package cn.hh.common.config;

import cn.hh.common.serializer.LongSerializer;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.data.redis.cache.RedisCacheConfiguration;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.RedisSerializationContext;

import java.time.Duration;

/**
 * @author shisl
 * @package cn.hh.common.config
 * @class RedisConfiguration
 * @description redis配置
 */
public class RedisConfiguration {

    /**
     * 天数
     *
     * @param days
     * @return
     */
    public static RedisCacheConfiguration getRedisCacheConfigurationWithDays(Long days) {
        return getRedisCacheConfigurationWithTtl(Duration.ofDays(days));
    }

    /**
     * 小时
     *
     * @param hours
     * @return
     */
    public static RedisCacheConfiguration getRedisCacheConfigurationWithHours(Long hours) {
        return getRedisCacheConfigurationWithTtl(Duration.ofHours(hours));
    }

    /**
     * 秒数
     *
     * @param seconds
     * @return
     */
    public static RedisCacheConfiguration getRedisCacheConfigurationWithSeconds(Long seconds) {
        return getRedisCacheConfigurationWithTtl(Duration.ofSeconds(seconds));
    }


    private static RedisCacheConfiguration getRedisCacheConfigurationWithTtl(Duration ttl) {
        Jackson2JsonRedisSerializer<Object> jackson2JsonRedisSerializer = new Jackson2JsonRedisSerializer<>(Object.class);
        ObjectMapper om = new ObjectMapper();
        om.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY);
        om.enableDefaultTyping(ObjectMapper.DefaultTyping.NON_FINAL);
        jackson2JsonRedisSerializer.setObjectMapper(om);

        RedisCacheConfiguration redisCacheConfiguration = RedisCacheConfiguration.defaultCacheConfig();
        redisCacheConfiguration = redisCacheConfiguration
                .serializeValuesWith(RedisSerializationContext.SerializationPair.fromSerializer(jackson2JsonRedisSerializer))
                .entryTtl(ttl);

        return redisCacheConfiguration;
    }


    public static RedisCacheConfiguration getRedisCacheConfigurationByLongWithHours(Long hours) {
        return getRedisCacheConfigurationByLongWithTtl(Duration.ofHours(hours));
    }


    public static RedisCacheConfiguration getRedisCacheConfigurationByLongWithSeconds(Long seconds) {
        return getRedisCacheConfigurationByLongWithTtl(Duration.ofSeconds(seconds));
    }


    public static RedisCacheConfiguration getRedisCacheConfigurationByLongWithDays(Long days) {
        return getRedisCacheConfigurationByLongWithTtl(Duration.ofDays(days));
    }

    /**
     * long serializer
     *
     * @param ttl
     * @return
     */
    private static RedisCacheConfiguration getRedisCacheConfigurationByLongWithTtl(Duration ttl) {
        LongSerializer longSerializer = new LongSerializer();
        RedisCacheConfiguration redisCacheConfiguration = RedisCacheConfiguration.defaultCacheConfig();
        redisCacheConfiguration = redisCacheConfiguration
                .serializeValuesWith(RedisSerializationContext.SerializationPair.fromSerializer(longSerializer))
                .entryTtl(ttl);

        return redisCacheConfiguration;
    }






}
