package cn.hh.common.utils;

import cn.hh.common.config.Constants;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.ZoneId;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

/**
 * @author shisl
 * @package cn.hh.common.util
 * @class DateUtils
 * @date 2023/1/5 下午4:06
 * @description 日期工具类
 */
@Slf4j
public class DateUtils {

    public static final String FORMAT_Y = "yyyy";
    public static final String FORMAT_D_1 = "yyyy/MM/dd";
    public static final String FORMAT_D_2 = "yyyy-MM-dd";
    public static final String FORMAT_D_3 = "yyyyMMdd";
    public static final String FORMAT_D_4 = "yyyy.MM.dd";
    public static final String FORMAT_D = "dd";
    public static final String FORMAT_DT_1 = "yyyy/MM/dd HH:mm:ss";
    public static final String FORMAT_DT_2 = "yyyy-MM-dd HH:mm:ss";
    public static final String FORMAT_DT_3 = "yyyyMMdd HH:mm:ss";
    public static final String FORMAT_DT_4 = "yyyy-MM-dd HH:mm";
    public static final String FORMAT_DT_5 = "yyyy.MM.dd HH:mm:ss";
    public static final String FORMAT_DT_6 = "yyyyMMddHHmmss";
    public static final String FORMAT_M_1 = "yyyy/MM";
    public static final String FORMAT_M_2 = "yyyy-MM";
    public static final String FORMAT_M_3 = "yyyyMM";
    public static final String FORMAT_M = "MM";
    public static final String FORMAT_MD_1 = "MM/dd";
    public static final String FORMAT_MD_2 = "MM-dd";
    public static final String FORMAT_MD_3 = "MMdd";
    public static final String FORMAT_T_1 = "HH:mm:ss";
    public static final String FORMAT_T_2 = "HH:mm";
    public static final String FORMAT_TH = "HH";
    public static final String FORMAT_TM = "mm";
    public static final String FORMAT_TS = "ss";
    public static final String FORMAT_UTC_ISO8601 = "yyyy-MM-dd'T'HH:mm:ss'Z'";

    public static String format(String format, Date date) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(date);
    }

    /**
     * @param format
     * @param date
     * @param timeZone 时区数字 -8, 0, 8 等
     * @return
     */
    public static String format(String format, Date date, int timeZone) {
        timeZone = timeZone % 13;
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        ZoneId zoneId = ZoneId.of("GMT" + (timeZone >= 0 ? "+" : "") + timeZone);
        TimeZone tz = TimeZone.getTimeZone(zoneId);

        sdf.setTimeZone(tz);

        return sdf.format(date);
    }

    public static Date parseDate(Date date) {
        String zoneDate = format(FORMAT_DT_2, date, 8);
        SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_DT_2);
        try {
            return sdf.parse(zoneDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static Date parse(String dateString, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);

        try {
            return sdf.parse(dateString);
        } catch (ParseException var4) {
            return null;
        }
    }


    public static Date parse(String dateString, String format, int timeZone) {

        SimpleDateFormat sdf = new SimpleDateFormat(format);
        ZoneId zoneId = ZoneId.of("GMT" + (timeZone >= 0 ? "+" : "") + timeZone);
        TimeZone tz = TimeZone.getTimeZone(zoneId);

        sdf.setTimeZone(tz);

        try {
            return sdf.parse(dateString);
        } catch (ParseException var4) {
            return null;
        }
    }

    public static Date todayStart() {
        Calendar calendar1 = Calendar.getInstance();
        calendar1.set(calendar1.get(Calendar.YEAR), calendar1.get(Calendar.MONTH), calendar1.get(Calendar.DAY_OF_MONTH),
                0, 0, 0);
        calendar1.setTimeZone(TimeZone.getTimeZone("GMT+8"));
        Date beginOfDate = calendar1.getTime();
        return beginOfDate;
    }

    public static Date todayEnd() {
        Calendar calendar1 = Calendar.getInstance();
        calendar1.set(calendar1.get(Calendar.YEAR), calendar1.get(Calendar.MONTH), calendar1.get(Calendar.DAY_OF_MONTH),
                23, 59, 59);
        calendar1.setTimeZone(TimeZone.getTimeZone("GMT+8"));
        Date endOfDate = calendar1.getTime();
        return endOfDate;
    }

    public static String format(Date date, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(date);
    }

    public static Long millSecToSec(Long millSeconds) {
        return millSeconds / 1000;
    }

    /**
     * 某天的开始时间
     *
     * @param date
     * @return
     */
    public static Date dateStart(Date date) {
        Calendar calendar1 = Calendar.getInstance();
        calendar1.setTime(date);
        calendar1.set(calendar1.get(Calendar.YEAR), calendar1.get(Calendar.MONTH), calendar1.get(Calendar.DAY_OF_MONTH), 0, 0, 0);
        calendar1.setTimeZone(TimeZone.getTimeZone("GMT+8"));
        Date end = calendar1.getTime();
        return end;
    }

    /**
     * 某天的最后时间
     *
     * @param date
     * @return
     */
    public static Date dateEnd(Date date) {
        Calendar calendar1 = Calendar.getInstance();
        calendar1.setTime(date);
        calendar1.set(calendar1.get(Calendar.YEAR), calendar1.get(Calendar.MONTH), calendar1.get(Calendar.DAY_OF_MONTH), 23, 59, 59);
        calendar1.setTimeZone(TimeZone.getTimeZone("GMT+8"));
        Date end = calendar1.getTime();
        return end;
    }


    /**
     * 为指定日期增加指定天数
     */
    public static String plusDay(int num, Date newDate, String format) {
        String enddate = null;
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        Date currdate = newDate;
        Calendar ca = Calendar.getInstance();
        ca.setTime(currdate);
        ca.add(Calendar.DATE, num);
        currdate = ca.getTime();
        enddate = sdf.format(currdate);
        return enddate;
    }

    /**
     * 几小时前的时间
     *
     * @param date
     * @param hours
     * @return
     */
    public static Date minusTimeByHour(Date date, Integer hours) {
        Calendar ca = Calendar.getInstance();
        ca.setTime(date);
        ca.set(Calendar.HOUR_OF_DAY, ca.get(Calendar.HOUR_OF_DAY) - hours);
        return ca.getTime();
    }

    /**
     * 几个月后的时间
     *
     * @param date
     * @param month
     * @return
     */
    public static Date addTimeByMonth(Date date, Integer month) {
        Calendar ca = Calendar.getInstance();
        ca.setTime(date);
        ca.set(Calendar.MONTH, ca.get(Calendar.MONTH) + month);
        return ca.getTime();
    }

    /**
     * 几小时后的时间
     *
     * @param date
     * @param hours
     * @return
     */
    public static Date addTimeByHour(Date date, Integer hours) {
        Calendar ca = Calendar.getInstance();
        ca.setTime(date);
        ca.set(Calendar.HOUR_OF_DAY, ca.get(Calendar.HOUR_OF_DAY) + hours);
        return ca.getTime();
    }


    public static Date addTimeByYear(Date date, Integer years) {
        Calendar ca = Calendar.getInstance();
        ca.setTime(date);
        ca.set(Calendar.YEAR, ca.get(Calendar.YEAR) + years);
        return ca.getTime();
    }

    /**
     * 间隔分钟
     *
     * @param firstTime
     * @param secondTime
     * @return
     */
    public static int getMinsBetweenTime(Long firstTime, Long secondTime) {
        int mins = (int) ((Math.abs(firstTime - secondTime)) / (1000 * 60));
        return mins;
    }

    /**
     * 间隔小时
     *
     * @param firstTime
     * @param secondTime
     * @return
     */
    public static int getHoursBetweenTime(Long firstTime, Long secondTime) {
        int hours = (int) ((Math.abs(firstTime - secondTime)) / (1000 * 60 * 60));
        return hours;
    }

    /**
     * 间隔天数
     *
     * @param firstTime
     * @param secondTime
     * @return
     */
    public static int getDaysBetweenTime(Long firstTime, Long secondTime) {
        int days = (int) ((Math.abs(firstTime - secondTime)) / (1000 * 60 * 60 * 24));
        return days;
    }

    /**
     * 获取当前时间所在月的开始时间
     *
     * @return
     */
    public static Date getMonthFirst(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        Date firstDayOfMonth = calendar.getTime();
        String format = DateUtils.format(firstDayOfMonth, DateUtils.FORMAT_D_2).concat(" 00:00:00");
        return DateUtils.parse(format, DateUtils.FORMAT_DT_2);

    }

    /**
     * 获取本月的结束时间
     *
     * @return
     */
    public static Date getMonthLast(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        calendar.add(Calendar.MONTH, 1);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        Date lastDayOfMonth = calendar.getTime();
        String format = DateUtils.format(lastDayOfMonth, DateUtils.FORMAT_D_2).concat(" 23:59:59");
        return DateUtils.parse(format, DateUtils.FORMAT_DT_2);
    }

    /**
     * 时区转换
     *
     * @param time           时间字符串
     * @param pattern        格式 "yyyy-MM-dd HH:mm"
     * @param nowTimeZone    eg:+8，0，+9，-1 等等
     * @param targetTimeZone 同nowTimeZone
     * @return
     */
    public static String timeZoneTransfer(String time, String pattern, int nowTimeZone, int targetTimeZone) {
        if (Preconditions.isBlank(time)) {
            return Constants.EMPTY_STRING;
        }
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("GMT" + (nowTimeZone >= 0 ? "+" : "") + nowTimeZone));
        Date date;
        try {
            date = simpleDateFormat.parse(time);
        } catch (ParseException e) {
            log.error("时间转换出错。", e);
            return Constants.EMPTY_STRING;
        }
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("GMT" + (targetTimeZone >= 0 ? "+" : "") + targetTimeZone));
        return simpleDateFormat.format(date);
    }

    /**
     * 计算两个时间相差几天几小时几秒
     *
     * @param diff
     * @return
     */
    public static String getTimeDiffString(long diff) {
        StringBuilder sb = new StringBuilder();
        // 只能精确到日 无法具细到年 月 不能确定一个月具体多少天 不能确定一年具体多少天
        // 获取日
        long day = diff / (1000 * 60 * 60 * 24);
        diff = diff % (1000 * 60 * 60 * 24);
        if (day > 0) {
            sb.append(day).append("天");
        }
        // 获取时
        long hour = diff / (1000 * 60 * 60);
        diff = diff % (1000 * 60 * 60);
        if (hour > 0) {
            sb.append(hour).append("时");
        }
        // 获取分
        long min = diff / (1000 * 60);
        diff = diff % (1000 * 60);
        if (min > 0) {
            sb.append(min).append("分");
        }
        // 获取秒
        long sec = diff / 1000;
        if (sec > 0) {
            sb.append(sec).append("秒");
        }
        return sb.toString();
    }

    /**
     * 获取当前系统的年份
     *
     * @return
     */
    public static Integer getSystemCurrentYear() {
        Calendar date = Calendar.getInstance();
        int year = date.get(Calendar.YEAR);
        return year;
    }

    /**
     * 获取当前系统的月份
     *
     * @return
     */
    public static Integer getSystemCurrentMonth() {
        Calendar date = Calendar.getInstance();
        int month = date.get(Calendar.MONTH) + 1;
        return month;
    }

    /**
     * 获取当前系统的季度
     *
     * @return
     */
    public static int getSystemCurrentQuarter() {
        Calendar c = Calendar.getInstance();
        //由于month从0开始，所有这里+1
        int month = c.get(Calendar.MONTH) + 1;
        return month % 3 == 0 ? month / 3 : month / 3 + 1;
    }

    /**
     * 获取某年第一天日期
     *
     * @param year 年份
     * @return Date 2023-01-01 00:00:00
     */
    public static Date getYearFirst(int year) {
        Calendar calendar = Calendar.getInstance();
        calendar.clear();
        calendar.set(Calendar.YEAR, year);
        Date currYearFirst = calendar.getTime();
        String nowTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(currYearFirst);
        log.info("{}年第一天为{}", year, nowTime);
        return currYearFirst;
    }

    /**
     * 获取某年最后一天日期
     *
     * @param year 年份
     * @return Date 2023-12-31 23:59:59
     */
    public static Date getYearLast(int year) {
        Calendar calendar = Calendar.getInstance();
        calendar.clear();
        calendar.set(Calendar.YEAR, year);
        calendar.roll(Calendar.DAY_OF_YEAR, -1);
        String concat = DateUtils.format(calendar.getTime(), DateUtils.FORMAT_D_2).concat(" 23:59:59");
        return DateUtils.parse(concat, DateUtils.FORMAT_DT_2);
    }

    /**
     * 相差年份
     *
     * @param startDate
     * @param endDate
     */
    public static long getYearsDiff(Date startDate, Date endDate) {
        return DateUtil.betweenYear(startDate, endDate, false);

    }


    public static void main(String[] args) {
//        long yearsDiff = getYearsDiff(DateUtils.parse("2020-03-26", DateUtils.FORMAT_D_2), new Date());
        log.info("" + DateUtil.endOfYear(new Date()));
    }

}
