package cn.hh.hhface.config;

import io.netty.channel.ChannelInitializer;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpServerCodec;
import io.netty.handler.codec.http.websocketx.WebSocketServerProtocolHandler;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.stream.ChunkedWriteHandler;
import io.netty.handler.timeout.ReadTimeoutHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * @author shisl
 * @package cn.hh.hhface.config
 * @class NioWebSocketChannelInitializer
 * @date 2023/12/10 下午6:05
 * @description WebSocket通道连接初始化
 */
@Component
public class NioWebSocketChannelInitializer extends ChannelInitializer<SocketChannel> {

    @Autowired
    private WebSocketProperties webSocketProperties;
    @Autowired
    private NioWebSocketHandler nioWebSocketHandler;

    @Override
    protected void initChannel(SocketChannel socketChannel) {
        socketChannel.pipeline()
                .addLast(new HttpServerCodec()) //用于解析HTTP请求和响应
                .addLast(new ChunkedWriteHandler())
                .addLast(new HttpObjectAggregator(8192)) //用于将HTTP消息的多个部分聚合成一个完整的HTTP消息。
                .addLast(nioWebSocketHandler) //WebSocket连接数据接收处理类
//                .addLast(new StringDecoder())
//                .addLast(new StringEncoder())
                //设置2min的超时时间，如果某个通道5min内未发送信号，则抛出异常删除当前通道
                .addLast(new ReadTimeoutHandler(2, TimeUnit.MINUTES))
                //可以对整个WebSocket通信进行初始化（当Http请求中有升级为WebSocket的请求时），以及握手处理
                .addLast(new WebSocketServerProtocolHandler(webSocketProperties.getPath(), null, true, 2147483647));
    }
}

