package cn.hh.hhface.config;

import io.netty.channel.Channel;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.util.concurrent.GlobalEventExecutor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Iterator;

/**
 * @author shisl
 * @package cn.hh.hhface.config
 * @class NioWebSocketChannelPool
 * @date 2023/12/10 下午6:00
 * @description WebSocket连接通道池
 */
@Slf4j
@Component
public class NioWebSocketChannelPool {

    private final ChannelGroup channels = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);

    /**
     * 新增一个客户端通道
     *
     * @param channel
     */
    public void addChannel(Channel channel) {
        channels.add(channel);
    }

    /**
     * 移除一个客户端连接通道
     *
     * @param channel
     */
    public void removeChannel(Channel channel) {
        channels.remove(channel);
    }

    /**
     * 获取指定通道
     *
     * @param channelId
     * @return
     */
    public Channel findChannel(String channelId) {
        Iterator var5 = channels.iterator();
        while (var5.hasNext()) {
            Channel c = (Channel) var5.next();
            if (c.id().asShortText().equals(channelId)) {
                return c;
            }
        }
        return null;
    }

    /**
     * 获取当前连接数
     *
     * @return
     */
    public int getConnectCount() {
        return channels.size();
    }


}

