package cn.hh.hhface.config;

import cn.hh.common.restful.BusinessException;
import cn.hh.common.restful.RestCode;
import cn.hh.common.utils.IPUtils;
import cn.hh.hhface.jwt.JwtService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;

/**
 * @author shisl
 * @package cn.hh.hhface.config
 * @class RequestInterceptor
 * @date 2022/12/20 上午10:32
 * @description
 */
@Slf4j
@Component
public class RequestInterceptor implements HandlerInterceptor {

    private static String HEADER_TOKEN = "Authorization";

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        //防止线程池环境下 ThreadLocal未被清理掉
        log.info("【=========preHandle  admin 管理员权限认证 begin==========】");
        log.info("管理员 IP={} : 请求 path={}", IPUtils.getIP(request), request.getServletPath());

        // 如果不是映射到方法直接通过
        if (!handler.getClass().isAssignableFrom(HandlerMethod.class)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) handler;
        Method method = handlerMethod.getMethod();
        //检查是否有Authority注释
        Authority annotation = method.getAnnotation(Authority.class);
        if (annotation != null) {
            if (!annotation.required()) {
                return true;
            }
        } else {
            annotation = handlerMethod.getBean().getClass().getAnnotation(Authority.class);
            if (annotation != null) {
                if (!annotation.required()) {
                    return true;
                }
            }
        }

        //检查有没有需要用户权限的注解
        String token = request.getHeader(HEADER_TOKEN);
        if (StringUtils.isEmpty(token)) {
            throw new BusinessException(RestCode.UNAUTHORIZED);
        }
        // 执行认证
        String accountId = JwtService.getUserId(token);
        if (StringUtils.isEmpty(accountId)) {
            throw new BusinessException(RestCode.UNAUTHORIZED);
        }

        if (annotation == null) {
            return true;
        }

        int[] needAuth = annotation.value();
        if (needAuth.length == 0) {
            return true;
        }
        if (needAuth[0] == 0) {
            return true;
        }
        throw new BusinessException(RestCode.UNAUTHORIZED);
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {

    }


}
