package cn.hh.hhface.jwt;

import cn.hh.common.restful.BusinessException;
import cn.hh.common.restful.RestCode;
import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTDecodeException;
import com.auth0.jwt.exceptions.JWTVerificationException;
import com.auth0.jwt.interfaces.DecodedJWT;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.util.Date;


/**
 * @author shisl
 * @package cn.hh.party.building.management.jwt
 * @class JwtService
 * @date 2022/12/20 上午10:37
 * @description
 */
@Slf4j
@Service
public class JwtService {

    private final static String SALT = "YcYp65l;uq234pwb;iIjHg872S-ajfL.khp636";

    /**
     * 校验token是否正确
     *
     * @param token 密钥
     * @param phone 用户的密码
     */
    public static void verify(String token, String userId, String phone) {
        try {
            String secret = DigestUtils.md5DigestAsHex(phone.concat(SALT).getBytes());
            Algorithm algorithm = Algorithm.HMAC256(secret);
            JWTVerifier verifier = JWT.require(algorithm)
                    .withClaim("userId", userId)
                    .build();
            verifier.verify(token);
        } catch (JWTVerificationException e) {
            throw new BusinessException(RestCode.UNAUTHORIZED.getCode(), RestCode.UNAUTHORIZED.getMsg());
        }
    }

    /**
     * 获得token中的信息无需secret解密也能获得
     *
     * @return token中包含的用户名
     */
    public static String getUserId(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim("userId").asString();
        } catch (JWTDecodeException e) {
            throw new BusinessException(RestCode.UNAUTHORIZED.getCode(), RestCode.UNAUTHORIZED.getMsg());
        }
    }

    /**
     * 生成签名,5min后过期
     *
     * @param userId 用户Id
     * @param password  用户的密码
     * @return 加密的token
     */
    public static String sign(String userId, String password) {
        return sign(userId, password, getExpireTime());
    }


    public static String sign(String userId, String password, Date expireTime) {
        String secret = DigestUtils.md5DigestAsHex(password.concat(SALT).getBytes());
        Algorithm algorithm = Algorithm.HMAC256(secret);
        return JWT.create()
                .withClaim("userId", userId)
                .withExpiresAt(expireTime)
                .withIssuedAt(new Date())
                .sign(algorithm);
    }

    /**
     * token是否过期
     *
     * @return true：过期
     */
    public static boolean isTokenExpired(String token) {
        Date now = new Date(System.currentTimeMillis());
        DecodedJWT jwt = JWT.decode(token);
        return jwt.getExpiresAt().before(now);
    }

    private static Date getExpireTime() {
        return DateUtils.addYears(new Date(), 1);
    }


}
