package com.yx.face.boot.mybatis;


import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.mapping.SqlSource;
import org.apache.ibatis.scripting.LanguageDriver;
import org.apache.ibatis.scripting.xmltags.XMLLanguageDriver;
import org.apache.ibatis.session.Configuration;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
/**
 * @author shisl
 * @package com.yx.face.boot.mybatis
 * @class LanguageDriverUpdateIn
 * @date 2024/10/23 下午5:12
 * @description 构造update in 动态sql更新
 */
public class LanguageDriverUpdateIn extends XMLLanguageDriver implements LanguageDriver {

    private final Pattern pattern = Pattern.compile("\\(#\\{(\\w+)\\}\\)");

    @Override
    public SqlSource createSqlSource(
            Configuration configuration, String script, Class<?> parameterType) {
        Matcher matcher = pattern.matcher(script);
        if (matcher.find()) {
            script =
                    matcher.replaceAll(
                            "(<foreach collection=\"$1\" item=\"__item\" separator=\",\" >#{__item}</foreach>)");
        }
        script = "<script>" + script + "</script>";
        SqlSource source = super.createSqlSource(configuration, script, parameterType);
        log.info("sql=>{}", script);
        return source;
    }
}
