package com.yx.face.boot.uitls;

import org.apache.commons.codec.binary.Base64;

import javax.crypto.*;
import javax.crypto.spec.DESedeKeySpec;
import java.nio.charset.Charset;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;


/**
 * @ProjectName: face-server
 * @Package: com.yx.face.boot.uitls
 * @ClassName: DESUtil
 * @Author: 崔哥
 * @Description:
 * @Date: 2021/10/10 15:31
 * @Version: 1.0
 */
public class DESUtil {


    private static final String KEY_ALGORITHM = "DESede";

    private static final String CIPHER_ALGORITHM = "DESede/ECB/PKCS5Padding";
    private static final String ENCODING = "UTF-8";

    /**
     * 加密
     *
     * @param data 参数json字符串
     * @param key  平台提供的deskey
     * @return
     */
    public static String encrypt(String data, String key) {
        byte[] encrypt = encrypt(data.getBytes(Charset.forName(ENCODING)), key);
        return encrypt == null ? null : Base64.encodeBase64String(encrypt);
    }

    private static byte[] encrypt(byte[] data, String key) {
        try {
            //实例化Des密钥
            DESedeKeySpec dks = new DESedeKeySpec(key.getBytes());
            //实例化密钥工厂
            SecretKeyFactory keyFactory = SecretKeyFactory.getInstance(KEY_ALGORITHM);
            //生成密钥
            SecretKey secretKey = keyFactory.generateSecret(dks);
            //实例化
            Cipher cipher = Cipher.getInstance(CIPHER_ALGORITHM);
            //初始化，设置为加密模式
            cipher.init(Cipher.ENCRYPT_MODE, secretKey);
            return cipher.doFinal(data);
        } catch (InvalidKeyException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (InvalidKeySpecException e) {
            e.printStackTrace();
        } catch (NoSuchPaddingException e) {
            e.printStackTrace();
        } catch (IllegalBlockSizeException e) {
            e.printStackTrace();
        } catch (BadPaddingException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static void main(String[] args) {
        String deskey = "c75a61acbc7b48cca67187de9c81c2fe";
        String data = "{\"idenNo\":\"91330104MA2J0CP852\"}";
        System.out.println(encrypt(data, deskey));
    }

}
