package com.yx.face.boot.uitls;

import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellUtil;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description:
 * @ClassName ExcelUtil
 * @Author WXG
 * @Date 2022/7/5 16:14
 */
public class ExcelUtil {
    private static final String EXCEL_XLS = "xls";
    private static final String EXCEL_XLSX = "xlsx";

    public static int DEFAULT_COLOUMN_WIDTH = 20;

    /**
     * 将excel文件解析为list
     *
     * @param file
     * @return List<List<Object>>
     * @throws IOException
     */
    public static List<Map<String, String>> getListByExcel(MultipartFile file) throws Exception {
        List<Map<String, String>> list = new ArrayList<Map<String, String>>();

        InputStream in = file.getInputStream();

        // 创建Excel工作薄
        Workbook work = getWorkbook(in, file.getOriginalFilename());
        if (null == work) {
            throw new Exception("创建Excel工作薄为空！");
        }
        Sheet sheet = null;
        Row row = null;
        Cell cell = null;
        sheet = work.getSheetAt(0);
        Row row1 = sheet.getRow(0);

        // 遍历Excel中所有的sheet
        for (int i = 0; i < work.getNumberOfSheets(); i++) {
            sheet = work.getSheetAt(i);
            if (sheet == null) {
                continue;
            }

            // 遍历当前sheet中的所有行
            for (int j = sheet.getFirstRowNum()+1; j <= sheet.getLastRowNum(); j++) {
                row = sheet.getRow(j);
                if (row == null) {
                    continue;
                }

                // 遍历所有的列
                Map<String, String> map = new HashMap<String, String>();
                for (int y = row.getFirstCellNum(); y < row.getLastCellNum(); y++) {
                    cell = row.getCell(y);
                    if(cell != null) {
                        if(cell.getCellType()==0) {
                            if(HSSFDateUtil.isCellDateFormatted(cell)){
                                SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                map.put(row1.getCell(y).getStringCellValue(), sdf.format(cell.getDateCellValue()));
                            }else {
                                DecimalFormat df = new DecimalFormat("########");
                                map.put(row1.getCell(y).getStringCellValue(), df.format(cell.getNumericCellValue()));
                            }
                        }else {
                            map.put(row1.getCell(y).getStringCellValue(), cell.getRichStringCellValue().getString());
                        }
                    }else {
                        map.put(row1.getCell(y).getStringCellValue(), null);
                    }
                }
                list.add(map);
            }
        }
        in.close();
        return list;
    }
    /**
     * 描述：根据文件后缀，自适应上传文件的版本
     *
     * @param inStr,fileName
     * @return
     * @throws Exception
     */
    private static Workbook getWorkbook(InputStream inStr, String fileName) throws Exception {
        Workbook wb = null;
        String fileType = fileName.substring(fileName.lastIndexOf("."));
        if (("."+EXCEL_XLS).equals(fileType)) {
            wb = new HSSFWorkbook(inStr); // 2003-
        }else if(("."+EXCEL_XLSX).equals(fileType)) {
            wb = new XSSFWorkbook(inStr);
        }else {
            throw new Exception("解析的文件格式有误！");
        }
        return wb;
    }


    public static void Excel2007AboveOperate(String filePath) throws IOException {
        XSSFWorkbook workbook1 = new XSSFWorkbook(new FileInputStream(new File(filePath)));
        SXSSFWorkbook sxssfWorkbook = new SXSSFWorkbook(workbook1, 100);
        Sheet first = sxssfWorkbook.getSheetAt(0);
        for (int i = 0; i < 100000; i++) {
            Row row = first.createRow(i);
            for (int j = 0; j < 11; j++) {
                if(i == 0) {
                    // 首行
                    row.createCell(j).setCellValue("column" + j);
                } else {
                    // 数据
                    if (j == 0) {
                        CellUtil.createCell(row, j, String.valueOf(i));
                    } else
                        CellUtil.createCell(row, j, String.valueOf(Math.random()));
                }
            }
        }
        FileOutputStream out = new FileOutputStream("workbook.xlsx");
        sxssfWorkbook.write(out);
        out.close();
    }
}
