package com.yx.face.boot.uitls.excel;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.*;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author lilei
 * @version V 0.1
 * @describe {}
 * @time 2017/8/9 16:10.
 */
@Slf4j
public class ExcelReader {
    private Workbook workbook;

    public ExcelReader read(File file) throws IOException, InvalidFormatException {
        return read(file, null);
    }

    public ExcelReader read(File file, String password) throws IOException, InvalidFormatException {
        this.workbook = WorkbookFactory.create(file, password);
        return this;
    }

    public ExcelReader read(InputStream inputStream) throws IOException, InvalidFormatException {
        return read(inputStream, null);
    }

    public ExcelReader read(InputStream inputStream, String password) throws IOException, InvalidFormatException {
        this.workbook = WorkbookFactory.create(inputStream, password);
        return this;
    }

    public List<Map<String, String>> toMaplist(String sheetName, int titleRowIndex) {
        return toMaplist(sheetName, 0, titleRowIndex);
    }

    public List<Map<String, String>> toMaplist(int sheetIndex, int titleRowIndex) {
        return toMaplist(null, sheetIndex, titleRowIndex);
    }

    public List<Map<String, String>> toMaplist() {
        return toMaplist(null, 0, 0);
    }

    /**
     * Workbook 中 指定Sheet 转换为 List<Map<String, Object>>
     *
     * @param sheetName     sheet名字
     * @param sheetIndex    sheet位置
     * @param titleRowIndex titleRow位置
     * @return
     * @throws IOException
     */
    public List<Map<String, String>> toMaplist(String sheetName, int sheetIndex, int titleRowIndex) {
        List<Map<String, String>> result = new ArrayList<>();
        Sheet sheet;
        if (sheetName != null && sheetName.length() > 0) {
            sheet = this.workbook.getSheet(sheetName);
        } else {
            sheet = this.workbook.getSheetAt(sheetIndex);
        }
        int lastRowNum = sheet.getLastRowNum();
        Row titleRow = null;
        int lastCellNum = 0;
        String[] titles = new String[0];
        for (int j = titleRowIndex; j < lastRowNum; j++) {
            Map<String, String> map = new HashMap<>();
            if (j == titleRowIndex) {
                titleRow = sheet.getRow(j);
                lastCellNum = titleRow.getLastCellNum();
                titles = new String[lastCellNum];
            }
            Row dataRow = sheet.getRow(j + 1);
            for (int k = 0; k < lastCellNum; k++) {
                if (j == titleRowIndex) {
                    titles[k] = titleRow.getCell(k).getStringCellValue();
                }

                if (dataRow.getCell(k) != null && dataRow.getCell(k).getCellType() == CellType.NUMERIC.getCode()) {
                    try {
                        if (DateUtil.isCellDateFormatted(dataRow.getCell(k))) {
                            map.put(titles[k], dataRow.getCell(k) == null ? StrUtil.EMPTY : cn.hutool.core.date.DateUtil.formatDateTime(dataRow.getCell(k).getDateCellValue()));
                        }else{
                            DecimalFormat df = new DecimalFormat("0");
                            map.put(titles[k], dataRow.getCell(k) == null ? StrUtil.EMPTY : df.format(dataRow.getCell(k).getNumericCellValue()));
                        }
                    }catch (Exception e){
                        log.info("数据转换错误");
                        e.printStackTrace();
                    }
                }else{
                    map.put(titles[k], dataRow.getCell(k) == null ? StrUtil.EMPTY : dataRow.getCell(k).getStringCellValue());
                }
            }
            result.add(map);
        }
        return result;
    }

    public Workbook getWorkbook() {
        return workbook;
    }
}
