package com.yx.face.model.entity;

import com.yx.face.boot.handle.IntegerTypeHandler;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;
import tk.mybatis.mapper.annotation.ColumnType;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @description: 管理人员信息表(Admin)实体类 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-06-04 14:25:42 <br>
 */
@Data
@Accessors(chain = true)
@Table(name = "admin")
public class Admin implements Serializable {

    /**
     * 管理人员表主键id
     */

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)

    private Integer id;

    /**
     * 呢称
     */


    private String nickname;

    /**
     * 管理员名称
     */


    private String username;

    /**
     * 管理员密码
     */


    private String password;

    /**
     * 头像路径
     */


    private String avatar;

    /**
     * 身份证号
     */


    @Column(name = "id_number")
    private String idNumber;

    /**
     * 管理员联系电话
     */


    private String phone;

    /**
     * 管理员邮箱
     */
    private String email;


    @Column(name = "zfb_email")
    private String zfbEmail;

    /**
     * 女-0 | 男- 1| 未选中-2
     */


    private Byte gender;

    /**
     * 管理员类型 1.管理员 2.省级总账号 3.省级 4.市级 5.区级
     */


    private Byte type;


    /**
     * 数据更新时间
     */


    @Column(name = "update_time")
    private Date updateTime;

    /**
     * 数据创建时间
     */


    @Column(name = "create_time")
    private Date createTime;
    /*
     * 省份地址编码
     * */
    @Column(name = "province_id")
    private Integer provinceId;
    /*
     * 省份
     * */
    @Column(name = "province_name")
    private String provinceName;
    /*
     * 城市地址编码
     * */
    @Column(name = "city_id")
    private Integer cityId;
    /*
     * 城市
     * */
    @Column(name = "city_name")
    private String cityName;
    /*
     * 县区地址编码
     * */
    @Column(name = "area_id")
    private Integer areaId;
    /*
     * 县区
     * */
    @Column(name = "area_name")
    private String areaName;
    /*
     * 街道编号
     * */
    @Column(name = "street_id")
    private Integer streetId;
    /*
     * 街道 乡 镇名称
     * */
    @Column(name = "street_name")
    private String streetName;
    /*
     * 详细地址
     * */
    private String address;
    /*
     * 派出所编码
     * */
    @Column(name = "police_station_code")
    private String policeStationCode;
    /*
     * 街道 乡 镇名称
     * */
    @Column(name = "police_station_name")
    private String policeStationName;
    /*
     * 标签id
     * */
    @Column(name = "tag_id")
    private Integer tagId;
    /*
     * 导出权限 1.可以 2.不可以
     * */
    @Column(name = "is_out")
    private Integer isOut;
    /*
     * 是否脱敏重要信息 1.脱敏 2.不脱敏
     * */
    @Column(name = "is_hide_important")
    private Integer isHideImportant;

    /*
     * 常访客类型1常客2访客
     * */
    @Column(name = "white_visitor_type")
    private String whiteVisitorType;

    @ApiModelProperty(value = "自定义字段")
    private String content;


    /*
     * 是否开启布控 1.开启  2.不开启 默认2
     * */
    @Column(name = "is_control_early_warning")
    private Integer isControlEarlyWarning;
    /*
     * 脱敏的字段 1.姓名 2. 手机号 3.身份证
     * */
    @Column(name = "no_hide_cloumn")
    private String noHideCloumn;

    @ApiModelProperty(value = "是否开启下发")
    @Column(name = "is_push")
    private Integer isPush;

    @ApiModelProperty(value = "二维码类型记录标识，0：通用码，1：常客码")
    @Column(name = "code_type")
    private String codeType;

    @ApiModelProperty(value = "场所类型，0：通用场所，1：预约场所")
    @Column(name = "place_type")
    private Integer placeType;

    @ApiModelProperty(value = "审核状态 0:待审核 1:已拒绝 2:已同意")
    @Column(name = "review_status")
    private Integer reviewStatus;

    @ApiModelProperty(value = "租户id")
    @Column(name = "tenant_id")
    private Integer tenantId;

    @ApiModelProperty("是否开启场所码，false：否，true: 是")
    private Boolean placeCodeStatus;

    @ApiModelProperty("注册到某个地址码")
    private String addressCode;

    @ApiModelProperty("场所码单位名称")
    private String unitName;

    @ApiModelProperty("场所码单位地址")
    private String unitAddress;

    @ApiModelProperty("场所码管理员健康码信息")
    private String managerHealthCode;

    /*
     * 是否开启私有区域 1-开启 2-不开启
     * 注释：楼宇场景下的公私有区域
     * */
    @Column(name = "is_open_private_area")
    private Integer isOpenPrivateArea;

    @ApiModelProperty("核酸有效期时间，单位小时，0:不限")
    @Column(name = "nucleic_expiration_time")
    private Integer nucleicExpirationTime;

    @Column(name = "scenic_flag")
    private Boolean scenicFlag;

    /*
     * 三码 1-健康码 2-核酸 3-行程码
     * */
    @Column(name = "three_code")
    private String threeCode;
    /*
     * 核酸时长 三码中如果存在核酸 则需要添加核酸时长
     * */
    @Column(name = "hs_hour")
    private Integer hsHour;

    /*
     * 常客有效时长（单位/天）
     * */
    @Column(name = "user_whitelist_effective_time")
    private BigDecimal userWhitelistEffectiveTime;
    /*
     * 访客有效时长（单位/天）
     * */
    @Column(name = "user_visiter_effective_time")
    private BigDecimal userVisiterEffectiveTime;
    /*
     * 风险区（中间逗号相隔）
     * */
    @Column(name = "risk_area")
    private String riskArea;
    /*
     * 是否开启物理卡号：1-开启 2-不开启
     * */
    @Column(name = "card_physics")
    private Integer cardPhysics;
    /*
     * 实名认证年龄段 为空则表示都需要实名认证 格式：20-50
     * */
    @Column(name = "real_authentication_age")
    private String realAuthenticationAge;
    /*
     * 打卡上报字段配置
     * */
    @Deprecated //已弃用
    @Column(name = "punch_report_fields_config")
    private String punchReportFieldsConfig;


    @ApiModelProperty("打卡方式 0 钉钉 1 通用")
    private String clockWayType;

    @Deprecated //已弃用
    @ApiModelProperty("打卡信息推送地址")
    private String clockWayPushUrl;
    /*
     * 不允许进入的年龄段 格式：20-50
     * */
    @Column(name = "not_allow_age")
    private String notAllowAge;
    /**
     * 不允许进入的年龄段提示信息
     */
    @Column(name = "not_allow_age_msg")
    private String notAllowAgeMsg;
    /**
     * 是否报警 true/false
     */
    @Column(name = "is_alarm")
    private Boolean isAlarm = false;

    @ApiModelProperty(value = "钉钉应用的唯一标识key")
    private String dingtalkAppKey;

    @ApiModelProperty(value = "钉钉应用的密钥")
    private String dingtalkAppSecret;

    /*
     * 没有行程信息是否开门 1-开门（默认）0-关门
     * */
    @Column(name = "no_xc_open_door")
    private Boolean noXcOpenDoor;

    /*
     * 24小时内核酸报告未出是否开门：1-开门（默认）0-关门
     * */
    @Column(name = "no_hs_open_door")
    private Boolean noHsOpenDoor;

    /*
     * 去过中高风险地区是否开门 1-开门（默认）0-关门
     * */
    @Column(name = "risk_xc_open_door")
    private Boolean riskXcOpenDoor;
    /*
     * 不允许通行的身份证地区代码，用/分隔
     * */
    @Column(name = "no_allow_area_code")
    private String noAllowAreaCode;

    /*
     * 行程码需检测是否开门 1-开门（默认）0-关门
     * */
    @Column(name = "xc_need_check_open_door")
    private Boolean xcNeedCheckOpenDoor;

    /*
     * 屏保照片
     * */
    @Column(name = "screen_photos")
    private String screenPhotos;
    /**
     * 首页屏幕图片
     */
    @Column(name = "home_img_url")
    private String homeImgUrl;

    /**
     * 开启问题上报 1:开启 0：不开启
     */
    @Column(name = "problem_report_flag")
    private Integer problemReportFlag;

    /**
     * 开启区域消杀 1:开启 0：不开启
     */
    @Column(name = "disinfect_flag")
    private Integer disinfectFlag;

    /**
     * 是否限制三天三检 1:是   0:否（默认）
     */
    @Column(name = "limit_gather")
    private Integer limitGather;

    /**
     * 微信小程序登录方式 1-身份证 2-卡号认证
     */
    @Column(name = "wx_login_type")
    private Integer wxLoginType;

    /**
     * 是否需要实名认证 1-需要 0-不需要
     */
    @Column(name = "is_need_real_authentication")
    private Boolean isNeedRealAuthentication;
    /*
     * 是否开启访客零点清除功能 1-开启 0-关闭
     * */
    @Column(name = "is_open_visitor_clear")
    private Boolean isOpenVisitorClear;

    /*
     * 企业账号的配置-小程序是否可被搜索选择 1-可以 0-不可以
     * */
    @Column(name = "is_can_search")
    private Integer isCanSearch;
    /*
     * 自定义输入手机号标志位 1输入 2获取
     * */
    @Column(name = "mobile_auto_input_flag")
    private Integer mobileAutoInputFlag;

    /**
     * ------ 账户添加配置信息2023-02-06 ------
     **/
    @ApiModelProperty(value = "是否开启健康码功能 0:关闭 1:开启 ")
    private Integer isOpenHealthCode;

    /**
     * 是否开启进出记录统计
     */
    @Column(name = "open_in_out_statistics")
    private Boolean openInOutStatistics;

    /**
     * ------ 账户添加配置信息2023-04-13 ------
     **/
    @ApiModelProperty("二维码是否显示 true/false")
    private Boolean showQrcode = false;
    @ApiModelProperty(value = "二维码的码值")
    private String qrcodeText;
    @ApiModelProperty(value = "广告图片的地址 ")
    private String advertise;

    /**
     * 考试设备集合 用逗号隔开
     */
    @Column(name = "exam_devices")
    private String examDevices;
    /*
     * 是否开启考试 1-开启 0-不开启
     * */
    @Column(name = "is_open_exam")
    private Boolean isOpenExam;

    @Deprecated
    @ApiModelProperty(value = "商户平台id 已弃用")
    private Long tenantPlatformId;

    /**
     * ------ 账户添加配置信息2023-04-25 ------
     **/
    @ApiModelProperty(value = "联动平台服务是否开启 true/false")
    private Boolean openThirdLinkPlatform;
    @ApiModelProperty(value = "顶部标题 设备显示配置 mainLine")
    private String topTitle;
    @ApiModelProperty(value = "成功-通行结果")
    private String subLineSuccess;
    @ApiModelProperty(value = "失败-通行结果")
    private String subLineFail;
    @ApiModelProperty(value = "左副框上层")
    private String mainLineLeft;
    @ApiModelProperty(value = "左副框下层")
    private String subLineLeft;
    @ApiModelProperty(value = "右副框上层")
    private String mainLineRight;
    @ApiModelProperty(value = "右副框下层")
    private String subLineRight;
    @ApiModelProperty(value = "成功-语音播报内容")
    private String openVoiceMsgSuccess;
    @ApiModelProperty(value = "失败-语音播报内容")
    private String openVoiceMsgFail;

    /**
     * 人脸数据保留天数
     */
    @ApiModelProperty(value = "人脸数据保留天数")
    private Integer faceLogHoldDays;
    /*
     * 常客小程序是否需要实名 0-否 1-需要
     * */
    private Boolean whitelistAppRealname;
    /*
     * 常客小程序是否需要照片 0-否 1-需要
     * */
    private Boolean whitelistAppPhoto;
    /*
    * 常客的登录参数列表
    * @see
    * */
    @ColumnType(column = "whitelist_app_login",typeHandler = IntegerTypeHandler.class)
    private List<Integer> whitelistAppLoginList;
    /*
     * 常客小程序是否需要手机号 0-否 1-需要
     * */
    private Boolean whitelistAppPhone;
    /*
     * 访客小程序是否需要实名 0-否 1-需要
     * */
    private Boolean visitorAppRealname;
    /*
     * 访客小程序是否需要照片 0-否 1-需要
     * */
    private Boolean visitorAppPhoto;
    /*
     * 访客的登录参数列表
     * */
    @ColumnType(column = "visitor_app_login",typeHandler = IntegerTypeHandler.class)
    private List<Integer> visitorAppLoginList;
    /*
     * 访客小程序是否需要手机号 0-否 1-需要
     * */
    private Boolean visitorAppPhone;

    /**
     * 楼宇审核方式 1-常客审核 2-指定人员审核
     * @see com.yx.face.model.enums.AdminBuildingCheckTypeEnum
     * */
    private Integer buildingCheckType;

    @ApiModelProperty(value = "设备结果页类型 false:老版  true:新版")
    private Boolean latestResultPage;
    /**
     * 机构编码
     */
    @Column(name = "org_out_id")
    private String orgOutId;
    /*
    * 上级名称-显示使用
    * */
    @Column(name = "tenant_enterprise_name")
    private String tenantEnterpriseName;
    /*
    * 下级名称-显示使用
    * */
    @Column(name = "enterprise_name")
    private String enterpriseName;
    @ApiModelProperty(value = "支付宝小程序单机二维码标题")
    private String zfbMiniEthTitle;
    @ApiModelProperty(value = "支付宝小程序联机二维码标题")
    private String zfbMiniVguangTitle;

    @ApiModelProperty(value = "是否开启进出预警 false-不开启 true-开启")
    private Boolean isOpenInoutWarning;

    /**
     * ------ 2024-03-18添加 ------
     **/
    @ApiModelProperty(value = "员工管理接口是否开启 false-不开启 true-开启")
    private Boolean userManageSwitch;
    @ApiModelProperty(value = "区域管理接口是否开启 false-不开启 true-开启")
    private Boolean zoneManageSwitch;

    @ApiModelProperty(value = "是否开启梯控 false-不开启 true-开启")
    private Boolean isOpenLadder;

}

