package com.yx.face.model.enums;

import com.yx.face.boot.uitls.Preconditions;
import lombok.Getter;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package com.yx.face.model.enums
 * @class FkjQrcodeFloorEnum
 * @date 2023/8/9 上午9:23
 * @description 访客机二维码 楼层参数2进制转16进制
 * 二进制
 * 00000000  00000000 00000000 00000000 00000000 00000000 00000000 00000000
 * 16进制
 * 00           00      00      00          00      00      00      00
 * Floor7     Floor6  Floor5  Floor4    Floor3     Floor2   Floor1  Floor0
 * 代表1-64层
 */
@Getter
public enum FkjQrcodeFloorV2Enum {

    ONE(1, "00000001", -3, -3),
    TWO(2, "00000010", -2, -2),
    THREE(3, "00000100", -1, -1),
    FOUR(4, "00001000", 1, 1),
    FIVE(5, "00010000", 2, 2),
    SIX(6, "00100000", null, 3),
    SEVEN(7, "01000000", null, 4),
    EIGHT(8, "10000000", null, 5),

    NINE(9, "00000001", null, 6),
    TEN(10, "00000010", null, 7),
    ELEVEN(11, "00000100", null, 8),
    TWELVE(12, "00001000", null, 9),
    THIRTEEN(13, "00010000", null, 10),
    FOURTEEN(14, "00100000", null, 11),
    FIFTEEN(15, "01000000", null, 12),
    SIXTEEN(16, "10000000", null, 13),

    SEVENTEEN(17, "00000001", null, 14),
    EIGHTEEN(18, "00000010", null, 15),
    NINETEEN(19, "00000100", null, 16),
    TWENTY(20, "00001000", null, 17),
    TWENTY_ONE(21, "00010000", null, 18),
    TWENTY_TWO(22, "00100000", null, 19),
    TWENTY_THREE(23, "01000000", null, 20),
    TWENTY_FOUR(24, "10000000", null, 21),

    TWENTY_FIVE(25, "00000001", 22, null),
    TWENTY_SIX(26, "00000010", 23, null),
    TWENTY_SEVEN(27, "00000100", 24, null),
    TWENTY_EIGHT(28, "00001000", 25, null),
    TWENTY_NINE(29, "00010000", 26, null),
    THIRTY(30, "00100000", 27, null),
    THIRTY_ONE(31, "01000000", 28, null),
    THIRTY_TWO(32, "10000000", 29, null),

    THIRTY_THREE(33, "00000001", 30, null),
    THIRTY_FOUR(34, "00000010", 31, null),
    THIRTY_FIVE(35, "00000100", 32, null),
    THIRTY_SIX(36, "00001000", 33, null),
    THIRTY_SEVEN(37, "00010000", 34, null),
    THIRTY_EIGHT(38, "00100000", 35, null),
    THIRTY_NINE(39, "01000000", 36, null),
    FORTY(40, "10000000", 37, null),

    FORTY_ONE(41, "00000001", 38, null),
    FORTY_TWO(42, "00000010", 39, null),
    FORTY_THREE(43, "00000100", 40, null),
    FORTY_FOUR(44, "00001000", 41, null),
    FORTY_FIVE(45, "00010000", 42, null),
    FORTY_SIX(46, "00100000", 43, null),
    FORTY_SEVEN(47, "01000000", 44, null),
    FORTY_EIGHT(48, "10000000", 45, null),

    FORTY_NINE(49, "00000001", 46, null),
    FIFTY(50, "00000010", 47, null),
    FIFTY_ONE(51, "00000100", 48, null),
    FIFTY_TWO(52, "00001000", 49, null),
    FIFTY_THREE(53, "00010000", 50, null),
    FIFTY_FOUR(54, "00100000", 51, null),
    FIFTY_FIVE(55, " 01000000", 52, null),
    FIFTY_SIX(56, "10000000", 53, null),

    FIFTY_SEVEN(57, "00000001", 54, null),
    FIFTY_EIGHT(58, "00000010", 55, null),
    FIFTY_NINE(59, "00000100", 56, null),
    SIXTY(60, "00001000", 57, null),
    SIXTY_ONE(61, "00010000", 58, null),
    SIXTY_TWO(62, "00100000", 59, null),
    SIXTY_THREE(63, "01000000", 60, null),
    SIXTY_FOUR(64, "10000000", 61, null);

    //UI 逻辑楼层
    Integer floor;
    String hexadecimal;
    //高层 实际楼层
    Integer highFloor;
    //低层 实际楼层
    Integer lowFloor;

    //默认16位16进制楼层
    private static String default16Hexadecimal = " 00 00 00 00 00 00 00 08";
    //默认2位16进制楼层
    private static String default2Hexadecimal = "00";
    //默认的二进制
    private static String defaultBinary = "00000000";

    FkjQrcodeFloorV2Enum(Integer floor, String hexadecimal, Integer highFloor, Integer lowFloor) {
        this.floor = floor;
        this.hexadecimal = hexadecimal;
        this.highFloor = highFloor;
        this.lowFloor = lowFloor;
    }

    public static FkjQrcodeFloorV2Enum getByFloor(Integer floor) {
        if (Preconditions.isNotBlank(floor)) {
            for (FkjQrcodeFloorV2Enum floorEnum : values()) {
                //低层
                if (floor.intValue() <= 21 && Preconditions.isNotBlank(floorEnum.getLowFloor()) && floorEnum.getLowFloor().intValue() == floor.intValue()) {
                    return floorEnum;
                }
                //高层
                if (floor.intValue() > 21 && Preconditions.isNotBlank(floorEnum.getHighFloor()) && floorEnum.getHighFloor().intValue() == floor.intValue()) {
                    return floorEnum;
                }
            }
        }
        return FkjQrcodeFloorV2Enum.FOUR;
    }

    /**
     * 逗号隔开的字符串 "1","2"
     *
     * @param floorsStr
     * @return
     */
    public static String getHexadecimalByFloorStr(String floorsStr) {
        List<Integer> floors = Arrays.asList(floorsStr.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        return getHexadecimalByFloor(floors);
    }

    /**
     * 逻辑楼层集合 [1,2,3]
     *
     * @param floors
     * @return
     */
    public static String getHexadecimalByFloor(List<Integer> floors) {
        if (Preconditions.isBlank(floors)) {
            return default16Hexadecimal;
        }
        List<String> from1To8 = new ArrayList<>();
        List<String> from9To16 = new ArrayList<>();
        List<String> from17To24 = new ArrayList<>();
        List<String> from25To32 = new ArrayList<>();
        List<String> from33To40 = new ArrayList<>();
        List<String> from41To48 = new ArrayList<>();
        List<String> from49To56 = new ArrayList<>();
        List<String> from57To64 = new ArrayList<>();
        for (Integer floor : floors) {
            if (floor.intValue() <= 8) {
                from1To8.add(get2HexadecimalByFloor(floor));
                continue;
            }
            if (floor.intValue() <= 16) {
                from9To16.add(get2HexadecimalByFloor(floor));
                continue;
            }
            if (floor.intValue() <= 24) {
                from17To24.add(get2HexadecimalByFloor(floor));
                continue;
            }
            if (floor.intValue() <= 32) {
                from25To32.add(get2HexadecimalByFloor(floor));
                continue;
            }
            if (floor.intValue() <= 40) {
                from33To40.add(get2HexadecimalByFloor(floor));
                continue;
            }
            if (floor.intValue() <= 48) {
                from41To48.add(get2HexadecimalByFloor(floor));
                continue;
            }
            if (floor.intValue() <= 56) {
                from49To56.add(get2HexadecimalByFloor(floor));
                continue;
            }
            if (floor.intValue() <= 64) {
                from57To64.add(get2HexadecimalByFloor(floor));
                continue;
            }
        }

        String from1To8Str = Preconditions.isNotBlank(from1To8) ? addBinary(from1To8) : default2Hexadecimal;
        String from9To16Str = Preconditions.isNotBlank(from9To16) ? addBinary(from9To16) : default2Hexadecimal;
        String from17To24Str = Preconditions.isNotBlank(from17To24) ? addBinary(from17To24) : default2Hexadecimal;
        String from25To32Str = Preconditions.isNotBlank(from25To32) ? addBinary(from25To32) : default2Hexadecimal;
        String from33To40Str = Preconditions.isNotBlank(from33To40) ? addBinary(from33To40) : default2Hexadecimal;
        String from41To48Str = Preconditions.isNotBlank(from41To48) ? addBinary(from41To48) : default2Hexadecimal;
        String from49To56Str = Preconditions.isNotBlank(from49To56) ? addBinary(from49To56) : default2Hexadecimal;
        String from57To64Str = Preconditions.isNotBlank(from57To64) ? addBinary(from57To64) : default2Hexadecimal;
        StringBuffer hexadecimalBuff = new StringBuffer();
        hexadecimalBuff.append(" ").append(from57To64Str)
                .append(" ").append(from49To56Str)
                .append(" ").append(from41To48Str)
                .append(" ").append(from33To40Str)
                .append(" ").append(from25To32Str)
                .append(" ").append(from17To24Str)
                .append(" ").append(from9To16Str)
                .append(" ").append(from1To8Str);
        return hexadecimalBuff.toString();
    }

    public static String get2HexadecimalByFloor(Integer floor) {
        if (Preconditions.isNotBlank(floor)) {
            for (FkjQrcodeFloorV2Enum floorEnum : values()) {
                if (floorEnum.getFloor().intValue() == floor.intValue()) {
                    return floorEnum.getHexadecimal();
                }
            }
        }
        return defaultBinary;
    }


    public static String addBinary(List<String> fromXToY) {
        if (Preconditions.isBlank(fromXToY)) {
            return default16Hexadecimal;
        }
        // 将二进制字符串转换为整数
        BigInteger result = new BigInteger(defaultBinary, 2);
        // 计算整数的和
        for (String item : fromXToY) {
            result = result.add(new BigInteger(item, 2));
        }
        // 将和转换回二进制字符串
        String binaryResult = String.format("%8s", result.toString(2)).replace(' ', '0');
        return String.format("%2s", new BigInteger(binaryResult, 2).toString(16)).replace(' ', '0');
    }

    public static void main(String[] args) {
        String result = getHexadecimalByFloor(Arrays.asList(32));
        System.out.println("Result in binary: " + result);
    }


}
