package com.yx.face.model.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.Date;
import java.util.List;

/**
 * @description: 管理人员信息表(Admin)VO <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-06-04 14:25:43 <br>
 */

@ApiModel("管理人员信息表VO model")
@Data
public class AdminVO {

    @ApiModelProperty(value = "管理人员表主键id")
    private Integer adminId;

    private List<Integer> adminIds;

    @ApiModelProperty(value = "呢称")
    private String name;

    @ApiModelProperty(value = "账号")
    private String username;

    @ApiModelProperty(value = "密码")
    private String password;

    @ApiModelProperty(value = "头像路径")
    private String avatar;

    @ApiModelProperty(value = "身份证号")
    private String idNumber;

    @ApiModelProperty(value = "管理员联系电话")
    private String phone;

    @ApiModelProperty(value = "管理员邮箱")
    private String email;

    @ApiModelProperty(value = "女-0 | 男- 1| 未选中-2")
    private Byte gender;

    @ApiModelProperty(value = "管理员类型 1.管理员 2.省级总账号 3.省级 4.市级 5.区级")
    private Byte type;

    @ApiModelProperty(value = "token", hidden = true)
    private String token;

    @ApiModelProperty(value = "数据创建时间")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    @ApiModelProperty(value = "省份地址编码")
    private Integer provinceId;

    @ApiModelProperty(value = "省份")
    private String provinceName;

    @ApiModelProperty(value = "城市地址编码")
    private Integer cityId;

    @ApiModelProperty(value = "城市")
    private String cityName;

    @ApiModelProperty(value = "县区地址编码")
    private Integer areaId;

    @ApiModelProperty(value = "县区")
    private String areaName;

    @ApiModelProperty(value = "街道编码")
    private Integer streetId;

    @ApiModelProperty(value = "街道/乡/镇")
    private String streetName;

    @ApiModelProperty(value = "详细地址")
    private String address;

    @ApiModelProperty(value = "派出所编码")
    private String policeStationCode;

    @ApiModelProperty(value = "派出所名称")
    private String policeStationName;

    @ApiModelProperty(value = "所属部门id")
    private Integer tagId;

    @ApiModelProperty(value = "所属部门名称")
    private String tagName;

    @ApiModelProperty(value = "导出权限 1.可以 2.不可以")
    private Integer isOut;

    @ApiModelProperty(value = "是否脱敏重要信息 1.脱敏 2.不脱敏")
    private Integer isHideImportant;

    @ApiModelProperty("脱敏数据查询是否配置 0-未配置 1 -配置")
    private Integer sysQueryConfiguration;

    @ApiModelProperty("脱敏数据查询配置是否开启 0-未开启 1 -开启")
    private Integer sysQueryConfigurationStatus;

    @ApiModelProperty("1常客2访客")
    private List<String> whiteVisitorType;

    /***
     * 1常客2访客 字符串
     */
    private String wvTypes;

    @ApiModelProperty(value = "是否开启布控 1.开启  2.不开启 默认2")
    private Integer isControlEarlyWarning;

    @ApiModelProperty(value = "不需要脱敏的字段 1.姓名 2. 手机号 3.身份证")
    private List<String> noHideCloumn;

    private String noHideCloumnString;

    @ApiModelProperty(value = "是否开启下发 0(默认)不开启  1开启")
    private Integer isPush;

    @ApiModelProperty(value = "场所类型，0：通用场所，1：预约场所")
    private Integer placeType = AdminPlaceTypeEnum.COMMON_PLACE.getCode();

    @ApiModelProperty("是否开启物理卡号：1-开启 2-不开启")
    private Integer cardPhysics;

    @ApiModelProperty(value = "自定义字段列表")
    private List<ContextVO>  contextVOList;

    @ApiModelProperty(value = "是否开启考试 1-开启 0-不开启")
    private Boolean isOpenExam;

    @ApiModelProperty(value = "考试设备列表")
    private String  examDevices;

    private Integer faceLogHoldDays;

    @ApiModelProperty(value = "是否开启进出预警 false-不开启 true-开启")
    private Boolean isOpenInoutWarning;

    @ApiModelProperty(value = "是否开启进出记录统计 false-不开启 true-开启")
    private Boolean openInOutStatistics;

}

