package com.yx.face.service.component.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FaceDeviceDetailDao;
import com.yx.face.dao.FaceDeviceTypeDao;
import com.yx.face.dao.FacePassDao;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.FaceDeviceDetail;
import com.yx.face.model.entity.FaceDeviceType;
import com.yx.face.model.entity.FacePass;
import com.yx.face.model.enums.FaceDeviceTypeEnum;
import com.yx.face.model.vo.UserInfoVO;
import com.yx.face.server.HandheldTerminalsServer;
import com.yx.face.server.dto.BindCompanyDTO;
import com.yx.face.server.dto.DelPersonDTO;
import com.yx.face.server.vo.BindCompanyVO;
import com.yx.face.server.vo.ResultVO;
import com.yx.face.service.UserInfoService;
import com.yx.face.service.component.HandheldTerminalsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/4/19 15:55
 * @description:
 */
@Component
public class HandheldTerminalsServiceImpl implements HandheldTerminalsService {
    @Autowired
    private HandheldTerminalsServer handheldTerminalsServer;
    @Resource
    private FaceDeviceTypeDao faceDeviceTypeDao;
    @Resource
    private FaceDeviceDetailDao faceDeviceDetailDao;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Resource
    private FacePassDao facePassDao;
    @Resource
    private UserInfoService userInfoService;

    /**
     * 异步删除手持终端人脸信息
     */
    @Async
    @Override
    public void delPerson(String sn) {
        FaceDevice faceDevice = faceDeviceDao.selectOne(new FaceDevice().setSn(sn).setEnable(1).setAuth(1));
        if (faceDevice == null) {
            return;
        }
        FacePass facePass = new FacePass();
        facePass.setDeviceId(faceDevice.getId());
        List<FacePass> passList = facePassDao.select(facePass);
        if (CollectionUtil.isEmpty(passList)) {
            return;
        }
        List<Long> userIds = passList.stream().map(FacePass::getUserId).collect(Collectors.toList());
        this.delPerson(sn, faceDevice, userIds);
    }

    @Async
    @Override
    public void delPerson(String sn, List<Long> userIds) {
        this.delPerson(sn, null, userIds);
    }

    @Async
    @Override
    public void delPerson(String sn, FaceDevice faceDevice, List<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return;
        }

        if (faceDevice == null) {
            faceDevice = faceDeviceDao.selectOne(new FaceDevice().setSn(sn).setEnable(1).setAuth(1));
        }

        if (faceDevice == null) {
            return;
        }

        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectByPrimaryKey(faceDevice.getFaceDeviceTypeId());
        if (!FaceDeviceTypeEnum.HANDHELD_TERMINALS.name().equals(faceDeviceType.getCode())) {
            return;
        }

        String companyId = null;
        //查询设备单位编号信息
        FaceDeviceDetail faceDeviceDetail = faceDeviceDetailDao.selectOne(new FaceDeviceDetail().setFaceDeviceId(faceDevice.getId()));
        if (faceDeviceDetail == null || StrUtil.isEmpty(faceDeviceDetail.getCompanyId())) {
            ResultVO resultVO = handheldTerminalsServer.bindCompany(new BindCompanyDTO().setCondition(faceDevice.getSn().replaceAll("hh_", "")));
            if (null != resultVO.getErrCode() && !resultVO.getErrCode().equals(1)) {
                return;
            }

            List<BindCompanyVO> companyList = resultVO.getContent();
            if (CollectionUtil.isEmpty(companyList)) {
                return;
            }

            companyId = companyList.stream().findFirst().get().getCompany_id();
            if (StrUtil.isEmpty(companyId)) {
                return;
            }

            if (faceDeviceDetail == null) {
                faceDeviceDetailDao.insert(new FaceDeviceDetail().setFaceDeviceId(faceDevice.getId()).setCompanyId(companyId));
            } else {
                faceDeviceDetailDao.updateByPrimaryKey(faceDeviceDetail.setCompanyId(companyId));
            }
        } else {
            companyId = faceDeviceDetail.getCompanyId();
        }
        List<UserInfoVO> userInfos = userInfoService.getUserInfoByUserIds(userIds);
        for (UserInfoVO userInfoVO : userInfos) {
            if (StrUtil.isEmpty(userInfoVO.getIdNumber())) {
                continue;
            }
            handheldTerminalsServer.delPerson(new DelPersonDTO().setCompanyId(companyId).setCardId(userInfoVO.getIdNumber()));
        }
    }
}
