package com.yx.face.service.dataFix.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.Base64Util;
import com.yx.face.boot.uitls.Sm4Util;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FaceLogDao;
import com.yx.face.dao.UserWhitelistDao;
import com.yx.face.model.dto.FaceLogDataPushDto;
import com.yx.face.model.dto.UserVisitorListDetailInfo;
import com.yx.face.model.dto.UserVisitorListDetailVisitor;
import com.yx.face.model.dto.dataFix.ClockWayPushVO;
import com.yx.face.model.entity.*;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.FaceService;
import com.yx.face.service.dataFix.DataFixService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.List;

/**
 * @author shisl
 * @package com.yx.face.service.dataFix.impl
 * @class DataFixServiceImpl
 * @date 2023/8/17 上午10:44
 * @description
 */
@Service
@Slf4j
public class DataFixServiceImpl implements DataFixService {

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private AdminService adminService;

    @Resource
    private FaceLogDao faceLogDao;

    @Resource
    private UserWhitelistDao userWhitelistDao;

    @Resource
    private FaceService faceService;

    @Override
    public RestResult<Boolean> faceLogDataPush(FaceLogDataPushDto dto) {
        if (dto.getSn() == null || dto.getPushUrl() == null || dto.getPrivateKey() == null) {
            throw new CommonException("参数有误，请检查后提交");
        }
        FaceDevice faceDeviceBySn = faceDeviceDao.getFaceDeviceBySn(dto.getSn());
        if (faceDeviceBySn == null) {
            throw new CommonException("设备不存在，请联系管理员");
        }
        AdminVO adminInfo = adminService.getAdminInfoById(faceDeviceBySn.getAdminId());
        List<FaceLog> list = faceLogDao.selectByExample(Example.builder(FaceLog.class)
                .where(WeekendSqls.<FaceLog>custom().andEqualTo(FaceLog::getDeviceSn, dto.getSn())).build());
        if (list != null && list.size() > 0) {
            list.stream().forEach(item -> pushGeneralClockWay(item, adminInfo, faceDeviceBySn, dto));
        }
        log.info("---->设备{}已同步完成{}条数据", dto.getSn(), list != null ? list.size() : 0);
        return RestResponse.ok(Boolean.TRUE);
    }

    //通用打卡数据推送
    private void pushGeneralClockWay(FaceLog faceLog, AdminVO adminInfo, FaceDevice device, FaceLogDataPushDto dto) {
        try {
            String privateKey = dto.getPrivateKey();
            String clockWayPushUrl = dto.getPushUrl();

            ClockWayPushVO pushVO = BeanUtil.copyProperties(faceLog, ClockWayPushVO.class);

            String photo = pushVO.getPhoto();
            if (photo != null ){
                try {
                    pushVO.setPhoto(Base64Util.imgToBase64(photo));
                } catch (Exception e) {
                    log.error("图片base64转化异常");
                }
            }
            //补全地址
            pushVO.setProvinceName(adminInfo.getProvinceName());
            pushVO.setCityName(adminInfo.getCityName());
            pushVO.setAreaName(adminInfo.getAreaName());
            pushVO.setStreetName(adminInfo.getStreetName());
            pushVO.setAddress(adminInfo.getAddress());
            //补全设备地址
            pushVO.setDeviceAddress(device.getName());
            //补全访客机用户字段
            completeVisitorData(pushVO);
            //查询授权配置文件 加密
            pushVO.setCardid(pushVO.getCardid() != null ? Sm4Util.encrypt(privateKey, pushVO.getCardid()) : null);
            pushVO.setName(pushVO.getName() != null ? Sm4Util.encrypt(privateKey, pushVO.getName()) : null);
            pushVO.setPhone(pushVO.getPhone() != null ? Sm4Util.encrypt(privateKey, pushVO.getPhone()) : null);
            pushVO.setIdCard(pushVO.getIdCard() != null ? Sm4Util.encrypt(privateKey, pushVO.getIdCard()) : null);
            // 通用回调
            log.info("筛选之后的参数接口：{}", JSONObject.toJSONString(pushVO));
            String body = HttpUtil.post(clockWayPushUrl, JSONObject.toJSONString(pushVO), 3000);
            log.info("推送接口地址：{},响应内容：{}", clockWayPushUrl, body);
        } catch (Exception e) {
            log.error("上报失败，请联系管理人员{} ", e);
            e.printStackTrace();
        }

    }

    //补全访客机字段
    private void completeVisitorData(ClockWayPushVO pushVO) {
        try {
            log.info("--->补全访客机字段----");
            Integer type = 104;
            Integer adminId = pushVO.getAdminId();
            String cardId = pushVO.getCardid();
            List<UserWhitelist> userWhitelists = userWhitelistDao.selectByPhoneOrIdNumber(cardId, null, Arrays.asList(pushVO.getAdminId()));
            if (userWhitelists != null && userWhitelists.size() > 0) {
                type = 105;
            }
            pushVO.setRightno(type);
            UserVisitorListDetailInfo userListFrom = null;
            String s = "yyyy-MM-dd HH:mm:ss";
            SimpleDateFormat f = new SimpleDateFormat(s);
            String time = f.format(pushVO.getCreateTime());
            if (type == 104) {
                userListFrom = faceService.getUserListFrom(adminId, cardId, time);
            } else if (type == 105) {
                userListFrom = faceService.getUserListFromTwo(adminId, cardId, time);
            }

            if (userListFrom == null) {
                return;
            }
            UserVisitorListDetailVisitor visitor = userListFrom.getUserVisitorListDetailVisitor();
            if (visitor == null) {
                return;
            }
            pushVO.setCompany(visitor.getCompany());
            pushVO.setTemperature(visitor.getTemperature());
            pushVO.setCarno(visitor.getCarno());
            pushVO.setReason(visitor.getReason());
            pushVO.setTime(visitor.getTime());
            pushVO.setOrg(visitor.getOrg());
            pushVO.setVisiteeCompany(visitor.getVisiteeCompany());
            pushVO.setVisiteeXm(visitor.getVisiteeXm());
            pushVO.setVisiteeBm(visitor.getVisiteeBm());
            pushVO.setVisiteeRoom(visitor.getVisiteeRoom());
            pushVO.setPcsdm(visitor.getPcsdm());
            pushVO.setPcsname(visitor.getPcsname());
            pushVO.setMachineId(visitor.getMachineId());
        } catch (Exception e) {
            log.error("--->获取访客机字段失败", e);
        }
    }


}
