package com.yx.face.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Maps;
import com.yx.face.boot.core.SnowflakeIdWorker;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.dao.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.vo.UserWhitelistVO;
import com.yx.face.service.UserWhitelistService;
import com.yx.face.service.feign.TBDeviceFaceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @description:
 * @ClassName AsyncTriggerRetry
 * @Author WXG
 * @Date 2021/11/30 13:16
 */
@Service
@Slf4j
public class AsyncTriggerRetryImpl {

    @Autowired
    private TBDeviceFaceService tbDeviceFaceService;

    @Autowired
    private UserWhitelistService userWhitelistService;

    @Resource
    private UserInfoDao userInfoDao;

    @Resource
    private FacePassDao facePassDao;
    @Resource
    private UserWhitelistDao whitelistDao;

    @Resource
    private FaceTaskDao faceTaskDao;
    @Autowired
    private RedisUtil redisUtil;
    @Resource
    private UserWhitelistDao userWhitelistdao;
    @Resource
    private FaceRequestLogDao faceRequestLogDao;


    @Async
    public void send(Map<String, Object> params, FacePass facePass) {
        try {
            Map<String, Object> result = tbDeviceFaceService.downUser(params);
            log.info("下发人脸downFace = {}", result);
            if (result.get("msg") != null) {
                String msg = result.get("msg").toString();
                if (msg.contains("不在线")) {
                    //设备不在线 重试 3次
                    Boolean retry = triggerRetry(params);
                    if (!retry) {
                        facePass.setStatus(3);
                        facePassDao.updateByPrimaryKeySelective(facePass);
                    }

                }
            }
        } catch (Exception e) {
            log.info("下发失败 原因 {}", e.getMessage());
            //设备不在线 重试 3次
            Boolean retry = triggerRetry(params);
            if (!retry) {
                facePass.setStatus(3);
                facePassDao.updateByPrimaryKeySelective(facePass);
            }
        }
    }

    public Boolean triggerRetry(Map<String, Object> params) {
        Map<String, Object> result = new HashMap<>();
        int times = 1;
        while (times <= 3) {
            try {
                wait(1000 * 20);   // 休眠20秒
                result = tbDeviceFaceService.downUser(params);
                log.info("下发人脸重试downFace = {} 重试次数 {}", params, times);
                if (result.get("msg") != null) {
                    String msg = result.get("msg").toString();
                    if (msg.contains("不在线")) {
                        times++;
                    } else {
                        //代表成功
                        times = 6;
                    }
                } else {
                    times++;
                }
            } catch (Exception e) {
                times++;
                log.info("下发失败 原因 {}", e.getMessage());
            }
        }
        //成功
        if (times == 6) {
            return true;
        }
        return false;
    }

    //发送失败重试三次
    @Async
    public void delUserRetry(Map<String, Object> params) {
        Map<String, Object> result = new HashMap<>();
        int times = 1;
        while (times <= 3) {
            try {
                log.info("删除人脸重试delUser = {} 重试次数 {}", params, times);
                wait(1000 * 20);   // 休眠20秒
                result = tbDeviceFaceService.delUser(params);
                if (result.get("msg") != null) {
                    String msg = result.get("msg").toString();
                    if (msg.contains("不在线")) {
                        times++;
                    } else {
                        //代表成功
                        times = 6;
                    }
                } else {
                    times++;
                }
            } catch (Exception e) {
                times++;
                log.info("下发失败 原因 {}", e.getMessage());
            }
        }
    }

    /**
     * 常客名单补全
     *
     * @param userWhitelist //账号id
     * @param name //姓名
     * @param cardid  //身份账号
     * @param photo   //图片
     */
    public void whiteVisitorCertification(UserWhitelist userWhitelist, String cardid, String photo, String name) {
        if(StrUtil.isBlank(photo)){
            return;
        }
        userWhitelist.setAvatar(photo);
        userWhitelist.setName(StringUtils.hasText(name)?name:null);
        userWhitelist.setPushType(4);
        userWhitelist.setStatus(1);
        int update = whitelistDao.updateByPrimaryKeySelective(userWhitelist);
        if (update == 1) {
            Long userId = null;
            if(userWhitelist.getUserId() != null){
                UserInfo userInfo = userInfoDao.selectByPrimaryKey(userWhitelist.getUserId());
                if(userInfo != null){
                    userId = userWhitelist.getUserId();
                }
            }
            if(userId == null){
                userId = userInfoDao.getUserInfoPhoneAndIdNumber(cardid, null);
            }
            if (userId != null) {
                UserInfo userInfo = new UserInfo();
                userInfo.setIdNumber(cardid);
                userInfo.setAvatar(photo);
                userInfo.setUsername(StringUtils.hasText(name)?name:null);
                userInfo.setId(userId);
                userInfoDao.updateByPrimaryKeySelective(userInfo);
                userWhitelist.setUserId(userId);
                userWhitelistService.addSingleWhite(userWhitelist, 2);
            }
        }
    }
    public void whiteVisitorCertificationCardIdEx(UserWhitelist userWhitelist, String cardIdEx, String photo, String name) {
        if(StrUtil.isBlank(photo)){
            return;
        }
        userWhitelist.setAvatar(photo);
        userWhitelist.setName(StringUtils.hasText(name)?name:null);
        userWhitelist.setStatus(1);
        userWhitelist.setPushType(5);
        int update = whitelistDao.updateByPrimaryKeySelective(userWhitelist);
        if (update == 1) {
            Long userId = null;
            if(userWhitelist.getUserId() != null){
                UserInfo userInfo = userInfoDao.selectByPrimaryKey(userWhitelist.getUserId());
                if(userInfo != null){
                    userId = userWhitelist.getUserId();
                }
            }
            if(userId == null){
                userId = userInfoDao.getUserInfoIdByCardIdEx(cardIdEx, userWhitelist.getAdminId());
            }
            if (userId != null) {
                UserInfo userInfo = new UserInfo();
                userInfo.setAvatar(photo);
                userInfo.setUsername(StringUtils.hasText(name)?name:null);
                userInfo.setId(userId);
                userInfoDao.updateByPrimaryKeySelective(userInfo);
                userWhitelist.setUserId(userId);
                userWhitelistService.addSingleWhite(userWhitelist, 2);
            }
        }
    }
    @Async
    public void faceInsert(FaceDevice sn , Long userId , Date startTime , Date endTime){
        Date now = new Date();

        FacePass facePass = new FacePass();
        facePass.setUserId(userId);
        facePass.setDeviceSn(sn.getSn());
        facePass.setDeviceId(sn.getId());
        facePass.setStartTime(startTime);
        facePass.setEndTime(endTime);
        facePass.setStatus(2);
        facePass.setType(3);
        facePass.setTaskId(String.valueOf(SnowflakeIdWorker.getSnowId()));
        facePass.setUpdateTime(now);
        facePass.setCreateTime(now);
        facePass.setAdminId(sn.getAdminId());
        facePassDao.insertSelective(facePass);


        Long faceTaskId = faceTaskDao.getFaceTaskId(sn.getSn(), userId);
        FaceTask faceTask = new FaceTask();
        faceTask.setId(faceTaskId);
        faceTask.setTaskTime(endTime);
        faceTask.setStatus(0);
        faceTask.setUpdateTime(now);
        if (faceTaskId == null) {
            faceTask.setDeviceSn(sn.getSn());
            faceTask.setUserId(userId);
            faceTask.setCreateTime(now);
            faceTaskDao.insertSelective(faceTask);
        }else {
            faceTaskDao.updateByPrimaryKeySelective(faceTask);
        }
    }

    @Async
    public void addSnIssued(FaceDevice faceDevice) {

        List<Map<String, Object>> data1 = new ArrayList<>();
        redisUtil.del("shishi");
        Date now = new Date();
        Date startTime  ;
        Date endTime ;
        int rightno = 105;//角色类型 104：访客 | 105：常客 | 101：黑名单
        //有效开始时间
        startTime = DateUtils.addMinutes(now, -10);//减10分钟 确保当前时间不会因为波动而没有存进去
        //有效结束时间
        endTime = DateUtils.addYears(now, 1);
        List<UserWhitelistVO> list = userWhitelistdao.selectList(faceDevice.getAdminId());
        for (UserWhitelistVO userWhitelist : list) {
            Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(9);
            Long userId = null;
            if (userWhitelist.getUserId() != null) {
                userId = userWhitelist.getUserId();
            } else {
                continue;
            }

            dataItem.put("pin", userId);
            dataItem.put("name", userWhitelist.getName());
            dataItem.put("validPeriodStart", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, startTime));
            dataItem.put("validPeriodEnd", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, endTime));
            dataItem.put("rightno", rightno);
            dataItem.put("userType", rightno == 104 ? 0 : 3);
            dataItem.put("opt", 1);
            dataItem.put("photo", userWhitelist.getAvatar());
            dataItem.put("cardid", userWhitelist.getIdNumber());

            data1.add(dataItem);
            faceInsert(faceDevice, userId, startTime, endTime);
        }
        redisUtil.set("shishi", JSON.toJSONString(data1));

        FaceRequestLog requestLog = intoFaceRequestLog("4.9远程触发分批同步人员信息","/sendControl");


        Map<String, Object> map = Maps.newHashMap();
        Map<String, Object> data = Maps.newHashMap();
        data.put("syncUsersType", "0");
        map.put("SN", faceDevice.getSn());
        map.put("taskId", requestLog.getId());
        map.put("type", "syncUsers");
        map.put("Data", data);

        try {
            Map<String, Object> sendControl = tbDeviceFaceService.sendControl(map);
            log.info("发送分批同步请求返回参数；{}", sendControl);
        }catch (Exception e) {
            log.info("下发失败 原因 {}", e.getMessage());
        }

    }
    private FaceRequestLog intoFaceRequestLog(String name , String context){
        FaceRequestLog requestLog = new FaceRequestLog();
        requestLog.setName(name);
        requestLog.setContext(context);
        requestLog.setParams("");
        requestLog.setStatus(1);
        requestLog.setUsedTime(0L);
        Date now = new Date();
        requestLog.setUpdateTime(now);
        requestLog.setCreateTime(now);
        faceRequestLogDao.insertSelective(requestLog);
        return requestLog;
    }
}
