package com.yx.face.service.impl;


import com.yx.face.dao.FaceLogDao;
import com.yx.face.dao.UserVisitorListDao;
import com.yx.face.dao.UserWhitelistDao;
import com.yx.face.model.dto.UserVisitorListDetailInfo;
import com.yx.face.model.dto.UserVisitorListDetailVisitor;
import com.yx.face.model.entity.*;
import com.yx.face.service.FaceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;


/**
 * @description: FaceServiceImpl <br>
 * @date: 2021/6/2 14:49 <br>
 * @author: PWB <br>
 */
@Slf4j
@Service
public class FaceServiceImpl implements FaceService {

    @Resource
    private FaceLogDao faceLogDao;

    @Resource
    private UserVisitorListDao userVisitorListDao;

    @Resource
    private UserWhitelistDao userWhitelistDao;

    @Override
    public UserVisitorListDetailInfo getUserListFrom(Integer adminId, String cardId, String time) {
        Integer id = faceLogDao.getContextByFaceLogVisitor(adminId, cardId);
        if (id == null) {
            return new UserVisitorListDetailInfo();
        }
        UserVisitorList userVisitorList = userVisitorListDao.selectByPrimaryKey(id);
        if (userVisitorList == null) {
            return new UserVisitorListDetailInfo();
        }
        UserVisitorListDetail userVisitorListDetail = userVisitorListDao.selectVisitorFromById(id, time);
        return getVisitorListFrom(userVisitorListDetail, userVisitorList);
    }

    @Override
    public UserVisitorListDetailInfo getUserListFromTwo(Integer adminId, String cardId, String time) {
        Integer id = faceLogDao.getContextByFaceLogWhite(adminId, cardId);
        if (id == null) {
            return new UserVisitorListDetailInfo();
        }
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(id);
        if (userWhitelist == null) {
            return new UserVisitorListDetailInfo();
        }
        UserWhitelistDetail userWhitelistDetail = userWhitelistDao.selectWhiteListFromById(id, time);
        return getWhiteListFrom(userWhitelistDetail, userWhitelist);
    }

    private UserVisitorListDetailInfo getWhiteListFrom(UserWhitelistDetail userWhitelistDetail, UserWhitelist userWhitelist) {
        if (userWhitelistDetail == null) {
            userWhitelistDetail = new UserWhitelistDetail();
        }
        UserVisitorListDetailInfo userVisitorListDetailInfo = new UserVisitorListDetailInfo();
        //2.访客机字段
        if (userWhitelist.getPushType() != null && userWhitelist.getPushType() == 3) {
            userVisitorListDetailInfo.setVisitorStatus(1);
            UserVisitorListDetailVisitor userVisitorListDetailVisitor = new UserVisitorListDetailVisitor();
            BeanUtils.copyProperties(userWhitelistDetail, userVisitorListDetailVisitor);
            userVisitorListDetailInfo.setUserVisitorListDetailVisitor(userVisitorListDetailVisitor);
        }
        //3.健康码信息
        userVisitorListDetailInfo.setThreeCodeList(new ArrayList<>());
        return userVisitorListDetailInfo;
    }

    private UserVisitorListDetailInfo getVisitorListFrom(UserVisitorListDetail userVisitorListDetail, UserVisitorList userVisitorList) {
        if (userVisitorListDetail == null) {
            userVisitorListDetail = new UserVisitorListDetail();
        }
        UserVisitorListDetailInfo userVisitorListDetailInfo = new UserVisitorListDetailInfo();
        userVisitorListDetailInfo.setContents(new ArrayList<>());
        //2.访客机字段
        if (userVisitorList != null && userVisitorList.getPushType() != null && userVisitorList.getPushType() == 3) {
            userVisitorListDetailInfo.setVisitorStatus(1);
            UserVisitorListDetailVisitor userVisitorListDetailVisitor = new UserVisitorListDetailVisitor();
            BeanUtils.copyProperties(userVisitorListDetail, userVisitorListDetailVisitor);
            userVisitorListDetailInfo.setUserVisitorListDetailVisitor(userVisitorListDetailVisitor);
        }
        userVisitorListDetailInfo.setThreeCodeList(new ArrayList<>());
        return userVisitorListDetailInfo;
    }


}
