package com.yx.face.service.impl;


import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FacePassDao;
import com.yx.face.dao.FaceRequestLogDao;
import com.yx.face.dao.FaceTaskDao;
import com.yx.face.model.entity.DownFaceNew;
import com.yx.face.model.entity.FacePass;
import com.yx.face.model.entity.FaceRequestLog;
import com.yx.face.model.entity.FaceTask;
import com.yx.face.model.vo.FaceDeviceLessVO;
import com.yx.face.service.FaceTBService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @description: FaceTBServiceImpl <br>
 * @date: 2021/6/3 10:40 <br>
 * @author: PWB <br>
 */
@Slf4j
@Service
public class FaceTBServiceImpl implements FaceTBService {
    @Resource
    private FacePassDao facePassDao;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Autowired
    @Lazy
    private AsyncTriggerRetryImpl asyncTriggerRetry;

    @Resource
    private FaceTaskDao faceTaskDao;
    @Lazy
    @Autowired
    RabbitTemplate rabbitTemplate;

    @Resource
    private FaceRequestLogDao faceRequestLogDao;

    @Async
    @Override
    public Map<String, Object> downUser(DownFaceNew downFace) {
        Map<String, Object> result = new HashMap<>();
        /*1.先进行数据的校验*/
        String msg = "";//存放错误信息
        if (downFace == null) {
            msg = "请传入参数";
            result.put("msg", msg);
            return result;
        }
        if (downFace.getSn() == null || downFace.getSn().equals("")) {
            msg = "设备编号不能为空";
            result.put("msg", msg);
            return result;
        }
        if (downFace.getUserId() == null) {
            msg = "唯一标识userId不能为空";
            result.put("msg", msg);
            return result;
        }
        if (downFace.getName() == null || downFace.getName().equals("")) {
            msg = "姓名不能为空";
            result.put("msg", msg);
            return result;
        }
        if (!StringUtils.hasText(downFace.getCardid()) && !StringUtils.hasText(downFace.getCardIdEx())) {
            msg = "身份证和物理卡号不能都为空";
            result.put("msg", msg);
            return result;
        }
        if (downFace.getRightno() == null) {
            msg = "权限级别不能为空";
            result.put("msg", msg);
            return result;
        }

        //当前时间
        Date now = new Date();
        //天波人脸服务请求日志表
        FaceRequestLog requestLog;
        //人脸下发表
        FacePass facePass;
        //唯一id
        Long faceTaskId;
        //人脸清除定时任务
        FaceTask faceTask;
        //人脸下发到设备里面
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(1);
        Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(8);
        FaceDeviceLessVO faceDeviceBySn = faceDeviceDao.getBySn(downFace.getSn());
        if (faceDeviceBySn == null) {
            msg = "设备编号不存在";
            result.put("msg", msg);
            return result;
        }
        /*2.根据角色的不同设置不同的有效时间*/
        Date startTime;
        Date endTime;
        Integer rightno = downFace.getRightno();//角色类型 104：访客 | 105：常客 | 101：黑名单
        //有效开始时间
        if (downFace.getValidPeriodStart() == null) {
            startTime = DateUtils.addMinutes(now, -10);//减10分钟 确保当前时间不会因为波动而没有存进去
        } else startTime = downFace.getValidPeriodStart();
        //有效结束时间
        if (downFace.getValidPeriodEnd() == null) {
            if (rightno == 104) endTime = DateUtils.addDays(now, 1);
            else if (rightno == 105) endTime = DateUtils.addYears(now, 1);
            else endTime = DateUtils.addYears(now, 100);
        } else endTime = downFace.getValidPeriodEnd();
        /*3.进行下发*/
        // 下发参数
        dataItem.put("pin", downFace.getUserId());
        dataItem.put("name", downFace.getName());
        dataItem.put("validPeriodStart", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, startTime));
        dataItem.put("validPeriodEnd", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, endTime));
        dataItem.put("rightno", rightno);
        dataItem.put("userType", rightno == 104 ? 0 : 3);
        if (org.apache.commons.lang3.StringUtils.isNotBlank(downFace.getPhoto())) {
            dataItem.put("photo", downFace.getPhoto());
        }
        dataItem.put("cardid", downFace.getCardid());
        dataItem.put("phone", downFace.getPhone());
        dataItem.put("cardIdEx", downFace.getCardIdEx());
        data.add(dataItem);
        params.put("SN", downFace.getSn());
        params.put("Data", data);

        //人脸下发表
        facePass = new FacePass();
        facePass.setUserId(downFace.getUserId());
        facePass.setDeviceSn(faceDeviceBySn.getSn());
        facePass.setDeviceId(faceDeviceBySn.getId());
        facePass.setStartTime(startTime);
        facePass.setEndTime(endTime);
        facePass.setAdminId(faceDeviceBySn.getAdminId());
        facePass.setStatus(2);
        facePass.setType(rightno == 105 ? 2 : 3);
        facePass.setTaskId(faceDeviceBySn.getAdminId().toString());
        facePass.setUpdateTime(now);
        facePass.setCreateTime(now);
        //添加人脸
        facePassDao.insertSelective(facePass);
        /*4.天波人脸服务请求日志表*/
        requestLog = intoFaceRequestLog("下发人脸", "/downUser");

        params.put("taskId", requestLog.getId());
        /*人脸下发的几种情况*/
        /*1.天波人脸下发*/
//        if (faceDeviceBySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.TB.getName()))
            asyncTriggerRetry.send(params, facePass);
            /*2.手持终端人脸下发*/
//        else if (faceDeviceBySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getName()))
//            handheldTerminalsImpl.send(params, facePass, faceDeviceBySn.getId());

        faceTaskId = faceTaskDao.getFaceTaskId(faceDeviceBySn.getSn(), downFace.getUserId());
        faceTask = new FaceTask();
        faceTask.setId(faceTaskId);
        faceTask.setTaskTime(facePass.getEndTime());
        faceTask.setStatus(0);
        faceTask.setUpdateTime(now);
        if (faceTaskId == null) {
            faceTask.setDeviceSn(faceDeviceBySn.getSn());
            faceTask.setUserId(downFace.getUserId());
            faceTask.setCreateTime(now);
            faceTaskDao.insertSelective(faceTask);
        } else {
            faceTaskDao.updateByPrimaryKeySelective(faceTask);
        }
        result.put("data", "命令下发成功");
        return result;
    }
    private FaceRequestLog intoFaceRequestLog(String name , String context){
        FaceRequestLog requestLog = new FaceRequestLog();
        requestLog.setName(name);
        requestLog.setContext(context);
        requestLog.setParams("");
        requestLog.setStatus(1);
        requestLog.setUsedTime(0L);
        Date now = new Date();
        requestLog.setUpdateTime(now);
        requestLog.setCreateTime(now);
        faceRequestLogDao.insertSelective(requestLog);
        return requestLog;
    }
}

