package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.dao.InOutStatisticsDao;
import com.yx.face.model.entity.InOutStatistics;
import com.yx.face.model.redis.InOutStatisticsBean;
import com.yx.face.service.InOutStatisticsService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Slf4j
@Service
@AllArgsConstructor(onConstructor_ = @Autowired)
public class InOutStatisticsServiceImpl implements InOutStatisticsService {

    private final InOutStatisticsDao inOutStatisticsDao;

    private final RedisUtil redisUtil;

    /**
     * 将当天的进出记录统计从redis中刷新到数据库
     */
    @Override
    public void refreshStatisticsCache() {
        long yesterdayTimestamp = System.currentTimeMillis() - 1000 * 3600 * 24;
        Date yesterday = new Date(yesterdayTimestamp);
        List<String> allByKey = redisUtil.getAllByKey(Constants.IN_OUT_STATISTICS);
        if(CollUtil.isEmpty(allByKey)){
            return;
        }
        for (String key : allByKey) {
            InOutStatisticsBean bean = (InOutStatisticsBean)redisUtil.get(key);
            if(bean == null){
                continue;
            }
            int adminId = Integer.parseInt(key.substring(key.lastIndexOf(":") + 1));

            InOutStatistics inOutStatistics = BeanUtil.copyProperties(bean, InOutStatistics.class);
            inOutStatistics.setStatisticDate(yesterday);
            inOutStatistics.setAdminId(adminId);
            inOutStatistics.setCreateTime(new Date());
            inOutStatisticsDao.insert(inOutStatistics);
            redisUtil.del(key);
        }
    }
}
