package com.yx.face.service.impl;

import cn.hutool.core.lang.Validator;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.uitls.OkHttpUtils;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.model.entity.UserVisitorListDetail;
import com.yx.face.model.enums.WXMessageTemplateEnum;
import com.yx.face.service.SmsService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;


/**
 * @description:
 * @ClassName SmsServiceImpl
 * @Author WXG
 * @Date 2022/6/7 18:58
 */
@Service
@Slf4j
public class SmsServiceImpl implements SmsService {

    private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    @Autowired
    private RedisUtil redisUtil;

    @Resource
    private UserInfoDao userInfoDao;

    public void getToken(){
        String appid = "wxa24c078a1704af94";
        String secret = "f04c3cb8fc552c1d7b68a2e282e95422";
        String grant_type = "client_credential";

        String sync = OkHttpUtils.builder().url("https://api.weixin.qq.com/cgi-bin/token")
                .addParam("grant_type",grant_type)
                .addParam("appid",appid)
                .addParam("secret",secret)
                .addHeader("Content-Type","application/x-www-form-urlencoded")
                .get()
                .sync();
        JSONObject jsonData = JSON.parseObject(sync);
        log.info("返回值；" + jsonData);
        if (jsonData != null) {
            redisUtil.set("smsToken", jsonData.getString("access_token"), jsonData.getInteger("expires_in")-200);
        }
    }
    @Override
    @Async
    public void createUrl(UserVisitorListDetail userVisitorListDetail){
        if (!redisUtil.hasKey("smsToken")) this.getToken();
        String smsToken = (String) redisUtil.get("smsToken");

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("path","pages/visitor_make/visitor_make");
        jsonObject.put("query","userVisitorDetailId=" + userVisitorListDetail.getId());
        jsonObject.put("env_version","release");//正式版
//        jsonObject.put("env_version","trial");//体验版
//        jsonObject.put("env_version","develop");//开发板
        jsonObject.put("expire_type","0");
        jsonObject.put("expire_time",userVisitorListDetail.getEndTime().getTime());

        String sync = OkHttpUtils.builder().url("https://api.weixin.qq.com/wxa/generate_urllink?access_token="+smsToken)
                .addHeader("Content-Type","application/json")
                .postString(JSONObject.toJSONString(jsonObject))
                .sync();
        if (sync.contains("access_token is invalid")){
            log.info("重新获取token");
            this.getToken();
            smsToken = (String) redisUtil.get("smsToken");
            sync = OkHttpUtils.builder().url("https://api.weixin.qq.com/wxa/generate_urllink?access_token="+smsToken)
                    .addHeader("Content-Type","application/json")
                    .postString(JSONObject.toJSONString(jsonObject))
                    .sync();
        }
        JSONObject jsonData = JSON.parseObject(sync);
        log.info("连接返回数据,{}",sync);
        if (jsonData != null) {
            if(jsonData.get("url_link") == null){
                log.info("{}生成短信邀请{}失败",userVisitorListDetail.getVisiteeXm(),userVisitorListDetail.getPhone());
            }else {
                String urlLink = jsonData.getString("url_link");
                String content = "您好，"+"楼宇综合管理"+"提醒您，您的好友"+userVisitorListDetail.getVisiteeXm()+"向您发送了一个访客邀请，点击链接打开小程序，接受邀请并填写访客信息，可享受"+userVisitorListDetail.getVisiteeCompany()+"区域快速通行，微信小程序链接： "+urlLink+" 。";

                String result = OkHttpUtils.builder().url("http://106.ihuyi.com/webservice/sms.php")
                        .addParam("account", "C23580469")
                        .addParam("password", "d7cbc559abd6f30561f35576bae49f66")
                        .addParam("mobile", userVisitorListDetail.getPhone())
                        .addParam("content", content)
                        .addParam("format", "json")
                        .addParam("method", "Submit")
                        .addHeader("Content-Type", "application/x-www-form-urlencoded")
                        .get()
                        .sync();
                JSONObject object = JSON.parseObject(result);
                if (object.get("smsid") == null) {
                    log.info("{}发送短信邀请{}失败", userVisitorListDetail.getVisiteeXm(), userVisitorListDetail.getPhone());
                }else {
                    log.info("短信发送成功：{}",result);
                }
            }
        }
    }

    @Override
    @Async
    public void subscribeMessage(String url, String phone, WXMessageTemplateEnum wxMessageTemplateEnum, String p1, String p2, String p3, String p4, String p5) {
        if (!redisUtil.hasKey("smsToken")) this.getToken();
        String smsToken = (String) redisUtil.get("smsToken");
        if(StringUtils.isEmpty(url)) url = "pages/resitor_scence/resitor_scence";
        String method = "【微信订阅消息发送】";
        log.info(method + "入参：【url：{}，phone：{}，template：{}，p1：{}，p2：{}，p3：{}，p4：{}，p5：{}】", url, phone, wxMessageTemplateEnum, p1, p2, p3, p4, p5);
        if (StringUtils.isEmpty(url) || StringUtils.isEmpty(phone) || wxMessageTemplateEnum == null) {
            log.error("必填参数都不能为空！");
            return;
        }
        Map<String, Object> paramMap = new HashMap<>();
        Map<String, Map<String, String>> data = new HashMap<>();
        paramMap.put("page", url);
        String oppenId = userInfoDao.selectByPhone(phone);
        if(StringUtils.isEmpty(oppenId)){
            return;
        }
        paramMap.put("touser", oppenId);//获取openId
        switch (wxMessageTemplateEnum) {
            case REVICE_APPLICATION://常客收到申请
                paramMap.put("template_id", WXMessageTemplateEnum.REVICE_APPLICATION.getTemplate());
                data.put("thing9", initMap(p1));//访客姓名
                data.put("phone_number6", initMap(p2));//联系电话
                data.put("thing10", initMap(p3));//来访单位
                data.put("thing7", initMap(p4));//备注
                break;
            case COMMIT_APPLICATION://访客申请提交
                if(!Validator.isMobile(p2)){
                    p2 = phone;
                }
                paramMap.put("template_id", WXMessageTemplateEnum.COMMIT_APPLICATION.getTemplate());
                data.put("thing9", initMap(p1));//访客姓名
                data.put("phone_number6", initMap(p2));//联系电话
                data.put("thing10", initMap(p3));//来访单位
                data.put("thing2", initMap(p4));//访客原由
                data.put("thing7", initMap(p5));//备注
                break;
            case COMMIT_RESULT://常客同意后受邀人收到的结果
                paramMap.put("template_id", WXMessageTemplateEnum.COMMIT_RESULT.getTemplate());
                data.put("thing14", initMap(p1));//申请人
                data.put("thing13", initMap(p2));//申请结果
                data.put("thing5", initMap(p3));//备注
                break;
            case INVITE_RESULT://受邀人接受邀请时 常客提醒
                paramMap.put("template_id", WXMessageTemplateEnum.INVITE_RESULT.getTemplate());
                data.put("name1", initMap(p1));//受邀者
                data.put("name2", initMap(p2));//邀请人
                data.put("thing3", initMap(p3));//邀请说明
                data.put("time10", initMap(p4));//预约时间
                data.put("thing6", initMap(p5));//备注
                break;
            case REMIND_REVICE_APPLICATION://提醒受邀人收到访客邀请
                paramMap.put("template_id", WXMessageTemplateEnum.REMIND_REVICE_APPLICATION.getTemplate());
                data.put("thing2", initMap(StringUtils.isEmpty(p1) ? "常客邀约": p1));//访客原由
                data.put("phone_number6", initMap(p2));//联系电话
                data.put("thing5", initMap(p3));//访客类型
                data.put("thing7", initMap(p4));//备注
                break;
            case COMPANY_COMMIT://用于企业申请提交告知用户
                paramMap.put("template_id", WXMessageTemplateEnum.COMPANY_COMMIT.getTemplate());
                data.put("thing1", initMap(p1));//公司名称
                data.put("phrase2", initMap(p2));//认证状态
                break;
            case COMPANY_RESULT://用于企业申请提交告知用户
                paramMap.put("template_id", WXMessageTemplateEnum.COMPANY_RESULT.getTemplate());
                data.put("thing1", initMap(p1));//公司名称
                data.put("phrase2", initMap(p2));//认证状态
                data.put("thing3", initMap(p3));//备注
                break;
            case VISITORY_IN_FIELD://用于企业申请提交告知用户
                paramMap.put("template_id", WXMessageTemplateEnum.VISITORY_IN_FIELD.getTemplate());
                data.put("name1", initMap(p1));//用户昵称
                data.put("thing3", initMap(p2));//备注
                break;
        }
        paramMap.put("miniprogram_state", "formal");//正式版
//        paramMap.put("miniprogram_state", "developer");//开发版
        paramMap.put("lang", "zh_CN");
        paramMap.put("data", data);
        System.out.println(JSONObject.toJSONString(paramMap));
        OkHttpUtils builder = OkHttpUtils.builder();
        String result = builder.url("https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token="+smsToken)
                .postString(JSONObject.toJSONString(paramMap))
                .sync();

        if (result.contains("access_token is invalid")){
            this.getToken();
            smsToken = (String) redisUtil.get("smsToken");
            result = builder.url("https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token="+smsToken)
                    .postString(JSONObject.toJSONString(paramMap))
                    .sync();
        }
        log.info(method + "结果为【" + result + "】");
        if (result != null) {
            try {
                JSONObject json = JSONObject.parseObject(result);
                if (!json.getString("errcode").equals("0")) {
                    log.error(method + "发送订阅消息出错【" + json.getString("errmsg") + "】");
                }
            } catch (Exception ex) {
                log.error(method + "解析返回结果时出错！", ex);
            }
        } else {
            log.error(method + "返回的结果为Null");
        }

    }
    private static Map<String,String> initMap(String info) {
        Map<String,String> map = new HashMap<>();
        map.put("value",info);
        return map;
    }
}
