package com.yx.face.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.UserVisitorListDetailDao;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.FaceLog;
import com.yx.face.model.entity.UserVisitorListDetail;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.enums.WXMessageTemplateEnum;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.service.SmsService;
import com.yx.face.service.UserVisitorListDetailService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.interceptor.AsyncExecutionAspectSupport;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName UserVisitorListServiceImpl
 * @Author cui
 * @Date 2021/11/5 14:16
 */
@Slf4j
@Service
public class UserVisitorListDetailServiceImpl implements UserVisitorListDetailService {
    @Resource
    private UserVisitorListDetailDao userVisitorListDetailDao;
    @Resource
    private AdminDao adminDao;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Resource
    private SmsService smsService;
    @Override
    @Async(AsyncExecutionAspectSupport.DEFAULT_TASK_EXECUTOR_BEAN_NAME)
    public void inField(AdminVO admin, FaceLog faceLog) {
        /*楼宇场景下的访客入场通知*/
        try{
            if(admin != null
                    && admin.getPlaceType().equals(AdminPlaceTypeEnum.BUILDING_PLACE.getCode())
                    && faceLog.getVerifyType() == 0
                    && faceLog.getUserId() > 0
            ){
                List<Integer> adminIdList = new ArrayList<>();
                adminIdList.add(admin.getAdminId());
                if(admin.getType().equals(AdminTypeEnum.DISTRICT.getCode())){
                    List<Admin> list = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, admin.getAdminId())).build());
                    adminIdList.addAll(list.stream().map(Admin::getId).collect(Collectors.toList()));
                }
                Date date = Convert.toDate(faceLog.getFaceTime());
                String faceTime = DateUtil.formatDateTime(date);
                List<UserVisitorListDetail> uvList  = userVisitorListDetailDao.selectInField(faceTime,adminIdList,faceLog.getUserId());
                if(CollUtil.isNotEmpty(uvList)){
                    for (int i = 0; i < uvList.size(); i++) {
                        UserVisitorListDetail userVisitorListDetail = uvList.get(i);
                        userVisitorListDetail.setReviewStatus(3);
                        userVisitorListDetail.setInTime(date);
                        userVisitorListDetail.setInPhoto(faceLog.getPhoto());
                        userVisitorListDetail.setSnAddr(faceDeviceDao.getFaceDeviceBySn(faceLog.getDeviceSn()).getName());
                        userVisitorListDetailDao.updateByPrimaryKeySelective(userVisitorListDetail);
                        /*发送消息订阅*/
                        smsService.subscribeMessage(null, userVisitorListDetail.getVisiteePhone(), WXMessageTemplateEnum.VISITORY_IN_FIELD,faceLog.getName(), "访客已入场", null, null,null);
                    }
                }
            }
        }catch (Exception e){
            log.error("楼宇场景下的访客入场通知异常",e);
        }

    }
}
