package com.yx.face.service.impl;


import cn.hutool.core.util.StrUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.boot.uitls.JsonUtils;
import com.yx.face.dao.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.AdminIsOpenPrivateAreaEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.enums.UserRightNoEnum;
import com.yx.face.model.vo.FaceDeviceLessVO;
import com.yx.face.model.vo.UserWhitelistVO;
import com.yx.face.service.FaceTBService;
import com.yx.face.service.UserWhitelistService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName UserWhitelistServiceImpl
 * @Author WXG
 * @Date 2021/11/5 14:16
 */
@Slf4j
@Service
public class UserWhitelistServiceImpl implements UserWhitelistService {
    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private UserWhitelistDao userWhitelistdao;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Resource
    private FacePassDao facePassDao;

    @Resource
    private FaceTaskDao faceTaskDao;
    @Autowired
    @Lazy
    private AsyncTriggerRetryImpl asyncTriggerRetryImpl;
    @Resource
    private FaceRequestLogDao faceRequestLogDao;
    @Resource
    private AdminDao adminDao;
    @Autowired
    @Lazy
    private FaceTBService faceTBService;
    public Boolean whiteVisitorType(Integer adminId) {
        String whiteVisitorType = userInfoDao.selectWhiteVisitorType(adminId.toString());
//      Boolean b = (StringUtils.isNotBlank(whiteVisitorType) && whiteVisitorType.contains("1") && !whiteVisitorType.contains("2"));
        Boolean b = (StringUtils.isNotBlank(whiteVisitorType) && whiteVisitorType.contains("1"));
        return b;
    }
    /**
     * 获取常客是否认证
     *
     * @param adminId
     * @param idNumber
     * @return
     */
    @Override
    public Integer getWhiteVisitorType(Integer adminId, String idNumber) {
        Integer status = userWhitelistdao.getWhiteVisitorType(adminId, idNumber);
        return status;
    }
    /**
     * @param userWhitelist
     * @param type          1   学校场景第一次下发 不携带人脸 2 补全信息下发人脸
     */
    public void addSingleWhite(UserWhitelist userWhitelist, int type) {
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userWhitelist.getUserId());
        //查询当前账号下所有设备
        List<String> list = faceDeviceDao.selectList(userWhitelist.getAdminId());
        //查询当前账号信息
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        if(admin.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.OPEN_PRIVATE_AREA.getCode())){
            List<FaceDeviceLessVO> snListByUserWhitelistId = faceDeviceDao.getSnListByUserWhitelistId(userWhitelist.getId());
            List<String> collect = snListByUserWhitelistId.stream().map(FaceDeviceLessVO::getSn).collect(Collectors.toList());
            if(list == null)list= new ArrayList<>();
            list.addAll(collect);
        }
        if (admin.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            //楼宇上级物业设备信息
            list.addAll(faceDeviceDao.selectList(admin.getTenantId()));
        }

        for (String sn : list) {
            DownFaceNew downFaceNew = new DownFaceNew();
            downFaceNew.setCardid(userInfo.getIdNumber());
            downFaceNew.setCardIdEx(userWhitelist.getCardIdEx());
            downFaceNew.setRightno(UserRightNoEnum.REGULAR.getCode());
            downFaceNew.setPhoto(userInfo.getAvatar());
            downFaceNew.setName(userInfo.getUsername());
            downFaceNew.setUserId(userInfo.getId());
            downFaceNew.setSn(sn);
            downFaceNew.setPhone(userInfo.getPhone());
            downFaceNew.setValidPeriodStart(userWhitelist.getStartTime());
            downFaceNew.setValidPeriodEnd(userWhitelist.getEndTime());
            faceTBService.downUser(downFaceNew);
        }
//        Boolean b = (StringUtils.isBlank(userWhitelist.getIdNumber()) && StringUtils.isBlank(userWhitelist.getPhone()));
//        if (b) {
//
//        }
//        Date now = new Date();
//        Date date = DateUtils.addYears(now, 1);
//        UserWhitelistVO userWhitelistVO = userWhitelistdao.selectUserWhitelistOn(userWhitelist.getIdNumber(), userWhitelist.getPhone());
//        if (Objects.isNull(userWhitelistVO)) {
//            userWhitelistVO = new UserWhitelistVO();
//        }
//        if (type == 1) {
//            userWhitelistVO.setAvatar("");
//        }
//        List<String> list = faceDeviceDao.selectList(userWhitelist.getAdminId());
//        if (list.isEmpty()) {
//
//        }
//        for (String s : list) {
//            userWhitelistVO.setSn(s);
//            userWhitelistVO.setAdminId(userWhitelist.getAdminId());
//            downFaceByTask(userWhitelistVO, DateUtils.addMinutes(now, -10), date, 2);
//        }

    }

    @Override
    public UserWhitelist getByPhoneAndIdNumber(String idNumber, String phone, String cardIdEx, Integer adminId) {
        UserWhitelist userWhitelist = null;
        if (StringUtils.isNotEmpty(idNumber)) {
            userWhitelist = userWhitelistdao.getByIdNumberAndId(idNumber, null, adminId);
            if (userWhitelist == null && StringUtils.isNotEmpty(phone)) {
                userWhitelist = userWhitelistdao.getByPhoneAndId(phone, null, adminId);
            }
        } else {
            if (StringUtils.isNotEmpty(phone)) {
                userWhitelist = userWhitelistdao.getByPhoneAndId(phone, null, adminId);
            }
        }
        if(userWhitelist == null && StrUtil.isNotEmpty(cardIdEx)){
            userWhitelist = userWhitelistdao.selectOne(new UserWhitelist().setCardIdEx(cardIdEx).setAdminId(adminId));
        }

        return userWhitelist;
    }

    /**
     * @param whitelistVO 对应数据
     * @param startTime   开始时间
     * @param endTime     结束时间
     * @param type        类型
     */
    @Transactional(rollbackFor = Exception.class)
    public Map<String, Object> downFaceByTask(UserWhitelistVO whitelistVO, Date startTime, Date endTime, Integer type) {
        Map<String, Object> result = new HashMap<>();
        //天波人脸服务请求日志表
        FaceRequestLog requestLog;
        Date now = new Date();
        //人脸下发表
        FacePass facePass;
        //唯一id
        Long faceTaskId;
        //人脸清除定时任务
        FaceTask faceTask;
        //信息
        String msg;
        //人脸下发到设备里面
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(1);
        Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(8);
        FaceDevice faceDeviceBySn = faceDeviceDao.getFaceDeviceBySn(whitelistVO.getSn());
        // 下发参数
        dataItem.put("pin", whitelistVO.getId());
        dataItem.put("name", whitelistVO.getName());
        dataItem.put("validPeriodStart", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, startTime));
        dataItem.put("validPeriodEnd", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, endTime));
        dataItem.put("userType", 0);
        if (StringUtils.isNotBlank(whitelistVO.getAvatar())) {
            dataItem.put("photo", whitelistVO.getAvatar());
        }
        dataItem.put("rightno", 105);
        dataItem.put("cardid", whitelistVO.getIdNumber());
        data.add(dataItem);
        params.put("SN", whitelistVO.getSn());
        params.put("Data", data);
        //天波人脸服务请求日志表
        requestLog = new FaceRequestLog();
        requestLog.setName("下发人脸");
        requestLog.setContext("/downUser");
        requestLog.setParams(JsonUtils.toJson(params));
        requestLog.setStatus(1);
        requestLog.setUsedTime(0L);
        requestLog.setUpdateTime(now);
        requestLog.setCreateTime(now);
        //人脸下发记录表
        faceRequestLogDao.insertSelective(requestLog);
        params.put("taskId", requestLog.getId());
        //人脸下发表
        facePass = new FacePass();
        facePass.setUserId(whitelistVO.getId());
        facePass.setDeviceSn(faceDeviceBySn.getSn());
        facePass.setDeviceId(faceDeviceBySn.getId());
        facePass.setStartTime(startTime);
        facePass.setEndTime(endTime);
        facePass.setAdminId(whitelistVO.getAdminId());
        facePass.setStatus(2);
        facePass.setType(type);
        facePass.setTaskId(whitelistVO.getAdminId().toString());
        facePass.setUpdateTime(now);
        facePass.setCreateTime(now);
        //添加人脸
        facePassDao.insertSelective(facePass);
        asyncTriggerRetryImpl.send(params, facePass);
        faceTaskId = faceTaskDao.getFaceTaskId(whitelistVO.getSn(), whitelistVO.getId());
        faceTask = new FaceTask();
        faceTask.setId(faceTaskId);
        faceTask.setTaskTime(facePass.getEndTime());
        faceTask.setStatus(0);
        faceTask.setUpdateTime(now);
        if (faceTaskId == null) {
            faceTask.setDeviceSn(whitelistVO.getSn());
            faceTask.setUserId(whitelistVO.getId());
            faceTask.setCreateTime(now);
            faceTaskDao.insertSelective(faceTask);
        }
        faceTaskDao.updateByPrimaryKeySelective(faceTask);
        return result;
    }
}
