package com.yx.face.service.job;

import cn.hutool.core.collection.CollectionUtil;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.model.entity.TaskInfo;
import com.yx.face.service.TaskInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author lejingjing
 * @version 1.0
 * @Description:
 * @date 2022/10/14
 **/
@Component
@EnableScheduling   
@Slf4j
public class ClearOfflineHistoryTask {

    @Resource
    private TaskInfoService taskInfoService;

    @Resource
    private RedisUtil redisUtil;

    @Scheduled(cron = "0 0 1 * * ?", zone = "Asia/Shanghai")
    @Async
    public void clearHistoryTask() {
        log.info("----------开始------清理过期1个月的任务-------------");
        List<TaskInfo> overDueTaskList = taskInfoService.getOverdueTask();
        Integer count = 0;
        log.info("一个月前的任务有{}个过期任务",overDueTaskList.size());
        if(CollectionUtil.isNotEmpty(overDueTaskList)){
            List<String> keys = overDueTaskList.stream().filter(e-> Objects.nonNull(e.getBody()))
                   .map(TaskInfo::getBody).collect(Collectors.toList());
            keys.forEach(e->{
                redisUtil.del(e);
                log.info("redis key #{}#已经过期被清理",e);
                    });
            count = taskInfoService.batchDelete(overDueTaskList);
        }
        log.info("已经清理了{}个过期任务",count);
        if(count.equals(overDueTaskList.size())){
            log.info("定时清理过期任务成功");
        }else{
            log.error("定时清理过期任务失败");
        }
        log.info("----------结束------清理过期1个月的任务-------------");
    }

}
