package com.yx.face.service.job;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yx.face.dao.*;
import com.yx.face.model.entity.FaceRequestLog;
import com.yx.face.model.entity.FaceTask;
import com.yx.face.service.feign.TBDeviceFaceService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: faceTask <br>
 * @date: 2021/6/8 20:39 <br>
 * @author: PWB <br>
 */
@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
@EnableAsync
@Slf4j
public class FaceServiceTask {

    @Resource
    private FaceTaskDao faceTaskDao;
    @Resource
    private FaceRequestLogDao faceRequestLogDao;
    @Autowired
    private TBDeviceFaceService tbDeviceFaceService;
    @Resource
    private UserBlackListDao userBlackListDao;
    @Resource
    private UserWhitelistDao userWhitelistDao;
    @Resource
    private UserVisitorListDao userVisitorListDao;
    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Scheduled(cron = "${cron.face.clear.user}", zone = "Asia/Shanghai")
    @Async
    public void doDeleteFaceUser() {
        log.info("----------开始------执行人脸过期清除计划-------------");
        Date now = new Date();
        List<FaceTask> taskList = faceTaskDao.getFaceTaskList(now);
        if (CollectionUtils.isEmpty(taskList)) {
            log.info("----------完成------执行人脸过期清除计划-------------");
            return;
        }
        //分组
        Map<String, List<FaceTask>> snMap = taskList.stream().collect(Collectors.groupingBy(FaceTask::getDeviceSn));
        // 1. entrySet遍历，在键和值都需要时使用（最常用）
        Map<String, Object> params;
        List<Map<String, Object>> data;
        FaceRequestLog requestLog;
        Map<String, Object> userMap;
        List<FaceTask> tasks;
        Map<String, Object> result;
        List<Long> userIds;
        for (Map.Entry<String, List<FaceTask>> entry : snMap.entrySet()) {
            params = Maps.newHashMap();
            tasks = entry.getValue();
            data = Lists.newArrayListWithExpectedSize(tasks.size());
            userIds = Lists.newArrayListWithExpectedSize(tasks.size());
            for (FaceTask task : tasks) {
                userMap = Maps.newHashMap();
                userMap.put("pin", task.getUserId());
                data.add(userMap);
                userIds.add(task.getUserId());
            }
            /*手持终端不走天波的删除逻辑*/
            if(entry.getKey().startsWith("hh_")){
                continue;
            }
            params.put("Data", data);
            params.put("SN", entry.getKey());
            requestLog = new FaceRequestLog();
            requestLog.setName("删除人员人脸信息");
            requestLog.setContext("/delUser");
            requestLog.setParams("");
            requestLog.setStatus(0);
            requestLog.setUsedTime(0L);
            requestLog.setUpdateTime(now);
            requestLog.setCreateTime(now);
            faceRequestLogDao.insertSelective(requestLog);
            params.put("taskId", requestLog.getId());
            try {
                Integer adminId = faceDeviceDao.selectAdminId(entry.getKey());
                if (adminId != null) {
                    //删除人脸
                    result = tbDeviceFaceService.delUser(params);
                    String msg = (String) result.get("msg");
                    if (StringUtils.equals("命令下发成功", msg)) {
                        //删除 常客
//                        List<Long> whiteIds = userWhitelistDao.selectId(userIds, adminId);
//                        if (!whiteIds.isEmpty()) {
//                            List<String> collect = whiteIds.stream().map(String::valueOf).collect(Collectors.toList());
//                            String join = String.join(",", collect);
//                            userWhitelistDao.deleteByIds(join);
//                        }
                        //删除 黑名单
                        List<Long> uBlackIdS = userBlackListDao.selectId(userIds, adminId);
                        if (!uBlackIdS.isEmpty()) {
                            List<String> collect = uBlackIdS.stream().map(String::valueOf).collect(Collectors.toList());
                            String join = String.join(",", collect);
                            userBlackListDao.deleteByIds(join);
                        }
                        //删除 访客
                        List<Long> uVisitorIdS = userVisitorListDao.selectId(userIds, adminId);
                        if (!uVisitorIdS.isEmpty()) {
                            List<String> collect = uVisitorIdS.stream().map(String::valueOf).collect(Collectors.toList());
                            String join = String.join(",", collect);
                            userVisitorListDao.deleteByIds(join);
                        }
                        //修改定时任务表
                        faceTaskDao.updateFaceTaskStatus(entry.getKey(), userIds);
                    }
                    //获取 设备基本信息
                    log.info("删除结果={}", result);
                }
            } catch (Exception e) {
                log.info("删除人脸异常 {}", e.getMessage());
            }
        }
        log.info("----------完成------执行人脸过期清除计划-------------");
    }

}
