package com.yx.face.service.job;

import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FkjHeartTrackingDao;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.FkjHeartTracking;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package com.yx.face.service.job
 * @class FkjDeviceTask
 * @date 2023/7/29 下午9:41
 * @description 访客机job
 */
@Configuration
@EnableScheduling
@EnableAsync
@Slf4j
public class FkjDeviceTask {

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private FkjHeartTrackingDao fkjHeartTrackingDao;

    /**
     * 每一分钟执行一次
     * 60秒未接收到心跳 访客机视为离线状态
     */
    @Scheduled(cron = "0 */2 * * * ?", zone = "Asia/Shanghai")
    public void opDeviceOffline() {
        log.info("--->开始执行访客机心跳监测");
        Date currentTime = new Date();
        Date lastTrackingTime = DateUtils.addSeconds(currentTime, -120);
        List<FkjHeartTracking> list = fkjHeartTrackingDao.selectByExample(Example.builder(FkjHeartTracking.class)
                .where(WeekendSqls.<FkjHeartTracking>custom().andLessThan(FkjHeartTracking::getLastTrackingTime, lastTrackingTime)).build());
        if (list == null || list.size() == 0) {
            log.info("--->完成执行访客机心跳监测，执行数{}", 0);
            return;
        }
        List<String> sns = list.stream().map(FkjHeartTracking::getSn).collect(Collectors.toList());
        List<FaceDevice> faceDevices = faceDeviceDao.selectByExample(Example.builder(FaceDevice.class)
                .where(WeekendSqls.<FaceDevice>custom().andIn(FaceDevice::getSn, sns).andEqualTo(FaceDevice::getOnline, 1)).build());
        if (faceDevices != null && faceDevices.size() > 0) {
            faceDevices.stream().forEach(d -> {
                d.setOnline(0);
                faceDeviceDao.updateByPrimaryKey(d);
            });
        }
        log.info("--->完成执行访客机心跳监测，执行数{}", (faceDevices == null || faceDevices.size() == 0) ? 0 : faceDevices.size());
    }

}
