package com.yx.face.service.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FaceDeviceTypeDao;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.FaceDeviceType;
import com.yx.face.model.enums.FaceDeviceTypeEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author lejingjing
 * @version 1.0
 * @Description:实时监听设备离线状态
 * @date 2022/11/22
 **/
@Component
@EnableScheduling
@Slf4j
public class MonitorDeviceOnlineStatusJob {

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private FaceDeviceTypeDao faceDeviceTypeDao;

    @Scheduled(cron = "0 */1 * * * ?", zone = "Asia/Shanghai")
    @Async
    public void monitorDeviceOnlineStatus() {
        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectOneByExample(Example.builder(FaceDeviceType.class).where(WeekendSqls
                .custom().andEqualTo("code",FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())).build());
        Date offset30Seconds = DateUtil.offsetSecond(new Date(),-30).toJdkDate();
        if(faceDeviceType != null){
            List<FaceDevice> faceDeviceList = faceDeviceDao.selectByExample(Example.builder(FaceDevice.class).where(WeekendSqls.custom()
                    .andLessThan("reportTime",offset30Seconds).andEqualTo("faceDeviceTypeId",faceDeviceType.getId())).build());
            log.info("=======手持终端设备列表:{}======", JSON.toJSONString(faceDeviceList));
            if(CollectionUtil.isNotEmpty(faceDeviceList)){
                faceDeviceList.forEach(e->{
                    e.setOnline(0);
                    e.setUpdateTime(new Date());
                    faceDeviceDao.updateByPrimaryKey(e);
                });
            }
            log.info("=======手持终端设备状态更新结束======");
        }
    }
}
