package com.yx.face.service.job;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.core.Constants;
import com.yx.face.dao.SpiIotUserWhitelistDao;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.model.dto.BatchAddUser2IotDto;
import com.yx.face.model.dto.DelUserFromIotDto;
import com.yx.face.model.dto.IotUserWhitelistBizContentDto;
import com.yx.face.model.dto.IotUserWhitelistDto;
import com.yx.face.model.entity.DownFaceNew;
import com.yx.face.model.entity.SpiIotUserWhitelist;
import com.yx.face.model.entity.UserInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Component
@Slf4j
public class SpiIotUserWhitelistTask {

    @Resource
    private SpiIotUserWhitelistDao spiIotUserWhitelistDao;
    @Resource
    private UserInfoDao userInfoDao;

    @RabbitListener(queues = Constants.IOT_USER_QUEUE_NAME)
    public synchronized void spiIotUserWhitelist(String message) {
        log.info("----------开始iot设备用户同步-------------");
        JSONObject obj = JSONObject.parseObject(message);
        String bizType = obj.get("bizType").toString();
        if (Constants.IOT_MESSAGE_TYPE_addSingleUser2iot.equals(bizType)) {
            IotUserWhitelistDto dto = JSONObject.parseObject(obj.get("data").toString(), IotUserWhitelistDto.class);
            handleAddSingleUser2Iot(dto);
        } else if (Constants.IOT_MESSAGE_TYPE_delUserFromIot.equals(bizType)) {
            DelUserFromIotDto dto = JSONObject.parseObject(obj.get("data").toString(), DelUserFromIotDto.class);
            handleDelUserFromIot(dto.getSn(), dto.getUserIds());
        } else if (Constants.IOT_MESSAGE_TYPE_batchAddUser2Iot.equals(bizType)) {
            BatchAddUser2IotDto dto = JSONObject.parseObject(obj.get("data").toString(), BatchAddUser2IotDto.class);
            handleBatchAddUser2Iot(dto.getSn(), dto.getUsers());
        } else if (Constants.IOT_MESSAGE_TYPE_delBySn.equals(bizType)) {
            handleDelBySn(obj.get("data").toString());
        }
        log.info("----------完成iot设备用户同步-------------");
    }

    public void handleBatchAddUser2Iot(String sn, List<DownFaceNew> users) {
        try {
            log.info("---->handleBatchAddUser2Iot:{}, {}", sn, JSONObject.toJSONString(users));
            List<String> idNumbers = users.stream().map(DownFaceNew::getCardid).collect(Collectors.toList());
            //根据idNumber查询IOT人员
            List<SpiIotUserWhitelist> list = spiIotUserWhitelistDao.selectByExample(Example.builder(SpiIotUserWhitelist.class)
                    .where(WeekendSqls.<SpiIotUserWhitelist>custom()
                            .andIn(SpiIotUserWhitelist::getIdNumber, idNumbers)
                            .andEqualTo(SpiIotUserWhitelist::getIsDeleted, Boolean.FALSE)).build());
            //数据库不存在全量下发
            if (list == null || list.size() == 0) {
                for (DownFaceNew item : users) {
                    SpiIotUserWhitelist iot = new SpiIotUserWhitelist();
                    iot.setIdNumber(item.getCardid());
                    iot.setName(item.getName());
                    iot.setPhone(item.getPhone());
                    iot.setIsDeleted(Boolean.FALSE);
                    iot.setCreateTime(new Date());
                    iot.setUpdateTime(new Date());

                    IotUserWhitelistBizContentDto bizContentDto = new IotUserWhitelistBizContentDto();
                    bizContentDto.setSn(sn);
                    bizContentDto.setStartTime(item.getValidPeriodStart());
                    bizContentDto.setEndTime(item.getValidPeriodEnd());
                    iot.setBizContent(JSONObject.toJSONString(Arrays.asList(bizContentDto)));
                    spiIotUserWhitelistDao.insert(iot);
                }
                return;
            }

            List<String> dbIdNumbers = list.stream().map(SpiIotUserWhitelist::getIdNumber).collect(Collectors.toList());
            Map<String, SpiIotUserWhitelist> dbIdNumberMap = list.stream().collect(Collectors.toMap(SpiIotUserWhitelist::getIdNumber, a -> a, (k1, k2) -> k1));
            for (DownFaceNew item : users) {
                if (dbIdNumbers.contains(item.getCardid())) {
                    SpiIotUserWhitelist user = dbIdNumberMap.get(item.getCardid());
                    if (user.getBizContent() == null || "".equals(user.getBizContent().trim())) {
                        IotUserWhitelistBizContentDto bizContentDto = new IotUserWhitelistBizContentDto();
                        bizContentDto.setSn(sn);
                        bizContentDto.setStartTime(item.getValidPeriodStart());
                        bizContentDto.setEndTime(item.getValidPeriodEnd());
                        user.setBizContent(JSONObject.toJSONString(Arrays.asList(bizContentDto)));
                        user.setUpdateTime(new Date());
                        spiIotUserWhitelistDao.updateByPrimaryKey(user);
                    } else {
                        List<IotUserWhitelistBizContentDto> bizContentDtos = JSONObject.parseArray(user.getBizContent(), IotUserWhitelistBizContentDto.class);
                        List<String> sns = bizContentDtos.stream().map(IotUserWhitelistBizContentDto::getSn).collect(Collectors.toList());
                        if (sns.contains(sn)) {
                            bizContentDtos.stream().forEach(dbItem -> {
                                if (dbItem.getSn().equals(sn)) {
                                    dbItem.setStartTime(item.getValidPeriodStart());
                                    dbItem.setEndTime(item.getValidPeriodEnd());
                                }
                            });
                        } else {
                            IotUserWhitelistBizContentDto bizContentDto = new IotUserWhitelistBizContentDto();
                            bizContentDto.setSn(sn);
                            bizContentDto.setStartTime(item.getValidPeriodStart());
                            bizContentDto.setEndTime(item.getValidPeriodEnd());
                            bizContentDtos.add(bizContentDto);
                        }
                        user.setUpdateTime(new Date());
                        user.setBizContent(JSONObject.toJSONString(bizContentDtos));
                        spiIotUserWhitelistDao.updateByPrimaryKey(user);
                    }
                } else {
                    SpiIotUserWhitelist iot = new SpiIotUserWhitelist();
                    iot.setIdNumber(item.getCardid());
                    iot.setName(item.getName());
                    iot.setPhone(item.getPhone());
                    iot.setIsDeleted(Boolean.FALSE);
                    iot.setCreateTime(new Date());
                    iot.setUpdateTime(new Date());

                    IotUserWhitelistBizContentDto bizContentDto = new IotUserWhitelistBizContentDto();
                    bizContentDto.setSn(sn);
                    bizContentDto.setStartTime(item.getValidPeriodStart());
                    bizContentDto.setEndTime(item.getValidPeriodEnd());
                    iot.setBizContent(JSONObject.toJSONString(Arrays.asList(bizContentDto)));
                    spiIotUserWhitelistDao.insert(iot);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("--->handleBatchAddUser2Iot exception");
        }
    }

    public void handleDelBySn(String sn) {
        try {
            log.info("---->handleDelBySn:{}", sn);
            List<SpiIotUserWhitelist> list = spiIotUserWhitelistDao.selectByExample(Example.builder(SpiIotUserWhitelist.class)
                    .where(WeekendSqls.<SpiIotUserWhitelist>custom()
                            .andLike(SpiIotUserWhitelist::getBizContent, "%" + sn + "%")
                            .andEqualTo(SpiIotUserWhitelist::getIsDeleted, Boolean.FALSE)).build());
            if (list == null || list.size() == 0) {
                return;
            }

            for (SpiIotUserWhitelist item : list) {
                if (item.getBizContent() != null && !"".equals(item.getBizContent().trim())) {
                    List<IotUserWhitelistBizContentDto> temp = new ArrayList<>();
                    List<IotUserWhitelistBizContentDto> dbBizContents = JSONObject.parseArray(item.getBizContent(), IotUserWhitelistBizContentDto.class);
                    for (IotUserWhitelistBizContentDto biz : dbBizContents) {
                        if (!biz.getSn().equals(sn)) {
                            temp.add(biz);
                        }
                    }
                    item.setBizContent(JSONObject.toJSONString(temp));
                    item.setUpdateTime(new Date());
                    spiIotUserWhitelistDao.updateByPrimaryKey(item);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("--->handleDelBySn exception");
        }
    }

    public void handleDelUserFromIot(String sn, List<Long> userIds) {
        try {
            log.info("---->handleDelUserFromIot:{}, {}", sn, JSONObject.toJSONString(userIds));
            List<UserInfo> userInfos = userInfoDao.selectByExample(Example.builder(UserInfo.class).where(WeekendSqls.<UserInfo>custom().andIn(UserInfo::getId, userIds)).build());
            List<String> idNumbers = userInfos.stream().map(UserInfo::getIdNumber).collect(Collectors.toList());
            //根据idNumber查询IOT人员
            List<SpiIotUserWhitelist> list = spiIotUserWhitelistDao.selectByExample(Example.builder(SpiIotUserWhitelist.class)
                    .where(WeekendSqls.<SpiIotUserWhitelist>custom()
                            .andIn(SpiIotUserWhitelist::getIdNumber, idNumbers)
                            .andEqualTo(SpiIotUserWhitelist::getIsDeleted, Boolean.FALSE)).build());
            if (list == null || list.size() == 0) {
                return;
            }
            for (SpiIotUserWhitelist item : list) {
                if (item.getBizContent() != null && !"".equals(item.getBizContent().trim())) {
                    List<IotUserWhitelistBizContentDto> temp = new ArrayList<>();
                    List<IotUserWhitelistBizContentDto> dbBizContents = JSONObject.parseArray(item.getBizContent(), IotUserWhitelistBizContentDto.class);
                    for (IotUserWhitelistBizContentDto biz : dbBizContents) {
                        if (!biz.getSn().equals(sn)) {
                            temp.add(biz);
                        }
                    }
                    item.setBizContent(JSONObject.toJSONString(temp));
                    item.setUpdateTime(new Date());
                    spiIotUserWhitelistDao.updateByPrimaryKey(item);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("--->handleDelUserFromIot exception");
        }
    }

    public void handleAddSingleUser2Iot(IotUserWhitelistDto dto) {
        try {

            log.info("---->handleAddSingleUser2Iot:{}", JSONObject.toJSONString(dto));
            SpiIotUserWhitelist user = spiIotUserWhitelistDao.selectOneByExample(Example.builder(SpiIotUserWhitelist.class)
                    .where(WeekendSqls.<SpiIotUserWhitelist>custom()
                            .andEqualTo(SpiIotUserWhitelist::getIdNumber, dto.getIdNumber())
                            .andEqualTo(SpiIotUserWhitelist::getIsDeleted, Boolean.FALSE)).build());
            if (user == null) {
                user = BeanUtil.copyProperties(dto, SpiIotUserWhitelist.class);
                IotUserWhitelistBizContentDto bizContentDto = new IotUserWhitelistBizContentDto();
                bizContentDto.setSn(dto.getSn());
                bizContentDto.setStartTime(dto.getStartTime());
                bizContentDto.setEndTime(dto.getEndTime());
                user.setBizContent(JSONObject.toJSONString(Arrays.asList(bizContentDto)));
                user.setIsDeleted(Boolean.FALSE);
                user.setCreateTime(new Date());
                user.setUpdateTime(new Date());
                spiIotUserWhitelistDao.insert(user);
            } else {
                if (user.getBizContent() == null || "".equals(user.getBizContent().trim())) {
                    IotUserWhitelistBizContentDto bizContentDto = new IotUserWhitelistBizContentDto();
                    bizContentDto.setSn(dto.getSn());
                    bizContentDto.setStartTime(dto.getStartTime());
                    bizContentDto.setEndTime(dto.getEndTime());
                    user.setBizContent(JSONObject.toJSONString(Arrays.asList(bizContentDto)));
                    user.setUpdateTime(new Date());
                    spiIotUserWhitelistDao.updateByPrimaryKey(user);
                } else {
                    List<IotUserWhitelistBizContentDto> bizContentDtos = JSONObject.parseArray(user.getBizContent(), IotUserWhitelistBizContentDto.class);
                    List<String> sns = bizContentDtos.stream().map(IotUserWhitelistBizContentDto::getSn).collect(Collectors.toList());
                    if (sns.contains(dto.getSn())) {
                        bizContentDtos.stream().forEach(item -> {
                            if (item.getSn().equals(dto.getSn())) {
                                item.setStartTime(dto.getStartTime());
                                item.setEndTime(dto.getEndTime());
                            }
                        });
                    } else {
                        IotUserWhitelistBizContentDto bizContentDto = new IotUserWhitelistBizContentDto();
                        bizContentDto.setSn(dto.getSn());
                        bizContentDto.setStartTime(dto.getStartTime());
                        bizContentDto.setEndTime(dto.getEndTime());
                        bizContentDtos.add(bizContentDto);
                    }
                    user.setUpdateTime(new Date());
                    user.setBizContent(JSONObject.toJSONString(bizContentDtos));
                    spiIotUserWhitelistDao.updateByPrimaryKey(user);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("--->handleAddSingleUser2Iot exception");
        }
    }

}
