package com.yx.face.service.job;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.StatDataDao;
import com.yx.face.dao.StatPeopleDao;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.StatData;
import com.yx.face.model.entity.StatPeople;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.enums.StatDataStatusEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.List;

/**
 * 批量删除用户库用户
 */
@Configuration
@EnableScheduling
@EnableAsync
@Slf4j
public class StatDataTask {

    @Resource
    private AdminDao adminDao;
    @Resource
    private StatPeopleDao statPeopleDao;
    @Resource
    private StatDataDao statDataDao;

    /**
     * 每个小时的0/15/30/45执行
     */
    @Scheduled(cron = "0 0,15,30,45 * * * ? ", zone = "Asia/Shanghai")
    public void statData() {
        log.info("--->开始执行数据统计");
        LocalDate now = LocalDate.now();
        LocalTime nowTime = LocalTime.now();
        Admin admin = new Admin();
        admin.setOpenInOutStatistics(true);
        admin.setType(AdminTypeEnum.DISTRICT.getCode());
        admin.setPlaceType(AdminPlaceTypeEnum.COMMON_PLACE.getCode());
        List<Admin> adminList = adminDao.select(admin);
        for (Admin item : adminList) {
            Long count = statDataDao.selectCount(
                    Wrappers.<StatData>query()
                            .select("DISTINCT id_number")
                            .lambda()
                            .eq(StatData::getAdminId, item.getId())
                            .eq(StatData::getStatus, StatDataStatusEnum.IN.getCode())
            );
            StatPeople statPeople = new StatPeople();
            statPeople.setAdminId(item.getId());
            statPeople.setStatDate(now);
            statPeople.setStatTime(nowTime);
            statPeople.setStatNum(Math.toIntExact(count));
            statPeopleDao.insert(statPeople);
        }
        log.info("--->完成执行数据统计");
    }

}
