package com.yx.face.service.job;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Maps;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.uitls.AddressUtil;
import com.yx.face.boot.uitls.FileUtils;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.dao.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.PassEnum;
import com.yx.face.model.enums.UserRightNoEnum;
import com.yx.face.model.mq.StatDataMq;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.ContextVO;
import com.yx.face.model.vo.WhiteVisitorCertificationVO;
import com.yx.face.service.*;
import com.yx.face.service.impl.AsyncTriggerRetryImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.aop.interceptor.AsyncExecutionAspectSupport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.nio.charset.Charset;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName UserInfoTask
 * @Author WXG
 * @Date 2021/11/16 16:56
 */
@Component
@Slf4j
public class TbFaceTestTask {
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Resource
    private FaceLogDao faceLogDao;
    @Autowired
    @Lazy
    private AsyncTriggerRetryImpl asyncTriggerRetry;
    @Autowired
    private AdminService adminService;
    @Autowired
    private UserWhitelistService userWhitelistService;
    @Autowired
    private UserControlEarlyWarningListService userControlEarlyWarningListService;
    @Resource
    private UserWhitelistDao userWhitelistDao;
    @Resource
    private ExamUserWhitelistDao examUserWhitelistDao;
    @Resource
    private UserVisitorListService userVisitorListService;
    @Resource
    private FaceLogDetailDao faceLogDetailDao;
    @Resource
    private ShortTermFaceLogDetailDao shortTermFaceLogDetailDao;
    @Autowired
    private UserVisitorListDetailService userVisitorListDetailService;
    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private AdminInoutWarnConfigDao adminInoutWarnConfigDao;
    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private RedisUtil redisUtil;
    @Resource
    private WarningLogDao warningLogDao;
   

//    //推送进出记录到公安网地址
//    @Value("${push.faceLogUrl}")
//    private String pushFaceLogUrl;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    /*    @Resource
        private ShortTermFaceLogDao shortTermFaceLogDao;*/
    @RabbitListener(queues = "faceLogQueue")
    public void toClearUserInfo(Map<String, Object> dto) {
        log.info("----------开始------进出记录存入-------------");
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        String sn = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        log.info("进入消费者参数{}", sn);
        //判断是否为空
        if (org.apache.commons.lang3.StringUtils.isBlank(sn)) {
            return;
        }
        WhiteVisitorCertificationVO whiteVisitorCertificationVO = faceDeviceDao.getFaceDeviceOne(sn);
        //sn是否存在
        if (whiteVisitorCertificationVO == null) {
            return;
        }

        //查询账号信息
        AdminVO admin = adminService.getAdminInfoById(whiteVisitorCertificationVO.getAdminId());
        //组合数据 <身份证，常访客信息> 目的得到 adminId
        //目的：为了将进入到楼宇的进出记录归档到企业的进出记录中去
//        Map<String, List<UserWhitelist>> userMap = Maps.newHashMap();
//        if(AdminTypeEnum.DISTRICT.getCode().equals(admin.getType()) && AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(admin.getPlaceType())){
//            admin.setAdminIds(Lists.newArrayList(admin.getAdminId()));
//            List<Admin> adminList = adminService.queryByTenantId(admin.getAdminId());
//            if(CollectionUtil.isNotEmpty(adminList)) {
//                admin.getAdminIds().addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
//            }
//
//            //查询常访客信息
//            List<UserWhitelist> userWhitelists = userWhitelistDao.selectByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andIn(UserWhitelist::getAdminId, admin.getAdminIds())).build());
//            List<UserVisitorList> userVisitorLists = userVisitorListDao.selectByExample(Example.builder(UserVisitorList.class).where(WeekendSqls.<UserVisitorList>custom().andIn(UserVisitorList::getAdminId, admin.getAdminIds())).build());
//            if(CollectionUtil.isNotEmpty(userVisitorLists)) {
//                userWhitelists.addAll(Convert.toList(UserWhitelist.class, userVisitorLists));
//            }
//
//            userMap = userWhitelists.stream().filter(v -> null != v && StrUtil.isNotEmpty(v.getIdNumber())).collect(Collectors.groupingBy(UserWhitelist::getIdNumber));
//        }

        Integer action = 0;
        String orgid = "";
        if (dto.get("action") != null) {
            action = (Integer) dto.get("action");
        }
        if (dto.get("orgid") != null) {
            orgid = dto.get("orgid").toString();
        }
        List<Map<String, Object>> dataList = (List) dto.get("data");
        if (CollectionUtils.isEmpty(dataList)) {
            return;
        }
        Integer recordid = 0;
        String pin = "0";
        String name = "";
        String photo = null;
        String cardid = "";
        String cardIdEx = "";
        Integer verifytype = 0;
        Integer rightno = 104;

        Integer outtype = 0;
        String idCard = "";
        String phone = "";
        //温度
        String extend3 = "";
        String extend1 = "";
        //健康码组装数据
        String extend5 = "";
        //体温是否异常 0 正常 1 异常
        String twStatus = "";
        //健康码状态00 绿码 01 黄码 10 红码
        String jkmStatus = "";
        //健康码状态 0 健康码 1 国康码
        String jkmType = "";
        //阴性 阳性
        String hsStatus = "";
        //疫苗状态 0针 等等依次累加
        String ymStatus = "";
        //行程信息
        String xcInfo = "";
        boolean status = true;//数据是否有效
        Integer reasonType = null;//原因
        String verify_score = null;
        Long userId;
        Date logTime;
        long nowL = System.currentTimeMillis();
        Date now = new Date(nowL);
        String faceTime = "";
        for (Map<String, Object> data : dataList) {
            if (data.get("recordid") != null) {
                recordid = Integer.valueOf(data.get("recordid").toString());
            }
//            else {
//                log.info("获取recordId为空");
//                continue;
//            }
            if (data.get("pin") != null && NumberUtil.isLong(data.get("pin").toString())) {
                pin = data.get("pin").toString();
            }
//            else {
//                log.info("获取pin为空");
//                continue;
//            }
            if (data.get("name") != null) {
                name = data.get("name").toString();
                name = name.replace(" ", "");
            }
            if (data.get("photo") != null) {
                photo = data.get("photo").toString().replaceAll("\n", "");
            }
            if (data.get("cardid") != null) {
                cardid = data.get("cardid").toString();

                if (HttpUtil.isHttp(cardid) || HttpUtil.isHttps(cardid)) {
                    //扫码推送参数为链接，提取杭州健康码barCode参数信息
                    cardid = HttpUtil.decodeParamMap(cardid, Charset.defaultCharset()).get("stateCouncilBarCode");
                }
            }
            if (data.get("cardIdEx") != null) {
                cardIdEx = data.get("cardIdEx").toString();
            }
            if (data.get("verifytype") != null) {
                verifytype = Integer.valueOf(data.get("verifytype").toString());
            }
            if (data.get("rightno") != null) {
                rightno = Integer.valueOf(data.get("rightno").toString());
            }
            if (data.get("time") != null) {
                faceTime = data.get("time").toString();
            }
            if (data.get("outtype") != null) {
                outtype = Integer.valueOf(data.get("outtype").toString()) ;
            }
            if (data.get("phone") != null) {
                phone = data.get("phone").toString();
            }
            if (data.get("extend3") != null) {
                extend3 = data.get("extend3").toString();
            }
            if (data.get("extend1") != null) {
                extend1 = data.get("extend1").toString();
            }
            if (data.get("verify_score") != null) {
                verify_score = data.get("verify_score").toString();
            }
            if (data.get("twStatus") != null) {
                twStatus = data.get("twStatus").toString();
            }
            if (data.get("jkmStatus") != null) {
                jkmStatus = data.get("jkmStatus").toString();
            }
            if (data.get("jkmType") != null) {
                jkmType = data.get("jkmType").toString();
            }
            if (data.get("hsStatus") != null) {
                hsStatus = data.get("hsStatus").toString();
                String[] split = hsStatus.split(",");
                if (split.length >= 1) {
                    hsStatus = split[0];
                }
            }
            if (data.get("ymStatus") != null) {
                ymStatus = data.get("ymStatus").toString();
                String[] split = ymStatus.split(";");
                if (split.length >= 1) {
                    ymStatus = split[0];
                } else {
                    ymStatus = "0";
                }
            }
            if (data.get("xcInfo") != null) {
                xcInfo = data.get("xcInfo").toString();
            }
            if (data.get("reasonType") != null) {
                reasonType = Integer.valueOf(data.get("reasonType").toString());
            }
            if (data.get("status") != null) {
                status = (boolean) data.get("status");
            }
            if (StringUtils.hasText(photo)) {
//                photo = buildImage(photo, sn + pin);
                String pathString = now.getTime() + String.valueOf((int) (Math.random() * 10000)) + ".jpg";
                String filePath = FileUtils.getDirectory(Constants.UPLOAD_IMAGE_NEW) + "/";
                photo = AddressUtil.address + FileUtils.transformBase64Image(photo, filePath, pathString);

            }
            if (verifytype != 0) {
                userId = Long.valueOf(pin);
            } else {
                userId = 0L;
            }

            if (!StringUtils.hasText(idCard)
                    && !StringUtils.hasText(phone)
                    && !StringUtils.hasText(name)
                    && !StringUtils.hasText(photo)
                    && !StringUtils.hasText(cardid)) {
                continue;
            }

            final FaceLog faceLog = new FaceLog();
            faceLog.setRecordid(Long.valueOf(recordid));
            faceLog.setAction(action);
            faceLog.setDeviceSn(sn);
            faceLog.setUserId(userId);
            faceLog.setName(name);
            faceLog.setPhoto(photo);
            faceLog.setCardid(cardid);
            faceLog.setCardidex(cardIdEx);
            faceLog.setVerifyType(verifytype);
            faceLog.setRightno(rightno);
            faceLog.setOutType(whiteVisitorCertificationVO.getPass());
            faceLog.setFaceTime(faceTime);
            faceLog.setOrgid(orgid);
            faceLog.setIdCard(idCard);
            faceLog.setPhone(phone);
            faceLog.setExtend3(extend3);
            faceLog.setExtend1(extend1);
            faceLog.setExtend5(extend5);
            faceLog.setTwStatus(twStatus);
            faceLog.setJkmStatus(jkmStatus);
            faceLog.setJkmType(jkmType);
            faceLog.setHsStatus(hsStatus);
            faceLog.setYmStatus(ymStatus);
            faceLog.setXcInfo(xcInfo);
            faceLog.setAdminId(whiteVisitorCertificationVO.getAdminId());
            faceLog.setUsername(admin.getUsername());
            faceLog.setVerifyScore(verify_score);
            faceLog.setCreateTime(now);
            faceLog.setDeviceName(whiteVisitorCertificationVO.getName());
            faceLog.setThirdPartyName(whiteVisitorCertificationVO.getThirdPartyName());
            faceLog.setFaceDeviceTypeName(whiteVisitorCertificationVO.getFaceDeviceTypeName());
            faceLog.setStatus(status);
            faceLog.setReasonType(reasonType);

            try {
                if (StringUtils.hasText(cardid)) {
                    List<UserInfo> userInfos = userInfoDao.selectByExample(Example.builder(UserInfo.class).where(WeekendSqls.<UserInfo>custom().andEqualTo(UserInfo::getIdNumber, cardid)).build());
                    if (userInfos != null && userInfos.size() > 0) {
                        List<UserInfo> ccUserInfos = userInfos.stream().filter(info -> info.getCountryCode() != null && !"".equals(info.getCountryCode().trim())).collect(Collectors.toList());
                        if (ccUserInfos != null && ccUserInfos.size() > 0) {
                            faceLog.setCountryCode(ccUserInfos.get(0).getCountryCode());
                        }

                        List<UserInfo> genderUserInfos = userInfos.stream().filter(info -> info.getGenderStr() != null && !"".equals(info.getGenderStr().trim())).collect(Collectors.toList());
                        if (genderUserInfos != null && genderUserInfos.size() > 0) {
                            faceLog.setGender(genderUserInfos.get(0).getGenderStr());
                        }

                        List<UserInfo> birthdayUserInfos = userInfos.stream().filter(info -> info.getBirthday() != null && !"".equals(info.getBirthday().trim())).collect(Collectors.toList());
                        if (birthdayUserInfos != null && birthdayUserInfos.size() > 0) {
                            faceLog.setBirthday(birthdayUserInfos.get(0).getBirthday());
                        }

                    }

                    //通过身份证查询常客信息
                    UserWhitelist userWhitelist = userWhitelistDao.selectOne(
                            new UserWhitelist().setAdminId(admin.getAdminId()).setIdNumber(cardid));
                    //查询到常客信息，并且为未认证的状态
                    if (userWhitelist != null) {
                        faceLog.setRightno(105);
                    }
                }

                //进出记录存入 保留天数为0 表示不保留
                if (admin.getFaceLogHoldDays() != null && admin.getFaceLogHoldDays().intValue() != 0) {
                    faceLogDao.insertSelective(faceLog);
                }
                //进出数据统计
                this.statData(admin,faceLog,whiteVisitorCertificationVO);

                /*无效的数据不用进行数据的处理只做存储*/
                if(!status){
                    return;
                }
                //进出预警
                this.adminInoutWarnConfig(faceLog, admin, whiteVisitorCertificationVO);
                /*楼宇场景下-访客入场通知*/
                userVisitorListDetailService.inField(admin, faceLog);
                //自定义字段的存入
                if (CollUtil.isNotEmpty(admin.getContextVOList())) {
                    List<ContextVO> contextVOList = this.getConTextVo(cardid, phone, cardIdEx, admin.getAdminId(), rightno);
                    if (CollUtil.isNotEmpty(contextVOList)) {
                        List<ContextVO> list = this.addList(admin.getContextVOList(), contextVOList);
                        this.insertContext(list, faceLog.getId(), now);
                    }
                }
                //考试场景现场照片
                if (admin.getIsOpenExam()) {
                    this.exam(photo, admin, cardid, sn);
                }

                //预警人员
                userControlEarlyWarningListService.addWarningInformation(faceLog.getDeviceSn(), faceLog.getCardid(), faceLog.getId());
                //常客人脸终端认证
                /*常客人脸终端认证 第一步：查看账号是否开启常客*/
                if (admin.getWhiteVisitorType().toString().contains("1")) {//表示开启了常客功能
                    /*常客人脸终端认证 第二步：第一种：账号开启了身份证认证*/
                    if (StringUtils.hasText(cardid) && admin.getIsPush() == 1) {
                        //通过身份证查询常客信息
                        UserWhitelist userWhitelist = userWhitelistDao.selectOne(
                                new UserWhitelist().setAdminId(admin.getAdminId()).setIdNumber(cardid));
                        //查询到常客信息，并且为未认证的状态
                        if (userWhitelist != null && userWhitelist.getStatus() == 2) {
                            asyncTriggerRetry.whiteVisitorCertification(userWhitelist, cardid, photo, name);
                        }
                    }
                    /*常客人脸终端认证 第二步：第二种：账号开启了物理卡号认证*/
                    if (StringUtils.hasText(cardIdEx) && admin.getCardPhysics() == 1) {
                        //通过物理卡号查询常客信息
                        UserWhitelist userWhitelist = userWhitelistDao.selectOne(
                                new UserWhitelist().setAdminId(admin.getAdminId()).setCardIdEx(cardIdEx));
                        //查询到常客信息，并且为未认证的状态
                        if (userWhitelist != null && userWhitelist.getStatus() == 2) {
                            asyncTriggerRetry.whiteVisitorCertificationCardIdEx(userWhitelist, cardIdEx, photo, name);
                        }
                    }
                }
            } catch (Exception e) {
                log.info("异常", e);
            }
        }
        log.info("----------完成------进出记录存入-------------");
    }

    private void statData(AdminVO admin, FaceLog faceLog, WhiteVisitorCertificationVO whiteVisitorCertificationVO) {
        if(!admin.getOpenInOutStatistics()){//没有开启数据统计
            return;
        }

        StatDataMq mq = new StatDataMq();
        mq.setAdminId(faceLog.getAdminId());
        mq.setName(faceLog.getName());
        mq.setPhone(faceLog.getPhone());
        mq.setIdNumber(faceLog.getCardid());
        mq.setCardIdEx(faceLog.getCardidex());
        int peopleType;
        if (UserRightNoEnum.REGULAR.getCode().equals(faceLog.getRightno())){//普通人员
            peopleType = 1;
        }else if (UserRightNoEnum.VISITORS.getCode().equals(faceLog.getRightno())){//访客
            peopleType = 2;
        }else{//其他
            peopleType = 3;
        }
        mq.setPeopleType(peopleType);
        mq.setSn(faceLog.getDeviceSn());
        mq.setSnName(whiteVisitorCertificationVO.getName());
        mq.setPass(faceLog.getOutType());
        mq.setPassTime(DateUtil.parse(faceLog.getFaceTime()));
        mq.setPhoto(faceLog.getPhoto());
        //推到mq中消费
        rabbitTemplate.convertAndSend(Constants.STAT_DATA_QUEUE_NAME, mq);
    }

    private void adminInoutWarnConfig(FaceLog faceLog, AdminVO admin, WhiteVisitorCertificationVO vo) {
        if (!admin.getIsOpenInoutWarning()) {
            return;
        }
        if (StrUtil.isBlank(faceLog.getCardid())) {
            return;
        }
        AdminInoutWarnConfig adminInoutWarnConfig = adminInoutWarnConfigDao.selectOne(new AdminInoutWarnConfig().setAdminId(admin.getAdminId()));
        if (ObjectUtil.isNull(adminInoutWarnConfig)) {
            log.info(admin.getUsername() + ":进出异常预警配置未填写");
            return;
        }
        if (!Objects.equals(adminInoutWarnConfig.getPass(), 0)
                && !adminInoutWarnConfig.getPass().equals(faceLog.getOutType())
        ) {//统计的不是通用。并且统计的设备与设备的进出不对应
            return;
        }
        Date faceTime = DateUtil.parse(faceLog.getFaceTime());
        String key = Constants.IN_OUT_WARNING + admin.getAdminId() + ":" + faceLog.getCardid();
        Date dateTime = DateUtil.offsetHour(new Date(), -adminInoutWarnConfig.getHour());//统计的时间
        long times = adminInoutWarnConfig.getHour() * 3600L;
        List<Date> effectiveDateList = null;
        synchronized (this) {
            redisUtil.lSet(key, faceTime, times);
            List<Date> dateList = redisUtil.lGet(key, 0, -1, Date.class);
            effectiveDateList = dateList.stream().filter(p -> DateUtil.compare(p, dateTime) >= 0).collect(Collectors.toList());
            if (dateList.size() != effectiveDateList.size()) {
                redisUtil.del(key);
                if (CollUtil.isNotEmpty(effectiveDateList)) {
                    for (Date date : effectiveDateList) {
                        redisUtil.lSet(key, date);
                    }
                    redisUtil.expire(key, times);
                }

            }
        }
        if (effectiveDateList.size() >= adminInoutWarnConfig.getNum()) {
            WarningLog warningLog = BeanUtil.copyProperties(faceLog, WarningLog.class);
            BeanUtil.copyProperties(vo, warningLog, "name");
            PassEnum enumByCode = PassEnum.getEnumByCode(adminInoutWarnConfig.getPass());
            String passString = ObjectUtil.isNull(enumByCode) ? "" : enumByCode.getDesc();
            warningLog.setId(null)
                    .setFaceDeviceId(vo.getFaceDeviceId())
                    .setFaceDeviceName(vo.getName())
                    .setFaceTime(faceTime)
                    .setWarningMsg("在" + adminInoutWarnConfig.getHour() + "小时内，该用户" + passString + "通道大于等于" + adminInoutWarnConfig.getNum() + "次");
            warningLogDao.insertSelective(warningLog);
        }
    }
    private void exam(String photo, AdminVO admin, String cardid, String sn) {
        log.info("照片：{}，设备列表：{}，身份照：{}，设备号{}", photo, admin.getExamDevices(), cardid, sn);
        if (StrUtil.isBlank(cardid)) {
            return;
        }
        if (StrUtil.isBlank(admin.getExamDevices())) {
            return;
        }
        if (!admin.getExamDevices().contains(sn)) {
            return;
        }
        Date date = new Date();
        String nowTime = DateUtil.format(date, DatePattern.NORM_DATETIME_PATTERN);
        ExamUserWhitelist examUserWhitelist = examUserWhitelistDao.
                selectByAdminIdAndIdNumber(
                        admin.getAdminId(),
                        cardid,
                        nowTime);
        if (examUserWhitelist != null) {
            examUserWhitelist.setPhoto(photo);
            examUserWhitelist.setStatus(1);
            examUserWhitelist.setInTime(date);
            examUserWhitelistDao.updateByPrimaryKeySelective(examUserWhitelist);
        }
    }

    private void insertContext(List<ContextVO> list, Long id, Date now) {
        if (CollUtil.isNotEmpty(list) && list.size() > 0) {
            for (ContextVO contextVO : list) {
                FaceLogDetail faceLogDetail = new FaceLogDetail();
                faceLogDetail.setFaceLogId(id);
                faceLogDetail.setContextId(contextVO.getId());
                faceLogDetail.setContextCustomNameVal(contextVO.getCustomNameVal());
                faceLogDetail.setCreateTime(now);
                faceLogDetailDao.insertSelective(faceLogDetail);
//                shortTermFaceLogDetailDao.insertSelective(BeanUtil.copyProperties(faceLogDetail, ShortTermFaceLogDetail.class));
            }
        }
    }

    private List<ContextVO> addList(List<ContextVO> adminContextVOList, List<ContextVO> contextVOList) {
        List<ContextVO> list = new ArrayList<>();
        for (ContextVO userVO : contextVOList) {
            for (ContextVO adminVo : adminContextVOList) {
                if (userVO.getId().equals(adminVo.getId())) {
                    list.add(userVO);
                    break;
                }
            }
        }
        return list;
    }

    private List<ContextVO> getConTextVo(String cardid, String phone, String cardIdEx, Integer adminId, Integer rightno) {
        List<ContextVO> contextVOList = null;
//        if(rightno.equals(105)){//在常客表中查询信息
        UserWhitelist userWhitelist = userWhitelistService.getByPhoneAndIdNumber(cardid, phone, cardIdEx, adminId);
        if (userWhitelist != null && StrUtil.isNotEmpty(userWhitelist.getContent())) {
            contextVOList = JSONArray.parseArray(userWhitelist.getContent(), ContextVO.class);
        }
        if (CollUtil.isNotEmpty(contextVOList)) {
            return contextVOList;
        }

//        }else if(rightno.equals(104)){//访客表中查询信息
        UserVisitorList userVisitorList = userVisitorListService.getByPhoneAndIdNumber(cardid, phone, adminId);
        if (userVisitorList != null && StrUtil.isNotEmpty(userVisitorList.getContent())) {
            contextVOList = JSONArray.parseArray(userVisitorList.getContent(), ContextVO.class);
        }
//        }
        return contextVOList;
    }

    public static void main(String[] args) {
        Map<String, String> stringListMap = HttpUtil.decodeParamMap("https://h5.dingtalk.com/healthAct/index.html?qrCode=V0ding03830b756f1ecf2635c2f4657eb6378f&stateCouncilBarCode=ffd631d12c5ee80fe77fe0fb4347b4e8&cityCode=#/result", Charset.defaultCharset());
        System.out.println(stringListMap);
        System.out.println(stringListMap.get("stateCouncilBarCode"));
    }
}
