package com.yx.face.service.job;

import com.yx.face.dao.FaceLogDao;
import com.yx.face.dao.UserCattleDao;
import com.yx.face.model.entity.FaceLog;
import com.yx.face.model.entity.UserCattle;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @description:
 * @ClassName UserCattleTask
 * @Author WXG
 * @Date 2022/1/1 15:35
 */
@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
@EnableAsync
@Slf4j
public class UserCattleTask {

    @Resource
    private FaceLogDao faceLogDao;
    @Resource
    private UserCattleDao userCattleDao;
    /*
     * 每三个小时统计下黄牛
     * */
    @Scheduled(cron = "0 0 0/3 * * ?", zone = "Asia/Shanghai")
    @Async
    @Transactional(rollbackFor = Exception.class)
    public void toClearUserInfo() {
        log.info("----------开始------查找黄牛数据-------------");
        /*
        * 黄牛统计标准：10天之内在同一账号下出现大于等于5次的人被标记为黄牛
        * 1.查询 face_log  并进行分组查询 先以身份证group by查询，再以天group by查询
        * */
        /*1.删除原有的内容*/
        userCattleDao.delete(new UserCattle());


        Date date  = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE,-10);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String faceTime = sdf.format(calendar.getTime());
        List<String> idNumberList = faceLogDao.getCattleList(faceTime);
        if(idNumberList !=null && idNumberList.size()>0){
            for (String idNumber : idNumberList) {
                UserCattle userCattle = new UserCattle();
                Set<String> faceTimeList = new LinkedHashSet<>();
                List<FaceLog> faceLogList = faceLogDao.getByIdnumber(idNumber,faceTime);
                FaceLog faceLog = faceLogList.get(0);
                userCattle.setName(faceLog.getName());
                userCattle.setPhone(faceLog.getPhone());
                userCattle.setAvatar(faceLog.getPhoto());
                userCattle.setStatus(2);
                userCattle.setUpdateTime(date);
                userCattle.setCreateTime(date);
                userCattle.setAdminId(faceLog.getAdminId());
                userCattle.setIdNumber(faceLog.getCardid());
                userCattle.setVerifyType(faceLog.getVerifyType());
                userCattle.setCurrentNum(faceLogList.size());
                for (FaceLog faceLog1 : faceLogList) {
                    faceTimeList.add(faceLog1.getFaceTime().substring(0,10));
                }
                userCattle.setCurrentDay(faceTimeList.size());

                userCattleDao.insertSelective(userCattle);
            }
        }


        log.info("----------完成------查找黄牛数据-------------");
    }
}
