package com.yx.face.service.job.fiveMinute;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.UserVisitorListDao;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.UserVisitorList;
import com.yx.face.model.po.VisitorClearPo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import tk.mybatis.mapper.weekend.Weekend;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName VisitorClearTask
 * @Author WXG
 * @Date 2023/8/18 10:53
 */
@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
@EnableAsync
@Slf4j
public class VisitorClearTask {

    @Resource
    private AdminDao adminDao;

    @Resource
    private UserVisitorListDao userVisitorListDao;

    @Scheduled(cron = "${cron.face.clear.user}", zone = "Asia/Shanghai")
    @Async
    @PostConstruct
    public void task() {
        //1.先查询
        List<Admin> adminList = adminDao.selectByExample(
                Weekend.builder(Admin.class).where(WeekendSqls.<Admin>custom()
                        .andIn(Admin::getType, CollUtil.toList(5, 99))).build()
        );
        //2.拼装数据
        List<VisitorClearPo> list = new LinkedList<>();
        Date date = new Date();
        for (Admin admin : adminList) {
            VisitorClearPo po = new VisitorClearPo();
            BigDecimal userVisiterEffectiveTime = admin.getUserVisiterEffectiveTime();
            userVisiterEffectiveTime = ObjectUtil.isNotNull(userVisiterEffectiveTime) ? userVisiterEffectiveTime : new BigDecimal(3);
            int timelong = userVisiterEffectiveTime.multiply(new BigDecimal(1440)).intValue();
            Date timeEnd = DateUtils.addMinutes(date, -timelong);
            String format = DateUtil.format(timeEnd, DatePattern.NORM_DATETIME_PATTERN);
            po.setAdminId(admin.getId());
            po.setClearTime(format);
            list.add(po);
        }
        if(CollUtil.isEmpty(list)){
            return;
        }
        List<UserVisitorList> userVisitorListList = userVisitorListDao.deleteByVisitorClearPo(list);
        if(CollUtil.isEmpty(userVisitorListList)){
            return;
        }
        List<Integer> UserVisitorListIdList = userVisitorListList.stream().map(UserVisitorList::getId).collect(Collectors.toList());
        userVisitorListDao.deleteByIds(StrUtil.join(",", UserVisitorListIdList));
    }
}
