package com.yx.face.service.job.zeroTask;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.FaceLogDao;
import com.yx.face.dao.FaceLogDetailDao;
import com.yx.face.model.entity.Admin;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 每天0点清除短期进出记录中一个月前的数据
 * @ClassName ClearShortTermFaceLog
 * @Author WXG
 * @Date 2022/9/15 17:26
 */
@Component   //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
@Slf4j
public class ClearFaceLogTask {

    @Resource
    private FaceLogDao faceLogDao;
    @Resource
    private FaceLogDetailDao faceLogDetailDao;

    @Resource
    private AdminDao adminDao;

    @Async
    public void toClearFaceLog() {
        log.info("----------开始------清除FaceLog的数据-------------");
        List<Admin> admins = adminDao.selectAll();
        //处理faceLogHoldDays为空的数据，并赋默认值60天
        admins.stream().forEach(item -> item.setFaceLogHoldDays(item.getFaceLogHoldDays() != null ? item.getFaceLogHoldDays() : 60));
        //根据保留天数分组
        Map<Integer, List<Admin>> faceLogHoldDaysMap = admins.stream().collect(Collectors.groupingBy(Admin::getFaceLogHoldDays));
        //清理数据 暂时不考虑多线程去处理
        faceLogHoldDaysMap.forEach((key, value) -> handleAdminsFaceLog(key, value));
        log.info("----------完成------清除FaceLog的数据-------------");

    }

    private void handleAdminsFaceLog(Integer faceLogHoldDays, List<Admin> admins) {
        try {
            if (admins != null && admins.size() > 0) {
                log.info("--->保留天数: {}, 执行账号数量：{}", faceLogHoldDays, admins.size());
                //保留天数
                faceLogHoldDays = faceLogHoldDays != null ? faceLogHoldDays : 60;
                Date dateTime = DateUtil.offsetDay(new DateTime(), -faceLogHoldDays);
                String formatzero = DateUtil.format(dateTime, DatePattern.NORM_DATE_PATTERN);
                dateTime = DateUtil.parse(formatzero);
                String formatDateStr = DateUtil.format(dateTime, DatePattern.NORM_DATETIME_PATTERN);
                List<Integer> adminIds = admins.stream().map(Admin::getId).collect(Collectors.toList());
                List<Long> faceLogIds = faceLogDao.getByAdminIds(adminIds, formatDateStr);
                if (faceLogIds != null && faceLogIds.size() > 0) {
                    faceLogDetailDao.deleteLastMonthData(faceLogIds);
                }
                faceLogDao.deleteLastMonthData(adminIds, formatDateStr);
            }
        } catch (Exception e) {
            log.error("--->", e);
            log.error("--->清除FaceLog的数据服务异常faceLogHoldDays:{}, 涉及账号:{}", faceLogHoldDays, JSONObject.toJSONString(admins));
        }

    }

    public static void main(String[] args) {
        Integer faceLogHoldDays = 42;
        Date dateTime = DateUtil.offsetDay(new DateTime(), -faceLogHoldDays);
        String formatzero = DateUtil.format(dateTime, DatePattern.NORM_DATE_PATTERN);
        dateTime = DateUtil.parse(formatzero);
        System.out.println(dateTime);
    }
}
