package com.yx.face.service.job.zeroTask;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FaceTaskDao;
import com.yx.face.dao.UserVisitorListDao;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.UserVisitorList;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName ClearVisitorDataTask
 * @Author WXG
 * @Date 2022/12/12 9:19
 */
@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
@EnableAsync
@Slf4j
public class ClearVisitorDataTask {

    @Resource
    private AdminDao adminDao;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Resource
    private UserVisitorListDao userVisitorListDao;
    @Resource
    private FaceTaskDao faceTaskDao;
    @Async
    public void clearVisitorDataTask() {
        log.info("----------开始------根据访客配置删除访客的数据-------------");
        /*1.查询开启访客删除的账号*/
        Example adminExample = Example.builder(Admin.class).where(WeekendSqls.<Admin>custom()
                .andEqualTo(Admin::getIsOpenVisitorClear, Boolean.TRUE)
                .andIn(Admin::getType, CollUtil.toList(5, 99))).build();
        List<Admin> adminList = adminDao.selectByExample(adminExample);
        List<Integer> adminIdList = adminList.stream().map(Admin::getId).collect(Collectors.toList());
        /*2.查询开启该配置账号的设备*/
        Example faceDeviceExample = new Example(FaceDevice.class);
        faceDeviceExample.createCriteria().andIn("adminId",adminIdList);
        List<FaceDevice> faceDeviceList = faceDeviceDao.selectByExample(faceDeviceExample);
        Map<Integer, List<FaceDevice>> listMap = faceDeviceList.stream().collect(Collectors.groupingBy(FaceDevice::getAdminId));
        /*3.查询账号下的访客*/
        Example userVisitorListExample = new Example(UserVisitorList.class);
        userVisitorListExample.createCriteria().andIn("adminId",adminIdList);
        List<UserVisitorList> userVisitorListList = userVisitorListDao.selectByExample(userVisitorListExample);
        /*4.修改人脸过期时间*/
//        String now = DateUtil.format(new Date(), DatePattern.NORM_DATETIME_PATTERN);
        Date now  = new Date();
        for (UserVisitorList userVisitorList : userVisitorListList) {
            if(listMap.containsKey(userVisitorList.getAdminId())){
                List<String> snList = listMap.get(userVisitorList.getAdminId()).stream().map(FaceDevice::getSn).collect(Collectors.toList());
                faceTaskDao.updateFaceTaskTaskTime(snList,userVisitorList.getUserId(),now);
            }
        }
        /*5.删除访客*/
        if(CollUtil.isNotEmpty(userVisitorListList)){
            List<Integer> userVisitorIdList = userVisitorListList.stream().map(UserVisitorList::getId).collect(Collectors.toList());
            String join = StrUtil.join(",", userVisitorIdList);
            userVisitorListDao.deleteByIds(join);
        }
        log.info("----------完成------根据访客配置删除访客的数据-------------");
    }
}
