package com.yixin.ms.server;

import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.model.dto.CodeOpenJudgeDto;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.ColorEnum;
import com.yixin.ms.model.enums.HealthOpenEnum;
import com.yixin.ms.model.enums.ThirdPlatformPassIconEnum;
import com.yixin.ms.model.vo.CodeOpenJudgeVo;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.server.vo.QueryExamUserWhitelistVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Date;

/**
 * @description:
 * 二维码扔给主服务进行识别
 * @ClassName CodeServer
 * @Author WXG
 * @Date 2024/4/10 14:20
 */
@Slf4j
@Component
public class CodeServer {
    @Value("${health.code.codeOpenJudgeUrl}")
    private String codeOpenJudgeUrl;

    public RestResult<HealtCodeUserVO> codeOpenJudge(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        CodeOpenJudgeVo data= null;
        try {
            CodeOpenJudgeDto dto = new CodeOpenJudgeDto();
            dto.setSn(shortTermDTO.getSn());
            dto.setCode(shortTermDTO.getHealthCode());
            RestResult<CodeOpenJudgeVo> request = HttpRequest.post(codeOpenJudgeUrl)
                    .body(JSON.toJSONString(dto)).timeout(5000).thenFunction(val -> {
                        String body = val.body();
                        log.info("请求二维码核查返回结果：{}",body);
                        return JSON.parseObject(body, new TypeReference<RestResult<CodeOpenJudgeVo>>() {
                        });
                    });
            data = request.getData();
        } catch (Exception e) {
            throw new ServiceException("请求二维码核查失败");
        }
        if(ObjectUtil.isNull(data)){
            throw new ServiceException("请求二维码核查失败");
        }



        HealtCodeUserVO userVO = this.initHealtCodeUserVO(data,placeInfoVO);
        return RestResponse.ok(userVO);
    }
    private void convertOpenReturn(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO) {
        //头部
        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getTopTitle())) {
            userVO.setTitle(placeInfoVO.getTopTitle());
        }
        //中下左部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        v.setTitle(Preconditions.isNotBlank(placeInfoVO.getMainLineLeft()) ? placeInfoVO.getMainLineLeft() : v.getTitle());
        v.setContent(Preconditions.isNotBlank(placeInfoVO.getSubLineLeft()) ? placeInfoVO.getSubLineLeft() : v.getContent());
        userVO.setNuclein(Arrays.asList(v));
        //中下右部
        userVO.setVaccineIcoTitle(Preconditions.isNotBlank(placeInfoVO.getMainLineRight()) ? placeInfoVO.getMainLineRight() : userVO.getVaccineIcoTitle());
        userVO.setVaccineNum(Preconditions.isNotBlank(placeInfoVO.getSubLineRight()) ? placeInfoVO.getSubLineRight() : userVO.getVaccineNum());
        //底部
        userVO.setShowQrcode(placeInfoVO.getShowQrcode());
        userVO.setQrcodeText(placeInfoVO.getShowQrcode() ? placeInfoVO.getQrcodeText() : StrUtil.EMPTY);
        userVO.setAdvertise((!placeInfoVO.getShowQrcode() && Preconditions.isBlank(placeInfoVO.getAdvertise())) ? HealthOpenReturnServer.defaultAdvertise : placeInfoVO.getAdvertise());
        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getLatestResultPage())
                && placeInfoVO.getLatestResultPage()) {
            userVO.setShowPageType("scenic");
        }
    }
    private HealtCodeUserVO initHealtCodeUserVO(CodeOpenJudgeVo data, PlaceInfoVO placeInfoVO) {
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        userVO.setIdNumber(StrUtil.isEmpty(data.getIdNumber()) ? StrUtil.EMPTY : data.getIdNumber());
        userVO.setShowIdNumber(StrUtil.isEmpty(data.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(data.getIdNumber(), 4, 4));
        userVO.setUsername(StrUtil.isEmpty(data.getName()) ? StrUtil.EMPTY : data.getName());
        userVO.setShowUsername(StrUtil.isEmpty(data.getName()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(data.getName()));
        convertOpenReturn(userVO,placeInfoVO);

        Integer isOpen = HealthOpenEnum.RELEASE.getCode();
        String codeTypeVOContent = "请通行";

        String iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/trip/green.png";

        if (data.getIsOpen() == 1) {
            isOpen = HealthOpenEnum.NO_THOROUGHFARE.getCode();
            codeTypeVOContent = data.getReason();
            iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            userVO.setAMainIcon(ThirdPlatformPassIconEnum.AFORBID.getUrl());
            userVO.setAMainLine(data.getReason());
        }
        //开门信号
        userVO.setIsOpen(isOpen);
        //语音播报
        userVO.setOpenMsg(codeTypeVOContent != null ? codeTypeVOContent : "请通行");
        userVO.setOpenVoiceMsg(codeTypeVOContent != null ? codeTypeVOContent : "请通行");

        //全局背景
        userVO.setBackgroundColor(ColorEnum.GREEN.getValue());

        //头部
        String title = "温馨提醒"; //温馨提醒
        if (placeInfoVO != null
                && placeInfoVO.getTopTitle() != null) {
            title = placeInfoVO.getTopTitle();
        }
        userVO.setTitle(title == null ? "温馨提醒" : title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());

        //中上部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt(StrUtil.EMPTY);
        codeTypeVO.setContent(codeTypeVOContent);
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        //中下部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        //左icon
        v.setIconUrl(iconUrl);
        v.setTitle("");
        v.setContent("").setContentColor(ColorEnum.GREEN.getValue());
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (Exception e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));

        //右icon
        userVO.setVaccineIconUrl(vaccineIconUrl);
        userVO.setVaccineIcoTitle("");
        userVO.setVaccineNum("");
        userVO.setVaccineNumColor(ColorEnum.GREEN.getValue());

        //底部
        userVO.setTravelValidationIconUrl(travelValidationIconUrl);
        userVO.setTravelValidationTitle("");
        userVO.setTravelValidation("");
        userVO.setTravelValidationDesc(StrUtil.EMPTY);
        //ui版本
        if(placeInfoVO != null && placeInfoVO.getLatestResultPage()){
            userVO.setShowPageType("scenic");
        }

        userVO.setASubLineFirst(Preconditions.isBlank(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        userVO.setASubLineSecond(Preconditions.isBlank(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setAMainIcon(Preconditions.isNotBlank(userVO.getAMainIcon()) ? userVO.getAMainIcon() : ThirdPlatformPassIconEnum.APASS.getUrl());
        userVO.setAMainLine(Preconditions.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : codeTypeVOContent);

        log.info("回参:{}",JSON.toJSONString(userVO));
        return userVO;
    }
}
