package com.yixin.ms.server;

import cn.hutool.core.map.MapUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.crypto.util.SzhzSm4Util;
import com.yixin.ms.server.vo.HealthCodeVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @author: lileilei
 * @date: 2022/8/12 17:35
 * @description: 杭州市数据局根据barcode查询健康码（国密接口）信息 仅政务网可访问
 */
@Slf4j
@Component
public class HealthCodeServer {
    @Value("${irs.health.code.appKey}")
    private String appKey;
    @Value("${irs.health.code.appSecret}")
    private String appSecret;
    @Value("${irs.health.code.AppKey}")
    private String AppKey;
    @Value("${irs.health.code.queryInfoByBarCodeUrl}")
    private String queryInfoByBarCodeUrl;

    /**
     * 通过barcode查询个人信息
     * @param barcode
     * @return
     */
    public RestResult<HealthCodeVO> queryInfoByBarCode(String barcode){
        Map<String, String> headers = MapUtil.newHashMap();
        headers.put("appKey", appKey);
        headers.put("appSecret", appSecret);
        Map<String, Object> paramMap = MapUtil.newHashMap();
        paramMap.put("AppKey", AppKey);
        paramMap.put("barcode", SzhzSm4Util.encrypt(AppKey, barcode));
        String body = HttpRequest.post(queryInfoByBarCodeUrl).form(paramMap).addHeaders(headers).timeout(5000).execute().body();
        log.info("通过barcode查询个人信息响应内容： {}", body);
        RestResult restResult = JSON.parseObject(body, RestResult.class);
        if(restResult.getCode() == RestCode.SUCCESS.getCode()){
            HealthCodeVO healthCodeVO = JSON.parseObject(JSON.toJSONString(restResult.getData()), HealthCodeVO.class);
            //反解密
            healthCodeVO.setUserName(SzhzSm4Util.decrypt(AppKey, healthCodeVO.getUserName()));
            healthCodeVO.setIdNum(SzhzSm4Util.decrypt(AppKey, healthCodeVO.getIdNum()));
            healthCodeVO.setPn(SzhzSm4Util.decrypt(AppKey, healthCodeVO.getPn()));
            restResult.setData(healthCodeVO);
        }

        return restResult;
    }
}
