package com.yixin.ms.server;

import cn.hutool.http.HttpUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.OkHttpUtils;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.crypto.util.SzhzSm4Util;
import com.yixin.ms.model.dto.IrsShortTermDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.HealthVaccineNumEnum;
import com.yixin.ms.model.enums.RestCodeEnum;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.dto.HealthCodeDTO;
import com.yixin.ms.server.vo.HealthCodeVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * @author: lileilei
 * @date: 2022/8/12 17:35
 * @description: 杭州市数据局根据barcode查询健康码（国密接口）信息
 */
@Slf4j
@Component
public class HealthServer {
    @Value("${health.appKey}")
    private String appKey;
    @Value("${health.appSecret}")
    private String appSecret;
    @Value("${health.queryInfoByBarCodeUrl}")
    private String queryInfoByBarCodeUrl;
    @Value("${health.getProvinceHealthCodeUrl}")
    private String getProvinceHealthCodeUrl;


    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        try {
            //增加上层接口 省健康码基本信息、疫苗注射数据信息、核酸检测数据 自定义封装三合一接口
            HealtCodeUserVO shortTermUser = getProvinceHealthCodeInfo(shortTermDTO.getIdNumber());
            log.info("---->省健康码三合一接口：{}", JSON.toJSONString(shortTermUser));
            if (shortTermUser != null
                    && !CollectionUtils.isEmpty(shortTermUser.getData())) {
                if (HealthVaccineNumEnum.VACCINATION_COMPLETE.getStatus().equals(shortTermUser.getVaccineNum())) {
                    shortTermUser.setVaccineNum(HealthVaccineNumEnum.VACCINATION_COMPLETE.getNum());
                } else if (HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getStatus().equals(shortTermUser.getVaccineNum())) {
                    shortTermUser.setVaccineNum(HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getNum());
                }
                return shortTermUser;
            }

            shortTermUser = getShortTermUser(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            if (shortTermUser != null && !CollectionUtils.isEmpty(shortTermUser.getData())) {
                shortTermUser.setIdNumber(shortTermDTO.getIdNumber());
                shortTermUser.setUsername(shortTermDTO.getName());
                return shortTermUser;
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("--->查询健康码异常", e);
        }
        //未获取到健康码信息
        throw new ServiceException(RestCodeEnum.HEALTH_CODE_INFO_NOT_FOUND.getCode(), RestCodeEnum.HEALTH_CODE_INFO_NOT_FOUND.getMsg());
    }

    /**
     * 通过barcode查询个人信息
     *
     * @param barcode
     * @return
     */
    public HealthCodeVO queryInfoByBarCode(String barcode) {
        String body = null;
        HealthCodeVO healthCodeVO = null;
        try {
            body = HttpUtil.post(queryInfoByBarCodeUrl, JSON.toJSONString(new HealthCodeDTO().setAppKey(appKey).setAppSecret(appSecret).setBarCode(barcode)));
            RestResult restResult = JSON.parseObject(body, RestResult.class);
            if (restResult.getCode() == RestCode.SUCCESS.getCode()) {
                healthCodeVO = JSON.parseObject(JSON.toJSONString(restResult.getData()), HealthCodeVO.class);
            }
        } catch (Exception e) {
            throw new ServiceException("barCode查询信息异常，请稍后重试");
        }
        return healthCodeVO;

    }

    /**
     * 获取省健康码信息
     *
     * @param idNumber
     * @return
     */
    public HealtCodeUserVO getProvinceHealthCodeInfo(String idNumber) {
        String body = null;
        try {
            if (Preconditions.isNotBlank(idNumber)) {
                body = HttpUtil.post(getProvinceHealthCodeUrl, JSON.toJSONString(new IrsShortTermDTO().setAppKey(appKey).setAppSecret(appSecret).setIdNumber(idNumber)));
                log.info("--->获取省健康码信息最终结果返回：{}", body);
                if (Preconditions.isNotBlank(body)) {
                    HealtCodeUserVO restResult = JSON.parseObject(body, HealtCodeUserVO.class);
                    return restResult;
                }
            }
        } catch (Exception e) {
//            throw new ServiceException("省健康码查询信息异常，请稍后重试");
        }
        return null;
    }

    /*
     * 获取江干分局数据
     * */
    public HealtCodeUserVO getShortTermUser(String sfzh, String name) throws Exception {
        log.info("江干分局健康码传参，身份证{} 姓名 {}", sfzh, name);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("sfzh", sfzh);
        jsonObject.put("xm", name);
        String sync = OkHttpUtils.builder().url("http://115.236.38.205:8903/id/hvn")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(jsonObject.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("江干分局请重试");
        }
        log.info("江干分局健康码返回结果，{}", sync);
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.isNotBlank(sync)) {
            JSONObject json = JSON.parseObject(sync);
            if (StringUtils.equals("00", json.getString("error_code"))) {
                healtCodeUserVO.setData(JSON.parseArray(json.getJSONArray("data").toJSONString(), HealtCodeTypeVO.class));
                healtCodeUserVO.setNuclein(JSON.parseArray(json.getJSONArray("nuclein").toJSONString(), HealtCodeNucleinVO.class));
                //源数据只返回报告时间，检测时间先等同于报告时间，方便后面逻辑处理
//                healtCodeUserVO.getNuclein().forEach(v -> v.setReportTime(v.getChecktime()));
                healtCodeUserVO.setVaccineNum(json.getString("vaccine_num"));
            } else {
                throw new ServiceException("江干分局异常:" + jsonObject.getString("msg"));
            }
        }
        return healtCodeUserVO;
    }
}
