package com.yixin.ms.server;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.server.dto.IrsCheckScheduleDTO;
import com.yixin.ms.server.dto.IrsSendMsgDTO;
import com.yixin.ms.server.vo.IrsTripResultVO;
import com.yixin.ms.server.vo.IrsTripTokenVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.Md5Crypt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.Map;

/**
 * @author: lileilei
 * @date: 2022/8/3 17:37
 * @description: 省共享行程信息服务
 */
@Slf4j
@Component
public class IrsTripServer {
    private static final String TOKEN_HEADERS_KEY = "token_session_id";

    private static final String REFRESH_TOKEN_HEADERS_KEY = "refresh_token_session_id";

    /**
     * 请求密钥有效时间为 15 分钟
     *
     * @see <a href="https://www.hutool.cn/docs/#/cache/TimedCache">定时缓存</a>
     */
//    private static TimedCache<String, String> tokenCache = CacheUtil.newTimedCache(14 * 60 * 1000);
//
//    /**
//     * 刷新密钥有效时间为 48 小时
//     */
//    private static TimedCache<String, String> refreshTokenCache = CacheUtil.newTimedCache(47 * 60 * 60 * 1000);


    @Value("${irs.appKey}")
    private String appKey;
    @Value("${irs.appSecret}")
    private String appSecret;
    /**
     * 签名密钥
     */
    @Value("${irs.secretKey}")
    private String secretKey;
    @Value("${irs.tokenUrl}")
    private String tokenUrl;
    @Value("${irs.refreshTokenUrl}")
    private String refreshTokenUrl;
    @Value("${irs.sendMsgUrl}")
    private String sendMsgUrl;
    @Value("${irs.checkScheduleUrl}")
    private String checkScheduleUrl;
    @Value("${irs.healthCodeBaseInfoUrl}")
    private String healthCodeBaseInfoUrl;
    @Value("${irs.vaccineUrl}")
    private String vaccineUrl;
    @Value("${irs.nucleinListUrl}")
    private String nucleinListUrl;

    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private RedisUtil redisUtil;
    /**
     * 获取token
     * @return
     */
    public IrsTripTokenVO getToken(){
        IrsTripTokenVO tokenVO = null;
        String tokenValue = (String) redisUtil.get(TOKEN_HEADERS_KEY);
        String refreshTokenValue =(String) redisUtil.get(REFRESH_TOKEN_HEADERS_KEY);
        log.info("tokenValue,{}",tokenValue);
        log.info("refreshTokenValue,{}",refreshTokenValue);
        if(StrUtil.isNotEmpty(tokenValue) && StrUtil.isNotEmpty(refreshTokenValue)){
            tokenVO = new IrsTripTokenVO();
            tokenVO.setRequestSecret(tokenValue).setRefreshSecret(refreshTokenValue);
            return tokenVO;
        }

//        if(StrUtil.isEmpty(tokenValue) && StrUtil.isNotEmpty(refreshTokenValue)){
//            return this.refreshToken(refreshTokenValue);
//        }

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        MultiValueMap<String, String> map= new LinkedMultiValueMap();
        String requestTime = String.valueOf(System.currentTimeMillis());
        String sign = SecureUtil.md5(new StringBuilder().append(appKey).append(appSecret).append(requestTime).toString()).toLowerCase();
        map.add("appKey", appKey);
        map.add("requestTime", requestTime);
        map.add("sign", sign);
        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(map, headers);
        ResponseEntity<String> response = restTemplate.postForEntity(tokenUrl, request , String.class);
        log.info("token返回数,{}", JSON.toJSONString(response));
        IrsTripResultVO irsTripResultVO = JSON.parseObject(response.getBody(), IrsTripResultVO.class);
        if(IrsTripResultVO.SUCCESS_FLAG.equals(irsTripResultVO.getCode())){
            tokenVO =  JSON.parseObject(JSON.toJSONString(irsTripResultVO.getDatas()), IrsTripTokenVO.class);
            redisUtil.set(TOKEN_HEADERS_KEY, tokenVO.getRequestSecret(),14*60);
            redisUtil.set(REFRESH_TOKEN_HEADERS_KEY, tokenVO.getRefreshSecret(),60*60*47);
        }

        return tokenVO;
    }

    /**
     * 刷新token
     * @param refreshSecret
     * @return
     */
    public IrsTripTokenVO refreshToken(String refreshSecret){
        IrsTripTokenVO tokenVO = null;
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        MultiValueMap<String, String> map= new LinkedMultiValueMap();
        String requestTime = String.valueOf(System.currentTimeMillis());
        String sign = SecureUtil.md5(new StringBuilder().append(appKey).append(refreshSecret).append(requestTime).toString()).toLowerCase();
        map.add("appKey", appKey);
        map.add("requestTime", requestTime);
        map.add("sign", sign);
        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(map, headers);
        ResponseEntity<String> response = restTemplate.postForEntity(refreshTokenUrl, request , String.class);

        IrsTripResultVO irsTripResultVO = JSON.parseObject(response.getBody(), IrsTripResultVO.class);
        if(irsTripResultVO != null && IrsTripResultVO.SUCCESS_FLAG.equals(irsTripResultVO.getCode())){
            tokenVO =  JSON.parseObject(JSON.toJSONString(irsTripResultVO.getDatas()), IrsTripTokenVO.class);
            redisUtil.set(TOKEN_HEADERS_KEY, tokenVO.getRequestSecret(),14*60);
            redisUtil.set(REFRESH_TOKEN_HEADERS_KEY, tokenVO.getRefreshSecret(),60*60*47);
        }
        log.info("数据刷新,返回数据,{}",irsTripResultVO);
        return tokenVO;
    }

    /**
     * 行程核验查询短信验证服务
     * @param phone
     * @return
     */
    public String sendMsg(String phone, String requestSecret){
        IrsSendMsgDTO irsSendMsgDTO = new IrsSendMsgDTO();
        String requestTime = String.valueOf(System.currentTimeMillis());
        String dateTime = DateUtil.formatDateTime(DateUtil.date(Long.valueOf(requestTime)));
        String sign = SecureUtil.md5(new StringBuilder().append(appKey).append(requestSecret).append(requestTime).toString()).toLowerCase();
        String secret = SecureUtil.md5(new StringBuilder().append(secretKey).append(dateTime).toString());
        irsSendMsgDTO.setRequestTime(requestTime).setSign(sign).setAppKey(appKey).setQueryId(IdUtil.fastSimpleUUID()).setSendTime(dateTime).setPhone(phone).setSecert(secret);
        String body = HttpUtil.get(sendMsgUrl, BeanUtil.beanToMap(irsSendMsgDTO), 5000);
        log.info("行程核验查询短信验证服务响应内容：{}", body);
        return body;
    }

    /**
     * 行程核验信息
     * @param phone
     * @param verification
     * @param requestSecret
     * @return
     */
    public String checkSchedule(String phone, String verification, String requestSecret, String city){
        IrsCheckScheduleDTO irsCheckScheduleDTO = new IrsCheckScheduleDTO();
        String requestTime = String.valueOf(System.currentTimeMillis());
        String dateTime = DateUtil.formatDateTime(DateUtil.date(Long.valueOf(requestTime)));
        String sign = SecureUtil.md5(new StringBuilder().append(appKey).append(requestSecret).append(requestTime).toString()).toLowerCase();
        String secret = SecureUtil.md5(new StringBuilder().append(secretKey).append(dateTime).toString());
        irsCheckScheduleDTO.setRequestTime(requestTime).setSign(sign).setAppKey(appKey).setQueryId(IdUtil.fastSimpleUUID()).setSendTime(dateTime).setPhone(phone).setSecert(secret);
        irsCheckScheduleDTO.setCity(city);
        irsCheckScheduleDTO.setVerification(verification);
        String body = HttpUtil.get(checkScheduleUrl, BeanUtil.beanToMap(irsCheckScheduleDTO), 5000);
        log.info("行程核验信息响应内容：{}", body);
        return body;
    }

    /**
     * 查询省健康码基本信息
     *
     * @param idNumber
     * @param name
     * @return
     */
    public String getHealthCodeBaseInfo(String idNumber, String name) {
        log.info("查询省健康码基本信息传参，身份证{} 姓名 {}", idNumber, name);
        IrsTripTokenVO irsTripTokenVO = getToken();
        String requestTime = String.valueOf(System.currentTimeMillis());
        String sign = SecureUtil.md5(appKey + irsTripTokenVO.getRequestSecret() + requestTime).toLowerCase();
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("sfzh", idNumber);

        String appendUrl = new StringBuilder().append(healthCodeBaseInfoUrl)
                .append("?requestTime=").append(requestTime)
                .append("&sign=").append(sign)
                .append("&appKey=").append(appKey).toString();

        String body = HttpRequest.post(appendUrl).form(paramMap).timeout(5000).execute().body();
        log.info("查询省健康码基本信息返回结果，{}", JSONObject.toJSONString(body));
        if (body.contains("请求超时")) {
            throw new ServiceException("省健康码基本信息查询请重试");
        }

        return body;
    }

    /**
     * 疫苗注射数据信息
     *
     * @param idNumber 身份证号
     * @param name     姓名
     */
    public String getVaccine(String idNumber, String name) {
        log.info("疫苗注射数据信息传参，身份证{} 姓名 {}", idNumber, name);
        String requestTime = String.valueOf(System.currentTimeMillis());
        IrsTripTokenVO irsTripTokenVO = getToken();
        String sign = SecureUtil.md5(new StringBuilder().append(appKey).append(irsTripTokenVO.getRequestSecret()).append(requestTime).toString()).toLowerCase();
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("idcardNo", idNumber);

        String appendUrl = new StringBuilder().append(vaccineUrl)
                .append("?requestTime=").append(requestTime)
                .append("&sign=").append(sign)
                .append("&appKey=").append(appKey).toString();

        String body = HttpRequest.post(appendUrl).form(paramMap).timeout(5000).execute().body();
        log.info("疫苗注射数据信息返回结果，{}", JSONObject.toJSONString(body));
        if (body.contains("请求超时")) {
            throw new ServiceException("疫苗注射数据信息查询请重试");
        }
        return body;

    }

    /**
     * 核酸检测数据列表
     *
     * @param idNumber 身份证号
     * @param name     姓名
     */
    public String getNucleinList(String idNumber, String name) {
        log.info("核酸检测数据列表传参，身份证{} 姓名 {}", idNumber, name);
        String requestTime = String.valueOf(System.currentTimeMillis());
        IrsTripTokenVO irsTripTokenVO = getToken();
        String sign = SecureUtil.md5(appKey + irsTripTokenVO.getRequestSecret() + requestTime).toLowerCase();
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("idcardNo", idNumber);

        String appendUrl = new StringBuilder().append(nucleinListUrl)
                .append("?requestTime=").append(requestTime)
                .append("&sign=").append(sign)
                .append("&appKey=").append(appKey).toString();
        String body = HttpRequest.post(appendUrl).form(paramMap).timeout(5000).execute().body();
        log.info("核酸检测数据列表返回结果，{}", JSONObject.toJSONString(body));
        if (body.contains("请求超时")) {
            throw new ServiceException("核酸检测数据列表查询请重试");
        }
        return body;
    }


    public static void main(String[] args) {
//        System.out.println(System.currentTimeMillis());
        String appKey = "A330104242107202107000001", appSecret = "9a92d8b9d41f41729ab8ccbfaa547f12";
        String requestTime = String.valueOf(System.currentTimeMillis());
        System.out.println(requestTime);
        String dateTime = DateUtil.formatDateTime(DateUtil.date(Long.valueOf(requestTime)));
        System.out.println(dateTime);
        String sign = SecureUtil.md5(appKey + appSecret + requestTime).toLowerCase();
        System.out.println("获取token: " + sign);
        sign = SecureUtil.md5(appKey + "2d04c24995d34989a139b088f6cf54ce" + requestTime).toLowerCase();
        System.out.println("刷新token: "+ sign);
        //bbf754f8d3134d7292b7f363f20f533a
        sign = SecureUtil.md5(appKey + "048755972cd24e1a9dca21dc6aac70cc" + requestTime).toLowerCase();
        System.out.println(sign);
        String secret = SecureUtil.md5("9@nr!#G8*Kuw" + dateTime);
        System.out.println(secret);
        System.out.println(RandomUtil.randomString(32));
        System.out.println("#############################");
        String pwd = SecureUtil.md5("admin@123" + "hh_f03920fbc2512e12" + "1659354702").toLowerCase();
        System.out.println(pwd);
        String str = "{\"account\":\"1510813918447009792\",\"idNumber\":\"456\",\"reservationCode\":\"123\",\"timestamp\":\"1660717723806\"}74d9aed80d7a4c7181ba9615fae0fc11";
        String signStr = SecureUtil.md5(str);
        System.out.println(signStr);

        String rt = String.valueOf(System.currentTimeMillis());

        String ak = "83d9174a3e614cf4bb95767551dd7ec6", sk = "fca0ee49199347f793e54175090193e6";
        String sn = SecureUtil.md5(ak+sk+rt);
        System.out.println(rt);
        System.out.println(sn);
    }

}
