package com.yixin.ms.server;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.yixin.ms.boot.config.properties.ScenicProperties;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.HealthOpenEnum;
import com.yixin.ms.model.enums.RestCodeEnum;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.dto.ScenicConsumeDTO;
import com.yixin.ms.server.dto.ScenicVerifyDTO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.server.vo.ScenicConsumeVO;
import com.yixin.ms.server.vo.ScenicVerifyVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/8/3 17:37
 * @description: 皋亭山景区预约服务
 */
@Slf4j
@Component
public class ScenicServer {
    @Autowired
    private ScenicProperties scenicProperties;
    @Resource
    private Executor executor;

    /**
     * 根据预约码或者身份证查询是否预约
     *
     * @param dto
     * @return
     */
    public ScenicVerifyVO verify(ScenicVerifyDTO dto, Integer adminId) {
        List<ScenicProperties.Config> configs = scenicProperties.getConfigs();
        Map<Integer, ScenicProperties.Config> configMap = configs.stream().collect(Collectors.toMap(ScenicProperties.Config::getAdminId, v -> v, (v1, v2) -> v1));
        ScenicProperties.Config config = configMap.get(adminId);
        if (config == null || !config.getAdminId().equals(adminId)) {
            throw new ServiceException(RestCodeEnum.SCENIC_NOT_CONFIG.getCode(), RestCodeEnum.SCENIC_NOT_CONFIG.getMsg());
        }

        dto.setTimestamp(String.valueOf(System.currentTimeMillis())).setAccount(config.getAccount());
        //签名
        dto.setSign(SecureUtil.md5(new StringBuilder().append(JSON.toJSONString(dto)).append(config.getKey()).toString()));
        String body = null;
        try {
            body = HttpUtil.post(scenicProperties.getVerifyUrl(), JSON.toJSONString(dto));
        } catch (Exception e) {
            throw new ServiceException(RestCodeEnum.SCENIC_QUERY_EXCEPTION.getCode(), RestCodeEnum.SCENIC_QUERY_EXCEPTION.getMsg());
        }
        log.info("根据预约码或者身份证查询是否预约，入参信息：{}, 响应信息：{}", JSON.toJSONString(dto), body);
        ScenicVerifyVO verifyVO = JSON.parseObject(body, ScenicVerifyVO.class);
        return verifyVO;
    }

    public void checkConsume(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {
        executor.execute(() -> {
            if (placeInfoVO != null && placeInfoVO.getScenicFlag() && userVO.getIsOpen().equals(HealthOpenEnum.RELEASE.getCode())) {
                this.consume(new ScenicConsumeDTO().setIdNumber(shortTermDTO.getIdNumber()).setReservationCode(NumberUtil.isNumber(shortTermDTO.getHealthCode()) ? shortTermDTO.getHealthCode() : StrUtil.EMPTY), placeInfoVO.getAdminId());
            }
        });
    }

    /**
     * 游客放行推送
     *
     * @param dto
     * @return
     */
    public ScenicConsumeVO consume(ScenicConsumeDTO dto, Integer adminId) {
        List<ScenicProperties.Config> configs = scenicProperties.getConfigs();
        Map<Integer, ScenicProperties.Config> configMap = configs.stream().collect(Collectors.toMap(ScenicProperties.Config::getAdminId, v -> v, (v1, v2) -> v1));
        ScenicProperties.Config config = configMap.get(adminId);
        if (!config.getAdminId().equals(adminId)) {
            throw new ServiceException("景区预约账号未配置，请检查");
        }

        dto.setTimestamp(String.valueOf(System.currentTimeMillis())).setAccount(config.getAccount());
        //签名
        dto.setSign(SecureUtil.md5(new StringBuilder().append(JSON.toJSONString(dto)).append(config.getKey()).toString()));
        String body = null;
        try {
            body = HttpUtil.post(scenicProperties.getConsumeUrl(), JSON.toJSONString(dto));
        } catch (Exception e) {
            throw new ServiceException("景区预约服务系统异常，请稍后重试");
        }
        log.info("游客放行入参信息：{}, 推送响应内容：{}", JSON.toJSONString(dto), body);
        return JSON.parseObject(body, ScenicConsumeVO.class);
    }
}
