package com.yixin.ms.service.impl;

import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.*;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.yixin.ms.boot.config.properties.HealthCodeProperties;
import com.yixin.ms.boot.restful.*;
import com.yixin.ms.boot.uitls.ClassUtil;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.yqk.VerifyCardByStationStrictlyDetailVo;
import com.yixin.ms.boot.yqk.VerifyCardByStationStrictlyDto;
import com.yixin.ms.boot.yqk.VerifyCardByStationStrictlyVo;
import com.yixin.ms.boot.yqk.YqkRestResult;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.dto.ThirdPlatformSm4RespDto;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.AlipayUserInfo;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.*;
import com.yixin.ms.server.dto.UserOpenJudgeDto;
import com.yixin.ms.server.vo.*;
import com.yixin.ms.service.HealthCodeService;
import io.swagger.annotations.ApiModelProperty;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.function.Function;

@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {

    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;
    @Autowired
    private CodeServer codeServer;
    @Autowired
    private HealthCodeServer healthCodeServer;
    @Autowired
    private HealthServer healthServer;
    @Resource
    private HealthOpenReturnServer healthOpenReturnServer;
    @Resource
    private HealthCodeProperties healthCodeProperties;
    @Resource
    private ThreadPoolTaskExecutor taskExecutor;
    @Value("${health.code.userOepnJudgeUrl}")
    private String userOepnJudgeUrl;
    @Value("${health.code.yqkOepnJudgeUrl}")
    private String yqkOepnJudgeUrl;

    @Override
    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        return healthServer.getShortTermCertificate(shortTermDTO);
    }

    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }
    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    @Override
    public RestResult<HealtCodeUserVO> getShortTerm(ShortTermDTO shortTermDTO) {
        long starTime = System.currentTimeMillis();
        log.info("--->联动服务查询入参信息：{}", JSON.toJSONString(shortTermDTO));
        if (StringUtils.isBlank(shortTermDTO.getToken())) {
            throw new ServiceException(RestCodeEnum.FORBIDDEN.getCode(), RestCodeEnum.FORBIDDEN.getMsg());
        }
        /*查询设备账户信息 以及设备所绑账号的开关门信息*/
        PlaceInfoVO placeInfoVO = placeHealthCodeServer.queryAdminInfoBySn(shortTermDTO.getSn());
        //初始化参数
        HealtCodeUserVO userVO = initHealthUserVo(shortTermDTO,placeInfoVO);
        //处理姓名、身份证、手机号数据
        convertUserInfo(userVO, shortTermDTO);
        /*
        * 银铅矿的逻辑的有两步
        * 1.我们本身系统的校验，通过身份证和对应账号查询用户是否有效
        * 2.调用银铅矿提供的结构，查看是否携带芯片卡
        * 银铅矿下矿需要本人持证下矿
        * */
        Future<RestResult<Boolean>> userOepnJudgeResult = taskExecutor.submit(() -> this.userOepnJudgeUrl(shortTermDTO.getIdNumber(),shortTermDTO.getCardIdEx(), placeInfoVO.getAdminId()));
        Future<YqkRestResult<VerifyCardByStationStrictlyVo>> yqkOepnJudgeResult = taskExecutor.submit(() -> this.yqkOepnJudgeUrl(shortTermDTO.getCardIdEx(), placeInfoVO.getPass()));

        try {
            RestResult<Boolean> userRestResult = userOepnJudgeResult.get();
            ClassUtil.throwBusinessException(RestCode.SUCCESS.code != userRestResult.getCode(),userRestResult.getMsg());
        }catch (ServiceException e){
            throw e;
        }catch (ExecutionException e){
            throw new ServiceException("主服务异常，请联系管理员");
        } catch(Exception e) {
            log.info("报错了:",e);
            throw new RuntimeException("系统异常，请联系管理员");
        }
//        try {
//            YqkRestResult<VerifyCardByStationStrictlyVo> yqkRestResult = yqkOepnJudgeResult.get();
//            ClassUtil.throwBusinessException(0 != yqkRestResult.getStatus(),yqkRestResult.getMessage());
//            VerifyCardByStationStrictlyVo result = yqkRestResult.getResult();
//            ClassUtil.throwBusinessException(0 != result.getState(),result.getErrMsg());
//        }catch (ServiceException e){
//            throw e;
//        }catch (ExecutionException e){
//            throw new ServiceException("银铅矿人员定位接口异常");
//        } catch(Exception e) {
//            log.info("报错了:",e);
//            throw new RuntimeException("系统异常，请联系管理员");
//        }

        log.info("开门信号结果返回:{}", JSON.toJSONString(userVO));
        log.info("----->总耗时：{}", System.currentTimeMillis() - starTime);
        return RestResponse.ok(userVO);
    }

    private RestResult<Boolean> userOepnJudgeUrl(String idNumber,String cardIdEx, Integer adminId) {
        UserOpenJudgeDto dto = new UserOpenJudgeDto();
        dto.setIdNumber(idNumber);
        dto.setAdminId(adminId);
        dto.setCardIdEx(cardIdEx);
        RestResult<Boolean> request = HttpRequest.post(userOepnJudgeUrl)
                .body(JSONObject.toJSONString(dto)).timeout(4000).thenFunction(val -> {
                    String body = val.body();
                    log.info("主服务用户通行结果:{}",body);
                    return JSON.parseObject(body, new TypeReference<RestResult<Boolean>>() {
                    });
                });
        return request;
    }
    private YqkRestResult<VerifyCardByStationStrictlyVo> yqkOepnJudgeUrl(String cardIdEx, Integer pass) {
        VerifyCardByStationStrictlyDto dto = new VerifyCardByStationStrictlyDto();
        dto.setPositionCardID(cardIdEx);
        if(pass != 0){
            dto.setAttendType(pass);
        }
        YqkRestResult<VerifyCardByStationStrictlyVo> request = HttpRequest.get(yqkOepnJudgeUrl)
                .body(JSONObject.toJSONString(dto)).timeout(4000).thenFunction(val -> {
                    String body = val.body();
                    log.info("三方用户通行结果:{}",body);
                    return JSON.parseObject(body, new TypeReference<YqkRestResult<VerifyCardByStationStrictlyVo>>() {
                    });
                });
        return request;
    }


    @Override
    public RestResult<HealthCodeVO> queryInfoByBarCode(HealthCodeDTO healthCodeDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(healthCodeDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(healthCodeDTO.getAppSecret())) {
            return RestResponse.error("调用权限校验失败，请检查");
        }
        return healthCodeServer.queryInfoByBarCode(healthCodeDTO.getBarCode());
    }

    /**
     * 处理用户信息 身份证 姓名 手机号
     *
     * @param userVO
     * @param shortTermDTO
     */
    private void convertUserInfo(HealtCodeUserVO userVO, ShortTermDTO shortTermDTO) {
        //姓名和身份证号脱敏处理
        userVO.setUsername(Preconditions.isBlank(userVO.getUsername()) ? shortTermDTO.getName() : userVO.getUsername());
        userVO.setIdNumber(Preconditions.isBlank(userVO.getIdNumber()) ? shortTermDTO.getIdNumber() : userVO.getIdNumber());
        userVO.setShowIdNumber(Preconditions.isBlank(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setShowUsername(Preconditions.isBlank(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        userVO.setPhone(Preconditions.isBlank(userVO.getPhone()) ? shortTermDTO.getPhone() : userVO.getPhone());
        userVO.setASubLineFirst(Preconditions.isBlank(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        userVO.setASubLineSecond(Preconditions.isBlank(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
    }

    public static HealtCodeUserVO initHealthUserVo(ShortTermDTO shortTermDTO,PlaceInfoVO placeInfoVO) {
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        userVO.setCardIdEx(shortTermDTO.getCardIdEx());
        userVO.setIsOpen(0);
        userVO.setOpenVoiceMsg("请通行");
        userVO.setOpenMsg("请通行");
        //验证方式
        String vaccineNum = "身份证验证";
        if (Preconditions.isNotBlank(shortTermDTO.getHealthCode())) {
            vaccineNum = "二维码";
        }
        //全局背景色
        userVO.setBackgroundColor(ColorEnum.BLUE.getValue());
        //头部
        String title = "温馨提示";
        userVO.setTitle(title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());
        //中上部
        userVO.setVaccineIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        userVO.setVaccineNumColor("#2b6ef5");

        //中下左部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        v.setTitle("温馨提示");
        if(StrUtil.isNotBlank(placeInfoVO.getTopTitle())){
            v.setTitle(placeInfoVO.getTopTitle());
        }
        v.setContent("携带好随身物品").setContentColor("#4e5c74");
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));
        //中下右部
        userVO.setVaccineIcoTitle("验证方式");
        userVO.setVaccineNum(vaccineNum);
        //底部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt(StrUtil.EMPTY);
        codeTypeVO.setContent("请通行");
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        userVO.setTravelValidationTitle("动态行程卡");
        userVO.setTravelValidationIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        userVO.setTravelValidationDesc("行程描述");

        return userVO;
    }

}

