package com.yx.face.boot.aop;

import cn.hutool.core.util.StrUtil;
import com.yx.face.annotation.CheckRepeatCommit;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.model.enums.ChannelEnum;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.service.AdminService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.lang.reflect.Method;
import java.util.Objects;

@Component
@Aspect
@Slf4j
public class CheckRepeatCommitAspect {

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private AdminService adminService;

    @Pointcut("@annotation(com.yx.face.annotation.CheckRepeatCommit)")
    private void checkRepeatCommit() {

    }

    @Around("checkRepeatCommit()")
    public Object checkRepeatCommit(ProceedingJoinPoint joinPoint) throws Throwable {
        String methodName = joinPoint.getSignature().getName();
        Object target = joinPoint.getTarget();
        Method method = getMethodByClassAndName(target.getClass(), methodName);
        log.info("验证是否重复提交:" + "调用方法:" + method);
        String className = joinPoint.getTarget().getClass().getName();
        String bizKey = method.getName();
        int expireTime = 0;
        String redisKey = "";
        CheckRepeatCommit checkRepeatCommit = method.getAnnotation(CheckRepeatCommit.class);
        if (Objects.nonNull(checkRepeatCommit) && checkRepeatCommit.channel().equals(ChannelEnum.WEB)) {
            ChannelEnum channel = checkRepeatCommit.channel();
            expireTime = checkRepeatCommit.expireTime();
            AdminVO admin = adminService.getInfo();
            redisKey = getRepeatCommitLock(channel, className, bizKey, String.valueOf(admin.getAdminId()), expireTime);
            if (StringUtils.isBlank(redisKey)) {
                throw new Exception("异常");
            }
        }
        return joinPoint.proceed();
    }

    /**
     * 根据类和方法名得到方法
     */
    public Method getMethodByClassAndName(Class c, String methodName) {
        Method[] methods = c.getDeclaredMethods();
        for (Method method : methods) {
            if (method.getName().equals(methodName)) {
                return method;
            }
        }
        return null;
    }

    public String getRepeatCommitLock(ChannelEnum channel, String module, String bizKey, String userNo, int expireTime) {
        if (StringUtils.isEmpty(module) || StringUtils.isEmpty(bizKey)) {
            throw new ServiceException("getRepeatCommitLock 参数不能为空！");
        }
        String redisKey = channel + StrUtil.COLON + module + StrUtil.COLON + bizKey + StrUtil.COLON + userNo;
        long count = redisUtil.incr(redisKey, 1);
        if (count == 1) {
            if (expireTime == 0) {
                expireTime = 60;
            }
            redisUtil.expire(redisKey, expireTime);
            return redisKey;
        } else {
            return null;
        }
    }

}
