package com.yx.face.boot.aop;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.yx.face.boot.component.aop.MyAnnotation;
import com.yx.face.boot.core.Constants;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @description:
 * 作用说明，在对账号和设备表进行修改之后
 * 要向发送一个请求，告知一下哪些设备的缓存失效，需要重新来
 *
 *
 *
 * @ClassName DeviceCacheAspect
 * @Author WXG
 * @Date 2024/3/22 13:29
 */
@Component
@Aspect
@Slf4j
public class DeviceCacheAspect {

    @Resource
    private AdminDao adminDao;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Autowired
    RabbitTemplate rabbitTemplate;
    @Pointcut("@annotation(DeviceCacheAnnotation)")
    public void cache() {
    }

    @Pointcut("@annotation(DeviceCacheSnAnnotation)")
    public void cacheSn() {
    }
    @Around("cacheSn()")
    public Object around(ProceedingJoinPoint point)throws Throwable{

        DeviceCacheSnAnnotation annotation = getSnAnnotation(point);
        DeviceCacheEnum deviceCacheEnum = annotation.value();
        String sn = null;
        if(deviceCacheEnum.code.equals(DeviceCacheResultTypeEnum.sn)){
            sn = deviceCacheEnum.getService().get(point).toString();
        }
        FaceDevice startFaceDevice = faceDeviceDao.getFaceDeviceBySn(sn);
        //运行
        Object result = point.proceed();
        FaceDevice endFaceDevice = faceDeviceDao.getFaceDeviceBySn(sn);

        if(ObjectUtil.isNotNull(startFaceDevice) && ObjectUtil.isNotNull(endFaceDevice)){
            if (Objects.equals(startFaceDevice.getAdminId(), endFaceDevice.getAdminId())
            && Objects.equals(startFaceDevice.getAuth(), endFaceDevice.getAuth())
            && Objects.equals(startFaceDevice.getEnable(), endFaceDevice.getEnable())
            ){
                return result;
            }
        }
        if(StrUtil.isNotBlank(sn)){
            log.info("开始进行缓存操作,sn:{}",sn);
            rabbitTemplate.convertAndSend(Constants.DEVICE_CACHE_QUQUE, sn);
        }
        return result;
    }


    @After("cache()")
    public void after(JoinPoint point) throws Throwable {
        long startTime = System.currentTimeMillis();
        DeviceCacheAnnotation annotation = getAnnotation(point);
        DeviceCacheEnum deviceCacheEnum = annotation.value();
        List<String> snList = null;
        if(deviceCacheEnum.code.equals(DeviceCacheResultTypeEnum.admin_id)){
            //1.返回是adminId的情况
            Admin admin = adminDao.selectByPrimaryKey(deviceCacheEnum.getService().get(point));
            if(ObjectUtil.isNull(admin)){
                return;
            }
            if(!AdminTypeEnum.DISTRICT.getCode().equals(admin.getType()) && !AdminTypeEnum.ENTERPRISE.getCode().equals(admin.getType())){
                return;
            }
            //2.获取账号Id列表
            List<Integer> adminIdList = CollUtil.toList(admin.getId());
            if (AdminTypeEnum.DISTRICT.getCode().equals(admin.getType()) && (AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(admin.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(admin.getPlaceType()))) {
                List<Admin> adminList = adminDao.selectByExample(
                        Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, admin.getId())).select("id").build());
                for (Admin item : adminList) {
                    adminIdList.add(item.getId());
                }
            }
            //3.获取对应的设备列表
            List<FaceDevice> faceDeviceList = faceDeviceDao.selectByExample(
                    Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andIn(FaceDevice::getAdminId, adminIdList)).select("sn").build()
            );
            snList = faceDeviceList.stream().map(FaceDevice::getSn).collect(Collectors.toList());
        }
//        else if (deviceCacheEnum.code.equals(DeviceCacheResultTypeEnum.sn)) {
//            snList = CollUtil.toList(deviceCacheEnum.getService().get(point).toString());
//        }
        if(CollUtil.isEmpty(snList)){
            log.info("设备号为空，不需要传递sn");
            return;
        }
        for (String sn : snList) {
            if(StrUtil.isBlank(sn)){
                continue;
            }
            log.info("开始进行缓存操作,sn:{}",sn);
            rabbitTemplate.convertAndSend(Constants.DEVICE_CACHE_QUQUE, sn);
        }
        long endTime = System.currentTimeMillis();
        log.info("花费时间:{}",endTime-startTime);
    }

    private DeviceCacheAnnotation getAnnotation(JoinPoint pjp) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        // 获取注解属性
        return  ((MethodSignature) pjp.getSignature()).getMethod().getAnnotation(DeviceCacheAnnotation.class);
    }
    private DeviceCacheSnAnnotation getSnAnnotation(ProceedingJoinPoint pjp) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        // 获取注解属性
        return  ((MethodSignature) pjp.getSignature()).getMethod().getAnnotation(DeviceCacheSnAnnotation.class);
    }
}
