package com.yx.face.boot.config;

import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.core.Constants;
import com.yx.face.model.dto.issue.WebsocketMsgReq;
import com.yx.face.service.issue.IssueDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.drafts.Draft_6455;
import org.java_websocket.handshake.ServerHandshake;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.net.URI;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.Executor;

/**
 * @author shisl
 * @package com.yx.face.boot.config
 * @class WebsocketConfig
 * @date 2024/4/7 下午4:00
 * @description
 */
@Slf4j
@Configuration
@Component
public class WebsocketConfig {

    @Resource
    private Executor executor;

    @Resource
    private IssueDeviceService issueDeviceService;

    @Value("${websocket.wsSwitch}")
    private Boolean wsSwitch;

    @Bean(name = "otaWebSocketClient")
    public WebSocketClient webSocketClient(@Value("${websocket.uri}") String websocketUri,
                                           @Value("${websocket.token}") String websocketToken) {
        try {

            WebSocketClient webSocketClient = new WebSocketClient(new URI(String.format(websocketUri, websocketToken, Constants.SOURCE_TQSERVER)), new Draft_6455(), null, 60000) {
                //连接服务端时触发
                @Override
                public void onOpen(ServerHandshake handshakedata) {
                    log.info("websocket客户端和服务器连接成功");
                }

                //收到服务端消息时触发
                @Override
                public void onMessage(String message) {
                    //TODO。。。处理业务逻辑
                    handleBiz(message);
                }

                //和服务端断开连接时触发
                @Override
                public void onClose(int code, String reason, boolean remote) {
                    log.info("websocket客户端退出连接");
                }

                //连接异常时触发
                @Override
                public void onError(Exception ex) {
                    ex.printStackTrace();
                    log.info("websocket客户端和服务器连接发生错误={}", ex.getMessage());
                }
            };

            webSocketClient.connect();
            Timer t = new Timer();
            t.scheduleAtFixedRate(new TimerTask() {
                @Override
                public void run() {
                    log.info("进入定时器");
                    if (webSocketClient.isClosed()) {
                        log.error("断线重连");
                        webSocketClient.reconnect();
                    }
                }
            }, 1000, 5000);
            return webSocketClient;
        } catch (Exception e) {
            e.printStackTrace();
            log.error("--->websocket连接异常");
        }
        return null;
    }

    /**
     * 处理设备消息
     *
     * @param message
     */
    private void handleBiz(String message) {

        if (!wsSwitch) {
            log.info("---->关闭ws开关");
            return;
        }
        WebsocketMsgReq msgReq = JSONObject.parseObject(message, WebsocketMsgReq.class);
        log.info("--->msgReq:{}", JSONObject.toJSONString(msgReq));
        issueDeviceService.handleCallback(msgReq);
    }

}
