package com.yx.face.boot.consumer;

import com.alibaba.fastjson.JSONObject;
import com.yx.face.constant.RabbitMqConstants;
import com.yx.face.model.dto.hhface.AddSnIssuedMsg;
import com.yx.face.model.dto.hhface.DelFaceBySnMsg;
import com.yx.face.model.dto.hhface.DeleteBySnMsg;
import com.yx.face.model.dto.hhface.DownUserMsg;
import com.yx.face.model.dto.issue.WebsocketMsgReq;
import com.yx.face.service.FaceTBService;
import com.yx.face.service.impl.AsyncTriggerRetryImpl;
import com.yx.face.service.issue.IssueDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author shisl
 * @package com.yx.face.boot.rabbitConfig.listener
 * @class HhfaceIssueConsumer
 * @date 2024/9/10 下午3:46
 * @description
 */
@Slf4j
@Component
public class HhfaceIssueConsumer {

    @Resource
    private FaceTBService faceTBService;

    @Resource
    private AsyncTriggerRetryImpl asyncTriggerRetryImpl;

    @Resource
    private IssueDeviceService issueDeviceService;

    @RabbitListener(queues = RabbitMqConstants.GENERAL_USER_QUEUE_NAME)
    public synchronized void generalUserQueue(String message) {
        try {
            JSONObject obj = JSONObject.parseObject(message);
            String bizType = obj.get("bizType").toString();
            if (RabbitMqConstants.GENERAL_MESSAGE_TYPE_addSingleUser2Device.equals(bizType)) {
                DownUserMsg dto = JSONObject.parseObject(obj.get("data").toString(), DownUserMsg.class);
                log.info("----->消费单个用户下发消息：{}", JSONObject.toJSONString(dto.getDownFace()));
                faceTBService.downUser(dto.getDownFace(), dto.getTaskInfo());

            } else if (RabbitMqConstants.GENERAL_MESSAGE_TYPE_batchAddUser2Device.equals(bizType)) {
                AddSnIssuedMsg dto = JSONObject.parseObject(obj.get("data").toString(), AddSnIssuedMsg.class);
                log.info("----->消费批量用户下发消息：{}", JSONObject.toJSONString(dto));
                asyncTriggerRetryImpl.addSnIssued(dto.getFaceDevice(), dto.getAdminVO(), dto.getTaskInfo(), dto.getIssueType());

            } else if (RabbitMqConstants.GENERA_MESSAGE_TYPE_delUserFromDevice.equals(bizType)) {
                DelFaceBySnMsg dto = JSONObject.parseObject(obj.get("data").toString(), DelFaceBySnMsg.class);
                log.info("----->消费指定用户删除消息：{}", JSONObject.toJSONString(dto));
                faceTBService.delFaceBySn(dto.getSn(), dto.getUserIds(), dto.getTaskInfo());

            } else if (RabbitMqConstants.GENERAL_MESSAGE_TYPE_delUserBySn.equals(bizType)) {
                DeleteBySnMsg dto = JSONObject.parseObject(obj.get("data").toString(), DeleteBySnMsg.class);
                log.info("----->消费清空设备用户消息：{}", JSONObject.toJSONString(dto));
                faceTBService.deleteBySn(dto.getSn(), dto.getTaskInfo());
            }
        } catch (Exception e) {
            log.error("消费下发消息失败的数据体：{}", message);
            log.error("消费下发消息失败：", e);
        }

    }

    @RabbitListener(queues = RabbitMqConstants.GENERAL_USER_CALLBACK_QUEUE_NAME)
    public synchronized void generalUserCallbackQueue(String message) {
        WebsocketMsgReq msgReq = JSONObject.parseObject(message, WebsocketMsgReq.class);
        log.info("----->消费HHFACE回调消息：{}", message);
        issueDeviceService.handleCallback(msgReq);
    }


}
