package com.yx.face.boot.rabbitConfig.config;

import org.springframework.amqp.core.*;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.HashMap;
import java.util.Map;

/**
 * @description:
 * @ClassName SnQuenueConfig
 * @Author WXG
 * @Date 2024/3/26 9:40
 */
@Configuration
public class SnQuenueConfig {

    public static final String QUEUE_NAME = "snCacheQueueDelay";
    @Bean
    public Queue demoQueue() {

        Map<String,Object> arguments=new HashMap<>(3);
        //设置TTL 单位是ms
        arguments.put("x-message-ttl",5000);
        return QueueBuilder.durable(QUEUE_NAME).withArguments(arguments).build();
    }

    //创建交换机
    @Bean(name = "topi-exchange")
    public TopicExchange getExchange(){
        return new TopicExchange("topi-exchange", true, false);
    }
    //创建队列
    @Bean(name = "topi-queue")
    public Queue getQueue(){
        Map<String, Object> args = new HashMap<>();
        args.put("x-message-ttl",1000*3600*6); // 定义过期时间
        args.put("x-dead-letter-exchange", "dead-exchange"); // 与交换机产生关联
        args.put("x-dead-letter-routing-key","*.dead.*"); // 路由规则
        return new Queue("topi-queue", true, false, false,args);
    }
    //绑定队列
    @Bean
    public Binding topiBinding(@Qualifier("topi-exchange") TopicExchange topicExchange, @Qualifier("topi-queue") Queue queue){
        return BindingBuilder.bind(queue).to(topicExchange).with("*.topi.*");
    }

    //创建死信MQ
    //创建交换机
    @Bean(name = "dead-exchange")
    public TopicExchange deadExchange(){
        return new TopicExchange("dead-exchange", true, false);
    }
    //创建队列
    @Bean(name = "dead-queue")
    public Queue deadQueue(){
        return new Queue("dead-queue", true, false, false);
    }

    //绑定队列
    @Bean
    public Binding deadBinding(@Qualifier("dead-exchange") TopicExchange topicExchange, @Qualifier("dead-queue") Queue queue){
        return BindingBuilder.bind(queue).to(topicExchange).with("*.dead.*");
    }

}
