package com.yx.face.boot.uitls;


import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Random;
import java.util.UUID;

/**
 * @author shisl
 * @package com.yx.face.boot.uitls
 * @class AppUtils
 * @date 2022/10/25 下午6:22
 * @description appKey appSecret 生成工具类
 */
public class AppUtils {
    private final static String APP_KEY_PREFIX = "hh";
    private final static String SERVER_NAME = "hanghui_api_nqj##361";
    private final static String[] chars = new String[]{"a", "b", "c", "d", "e", "f",
            "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s",
            "t", "u", "v", "w", "x", "y", "z", "0", "1", "2", "3", "4", "5",
            "6", "7", "8", "9", "A", "B", "C", "D", "E", "F", "G", "H", "I",
            "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V",
            "W", "X", "Y", "Z"};

    /**
     * <p>
     * 短8位UUID思想其实借鉴微博短域名的生成方式，但是其重复概率过高，而且每次生成4个，需要随即选取一个。
     * 本算法利用62个可打印字符，通过随机生成32位UUID，由于UUID都为十六进制，
     * 所以将UUID + 时间戳 分成9组，每5个为一组，然后通过模62操作，结果作为索引取出字符，
     * 这样重复率大大降低。
     * 经测试，在生成一千万个数据也没有出现重复，完全满足大部分需求。
     * </p>
     *
     * @return
     */
    public static String getAppKey() {
        StringBuffer shortBuffer = new StringBuffer().append(APP_KEY_PREFIX);
        String uuid = UUID.randomUUID().toString().replace("-", "") + System.currentTimeMillis();
        for (int i = 0; i < 9; i++) {
            String str = uuid.substring(i * 5, i * 5 + 5);
            int x = Integer.parseInt(str, 16);
            shortBuffer.append(chars[x % 0x3E]);
        }
        return shortBuffer.toString();
    }

    /**
     * 算法sha1生成AppSecret
     *
     * @param appKey
     * @return
     */
    public static String getAppSecret(String appKey) {
        try {

            StringBuffer sb = new StringBuffer();
            String uuid = UUID.randomUUID().toString();
            sb.append(appKey).append(SERVER_NAME).append(uuid);

            String str = sb.toString();
            MessageDigest md = MessageDigest.getInstance("SHA-1");
            md.update(str.getBytes());
            byte[] digest = md.digest();

            StringBuffer hexstr = new StringBuffer();
            String shaHex = "";
            for (int i = 0; i < digest.length; i++) {
                shaHex = Integer.toHexString(digest[i] & 0xFF);
                if (shaHex.length() < 2) {
                    hexstr.append(0);
                }
                hexstr.append(shaHex);
            }
            return hexstr.toString();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return appKey;
    }

    /**
     * 返回一个定长的随机字符串(只包含大小写字母、数字)
     *
     * @return 随机字符串
     */
    public static String getPrivateKey() {

        return RandomUtils.generateString(16).toLowerCase();
    }

    public static void main(String[] args) {
        String appKey = getAppKey();
        String appSecret = getAppSecret(appKey);
        System.out.println("--->time: " + System.currentTimeMillis());

        System.out.println("--->appKey: " + appKey);
        System.out.println("--->appSecret: " + appSecret);
        System.out.println("--->privateKey: " + getPrivateKey());

    }
}
