package com.yx.face.boot.uitls;

import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.SymmetricCrypto;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;

import java.nio.charset.Charset;
/**
 * @author shisl
 * @package com.yx.face.boot.uitls
 * @class Sm4Util
 * @date 2023/6/2 上午10:52
 * @description
 */
@Slf4j
public class Sm4Util {

    /**
     * 加密
     *
     * @param privateKey 管理员提供
     * @param str
     * @return
     */
    public static String encrypt(final String privateKey, final String str) {
        if (StrUtil.isBlank((CharSequence) str)) {
            return "";
        }
        SymmetricCrypto sm4 = new SymmetricCrypto("SM4/ECB/PKCS5Padding", privateKey.getBytes());
        String sm4Encrypt = sm4.encryptHex(str, Charset.forName("UTF-8"));
        return sm4Encrypt;
    }

    /**
     * 解密
     *
     * @param privateKey 管理员提供
     * @param str
     * @return
     */
    public static String decrypt(final String privateKey, final String str) {
        if (StrUtil.isBlank((CharSequence) str)) {
            return null;
        }
        try {
            SymmetricCrypto sm4 = new SymmetricCrypto("SM4/ECB/PKCS5Padding", privateKey.getBytes());
            String sm4Decrypt = sm4.decryptStr(str, Charset.forName("UTF-8"));
            return sm4Decrypt;
        } catch (Exception e) {
            log.error("解密失败", e);
            return null;
        }
    }

    /**
     * 验证签名
     *
     * @param appKey
     * @param appSecret
     * @param sign
     * @param timestamp
     * @return
     */
    public static Boolean checkSign(String appKey, String appSecret, String sign, String timestamp) {
        if (Preconditions.isBlank(sign) || Preconditions.isBlank(timestamp)) {
            return Boolean.FALSE;
        }

        String dbSign = SecureUtil.md5(new StringBuilder().append(appKey)
                .append(appSecret)
                .append(timestamp).toString()).toLowerCase();
        if (!dbSign.equals(sign)) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    /**
     * 获取签名
     *
     * @param appKey
     * @param appSecret
     * @param timestamp
     * @return
     */
    public static String getSign(String appKey, String appSecret, String timestamp) {
        String sign = SecureUtil.md5(new StringBuilder().append(appKey)
                .append(appSecret)
                .append(timestamp).toString()).toLowerCase();
        return sign;
    }


    public static void main(String[] args) {
        //基础信息
//        Map<String, Object> map = new HashMap<>();
//        map.put("idNumber", "330481199704031831");
//        map.put("name", "王鑫刚");
//        map.put("phone", "13515834834");
//        map.put("qrcode", "");
//        map.put("sn", "1351583T234834");
//        map.put("verifyType", "IDCARD");

        System.out.println(AesUtils.generateKey());
        String s = AesUtils.encode("a2f0e24913f378add01b9f36546a2edb", "123456");
        System.out.println(s);
        System.out.println(AesUtils.decode("a2f0e24913f378add01b9f36546a2edb",s));


        JSONObject jsonObject = new JSONObject();
        jsonObject.put("sfzh", "330481199704031831");
//        String str = "欧阳明";
        String encrypt = Sm4Util.encrypt("dvhrjn7qycki05pd", jsonObject.toJSONString());
        System.out.println(encrypt);
        String decrypt = Sm4Util.decrypt("dvhrjn7qycki05pd", encrypt);
        System.out.println(decrypt);
        String timestamp = String.valueOf(System.currentTimeMillis());
        System.out.println(timestamp);
        System.out.println(Sm4Util.getSign("hha9LJw4TgV", "605f83d833be21f94582a920228e03fc9a072809", timestamp));

    }

}
