package com.yx.face.controller;
import java.util.Date;

import com.yx.face.dao.AdminDao;
import com.yx.face.dao.AdminPlaceInfoDao;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.AdminPlaceInfo;
import com.yx.face.model.entity.FaceDevice;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/7/6 16:51
 * @description:
 */
@Api(tags = "场所码数据清洗 API")
@RestController
@RequestMapping("/data/clean")
public class DataCleanController {
    @Autowired
    private AdminDao adminDao;
    @Autowired
    private AdminPlaceInfoDao adminPlaceInfoDao;
    @Autowired
    private FaceDeviceDao faceDeviceDao;

    @GetMapping
    public String dataClean(){
        //场所码信息
        List<AdminPlaceInfo> placeInfoList = adminPlaceInfoDao.selectAll();
        Map<Integer, List<AdminPlaceInfo>> placeMap = placeInfoList.stream().collect(Collectors.groupingBy(AdminPlaceInfo::getAdminId));
        //查询所有开启场所码的账号信息
        List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getPlaceCodeStatus, true)).build());
        adminList.forEach(v -> {
            //场所码信息为空处理
            if(placeMap.get(v.getId()) != null){
               return;
            }

            AdminPlaceInfo adminPlaceInfo = new AdminPlaceInfo().setAddressCode(v.getAddressCode()).setUnitName(v.getUnitName()).setUnitAddress(v.getUnitAddress()).setManagerHealthCode(v.getManagerHealthCode()).setAdminId(v.getId()).setUpdateTime(new Date()).setCreateTime(new Date());
            //新增场所码信息
            adminPlaceInfoDao.insertSelective(adminPlaceInfo);
            //更新设备绑定场所码信息
            faceDeviceDao.updateByExampleSelective(new FaceDevice().setAdminPlaceInfoId(adminPlaceInfo.getId()), Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getAdminId, v.getId())).build());
        });

        return "数据清洗完成";
    }
}
