package com.yx.face.controller;

import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.google.common.collect.Maps;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.boot.uitls.JsonUtils;
import com.yx.face.model.dto.fkj.*;
import com.yx.face.model.vo.fkj.CodeOpenCodeJudgeVo;
import com.yx.face.model.vo.fkj.CodeOpenJudgeVo;
import com.yx.face.model.vo.fkj.VisitorCodeCheckVo;
import com.yx.face.model.vo.third.VisitorReportPageVo;
import com.yx.face.model.vo.third.VisitorReportVo;
import com.yx.face.service.ThirdFaceService;
import com.yx.face.service.fkj.FkjService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @description:
 * @ClassName ThirdFaceController
 * @Author WXG
 * @Date 2022/4/7 12:53
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "C 第三方设备数据推送 API")
@RestController
@Validated
public class ThirdFaceController {

    private final ThirdFaceService thirdFaceService;

    private final FkjService fkjService;

    @ApiOperation("访客登记信息推送")
    @PostMapping("VisitorRecords")
    @ApiOperationSupport(order = 10)
    public Map<String, Object> VisitorRecords(HttpServletRequest request) {
        try {
            Map<String, Object> map = thirdFaceService.visitorRecords(getParam(request));
            log.info("访客登记返回数据:{}", map);
            return map;
        } catch (Exception e) {
            e.printStackTrace();
            log.error("访客机推送刷脸刷卡记录 异常={}", e.getMessage(), e);
        }
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "0");
        return result;
    }

    private Map<String, Object> getParam(HttpServletRequest request) {
        StringBuilder sb = new StringBuilder();
        try (InputStream is = request.getInputStream();
             BufferedInputStream bis = new BufferedInputStream(is)) {
            byte[] buffer = new byte[1024];
            int read;
            while ((read = bis.read(buffer)) != -1) {
                sb.append(new String(buffer, 0, read, StandardCharsets.UTF_8));
            }
        } catch (IOException e) {
            return Collections.EMPTY_MAP;
        }
        String paramStr = sb.toString();
        if (!StringUtils.hasText(paramStr)) {
            return Collections.EMPTY_MAP;
        }
        return JsonUtils.toMap(paramStr);
    }

    /*
     * 上面的接口一般都是李工使用
     * 为实现登记之后返回对应的登记二维码
     *
     * */
    @ApiOperation("访客机登记信息推送返回唯一值")
    @PostMapping("visitorReport")
    @ApiOperationSupport(order = 20)
    public RestResult<VisitorReportVo> visitorReport(@RequestBody @Valid VisitorReportDto dto) {
        return thirdFaceService.visitorReport(dto);
    }

    @ApiOperation("访客机-登记信息列表")
    @PostMapping("visitorReportPageList")
    @ApiOperationSupport(order = 30)
    public RestResult<PageInfo<VisitorReportPageVo>> visitorReportPageList(@RequestBody @Valid RestDTO<VisitorReportSearch> dto) {
        ClassUtil.searchInit(dto, VisitorReportSearch.class);
        return thirdFaceService.visitorReportPageList(dto);
    }

    @ApiOperation("访客机-列表签离")
    @PostMapping("visitorSignOut")
    @ApiOperationSupport(order = 40)
    public RestResult<Boolean> visitorSignOut(@RequestBody @Valid VisitorSignOutDto dto) {
        return thirdFaceService.visitorSignOut(dto);
    }

    @ApiOperation("设备刷码-开门判定（前端不用接）")
    @PostMapping("codeOpenJudge")
    @ApiOperationSupport(order = 50)
    public RestResult<CodeOpenJudgeVo> codeOpenJudge(@RequestBody @Valid CodeOpenJudgeDto dto) {
        return thirdFaceService.codeOpenJudge(dto);
    }

    @ApiOperation("访客机-刷码签离")
    @PostMapping("visitorCodeSignOut")
    @ApiOperationSupport(order = 60)
    public RestResult<CodeOpenCodeJudgeVo> visitorSignOut(@RequestBody @Valid CodeOpenJudgeDto dto) {
        return thirdFaceService.visitorSignOut(dto);
    }

    @ApiOperation("保安巡逻-刷码验签")
    @PostMapping("visitorCodeCheck")
    @ApiOperationSupport(order = 70)
    public RestResult<VisitorCodeCheckVo> visitorCodeCheck(@RequestBody @Valid VisitorCodeCheckDto dto) {
        return thirdFaceService.visitorCodeCheck(dto);
    }

    @ApiOperationSupport(order = 80)
    @ApiOperation(value = "获取楼层对应关系配置列表")
    @PostMapping("/floor/config/list")
    public RestResult<List<FkjFloorInitConfig>> floorConfigList(@RequestBody @Valid FkjFloorSearch search) {
        return RestResponse.ok(fkjService.floorConfigList(search));
    }


}
