/**
 * Copyright (C), 2018-2019, XXX有限公司
 * FileName: UploadAndDownController
 * Author:   pwb
 * Date:     2019/7/15 13:50
 * Description: 文件传输接口
 * History:
 * <author>          <time>          <version>          <desc>
 * frank           修改时间           1.0.0              ycypApp
 */
package com.yx.face.controller;

import com.yx.face.boot.component.aop.ServiceLog;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.AddressUtil;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.boot.uitls.FileUtils;
import com.yx.face.dao.SystemConfigDao;
import com.yx.face.service.FaceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * 〈文件传输接口〉
 *
 * @author pwb
 * @create 2019/7/15
 * @since 1.0.0
 */

@Slf4j
@Api(tags = "文件传输接口")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@RestController
@RequestMapping(value = "/upload")
public class UploadController extends BaseController {

    private final FaceService faceService;
    private final SystemConfigDao systemConfigDao;

   

    //上传图片
    @ApiOperation(value = "单图片上传接口", notes = "单图片上传接口")
    @PostMapping(value = "/picture")
    public RestResult<String> uploadImage(@RequestParam MultipartFile file) throws IOException {
        String fogImg = faceService.addFaceDeviceLogImg(file.getInputStream(), getUserId());
        return RestResponse.ok(fogImg);
    }

    //上传图片
    @ApiOperation(value = "单图片上传接口", notes = "单图片上传接口")
    @PostMapping(value = "/pictureLocal")
    public RestResult<String> pictureLocal(@RequestParam MultipartFile file) throws IOException {
        String filePath = FileUtils.getDirectory(Constants.UPLOAD_IMAGE_NEW);
        filePath = FileUtils.saveByCopy(filePath, file);
        filePath = AddressUtil.address.concat(filePath);
        return RestResponse.ok(filePath);
    }

    @ApiOperation(value = "视频上传接口", notes = "视频上传接口")
    @PostMapping(value = "/video")
    public RestResult<String> uploadVideo(@RequestParam MultipartFile file) throws IOException, ServletException {
        String filePath = FileUtils.getDirectory(Constants.UPLOAD_VIDEO);
        String rootFilePath = FileUtils.saveByCopy(filePath, file);
        rootFilePath = AddressUtil.address.concat(rootFilePath);
        return RestResponse.ok(rootFilePath);
    }

    @ApiOperation(value = "获取图片后台图片地址", notes = "获取图片后台图片地址")
    @PostMapping(value = "/getPicture")
    public RestResult<Map<String,String>> getPicture(){
        Map<String,Map<String,String>> map = systemConfigDao.getPicture();
        Map<String,String> stringMap = new HashMap<>();
        for(String key :map.keySet()){
            stringMap.put(key,map.get(key).get("system_value"));
        }
        return RestResponse.ok(stringMap);
    }
    @ApiOperation(value = "日志等级less/all")
    @GetMapping(value = "/updateLogPrint")
    public void updateLogPrint(@RequestParam String logPrint){
        ClassUtil.throwBusinessException(!logPrint.equals("less") && !logPrint.equals("all"),"请传入less/all");
        ServiceLog.updateLogPrint(logPrint);
    }
}
