package com.yx.face.controller.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.AdminDataDTO;
import com.yx.face.model.vo.AdminDeviceVO;
import com.yx.face.model.vo.stat.*;
import com.yx.face.service.AdminDataShowService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @description:
 * @ClassName AdminDataShow
 * @Author WXG
 * @Date 2021/11/12 13:40
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "B 数据大屏显示")
@RestController
@RequestMapping("admin/dataShow")
@Validated
public class AdminDataShowController {

    private final AdminDataShowService adminDataShowService;

    @ApiOperation("获取设备数量")
    @PostMapping("getDeviceNumber")
    @ApiOperationSupport(order = 10)
    public RestResult<AdminDeviceVO> getDeviceNumber() {
        return RestResponse.ok(adminDataShowService.getDeviceNumber());
    }
//    @ApiOperation("获取通行信息")
//    @PostMapping("getAdminData")
//    public RestResult<AdminDataVO> getAdminData() {
//        return RestResponse.ok(adminDataShowService.getGloablePassData());
//    }

    @ApiOperation("人员通行情况")
    @PostMapping("stat/pass")
    @ApiOperationSupport(order = 20)
    public RestResult<StatPassVO> statPass(@RequestBody @Valid AdminDataDTO dto) {
        return RestResponse.ok(adminDataShowService.statPass(dto));
    }
    @ApiOperation("人员年龄分布情况")
    @PostMapping("stat/age")
    @ApiOperationSupport(order = 30)
    public RestResult<StatAgeVO> statAge(@RequestBody @Valid AdminDataDTO dto) {
        return RestResponse.ok(adminDataShowService.statAge(dto));
    }
    @ApiOperation("人员性别分布情况")
    @PostMapping("stat/sex")
    @ApiOperationSupport(order = 40)
    public RestResult<StatSexVO> statSex(@RequestBody @Valid AdminDataDTO dto) {
        return RestResponse.ok(adminDataShowService.statSex(dto));
    }
    @ApiOperation("各时段在场人数情况")
    @PostMapping("stat/people")
    @ApiOperationSupport(order = 50)
    public RestResult<StatPeopleVO> statPeople(@RequestBody @Valid AdminDataDTO dto) {
        return RestResponse.ok(adminDataShowService.statPeople(dto));
    }
    @ApiOperation("人员来源地分布情况")
    @PostMapping("stat/source")
    @ApiOperationSupport(order = 60)
    public RestResult<List<StatSourceVO>> statSource(@RequestBody @Valid AdminDataDTO dto) {
        return RestResponse.ok(adminDataShowService.statSource(dto));
    }
    @ApiOperation("人员进出统计数据")
    @PostMapping("stat/pageList")
    @ApiOperationSupport(order = 70)
    public RestResult<IPage<StatDataVO>> pageList(@RequestBody RestDTO<StatDataSearch> dto) {
        return RestResponse.ok(adminDataShowService.pageList(dto));
    }
    @ApiOperation(value = "人员进出统计数据-导出",notes = "最多导出最前面的5万条数据")
    @PostMapping("stat/excelOut")
    @ApiOperationSupport(order = 80)
    public void excelOut(@RequestBody StatDataSearch dto, HttpServletResponse response) {
        adminDataShowService.excelOut(dto,response);
    }
}
