package com.yx.face.controller.admin;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yx.face.boot.component.security.Authority;
import com.yx.face.boot.config.CustomCellWriteHandler;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.AddressUtil;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.dao.FaceLogDao;
import com.yx.face.dao.UserVisitorListDao;
import com.yx.face.dao.UserWhitelistDao;
import com.yx.face.model.dto.FaceDeviceDTO;
import com.yx.face.model.dto.WaringLogDTO;
import com.yx.face.model.dto.WaringLogSureDTO;
import com.yx.face.model.entity.UserVisitorListDetail;
import com.yx.face.model.entity.UserWhitelistDetail;
import com.yx.face.model.enums.*;
import com.yx.face.model.excel.FaceLogVoExcel;
import com.yx.face.model.excel.UserWhitelistDetailVo;
import com.yx.face.model.query.FaceServerLogQu;
import com.yx.face.model.query.QueryUser;
import com.yx.face.model.search.FaceDeviceSearch;
import com.yx.face.model.search.FaceServerLogSearch;
import com.yx.face.model.search.WaringLogSearch;
import com.yx.face.model.vo.*;
import com.yx.face.model.vo.visitor.UserVisitorListDetailInfo;
import com.yx.face.service.AdminService;
import com.yx.face.service.FaceService;
import com.yx.face.service.UserInfoService;
import com.yx.face.service.WarningLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.thymeleaf.util.ListUtils;
import org.thymeleaf.util.SetUtils;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description: 地址编码表(AreaCode)表控制层 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-06-04 14:25:45 <br>
 */

@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "B 管理人脸下发相关API")
@RestController
@RequestMapping("admin/face")
public class AdminFaceController extends BaseController {

    private final FaceService faceService;
    private final WarningLogService warningService;

    private final AdminService adminService;

    private final UserInfoService userInfoService;

    private final FaceLogDao faceLogDao;

    private final UserVisitorListDao userVisitorListDao;

    private final UserWhitelistDao userWhitelistDao;

   

    @ApiOperation("获取设备列表")
    @PostMapping("doGetFaceDevices")
    public RestResult<PageInfo<FaceDeviceVO>> doGetFaceDevices(@RequestBody RestDTO<FaceDeviceSearch> dto) {
        return RestResponse.ok(faceService.getFaceDevices(dto));
    }

    @ApiOperation("获取商户平台列表")
    @PostMapping("doGetTenants")
    public RestResult<List<TenantVo>> doGetTenants() {
        return RestResponse.ok(faceService.doGetTenants());
    }

    @ApiOperation("获取设备列表 sn / name")
    @PostMapping("getFaceDeviceList")
    public RestResult<List<FaceDeviceVO>> getFaceDeviceList() {
        return RestResponse.ok(faceService.getFaceDeviceList());
    }

    @ApiOperation("授权设备使用权限")
    @Authority({1, 2})
    @GetMapping("doAuthFaceDevice/{deviceId}")
    public RestResult<Boolean> doAuthFaceDevice(@PathVariable Integer deviceId) {
        return RestResponse.ok(faceService.authFaceDevice(deviceId));
    }

    @ApiOperation("停止设备使用权限")
    @Authority({1, 2})
    @GetMapping("doStopFaceDevice/{deviceId}")
    public RestResult<Boolean> doStopFaceDevice(@PathVariable Integer deviceId) {
        return RestResponse.ok(faceService.stopFaceDevice(deviceId));
    }

    @ApiOperation("新增/编辑设备信息")
    @PostMapping("doEditFaceDevices")
    public RestResult<Boolean> doEditFaceDevices(@RequestBody FaceDeviceDTO dto) {
        if (dto.getAdminAreaType() != null) {
            if (dto.getAdminAreaType() == 3) {
                dto.setAreaType(FaceDeviceAreatypeEnum.PRIVATE_AREA.getCode());
            } else {
                dto.setAreaType(FaceDeviceAreatypeEnum.PUBLIC_AREA.getCode());
            }
        }
        AdminVO admin = adminService.getInfo();
        if (admin.getType().equals(AdminTypeEnum.ENTERPRISE.getCode()) && dto.getDeviceId() == null) {
            throw new ServiceException("企业不能添加设备");
        }
        if (dto.getAdminId() == null) {
            if (AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(admin.getPlaceType()) && AdminTypeEnum.DISTRICT.getCode().equals(admin.getType())) {
                dto.setAdminId(admin.getAdminId());
            } else {
                throw new ServiceException("请传入adminId");
            }
        }
        admin = adminService.getInfo(dto.getAdminId());
        /*如果是楼宇场景，并且是企业的情况下，而且开启私有区域的情况下 公私有信息填入*/
        if (AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(admin.getPlaceType()) //楼宇场景
                && AdminTypeEnum.ENTERPRISE.getCode().equals(admin.getType())//企业账号
                && AdminIsOpenPrivateAreaEnum.OPEN_PRIVATE_AREA.getCode().equals(admin.getIsOpenPrivateArea())//开启私有区域
        ) {
            if (dto.getAreaType() == null) {
                throw new ServiceException("请选择公私有区域");
            }
            if (FaceDeviceAreatypeEnum.PRIVATE_AREA.getCode().equals(dto.getAreaType())) {//如果选择了私有区域
                if (dto.getPrivateAreaId() == null) {
                    throw new ServiceException("请选择私有区域填入");
                }
            }
        }
        if (AdminTypeEnum.DISTRICT.getCode().equals(admin.getType())) {
            dto.setAdminAreaType(1);
        }
        if (dto.getDeviceId() == null) {
            if (!StringUtils.hasText(dto.getSn())) {
                throw new ServiceException("请填写设备编号");
            }
            if (dto.getEnable() == null) {
                throw new ServiceException("请设置使用状态");
            }
            if (dto.getEnable() < 0 || dto.getEnable() > 1) {
                throw new ServiceException("请设置使用状态不规范");
            }
            faceService.addFaceDevice(dto);
        } else {
            if (dto.getEnable() != null) {
                if (dto.getEnable() < 0 || dto.getEnable() > 1) {
                    dto.setEnable(null);
                }
            }
            if (!StringUtils.hasText(dto.getSn())) {
                dto.setSn(null);
            }
            faceService.modifyFaceDevice(dto);
        }
        return RestResponse.ok(true);
    }

    @ApiOperation("删除设备")
    @DeleteMapping("doDeleteDevice/{deviceId}")
    public RestResult<Boolean> doDeleteDevice(@PathVariable Integer deviceId) {
        return RestResponse.ok(faceService.deleteDevice(deviceId));
    }

    @ApiOperation("设备重启")
    @GetMapping("doReboot")
    public RestResult<Boolean> doReboot(String sn) {
        if (org.apache.commons.lang3.StringUtils.isBlank(sn)) {
            throw new ServiceException("设备编号为空");
        }
        return RestResponse.ok(faceService.doReboot(sn));
    }


    @ApiOperation("获取人脸下发列表")
    @PostMapping("doGetFacePass")
    public RestResult<PageInfo<FacePassVO>> doGetFacePass(@RequestBody RestDTO<QueryUser> dto) {
        reBuildQuery(dto);
        return RestResponse.ok(faceService.getFacePasses(dto));
    }
    @ApiOperation("获取原因列表")
    @GetMapping("getFaceLogStatusEnum")
    public RestResult<List<UserWhitePersonelTypeVO>> getUserWhitePersonnelTypeEnum() {
        List<UserWhitePersonelTypeVO> list = new LinkedList<>();
        for (FaceLogStatusEnum value : FaceLogStatusEnum.values()) {
            list.add(new UserWhitePersonelTypeVO().setCode(value.getCode()).setDesc(value.getDesc()));
        }
        return RestResponse.ok(list);
    }

    @ApiOperation("获取人脸服务日志列表")
    @PostMapping("doGetFaceServerLog")
    public RestResult<PageInfo<FaceLogVO>> doGetFaceServerLog(@RequestBody RestDTO<FaceServerLogSearch> dto) {
        return RestResponse.ok(faceService.getFaceLogs(dto));
    }

    @ApiOperation("获取通行记录详情")
    @PostMapping("/getDetails")
    public RestResult<FaceLogVO> getDetails(@RequestBody FaceServerLogQu qu) {
        if (qu.getId() == null) {
            throw new ServiceException("缺少参数哦~");
        }
        return RestResponse.ok(faceService.getDetails(qu, getUserId()));
    }

    @ApiOperation("导出人脸日志Excel")
    @PostMapping("doGetFaceServerLogExcel")
    public RestResult<String> doGetFaceServerLogExcel(@RequestBody FaceServerLogSearch dto) {
        List<FaceLogVO> list2 = Lists.newArrayList();
        List<FaceLogVoExcel> faceLogsExcel = faceService.getFaceLogsExcel(dto, list2);
        AdminVO info = adminService.getInfo();
        //常客身份证
        List<FaceLogVO> white = list2.stream().filter(p -> p.getRightno().equals(105)).collect(Collectors.toList());
        //访客身份证
        List<FaceLogVO> visitor = list2.stream().filter(p -> p.getRightno().equals(104)).collect(Collectors.toList());
        Set<String> whiteCardIds = white.stream().map(v -> v.getCardid()).collect(Collectors.toSet());
        Set<String> visitorCardIds = visitor.stream().map(i -> i.getCardid()).collect(Collectors.toSet());
        HashMap<Object, List<ContextVO>> whiteMap = new HashMap<>();

        Map<String, List<UserWhitelistDetailVo>> whitelistIdNumberMap = new HashMap<>();
        if (!SetUtils.isEmpty(whiteCardIds)) {
            //常客信息
            List<FaceLogVO> whiteContent = faceLogDao.getContentByWhiteCardIds(whiteCardIds, info.getAdminId());
            for (int i = 0; i < whiteContent.size(); i++) {
                List<ContextVO> contextVOS = JSONArray.parseArray(whiteContent.get(i).getContent(), ContextVO.class);
                whiteMap.put(whiteContent.get(i).getCardid(), contextVOS);
            }
            //获取所有常客列表id集合
            List<Integer> userWhitelistIds = whiteContent.stream().filter(item -> Preconditions.isNotBlank(item.getUserWhitelistId())).map(FaceLogVO::getUserWhitelistId).collect(Collectors.toList());
            //获取访客机字段
            if (Preconditions.isNotBlank(userWhitelistIds)) {
                List<UserWhitelistDetail> userWhitelistDetails = userWhitelistDao.selectWhiteListFromByUserWhitelistIds(userWhitelistIds);
                if (Preconditions.isNotBlank(userWhitelistDetails)) {
                    Map<Integer, FaceLogVO> userWhitelistMap = whiteContent.stream().collect(Collectors.toMap(FaceLogVO::getUserWhitelistId, v -> v, (v1, v2) -> v1));
                    List<UserWhitelistDetailVo> userWhitelistDetailVos = BeanUtil.copyToList(userWhitelistDetails, UserWhitelistDetailVo.class);
                    userWhitelistDetailVos.stream().forEach(item -> {
                        if (Preconditions.isNotBlank(userWhitelistMap.get(item.getUserWhitelistId()))) {
                            item.setIdNumber(userWhitelistMap.get(item.getUserWhitelistId()).getCardid());
                        }
                    });
                    whitelistIdNumberMap = userWhitelistDetailVos.stream().collect(Collectors.groupingBy(UserWhitelistDetailVo::getIdNumber));
                }
            }
        }

        Map<String, List<UserVisitorListDetail>> visitorIdNumberMap = new HashMap<>();
        if (!SetUtils.isEmpty(visitorCardIds)) {
            //访客信息
            List<FaceLogVO> visitorContent = faceLogDao.getContentByVisitorCardIds(visitorCardIds, info.getAdminId());
            for (int i = 0; i < visitorContent.size(); i++) {
                List<ContextVO> contextVOS2 = JSONArray.parseArray(visitorContent.get(i).getContent(), ContextVO.class);
                whiteMap.put(visitorContent.get(i).getCardid(), contextVOS2);
            }
            //获取所有访客列表id集合
            List<Integer> userVisitorListIds = visitorContent.stream().filter(item -> Preconditions.isNotBlank(item.getUserVisitorListId())).map(FaceLogVO::getUserVisitorListId).collect(Collectors.toList());
            //获取访客机字段
            if (Preconditions.isNotBlank(userVisitorListIds)) {
                Map<Integer, FaceLogVO> userVisitorListMap = visitorContent.stream().collect(Collectors.toMap(FaceLogVO::getUserVisitorListId, v -> v, (v1, v2) -> v1));
                List<UserVisitorListDetail> userVisitorListDetails = userVisitorListDao.selectVisitorFromByUserVisitorListIds(userVisitorListIds);
                if (Preconditions.isNotBlank(userVisitorListDetails)) {
                    userVisitorListDetails.stream().forEach(item -> {
                        if (Preconditions.isNotBlank(userVisitorListMap.get(item.getUserVisitorListId()))) {
                            item.setIdNumber(userVisitorListMap.get(item.getUserVisitorListId()).getCardid());
                        }
                    });
                    visitorIdNumberMap = userVisitorListDetails.stream().collect(Collectors.groupingBy(UserVisitorListDetail::getIdNumber));
                }
            }
        }

        //补全访客机字段
        completeVisitorInfo(faceLogsExcel, whitelistIdNumberMap, visitorIdNumberMap);

        //访客信息
        Map<String, String> stringMap = Maps.newHashMap();
        String contextById = faceLogDao.getContextById(info.getAdminId());
        List<ContextVO> titleList = JSONArray.parseArray(contextById, ContextVO.class);
        //构造无值头部信息
        List<ContextVO> headTitleList = JSON.parseArray(JSON.toJSONString(titleList), ContextVO.class);
        for (int i = 0; i < list2.size(); i++) {
            String cardId = list2.get(i).getCardid();
            List<ContextVO> contextVOList = whiteMap.get(cardId);
            if (!ListUtils.isEmpty(contextVOList)) {
                stringMap = contextVOList.stream().
                        filter(v -> v != null && StrUtil.isNotEmpty(v.getCustomNameVal()))
                        .collect(Collectors.toMap(ContextVO::getId, ContextVO::getCustomNameVal, (v1, v2) -> v1));
                if(CollUtil.isNotEmpty(titleList)){
                    for (ContextVO context : titleList) {
                        context.setCustomNameVal(stringMap.get(context.getId()));
                    }
                }
            } else {
                titleList = JSON.parseArray(JSON.toJSONString(headTitleList), ContextVO.class);
            }

            extracted(faceLogsExcel, i, titleList);
        }

        String uploadExcel = Constants.UPLOAD_EXCEL;
        String fileName = uploadExcel + "/游客进出记录-" + System.currentTimeMillis() + ".xlsx";
        // 这里 需要指定写用哪个class去写
        ExcelWriter excelWriter = null;
        try {
            Path directory = Paths.get(uploadExcel);
            if (!Files.exists(directory)) {
                Files.createDirectories(directory);
            }

            List<ContextVO> context = faceService.getContextById(info.getAdminId());
            List<List<String>> head = head();
            if (!StringUtils.isEmpty(context)) {
                for (int i = 0; i < context.size(); i++) {
                    List<String> list1 = new ArrayList<String>();
                    list1.add(context.get(i).getCustomName());
                    head.add(list1);
                }
            }
            //     excelWriter.write(faceLogsExcel, writeSheet);
            EasyExcel.write(fileName).head(head)
                    .registerWriteHandler(new CustomCellWriteHandler())
                    .sheet("游客进出")
                    .doWrite(faceLogsExcel);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 千万别忘记finish 会帮忙关闭流
            if (excelWriter != null) {
                excelWriter.finish();
            }
        }
        String filePath = AddressUtil.address.concat(fileName);
        return RestResponse.ok(filePath);
    }

    /**
     * 补全访客机字段
     *
     * @param faceLogsExcel
     * @param whitelistIdNumberMap
     * @param visitorIdNumberMap
     */
    private void completeVisitorInfo(List<FaceLogVoExcel> faceLogsExcel, Map<String, List<UserWhitelistDetailVo>> whitelistIdNumberMap, Map<String, List<UserVisitorListDetail>> visitorIdNumberMap) {
        if (Preconditions.isBlank(faceLogsExcel)) {
            return;
        }
        faceLogsExcel.stream().forEach(item -> {
            Date createTime = item.getCreateTime();
            if (Preconditions.isNotBlank(item.getRightNoInt()) && item.getRightNoInt().intValue() == 105) {
                //常客
                if (Preconditions.isNotBlank(whitelistIdNumberMap) && Preconditions.isNotBlank(whitelistIdNumberMap.get(item.getShowCardid()))) {
                    List<UserWhitelistDetailVo> userWhitelistDetailVos = whitelistIdNumberMap.get(item.getShowCardid()).stream().filter(f -> Preconditions.isNotBlank(f.getTime())).collect(Collectors.toList());
                    if (Preconditions.isNotBlank(userWhitelistDetailVos)) {
                        userWhitelistDetailVos.forEach(v -> v.setSortTimeSub(Math.abs(createTime.getTime() - DateUtils.parse(v.getTime(), DateUtils.YYYY_MM_DD_HH_MM_SS).getTime())));
                        //根据时间倒叙
                        List<UserWhitelistDetailVo> desc = userWhitelistDetailVos.stream().sorted(Comparator.comparing(UserWhitelistDetailVo::getSortTimeSub)).collect(Collectors.toList());
                        UserWhitelistDetailVo detail = desc.get(0);
                        item.setCompany(detail.getCompany());
                        item.setTemperature(detail.getTemperature());
                        item.setCarno(detail.getCarno());
                        item.setReason(detail.getReason());
                        item.setTime(detail.getTime());
                        item.setOrg(detail.getOrg());
                        item.setVisiteeCompany(detail.getVisiTeeCompany());
                        item.setVisiteeXm(detail.getVisiTeeXm());
                        item.setVisiteeBm(detail.getVisiTeeBm());
                        item.setVisiteeRoom(detail.getVisiTeeRoom());
                        item.setPcsdm(detail.getPcsdm());
                        item.setPcsname(detail.getPcsname());
                    }
                }
            }

            if (Preconditions.isNotBlank(item.getRightNoInt()) && item.getRightNoInt().intValue() == 104) {
                //访客
                if (Preconditions.isNotBlank(visitorIdNumberMap) && Preconditions.isNotBlank(visitorIdNumberMap.get(item.getShowCardid()))) {
                    List<UserVisitorListDetail> userVisitorListDetails = visitorIdNumberMap.get(item.getShowCardid()).stream().filter(f -> Preconditions.isNotBlank(f.getTime())).collect(Collectors.toList());
                    if (Preconditions.isNotBlank(userVisitorListDetails)) {
                        userVisitorListDetails.forEach(v -> v.setSortTimeSub(createTime.getTime() - DateUtils.parse(v.getTime(), DateUtils.YYYY_MM_DD_HH_MM_SS).getTime()));
                        //根据时间倒叙
                        UserVisitorListDetail detail = userVisitorListDetails.stream().sorted(Comparator.comparing(UserVisitorListDetail::getSortTimeSub)).collect(Collectors.toList()).get(0);
                        item.setCompany(detail.getCompany());
                        item.setTemperature(detail.getTemperature());
                        item.setCarno(detail.getCarno());
                        item.setReason(detail.getReason());
                        item.setTime(detail.getTime());
                        item.setOrg(detail.getOrg());
                        item.setVisiteeCompany(detail.getVisiteeCompany());
                        item.setVisiteeXm(detail.getVisiteeXm());
                        item.setVisiteeBm(detail.getVisiteeBm());
                        item.setVisiteeRoom(detail.getVisiteeRoom());
                        item.setPcsdm(detail.getPcsdm());
                        item.setPcsname(detail.getPcsname());
                        item.setMachineId(detail.getMachineId());
                    }
                }
            }
            item.setShowName(null);
            item.setShowCardid(null);
            item.setShowPhone(null);
            item.setRightNoInt(null);
        });
    }

    private void extracted(List<FaceLogVoExcel> faceLogsExcel, int i, List<ContextVO> visitorContent) {
        if (!ListUtils.isEmpty(visitorContent)) {
            List<String> customNameVal = visitorContent.stream().map(ContextVO::getCustomNameVal).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(customNameVal)) {
                int contentSize = customNameVal.size();
                if (contentSize == BigDecimal.ROUND_DOWN) {
                    faceLogsExcel.get(i).setContent1(customNameVal.get(0));
                } else if (contentSize == BigDecimal.ROUND_CEILING) {
                    faceLogsExcel.get(i).setContent1(customNameVal.get(0));
                    faceLogsExcel.get(i).setContent2(customNameVal.get(1));
                } else if (contentSize == BigDecimal.ROUND_FLOOR) {
                    faceLogsExcel.get(i).setContent1(customNameVal.get(0));
                    faceLogsExcel.get(i).setContent2(customNameVal.get(1));
                    faceLogsExcel.get(i).setContent3(customNameVal.get(2));
                } else if (contentSize == BigDecimal.ROUND_HALF_UP) {
                    faceLogsExcel.get(i).setContent1(customNameVal.get(0));
                    faceLogsExcel.get(i).setContent2(customNameVal.get(1));
                    faceLogsExcel.get(i).setContent3(customNameVal.get(2));
                    faceLogsExcel.get(i).setContent4(customNameVal.get(3));
                } else {
                    faceLogsExcel.get(i).setContent1(customNameVal.get(0));
                    faceLogsExcel.get(i).setContent2(customNameVal.get(1));
                    faceLogsExcel.get(i).setContent3(customNameVal.get(2));
                    faceLogsExcel.get(i).setContent4(customNameVal.get(3));
                    faceLogsExcel.get(i).setContent5(customNameVal.get(4));
                }
            }
        }

    }

    private List<List<String>> head() {
        List<String> head = CollUtil.toList
                ("设备编号","设备地点","用户姓名","现场照片","证件号","卡号","手机号码","进/出","验证类型","人员类型","通行时间","状态","原因",
                        "来访单位","体温","车牌号码","访问事由","来访时间","访客机所在的使用单位","受访人的单位名称","受访人姓名","受访人部门","受访人单位的房间号","派出所代码","派出所名称","访客机编号");

        List<List<String>> list = new ArrayList<List<String>>();
        for (String headString : head) {
            List<String> head0 = new ArrayList<>();
            head0.add(headString);
            list.add(head0);
        }
        return list;
    }

    @ApiOperation("获取人员列表")
    @PostMapping("doGetUsers")
    public RestResult<PageInfo<UserInfoVO>> doGetUsers(@RequestBody RestDTO<QueryUser> dto) {
        reBuildQuery(dto);
        return RestResponse.ok(userInfoService.getUsers(dto));
    }


    private void reBuildQuery(@RequestBody RestDTO<QueryUser> dto) {
        AdminVO adminVO = adminService.getAuthInfo(getUserId());
        QueryUser queryUser = dto.getData();
        if (queryUser == null) {
            queryUser = new QueryUser();
        }
        if (adminVO.getType() == 2) {
            queryUser.setAdminId(adminVO.getAdminId());
        }
        dto.setData(queryUser);
    }

    @ApiOperation("设备logo下发")
    @GetMapping("issuedAll")
    public RestResult<Boolean> issuedAll(Integer type, Integer adminId, String sn) {
        if (type == null || type >= 4) {
            throw new ServiceException("所选下发类型不存在");
        }
        return RestResponse.ok(faceService.issuedAll(type, adminId, sn));
    }

    @ApiOperation("获取logo")
    @GetMapping("getLogo")
    public RestResult<String> getLogo() {
        return RestResponse.ok(faceService.getLogo(getUserId()));
    }

    @ApiOperation("根据访客id获取访客自定义字段")
    @GetMapping("getContext")
    public RestResult<List<ContextVO>> getContext(Integer id) {
        return RestResponse.ok(faceService.getContext(id));
    }

    @ApiOperation("根据账号id获取自定义字段")
    @GetMapping("getContextById")
    public RestResult<List<ContextVO>> getContextById(Integer adminId) {
        return RestResponse.ok(faceService.getContextById(adminId));
    }

    @ApiOperation("根据常客id获取自定义字段")
    @GetMapping("getContextByWhiteId")
    public RestResult<List<ContextVO>> getContextByWhiteId(Integer id) {
        return RestResponse.ok(faceService.getContextByWhiteId(id));
    }

    @ApiOperation("根据进出纪录获取自定义字段")
    @GetMapping("getContextByFaceLog")
    public RestResult<List<ContextVO>> getContextByFaceLog(Integer logId, Integer type) {
        FaceLogVO faceLogVO = faceService.getFaceLog(logId);
        if (ObjectUtils.isEmpty(faceLogVO)) {
            List<ContextVO> contextVO = new ArrayList<>();
            return RestResponse.ok(contextVO);
        }
        Integer adminId = faceLogVO.getAdminId();
        String cardId = faceLogVO.getCardid();
        if (type == 104) {
            return RestResponse.ok(faceService.getContextByFaceLogVisitor(adminId, cardId));
        } else if (type == 105) {
            return RestResponse.ok(faceService.getContextByFaceLogWhite(adminId, cardId, faceLogVO));
        }
        List<ContextVO> contextById = faceService.getContextById(adminId);
        return RestResponse.ok(contextById);
    }

    @ApiOperation("根据常客id获取常客信息")
    @GetMapping("getWhiteListFrom")
    public RestResult<UserVisitorListDetailInfo> getWhiteListFrom(Integer id) {
        return RestResponse.ok(faceService.getWhiteListFrom(id));
    }

    @ApiOperation("根据访客id获取访客信息")
    @GetMapping("getVisitorListFrom")
    public RestResult<UserVisitorListDetailInfo> getVisitorListFrom(Integer id) {
        return RestResponse.ok(faceService.getVisitorListFrom(id));
    }

    @ApiOperation("根据进出记录获取访客/常客信息")
    @GetMapping("getUserListFrom")
    public RestResult<UserVisitorListDetailInfo> getUserListFrom(Integer logId, Integer type) {
        FaceLogVO faceLogVO = faceService.getFaceLog(logId);
        if (ObjectUtils.isEmpty(faceLogVO)) {
            return RestResponse.ok(new UserVisitorListDetailInfo());
        }
        Integer adminId = faceLogVO.getAdminId();
        String cardId = faceLogVO.getCardid();
        String s = "yyyy-MM-dd HH:mm:ss";
        SimpleDateFormat f = new SimpleDateFormat(s);
        String time = f.format(faceLogVO.getCreateTime());
        if (type == 104) {
            return RestResponse.ok(faceService.getUserListFrom(adminId, cardId, time));
        } else if (type == 105) {
            return RestResponse.ok(faceService.getUserListFromTwo(adminId, cardId, time));
        }
        return RestResponse.ok(new UserVisitorListDetailInfo());
    }


    @ApiOperation("预警记录-分页列表")
    @PostMapping("doGetWaringServerLog")
    @ApiOperationSupport(order = 50)
    public RestResult<PageInfo<WaringLogVo>> doGetWaringServerLog(@RequestBody RestDTO<WaringLogSearch> dto) {
        ClassUtil.searchInit(dto,WaringLogSearch.class);
        return RestResponse.ok(warningService.getWaringLogs(dto));
    }
    @ApiOperation("预警记录excel-导出")
    @PostMapping("doGetWaringServerLogExcel")
    @ApiOperationSupport(order = 60)
    public String doGetWaringServerLogExcel(@RequestBody WaringLogSearch dto, HttpServletResponse response){
        return warningService.doGetWaringServerLogExcel(dto,response);
    }
    @ApiOperation("预警记录-确认")
    @PostMapping("warningSure")
    @ApiOperationSupport(order = 70)
    public RestResult<Boolean> warningSure(@RequestBody WaringLogSureDTO dto){
        return warningService.warningSure(dto);
    }
    @ApiOperation("预警记录-删除")
    @PostMapping("warningDelete")
    @ApiOperationSupport(order = 80)
    public RestResult<Boolean> warningDelete(@RequestBody WaringLogDTO dto){
        return warningService.warningDelete(dto);
    }
    @ApiOperation("预警记录-拉黑")
    @PostMapping("warningBlack")
    @ApiOperationSupport(order = 90)
    public RestResult<Boolean> warningBlack(@RequestBody WaringLogDTO dto){
        return warningService.warningBlack(dto);
    }
}

