package com.yx.face.controller.admin;

import com.github.pagehelper.PageInfo;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.model.api.fkj.dto.VisiteeDto;
import com.yx.face.model.api.fkj.dto.VisiteePageDto;
import com.yx.face.model.dto.fkj.FkjFloorInitConfig;
import com.yx.face.model.entity.Visitee;
import com.yx.face.service.fkj.FkjService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;


@Api(tags = "访客机管理")
@Slf4j
@RestController
@RequestMapping("admin/fkj")
public class AdminFkjController extends BaseController {

    @Autowired
    private FkjService fkjService;


    @ApiOperation(value = "添加/编辑受访人")
    @PostMapping("/visitee/add")
    public RestResult<Boolean> add(@Valid @RequestBody VisiteeDto visiteeDto) {

        if (Preconditions.isBlank(visiteeDto.getAdminId())) {
            visiteeDto.setAdminId(getUserId());
        }
        return RestResponse.ok(fkjService.add(visiteeDto));
    }

    @ApiOperation(value = "删除受访人")
    @PostMapping("/visitee/del")
    public RestResult<Boolean> del(@RequestBody List<Long> ids) {
        return RestResponse.ok(fkjService.del(ids));
    }

    @ApiOperation(value = "获取受访人列表（分页）")
    @PostMapping("/visitee/page")
    public RestResult<PageInfo<Visitee>> page(@RequestBody RestDTO<VisiteePageDto> pageDtoRestDTO) {
        reBuildQuery(pageDtoRestDTO);
        return RestResponse.ok(fkjService.page(pageDtoRestDTO));
    }

    @ApiOperation(value = "批量删除受访人")
    @PostMapping("/visitee/batch/del")
    public RestResult<Boolean> batchDel(@RequestBody RestDTO<VisiteePageDto> pageDtoRestDTO) {
        reBuildQuery(pageDtoRestDTO);
        return RestResponse.ok(fkjService.batchDel(pageDtoRestDTO));
    }

    @ApiOperation(value = "获取楼层对应关系配置列表")
    @PostMapping("/floor/config/list")
    public RestResult<List<FkjFloorInitConfig>> floorConfigList() {
        return RestResponse.ok(fkjService.floorConfigList(getUserId()));
    }

    @ApiOperation(value = "添加楼层对应关系配置")
    @PostMapping("/floor/config/add")
    public RestResult floorConfigAdd(@RequestBody List<FkjFloorInitConfig> dto) {
        fkjService.floorConfigAdd(dto, getUserId());
        return RestResponse.ok();
    }

    private void reBuildQuery(@RequestBody RestDTO<VisiteePageDto> dto) {
        VisiteePageDto object = dto.getData();
        if (object == null) {
            object = new VisiteePageDto();
        }
        if (Preconditions.isBlank(object.getAdminId())) {
            object.setAdminId(getUserId());
        }
        dto.setData(object);
    }


}
