package com.yx.face.controller.admin;


import com.alibaba.excel.EasyExcel;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.ExcelListenerUtils;
import com.yx.face.model.dto.BussinessNumberDTO;
import com.yx.face.model.excel.BussinessNubmerExcel;
import com.yx.face.model.search.BussinessNumberSearch;
import com.yx.face.model.vo.BussinessNumberVO;
import com.yx.face.service.BussinessNumberService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author wxg
 * @since 2022-03-17
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "C 商户号管理")
@RestController
@RequestMapping("admin/bussinessNumber")
public class BussinessNumberController {
    private final BussinessNumberService bussinessNumberService;

    @ApiOperation("编辑商户号")
    @PostMapping("edit")
    public RestResult<Boolean> edit(@RequestBody BussinessNumberDTO bussinessNumberDTO) {
        Integer insert = bussinessNumberService.edit(bussinessNumberDTO);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }

    @ApiOperation("删除商户号")
    @GetMapping("delete")
    public RestResult<Boolean> delete(@RequestParam("bussinessNumberId") Integer bussinessNumberId) {
        Integer insert = bussinessNumberService.delete(bussinessNumberId);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("删除失败");
    }
    @ApiOperation("商户号列表(id 和 name)")
    @PostMapping("/getList")
    public RestResult<List<BussinessNumberVO>> getList() {
        List<BussinessNumberVO> list = bussinessNumberService.getList();
        if (list != null) {
            return RestResponse.ok(list);
        }
        return RestResponse.error("获取商户号列表失败");
    }
    @ApiOperation("商户号列表分页")
    @PostMapping("/getPageList")
    public RestResult<PageInfo<BussinessNumberVO>> getPageList(@RequestBody RestDTO<BussinessNumberSearch> dto) {
        reBuildQuery(dto);
        List<BussinessNumberVO> list = bussinessNumberService.getPageList(dto);
        if (list != null) {
            return RestResponse.ok(new PageInfo<>(list));
        }
        return RestResponse.error("获取商户号列表分页失败");
    }
    private void reBuildQuery(@RequestBody RestDTO<BussinessNumberSearch> dto) {
        BussinessNumberSearch object = dto.getData();
        if (object == null) {
            object = new BussinessNumberSearch();
        }
        dto.setData(object);
    }
    @ApiOperation("商户号批量导入")
    @PostMapping("uploadBussinessNumberExcel")
    public RestResult<List<String>> uploadBussinessNumberExcel(@RequestParam("file") MultipartFile excelFile , @RequestParam("adminId") Integer adminId ) {
        ExcelListenerUtils listener = new ExcelListenerUtils();  //ExcelListener
        try {
            EasyExcel.read(excelFile.getInputStream(), BussinessNubmerExcel.class, listener).sheet(0).doReadSync();  //ExcelModel 上面创建的实体类
        } catch (IOException e) {
            e.printStackTrace();
        }
        List<Object> datas = listener.getDatas(); //把拿到的数据处理到集合里去(根据所需自行修改)
        return bussinessNumberService.uploadBussinessNumberExcel(datas,adminId);
    }
}
