package com.yx.face.controller.admin;


import com.alibaba.excel.EasyExcel;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.ExcelListenerUtils;
import com.yx.face.model.dto.ExamUserWhitelistDTO;
import com.yx.face.model.excel.ExamUserWhitelistExcel;
import com.yx.face.model.search.ExamUserWhitelistSearch;
import com.yx.face.model.vo.ExamUserWhitelistExcelVO;
import com.yx.face.model.vo.ExamUserWhitelistVO;
import com.yx.face.service.ExamUserWhitelistService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

/**
 * <p>
 * 考试用户白名单 前端控制器
 * </p>
 *
 * @author wxg
 * @since 2023-04-14
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "C 考试名单管理")
@RestController
@Validated
@RequestMapping("admin/examUserWhitelist")
public class ExamUserWhitelistController extends BaseController {

    private final ExamUserWhitelistService examUserWhitelistService;

    @ApiOperation("考试人员-新增")
    @PostMapping("insert")
    public RestResult<Boolean> insert(@RequestBody @Valid ExamUserWhitelistDTO examUserWhitelistDTO) {
        return examUserWhitelistService.insert(examUserWhitelistDTO);
    }

    @ApiOperation("考试人员-分页")
    @PostMapping("/getPageList")
    public RestResult<PageInfo<ExamUserWhitelistVO>> getPageList(@RequestBody RestDTO<ExamUserWhitelistSearch> dto) {
        reBuildQuery(dto);
        PageInfo<ExamUserWhitelistVO> pageList = examUserWhitelistService.getPageList(dto);
        if (pageList != null) {
            return RestResponse.ok(pageList);
        }
        return RestResponse.error("考试人员列表分页失败");
    }

    private void reBuildQuery(@RequestBody RestDTO<ExamUserWhitelistSearch> dto) {
        ExamUserWhitelistSearch object = dto.getData();
        if (object == null) {
            object = new ExamUserWhitelistSearch();
        }
        dto.setData(object);
    }
    @ApiOperation("考试人员-删除")
    @GetMapping("delete")
    public RestResult<Boolean> delete(@RequestParam("examUserWhitelistId") Integer examUserWhitelistId) {
        Integer insert = examUserWhitelistService.delete(examUserWhitelistId);
        if (insert == 1) {
            return RestResponse.ok(true);
        } else {
            return RestResponse.error("删除失败");
        }
    }
    @ApiOperation("考试人员-批量导入")
    @PostMapping("uploadExcel")
    public RestResult<List<ExamUserWhitelistExcelVO>> uploadExcel(@RequestParam("file") MultipartFile excelFile) {
        ExcelListenerUtils listener = new ExcelListenerUtils();  //ExcelListener
        try {
            EasyExcel.read(excelFile.getInputStream(), ExamUserWhitelistExcel.class, listener).sheet(0).headRowNumber(2).doReadSync();  //ExcelModel 上面创建的实体类
        } catch (IOException e) {
            e.printStackTrace();
        }
        List<Object> datas = listener.getDatas(); //把拿到的数据处理到集合里去(根据所需自行修改)
        return examUserWhitelistService.uploadExcel(datas);
    }
    @ApiOperation("考试人员-导出")
    @PostMapping("/excelOut")
    public void excelOut(@RequestBody ExamUserWhitelistSearch dto, HttpServletResponse response){
        examUserWhitelistService.excelOut(dto,response);
    }

    @ApiOperation("考试人员-一键删除")
    @GetMapping("deleteAll")
    public RestResult<Boolean> deletedeleteAll() {
        return examUserWhitelistService.deletedeleteAll();
    }
}
