package com.yx.face.controller.admin;


import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.model.dto.FaceDeviceTypeDTO;
import com.yx.face.model.search.FaceDeviceTypeSearch;
import com.yx.face.model.vo.DeviceCodeVO;
import com.yx.face.model.vo.FaceDeviceTypeVO;
import com.yx.face.service.FaceDeviceTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 设备类型
 * </p>
 *
 * @author wxg
 * @since 2022-01-10
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "C 设备类型 API")
@RestController
@RequestMapping("admin/deviceType")

public class FaceDeviceTypeController {

    private final FaceDeviceTypeService faceDeviceTypeService;

    @ApiOperation("设备类型编码")
    @GetMapping("/queryDeviceCode")
    public RestResult<List<DeviceCodeVO>> queryDeviceCode(){
        return RestResponse.ok(faceDeviceTypeService.queryDeviceCode());
    }

    @ApiOperation("体检/编辑设备类型")
    @PostMapping("addOrEdit")
    public RestResult<Boolean> addOrEdit(@RequestBody FaceDeviceTypeDTO faceDeviceTypeDTO) {
        faceDeviceTypeDTO.getFaceDeviceTypeId();
        if (StringUtils.isBlank(faceDeviceTypeDTO.getName())) {
            throw new ServiceException("名称不能为空哦~");
        }
        Integer insert = faceDeviceTypeService.addOrEdit(faceDeviceTypeDTO);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }

    @ApiOperation("删除设备类型")
    @GetMapping("deleteType")
    public RestResult<Boolean> deleteType(@RequestParam("faceDeviceTypeId") Integer faceDeviceTypeId) {
        if (faceDeviceTypeId == null || faceDeviceTypeId <= 0) {
            throw new ServiceException("数据id为空或不匹配哦~");
        }
        Integer insert = faceDeviceTypeService.delete(faceDeviceTypeId);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("删除失败");
    }

    @ApiOperation("获取设备类型列表")
    @PostMapping("/getList")
    public RestResult<List<FaceDeviceTypeVO>> getList() {
        List<FaceDeviceTypeVO> list = faceDeviceTypeService.getList();
        if (list != null) {
            return RestResponse.ok(list);
        }
        return RestResponse.error("获取第三方列表失败");
    }

    @ApiOperation("设备类型列表分页")
    @PostMapping("/getPageList")
    public RestResult<PageInfo<FaceDeviceTypeVO>> getPageList(@RequestBody RestDTO<FaceDeviceTypeSearch> dto) {
        reBuildQuery(dto);
        List<FaceDeviceTypeVO> list = faceDeviceTypeService.getPageList(dto);
        if (list != null) {
            return RestResponse.ok(new PageInfo<>(list));
        }
        return RestResponse.error("获取第三方列表分页失败");
    }

    private void reBuildQuery(@RequestBody RestDTO<FaceDeviceTypeSearch> dto) {
        FaceDeviceTypeSearch object = dto.getData();
        if (object == null) {
            object = new FaceDeviceTypeSearch();
        }
        dto.setData(object);
    }
}
