package com.yx.face.controller.admin;

import com.alibaba.excel.ExcelWriter;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.AddressUtil;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.model.dto.issue.IssueDeviceDetailVo;
import com.yx.face.model.dto.issue.IssueDeviceItemSearch;
import com.yx.face.model.dto.issue.IssueDeviceItemVo;
import com.yx.face.model.dto.issue.IssueDeviceSearch;
import com.yx.face.model.dto.issue.IssueDeviceVo;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.excel.ExcelOperation;
import com.yx.face.service.issue.IssueDeviceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

/**
 * @author shisl
 * @package com.yx.face.controller.admin
 * @class IssueDeviceController
 * @date 2024/4/18 下午2:23
 * @description
 */
@Slf4j
@Api(tags = "websocket下发")
@RestController
@RequestMapping("admin/issue")
public class IssueDeviceController extends BaseController {

    @Resource
    private IssueDeviceService issueDeviceService;

    @Resource
    private AdminService adminService;

   

    @ApiOperation("下发设备分页列表")
    @PostMapping("/device/page")
    public RestResult<PageInfo<IssueDeviceVo>> page(@RequestBody RestDTO<IssueDeviceSearch> dto) {
        ClassUtil.searchInit(dto, IssueDeviceSearch.class);
        IssueDeviceSearch data = dto.getData();
        AdminVO info = adminService.getInfo();
        if (info.getType().intValue() != 1 && Preconditions.isBlank(data.getAdminId())) {
            data.setAdminId(info.getAdminId());
        }
        PageInfo<IssueDeviceVo> page = issueDeviceService.page(dto);
        return RestResponse.ok(page);
    }

    @ApiOperation("下发设备详情")
    @GetMapping("/device/detail/{issueDeviceId}")
    public RestResult<IssueDeviceDetailVo> detail(@PathVariable("issueDeviceId") Long issueDeviceId) {
        IssueDeviceDetailVo detailVo = issueDeviceService.detail(issueDeviceId);
        return RestResponse.ok(detailVo);
    }

    @ApiOperation("取消下发")
    @GetMapping("/device/cancel/{issueDeviceId}")
    public RestResult<Boolean> cancel(@PathVariable("issueDeviceId") Long issueDeviceId) {
        return issueDeviceService.cancel(issueDeviceId);
    }

    @ApiOperation("重试")
    @GetMapping("/device/retry/{issueDeviceId}")
    public RestResult<Boolean> retryIssue(@PathVariable("issueDeviceId") Long issueDeviceId) {
        return issueDeviceService.retryIssue(issueDeviceId);
    }

    @ApiOperation("下发设备详情分页列表")
    @PostMapping("/device/item/page")
    public RestResult<PageInfo<IssueDeviceItemVo>> pageItem(@RequestBody RestDTO<IssueDeviceItemSearch> dto) {
        ClassUtil.searchInit(dto, IssueDeviceItemSearch.class);
        PageInfo<IssueDeviceItemVo> page = issueDeviceService.pageItem(dto);
        return RestResponse.ok(page);
    }

    @ApiOperation("导出下发详情列表")
    @PostMapping("/device/item/export")
    public RestResult<String> issueDeviceItemsExport(@RequestBody IssueDeviceItemSearch dto) {
        List<IssueDeviceItemVo> list = issueDeviceService.issueDeviceItems(dto);
        String uploadExcel = Constants.UPLOAD_EXCEL;
        String fileName = String.format("%s下发名单-%s.xlsx", uploadExcel, System.currentTimeMillis());
        ExcelWriter excelWriter = null;
        try {
            Path directory = Paths.get(uploadExcel);
            if (!Files.exists(directory)) {
                Files.createDirectories(directory);
            }
            OutputStream out = new FileOutputStream(fileName);
            ExcelOperation.issueItemExcelOutPut(list, out);
            out.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (excelWriter != null) {
                excelWriter.finish();
            }
        }
        String filePath = AddressUtil.address.concat(fileName);
        return RestResponse.ok(filePath);
    }


}
