package com.yx.face.controller.admin;


import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.*;
import com.yx.face.model.search.PrivateAreaDeviceSearch;
import com.yx.face.model.search.PrivateAreaSearch;
import com.yx.face.model.search.PrivateAreaWhiteListSearch;
import com.yx.face.model.vo.*;
import com.yx.face.service.PrivateAreaDeviceService;
import com.yx.face.service.PrivateAreaService;
import com.yx.face.service.PrivateAreaWhitelistService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 私有区域表 前端控制器
 * </p>
 *
 * @author wxg
 * @since 2022-06-29
 */
@Api(tags = "私有区域")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@RestController
@RequestMapping("admin/privateArea")
public class PrivateAreaController {

    private final PrivateAreaService privateAreaService;
    private final PrivateAreaDeviceService privateAreaDeviceService;
    private final PrivateAreaWhitelistService privateAreaWhitelistService;

    @ApiOperation(value = "私有区域列表分页")
    @ApiOperationSupport(order = 1)
    @PostMapping("privateAreaPageList")
    
    public RestResult<PageInfo<PrivateAreaVO>> privateAreaPageList(@RequestBody RestDTO<PrivateAreaSearch> dto) {
        reBuildQuery(dto);
        return privateAreaService.privateAreaPageList(dto);
    }
    private void reBuildQuery(RestDTO<PrivateAreaSearch> dto) {
        PrivateAreaSearch object = dto.getData();
        if (object == null) {
            object = new PrivateAreaSearch();
        }
        dto.setData(object);
    }
    @ApiOperation("添加/修改私有区域")
    @PostMapping("addOrUpdatePrivateArea")
    @ApiOperationSupport(order = 2)
    
    public RestResult<Boolean> addOrUpdatePrivateArea(@RequestBody PrivateAreaDTO privateAreaDTO) {
        Integer insert = privateAreaService.addOrUpdatePrivateArea(privateAreaDTO);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }

    @ApiOperation("删除私有区域")
    @GetMapping("deletePrivateArea")
    @ApiOperationSupport(order = 3)
    public RestResult<Boolean> addOrUpdatePrivateArea(@RequestParam("privateAreaId") Integer privateAreaId) {
        Integer insert = privateAreaService.deletePrivateArea(privateAreaId);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }
    @ApiOperation("获取私有区域列表")
    @GetMapping("getPrivateAreaList")
    @ApiOperationSupport(order = 4)
    
    public RestResult<List<PrivateAreaVO>> getPrivateAreaList(@RequestParam("adminId") Integer adminId) {
        return privateAreaService.getPrivateAreaList(adminId);
    }

    @ApiOperation(value = "私有区域设备列表分页")
    @ApiOperationSupport(order = 11)
    @PostMapping("privateAreaDevicePageList")
    
    public RestResult<PageInfo<PrivateAreaDeviceVO>> privateAreaDevicePageList(@RequestBody RestDTO<PrivateAreaDeviceSearch> dto) {
        reBuildQuery1(dto);
        return privateAreaDeviceService.privateAreaDevicePageList(dto);
    }
    private void reBuildQuery1(RestDTO<PrivateAreaDeviceSearch> dto) {
        PrivateAreaDeviceSearch object = dto.getData();
        if (object == null) {
            object = new PrivateAreaDeviceSearch();
        }
        dto.setData(object);
    }

    @ApiOperation("获取私有区域设备列表")
    @GetMapping("getPrivateAreaDeviceList")
    @ApiOperationSupport(order = 12)
    
    public RestResult<List<FaceDeviceVO>> getPrivateAreaDeviceList(@RequestParam("privateAreaId") Integer privateAreaId) {
        return privateAreaDeviceService.getPrivateAreaDeviceList(privateAreaId);
    }

    @ApiOperation("添加私有区域设备")
    @PostMapping("addPrivateAreaDevice")
    @ApiOperationSupport(order = 13)
    
    public RestResult<Boolean> addPrivateAreaDevice(@RequestBody PrivateAreaDeviceDTO privateAreaDeviceDTO) {
        Integer insert = privateAreaDeviceService.addPrivateAreaDevice(privateAreaDeviceDTO);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }

    @ApiOperation("删除私有区域设备")
    @GetMapping("deletePrivateAreaDevice")
    @ApiOperationSupport(order = 14)
    
    public RestResult<Boolean> deletePrivateAreaDevice(@RequestParam("privateAreaDeviceId") Integer privateAreaDeviceId) {
        Integer insert = privateAreaDeviceService.deletePrivateAreaDevice(privateAreaDeviceId);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }

    @ApiOperation(value = "私有区域人员列表分页")
    @ApiOperationSupport(order = 21)
    
    @PostMapping("privateAreaWhitelistPageList")
    public RestResult<PageInfo<PrivateAreaWhitelistVO>> privateAreaWhitelistPageList(@RequestBody RestDTO<PrivateAreaWhiteListSearch> dto) {
        reBuildQuery2(dto);
        return privateAreaWhitelistService.privateAreaWhitelistPageList(dto);
    }
    private void reBuildQuery2(RestDTO<PrivateAreaWhiteListSearch> dto) {
        PrivateAreaWhiteListSearch object = dto.getData();
        if (object == null) {
            object = new PrivateAreaWhiteListSearch();
        }
        dto.setData(object);
    }

    @ApiOperation("获取可以加入私有区域人员列表")
    @PostMapping("getPrivateAreaWhitelistList")
    @ApiOperationSupport(order = 22)
    
    public RestResult<List<UserWhitelistVO>> getPrivateAreaWhitelistList(@RequestBody PrivateAreaWhitelistListDTO privateAreaWhitelistListDTO) {
        return privateAreaWhitelistService.getPrivateAreaWhitelistList(privateAreaWhitelistListDTO);
    }

    @ApiOperation("添加私有区域人员")
    @PostMapping("addPrivateAreaWhitelist")
    @ApiOperationSupport(order = 23)
    
    public RestResult<Boolean> addPrivateAreaWhitelist(@RequestBody PrivateAreaWhitelistDTO privateAreaWhitelistDTO) {
        Integer insert = privateAreaWhitelistService.addPrivateAreaWhitelist(privateAreaWhitelistDTO);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }

    @ApiOperation("删除私有区域人员")
    @GetMapping("deletePrivateAreaWhitelist")
    @ApiOperationSupport(order = 24)
    
    public RestResult<Boolean> deletePrivateAreaWhitelist(@RequestParam("privateAreaWhitelistId") Integer privateAreaDeviceId) {
        Integer insert = privateAreaWhitelistService.deletePrivateAreaWhitelist(privateAreaDeviceId);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }
}
