package com.yx.face.controller.admin;


import com.github.pagehelper.PageInfo;
import com.yx.face.boot.component.security.Authority;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.ThirdPartyDTO;
import com.yx.face.model.search.ThirdPartySearch;
import com.yx.face.model.vo.ThirdPartyVO;
import com.yx.face.service.ThirdPartyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 第三方列表 前端控制器
 * </p>
 *
 * @author wxg
 * @since 2022-01-10
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "C 第三方列表管理 API")
@RestController
@RequestMapping("admin/thirdParty")
public class ThirdPartyController {

    private final ThirdPartyService thirdPartyService;

    @ApiOperation("编辑第三方")
    @PostMapping("edit")
    @Authority(value = {1,2})
    public RestResult<Boolean> edit(@RequestBody ThirdPartyDTO thirdPartyDTO) {
        Integer insert = thirdPartyService.edit(thirdPartyDTO);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }

    @ApiOperation("删除第三方")
    @GetMapping("delete")
    @Authority(value = {1,2})
    public RestResult<Boolean> delete(@RequestParam("thirdPartyId") Integer thirdPartyId) {
        Integer insert = thirdPartyService.delete(thirdPartyId);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("删除失败");
    }
    @ApiOperation("第三方列表(id 和 name)")
    @PostMapping("/getList")
    public RestResult<List<ThirdPartyVO>> getList() {
        List<ThirdPartyVO> list = thirdPartyService.getList();
        if (list != null) {
            return RestResponse.ok(list);
        }
        return RestResponse.error("获取第三方列表失败");
    }
    @ApiOperation("第三方列表分页")
    @PostMapping("/getPageList")
    @Authority(value = {1,2})
    public RestResult<PageInfo<ThirdPartyVO>> getPageList(@RequestBody RestDTO<ThirdPartySearch> dto) {
        reBuildQuery(dto);
        List<ThirdPartyVO> list = thirdPartyService.getPageList(dto);
        if (list != null) {
            return RestResponse.ok(new PageInfo<>(list));
        }
        return RestResponse.error("获取第三方列表分页失败");
    }
    private void reBuildQuery(@RequestBody RestDTO<ThirdPartySearch> dto) {
        ThirdPartySearch object = dto.getData();
        if (object == null) {
            object = new ThirdPartySearch();
        }
        dto.setData(object);
    }
}
