package com.yx.face.controller.admin;

import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.third.ZoneDeviceRDto;
import com.yx.face.model.vo.third.ZoneDeviceRPageReq;
import com.yx.face.model.vo.third.ZoneSnReq;
import com.yx.face.service.AdminService;
import com.yx.face.service.ZoneDeviceRService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author shisl
 * @package com.yx.face.controller.admin
 * @class ThirdPlatformApiController
 * @date 2023/7/5 上午9:44
 * @description 三方平台API管理
 */
@Api(tags = "三方平台API管理")
@RestController
@RequestMapping("admin/third/platform")
public class ThirdPlatformApiController {

    @Autowired
    private ZoneDeviceRService zoneDeviceRService;

    @Autowired
    private AdminService adminService;

    @ApiOperation("添加/编辑区域")
    @PostMapping("/create")
    public RestResult create(@RequestBody ZoneDeviceRDto zoneDeviceRDto) {

        if (Preconditions.isBlank(zoneDeviceRDto.getName())) {
            return RestResponse.error("参数有误，请核验后提交");
        }

        AdminVO info = adminService.getInfo();
        zoneDeviceRDto.setAdminId(info.getAdminId());
        zoneDeviceRService.create(zoneDeviceRDto);
        return RestResponse.ok();
    }

    @ApiOperation("删除指定区域")
    @GetMapping("/zone/delete/{id}")
    public RestResult delete(@PathVariable("id") Integer id) {
        zoneDeviceRService.delete(id);
        return RestResponse.ok();
    }

    @ApiOperation("删除指定区域的设备")
    @GetMapping("/zone/sn/delete/{zoneId}/{sn}")
    public RestResult deleteSn(@PathVariable("zoneId") Integer zoneId,
                               @PathVariable("sn") String sn) {
        zoneDeviceRService.deleteSn(zoneId, sn);
        return RestResponse.ok();
    }

    @ApiOperation("获取区域列表分页")
    @PostMapping("/page")
    public RestResult<PageInfo<ZoneDeviceRDto>> page(@RequestBody RestDTO<ZoneDeviceRPageReq> dto) {
        if (Preconditions.isBlank(dto.getData())) {
            ZoneDeviceRPageReq req = new ZoneDeviceRPageReq();
            dto.setData(req);
        }
        AdminVO info = adminService.getInfo();
        dto.getData().setAdminId(info.getAdminId());
        PageInfo<ZoneDeviceRDto> page = zoneDeviceRService.page(dto);
        return RestResponse.ok(page);
    }


    @ApiOperation("获取区域列表")
    @PostMapping("/list")
    public RestResult<List<ZoneDeviceRDto>> list(@RequestBody ZoneDeviceRPageReq dto) {
        AdminVO info = adminService.getInfo();
        Integer adminId = null;
        if(Objects.equals(info.getType(), AdminTypeEnum.DISTRICT.getCode())
                || Objects.equals(info.getType(), AdminTypeEnum.ENTERPRISE.getCode())){
            adminId = info.getAdminId();
        }else {
            if(dto.getAdminId() != null){
                adminId = dto.getAdminId();
            }
        }
        List<ZoneDeviceRDto> list = zoneDeviceRService.list(adminId);
        return RestResponse.ok(Preconditions.isNotBlank(list) ? list : new ArrayList<>());
    }

    @ApiOperation("获取设备列表")
    @PostMapping("/sn/list")
    public RestResult<List<String>> snList(@RequestBody ZoneSnReq dto) {
        if (Preconditions.isBlank(dto)) {
            dto = new ZoneSnReq();
        }
        AdminVO info = adminService.getInfo();
        dto.setAdminId(info.getAdminId());
        List<String> list = zoneDeviceRService.snList(dto);
        return RestResponse.ok(Preconditions.isNotBlank(list) ? list : new ArrayList<>());
    }

}
