package com.yx.face.controller.admin;

import com.github.pagehelper.PageInfo;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.UserBlackListDTO;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.search.UserBlacklistSearch;
import com.yx.face.model.search.UserWhitelistSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.UserBlackListVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.UserBlackListService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @description:
 * @ClassName UserWhitelistController
 * @Author WXG
 * @Date 2021/11/5 14:12
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "B 黑名单 API")
@RestController
@RequestMapping("admin/userBlackList")
public class UserBlackListController extends BaseController {

    private final UserBlackListService userBlackListService;
    private final AdminService adminService;


    @ApiOperation("新增黑名单")
    @PostMapping("insert")
    public RestResult<Boolean> insert(@RequestBody UserBlackListDTO userBlackListDTO) {
        if (userBlackListDTO.getAdminId() == null) {
            userBlackListDTO.setAdminId(getUserId());
        }
        AdminVO info  = adminService.getInfo(userBlackListDTO.getAdminId());;
        if(!info.getType().equals(AdminTypeEnum.DISTRICT.getCode())
           && !info.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())
        ){
            return RestResponse.error("不能添加到区级及企业外的账号");
        }

        try {
            Integer insert = userBlackListService.insert(userBlackListDTO);
            if (insert == 1) {
                return RestResponse.ok(true);
            }
            return RestResponse.error("增加失败");
        } catch (Exception e) {
            log.info("添加失败",e);
            return RestResponse.error("增加失败" + e.getMessage());
        }
    }


    @ApiOperation("黑名单列表分页")
    @PostMapping("/getPageList")
    public RestResult<PageInfo<UserBlackListVO>> getPageList(@RequestBody RestDTO<UserWhitelistSearch> dto) {
        reBuildQuery(dto);
        PageInfo<UserBlackListVO> pageList = userBlackListService.getPageList(dto);
        if (pageList != null) {
            return RestResponse.ok(pageList);
        }
        return RestResponse.error("获取常客列表分页失败");
    }


    @SneakyThrows
    @ApiOperation("删除黑名单")
    @GetMapping("delete")
    public RestResult<Boolean> delete(@RequestParam("id") Integer id){
        Integer insert = userBlackListService.delete(id, getUserId());
        if (insert == 1) {
            return RestResponse.ok(true);
        } else {
            return RestResponse.error("删除失败");
        }
    }


    private void reBuildQuery(@RequestBody RestDTO<UserWhitelistSearch> dto) {
        UserWhitelistSearch object = dto.getData();
        if (object == null) {
            object = new UserWhitelistSearch();
        }
        dto.setData(object);
    }

    @ApiOperation("黄牛黑名单列表分页")
    @PostMapping("/getCattlePageList")
    public RestResult<PageInfo<UserBlackListVO>> getCattlePageList(@RequestBody RestDTO<UserBlacklistSearch> dto) {
        reBuildQuery1(dto);
        PageInfo<UserBlackListVO> pageList = userBlackListService.getCattlePageList(dto);
        if (pageList != null) {
            return RestResponse.ok(pageList);
        }
        return RestResponse.error("获取常客列表分页失败");
    }
    private void reBuildQuery1(@RequestBody RestDTO<UserBlacklistSearch> dto) {
        UserBlacklistSearch object = dto.getData();
        if (object == null) {
            object = new UserBlacklistSearch();
        }
        dto.setData(object);
    }
    @SneakyThrows
    @ApiOperation("删除黄牛黑名单")
    @GetMapping("deleteCattle")
    public RestResult<Boolean> deleteCattle(@RequestParam("id") Integer id){
        Integer insert = userBlackListService.deleteCattle(id);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("删除失败");
    }
}
