package com.yx.face.controller.admin;

import com.alibaba.excel.ExcelWriter;
import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.AddressUtil;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.model.search.UserVisitorlistDetailSearch;
import com.yx.face.model.search.UserWhitelistSearch;
import com.yx.face.model.vo.UserVisitorListDetailPageVO;
import com.yx.face.model.vo.UserVisitorListVO;
import com.yx.face.model.vo.visitor.UserVisitorListDetailInfo;
import com.yx.face.service.UserVisitorListService;
import com.yx.face.service.excel.ExcelOperation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;


/**
 * @description:
 * @ClassName UserWhitelistController
 * @Author WXG
 * @Date 2021/11/5 14:12
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "B 访客 API")
@RestController
@RequestMapping("admin/userVisitorList")
public class UserVisitorListController extends BaseController {

    private final UserVisitorListService userVisitorListService;
   

    @ApiOperation("访客列表分页")
    @PostMapping("/getPageList")
    public RestResult<PageInfo<UserVisitorListVO>> getPageList(@RequestBody RestDTO<UserWhitelistSearch> dto) {
        reBuildQuery(dto);
        PageInfo<UserVisitorListVO> pageList = userVisitorListService.getPageList(dto);
        if (pageList != null) {
            return RestResponse.ok(pageList);
        }
        return RestResponse.error("获取常客列表分页失败");
    }

    @ApiOperation("访客申请记录分页")
    @PostMapping("/getDetailPageList")
    public RestResult<PageInfo<UserVisitorListDetailPageVO>> getDetailPageList(@RequestBody RestDTO<UserVisitorlistDetailSearch> dto) {
        ClassUtil.searchInit(dto,UserVisitorlistDetailSearch.class);
        PageInfo<UserVisitorListDetailPageVO> pageList = userVisitorListService.getDetailPageList(dto);
        if (pageList != null) {
            return RestResponse.ok(pageList);
        }
        return RestResponse.error("获取常客列表分页失败");
    }

    @ApiOperation("访客记录详情")
    @GetMapping("/getDetailInfo/{detailId}")
    public RestResult<UserVisitorListDetailInfo> getDetailInfo(@PathVariable("detailId") Integer detailId) {
        return RestResponse.ok(userVisitorListService.getDetailInfo(detailId));
    }


    @ApiOperation("添加白名单")
    @GetMapping("/addWhite")
    public RestResult<Boolean> addWhite(Integer id) throws Exception {
        Integer insert = null;
        insert = userVisitorListService.addWhite(id, getUserId());
        if (insert == 1) {
            return RestResponse.ok(true);
        } else {
            return RestResponse.error("删除失败");
        }
    }

    @ApiOperation("批量添加白名单")
    @PostMapping("addWhiteAll")
    public RestResult<Boolean> addWhiteAll() {
        return RestResponse.ok(userVisitorListService.addWhiteAll(getUserId()));
    }

    @ApiOperation("添加黑名单")
    @GetMapping("/addBlack")
    public RestResult<Boolean> addBlack(Integer id) {
        Integer insert = null;
        try {
            insert = userVisitorListService.addBlack(id, getUserId());
        } catch (Exception exception) {
            log.info("添加黑名单异常了", exception);
        }
        if (insert != null && insert == 1) {
            return RestResponse.ok(true);
        } else {
            return RestResponse.error("删除失败");
        }
    }

    @ApiOperation("删除访客")
    @GetMapping("/deleteVisitor")
    public RestResult<Boolean> deleteVisitor(Integer id) {
        Integer insert = userVisitorListService.delete(id);
        if (insert == 1) {
            return RestResponse.ok(true);
        } else {
            return RestResponse.error("删除失败");
        }
    }

    @ApiOperation("访客下发")
    @GetMapping("addSingle")
    public RestResult<String> addSingle(Integer id) throws Exception {
        RestResult<String> restResult = userVisitorListService.addSingle(getUserId(), id);
        return restResult;
    }

    @ApiOperation("导出访客列表Excel")
    @PostMapping("userVisitorlistExcelOut")
    public RestResult<String> userVisitorlistExcelOut(@RequestBody UserWhitelistSearch userWhitelistSearch) {
        List<UserVisitorListVO> list = userVisitorListService.userVisitorlistExcelOut(userWhitelistSearch);
        String uploadExcel = Constants.UPLOAD_EXCEL;
        String fileName = uploadExcel + "/访客信息记录-" + System.currentTimeMillis() + ".xlsx";
        ExcelWriter excelWriter = null;
        try {
            Path directory = Paths.get(uploadExcel);
            if (!Files.exists(directory)) {
                Files.createDirectories(directory);
            }
            OutputStream out = new FileOutputStream(fileName);
            ExcelOperation.userVisitorlistExcelOut(list, out);
            out.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (excelWriter != null) {
                excelWriter.finish();
            }
        }
        String filePath = AddressUtil.address.concat(fileName);
        return RestResponse.ok(filePath);
    }

    private void reBuildQuery(@RequestBody RestDTO<UserWhitelistSearch> dto) {
        UserWhitelistSearch object = dto.getData();
        if (object == null) {
            object = new UserWhitelistSearch();
        }
        dto.setData(object);
    }
    @ApiOperation(value = "访客记录-导出",notes = "最多导出最前面的5万条数据")
    @PostMapping("detailExcelOut")
    @ApiOperationSupport(order = 110)
    public void excelOut(@RequestBody UserVisitorlistDetailSearch dto, HttpServletResponse response){
        userVisitorListService.excelOut(dto,response);
    }
}
