package com.yx.face.controller.admin;


import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.component.security.Authority;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.model.dto.UserWhitelistCheckDto;
import com.yx.face.model.search.UserWhitelistCheckSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.UserWhitelistCheckVO;
import com.yx.face.model.vo.UserWhitelistLittleVO;
import com.yx.face.service.UserWhitelistCheckService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 楼宇审核人员 前端控制器
 * </p>
 *
 * @author wxg
 * @since 2023-08-23
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "B 楼宇常客审核权限")
@RestController
@RequestMapping("admin/userWhitelistCheck")
@Validated
public class UserWhitelistCheckController {

    private final UserWhitelistCheckService userWhitelistCheckService;

    @ApiOperation("权限人员列表")
    @Authority({5})
    @PostMapping("/getPageList")
    @ApiOperationSupport(order = 10)
    public RestResult<PageInfo<UserWhitelistCheckVO>> getPageList(@RequestBody RestDTO<UserWhitelistCheckSearch> dto) {
        ClassUtil.searchInit(dto,UserWhitelistCheckSearch.class);
        return RestResponse.ok(userWhitelistCheckService.getPageList(dto));
    }

    @ApiOperation("获取公司列表列表")
    @Authority({5})
    @GetMapping("/getAdminList")
    @ApiOperationSupport(order = 15)
    public RestResult<List<AdminVO>> getAdminList() {
        return RestResponse.ok(userWhitelistCheckService.getAdminList());
    }

    @ApiOperation("获取常客列表")
    @Authority({5})
    @GetMapping("/getUserWhitelist")
    @ApiOperationSupport(order = 20)
    public RestResult<List<UserWhitelistLittleVO>> getUserWhitelist(@RequestParam Integer adminId) {
        return RestResponse.ok(userWhitelistCheckService.getUserWhitelist(adminId));
    }

    @ApiOperation("添加人员")
    @Authority({5})
    @PostMapping("/insert")
    @ApiOperationSupport(order = 30)
    public RestResult<Boolean> insert(@RequestBody @Valid UserWhitelistCheckDto dto) {
        return RestResponse.ok(userWhitelistCheckService.insert(dto));
    }
    @ApiOperation("删除人员")
    @Authority({5})
    @GetMapping("/delete")
    @ApiOperationSupport(order = 40)
    public RestResult<Boolean> delete(@RequestParam Long userWhitelistCheckId) {
        return RestResponse.ok(userWhitelistCheckService.delete(userWhitelistCheckId));
    }
}
