package com.yx.face.controller.admin;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.google.common.collect.Lists;
import com.yx.face.boot.component.security.Authority;
import com.yx.face.boot.config.OldCellWriteHandler;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.AddressUtil;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.model.dto.IssueAllDTO;
import com.yx.face.model.dto.UserWhitelistBatchDTO;
import com.yx.face.model.dto.UserWhitelistBatchDelDTO;
import com.yx.face.model.dto.UserWhitelistBatchDistributionDTO;
import com.yx.face.model.dto.UserWhitelistBatchUpdateDTO;
import com.yx.face.model.dto.UserWhitelistDTO;
import com.yx.face.model.entity.UserWhitelist;
import com.yx.face.model.enums.*;
import com.yx.face.model.search.UserWhitelistSearch;
import com.yx.face.model.search.WhitelistPrivateAreaWhiteListSearch;
import com.yx.face.model.vo.*;
import com.yx.face.service.AdminService;
import com.yx.face.service.FaceService;
import com.yx.face.service.PrivateAreaWhitelistService;
import com.yx.face.service.UserWhitelistService;
import com.yx.face.service.excel.ExcelOperation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;

/**
 * @description:
 * @ClassName UserWhitelistController
 * @Author WXG
 * @Date 2021/11/5 14:12
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "B 常客白名单 API")
@RestController
@RequestMapping("admin/userWhitelis")
public class UserWhitelistController extends BaseController {

    private final UserWhitelistService userWhitelistService;

    private final AdminService adminService;

    private final FaceService faceService;

    private final PrivateAreaWhitelistService privateAreaWhitelistService;

   

    @ApiOperation("获取常客有效期")
    @GetMapping("getUserWhitelistEffectiveTime")
    public RestResult<String> getUserWhitelistEffectiveTime(@RequestParam(value = "adminId", required = false) Integer adminId) {
        return RestResponse.ok(adminService.getUserWhitelistEffectiveTime(adminId));
    }

    @ApiOperation("获取人员类型列表")
    @GetMapping("getUserWhitePersonnelTypeEnum")
    public RestResult<List<UserWhitePersonelTypeVO>> getUserWhitePersonnelTypeEnum() {
        List<UserWhitePersonelTypeVO> list = new LinkedList<>();
        for (UserWhitePersonnelTypeEnum value : UserWhitePersonnelTypeEnum.values()) {
            list.add(new UserWhitePersonelTypeVO().setCode(value.getCode()).setDesc(value.getDesc()));
        }
        return RestResponse.ok(list);
    }

    @ApiOperation("获取操作人（家长）信息")
    @GetMapping("getParentInfo")
    public RestResult<UserWhitelistParentVO> getParentInfo(@RequestParam("UserWhitelistId") Integer UserWhitelistId) {
        return userWhitelistService.getParentInfo(UserWhitelistId);
    }

    @ApiOperation("新增常客")
    @PostMapping("insert")
    public RestResult<Boolean> insert(@RequestBody UserWhitelistDTO userWhitelistDTO) {
        if (userWhitelistDTO.getAdminId() == null) {
            userWhitelistDTO.setAdminId(getUserId());
        }
        AdminVO info = adminService.getInfo(userWhitelistDTO.getAdminId());
        if (!info.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            userWhitelistDTO.setPrivateAreaIdList(null);
        } else {
            if (info.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.CLOSE_PRIVATE_AREA.getCode())) {
                userWhitelistDTO.setAdminAreaType(2);
            } else {
                userWhitelistDTO.setAdminAreaType(CollUtil.isNotEmpty(userWhitelistDTO.getPrivateAreaIdList()) ? 3 : 2);
            }
        }
        if (info.getType().equals(AdminTypeEnum.DISTRICT.getCode()) &&
                (AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType())
                        || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())
                )
        ) {
            throw new ServiceException("常客无法添加入当前配置的（楼宇/学校）区级账号");
        }
        if (!CollectionUtils.isEmpty(userWhitelistDTO.getAddCustomList())) {
            String s = JSON.toJSONString(userWhitelistDTO.getAddCustomList());
            userWhitelistDTO.setContent(s);
        }
        try {
            UserWhitelist userWhitelist = userWhitelistService.insert(userWhitelistDTO, 0);
            if (Objects.nonNull(userWhitelist) && Objects.nonNull(userWhitelist.getId())) {
                return RestResponse.ok(true);
            }
            return RestResponse.error("增加失败");
        } catch (Exception e) {
            log.info("添加失败", e);
            return RestResponse.error(e.getMessage());
        }
    }

    @ApiOperation("修改常客")
    @PostMapping("update")
    public RestResult<Boolean> update(@RequestBody UserWhitelistDTO userWhitelistDTO) {
        if (!CollectionUtils.isEmpty(userWhitelistDTO.getAddCustomList())) {
            String s = JSON.toJSONString(userWhitelistDTO.getAddCustomList());
            userWhitelistDTO.setContent(s);
        }
        AdminVO info = null;
        if (userWhitelistDTO.getAdminId() == null) {
            info = adminService.getInfo();
            userWhitelistDTO.setAdminId(info.getAdminId());
        } else {
            info = adminService.getInfo(userWhitelistDTO.getAdminId());
        }
        if (!info.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            userWhitelistDTO.setPrivateAreaIdList(null);
        }
        UserWhitelist insert = userWhitelistService.update(userWhitelistDTO);
        return RestResponse.ok(true);
    }

    @ApiOperation("删除常客")
    @GetMapping("delete")
    public RestResult<Boolean> delete(@RequestParam("UserWhitelistId") Integer id) {
        Integer insert = userWhitelistService.delete(id, getUserId());
        if (insert == 1) {
            return RestResponse.ok(true);
        } else {
            return RestResponse.error("删除失败");
        }
    }

    @ApiOperation("清空数据")
    @GetMapping("/deleteAll")
    public RestResult<Boolean> deleteAll() {
        Integer insert = userWhitelistService.deleteAll(getUserId());
        if (insert == 1) {
            return RestResponse.ok(true);
        } else {
            return RestResponse.error("删除失败");
        }
    }

    @ApiOperation("常客列表分页")
    @PostMapping("/getPageList")
    public RestResult<PageInfo<UserWhitelistVO>> getPageList(@RequestBody RestDTO<UserWhitelistSearch> dto) {
        ClassUtil.searchInit(dto, UserWhitelistSearch.class);
        PageInfo<UserWhitelistVO> pageList = userWhitelistService.getPageList(dto);
        return RestResponse.ok(pageList);

    }

    @ApiOperation("全量白名单下发")
    @GetMapping("addSnIssued")
    public RestResult<String> addSnIssued(Integer deviceId, String startTime, String endTime, Boolean validityPeriodStatus, String issueType) {
        IssueAllDTO dto = new IssueAllDTO();
        dto.setDeviceId(deviceId);
        dto.setStartTime(startTime);
        dto.setEndTime(endTime);
        dto.setValidityPeriodStatus(validityPeriodStatus);
        dto.setIssueType(issueType);
        return userWhitelistService.addSnIssued(dto);
    }

    @ApiOperation("常客重新下发")
    @GetMapping("addSingle")
    public RestResult<String> addSingle(Integer id) {
        UserWhitelist whitelist = userWhitelistService.getOne(id);
        RestResult<String> restResult = userWhitelistService.addSingle(id, whitelist.getStartTime(), whitelist.getEndTime());
        return restResult;
    }

    @ApiOperation("常客模板")
    @GetMapping("doGetWhiteExcel")
    public void doGetFaceServerLogExcel(HttpServletResponse resp) {
        AdminVO info = adminService.getInfo();
        String uploadExcel = Constants.UPLOAD_EXCEL;
        String fileName = uploadExcel + "常客模板-" + System.currentTimeMillis() + ".xlsx";
        try {
            Path directory = Paths.get(uploadExcel);
            if (!Files.exists(directory)) {
                Files.createDirectories(directory);
            }
            List<ContextVO> context = faceService.getContextById(info.getAdminId());
            List<List<String>> head = head();
            if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType())
                    || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())
            )
                    && AdminTypeEnum.DISTRICT.getCode().equals(info.getType())) {
                head.add(Lists.newArrayList("账号名称"));
            }
            if (AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())) {
                head.add(Lists.newArrayList("人员类型"));
            }

            if (AdminIsOpenPrivateAreaEnum.OPEN_PRIVATE_AREA.getCode().equals(info.getIsOpenPrivateArea())) {
                head.add(Lists.newArrayList("通行区域权限"));
                head.add(Lists.newArrayList("私有区域"));
            }
            if (AdminCardPhysicsEnum.Open.getCode().equals(info.getCardPhysics())) {
                head.add(Lists.newArrayList("物理卡号"));
            }
            if (CollUtil.isNotEmpty(context)) {
                for (int i = 0; i < context.size(); i++) {
                    head.add(Lists.newArrayList(context.get(i).getCustomName()));
                }
            }

            //预设500行
            List<List<String>> list = new ArrayList<>();
            for (int i = 0; i < 1000; i++) {
                int size = head.size();
                List<String> celllist = new ArrayList<String>();
                for (int j = 0; j < size; j++) {
                    celllist.add("  ");
                }
                list.add(celllist);
            }
            EasyExcel.write(fileName)
                    .head(head).sheet("常客模板")
                    .registerWriteHandler(new OldCellWriteHandler()).doWrite(list);
            InputStream inputStream = Files.newInputStream(Paths.get(fileName));
            int b = 0;
            byte[] buffer = new byte[1024];
            while ((b = inputStream.read(buffer)) != -1) {
                resp.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode("常客模板.xlsx", "UTF-8"));
                resp.addHeader("Access-Control-Expose-Headers", "Content-Disposition");
                resp.setContentType("application/vnd.ms-excel;charset=UTF-8");
                resp.getOutputStream().write(buffer, 0, b);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private List<List<String>> head() {
        List<List<String>> list = new ArrayList<>();
        list.add(Lists.newArrayList("姓名"));
        list.add(Lists.newArrayList("手机号"));
        list.add(Lists.newArrayList("身份证号"));
        list.add(Lists.newArrayList("开始有效期"));
        list.add(Lists.newArrayList("结束有效期"));
        return list;
    }

    @ApiOperation("清洗自定义数据")
    @GetMapping("clear")
    public RestResult<Boolean> clear() {
        return RestResponse.ok(faceService.clear());
    }

    @ApiOperation("导出常客列表Excel")
    @PostMapping("userWhitelistExcelOut")
    public RestResult<String> userWhitelistExcelOut(@RequestBody UserWhitelistSearch userWhitelistSearch) {
        List<UserWhitelistVO> list = userWhitelistService.userWhitelistExcelOut(userWhitelistSearch);
        String uploadExcel = Constants.UPLOAD_EXCEL;
        String fileName = uploadExcel + "/常客信息记录-" + System.currentTimeMillis() + ".xlsx";
        ExcelWriter excelWriter = null;
        try {
            Path directory = Paths.get(uploadExcel);
            if (!Files.exists(directory)) {
                Files.createDirectories(directory);
            }
            OutputStream out = new FileOutputStream(fileName);
            ExcelOperation.onlineOrderExcelOutPut(list, out);
            out.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (excelWriter != null) {
                excelWriter.finish();
            }
        }
        String filePath = AddressUtil.address.concat(fileName);
        return RestResponse.ok(filePath);
    }

    @ApiOperation(value = "查看常客所在的私有区域")
    @ApiOperationSupport(order = 21)
    @Authority({5, 99})
    @PostMapping("privateAreaPageListByWhitelistId")
    public RestResult<PageInfo<WhitelistPrivateAreaVO>> privateAreaPageListByWhitelistId(@RequestBody RestDTO<WhitelistPrivateAreaWhiteListSearch> dto) {
        ClassUtil.searchInit(dto, WhitelistPrivateAreaWhiteListSearch.class);
        return privateAreaWhitelistService.privateAreaPageListByWhitelistId(dto);
    }

    @ApiOperation("常客的批量编辑")
    @ApiOperationSupport(order = 100)
    @PostMapping("batchEditUserWhitelist")
    @Authority(value = {5, 99})
    public RestResult<String> batchEditUserWhitelist(@RequestBody @Valid UserWhitelistBatchDTO dto) {
        return userWhitelistService.batchEditUserWhitelist(dto);
    }

    @ApiOperation("批量修改常客")
    @ApiOperationSupport(order = 101)
    @PostMapping("batchUpdate")
    public RestResult<Boolean> batchUpdate(@RequestBody UserWhitelistBatchUpdateDTO userWhitelistDTO) {
        if (!CollectionUtils.isEmpty(userWhitelistDTO.getAddCustomList())) {
            String s = JSON.toJSONString(userWhitelistDTO.getAddCustomList());
            userWhitelistDTO.setContent(s);
        }
        AdminVO info = null;
        if (userWhitelistDTO.getAdminId() == null) {
            info = adminService.getInfo();
            userWhitelistDTO.setAdminId(info.getAdminId());
        }
        Integer insert = userWhitelistService.batchUpdate(userWhitelistDTO);
        if (insert > 0) {
            return RestResponse.ok(true);
        }
        return RestResponse.error("修改失败");
    }

    @ApiOperation("批量删除用户数据")
    @ApiOperationSupport(order = 102)
    @PostMapping("/batchDelete")
    public RestResult<Boolean> batchDelete(@RequestBody UserWhitelistBatchDelDTO dto) {
        AdminVO info = null;
        if (dto.getAdminId() == null) {
            info = adminService.getInfo();
            dto.setAdminId(info.getAdminId());
        }
        Integer del = userWhitelistService.batchDelete(dto);
        if (del > 0) {
            return RestResponse.ok(true);
        } else {
            return RestResponse.error("删除失败");
        }
    }

    @ApiOperation("批量下发用户数据")
    @ApiOperationSupport(order = 103)
    @PostMapping("batchDistribution")
    public RestResult<String> batchDistribution(@RequestBody UserWhitelistBatchDistributionDTO dto) {
        AdminVO info = null;
        if (dto.getAdminId() == null) {
            info = adminService.getInfo();
            dto.setAdminId(info.getAdminId());
        }
        return userWhitelistService.batchDistribution(dto);
    }


}
