package com.yx.face.controller.api;

import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.iot.AlipayAmpeSignOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayGroupIdOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayOpenIotvspUserwithimageEncryptionRequest;
import com.yx.face.model.dto.iot.AlipayOpenIotvspUserwithimageRequest;
import com.yx.face.model.dto.iot.AlipayOperateRecordCreateWithVidOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayUserDeleteEncryptionRequest;
import com.yx.face.model.dto.iot.AlipayUserDeleteOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayUserStateOpenApiRequest;
import com.yx.face.service.AlipayOpenApiService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author shisl
 * @package cn.hanghui.outapi.platform.alipay.controller
 * @class AlipayOpenApiController
 * @date 2023/5/5 下午2:19
 * @description
 */
@Validated
@Api(tags = "支付宝开放接口")
@RestController
@RequestMapping("/alipay/api/v1/open")
public class AlipayOpenApiController {

    @Autowired
    private AlipayOpenApiService alipayOpenApiService;

    @ApiOperation(value = "查询封闭场景的机构用户库id")
    @PostMapping("/iotvsp/groupid/query")
    public RestResult queryOrgGroupId(@RequestBody AlipayGroupIdOpenApiRequest req) {
        return alipayOpenApiService.queryOrgGroupId(req);
    }

    @ApiOperation(value = "查询机构用户在库结果")
    @PostMapping("/iotvsp/userstate/query")
    public RestResult queryUserState(@RequestBody AlipayUserStateOpenApiRequest req) {
        return alipayOpenApiService.queryUserState(req);
    }

    @ApiOperation(value = "机构用户在库出库")
    @PostMapping("/iotvsp/user/delete")
    public RestResult queryUserState(@RequestBody AlipayUserDeleteOpenApiRequest req) {
        alipayOpenApiService.deleteUser(req);
        return RestResponse.ok();
    }

    @ApiOperation(value = "录入机构用户信息（自定义vid）")
    @PostMapping("/iotvsp/userwithvid/create")
    public RestResult queryUserState(@RequestBody AlipayOperateRecordCreateWithVidOpenApiRequest req) {
        return alipayOpenApiService.createOperateRecordWithVidV2(req);
    }

    @ApiOperation(value = "获取key8设备签名等信息")
    @PostMapping("/ampe/sign")
    public RestResult queryKey8Sign(@RequestBody AlipayAmpeSignOpenApiRequest req) {
        return alipayOpenApiService.queryKey8Sign(req);
    }

    @ApiOperation(value = "录入机构用户信息并直接入库（自定义vid） 非加密版")
    @PostMapping("/iotvsp/decrypt/userwithimage/create")
    public RestResult userwithimage(@RequestBody AlipayOpenIotvspUserwithimageRequest req) {
        return alipayOpenApiService.createOperateRecordWithoutImage(req);
    }

    @ApiOperation(value = "苏州园林录入机构用户信息并直接入库（自定义vid）加密版")
    @PostMapping("/iotvsp/userwithimage/create")
    public RestResult userwithimageEncryption(@RequestBody AlipayOpenIotvspUserwithimageEncryptionRequest req) {
        return alipayOpenApiService.createOperateRecordWithoutImageEncryption(req);
    }

    @ApiOperation(value = "苏州园林机构用户出库（加密版）")
    @PostMapping("/iotvsp/encryption/user/delete")
    public RestResult deleteUserEncryption(@RequestBody AlipayUserDeleteEncryptionRequest req) {
        return alipayOpenApiService.deleteUserEncryption(req);
    }

}
