package com.yx.face.controller.api;

import cn.hutool.core.bean.BeanUtil;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.model.api.building.dto.OpenIdSetDTO;
import com.yx.face.model.api.common.dto.CardIdExSetDTO;
import com.yx.face.model.dto.*;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.vo.*;
import com.yx.face.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;

/**
 * @description: 地址编码表(AreaCode)表控制层 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-06-04 14:25:45 <br>
 */

@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "A 人脸下发相关功能接口 ＡＰＩ")
@Slf4j
@RestController
@RequestMapping("api/next")
public class ApiController extends BaseController {

    /**
     * 服务对象
     */
    private final FaceService faceService;
    private final UserInfoService userInfoService;
    private final AdminService adminService;
    private final AreaCodeService areaCodeService;
    private final VisitorUserInfoService visitorUserInfoService;
    private final AlipayOpenApiService alipayOpenApiService;

    @ApiOperation(value = "获取账号的配置信息")
    @GetMapping("getAdminConfig")
    @ApiOperationSupport(order = 10)
    public RestResult<AdminLessVO> getAdminConfig(@RequestParam Integer adminId) {
        AdminVO info = adminService.getInfo(adminId);
        if (info.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())){
            info = adminService.getInfo(info.getTenantId());
        }

        return RestResponse.ok(BeanUtil.copyProperties(info, AdminLessVO.class));
    }

    @ApiOperation(value = "获取微信用户绑定的手机号")
    @PostMapping("doGetPhone")
    @ApiOperationSupport(order = 20)
    public RestResult<String> phone(@RequestBody WxInfoDTO dto) {
        return RestResponse.ok(userInfoService.getWxPhone(dto));
    }

    /*根据身份证 和 adminId去获取年龄段 判断用户的认证方式
     * 返回值 1-调用第三方（原先的实名认证）  2-前端获取人脸的实名认证
     * */
    @ApiOperation(value = "实名认证方式：返回1-老 2-新")
    @PostMapping("realAuthenticationType")
    @ApiOperationSupport(order = 25)
    public RestResult<Integer> realAuthenticationType(@RequestBody RealAuthenticationTypeDTO realAuthenticationTypeDTO) {
        return RestResponse.ok(userInfoService.realAuthenticationType(realAuthenticationTypeDTO));
    }
    @ApiOperation(value = "注册人脸认证回调接口")
    @PostMapping("doConfirmFaceNotify")
    @ApiOperationSupport(order = 30)
    public void doConfirmFaceNotify(@RequestBody ConfirmFaceNotifyDTO confirmFaceNotifyDTO) {
        userInfoService.confirmFaceNotify(confirmFaceNotifyDTO);
    }
    @ApiOperation(value = "实名认证方式：新")
    @PostMapping("doConfirmFaceNotifyNew")
    @ApiOperationSupport(order = 40)
    public RestResult<ConfirmFaceNotifyNewVO> doConfirmFaceNotifyNew(@RequestBody ConfirmFaceNotifyNewDTO confirmFaceNotifyNewDTO) {
        return userInfoService.doConfirmFaceNotifyNew(confirmFaceNotifyNewDTO);
    }
    @ApiOperation(value = "taskId换用户信息")
    @GetMapping("doQueryFaceSuccess")
    @ApiOperationSupport(order = 50)
    public RestResult<UserInfoIdentificationVO> doQueryFaceSuccess(@RequestParam String taskId) {
        return RestResponse.ok(faceService.queryFaceSuccess(taskId));
    }
    @ApiOperation(value = "用户OpenId填入")
    @ApiOperationSupport(order = 60)
    @PostMapping("openIdSet")
    public RestResult<Boolean> openIdSet(@RequestBody OpenIdSetDTO openIdSetDTO) {
        return userInfoService.openIdSet(openIdSetDTO);
    }
    @ApiOperation(value = "物理卡号填入")
    @ApiOperationSupport(order = 70)
    @PostMapping("cardIdExSet")
    public RestResult<Boolean> cardIdExSet(@RequestBody CardIdExSetDTO dto) {
        return userInfoService.cardIdExSet(dto);
    }
    @ApiOperation(value = "用户信息录入常客/访客表")
    @PostMapping("userInfoInsert")
    @ApiOperationSupport(order = 80)
    public RestResult<UserInfoInsertVO> userInfoInsert(@RequestBody UserInfoInsertDTO dto) {
        return RestResponse.ok(userInfoService.userInfoInsert(dto));
    }


    @ApiOperation("小程序获取账号是否有自定义字段")
    @GetMapping("getContextByAdminId")
    public RestResult<Boolean> getContextByAdminId(Integer adminId) {
        return RestResponse.ok(faceService.getContextByAdminId(adminId));
    }

    @ApiOperation("小程序根据账号id获取自定义字段")
    @GetMapping("getContextById")
    public RestResult<List<ContextVO>> getContextById(Integer adminId) {
        return RestResponse.ok(faceService.getContextById(adminId));
    }

    @ApiOperation("小程序根据用户id添加自定义字段")
    @PostMapping("InsertContentByCardId")
    public RestResult<Boolean> InsertContentById(@RequestBody Map map) {
        Integer id = (Integer) map.get("id");
        Integer adminId = (Integer) map.get("adminId");
        List<ContextVO> list = (List<ContextVO>) map.get("list");
        faceService.modifyUserInfo(id, adminId);
        UserInfoVO user = faceService.getUser(id);
        String idNumber = user.getIdNumber();
        Integer count = faceService.getUserVisitorList(adminId, idNumber);
        if (count == 1) {
            return RestResponse.ok(faceService.updateVisitorContent(adminId, idNumber, list));
        } else if (count == 0) {
            return RestResponse.ok(faceService.updateWhiteContent(adminId, idNumber, list));
        } else {
            throw new ServiceException("添加自定义字段失败!");
        }
    }

    @ApiOperation("小程序字段是否填写")
    @GetMapping("selectContextByCardId")
    public RestResult<List<ContextVO>> getContextByCardId(Integer id, Integer adminId) {
        return RestResponse.ok(userInfoService.getContextByCardId(id, adminId));
    }

    @ApiOperation("获取省级列表")
    @GetMapping("getProvince")
    public RestResult<List<AreaCodeVOList>> doGetInfo() {
        return RestResponse.ok(areaCodeService.getProvince());
    }

    @ApiOperation("获取城市列表")
    @GetMapping("getCity")
    public RestResult<List<AreaCodeVOList>> getCity(@ApiParam(name = "areaCodeId", value = "省份的id值", required = true) Integer areaCodeId) {
        if (areaCodeId == null) {
            return RestResponse.error("省份的主键值不能为空");
        }
        return RestResponse.ok(areaCodeService.getCity(areaCodeId));
    }

    @ApiOperation("获取县区列表")
    @GetMapping("getArea")
    public RestResult<List<AreaCodeVOList>> getArea(@ApiParam(name = "areaCodeId", value = "市级的id值", required = true) Integer areaCodeId) {
        if (areaCodeId == null) {
            return RestResponse.error("城市的主键值不能为空");
        }
        return RestResponse.ok(areaCodeService.getArea(areaCodeId));
    }

    @ApiOperation("获取街道乡镇列表")
    @GetMapping("getStreet")
    public RestResult<List<AreaCodeVOList>> getStreet(@ApiParam(name = "areaCodeId", value = "县区的id值", required = true) Integer areaCodeId) {
        if (areaCodeId == null) {
            return RestResponse.error("县区的主键值不能为空");
        }
        return RestResponse.ok(areaCodeService.getStreet(areaCodeId));
    }

    @ApiOperation(value = "taskId换取查询访客机用户数据")
    @GetMapping("getVisitorUserInfo")
    @ApiOperationSupport(order = 112)
    public RestResult<VisitorUserInfoVO> getVisitorUserInfo(@RequestParam String taskId) {
        return visitorUserInfoService.getVisitorUserInfo(taskId);
    }

    @ApiOperation("更新访客机用户")
    @PostMapping("updateVisitDeviceUser")
    @ApiOperationSupport(order = 113)
    public RestResult<Long> updateVisitDeviceUser(@RequestBody UpdateVisitDeviceUserVO updateVisitDeviceUserVO) {
        return RestResponse.ok(visitorUserInfoService.updateVisitDeviceUser(updateVisitDeviceUserVO));
    }

    @ApiOperation(value = "访客机用户数据填入")
    @PostMapping("visitorUserInfoInPut")
    @ApiOperationSupport(order = 111)
    public void visitorUserInfoInPut(HttpServletRequest request) {
        log.info("访客机数据填入");
        StringBuilder sb = new StringBuilder();
        try (InputStream is = request.getInputStream();
             BufferedInputStream bis = new BufferedInputStream(is)) {
            byte[] buffer = new byte[1024];
            int read;
            while ((read = bis.read(buffer)) != -1) {
                sb.append(new String(buffer, 0, read, StandardCharsets.UTF_8));
            }
        } catch (IOException e) {
            return;
        }
        visitorUserInfoService.visitorUserInfoInPut(sb.toString());
    }

    @ApiOperation(value = "支付宝-实名认证")
    @PostMapping("ZFBConfirmFace")
    public RestResult<ConfirmFaceNotifyNewVO> ZFBConfirmFace(@RequestBody ZFBConfirmFaceDTO zfbConfirmFaceDTO) {
        return userInfoService.doConfirmFaceNotifyNew(zfbConfirmFaceDTO);
    }

    @ApiOperation(value = "支付宝-开放平台解密")
    @PostMapping("alipay/decrypt")
    public RestResult<String> decryptContent(@RequestBody String encryptedResponse) {
        return RestResponse.ok(alipayOpenApiService.decryptContent(encryptedResponse));
    }

}
