package com.yx.face.controller.api;

import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ScanCodeRestCode;
import com.yx.face.model.api.fkj.dto.FkjAreaInfoDto;
import com.yx.face.model.api.fkj.dto.FkjEthQrCodeDto;
import com.yx.face.model.api.fkj.dto.FkjJshunQrCodeDto;
import com.yx.face.model.api.fkj.dto.FkjQrCodeDto;
import com.yx.face.model.api.fkj.dto.FkjVguangQrCodeDto;
import com.yx.face.model.api.fkj.vo.FkjAreaInfoVo;
import com.yx.face.model.api.fkj.vo.FkjEthQrCodeVo;
import com.yx.face.model.api.fkj.vo.FkjQrCodeVo;
import com.yx.face.model.api.fkj.vo.VisiteeVo;
import com.yx.face.service.fkj.FkjService;
import com.yx.face.service.fkj.ScanCodeBoxService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;


@Api(tags = "A 访客机相关功能接口API")
@Slf4j
@RestController
@RequestMapping("api/v1/fkj")
public class ApiFkjController {

    @Autowired
    private FkjService fkjService;

    @Autowired
    private ScanCodeBoxService scanCodeBoxService;

    @ApiOperation(value = "获取访客机辖区配置信息")
    @PostMapping("/area/info")
    public RestResult<FkjAreaInfoVo> areaInfo(@Valid @RequestBody FkjAreaInfoDto fkjAreaInfoDto) {
        return RestResponse.ok(fkjService.getAreaInfoBySn(fkjAreaInfoDto.getSn()));
    }

    @ApiOperation(value = "获取受访人列表")
    @PostMapping("/visitee/list")
    public RestResult<List<VisiteeVo>> list(@Valid @RequestBody FkjAreaInfoDto fkjAreaInfoDto) {
        return RestResponse.ok(fkjService.list(fkjAreaInfoDto.getSn()));
    }

    @ApiOperation(value = "设备上报心跳（30秒一次）")
    @PostMapping("/heart/tracking")
    public RestResult<Boolean> tracking(@Valid @RequestBody FkjAreaInfoDto fkjAreaInfoDto) {
        return RestResponse.ok(fkjService.tracking(fkjAreaInfoDto.getSn()));
    }

    //访客机生成二维码
    @ApiOperation(value = "访客单机版二维码生成")
    @PostMapping("/visitor/eth/qrCode")
    public RestResult<FkjEthQrCodeVo> ethQrCode(@Valid @RequestBody FkjEthQrCodeDto dto) {
        return RestResponse.ok(fkjService.ethQrCode(dto));
    }

    @ApiOperation(value = "微光互联访客二维码生成")
    @PostMapping("/visitor/vguang/qrCode")
    public RestResult<FkjEthQrCodeVo> vguangQrCode(@Valid @RequestBody FkjVguangQrCodeDto dto) {
        return RestResponse.ok(fkjService.vguangQrCode(dto));
    }

    @ApiOperation(value = "捷顺访客二维码生成")
    @PostMapping("/visitor/jslife/qrCode")
    public RestResult<FkjEthQrCodeVo> jslifeQrCode(@Valid @RequestBody FkjJshunQrCodeDto dto) {
//        fkjService.jslifeQrCode(dto);
        FkjEthQrCodeVo vo = new FkjEthQrCodeVo();
        vo.setQrCodeContent("开发中。。。");
        return RestResponse.ok(vo);
    }

    @ApiOperation(value = "根据三要素获取访客二维码生成")
    @PostMapping("/visitor/qrCode")
    public RestResult<List<FkjQrCodeVo>> visitorQrCode(@Valid @RequestBody FkjQrCodeDto dto) {
        log.info("--->根据三要素获取访客二维码入参:{}", JSONObject.toJSONString(dto));
        List<FkjQrCodeVo> fkjQrCodeVos = fkjService.visitorQrCode(dto);
        return RestResponse.ok(fkjQrCodeVos);
    }

    @ApiOperation("核销微光互联二维码")
    @PostMapping("/visitor/qrcode/verification")
    public void verification(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String message = null;
        try {
            Map<String, Object> param = getParam(request);
            if (param.isEmpty()) {
                message = "code=" + ScanCodeRestCode.FAIL.getCode() + "&&desc=" + ScanCodeRestCode.FAIL.getMsg();
            }
            if (message == null) {
                log.info("---->request:{}", JSONObject.toJSONString(param));
                //校验数据是否存在
                String code = param.get("vgdecoderesult").toString();
                //校验数据是否存在
                Boolean existFlag = scanCodeBoxService.checkQrcodeExist(code);
                if (!existFlag) {
                    //不存在
                    message = "code=" + ScanCodeRestCode.FAIL.getCode() + "&&desc=" + ScanCodeRestCode.FAIL.getMsg();
                }

//                //校验数据是否重复
//                if (message == null) {//数据已重复提交
//                    Boolean addFlag = scanCodeBoxService.addScanCodeBoxCode(code);
//                    if (!addFlag) {
//                        message = "code=" + ScanCodeRestCode.USED.getCode() + "&&desc=" + ScanCodeRestCode.USED.getMsg();
//                    }
//                }

                if (message == null) {//提交数据
                    Boolean success = scanCodeBoxService.dataCreate(code);
                    if (!success) {
                        message = "code=" + ScanCodeRestCode.FAIL.getCode() + "&&desc=" + ScanCodeRestCode.FAIL.getMsg();
                    }
                }
            }

            if (message == null) {//成功
                message = "code=" + ScanCodeRestCode.SUCCESS.getCode() + "&&desc=" + ScanCodeRestCode.SUCCESS.getMsg();
            }
        } catch (Exception e) {
            e.printStackTrace();
            message = "code=" + ScanCodeRestCode.FAIL.getCode() + "&&desc=" + ScanCodeRestCode.FAIL.getMsg();
        }

        // 以流的形式返回
        ServletOutputStream out = null;
        ByteArrayOutputStream baos = null;
        try {
            InputStream inStream = new ByteArrayInputStream(message.getBytes());
            byte[] buffer = new byte[1024];
            int len;
            baos = new ByteArrayOutputStream();
            while ((len = inStream.read(buffer)) != -1) {
                baos.write(buffer, 0, len);
            }
            out = response.getOutputStream();
            out.write(baos.toByteArray());
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            Objects.requireNonNull(baos).flush();
            baos.close();
            Objects.requireNonNull(out).flush();
            out.close();
        }
    }

    private Map<String, Object> getParam(HttpServletRequest request) {
        StringBuilder sb = new StringBuilder();
        try (InputStream is = request.getInputStream();
             BufferedInputStream bis = new BufferedInputStream(is)) {
            byte[] buffer = new byte[1024];
            int read;
            while ((read = bis.read(buffer)) != -1) {
                sb.append(new String(buffer, 0, read, StandardCharsets.UTF_8));
            }
            String paramStr = sb.toString();
            if (!StringUtils.hasText(paramStr)) {
                return Collections.EMPTY_MAP;
            }
            String[] split = paramStr.split("&&");
            List<String> params = Arrays.asList(split);
            Map<String, Object> map = new HashMap<>();
            for (String param : params) {
                String[] splits = param.split("=");
                if (splits.length > 1) {
                    String key = param.split("=")[0];
                    String value = param.split("=")[1];
                    map.put(key, value);
                }
            }
            return map;
        } catch (Exception e) {
            return Collections.EMPTY_MAP;
        }
    }


}
