package com.yx.face.controller.api;

import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.HealthCodeConfigDTO;
import com.yx.face.model.dto.OpenIssuedDTO;
import com.yx.face.model.dto.PlaceRecordDTO;
import com.yx.face.model.dto.UserOpenJudgeDto;
import com.yx.face.model.dto.iot.IotUserWhitelistDto;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.enums.AdminSecretEnum;
import com.yx.face.model.search.QueryExamUserWhitelistSearch;
import com.yx.face.model.vo.PlaceInfoVO;
import com.yx.face.model.vo.QueryExamUserWhitelistVO;
import com.yx.face.server.dto.TokenDataDTO;
import com.yx.face.server.vo.HealthResultVO;
import com.yx.face.service.OpenIssuedService;
import com.yx.face.service.SpiIotUserWhitelistService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author: lileilei
 * @date: 2022/6/4 19:45
 * @description:
 */
@Api(tags = "开放门禁接口服务")
@RestController
@RequestMapping("api/open")
public class ApiOpenController {
    @Autowired
    private OpenIssuedService openIssuedService;

    @Autowired
    private SpiIotUserWhitelistService spiIotUserWhitelistService;

    @ApiOperation(value = "常访客人脸新增/删除下发")
    @PostMapping("/issued")
    public RestResult issued(@RequestBody OpenIssuedDTO openIssuedDTO) {
        if (AdminSecretEnum.adminIdOf(openIssuedDTO.getAdminId()) == null
                || !AdminSecretEnum.adminIdOf(openIssuedDTO.getAdminId()).getSecret().equals(openIssuedDTO.getSecret())) {
            return RestResponse.error("密钥验签失败，请检查");
        }
        openIssuedService.issued(openIssuedDTO);
        return RestResponse.ok();
    }

    @ApiOperation(value = "查询账号所属设备信息")
    @GetMapping("/queryFaceDevices")
    public RestResult<List<FaceDevice>> queryFaceDevices(Long adminId, String secret) {
        if (AdminSecretEnum.adminIdOf(adminId) == null || !AdminSecretEnum.adminIdOf(adminId).getSecret().equals(secret)) {
            return RestResponse.error("密钥验签失败，请检查");
        }

        return RestResponse.ok(openIssuedService.queryFaceDevices(adminId));
    }

    @ApiOperation("通过设备sn查询场所码信息")
    @GetMapping("/queryPlaceInfoBySn")
    public RestResult<PlaceInfoVO> queryPlaceInfoBySn(String sn) {
        return openIssuedService.queryPlaceInfoBySn(sn);
    }

    @ApiOperation("通过设备sn查询开放场所码配置信息")
    @GetMapping("/queryOpenPlaceInfoBySn")
    public RestResult<PlaceInfoVO> queryOpenPlaceInfoBySn(String sn) {
        return openIssuedService.queryOpenPlaceInfoBySn(sn);
    }

    @ApiOperation("场所码注册")
    @PostMapping("/saveHealthCodeConfig")
    public RestResult saveHealthCodeConfig(@RequestBody HealthCodeConfigDTO configDTO) {
        return openIssuedService.saveHealthCodeConfig(configDTO);
    }

    @ApiOperation("保存场所记录")
    @PostMapping("/savePlaceRecord")
    public RestResult savePlaceRecord(@RequestBody PlaceRecordDTO dto) {
        return RestResponse.ok(openIssuedService.savePlaceRecord(dto));
    }

    @ApiOperation("场所码token凭证")
    @GetMapping("/getToken")
    public HealthResultVO<TokenDataDTO> getToken(Boolean cleanSession, Integer placeAreaCode) {
        return openIssuedService.getToken(cleanSession, placeAreaCode);
    }

    @ApiOperation("获取考生信息")
    @PostMapping("/getExamUserWhitelist")
    public RestResult<QueryExamUserWhitelistVO> getExamUserWhitelist(@RequestBody QueryExamUserWhitelistSearch dto) {
        return RestResponse.ok(openIssuedService.getExamUserWhitelist(dto));
    }

    @ApiOperation("校验iot用户是否可以通行")
    @PostMapping("/checkIotUserPass")
    public RestResult checkIotUserPass(@RequestBody IotUserWhitelistDto dto) {
        return spiIotUserWhitelistService.checkIotUserPass(dto);
    }
    @ApiOperation("校验用户是否可以通行")
    @PostMapping("/userOpenJudgeUrl")
    public RestResult<Boolean> userOepnJudgeUrl(@RequestBody UserOpenJudgeDto dto) {
        return spiIotUserWhitelistService.userOepnJudgeUrl(dto);
    }


}
