package com.yx.face.controller.api;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.model.api.appointment.dto.*;
import com.yx.face.model.api.appointment.vo.*;
import com.yx.face.model.vo.UserInfoVO;
import com.yx.face.model.vo.eth.EthCodeDto;
import com.yx.face.model.vo.eth.EthCodeVO;
import com.yx.face.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/*
 * @创建者  WXG
 * @描述
 * @时间  2022/5/24 13:12
 * @参数
 **/
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "A 小程序场所预约相关 API")
@RestController
@RequestMapping("api/appointmentApi")
public class AppointmentApiController extends BaseController {

    private final AppointmentApiService appointmentApiService;
    private final OCRService ocrService;
    private final BaiDuFaceService baiDuFaceService;

    @ApiOperation(value = "查询账号基础信息")
    @ApiOperationSupport(order = 1)
    @GetMapping("getInfo")
    public RestResult<PlaceInfoVO> getInfo(@RequestParam("adminId") Integer adminId) {
        return RestResponse.ok(appointmentApiService.getInfo(adminId));
    }

    @ApiOperation(value = "查询预约信息")
    @ApiOperationSupport(order = 1)
    @PostMapping("getAppointmentInfo")
    public RestResult<AppointmentInfoVO> getAppointmentInfo(@RequestBody AppointmentInfoDTO appointmentInfoDTO) {
        return RestResponse.ok(appointmentApiService.getAppointmentInfo(appointmentInfoDTO));
    }

    @ApiOperation(value = "查询可以预约的日期")
    @ApiOperationSupport(order = 2)
    @GetMapping("getTimeList")
    public RestResult<List<DateVO>> getTimeList(@RequestParam(name = "adminId") Integer adminId) {
        return RestResponse.ok(appointmentApiService.getTimeList(adminId));
    }

    @ApiOperation(value = "查询预约的时间段列表")
    @ApiOperationSupport(order = 3)
    @PostMapping("getTimeSlotList")
    public RestResult<List<TimeSlotVO>> getTimeSlotList(@RequestBody TimeSlotDTO timeSlotDTO) {
        return RestResponse.ok(appointmentApiService.getTimeSlotList(timeSlotDTO));
    }

    @ApiOperation(value = "查询是否可以进行实名认证")
    @ApiOperationSupport(order = 4)
    @PostMapping("selectAuthenticationAuthority")
    public RestResult<Boolean> selectAuthenticationAuthority(@RequestBody AuthenticationAuthorityDTO authenticationAuthorityDTO) {
        return appointmentApiService.selectAuthenticationAuthority(authenticationAuthorityDTO);
    }

    @ApiOperation(value = "OCR数据获取")
    @ApiOperationSupport(order = 5)
    @GetMapping("getPassportDetail")
    public RestResult<OCRVO> getPassportDetail(@RequestParam(name = "type") Integer type, @RequestParam("passportUrl") String passportUrl) {
        return RestResponse.ok(ocrService.getPassportDetail(passportUrl, type));
    }

    @ApiOperation(value = "外宾实名认证/修改")
    @ApiOperationSupport(order = 6)
    @PostMapping("foreignAuthentication")
    public RestResult<UserInfoVO> foreignAuthentication(@Valid @RequestBody ForeignAuthenticationDTO foreignAuthenticationDTO) {
        return appointmentApiService.foreignAuthentication(foreignAuthenticationDTO);
    }

    @ApiOperation(value = "人证比对")
    @ApiOperationSupport(order = 6)
    @PostMapping("BaiDuFaceComparison")
    public RestResult<Boolean> BaiDuFaceComparison(@RequestBody BaiduFaceDTO baiduFaceDTO) {
        return baiDuFaceService.BaiDuFaceComparison(baiduFaceDTO);
    }

    @ApiOperation(value = "预约登记")
    @ApiOperationSupport(order = 7)
    @PostMapping("appointment")
    public RestResult<Boolean> appointment(@RequestBody AppointmentDTO appointmentDTO) {
        return appointmentApiService.appointment(appointmentDTO);
    }

    //    @ApiOperation(value = "修改人员信息")
//    @ApiOperationSupport(order = 7)
//    @PostMapping("updateUserInfo")
//    public RestResult<Boolean> updateUserInfo(@Valid @RequestBody UpdateUserInfoDTO updateUserInfoDTO) {
//        return appointmentApiService.updateUserInfo(updateUserInfoDTO);
//    }
    @ApiOperation(value = "查看预约单")
    @ApiOperationSupport(order = 8)
    @PostMapping("lookAppointment")
    public RestResult<LookAppointmentVO> lookAppointment(@RequestBody UserAdminDTO userAdminDTO) {
        return appointmentApiService.lookAppointment(userAdminDTO);
    }

    @ApiOperation(value = "查看预约记录")
    @ApiOperationSupport(order = 9)
    @PostMapping("lookAppointmentRecord")
    public RestResult<List<LookAppointmentVO>> lookAppointmentRecord(@RequestBody UserAdminDTO userAdminDTO) {
        return appointmentApiService.lookAppointmentRecord(userAdminDTO);
    }

    @ApiOperation(value = "取消预约")
    @ApiOperationSupport(order = 10)
    @PostMapping("cancelAppointment")
    public RestResult<Boolean> cancelAppointment(@RequestBody CancelAppointmentDTO cancelAppointmentDTO) {
        return appointmentApiService.cancelAppointment(cancelAppointmentDTO);
    }

    @ApiOperation(value = "查看用户数据")
    @ApiOperationSupport(order = 11)
    @GetMapping("getUserInfoVO")
    public RestResult<UserInfoVO> getUserInfoVO(@RequestParam(name = "userId") Integer userId) {
        return appointmentApiService.getUserInfoVO(userId);
    }

    @ApiOperation(value = "获取用户梯控码信息")
    @ApiOperationSupport(order = 12)
    @PostMapping("whitelist/eth/get")
    public RestResult<EthCodeVO> getUserInfoEthCode(@RequestBody EthCodeDto dto) {
        if (Preconditions.isBlank(dto.getAdminId()) || Preconditions.isBlank(dto.getUserId())) {
            throw new CommonException("参数有误");
        }
        return appointmentApiService.getUserEthCode(dto);
    }


}
