package com.yx.face.controller.api;

import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.model.api.building.dto.*;
import com.yx.face.model.api.building.vo.*;
import com.yx.face.model.api.building.vo.AdminLessVO;
import com.yx.face.model.dto.VisitorQrcodeDto;
import com.yx.face.model.dto.VisitorQrcodeIdDto;
import com.yx.face.model.entity.UserInfo;
import com.yx.face.model.vo.*;
import com.yx.face.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/*
 * @创建者  WXG
 * @描述  常访预约登记和常客邀约
 * @时间  2022/5/31 16:12
 * @参数
 **/
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "A 小程序常访客相关 API")
@RestController
@RequestMapping("api/buildingApi")
public class BuildingApiController extends BaseController {
    private final OCRService ocrService;
    private final BuildingApiService buildingApiService;
    private final UserInfoDao userInfoDao;
    private final SmsService smsService;

    /*通用接口*/
    @ApiOperation(value = "检查常客是否仍然存在")
    @ApiOperationSupport(order = 1)
    @GetMapping("checkUserWhiteListId")
    public RestResult<Boolean> checkUserWhiteListId(@RequestParam("userWhiteListId") @ApiParam(value = "常客Id") Integer userWhiteListId) {
        return buildingApiService.checkUserWhiteListId(userWhiteListId);
    }

    @ApiOperation(value = "查询楼宇列表")
    @ApiOperationSupport(order = 2)
    @GetMapping("getBuildingList")
    public RestResult<List<BuildingLessVO>> getBuildingList() {
        return RestResponse.ok(buildingApiService.getBuildingList());
    }
    @ApiOperation(value = "通过公司Id查询楼宇和公司信息")
    @ApiOperationSupport(order = 4)
    @GetMapping("getBuildingAndCompanyInfo")
    public RestResult<AdminLessVO> getBuildingAndCompanyInfo(@RequestParam("adminId") @ApiParam(value = "公司Id") Integer adminId) {
        return buildingApiService.getBuildingAndCompanyInfo(adminId);
    }

    @ApiOperation(value = "查询公司列表")
    @ApiOperationSupport(order = 5)
    @GetMapping("getCompanyList")
    public RestResult<List<BuildingLessVO>> getCompanyList(@RequestParam("adminId") Integer adminId) {
        return RestResponse.ok(buildingApiService.getCompanyList(adminId));
    }

    @ApiOperation(value = "查询三码信息")
    @ApiOperationSupport(order = 6)
    @GetMapping("getThreeCodeByAdminId")
    public RestResult<AdminVO> getThreeCodeByAdminId(@RequestParam("adminId") Integer adminId) {
        return buildingApiService.getThreeCodeByAdminId(adminId);
    }

    @ApiOperation(value = "查看用户是否有OpenId")
    @ApiOperationSupport(order = 7)
    @GetMapping("selectOpenIdByuserId")
    public RestResult<Boolean> selectOpenIdByuserId(@RequestParam("userId") Integer userId) {
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
        if (userInfo == null) {
            return RestResponse.error("数据有误");
        }
        if (StringUtils.isEmpty(userInfo.getBuildingOpenId())) {
            return RestResponse.ok(false);
        } else {
            return RestResponse.ok(true);
        }
    }


    /*常客接口*/
    @ApiOperation(value = "常客认证校验")
    @ApiOperationSupport(order = 10)
    @PostMapping("userWhiteCheck")
    public RestResult<List<UserWhiteCheckVO>> userWhiteCheck(@Valid @RequestBody UserWhiteCheckDTO userWhiteCheckDTO) {
        return buildingApiService.userWhiteCheck(userWhiteCheckDTO);
    }

    @ApiOperation(value = "常客获取楼宇的设备列表(sn和name)")
    @ApiOperationSupport(order = 11)
    @GetMapping("userWhiteGetSnList")
    public RestResult<List<FaceDeviceVO>> userWhiteGetSnList(@RequestParam("adminId") @ApiParam(value = "楼宇Id") Integer adminId, @RequestParam("companyId") @ApiParam(value = "公司账户Id") Integer companyId) {
        return buildingApiService.userWhiteGetSnList(adminId, companyId);
    }

    @ApiOperation(value = "常客邀请访客")
    @ApiOperationSupport(order = 12)
    @PostMapping("userWhiteInviteUserVisitor")
    public RestResult<List<UserWhiteInviteVO>> userWhiteInviteUserVisitor(@Valid @RequestBody UserWhiteInviteDTO userWhiteInviteDTO) {
        return buildingApiService.userWhiteInviteUserVisitor(userWhiteInviteDTO);
    }

    @ApiOperation(value = "常客查询拜访记录列表")
    @ApiOperationSupport(order = 13)
    @PostMapping("userWhiteVisitList")
    public RestResult<PageInfo<VisitoryVisitListVO>> userWhiteVisitList(@Valid @RequestBody RestDTO<UserWhiteVisitDTO> dto) {
        return buildingApiService.userWhiteVisitList(dto);
    }

    @ApiOperation(value = "常客处理访客申请记录")
    @ApiOperationSupport(order = 14)
    @PostMapping("userWhiteHandleVisitList")
    public RestResult<Boolean> userWhiteHandleVisitList(@Valid @RequestBody UserWhiteHandleDTO userWhiteHandleDTO) {
        return buildingApiService.userWhiteHandleVisitList(userWhiteHandleDTO);
    }

    @ApiOperation(value = "常客处理访客申请记录(支付宝)")
    @ApiOperationSupport(order = 14)
    @PostMapping("/alipay/userWhiteHandleVisitList")
    public RestResult<Boolean> alipayUserWhiteHandleVisitList(@Valid @RequestBody UserWhiteHandleDTO userWhiteHandleDTO) {
        return buildingApiService.alipayUserWhiteHandleVisitList(userWhiteHandleDTO);
    }

    @ApiOperation(value = "常客获取设备列表")
    @ApiOperationSupport(order = 15)
    @GetMapping("userWhiteSnList")
    public RestResult<List<FaceDeviceVO>> userWhiteSnList(@RequestParam("adminId") @ApiParam("楼宇id") Integer adminId) {
        return buildingApiService.userWhiteSnList(adminId);
    }

    @ApiOperation(value = "常客查看公司信息")
    @ApiOperationSupport(order = 15)
    @GetMapping("userWhiteLookCompanyInfo")
    public RestResult<List<CompanyVO>> userWhiteLookCompanyInfo(@RequestParam("userWhiteCheckVOList") List<UserWhiteCheckVO> userWhiteCheckVOList) {
        return buildingApiService.userWhiteLookCompanyInfo(userWhiteCheckVOList);
    }

    /*访客接口*/
    @ApiOperation(value = "根据访客记录Id获取记录")
    @ApiOperationSupport(order = 20)
    @GetMapping("getVisitoryByUserVisitorListDetailId")
    public RestResult<UserVisitorListDetailVO> getVisitoryByUserVisitorListDetailId(@RequestParam("userVisitorListDetailId") @ApiParam(value = "访客记录Id") Integer userVisitorListDetailId) {
        return buildingApiService.getVisitoryByUserVisitorListDetailId(userVisitorListDetailId);
    }

    @ApiOperation(value = "根据受访人手机号和楼宇获取受访人信息")
    @ApiOperationSupport(order = 21)
    @PostMapping("getUserWhiteByphoneAndAdminId")
    public RestResult<List<UserWhiteIntervieweeBuildingVO>> getUserWhiteByphoneAndAdminId(@RequestBody UserWhiteIntervieweeDTO userWhiteIntervieweeDTO) {
        return buildingApiService.getUserWhiteByphoneAndAdminId(userWhiteIntervieweeDTO);
    }

    @ApiOperation(value = "访客申请提交")
    @ApiOperationSupport(order = 22)
    @PostMapping("visitorySubmit")
    public RestResult<Boolean> visitorySubmit(@Valid @RequestBody VisitorySubmitDTO visitorySubmitDTO) {
        return buildingApiService.visitorySubmit(visitorySubmitDTO);
    }

    @ApiOperation(value = "访客申请提交(支付宝)")
    @ApiOperationSupport(order = 22)
    @PostMapping("/alipay/visitorySubmit")
    public RestResult<Boolean> alipayVisitorySubmit(@Valid @RequestBody VisitorySubmitDTO visitorySubmitDTO) {
        return buildingApiService.alipayVisitorySubmit(visitorySubmitDTO);
    }

    @ApiOperation(value = "访客查询拜访记录列表")
    @ApiOperationSupport(order = 23)
    @PostMapping("visitoryVisitList")
    public RestResult<PageInfo<VisitoryVisitListVO>> visitoryVisitList(@Valid @RequestBody RestDTO<VisitoryVisitDTO> dto) {
        return buildingApiService.visitoryVisitList(dto);
    }

    @ApiOperation(value = "访客的重新下发")
    @ApiOperationSupport(order = 24)
    @GetMapping("visitoryDownFaceAgain")
    public RestResult<Boolean> visitoryDownFaceAgain(@RequestParam("userVisitorListDetailId") @ApiParam(value = "访客记录Id") Integer userVisitorListDetailId) {
        return buildingApiService.visitoryDownFaceAgain(userVisitorListDetailId);
    }

    /*企业录入相关接口*/
    @ApiOperation(value = "营业执照OCR数据获取")
    @ApiOperationSupport(order = 31)
    @GetMapping("getOCRDetail")
    public RestResult<BusinessLicenseVO> getOCRDetail(@RequestParam("passportUrl") String passportUrl) {
        return RestResponse.ok(ocrService.getOCRDetail(passportUrl));
    }

    @ApiOperation(value = "企业信息提交审核")
    @ApiOperationSupport(order = 32)
    @PostMapping("companyCommit")
    public RestResult<Boolean> companyCommit(@Valid @RequestBody CompanyCommitDTO companyCommitDTO) {
        return buildingApiService.companyCommit(companyCommitDTO);
    }

    @ApiOperation(value = "企业信息提交审核(支付宝)")
    @ApiOperationSupport(order = 32)
    @PostMapping("/alipay/companyCommit")
    public RestResult<Boolean> alipayCompanyCommit(@Valid @RequestBody CompanyCommitDTO companyCommitDTO) {
        return buildingApiService.alipayCompanyCommit(companyCommitDTO);
    }

    @ApiOperation(value = "查询企业审核情况")
    @ApiOperationSupport(order = 33)
    @GetMapping("companyCommitInfoList")
    public RestResult<List<CompanyVO>> companyCommitInfoList(@RequestParam("userId") Integer userId) {
        return buildingApiService.companyCommitInfoList(userId);
    }

    /**
     * 获取访客qrcodeId
     *
     * @return QrcodeId
     */
    @ApiOperation(value = "获取访客qrcodeId")
    @ApiOperationSupport(order = 34)
    @PostMapping("getVisitorQrcodeId")
    public RestResult<VisitorQrcodeIdDto> getVisitorQrcodeId(@RequestBody VisitorQrcodeDto visitorQrcodeDto) {
        return RestResult.ok(buildingApiService.getVisitorQrcodeId(visitorQrcodeDto));
    }

    @ApiOperation(value = "常客邀请访客(支付宝短信)")
    @ApiOperationSupport(order = 12)
    @PostMapping("/alipay/userWhiteInviteUserVisitor")
    public RestResult<List<UserWhiteInviteVO>> alipayUserWhiteInviteUserVisitor(@Valid @RequestBody UserWhiteInviteDTO userWhiteInviteDTO) {
        return buildingApiService.alipayUserWhiteInviteUserVisitor(userWhiteInviteDTO);
    }

//    /*
//     * 测试微信订阅用的调用接口
//     *
//     * */
//    @ApiOperation(value = "用于测试的")
//    @ApiOperationSupport(order = 34)
//    @GetMapping("test")
//    public void companyCommitInfoList() {
//        UserVisitorListDetail userVisitorListDetail = new UserVisitorListDetail();
//        userVisitorListDetail.setVisiteeCompany("测试");
//        userVisitorListDetail.setPhone("13515834834");
//        userVisitorListDetail.setId((long) 1);
//        userVisitorListDetail.setVisiteeXm("姓名");
//        userVisitorListDetail.setEndTime(DateUtils.addDays(new Date(), 2));
//        smsService.createUrl(userVisitorListDetail);
////        smsService.subscribeMessage(null, "13515834834",WXMessageTemplateEnum.COMPANY_COMMIT,"测试","待审核",null,null,null);
//    }
}
