package com.yx.face.controller.api;

import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.core.BaseController;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.api.district.DistrictEncryptionRequest;
import com.yx.face.model.api.district.DistrictEncryptionResponse;
import com.yx.face.model.api.user.UserEncryptionRequest;
import com.yx.face.model.api.user.UserEncryptionResponse;
import com.yx.face.model.api.websocket.SendMsg2ServerRequest;
import com.yx.face.service.udistrict.OpenUserDistrictService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.java_websocket.client.WebSocketClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * @author shisl
 * @package com.yx.face.controller.api
 * @class OpenUserDistrictController
 * @date 2024/3/19 下午5:19
 * @description
 */
@Validated
@Api(tags = "用户区域管理")
@RestController
@RequestMapping("/api/v1/open")
public class OpenUserDistrictController extends BaseController {

    @Autowired
    private OpenUserDistrictService openUserDistrictService;

    @Resource
    private WebSocketClient otaWebSocketClient;

    @ApiOperation("根据账号id获取自定义字段")
    @GetMapping("/user/customField/query")
    public RestResult<UserEncryptionResponse> userCustomFieldQuery(@RequestBody @Valid UserEncryptionRequest req) {
        return openUserDistrictService.userCustomFieldQuery(req);
    }

    @ApiOperation("添加/修改用户信息")
    @PostMapping("/user/create")
    public RestResult<UserEncryptionResponse> userCreate(@RequestBody @Valid UserEncryptionRequest req) {
        return openUserDistrictService.userCreate(req);
    }

    @ApiOperation("删除常客")
    @PostMapping("user/delete")
    public RestResult<UserEncryptionResponse> userDelete(@RequestBody @Valid UserEncryptionRequest req) {
        return openUserDistrictService.userDelete(req);
    }

    @ApiOperation("查询常客")
    @PostMapping("user/query")
    public RestResult<UserEncryptionResponse> userQuery(@RequestBody @Valid UserEncryptionRequest req) {
        return openUserDistrictService.userQuery(req);
    }

    @ApiOperation("常客下发")
    @PostMapping("user/distribute")
    public RestResult<UserEncryptionResponse> userDistribute(@RequestBody @Valid UserEncryptionRequest req) {
        return openUserDistrictService.userDistribute(req);
    }

    @ApiOperation("添加区域")
    @PostMapping("/district/create")
    public RestResult<DistrictEncryptionResponse> districtCreate(@RequestBody @Valid DistrictEncryptionRequest req) {
        return openUserDistrictService.districtCreate(req);
    }

    @ApiOperation("编辑区域")
    @PostMapping("/district/update")
    public RestResult<DistrictEncryptionResponse> districtUpdate(@RequestBody @Valid DistrictEncryptionRequest req) {
        return openUserDistrictService.districtUpdate(req);
    }

    @ApiOperation("删除指定区域")
    @PostMapping("/district/delete")
    public RestResult<DistrictEncryptionResponse> districtDelete(@RequestBody @Valid DistrictEncryptionRequest req) {
        return openUserDistrictService.districtDelete(req);
    }

    @ApiOperation("查询区域")
    @PostMapping("/district/query")
    public RestResult<DistrictEncryptionResponse> districtQuery(@RequestBody @Valid DistrictEncryptionRequest req) {
        return openUserDistrictService.districtQuery(req);
    }

    @ApiOperation("websocket测试")
    @PostMapping("/websocket/test")
    public RestResult websockettest(@RequestBody SendMsg2ServerRequest req) {
        otaWebSocketClient.send(JSONObject.toJSONString(req));
        return RestResult.ok(Boolean.TRUE);
    }


}
