package com.yx.face.controller.api;


import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.school.dto.UserWhitelistSchoolCallBackDTO;
import com.yx.face.model.school.dto.UserWhitelistSchoolDTO;
import com.yx.face.model.school.vo.UserWhitelistSchoolDetailVO;
import com.yx.face.service.UserWhitelistSchoolService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 学校场景入库 前端控制器
 * </p>
 *
 * @author wxg
 * @since 2023-09-07
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "E 学校场景（小程序） API")
@RestController
@RequestMapping("api/whitelistSchool")
@ApiSupport(order = 60)
@Validated
public class UserWhitelistSchoolController {

    private final UserWhitelistSchoolService userWhitelistSchoolService;

    @ApiOperation("获取刷脸用户列表")
    @GetMapping("/getUserWhitelist")
    @ApiOperationSupport(order = 10)
    public RestResult<List<UserWhitelistSchoolDetailVO>> getUserWhitelist(@RequestParam(value = "userId") Long userId) {
        return RestResult.ok(userWhitelistSchoolService.getUserWhitelist(userId));
    }

    @ApiOperation("通过身份证号获取用户信息")
    @GetMapping("/getUserWhitelistByIdNumber")
    @ApiOperationSupport(order = 20)
    public RestResult<List<UserWhitelistSchoolDetailVO>> getUserWhitelistByIdNumber(@RequestParam(value = "idNumber") String idNumber,@RequestParam(value = "personnelType") Integer personnelType) {
        return RestResult.ok(userWhitelistSchoolService.getUserWhitelistByIdNumber(idNumber,personnelType));
    }
    @ApiOperation("获取用户入库参数")
    @PostMapping("/userInsertData")
    @ApiOperationSupport(order = 30)
    public RestResult userInsertData(@RequestBody UserWhitelistSchoolDTO userWhitelistSchoolDTO) {
        return userWhitelistSchoolService.userInsertData(userWhitelistSchoolDTO);
    }
    @ApiOperation("用户点击确定入库调用")
    @PostMapping("/userInsertCallback")
    @ApiOperationSupport(order = 40)
    public RestResult<Boolean> userInsertCallback(@RequestBody UserWhitelistSchoolCallBackDTO dto) {
        return userWhitelistSchoolService.userInsertCallback(dto);
    }
    @ApiOperation("关闭刷脸")
    @PostMapping("/closeFace")
    @ApiOperationSupport(order = 50)
    public RestResult<Boolean> closeFace(@RequestBody UserWhitelistSchoolCallBackDTO dto) {
        return userWhitelistSchoolService.closeFace(dto);
    }
}
