package com.yx.face.controller.disinfect.api;


import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.api.building.dto.OpenIdSetDTO;
import com.yx.face.model.dto.disinfect.api.*;
import com.yx.face.model.enums.WxInfoEnum;
import com.yx.face.model.vo.disinfect.DisinfectAreaVO;
import com.yx.face.model.vo.disinfect.DisinfectRecordVO;
import com.yx.face.model.vo.disinfect.api.AdminShowCheckVO;
import com.yx.face.model.vo.disinfect.api.DisinfectScanQrcodeVO;
import com.yx.face.service.UserInfoService;
import com.yx.face.service.disinfect.ApiDisinfectService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author wxg
 * @since 2022-11-11
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "E 小程序-区域消杀 API")
@RestController
@RequestMapping("api/disinfect")
public class ApiDisinfectController {

    private final UserInfoService userInfoService;
    private final ApiDisinfectService apiDisinfectService;

    @ApiOperation("账号判定回显")
    @PostMapping("/adminShowCheck")
    @ApiOperationSupport(order = 1)
    public RestResult<AdminShowCheckVO> adminShowCheck(@RequestBody DisinfectApiBaseDTO disinfectApiBaseDTO) {
        return apiDisinfectService.adminShowCheck(disinfectApiBaseDTO);
    }
    @ApiOperation("消杀登记-openId填入")
    @PostMapping("/disinfectSetOpenId")
    @ApiOperationSupport(order = 4)
    public RestResult<Boolean> disinfectSetOpenId(@RequestBody OpenIdSetDTO dto) {
        dto.setType(WxInfoEnum.ZHEJIANG.getCode());
        return userInfoService.openIdSet(dto);
    }
    @ApiOperation("消杀登记-扫码")
    @PostMapping("/disinfectScanQrcode")
    @ApiOperationSupport(order = 5)
    public RestResult<DisinfectScanQrcodeVO> disinfectScanQrcode(@RequestBody DisinfectScanQrcodeDTO disinfectScanQrcodeDTO) {
        return apiDisinfectService.disinfectScanQrcode(disinfectScanQrcodeDTO);
    }
    @ApiOperation("消杀登记-上报")
    @PostMapping("/disinfectCommit")
    @ApiOperationSupport(order = 10)
    public RestResult<Boolean> disinfectCommit(@RequestBody DisinfectCommitDTO disinfectCommitDTO) {
        return apiDisinfectService.disinfectCommit(disinfectCommitDTO);
    }

    @ApiOperation("消杀记录-消杀区域列表")
    @PostMapping("/disinfectList")
    @ApiOperationSupport(order = 15)
    public RestResult<List<DisinfectAreaVO>> disinfectPageList(@RequestBody DisinfectPageListDTO disinfectPageListDTO) {
        return apiDisinfectService.disinfectPageList(disinfectPageListDTO);
    }

    @ApiOperation("消杀记录-消杀区域今日消杀")
    @PostMapping("/disinfectTodayList")
    @ApiOperationSupport(order = 20)
    public RestResult<List<DisinfectRecordVO>> disinfectTodayList(@RequestBody DisinfectTodayListDTO disinfectTodayListDTO) {
        return apiDisinfectService.disinfectTodayList(disinfectTodayListDTO);
    }
    @ApiOperation("消杀记录-消杀区域提醒消杀")
    @PostMapping("/disinfectRemind")
    @ApiOperationSupport(order = 25)
    public RestResult<String> disinfectRemind(@RequestBody DisinfectScanQrcodeDTO disinfectScanQrcodeDTO) {
        return apiDisinfectService.disinfectRemind(disinfectScanQrcodeDTO);
    }
    @ApiOperation("消杀记录-消杀区域编辑")
    @PostMapping("/disinfectEdit")
    @ApiOperationSupport(order = 30)
    public RestResult<Boolean> disinfectEdit(@RequestBody DisinfectEditDTO disinfectEditDTO) {
        return apiDisinfectService.disinfectEdit(disinfectEditDTO);
    }
}
