package com.yx.face.controller.disinfect.backstage;


import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.component.security.Authority;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.disinfect.DisinfectAreaDTO;
import com.yx.face.model.search.DisinfectAreaSearch;
import com.yx.face.model.vo.disinfect.DisinfectAreaVO;
import com.yx.face.service.disinfect.DisinfectAreaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author wxg
 * @since 2022-11-11
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "E 后台-区域消杀 API")
@RestController
@RequestMapping("admin/disinfectArea")
public class DisinfectAreaController {
    private final DisinfectAreaService disinfectAreaService;

    @ApiOperation("消杀区域-列表分页")
    @PostMapping("/getPageList")
    @ApiOperationSupport(order = 1)
    @Authority(value = {5})
    public RestResult<PageInfo<DisinfectAreaVO>> getPageList(@RequestBody RestDTO<DisinfectAreaSearch> dto) {
        reBuildQuery(dto);
        PageInfo<DisinfectAreaVO> pageList = disinfectAreaService.getPageList(dto);
        if (pageList != null) {
            return RestResponse.ok(pageList);
        }
        return RestResponse.error("获取消杀区域列表分页失败");
    }
    private void reBuildQuery(@RequestBody RestDTO<DisinfectAreaSearch> dto) {
        DisinfectAreaSearch object = dto.getData();
        if (object == null) {
            object = new DisinfectAreaSearch();
        }
        dto.setData(object);
    }

    @ApiOperation("消杀区域-新增/修改")
    @PostMapping("edit")
    @ApiOperationSupport(order = 2)
    @Authority(value = {5})
    public RestResult<Boolean> edit(@RequestBody DisinfectAreaDTO disinfectAreaDTO) {
        Integer result = disinfectAreaService.edit(disinfectAreaDTO);
        if (result == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }

    @ApiOperation("消杀区域-删除")
    @GetMapping("delete")
    @ApiOperationSupport(order = 3)
    @Authority(value = {5})
    public RestResult<Boolean> delete(@RequestParam("disinfectAreaId") Integer disinfectAreaId) {
        Integer insert = disinfectAreaService.delete(disinfectAreaId);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("删除失败");
    }
    @ApiOperation("消杀区域-列表")
    @GetMapping("getList")
    @ApiOperationSupport(order = 4)
    @Authority(value = {5})
    public RestResult<List<DisinfectAreaVO>> getList() {
        return disinfectAreaService.getList();
    }
}
