package com.yx.face.controller.disinfect.backstage;


import com.alibaba.excel.EasyExcel;
import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.yx.face.boot.component.security.Authority;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.ExcelListenerUtils;
import com.yx.face.model.dto.disinfect.DisinfectUserDTO;
import com.yx.face.model.excel.DisinfectUserExcel;
import com.yx.face.model.search.DisinfectUserSearch;
import com.yx.face.model.vo.disinfect.DisinfectUserVO;
import com.yx.face.service.disinfect.DisinfectUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * <p>
 * 消杀人员表 前端控制器
 * </p>
 *
 * @author wxg
 * @since 2022-11-11
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "E 后台-区域消杀 API")
@RestController
@RequestMapping("admin/disinfectUser")
public class DisinfectUserController {

    private final DisinfectUserService disinfectUserService;

    @ApiOperation("消杀人员-列表分页")
    @PostMapping("/getPageList")
    @ApiOperationSupport(order = 11)
    @Authority(value = {5})
    public RestResult<PageInfo<DisinfectUserVO>> getPageList(@RequestBody RestDTO<DisinfectUserSearch> dto) {
        reBuildQuery(dto);
        PageInfo<DisinfectUserVO> pageList = disinfectUserService.getPageList(dto);
        if (pageList != null) {
            return RestResponse.ok(pageList);
        }
        return RestResponse.error("获取消杀人员列表分页失败");
    }
    private void reBuildQuery(@RequestBody RestDTO<DisinfectUserSearch> dto) {
        DisinfectUserSearch object = dto.getData();
        if (object == null) {
            object = new DisinfectUserSearch();
        }
        dto.setData(object);
    }

    @ApiOperation("消杀人员-新增/修改")
    @PostMapping("edit")
    @ApiOperationSupport(order = 12)
    @Authority(value = {5})
    public RestResult<Boolean> edit(@RequestBody DisinfectUserDTO disinfectUserDTO) {
        Integer result = disinfectUserService.edit(disinfectUserDTO);
        if (result == 1) return RestResponse.ok(true);
        else return RestResponse.error("操作失败");
    }

    @ApiOperation("消杀人员-删除")
    @GetMapping("delete")
    @ApiOperationSupport(order = 13)
    @Authority(value = {5})
    public RestResult<Boolean> delete(@RequestParam("disinfectUserId") Integer disinfectUserId) {
        Integer insert = disinfectUserService.delete(disinfectUserId);
        if (insert == 1) return RestResponse.ok(true);
        else return RestResponse.error("删除失败");
    }
    @ApiOperation("消杀人员的批量导入")
    @PostMapping("excelInput")
    @ApiOperationSupport(order = 14)
    @Authority(value = {5})
    public RestResult<List<String>> excelInput(@RequestParam("file") MultipartFile excelFile) {
        ExcelListenerUtils listener = new ExcelListenerUtils();  //ExcelListener
        try {
            EasyExcel.read(excelFile.getInputStream(), DisinfectUserExcel.class, listener).headRowNumber(2).sheet(0).doReadSync();  //ExcelModel 上面创建的实体类
        } catch (IOException e) {
            e.printStackTrace();
        }
        List<Object> datas = listener.getDatas(); //把拿到的数据处理到集合里去(根据所需自行修改)
        return disinfectUserService.excelInput(datas);
    }
}
