package com.yx.face.controller.handle;

import com.yx.face.boot.component.security.Authority;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.JsonUtils;
import com.yx.face.model.dto.ShortTermDTO;
import com.yx.face.model.vo.health.HealtCodeUserVO;
import com.yx.face.service.handle.HandleDeviceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.Map;

/**
 * @author lejingjing
 * @version 1.0
 * @Description:手持终端查询
 * @date 2022/11/22
 **/
@Api(tags = "手持终端 API")
@RestController
@RequestMapping("/handleDevice")
public class HandheldDeviceController {

    @Resource
    private HandleDeviceService handleDeviceService;

    @ApiOperation(value = "查询健康码接口")
    @Authority(required = false)
    @PostMapping("/queryHealthCode")
    public RestResult<HealtCodeUserVO> queryHealthCode(@RequestBody ShortTermDTO shortTermDTO){
      return RestResponse.ok(handleDeviceService.queryHealthCode(shortTermDTO));
    }

    @ApiOperation(value = "进出记录上报")
    @Authority(required = false)
    @PostMapping("/pushPassRecords")
    public RestResult<Map<String,Object>> pushPassRecords(HttpServletRequest request){
        return RestResponse.ok(handleDeviceService.pushPassRecords(getParam(request)));
    }

    @ApiOperation(value = "心跳上报")
    @Authority(required = false)
    @PostMapping("/heartBeat")
    public RestResult<Map<String,Object>> heartBeat(HttpServletRequest request){
        return RestResponse.ok(handleDeviceService.heartBeat(getParam(request)));
    }

    private Map<String, Object> getParam(HttpServletRequest request) {
        StringBuilder sb = new StringBuilder();
        try (InputStream is = request.getInputStream();
             BufferedInputStream bis = new BufferedInputStream(is)) {
            byte[] buffer = new byte[1024];
            int read;
            while ((read = bis.read(buffer)) != -1) {
                sb.append(new String(buffer, 0, read, StandardCharsets.UTF_8));
            }
        } catch (IOException e) {
            return Collections.EMPTY_MAP;
        }
        String paramStr = sb.toString();
        if (!StringUtils.hasText(paramStr)) {
            return Collections.EMPTY_MAP;
        }
        return JsonUtils.toMap(paramStr);
    }

}
