package com.yx.face.dao;

import com.yx.face.boot.aop.DeviceCacheEnum;
import com.yx.face.boot.aop.DeviceCacheSnAnnotation;
import com.yx.face.boot.component.tk.TKMapper;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.vo.FaceDeviceLessVO;
import com.yx.face.model.vo.FaceDeviceVO;
import com.yx.face.model.vo.WhiteVisitorCertificationVO;
import org.apache.ibatis.annotations.InsertProvider;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Update;
import org.apache.ibatis.annotations.UpdateProvider;
import tk.mybatis.mapper.provider.base.BaseInsertProvider;
import tk.mybatis.mapper.provider.base.BaseUpdateProvider;

import java.util.List;
import java.util.Map;

/**
 * @description: 人脸设备信息表(FaceDevice)表数据库访问层 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-06-04 14:25:45 <br>
 */
public interface FaceDeviceDao extends TKMapper<FaceDevice> {

    Integer getIdBySn(@Param("sn") String sn);

    Integer getAdminIdBySn(@Param("sn") String sn);

    FaceDevice getFaceDeviceBySn(@Param("sn") String sn);

    List<FaceDeviceVO> getFaceDevices(Map<String, Object> map);

    List<FaceDeviceLessVO> getSnList(@Param("adminId") Integer adminId);

    List<FaceDeviceVO> getFaceDeviceList(Map<String, Object> map);

    List<String> selectList(@Param("adminId") Integer adminId);

    List<String> selectListAll(@Param("adminId") Integer adminId);

    WhiteVisitorCertificationVO getFaceDeviceOne(@Param("sn") String sn);

    List<String> selectListNo(@Param("adminId") Integer adminId);

    Integer selectAdminId(@Param("sn") String sn);

    /**
     * @param type       类型 1 全量 2 账号 3 sn
     * @param sn         sn码
     * @param provinceId //leixng
     * @param adminId    账号id
     * @return
     */
    List<FaceDevice> selectSnList(@Param("type") Integer type, @Param("sn") String sn, @Param("provinceId") Integer provinceId, @Param("adminId") Integer adminId);

    /*查询所有贵州省车管所的设备*/
    List<String> selectGuiSnList();

    List<FaceDeviceVO> getBySnList(List<String> snList);

    List<String> getDeviceListSearch(@Param("pass") Integer pass, @Param("faceDeviceTypeId") Integer faceDeviceTypeId, @Param("thirdPartyId") Integer thirdPartyId);

    FaceDeviceLessVO getBySn(@Param("sn") String sn);

    /*获取私有区域的设备*/
    List<FaceDeviceLessVO> getSnListByUserWhitelistId(@Param("userWhitelistId") Integer userWhitelistId);

    /*将设备的区域类型都改为公有*/
    void updateAreaType(Integer adminId);

    List<FaceDevice> selectOfflineDeviceList(@Param("snArray") List<String> faceDeviceOne);

    List<FaceDevice> selectOnlineDeviceList(@Param("snArray") List<String> faceDeviceOne);

    List<String> selectOnlineDevceSNsByAdminId(@Param("adminId") Integer adminId);

    List<FaceDevice> getFaceDeviceByAdminId(@Param("adminId") Integer admin);

    List<FaceDevice> selectSimpleDeviceList(@Param("snArray") List<String> faceDeviceOne, @Param("faceTypeIds") List<Integer> faceTypeIds);

    List<FaceDevice> selectOnlineList(@Param("snArray") List<String> faceDeviceOne);

    void cancelAllFaceDeviceRegister(@Param("adminId") Integer adminId, @Param("adminPlaceInfoIds") List<Integer> adminPlaceInfoIds);

    List<FaceDevice> getDeviceListByAdminPlaceInfoId(@Param("adminId") Integer adminId, @Param("adminPlaceInfoId") Integer adminPlaceInfoId);

//    @UpdateProvider(type = BaseUpdateProvider.class, method = "dynamicSQL")
//    @DeviceCacheSnAnnotation(value = DeviceCacheEnum.sn_insert_and_update)
//    int updateByPrimaryKey(FaceDevice record);

    @UpdateProvider(type = BaseUpdateProvider.class, method = "dynamicSQL")
    @DeviceCacheSnAnnotation(value = DeviceCacheEnum.sn_insert_and_update)
    int updateByPrimaryKeySelective(FaceDevice record);

    @InsertProvider(type = BaseInsertProvider.class, method = "dynamicSQL")
    @DeviceCacheSnAnnotation(value = DeviceCacheEnum.sn_insert_and_update)
    int insertSelective(FaceDevice record);

    @Update("UPDATE face_device SET online = #{online} WHERE id = #{id}")
    void updateDeviceOnline(Integer id, int online);

}

