package com.yx.face.model.model.entity;

import lombok.Data;
import lombok.experimental.Accessors;

import javax.persistence.*;
import java.util.Date;

/**
 * @description: 天波人脸设备进出记录表(FaceLog)实体类 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-06-04 14:25:46 <br>
 */
@Data
@Accessors(chain = true)
@Table(name = "face_log")
public class FaceLog {

    // 通用
    public static final Integer NORMAL = 0;

    // 进门
    public static final Integer IN = 1;

    // 出门
    public static final Integer OUT = 2;

    /**
     * 天波人脸记录id
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    /**
     * 记录唯一序号
     */
    private Long recordid;

    /**
     * 0：离线（即开门提示逻辑由设备离线处理事后上报）
     * 1：在线（即开门提示逻辑由平台实时控制
     */
    private Integer action;

    /**
     * 设备SN
     */
    @Column(name = "device_sn")
    private String deviceSn;

    /**
     * 用户id
     */
    @Column(name = "user_id")
    private Long userId;

    /**
     * 用户姓名
     */
    private String name;

    /**
     * 手机号码
     */
    private String phone;

    /**
     * 照片
     */
    private String photo;

    /**
     * 10进制（当verifytype=2 时，表示身份证号码）
     */
    private String cardid;

    /**
     * 物理卡号（卡号2）
     */
    @Column(name = "cardIdEx")
    private String cardidex;

    /**
     * 0人脸
     * 1卡
     * 2 身份证
     * 3 二维码
     * 4 远程
     * 6 IC卡+人脸
     */
    @Column(name = "verify_type")
    private Integer verifyType;

    /**
     * 106 (管理员)
     * 105（操作员）
     * 104（普通员工 默认）
     * 103（访客）
     * 102（禁止）
     * 101 (黑名单）
     */
    private Integer rightno;

    /**
     * 考勤时间格式 yyyy-MM-dd HH:mm:ss
     */
    @Column(name = "face_time")
    private String faceTime;

    /**
     * 终端上设置的组织id
     */
    private String orgid;

    /**
     * 0 通用
     * 1进门
     * 2出门
     */
    @Column(name = "out_type")
    private Integer outType;

    /**
     * 身份证信息JSON串
     * name 姓名
     * sex 性别
     * nation民族
     * born出生年月
     * address地址
     * apartment签发机关
     * period有效期限
     * no身份证号
     * country国籍
     * cn_name中文名
     * Idcard_version证件版本
     * card_type身份证类型
     * photo 身份证照片
     */
    @Column(name = "id_card")
    private String idCard;

    /**
     * 体温测量度数
     */
    private String extend3;

    /**
     * 佩戴口罩
     * 0：未戴
     * 1：已戴
     * 2：未启用口罩检测/开启仅检测体温
     */
    private String extend1;

    /**
     * 人脸相似度分值
     */
    @Column(name = "verify_score")
    private String verifyScore;

    /**
     * 体温是否异常 0 正常 1 异常
     */
    @Column(name = "tw_status")
    private String twStatus;

    /**
     * 健康码状态00 绿码 01 黄码 10 红码
     */
    @Column(name = "jkm_status")
    private String jkmStatus;

    /**
     * .
     * 健康码状态 0 健康码 1 国康码
     */
    @Column(name = "jkm_type")
    private String jkmType;

    /**
     * 阴性 阳性
     */
    @Column(name = "hs_status")
    private String hsStatus;

    /**
     * 疫苗状态 0针 等等依次累加
     */
    @Column(name = "ym_status")
    private String ymStatus;

    /**
     * 行程信息
     */
    @Column(name = "xc_info")
    private String xcInfo;

    /**
     * admin id
     */
    @Column(name = "admin_id")
    private Integer adminId;

    /**
     * 健康码组装数据
     */
    private String extend5;

    /**
     * 创建时间
     */
    @Column(name = "create_time")
    private Date createTime;


    /**
     * 性别
     */
    private String gender;
    /**
     * 生日 出生日期
     */
    private String birthday;
    /** 是否有效 true-有效 false-无效*/
    private Boolean status;
    /** 原因*/
    /** @see com.yx.face.model.enums.FaceLogStatusEnum */
    private Integer reasonType;

}

