package com.yx.face.model.model.entity;

import com.yx.face.boot.handle.IntegerTypeHandler;
import com.yx.face.model.vo.ContextVO;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;
import tk.mybatis.mapper.annotation.ColumnType;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.List;


@Data
@Accessors(chain = true)
@Table(name = "user_visitor_list_detail")
public class UserVisitorListDetail implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @ApiModelProperty("访客id")
    @Column(name = "user_visitor_list_id")
    private Integer userVisitorListId;

    @ApiModelProperty("照片暂时没用")
    @Column(name = "img_base")
    private byte[] imgBase;

    @ApiModelProperty("来访单位")
    private String company;

    @ApiModelProperty("车牌号码，")
    private String carno;

    @ApiModelProperty("访问事由")
    private String reason;

    @ApiModelProperty("人数")
    @Column(name = "num_peoples")
    private String numPeoples;

    @ApiModelProperty("是否携带身份证")
    @Column(name = "has_idcard")
    private String hasIdCard;

    @ApiModelProperty("健康码")
    @Column(name = "health_code")
    private String healthCode;

    /**
     * 疫苗针数，可能是空字符串，或 “1”、“2”
     */
    @ApiModelProperty("疫苗针数")
    @Column(name = "vaccine_num")
    private String vaccineNum;

    /**
     * 体温
     */
    @ApiModelProperty("体温")
    private String temperature;

    /**
     * 受访人姓名
     */
    @ApiModelProperty("受访人姓名")
    @Column(name = "visitee_xm")
    private String visiteeXm;

    /**
     * 受访人部门
     */
    @ApiModelProperty("受访人部门")
    @Column(name = "visitee_bm")
    private String visiteeBm;

    /**
     * 受访人的单位名称
     */
    @ApiModelProperty("受访人的单位名称")
    @Column(name = "visitee_company")
    private String visiteeCompany;

    /**
     * 受访人单位的房间号
     */
    @ApiModelProperty("受访人单位的房间号")
    @Column(name = "visitee_room")
    private String visiteeRoom;

    /**
     * 派出所代码（新增） 如：330104502
     */
    @ApiModelProperty("派出所代码")
    private String pcsdm;

    /**
     * 派出所名称（新增），如：采荷派出所
     */
    @ApiModelProperty("派出所名称")
    private String pcsname;

    /**
     * 身份证（或公安网）相片，base64编码
     */
    @ApiModelProperty("身份证")
    private String xp;

    /**
     * 现场采集照片，base64编码
     */
    @ApiModelProperty("现场采集照片")
    @Column(name = "scene_xp")
    private String sceneXp = "";

    /**
     * 来访时间，格式 2019-04-03 12:00:00
     */
    @ApiModelProperty("来访时间")
    private String time;

    /**
     * 访客机编号
     */
    @ApiModelProperty("访客机编号")
    @Column(name = "machine_id")
    private String machineId;

    /**
     * 访客机所在的使用单位如：浙江杭州上城区A单位
     */
    @ApiModelProperty("访客机所在的使用单位如：浙江杭州上城区A单位")
    private String org;

    /**
     * 门向，如东门、西门等
     */
    @ApiModelProperty("门向，如东门、西门等")
    private String door;

    @ApiModelProperty("常客id")
    private Integer userWhitelistId;

    @ApiModelProperty("人员类型 0:访客 1:同事 2:亲属")
    private Integer personnelType;

    @ApiModelProperty("通道权限（设备sn）")
    private String sn;

    @ApiModelProperty("邀请方式 0:微信 1:短信")
    private Integer invitationWay;

    @ApiModelProperty("审核状态 0:待审核 1:已拒绝 2:已同意 3-已入场")
    private Integer reviewStatus;

    @ApiModelProperty("姓名")
    private String name;

    @ApiModelProperty("拜访开始时间")
    private Date startTime;

    @ApiModelProperty("拜访结束时间")
    private Date endTime;

    @ApiModelProperty("签离时间")
    private Date signOutTime;
    @ApiModelProperty("签离设备")
    private String signOutSn;
    @ApiModelProperty("签离地点")
    private String signOutAddress;
    @ApiModelProperty("受访人手机号")
    private String visiteePhone;
    /**
     * @see com.yx.face.model.enums.VisitingTypeEnum
     */
    @ApiModelProperty("来访记录类型")
    private Integer visitingType;

    @ApiModelProperty("访问人手机号")
    private String phone;

    @ApiModelProperty("访问人手机号")
    private String idNumber;

    @ApiModelProperty("用户Id")
    private Integer userId;

    @ApiModelProperty("访问人照片地址")
    private String photo;

    @ApiModelProperty("创建时间")
    @Column(name = "create_time")
    private Date createTime;

    @ApiModelProperty("账号id")
    private Integer adminId;

    @ApiModelProperty("是否签离 true-签离 false-未签离")
    private Boolean isSignOut;

    @ApiModelProperty(value = "自定义字段")
    @Column(name = "content")
    private String content;

    @ApiModelProperty(value = "自定义字段列表")
    private List<ContextVO> contents;

    @ApiModelProperty("健康码")
    private String jkm;
    @ApiModelProperty("健康码图片")
    private String jkmPhoto;
    @ApiModelProperty("核酸")
    private String hs;
    @ApiModelProperty("核酸检测时间")
    private String hsCheckTime;
    @ApiModelProperty("核酸结果时间")
    private String hsResultTime;
    @ApiModelProperty("核酸图片")
    private String hsPhoto;
    @ApiModelProperty("行程信息1 代表没有去过高风险地区 2 代表去过高风险地区 3 代表用户没有行程记录")
    private String xc;
    @ApiModelProperty("行程码图片")
    private String xcPhoto;
    @ApiModelProperty("获取行程码的手机号")
    private String xcPhone;
    @ApiModelProperty("备注信息")
    private String remark;
    @ApiModelProperty("入场时间")
    private Date inTime;
    @ApiModelProperty("通过照片")
    private String inPhoto;
    @ApiModelProperty("设备地点")
    private String snAddr;

    /**
     * 用于排序的时间差
     */
    @Transient
    private long sortTimeSub;

    @ApiModelProperty("楼层")
    private Integer floor;
    @ApiModelProperty("通用码码值")
    private String commonQrCode;
    @ApiModelProperty("梯控码码值")
    private String ladderQrCode;
    @ApiModelProperty("登记地点")
    private String registerAddress;

    @ApiModelProperty("通行区域区域列表")
    @ColumnType(typeHandler = IntegerTypeHandler.class, column = "zone_pass_json")
    private List<Integer> zoneDevicePassIdList;

}
