package com.yx.face.model.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.yx.face.model.dto.AdminInoutWarnConfigDTO;
import com.yx.face.model.entity.AdminPlaceInfo;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.vo.third.InterfAddrJsonVo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @description: 管理人员信息表(Admin)VO <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-06-04 14:25:43 <br>
 */

@ApiModel("管理人员信息表VO model")
@Data
public class AdminVO {

    @ApiModelProperty(value = "管理人员表主键id")
    private Integer adminId;

    private List<Integer> adminIds;

    private List<Integer> notInAdminIds;

    @ApiModelProperty(value = "呢称")
    private String name;

    @ApiModelProperty(value = "账号")
    private String username;

    @ApiModelProperty(value = "密码")
    private String password;

    @ApiModelProperty(value = "头像路径")
    private String avatar;

    @ApiModelProperty(value = "身份证号")
    private String idNumber;

    @ApiModelProperty(value = "管理员联系电话")
    private String phone;

    @ApiModelProperty(value = "管理员邮箱")
    private String email;
    private String zfbEmail;

    @ApiModelProperty(value = "女-0 | 男- 1| 未选中-2")
    private Byte gender;

    @ApiModelProperty(value = "管理员类型 1.管理员 2.省级总账号 3.省级 4.市级 5.区级")
    private Byte type;

    @ApiModelProperty(value = "token", hidden = true)
    private String token;

    @ApiModelProperty(value = "数据创建时间")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    @ApiModelProperty(value = "省份地址编码")
    private Integer provinceId;

    @ApiModelProperty(value = "省份")
    private String provinceName;

    @ApiModelProperty(value = "城市地址编码")
    private Integer cityId;

    @ApiModelProperty(value = "城市")
    private String cityName;

    @ApiModelProperty(value = "县区地址编码")
    private Integer areaId;

    @ApiModelProperty(value = "县区")
    private String areaName;

    @ApiModelProperty(value = "街道编码")
    private Integer streetId;

    @ApiModelProperty(value = "街道/乡/镇")
    private String streetName;

    @ApiModelProperty(value = "详细地址")
    private String address;

    @ApiModelProperty(value = "派出所编码")
    private String policeStationCode;

    @ApiModelProperty(value = "派出所名称")
    private String policeStationName;

    @ApiModelProperty(value = "所属部门id")
    private Integer tagId;

    @ApiModelProperty(value = "所属部门名称")
    private String tagName;

    @ApiModelProperty(value = "导出权限 1.可以 2.不可以")
    private Integer isOut;

    @ApiModelProperty(value = "是否脱敏重要信息 1.脱敏 2.不脱敏")
    private Integer isHideImportant;

    @ApiModelProperty("脱敏数据查询是否配置 0-未配置 1 -配置")
    private Integer sysQueryConfiguration;

    @ApiModelProperty("脱敏数据查询配置是否开启 0-未开启 1 -开启")
    private Integer sysQueryConfigurationStatus;

    @ApiModelProperty("1常客2访客")
    private List<String> whiteVisitorType;

    /***
     * 1常客2访客 字符串
     */
    private String wvTypes;

    @ApiModelProperty(value = "是否开启布控 1.开启  2.不开启 默认2")
    private Integer isControlEarlyWarning;

    @ApiModelProperty(value = "不需要脱敏的字段 1.姓名 2. 手机号 3.身份证")
    private List<String> noHideCloumn;

    private String noHideCloumnString;

    @ApiModelProperty(value = "是否开启下发 0(默认)不开启  1开启")
    private Integer isPush;

    @ApiModelProperty(value = "自定义字段")
    private List<ContextVO> addCustomList;

    @ApiModelProperty(value = "自定义字段")
    private String content;

    private String codeTypeStr;

    @ApiModelProperty(value = "二维码类型记录标识，0：通用码，1：常客码")
    private List<String> codeType;

    @ApiModelProperty(value = "场所类型，0：通用，1：预约 2：楼宇 3：学校")
    private Integer placeType = AdminPlaceTypeEnum.COMMON_PLACE.getCode();

    @ApiModelProperty(value = "审核状态 0:待审核 1:已拒绝 2:已同意")
    private Integer reviewStatus;

    @ApiModelProperty("二维码信息")
    private AdminDetailVO adminDetail;

    @ApiModelProperty(value = "租户id")
    private Integer tenantId;

    @ApiModelProperty("是否开启场所码，false：否，true: 是")
    private Boolean placeCodeStatus;

    @ApiModelProperty("注册到某个地址码")
    private String addressCode;

    @ApiModelProperty("场所码单位名称")
    private String unitName;

    @ApiModelProperty("场所码单位地址")
    private String unitAddress;

    @ApiModelProperty("场所码管理员健康码信息")
    private String managerHealthCode;

    @ApiModelProperty("是否开启私有区域 1-开启 2-不开启")
    private Integer isOpenPrivateArea;

    @ApiModelProperty("场所码信息")
    private List<AdminPlaceInfo> placeInfoList;

    @ApiModelProperty("核酸有效期时间，单位小时，0:不限")
    private Integer nucleicExpirationTime;

    @ApiModelProperty(value = "三码 1-健康码 2-核酸 3-行程码", hidden = true)
    private String threeCode;

    @ApiModelProperty("三码 1-健康码 2-核酸 3-行程码")
    private List<String> threeCodeList;

    @ApiModelProperty("核酸时长（单位小时）")
    private Integer hsHour;

    @ApiModelProperty("常客有效时长（单位/天）")
    private BigDecimal userWhitelistEffectiveTime;

    @ApiModelProperty("访客有效时长（单位/天）")
    private BigDecimal userVisiterEffectiveTime;

    @ApiModelProperty(value = "风险区", hidden = true)
    private String riskArea;

    @ApiModelProperty("风险区列表")
    private List<String> riskAreaList;

    @ApiModelProperty("是否开启物理卡号：1-开启 2-不开启")
    private Integer cardPhysics;

    @ApiModelProperty("实名认证年龄段 为空则表示都需要实名认证 格式：20-50")
    private String realAuthenticationAge;

    @ApiModelProperty("实名认证年龄段-开始年龄")
    private Integer realAuthenticationAgeStart;

    @ApiModelProperty("实名认证年龄段-结束年龄")
    private Integer realAuthenticationAgeEnd;

    @Deprecated //已弃用 目前为区域推送配置
    @ApiModelProperty("打卡上报字段配置项, 多个逗号分割 已弃用")
    private List<String> punchReportFieldsConfig;

    @Deprecated //已弃用 目前为区域推送配置
    @ApiModelProperty("打卡上报字段配置项, 多个逗号分割 已弃用")
    private String punchReportFieldsConfigStr;

    @ApiModelProperty("打卡方式 0 不启用 1 通用 2 钉钉")
    private String clockWayType;

    @Deprecated //已弃用 目前为区域推送配置
    @ApiModelProperty("打卡信息推送地址 已弃用")
    private String clockWayPushUrl;

    @ApiModelProperty("内宾不允许进入的年龄段 格式：20-50")
    private String notAllowAge;
    @ApiModelProperty("不允许进入的年龄段提示信息")
    private String notAllowAgeMsg;
    /**
     * 是否报警 true/false
     */
    @ApiModelProperty("是否报警 true/false")
    private Boolean isAlarm = false;

    @ApiModelProperty("不允许进入的年龄段-开始年龄")
    private Integer notAllowAgeStart;

    @ApiModelProperty("不允许进入的年龄段-结束年龄")
    private Integer notAllowAgeEnd;

    @ApiModelProperty(value = "钉钉应用的唯一标识key")
    private String dingtalkAppKey;

    @ApiModelProperty(value = "钉钉应用的密钥")
    private String dingtalkAppSecret;

    @ApiModelProperty(value = "没有行程信息是否开门 1-开门（默认）0-关门")
    private Boolean noXcOpenDoor;

    @ApiModelProperty(value = "24小时内核酸报告未出是否开门：1-开门（默认）0-关门")
    private Boolean noHsOpenDoor;

    @ApiModelProperty(value = "去过中高风险地区是否开门 1-开门（默认）0-关门")
    private Boolean riskXcOpenDoor;

    @ApiModelProperty(value = "不允许通行的身份证地区代码，用/分隔")
    private String noAllowAreaCode;

    @ApiModelProperty(value = "行程码需检测是否开门 1-开门（默认）0-关门")
    private Boolean xcNeedCheckOpenDoor;

    @ApiModelProperty(value = "屏保照片数组")
    private String screenPhotoStr;

    @ApiModelProperty(value = "首页屏幕图片")
    private String homeImgUrl;

    @ApiModelProperty(value = "屏保照片数组")
    private List<ScreenPhotoVO> screenShowPhotos;

    @ApiModelProperty(value = "开启问题上报 1:开启 0：不开启")
    private Integer problemReportFlag;

    @ApiModelProperty(value = "开启区域消杀 1:开启 0：不开启")
    private Integer disinfectFlag;

    @ApiModelProperty(value = "是否限制三天三检 1:是   0:否（默认")
    private Integer limitGather;

    @ApiModelProperty(value = "微信小程序登录方式 1-身份证 2-卡号认证")
    private Integer wxLoginType;

    @ApiModelProperty(value = "是否需要实名认证 1-需要 0-不需要")
    private Boolean isNeedRealAuthentication;
    @ApiModelProperty("企业账号的配置-小程序是否可被搜索选择 1-可以 0-不可以")
    private Integer isCanSearch;


    @ApiModelProperty(value = "是否开启访客零点清除功能 1-开启 0-关闭")
    private Boolean isOpenVisitorClear;

    @ApiModelProperty("自定义手机号输入标志位 1输入 2获取")
    private Integer mobileAutoInputFlag;

    /**
     * ------ 账户添加配置信息2023-02-06 ------
     **/
    @ApiModelProperty(value = "是否开启健康码功能 0:关闭 1:开启 ")
    private Integer isOpenHealthCode;

    @ApiModelProperty(value = "顶部标题 设备显示配置")
    private String topTitle;
    /**
     * ------ 账户添加配置信息2023-04-13 ------
     **/
    @ApiModelProperty("二维码是否显示 true/false")
    private Boolean showQrcode = false;
    @ApiModelProperty(value = "二维码的码值")
    private String qrcodeText;
    @ApiModelProperty(value = "广告图片的地址 ")
    private String advertise;

    @ApiModelProperty(value = "是否开启进出记录统计")
    private Boolean openInOutStatistics;

    @ApiModelProperty(value = "考试设备列表")
    private List<String> examDeviceList;

    @ApiModelProperty(value = "考试设备列表")
    private String examDevices;

    @ApiModelProperty(value = "是否开启考试 1-开启 0-不开启")
    private Boolean isOpenExam;

    @ApiModelProperty(value = "商户平台id ")
    private Long tenantPlatformId;

    /**
     * ------ 账户添加配置信息2023-04-25 ------
     **/
    @ApiModelProperty(value = "联动平台服务是否开启 true/false")
    private Boolean openThirdLinkPlatform;
    @ApiModelProperty(value = "成功-通行结果")
    private String subLineSuccess;
    @ApiModelProperty(value = "失败-通行结果")
    private String subLineFail;
    @ApiModelProperty(value = "左副框上层")
    private String mainLineLeft;
    @ApiModelProperty(value = "左副框下层")
    private String subLineLeft;
    @ApiModelProperty(value = "右副框上层")
    private String mainLineRight;
    @ApiModelProperty(value = "右副框下层")
    private String subLineRight;
    @ApiModelProperty(value = "成功-语音播报内容")
    private String openVoiceMsgSuccess;
    @ApiModelProperty(value = "失败-语音播报内容")
    private String openVoiceMsgFail;

    @Deprecated //已弃用
    @ApiModelProperty(value = "接口地址 已弃用")
    private String interfaceAddress;
    @ApiModelProperty(value = "三方API 接口地址 [{\"interfAddr\":\"三方接口地址\",\"zoneId\":\"所在区域ID\"},{\"interfAddr\":\"三方接口地址\",\"zoneId\":\"所在区域ID\"}]")
    private List<InterfAddrJsonVo> interfaceAddresses;
    @ApiModelProperty(value = "三方打卡信息推送 接口地址 [{\"interfAddr\":\"三方接口地址\",\"punchReportFields\":\"打卡上报字段, 多个逗号分割\",\"zoneId\":\"所在区域ID\"}]")
    private List<InterfAddrJsonVo> clockWayPushInterfaceAddresses;

    /**
     * 三方平台接口
     */
    private String appKey;
    /**
     * 三方平台接口
     * 权限校验的appSecret
     */
    private String appSecret;
    /**
     * 三方平台接口
     * 权限校验的privateKey 16位随机数 包含字母和数据
     */
    private String privateKey;

    @ApiModelProperty(value = "三方平台接口授权")
    private AdminAuthVO mAdminAuthVO;

    @ApiModelProperty(value = "用户区域管理接口授权")
    private AdminAuthVO userDistrictAdminAuthVO;

    @ApiModelProperty(value = "普通打卡接口授权")
    private AdminAuthVO clockWayAdminAuthVO;

    @ApiModelProperty(value = "人脸数据保留天数")
    private Integer faceLogHoldDays;

    @ApiModelProperty(value = "设备结果页类型 false:老版  true:新版")
    private Boolean latestResultPage;

    @ApiModelProperty(value = "支付宝小程序单机二维码标题")
    private String zfbMiniEthTitle;
    @ApiModelProperty(value = "支付宝小程序联机二维码标题")
    private String zfbMiniVguangTitle;


    @ApiModelProperty(value = "常客小程序是否需要实名 0-否 1-需要")
    private Boolean whitelistAppRealname;

    @ApiModelProperty(value = "常客小程序是否需要照片 0-否 1-需要")
    private Boolean whitelistAppPhoto;

    @ApiModelProperty(value = "常客的登录参数列表")
    private List<Integer> whitelistAppLoginList;

    @ApiModelProperty(value = "常客小程序是否需要手机号 0-否 1-需要")
    private Boolean whitelistAppPhone;

    @ApiModelProperty(value = "访客小程序是否需要实名 0-否 1-需要")
    private Boolean visitorAppRealname;

    @ApiModelProperty(value = "访客小程序是否需要照片 0-否 1-需要")
    private Boolean visitorAppPhoto;

    @ApiModelProperty(value = "访客的登录参数列表")
    private List<Integer> visitorAppLoginList;

    @ApiModelProperty(value = "访客小程序是否需要手机号 0-否 1-需要")
    private Boolean visitorAppPhone;

    @ApiModelProperty(value = "楼宇审核方式 1-常客审核 2-指定人员审核")
    private Integer buildingCheckType;

    @ApiModelProperty(value = "机构编码")
    private String orgOutId;

    @ApiModelProperty(value = "上级名称-显示使用")
    private String tenantEnterpriseName;

    @ApiModelProperty(value = "下级名称-显示使用")
    private String enterpriseName;

    @ApiModelProperty(value = "是否开启进出预警 false-不开启 true-开启")
    private Boolean isOpenInoutWarning;

    @ApiModelProperty(value = "预警配置")
    private AdminInoutWarnConfigDTO adminInoutWarnConfigDTO;

    @ApiModelProperty(value = "员工管理接口是否开启 false-不开启 true-开启")
    private Boolean userManageSwitch;
    @ApiModelProperty(value = "区域管理接口是否开启 false-不开启 true-开启")
    private Boolean zoneManageSwitch;
    @ApiModelProperty(value = "是否开启梯控 false-不开启 true-开启")
    private Boolean isOpenLadder;
}

