package com.yx.face.server;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.OapiAttendanceRecordUploadRequest;
import com.dingtalk.api.request.OapiGettokenRequest;
import com.dingtalk.api.request.OapiV2UserGetbymobileRequest;
import com.dingtalk.api.response.OapiAttendanceRecordUploadResponse;
import com.dingtalk.api.response.OapiGettokenResponse;
import com.dingtalk.api.response.OapiV2UserGetbymobileResponse;
import com.taobao.api.ApiException;
import com.yx.face.server.dto.DingTalkRecordUpload;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

/**
 * @author: lileilei
 * @date: 2022/6/30 13:37
 * @description: 钉钉开放平台服务
 */
@Component
@Slf4j
public class DingTalkServer {

    @Autowired
    private RestTemplate restTemplate;

    /**
     * 上传打卡记录
     * @param dingTalkRecordUpload
     * @return
     */
    public String attendanceRecordUpload(DingTalkRecordUpload dingTalkRecordUpload) {
        log.info("appkey:{},appscret:{}",dingTalkRecordUpload.getAppkey(),dingTalkRecordUpload.getAppsecret());
        String token = this.getToken(dingTalkRecordUpload.getAppkey(), dingTalkRecordUpload.getAppsecret());
        JSONObject tokenObject = JSON.parseObject(token);
        String accessToken = tokenObject.getString("access_token");
        String userInfo = this.getByMobile(accessToken, dingTalkRecordUpload.getMobile());
        JSONObject userObject = JSON.parseObject(userInfo);
        String userId = JSON.parseObject(userObject.get("result").toString()).getString("userid");
        return this.attendanceRecordUpload(accessToken, userId, dingTalkRecordUpload.getDeviceId(),
                dingTalkRecordUpload.getDeviceName(), dingTalkRecordUpload.getUserCheckTime());
    }

    public String getToken(String appkey, String appsecret) {
        DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/gettoken");
        OapiGettokenRequest request = new OapiGettokenRequest();
        request.setAppkey(appkey);
        request.setAppsecret(appsecret);
        request.setHttpMethod("GET");
        OapiGettokenResponse response = null;
        try {
            response = client.execute(request);
        } catch (ApiException e) {
            e.printStackTrace();
        }

        log.info("获取企业内部应用的access_token响应内容: {}", response.getBody());
        return response.getBody();
    }

    public String getByMobile(String accessToken, String mobile) {
        OapiV2UserGetbymobileResponse rsp = null;
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/getbymobile");
            OapiV2UserGetbymobileRequest req = new OapiV2UserGetbymobileRequest();
            req.setMobile(mobile);
            rsp = client.execute(req, accessToken);
        } catch (ApiException e) {
            e.printStackTrace();
        }

        log.info("根据手机号:{}查询用户响应内容: {}", mobile, rsp.getBody());
        return rsp.getBody();
    }

    public String attendanceRecordUpload(String accessToken, String userid, String deviceId, String deviceName, Long userCheckTime) {
        DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/attendance/record/upload");
        OapiAttendanceRecordUploadRequest req = new OapiAttendanceRecordUploadRequest();
        req.setUserid(userid);
        req.setDeviceName(deviceName);
        req.setDeviceId(deviceId);
        req.setUserCheckTime(userCheckTime);
        OapiAttendanceRecordUploadResponse rsp = null;
        try {
            rsp = client.execute(req, accessToken);
        } catch (ApiException e) {
            e.printStackTrace();
        }
        log.info("上传打卡记录响应内容：{}", rsp.getBody());
        return rsp.getBody();
    }
}
