package com.yx.face.server;

import com.alibaba.fastjson.JSON;
import com.yx.face.server.dto.AddPersonDTO;
import com.yx.face.server.dto.BindCompanyDTO;
import com.yx.face.server.dto.DelPersonDTO;
import com.yx.face.server.vo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/4/19 12:47
 * @description: 手持终端第三方服务
 */
@Component
@Slf4j
public class HandheldTerminalsServer {
    @Value("${handheld.terminals.addPersonUrl}")
    private String addPersonUrl;
    @Value("${handheld.terminals.delPersonUrl}")
    private String delPersonUrl;
    @Value("${handheld.terminals.bindCompanyUrl}")
    private String bindCompanyUrl;

    @Autowired
    private RestTemplate restTemplate;

    /**
     * 设备sn查询单位编号
     *
     * @param dto
     * @return
     */
    public ResultVO bindCompany(BindCompanyDTO dto) {
        //入参数据格式转换
        dto.setCondition(Arrays.stream(dto.getCondition().split(",")).map(s -> "\'" + s + "\'").collect(Collectors.joining(", ")));
        return JSON.parseObject(restTemplate.postForObject(bindCompanyUrl, dto, String.class), ResultVO.class);
    }

    /**
     * 插入或更新人员
     */
    public ResultVO addPerson(AddPersonDTO dto) {
//        log.info("手持终端更新人员下发，入参：{}", JSON.toJSONString(dto));
        log.info("手持终端更新人员下发开始");
        String result = restTemplate.postForObject(addPersonUrl, dto, String.class);
        log.info("手持终端更新人员下发，响应结果：{}", result);
        return JSON.parseObject(result, ResultVO.class);
    }

    /**
     * 删除人员
     * @param dto
     * @return
     */
    public ResultVO delPerson(DelPersonDTO dto) {
//        log.info("手持终端删除人员下发，入参：{}", JSON.toJSONString(dto));
        log.info("手持终端删除人员下发开始");
        String result = restTemplate.postForObject(delPersonUrl, dto, String.class);
        log.info("手持终端删除人员下发，响应结果：{}", result);
        return JSON.parseObject(result, ResultVO.class);
    }
}
