package com.yx.face.service;

import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.AdminDTO;
import com.yx.face.model.dto.EnterpriseAccountDTO;
import com.yx.face.model.entity.AdminPlaceInfo;
import com.yx.face.model.entity.RiskArea;
import com.yx.face.model.search.AdminListNewSearch;
import com.yx.face.model.search.AdminSearch;
import com.yx.face.model.update.AdminUpdate;
import com.yx.face.model.vo.AdminCodeVO;
import com.yx.face.model.vo.AdminVO;

import java.util.List;
import java.util.Map;


/**
 * @description: 管理人员信息表(Admin)表服务接口 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-05-21 13:37:05 <br>
 */

public interface AdminService {
    /**
     * @param name     <br>
     * @param password <br>
     * @return com.yixin.review.model.vo.AdminLoginVO <br>
     * @description: login <br>
     * @date: 2021/4/7 16:50 <br>
     * @author: PWB <br>
     */
    AdminVO login(String name, String password);


    /**
     * @return com.yixin.review.boot.restful.RestResult<java.lang.Boolean> <br>
     * @description: 后台管理退出登录 <br>
     * @date: 2021/4/10 14:23 <br>
     * @author: PWB <br>
     */
    Boolean logout();

    /**
     * @param userId <br>
     * @return com.yixin.review.model.vo.AdminLoginVO <br>
     * @description: 获取管理员信息 <br>
     * @date: 2021/4/10 14:23 <br>
     * @author: PWB <br>
     */
    AdminVO getInfo(Integer userId);

    /**
     * @param dto <br>
     * @return java.lang.Boolean <br>
     * @description: 添加/编辑管理员 <br>
     * @date: 2021/4/7 16:50 <br>
     * @author: PWB <br>
     */
    Boolean editAdmin(AdminDTO dto);


    /**
     * @param dto <br>
     * @return com.github.pagehelper.PageInfo<com.yixin.review.model.vo.AdminInfoVO> <br>
     * @description: 获取管理员列表 <br>
     * @date: 2021/4/7 16:50 <br>
     * @author: PWB <br>
     */
    PageInfo<AdminVO> getAdmins(RestDTO<AdminSearch> dto);


    /**
     * @param id <br>
     * @return java.lang.Boolean <br>
     * @description: 删除管理员 <br>
     * @date: 2021/4/7 16:50 <br>
     * @author: PWB <br>
     */
    Boolean deleteAdmin(Long id);


    /**
     * @param id 管理员id <br>
     * @return com.yixin.review.model.vo.AdminAuthVO <br>
     * @description: 获取管理员权限信息 <br>
     * @date: 2021/4/7 17:47 <br>
     * @author: PWB <br>
     */
    AdminVO getAuthInfo(Integer id);


    AdminVO getInfo();

    /*
    * 管理员批量导入
    * */
    RestResult<List<String>> uploadAdminInfoExcel(List<Object> datas);


    List<AdminVO> getAdminlist();

    Boolean updateAdmin(AdminUpdate adminUpdate);

    /*
    * 大屏获取少量的账号信息
    * */
    AdminVO showDataGetInfo(Integer userId);
    /*
     * 大屏获取少量的账号信息
     * */
    AdminVO showDataGetInfo();
    /*查询*/
    List<Long> getAdminListSearch(Map<String, Object> map);
    /*根据adminIdList查询到所有的adminVO*/
    List<AdminVO> getByAdminlist(List<Integer> list);

    /**
     * 根据admin查询基本信息
     * @param adminId
     * @return
     */
    AdminVO getAdminInfoById(Integer adminId);

    /**
     * 二维码类型记录标识
     * @return
     */
    List<AdminCodeVO> getCodeType();

    /**
     * 添加/编辑企业
     * @param dto
     * @return
     */
    Boolean saveEnterpriseAccount(EnterpriseAccountDTO dto);

    /**
     * 添加企业
     * @param dto
     * @return
     */
    Boolean addEnterpriseAccount(EnterpriseAccountDTO dto);

    /**
     * 修改企业
     * @param dto
     * @return
     */
    Boolean modifyEnterpriseAccount(EnterpriseAccountDTO dto);

    /**
     * 企业审核
     * @param id
     * @param reviewStatus
     * @return
     */
    Boolean audit(Integer id, Integer reviewStatus);

    /**
     * 通过账号id查询详情
     * @param id
     * @return
     */
    EnterpriseAccountDTO queryAdminById(Integer id);

    /**
     * 查询企业账号信息
     * @param userId
     * @return
     */
    List<EnterpriseAccountDTO> queryEnterpriseAccount(Integer userId);

    /**
     * 查询场所码信息
     * @param adminId
     * @return
     */
    List<AdminPlaceInfo> queryPlaceInfos(Integer adminId);
    /*
     * @创建者  WXG
     * @描述  之前的获取列表不满足现在的需求，不修改原有的情况下，所以新增一个
     * @时间  2022/7/11 10:23
     * @参数
     **/
    List<AdminVO> getAdminListNew(AdminListNewSearch adminListNewSearch);
    /*
     * @创建者  WXG
     * @描述  行政区域列表
     * @时间  2022/8/17 12:45
     * @参数
     **/
    List<RiskArea> riskAreaList(RiskArea riskArea);
    /*
     * @创建者  WXG
     * @描述  获取默认的有效期 返回格式yyyy-MM-dd HH:mm:ss
     * @时间  2022/8/17 13:12
     * @参数
     **/
    String getUserWhitelistEffectiveTime(Integer adminId);
}

