package com.yx.face.service;

import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.iot.AlipayAmpeSignOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayGroupIdOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayOpenIotvspUserwithimageEncryptionRequest;
import com.yx.face.model.dto.iot.AlipayOpenIotvspUserwithimageRequest;
import com.yx.face.model.dto.iot.AlipayOperateRecordCreateWithVidOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayUserDeleteEncryptionRequest;
import com.yx.face.model.dto.iot.AlipayUserDeleteOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayUserStateOpenApiRequest;
import com.yx.face.model.entity.UserVisitorListDetail;
import com.yx.face.model.enums.WXMessageTemplateEnum;

import java.util.List;

/**
 * @author shisl
 * @package com.yx.face.service
 * @class AlipayOpenApiService
 * @date 2023/5/22 上午11:00
 * @description
 */
public interface AlipayOpenApiService {

    /**
     * 解密
     *
     * @param encryptedResponse 加密的内容
     * @return
     */
    String decryptContent(String encryptedResponse);

    /**
     * 消息订阅
     *
     * @param url
     * @param phone
     * @param wxMessageTemplateEnum
     * @param p1
     * @param p2
     * @param p3
     * @param p4
     * @param p5
     */
    void subscribeAlipayMessage(String url, Long userId, WXMessageTemplateEnum wxMessageTemplateEnum, String p1, String p2, String p3, String p4, String p5);


    /**
     * 发送邀请短信
     *
     * @param userVisitorListDetail
     */
    void createAlipayUrlAndSendMsg(UserVisitorListDetail userVisitorListDetail);

    /**
     * 生成小程序码
     *
     * @param bizContent
     * @return 二维码链接
     */
    String createQrcode(String bizContent);

    /**
     * 通用常客二维码
     *
     * @param toJSONString
     * @return
     */
    String createQrcode4701(String toJSONString);

    /**
     * 查询封闭场景的机构用户库id
     *
     * @param req
     * @return
     */
    RestResult queryOrgGroupId(AlipayGroupIdOpenApiRequest req);


    /**
     * 机构用户出库
     *
     * @param req
     * @return
     */
    void deleteUser(AlipayUserDeleteOpenApiRequest req);

    /**
     * 查询机构用户在库结果
     *
     * @param request
     * @return
     */
    RestResult queryUserState(AlipayUserStateOpenApiRequest request);


    /**
     * 获取key8设备签名等信息
     *
     * @param openApiRequest
     * @return
     */
    RestResult queryKey8Sign(AlipayAmpeSignOpenApiRequest openApiRequest);


    void deleteIotUser(List<Long> userIds);


    /**
     * 自定义vid录入机构用户信息获取操作流水id v2版本
     *
     * @param req
     * @return
     */
    RestResult createOperateRecordWithVidV2(AlipayOperateRecordCreateWithVidOpenApiRequest req);

    /**
     * 录入机构用户信息并直接入库（自定义vid）
     *
     * @param req
     * @return
     */
    RestResult createOperateRecordWithoutImage(AlipayOpenIotvspUserwithimageRequest req);

    /**
     * 录入机构用户信息并直接入库（自定义vid） 加密版 对外 苏州园林
     *
     * @param req
     * @return
     */
    RestResult createOperateRecordWithoutImageEncryption(AlipayOpenIotvspUserwithimageEncryptionRequest req);

    /**
     * 用户出库（加密版） 对外 苏州园林
     *
     * @param req
     */
    RestResult deleteUserEncryption(AlipayUserDeleteEncryptionRequest req);
}
