package com.yx.face.service;

import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.model.dto.FaceDeviceDTO;
import com.yx.face.model.dto.SaveFaceLogDto;
import com.yx.face.model.entity.*;
import com.yx.face.model.excel.FaceLogVoExcel;
import com.yx.face.model.query.FaceServerLogQu;
import com.yx.face.model.query.QueryUser;
import com.yx.face.model.search.FaceDeviceSearch;
import com.yx.face.model.search.FaceServerLogSearch;
import com.yx.face.model.vo.*;
import com.yx.face.model.vo.visitor.UserVisitorListDetailInfo;

import java.io.InputStream;
import java.util.List;

public interface FaceService {

    Boolean authFaceDevice(Integer deviceId);

    Boolean stopFaceDevice(Integer deviceId);

    Boolean addFaceDevice(FaceDeviceDTO deviceDTO);

    Boolean modifyFaceDevice(FaceDeviceDTO deviceDTO);

    Boolean deleteDevice(Integer deviceId);

    PageInfo<FaceDeviceVO> getFaceDevices(RestDTO<FaceDeviceSearch> dto);

    PageInfo<FacePassVO> getFacePasses(RestDTO<QueryUser> dto);

    PageInfo<FaceLogVO> getFaceLogs(RestDTO<FaceServerLogSearch> dto);

    UserInfoIdentificationVO queryFaceSuccess(String taskId);

    List<FaceLogVoExcel> getFaceLogsExcel(FaceServerLogSearch dto, List<FaceLogVO> faceLogVOList);

    List<FaceDeviceVO> getFaceDeviceList();

    List<FaceDeviceVO> getFaceDeviceListByAdminId(Admin admin);

    /**
     * 设备图片上传
     *
     * @param file
     * @param adminId
     * @return
     */
    String addFaceDeviceLogImg(InputStream file, Integer adminId);

    Boolean issuedAll(Integer type, Integer adminId, String sn);

    String getLogo(Integer userId);

    /**
     * 设备重启
     *
     * @param sn
     * @return
     */
    Boolean doReboot(String sn);

    FaceLogVO getDetails(FaceServerLogQu id, Integer userId);

    /**
     * 接收第三方推送数据
     *
     * @param saveFaceLogDto
     * @return
     */
    Boolean saveFaceLog(SaveFaceLogDto saveFaceLogDto);

    List<ContextVO> getContext(Integer id);

    List<ContextVO> getContextById(Integer adminId);

    List<ContextVO> getContextByWhiteId(Integer id);

    Boolean getContextByAdminId(Integer adminId);

    Integer getUserVisitorList(Integer adminId, String idNumber);

    Boolean updateWhiteContent(Integer adminId, String idNumber, List<ContextVO> list);

    Boolean updateVisitorContent(Integer adminId, String idNumber, List<ContextVO> list);

    List<ContextVO> getContextByFaceLogVisitor(Integer adminId, String cardId);

    List<ContextVO> getContextByFaceLogWhite(Integer adminId, String cardId, FaceLogVO faceLogVO);

    FaceLogVO getFaceLog(Integer logId);

    UserInfoVO getUser(Integer id);

    UserVisitorListDetailInfo getWhiteListFrom(Integer id);

    UserVisitorListDetailInfo getVisitorListFrom(Integer id);

    UserVisitorListDetailInfo getUserListFrom(Integer adminId, String cardId, String time);

    UserVisitorListDetailInfo getUserListFromTwo(Integer adminId, String cardId, String time);

    List<ContextVO> getContextTwo(Integer id, Admin admin, UserVisitorListVO list);

    List<ContextVO> getContextByWhiteIdTwo(Integer id, Admin admin, UserWhitelistVO whitelistVO);

    /**
     * 判断填充用户类型信息
     *
     * @param userId
     * @param adminId
     */
    void modifyUserInfo(Integer userId, Integer adminId);

    Boolean clear();

    Boolean downFile(List<String> deviceSNs, List<String> screenPhotos, String fileName);

    /**
     * 获取商户平台列表
     *
     * @return
     */
    List<TenantVo> doGetTenants();

    /**
     * 推送数据到银川公安
     * @param pushVO
     * @return
     */
    void pushUserData2YinChuanGongAn(ClockWayPushVO pushVO);
}
