package com.yx.face.service;

import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.HealthCodeConfigDTO;
import com.yx.face.model.dto.OpenIssuedDTO;
import com.yx.face.model.dto.PlaceRecordDTO;
import com.yx.face.model.entity.AdminPlaceInfo;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.search.QueryExamUserWhitelistSearch;
import com.yx.face.model.vo.PlaceInfoVO;
import com.yx.face.model.vo.QueryExamUserWhitelistVO;
import com.yx.face.server.dto.TokenDataDTO;
import com.yx.face.server.vo.HealthResultVO;

import java.util.List;

/**
 * @author: lileilei
 * @date: 2022/6/4 19:52
 * @description:
 */
public interface OpenIssuedService {
    /**
     * 常访客人脸新增/删除下发
     *
     * @param openIssuedDTO
     * @return
     */
    RestResult issued(OpenIssuedDTO openIssuedDTO);

    /**
     * 删除人员
     *
     * @param openIssuedDTO
     * @return
     */
    RestResult delData(OpenIssuedDTO openIssuedDTO);

    /**
     * 新增常客人员
     *
     * @param openIssuedDTO
     * @return
     */
    RestResult addWhiteData(OpenIssuedDTO openIssuedDTO);

    /**
     * 新增访客人员
     *
     * @param openIssuedDTO
     * @return
     */
    RestResult addVisitorData(OpenIssuedDTO openIssuedDTO);

    /**
     * 通过设备sn查询场所码信息
     *
     * @param sn
     * @return
     */
    RestResult<PlaceInfoVO> queryPlaceInfoBySn(String sn);

    /**
     * 通过设备sn查询开放场所码配置信息
     *
     * @param sn
     * @return
     */
    RestResult<PlaceInfoVO> queryOpenPlaceInfoBySn(String sn);

    /**
     * 保存场所记录
     *
     * @param dto
     * @return
     */
    Boolean savePlaceRecord(PlaceRecordDTO dto);

    /**
     * 场所码token凭证
     *
     * @return
     */
    HealthResultVO<TokenDataDTO> getToken(Boolean cleanSession, Integer placeAreaCode);

    /**
     * 根据账号查询设备信息
     *
     * @param adminId
     * @return
     */
    List<FaceDevice> queryFaceDevices(Long adminId);

    /**
     * 场所码注册
     *
     * @param configDTO
     * @return
     */
    RestResult saveHealthCodeConfig(HealthCodeConfigDTO configDTO);

    /**
     * 设备注册
     *
     * @param faceDevice
     * @param adminPlaceInfo
     * @return
     */
    Boolean deviceRegister(FaceDevice faceDevice, AdminPlaceInfo adminPlaceInfo);
    /*
     * @创建者  WXG
     * @描述  根据设备号 和 身份证号 查询考试人员信息
     * @时间  2023/4/14 13:08
     * @参数
     **/
    QueryExamUserWhitelistVO getExamUserWhitelist(QueryExamUserWhitelistSearch dto);
}
