package com.yx.face.service;

import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.api.building.dto.OpenIdSetDTO;
import com.yx.face.model.api.common.dto.CardIdExSetDTO;
import com.yx.face.model.dto.*;
import com.yx.face.model.entity.UserInfo;
import com.yx.face.model.entity.UserWhitelist;
import com.yx.face.model.query.QueryUser;
import com.yx.face.model.vo.ConfirmFaceNotifyNewVO;
import com.yx.face.model.vo.ContextVO;
import com.yx.face.model.vo.UserInfoInsertVO;
import com.yx.face.model.vo.UserInfoVO;

import java.util.List;


/**
 * @description: 用户基础信息表(UserInfo)表服务接口 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-06-04 14:25:50 <br>
 */

public interface UserInfoService {

    String getWxPhone(WxInfoDTO dto);

    /**
     * 脸识别回调接口
     *
     * @param result 回调的相关用户信息 <br>
     * @description: 人脸识别回调接口 <br>
     * @date: 2021/4/2 18:28 <br>
     * @author: PWB <br>
     */
    Long confirmFaceNotify(ConfirmFaceNotifyDTO confirmFaceNotifyDTO);


    PageInfo<UserInfoVO> getUsers(RestDTO<QueryUser> dto);

    Long addOrUpdateUserInfo(UserWhitelist userWhitelist);

    Long checkingIdNumberAndPhone(String idNumber, String phone);
    /*实名认证方式：返回1-老 2-新*/
    Integer realAuthenticationType(RealAuthenticationTypeDTO realAuthenticationTypeDTO);

    List<UserInfoVO> getUserInfoByUserIds(List<Long> userIds);
  /*openId获取并填入*/
    RestResult<Boolean> openIdSet(OpenIdSetDTO openIdSetDTO);
    /*实名认证方式：新*/
    RestResult<ConfirmFaceNotifyNewVO> doConfirmFaceNotifyNew(ConfirmFaceNotifyNewDTO confirmFaceNotifyNewDTO);

    /**
     * 新增常/访客
     * @param dto
     * @return
     */
    UserInfoInsertVO userInfoInsert(UserInfoInsertDTO dto);

    RestResult<Boolean> cardIdExSet(CardIdExSetDTO dto);

    List<ContextVO> getContextByCardId(Integer userId, Integer adminId);
    UserInfo getUserInfo(String idNumber, String phone, String cardIdEx);
}

