package com.yx.face.service.component.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FaceDeviceTypeDao;
import com.yx.face.dao.FacePassDao;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.FaceDeviceTypeEnum;
import com.yx.face.service.component.RuiYingService;
import com.yx.face.service.feign.RemoteRYDeviceDownloadFaceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName RuiYingServiceImpl
 * @Author WXG
 * @Date 2022/12/21 13:34
 */
@Service
@Slf4j
public class RuiYingServiceImpl implements RuiYingService {
    @Autowired
    private RemoteRYDeviceDownloadFaceService remoteRYDeviceDownloadFaceService;
    @Resource
    private FaceDeviceTypeDao faceDeviceTypeDao;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Resource
    private FacePassDao facePassDao;

    @Async
    @Override
    public void delPerson(String sn) {
        FaceDevice faceDevice = faceDeviceDao.selectOne(new FaceDevice().setSn(sn).setEnable(1).setAuth(1));
        if (faceDevice == null) {
            return;
        }
        List<FacePass> passList = facePassDao.select(new FacePass().setDeviceId(faceDevice.getId()));
        if (CollectionUtil.isEmpty(passList)) {
            return;
        }
        List<Long> userIds = passList.stream().map(FacePass::getUserId).collect(Collectors.toList());
        this.delPerson(sn, faceDevice, userIds);
    }

    @Async
    @Override
    public void delPerson(String sn, List<Long> userIds) {
        this.delPerson(sn, null, userIds);
    }

    @Async
    @Override
    public void delPerson(String sn, FaceDevice faceDevice, List<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return;
        }
        if (faceDevice == null) {
            faceDevice = faceDeviceDao.selectOne(new FaceDevice().setSn(sn).setEnable(1).setAuth(1));
        }
        if (faceDevice == null) {
            return;
        }
        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectByPrimaryKey(faceDevice.getFaceDeviceTypeId());
        if (!FaceDeviceTypeEnum.FOUR_INCH_SCREEN.getCode().equals(faceDeviceType.getCode())) {
            return;
        }
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(userIds.size());
        Map<String, Object> dataItem;
        for (Long userId : userIds) {
            dataItem = Maps.newHashMapWithExpectedSize(1);
            dataItem.put("pin", userId);
            data.add(dataItem);
        }
        params.put("Data", data);
        params.put("SN", sn);
        Map<String, Object> result = remoteRYDeviceDownloadFaceService.delUser(params);
        log.info("锐颖删除人脸返回结果：{}", JSON.toJSONString(result));
    }
}
