package com.yx.face.service.disinfect.impl;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.yx.face.boot.component.wx.WxMaConfiguration;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.dao.disinfect.DisinfectAreaDao;
import com.yx.face.dao.disinfect.DisinfectRecordDao;
import com.yx.face.dao.disinfect.DisinfectUserDao;
import com.yx.face.model.dto.disinfect.api.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.DisinfectAuthEnum;
import com.yx.face.model.enums.WXMessageTemplateEnum;
import com.yx.face.model.enums.WxInfoEnum;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.disinfect.DisinfectAreaTimeVO;
import com.yx.face.model.vo.disinfect.DisinfectAreaVO;
import com.yx.face.model.vo.disinfect.DisinfectRecordVO;
import com.yx.face.model.vo.disinfect.api.AdminShowCheckVO;
import com.yx.face.model.vo.disinfect.api.DisinfectScanQrcodeVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.disinfect.ApiDisinfectService;
import com.yx.face.service.disinfect.DisinfectAreaService;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName DisinfectApiServiceImpl
 * @Author WXG
 * @Date 2022/11/15 14:52
 */
@Service
@Slf4j
public class ApiDisinfectServiceImpl implements ApiDisinfectService {
    @Resource
    private AdminService adminService;
    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private DisinfectUserDao disinfectUserDao;
    @Resource
    private DisinfectAreaDao disinfectAreaDao;
    @Resource
    private DisinfectRecordDao disinfectRecordDao;
    @Resource
    private DisinfectAreaService disinfectAreaService;
    @Value("${weixin.appid}")
    private String appid;
    @Value("${weixin.miniprogramState}")
    private String miniprogramState;
    @Override
    public RestResult<AdminShowCheckVO> adminShowCheck(DisinfectApiBaseDTO disinfectApiBaseDTO) {
        AdminShowCheckVO adminShowCheckVO = new AdminShowCheckVO();
        if(disinfectApiBaseDTO.getAdminId() == null){
            return RestResponse.ok(adminShowCheckVO);
        }
        AdminVO info = adminService.getInfo(disinfectApiBaseDTO.getAdminId());
        if(info == null){
            return RestResponse.ok(adminShowCheckVO);
        }
        if(info.getProblemReportFlag().equals(1)){
            adminShowCheckVO.setShowProblemReport(Boolean.TRUE);
        }
        if(info.getDisinfectFlag().equals(0)){
            adminShowCheckVO.setShowDisinfect(Boolean.FALSE);
            return RestResponse.ok(adminShowCheckVO);
        }

        if(disinfectApiBaseDTO.getUserId() == null){
            return RestResponse.ok(adminShowCheckVO);
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(disinfectApiBaseDTO.getUserId());
        if(userInfo == null){
            return RestResponse.ok(adminShowCheckVO);
        }
        try {
            DisinfectUser disinfectUser = this.getDisinfectUser(userInfo.getIdNumber(), disinfectApiBaseDTO.getAdminId());
            adminShowCheckVO.setShowDisinfect(Boolean.TRUE);
            if(StrUtil.isNotEmpty(disinfectUser.getDisinfectAuth()) && disinfectUser.getDisinfectAuth().contains(DisinfectAuthEnum.CHECK.getCode()+"")){
                adminShowCheckVO.setCheckAuth(Boolean.TRUE);
            }
        }catch (Exception e){
        }
        return RestResponse.ok(adminShowCheckVO);
    }

    @Override
    public RestResult<DisinfectScanQrcodeVO> disinfectScanQrcode(DisinfectScanQrcodeDTO disinfectScanQrcodeDTO) {
        //账号权限判定
        AdminVO adminVO = this.adminCheck(disinfectScanQrcodeDTO);
        //用户信息查询
        UserInfo userInfo = this.userInfoCheck(disinfectScanQrcodeDTO);
        //消杀人员查询
        DisinfectUser disinfectUser = this.getDisinfectUser(userInfo.getIdNumber(), adminVO.getAdminId());
        if(!disinfectUser.getDisinfectAuth().contains(DisinfectAuthEnum.DISINFECT.getCode().toString())) {
            throw new ServiceException("您没有消杀的权限，请联系管理员!");
        }
        //消杀区域
        DisinfectArea disinfectArea = this.disinfectAreaCheck(disinfectScanQrcodeDTO.getDisinfectAreaId(), adminVO.getAdminId());
        //消杀人员是否可以消杀该区域
        this.disinfectAuthCheck(disinfectUser,disinfectArea);
        //消杀记录
        List<DisinfectRecord> disinfectRecordList = disinfectRecordDao.selectByDisinfectAreaIdList(CollUtil.toList(disinfectArea.getId()));
        Map<String, DisinfectRecord> disinfectRecordMap = disinfectRecordList.stream().collect(Collectors.toMap(DisinfectRecord::getDisinfectTimeBreakId, Function.identity()));
        //数据组装
        DisinfectScanQrcodeVO disinfectScanQrcodeVO = BeanUtil.copyProperties(disinfectArea,DisinfectScanQrcodeVO.class);
        List<DisinfectAreaTimeVO> disinfectAreaTimeVOList = JSON.parseArray(disinfectArea.getDisinfectTimeBreak(), DisinfectAreaTimeVO.class);
        Integer todayDisinfectNum = 0;

        //将区域中的时间端 的消杀情况填入
        if(CollUtil.isNotEmpty(disinfectAreaTimeVOList)){
            for (DisinfectAreaTimeVO disinfectAreaTimeVO : disinfectAreaTimeVOList) {
                disinfectAreaTimeVO.setTimeBreak(new StringBuilder().append(DateUtil.format(disinfectAreaTimeVO.getStartTime(), "HH:mm")).append("-").append(DateUtil.format(disinfectAreaTimeVO.getEndTime(), "HH:mm")).toString());
                if(disinfectRecordMap.containsKey(disinfectAreaTimeVO.getDisinfectAreaTimeId())){
                    DisinfectRecord disinfectRecord = disinfectRecordMap.get(disinfectAreaTimeVO.getDisinfectAreaTimeId());
                    disinfectAreaTimeVO.setDisinfectAreaTimeType(disinfectRecord.getStatus());
                    todayDisinfectNum++;
                }
            }
        }

        StringBuilder waitDisinfect = this.toString(disinfectAreaTimeVOList);
        disinfectScanQrcodeVO.setTodayTotalDisinfectNum(disinfectAreaTimeVOList.size());
        disinfectScanQrcodeVO.setDisinfectAreaId(disinfectArea.getId());
        disinfectScanQrcodeVO.setTodayDisinfectNum(todayDisinfectNum);
        disinfectScanQrcodeVO.setDisinfectAreaTimeVOList(disinfectAreaTimeVOList);
        disinfectScanQrcodeVO.setWaitDisinfect(waitDisinfect.length() == 0? "待消杀:无":waitDisinfect.toString());
        disinfectScanQrcodeVO.setWaitDisinfectPipei(waitDisinfect.toString().replace("待消杀:",""));
        return RestResponse.ok(disinfectScanQrcodeVO);
    }

    private StringBuilder toString(List<DisinfectAreaTimeVO> disinfectAreaTimeVOList){
        StringBuilder waitDisinfect = new StringBuilder();
        List<DisinfectAreaTimeVO> unFinishDisinfectAreaTimeVOList = disinfectAreaTimeVOList.stream().filter(p -> p.getDisinfectAreaTimeType() == 3).collect(Collectors.toList());
        long time;
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
        try {
            time = sdf.parse(sdf.format(new Date())).getTime();
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        for (DisinfectAreaTimeVO disinfectAreaTimeVO : unFinishDisinfectAreaTimeVOList) {
            long startTime = disinfectAreaTimeVO.getStartTime().getTime();
            long endTime = disinfectAreaTimeVO.getEndTime().getTime();
            if(startTime > time || time < endTime){
                waitDisinfect.append("待消杀:").append(DateUtil.format(disinfectAreaTimeVO.getStartTime(), "HH:mm")).append("-").append(DateUtil.format(disinfectAreaTimeVO.getEndTime(), "HH:mm"));
                break;
            }
        }
        return waitDisinfect;
    }

    @Override
    public RestResult<Boolean> disinfectCommit(DisinfectCommitDTO disinfectCommitDTO) {
        //账号权限判定
        AdminVO adminVO = this.adminCheck(disinfectCommitDTO);
        //用户信息查询
        UserInfo userInfo = this.userInfoCheck(disinfectCommitDTO);
        //消杀人员查询
        DisinfectUser disinfectUser = this.disinfectUserCheck(userInfo.getIdNumber(), adminVO.getAdminId());
        //消杀区域
        DisinfectArea disinfectArea = this.disinfectAreaCheck(disinfectCommitDTO.getDisinfectAreaId(), adminVO.getAdminId());
        //数据填入
        DisinfectRecord disinfectRecord = BeanUtil.copyProperties(userInfo,DisinfectRecord.class);
        BeanUtil.copyProperties(disinfectArea,disinfectRecord);
        BeanUtil.copyProperties(disinfectCommitDTO,disinfectRecord);
        disinfectRecord.setName(userInfo.getUsername());
        Date now = new Date();
        String todayDate = DateUtil.format(now, "yyyy-MM-dd");
        disinfectRecord.setCreateTime(now);
        //消杀时间段
        List<DisinfectAreaTimeVO> disinfectAreaTimeVOList = JSON.parseArray(disinfectArea.getDisinfectTimeBreak(), DisinfectAreaTimeVO.class);
        Map<String, DisinfectAreaTimeVO> disinfectAreaTimeVOMap = disinfectAreaTimeVOList.stream().collect(Collectors.toMap(DisinfectAreaTimeVO::getDisinfectAreaTimeId, Function.identity()));
        if(StrUtil.isNotEmpty(disinfectCommitDTO.getDisinfectTimeBreakId()) && disinfectAreaTimeVOMap.containsKey(disinfectCommitDTO.getDisinfectTimeBreakId())){
            DisinfectAreaTimeVO disinfectAreaTimeVO = disinfectAreaTimeVOMap.get(disinfectCommitDTO.getDisinfectTimeBreakId());
            disinfectRecord.setDisinfectTimeDate(todayDate);
            disinfectRecord.setDisinfectTimeStart(DateUtil.format(disinfectAreaTimeVO.getStartTime(),"HH:mm"));
            disinfectRecord.setDisinfectTimeEnd(DateUtil.format(disinfectAreaTimeVO.getEndTime(),"HH:mm"));
        }else {
            return RestResponse.error("请正确选择时间段");
        }
        //消杀记录查询
        List<DisinfectRecord> disinfectRecordList = disinfectRecordDao.selectTodayIsFinishDisinfect(disinfectCommitDTO.getDisinfectAreaId(),disinfectCommitDTO.getDisinfectTimeBreakId());
        if(CollUtil.isNotEmpty(disinfectRecordList)){
            return RestResponse.error("今日该时段已有消杀记录，不可更改");
        }
        disinfectRecord.setId(null);
        disinfectRecordDao.insertSelective(disinfectRecord);
        return RestResponse.ok(true);
    }

    @Override
    public RestResult<List<DisinfectAreaVO>> disinfectPageList(DisinfectPageListDTO disinfectPageListDTO) {
        if(disinfectPageListDTO == null){
            throw new ServiceException("请输入基础数据");
        }
        //账号权限判定
        AdminVO adminVO = this.adminCheck(disinfectPageListDTO);
        //用户信息查询
        UserInfo userInfo = this.userInfoCheck(disinfectPageListDTO);
        //消杀人员信息查询
        DisinfectUser disinfectUser = this.getDisinfectUser(userInfo.getIdNumber(), adminVO.getAdminId());
        //用户消杀区域查询
        List<Integer> disinfectAreaIdList = JSON.parseArray(disinfectUser.getDisinfectAreaIdList(), Integer.class);
        if(CollUtil.isEmpty(disinfectAreaIdList)){
            return RestResponse.ok(new ArrayList<>());
        }
        //查询该人员负责的区域
        List<DisinfectArea> disinfectAreaList = disinfectAreaDao.selectByIdListAndAreaName(disinfectAreaIdList,disinfectPageListDTO.getDisinfectAreaName());
        List<DisinfectAreaVO> disinfectAreaVOList = disinfectAreaService.disinfectAreaToDisinfectAreaVO(disinfectAreaList);
        if(CollectionUtil.isNotEmpty(disinfectAreaVOList)){
            for (DisinfectAreaVO disinfectAreaVO : disinfectAreaVOList) {
                StringBuilder waitDisinfect = this.toString(disinfectAreaVO.getDisinfectAreaTimeVOList());
                disinfectAreaVO.setWaitDisinfect(waitDisinfect.length() == 0? "待消杀:无":waitDisinfect.toString());
            }
        }
        return RestResponse.ok(disinfectAreaVOList);
    }

    @Override
    public RestResult<List<DisinfectRecordVO>> disinfectTodayList(DisinfectTodayListDTO disinfectTodayListDTO) {
        if(disinfectTodayListDTO.getDisinfectAreaId() == null){
            return RestResponse.error("区域不能为空");
        }
        List<DisinfectRecord> list = disinfectRecordDao.selectByDisinfectAreaIdList(CollectionUtil.toList(disinfectTodayListDTO.getDisinfectAreaId()));
        List<DisinfectRecordVO> disinfectRecordVOList = list.stream().map(p -> {
            DisinfectRecordVO disinfectRecordVO = BeanUtil.copyProperties(p,DisinfectRecordVO.class);
            String disinfectTime = new StringBuffer().append(p.getDisinfectTimeDate()).append(" ").append(p.getDisinfectTimeStart()).append("-").append(p.getDisinfectTimeEnd()).toString();
            disinfectRecordVO.setDisinfectTime(disinfectTime);
            return disinfectRecordVO;
        }).collect(Collectors.toList());
        return RestResponse.ok(disinfectRecordVOList);
    }

    @Override
    public RestResult<String> disinfectRemind(DisinfectScanQrcodeDTO disinfectScanQrcodeDTO) {
        //账号权限判定
        AdminVO adminVO = this.adminCheck(disinfectScanQrcodeDTO);
        //用户信息查询
        UserInfo userInfo = this.userInfoCheck(disinfectScanQrcodeDTO);
        //消杀人员查询
        DisinfectUser disinfectUser = this.getDisinfectUser(userInfo.getIdNumber(), adminVO.getAdminId());
        if(!disinfectUser.getDisinfectAuth().contains(DisinfectAuthEnum.CHECK.getCode().toString())){
            throw new ServiceException("您没有检查的权限，请联系管理员!");
        }
        List<Integer> disinfectAreaIdList = JSON.parseArray(disinfectUser.getDisinfectAreaIdList(), Integer.class);
        if(!disinfectAreaIdList.contains(disinfectScanQrcodeDTO.getDisinfectAreaId())){
            throw new ServiceException("您不负责该区域!");
        }
        //查询负责该区域且拥有消杀权限的人员
        List<DisinfectUser> disinfectUserList = disinfectUserDao.selectResponsibleDisinfectArea(disinfectScanQrcodeDTO.getDisinfectAreaId()+"");
        if (CollUtil.isEmpty(disinfectUserList)){
            throw new ServiceException("该区域没有负责的消杀人员!");
        }
        //获取身份证列表
        List<String> idNumberList = disinfectUserList.stream().map(DisinfectUser::getIdNumber).collect(Collectors.toList());
        //获取用户的openId
        List<UserInfo> userInfoList = userInfoDao.selectOpenIdByIdNumberList(WxInfoEnum.ZHEJIANG.getCode(),idNumberList);
        if(CollUtil.isEmpty(userInfoList)){
            return RestResponse.error("该区域的消杀人员还没有订阅该信息");
        }
        DisinfectScanQrcodeVO data = this.disinfectScanQrcode(disinfectScanQrcodeDTO).getData();
        if(StrUtil.isEmpty(data.getWaitDisinfect())){
            return RestResponse.error("该区域今天没有需要消杀的时间段了");
        }
        final WxMaService wxService = WxMaConfiguration.getMaService(appid);
        String disinfectAreaName = "消杀区域:"+data.getDisinfectAreaName();
        String disinfectTimeBreak = data.getWaitDisinfect();
        for (UserInfo info : userInfoList) {
            //发送消息订阅
            this.sendWxMsg(disinfectAreaName,disinfectTimeBreak,info.getBuildingOpenId(),wxService);
        }
        StringBuilder result = new StringBuilder("已向下列人员发送消息订阅:");
        List<String> collect = userInfoList.stream().map(UserInfo::getIdNumber).collect(Collectors.toList());
        for (DisinfectUser user : disinfectUserList) {
            if(collect.contains(user.getIdNumber())){
                result.append("[").append(user.getName()).append("]");
            }
        }
        return RestResponse.ok(result.toString());
    }
    private void sendWxMsg(String disinfectAreaName,String disinfectTimeBreak ,String openId,WxMaService wxService) {

        WxMaSubscribeMessage.WxMaSubscribeMessageBuilder builder = WxMaSubscribeMessage.builder();
        //发送人openid
        builder.toUser(openId);
        //发送的模板id
        builder.templateId(WXMessageTemplateEnum.DISINFECT_REMIND.getTemplate());
        //消息主体
        builder.data(setMeetingMsg(disinfectAreaName,disinfectTimeBreak));
        builder.miniprogramState(miniprogramState);
        //点击订阅消息的跳转链接（默认是正式发布的小程序页面，当然也可以通过参数进行控制）
        WxMaSubscribeMessage msg = builder.build();
        try {
            wxService.getMsgService().sendSubscribeMsg(msg);
        } catch (WxErrorException e) {
            log.error("会议通知发送失败，用户openId:{}, 错误码：{},错误信息：{}",
                    openId, e.getError().getErrorCode(), e.getError().getErrorMsg());
        }
    }

    public static List<WxMaSubscribeMessage.MsgData> setMeetingMsg (String disinfectAreaName,String disinfectTimeBreak) {
        List<WxMaSubscribeMessage.MsgData> dataList = new ArrayList<>();
        //这里要完全按照订阅消息模板详情去设置，thing1 ，thing2...不能变
        WxMaSubscribeMessage.MsgData data1 = new WxMaSubscribeMessage.MsgData();
        data1.setName("thing2");
        data1.setValue(disinfectAreaName);
        dataList.add(data1);
        WxMaSubscribeMessage.MsgData data3 = new WxMaSubscribeMessage.MsgData();
        data3.setName("thing7");
        data3.setValue(disinfectTimeBreak);
        dataList.add(data3);
        return dataList;
    }
    @Override
    public RestResult<Boolean> disinfectEdit(DisinfectEditDTO disinfectEditDTO) {
        //账号权限判定
        AdminVO adminVO = adminService.getInfo(disinfectEditDTO.getAdminId());
        if(adminVO == null){
            throw new ServiceException("账号不存在");
        }
        //用户信息查询
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(disinfectEditDTO.getUserId());
        if(userInfo == null){
            throw new ServiceException("用户不存在");
        }
        //消杀人员信息查询
        DisinfectUser disinfectUser = this.getDisinfectUser(userInfo.getIdNumber(), adminVO.getAdminId());
        //用户消杀区域查询
        List<Integer> disinfectAreaIdList = JSON.parseArray(disinfectUser.getDisinfectAreaIdList(), Integer.class);
        if(!disinfectAreaIdList.contains(disinfectEditDTO.getDisinfectAreaId())){
            throw new ServiceException("你没有该区域的权限");
        }
        if(!disinfectUser.getDisinfectAuth().contains(DisinfectAuthEnum.CHECK.getCode()+"")){
            throw new ServiceException("你没有编辑该区域的权限");
        }
        DisinfectArea disinfectArea = disinfectAreaService.disinfectAreaDTOToDisinfectArea(adminVO, disinfectEditDTO);
        disinfectArea.setId(disinfectEditDTO.getDisinfectAreaId());
        disinfectAreaDao.updateByPrimaryKeySelective(disinfectArea);
        return RestResponse.ok(true);
    }

    private AdminVO adminCheck(DisinfectApiBaseDTO disinfectApiBaseDTO) {
        if(disinfectApiBaseDTO.getAdminId() == null){
            throw new ServiceException("请输入账号id");
        }
        AdminVO info = adminService.getInfo(disinfectApiBaseDTO.getAdminId());
        if(info == null){
            throw new ServiceException("账号不存在");
        }
        //todo 需要在这增加对于admin是否开启了消杀权限的判定
        if(!info.getDisinfectFlag().equals(1)){
            throw new ServiceException("账号未开启消杀");
        }
        return info;
    }
    private UserInfo userInfoCheck(DisinfectApiBaseDTO disinfectApiBaseDTO) {
        if(disinfectApiBaseDTO.getUserId() == null){
            throw new ServiceException("请输入用户id");
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(disinfectApiBaseDTO.getUserId());
        if(userInfo == null){
            throw new ServiceException("用户不存在");
        }
        return userInfo;
    }
    private DisinfectUser disinfectUserCheck(String idNumber,Integer adminId){
        //消杀人员查询
        DisinfectUser disinfectUser = this.getDisinfectUser(idNumber, adminId);
        if(!disinfectUser.getDisinfectAuth().contains(DisinfectAuthEnum.DISINFECT.getCode().toString())){
            throw new ServiceException("您没有消杀的权限，请联系管理员!");
        }
        return disinfectUser;
    }
    private DisinfectUser getDisinfectUser(String idNumber,Integer adminId){
        DisinfectUser disinfectUser = disinfectUserDao.selectOne(
                new DisinfectUser()
                        .setIsDel(Boolean.FALSE)
                        .setIdNumber(idNumber)
                        .setAdminId(adminId));
        if(disinfectUser == null){
            throw new ServiceException("请先联系管理员，添加为消杀人员!");
        }
        return disinfectUser;
    }
    private DisinfectArea disinfectAreaCheck(Integer disinfectAreaId,Integer adminId){
        //消杀区域查询
        DisinfectArea disinfectArea = disinfectAreaDao.selectByPrimaryKey(disinfectAreaId);
        if(disinfectArea == null){
            throw new ServiceException("消杀区域不存在，请扫描正确的消杀码!");
        }
        return disinfectArea;
    }
    private void disinfectAuthCheck(DisinfectUser disinfectUser, DisinfectArea disinfectArea) {
        String disinfectAreaIdList = disinfectUser.getDisinfectAreaIdList();
        if(StrUtil.isEmpty(disinfectAreaIdList)){
            throw new ServiceException("你不负责该区域的消杀");
        }
        List<Integer> list = JSON.parseArray(disinfectAreaIdList, Integer.class);
        for (Integer disinfectAreaId : list) {
            if(disinfectArea.getId().equals(disinfectAreaId)){
                return;
            }
        }
        throw new ServiceException("你不负责该区域的消杀 ");
    }
}
