package com.yx.face.service.disinfect.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.AddressUtil;
import com.yx.face.boot.uitls.FileUtils;
import com.yx.face.boot.uitls.QRCodeUtil;
import com.yx.face.dao.disinfect.DisinfectAreaDao;
import com.yx.face.dao.disinfect.DisinfectRecordDao;
import com.yx.face.model.dto.disinfect.DisinfectAreaDTO;
import com.yx.face.model.dto.disinfect.DisinfectAreaTimeDTO;
import com.yx.face.model.entity.DisinfectArea;
import com.yx.face.model.entity.DisinfectRecord;
import com.yx.face.model.search.DisinfectAreaSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.disinfect.DisinfectAreaTimeVO;
import com.yx.face.model.vo.disinfect.DisinfectAreaVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.disinfect.DisinfectAreaService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-11-11
 */
@Service
public class DisinfectAreaServiceImpl implements DisinfectAreaService {

    @Resource
    private DisinfectAreaDao disinfectAreaDao;
    @Resource
    private DisinfectRecordDao disinfectRecordDao;
    @Resource
    private AdminService adminService;
   
    @Override
    public PageInfo<DisinfectAreaVO> getPageList(RestDTO<DisinfectAreaSearch> dto) {
        Map<String, Object> map = new HashMap<>();
        AdminVO info = this.adminCheck();
        map.put("info", info);
        map.put("search", dto.getData());
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<DisinfectArea> list = disinfectAreaDao.selectListBySearch(map);
        return new PageInfo<>(this.disinfectAreaToDisinfectAreaVO(list));
    }

    public List<DisinfectAreaVO> disinfectAreaToDisinfectAreaVO(List<DisinfectArea> list){
        //根据list的时间端获取数据,转换为disinfectAreaIdList,根据这个获取当天消杀记录
        List<Integer> disinfectAreaIdList = list.stream().map(DisinfectArea::getId).collect(Collectors.toList());
        Map<String, DisinfectRecord> disinfectRecordMap = new HashMap<>();
        if(CollUtil.isNotEmpty(disinfectAreaIdList)){
            /*获取当前消杀区域的消杀记录*/
            List<DisinfectRecord> disinfectRecordList = disinfectRecordDao.selectByDisinfectAreaIdList(disinfectAreaIdList);
            disinfectRecordMap = disinfectRecordList.stream().collect(Collectors.toMap(DisinfectRecord::getDisinfectTimeBreakId,Function.identity()));
        }
        //数据组装
        Map<String, DisinfectRecord> finalDisinfectRecordMap = disinfectRecordMap;
        List<DisinfectAreaVO> disinfectAreaVOList = list.stream().map(p -> {
            DisinfectAreaVO disinfectAreaVO = BeanUtil.copyProperties(p,DisinfectAreaVO.class);
            //获取时间段数据
            List<DisinfectAreaTimeVO> disinfectAreaTimeVOList = JSON.parseArray(p.getDisinfectTimeBreak(), DisinfectAreaTimeVO.class);
            Integer todayDisinfectNum = 0;
            //将区域中的时间端 的消杀情况填入
            if(CollUtil.isNotEmpty(disinfectAreaTimeVOList)){
                for (DisinfectAreaTimeVO disinfectAreaTimeVO : disinfectAreaTimeVOList) {
                    if(finalDisinfectRecordMap.containsKey(disinfectAreaTimeVO.getDisinfectAreaTimeId())){
                        DisinfectRecord disinfectRecord = finalDisinfectRecordMap.get(disinfectAreaTimeVO.getDisinfectAreaTimeId());
                        disinfectAreaTimeVO.setDisinfectAreaTimeType(disinfectRecord.getStatus());
                        todayDisinfectNum++;
                    }
                }
            }
            disinfectAreaVO.setDisinfectAreaTimeVOList(disinfectAreaTimeVOList);
            disinfectAreaVO.setTodayDisinfectNum(todayDisinfectNum);
            disinfectAreaVO.setTodayTotalDisinfectNum(disinfectAreaTimeVOList.size());
            disinfectAreaVO.setDisinfectAreaId(p.getId());
            disinfectAreaVO.setDisinfectQrcode(p.getDisinfectQrcode());
            return disinfectAreaVO;
        }).collect(Collectors.toList());
        return disinfectAreaVOList;
    }

    @Override
    public Integer edit(DisinfectAreaDTO disinfectAreaDTO) {
        AdminVO info = this.adminCheck();
        DisinfectArea disinfectArea = this.disinfectAreaDTOToDisinfectArea(info,disinfectAreaDTO);
        //新增/修改
        if(disinfectAreaDTO.getDisinfectAreaId() == null){
            disinfectArea.setAdminId(info.getAdminId());
            disinfectArea.setCreateTime(new Date());
            disinfectAreaDao.insertSelective(disinfectArea);
        }else {
            disinfectArea.setId(disinfectAreaDTO.getDisinfectAreaId());
            disinfectAreaDao.updateByPrimaryKeySelective(disinfectArea);
        }
        //区域二维码填入
        try {
            String content = AddressUtil.address.substring(0, AddressUtil.address.length() - 1).concat("?disinfectAreaId=") + disinfectArea.getId()+ "&orgId=" + info.getAdminId() + "&org=" + info.getUsername();
            String path = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/disinfect_area_" + disinfectArea.getId() + ".png";
            QRCodeUtil.encode(content, "", path, true);
            disinfectArea.setDisinfectQrcode(AddressUtil.address + path);
            disinfectAreaDao.updateByPrimaryKeySelective(disinfectArea);
        } catch (Exception e) {
            throw new ServiceException("生成二维码失败,请编辑后重新确定");
        }
        return 1;
    }
    @Override
    public DisinfectArea disinfectAreaDTOToDisinfectArea(AdminVO info, DisinfectAreaDTO disinfectAreaDTO) {
        //查看消杀区域名称是否重复
        DisinfectArea disinfectArea1 = disinfectAreaDao.selectOne(new DisinfectArea()
                .setAdminId(info.getAdminId())
                .setDisinfectAreaName(disinfectAreaDTO.getDisinfectAreaName())
                .setIsDel(Boolean.FALSE));
        if(disinfectArea1 != null){
            if(disinfectAreaDTO.getDisinfectAreaId() == null){
                throw new ServiceException("消杀区域名称不能重复");
            }else {
                if(!disinfectAreaDTO.getDisinfectAreaId().equals(disinfectArea1.getId())){
                    throw new ServiceException("消杀区域名称不能重复");
                }
            }
        }
        //数据填入
        DisinfectArea disinfectArea = BeanUtil.copyProperties(disinfectAreaDTO,DisinfectArea.class);
        if(disinfectAreaDTO.getDisinfectAreaPhone().contains("*")){
            disinfectArea.setDisinfectAreaPhone(null);
        }
        if(disinfectAreaDTO.getDisinfectAreaOnus().contains("*")){
            disinfectArea.setDisinfectAreaOnus(null);
        }
        //消杀时间段填入
        if(CollUtil.isNotEmpty(disinfectAreaDTO.getDisinfectAreaTimeDTOList())){
            disinfectAreaDTO.getDisinfectAreaTimeDTOList().sort(Comparator.comparingLong(p -> p.getStartTime().getTime()));
            List<DisinfectAreaTimeDTO> disinfectAreaTimeDTOList = disinfectAreaDTO.getDisinfectAreaTimeDTOList();
            disinfectAreaTimeDTOList.stream().forEach(p ->{
                if(p.getStartTime().getTime() > p.getEndTime().getTime()){
                    throw new ServiceException("时间段中：开始时间不能大于结束时间");
                }
            });
            if(disinfectAreaTimeDTOList.size() > 1){
                for (int i = 0; i < disinfectAreaTimeDTOList.size() - 1; i++) {
                    long endTime = disinfectAreaTimeDTOList.get(i).getEndTime().getTime();
                    long startTime = disinfectAreaTimeDTOList.get(i + 1).getStartTime().getTime();
                    if(startTime < endTime){
                        throw new ServiceException("时间段之间不能存在包含关系");
                    }
                }
            }
            disinfectAreaTimeDTOList.stream().filter(p ->StrUtil.isEmpty(p.getDisinfectAreaTimeId())).forEach(p -> p.setDisinfectAreaTimeId(IdUtil.fastSimpleUUID()));
            disinfectArea.setDisinfectNum(disinfectAreaTimeDTOList.size());
            disinfectArea.setDisinfectTimeBreak(JSON.toJSONString(disinfectAreaTimeDTOList));
        }else {
            disinfectArea.setDisinfectNum(0);
            disinfectArea.setDisinfectTimeBreak(StrUtil.EMPTY);
        }
        return disinfectArea;
    }

    @Override
    public Integer delete(Integer disinfectAreaId) {
        DisinfectArea disinfectArea = disinfectAreaDao.selectByPrimaryKey(disinfectAreaId);
        if(disinfectArea == null){
            throw new ServiceException("该消杀区域不存在");
        }
        if(disinfectArea.getIsDel()){
            throw new ServiceException("该消杀区域已经被删除了");
        }
        disinfectArea.setIsDel(Boolean.TRUE);
        return disinfectAreaDao.updateByPrimaryKeySelective(disinfectArea);
    }

    @Override
    public RestResult<List<DisinfectAreaVO>> getList() {
        AdminVO info = this.adminCheck();
        List<DisinfectArea> disinfectAreaList = disinfectAreaDao.select(
                new DisinfectArea().setAdminId(info.getAdminId()).setIsDel(Boolean.FALSE));
        List<DisinfectAreaVO> disinfectAreaVOList = disinfectAreaList.stream().map(p -> {
            DisinfectAreaVO disinfectAreaVO = new DisinfectAreaVO();
            disinfectAreaVO.setDisinfectAreaId(p.getId());
            disinfectAreaVO.setDisinfectAreaName(p.getDisinfectAreaName());
            return disinfectAreaVO;
        }).collect(Collectors.toList());
        return RestResponse.ok(disinfectAreaVOList);
    }

    private AdminVO adminCheck() {
        AdminVO info = adminService.getInfo();
        //todo 需要在这增加对于admin是否开启了消杀权限的判定
        if(!info.getDisinfectFlag().equals(1)){
            throw new ServiceException("账号未开启消杀");
        }
        return info;
    }
}
