package com.yx.face.service.disinfect.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.disinfect.DisinfectRecordDao;
import com.yx.face.model.entity.DisinfectRecord;
import com.yx.face.model.search.DisinfectRecordSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.disinfect.DisinfectRecordVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.disinfect.DisinfectRecordService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-11-11
 */
@Service
public class DisinfectRecordServiceImpl implements DisinfectRecordService {
    @Resource
    private AdminService adminService;
    @Resource
    private DisinfectRecordDao disinfectRecordDao;

    @Override
    public PageInfo<DisinfectRecordVO> getPageList(RestDTO<DisinfectRecordSearch> dto) {
        Map<String, Object> map = new HashMap<>();
        AdminVO info = this.adminCheck();
        map.put("info", info);
        map.put("search", dto.getData());
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<DisinfectRecord> list = disinfectRecordDao.selectListBySearch(map);
        boolean isShowName = false;
        boolean isShowPhone = false;
        boolean isShowNumber = false;
        if (info.getNoHideCloumn() != null && info.getNoHideCloumn().size() > 0) {
            if (info.getNoHideCloumn().contains("1")) {
                isShowName = true;
            }
            if (info.getNoHideCloumn().contains("2")) {
                isShowPhone = true;
            }
            if (info.getNoHideCloumn().contains("3")) {
                isShowNumber = true;
            }
        }
        boolean finalIsShowPhone = isShowPhone;
        boolean finalIsShowNumber = isShowNumber;
        boolean finalIsShowName = isShowName;
        List<DisinfectRecordVO> disinfectRecordVOList = list.stream().map(p -> {
            DisinfectRecordVO disinfectRecordVO = BeanUtil.copyProperties(p,DisinfectRecordVO.class);
            String disinfectTime = new StringBuffer().append(p.getDisinfectTimeDate()).append(" ").append(p.getDisinfectTimeStart()).append("-").append(p.getDisinfectTimeEnd()).toString();
            disinfectRecordVO.setDisinfectTime(disinfectTime);
            if (StrUtil.isNotEmpty(p.getPhone()) && finalIsShowPhone) {
                disinfectRecordVO.setPhone(SMSOrIdCardUtils.hidePhone(p.getPhone()));
            }
            if (StrUtil.isNotEmpty(p.getIdNumber()) && finalIsShowNumber) {
                disinfectRecordVO.setIdNumber(SMSOrIdCardUtils.hideIdNumber(p.getIdNumber()));
            }
            if (StrUtil.isNotEmpty(p.getName()) && finalIsShowName) {
                disinfectRecordVO.setName(SMSOrIdCardUtils.hideName(p.getName().trim()));
            }
            return disinfectRecordVO;
        }).collect(Collectors.toList());
        return new PageInfo<>(disinfectRecordVOList);
    }
    private AdminVO adminCheck() {
        AdminVO info = adminService.getInfo();
        //todo 需要在这增加对于admin是否开启了消杀权限的判定
        if(!info.getDisinfectFlag().equals(1)){
            throw new ServiceException("账号未开启消杀");
        }
        return info;
    }
}
