package com.yx.face.service.disinfect.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.disinfect.DisinfectAreaDao;
import com.yx.face.dao.disinfect.DisinfectUserDao;
import com.yx.face.model.dto.disinfect.DisinfectUserDTO;
import com.yx.face.model.entity.DisinfectArea;
import com.yx.face.model.entity.DisinfectUser;
import com.yx.face.model.excel.DisinfectUserExcel;
import com.yx.face.model.search.DisinfectUserSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.disinfect.DisinfectUserVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.disinfect.DisinfectUserService;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.weekend.Weekend;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 消杀人员表 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-11-11
 */
@Service
public class DisinfectUserServiceImpl implements DisinfectUserService {

    @Resource
    private AdminService adminService;
    @Resource
    private DisinfectUserDao disinfectUserDao;
    @Resource
    private DisinfectAreaDao disinfectAreaDao;
    @Override
    public PageInfo<DisinfectUserVO> getPageList(RestDTO<DisinfectUserSearch> dto) {
        Map<String, Object> map = new HashMap<>();
        AdminVO info = this.adminCheck();
        map.put("info", info);
        map.put("search", dto.getData());
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<DisinfectUser> list = disinfectUserDao.selectListBySearch(map);
        boolean isShowName = false;
        boolean isShowNumber = false;
        if (info.getNoHideCloumn() != null && info.getNoHideCloumn().size() > 0) {
            if (info.getNoHideCloumn().contains("1")) {
                isShowName = true;
            }
            if (info.getNoHideCloumn().contains("3")) {
                isShowNumber = true;
            }
        }
        boolean finalIsShowNumber = isShowNumber;
        boolean finalIsShowName = isShowName;
        List<DisinfectUserVO> disinfectUserVOList = list.stream().map(p -> {
            DisinfectUserVO disinfectUserVO = BeanUtil.copyProperties(p,DisinfectUserVO.class);
            disinfectUserVO.setDisinfectUserId(p.getId());
            disinfectUserVO.setDisinfectAuthList(JSON.parseArray(p.getDisinfectAuth(),Integer.class));
            disinfectUserVO.setDisinfectAreaIdList(StrUtil.isEmpty(p.getDisinfectAreaIdList())?new ArrayList<>():JSON.parseArray(p.getDisinfectAreaIdList(),Integer.class));

            if(CollUtil.isNotEmpty(disinfectUserVO.getDisinfectAreaIdList())){
                Weekend<DisinfectArea> weekend = new Weekend<>(DisinfectArea.class);
                weekend.weekendCriteria()
                        .andIn(DisinfectArea::getId,disinfectUserVO.getDisinfectAreaIdList())
                        .andEqualTo(DisinfectArea::getIsDel,Boolean.FALSE);
                List<DisinfectArea> disinfectAreaList = disinfectAreaDao.selectByExample(weekend);
                List<String> disinfectAreaNameList = disinfectAreaList.stream().map(DisinfectArea::getDisinfectAreaName).collect(Collectors.toList());
                List<Integer> disinfectAreaIdList = disinfectAreaList.stream().map(DisinfectArea::getId).collect(Collectors.toList());
                disinfectUserVO.setDisinfectAreaIdList(disinfectAreaIdList);
                StringBuilder disinfectAreaNameString = new StringBuilder();
                for (int i = 0; i < disinfectAreaNameList.size(); i++) {
                    disinfectAreaNameString.append(disinfectAreaNameList.get(i));
                    if(i != disinfectAreaNameList.size() - 1){
                        disinfectAreaNameString.append(",");
                    }
                }
                disinfectUserVO.setDisinfectAreaNameStringList(disinfectAreaNameString.toString());
            }

            if (StrUtil.isNotEmpty(p.getIdNumber()) && finalIsShowNumber) {
                disinfectUserVO.setIdNumber(SMSOrIdCardUtils.hideIdNumber(p.getIdNumber()));
            }
            if (StrUtil.isNotEmpty(p.getName()) && finalIsShowName) {
                disinfectUserVO.setName(SMSOrIdCardUtils.hideName(p.getName().trim()));
            }

            return disinfectUserVO;
        }).collect(Collectors.toList());
        return new PageInfo<>(disinfectUserVOList);
    }

    @Override
    public Integer edit(DisinfectUserDTO disinfectUserDTO) {
        AdminVO info = this.adminCheck();
        DisinfectUser disinfectUser = new DisinfectUser();
        if(StrUtil.isNotEmpty(disinfectUserDTO.getName()) && !disinfectUserDTO.getName().contains("*")){
            disinfectUser.setName(disinfectUserDTO.getName());
        }
        //权限
        if(CollUtil.isNotEmpty(disinfectUserDTO.getDisinfectAuthList())){
            disinfectUser.setDisinfectAuth(JSON.toJSONString(disinfectUserDTO.getDisinfectAuthList()));
        }else {
            throw new ServiceException("请选择权限");
        }
        //负责消杀区域
        if(CollUtil.isNotEmpty(disinfectUserDTO.getDisinfectAreaIdList())){
            disinfectUser.setDisinfectAreaIdList(JSON.toJSONString(disinfectUserDTO.getDisinfectAreaIdList()));
        }
        //根据是否存在主键id 判定为 增加 或者 修改
        if(disinfectUserDTO.getDisinfectUserId() == null){
            if(!IdcardUtil.isValidCard(disinfectUserDTO.getIdNumber())){
                throw new ServiceException("身份证有误");
            }
            DisinfectUser disinfectUser1 = disinfectUserDao.selectOne(new DisinfectUser()
                    .setAdminId(info.getAdminId())
                    .setIdNumber(disinfectUserDTO.getIdNumber())
                    .setIsDel(Boolean.FALSE));
            if(disinfectUser1 != null){
                throw new ServiceException("身份证已存在");
            }
            disinfectUser.setIdNumber(disinfectUserDTO.getIdNumber());
            disinfectUser.setAdminId(info.getAdminId());
            disinfectUser.setCreateTime(new Date());
            disinfectUserDao.insertSelective(disinfectUser);
        }else{
            disinfectUser.setId(disinfectUserDTO.getDisinfectUserId());
            disinfectUserDao.updateByPrimaryKeySelective(disinfectUser);
        }
        return 1;
    }

    @Override
    public Integer delete(Integer disinfectUserId) {
        DisinfectUser disinfectUser = disinfectUserDao.selectByPrimaryKey(disinfectUserId);
        if(disinfectUser == null){
            throw new ServiceException("该人员不存在");
        }
        if(disinfectUser.getIsDel()){
            throw new ServiceException("该人员已经被删除了");
        }
        disinfectUser.setIsDel(Boolean.TRUE);
        return disinfectUserDao.updateByPrimaryKeySelective(disinfectUser);
    }

    @Override
    public RestResult<List<String>> excelInput(List<Object> datas) {
        AdminVO adminVO = this.adminCheck();
        List<String> list = new ArrayList<>();

        List<DisinfectArea> disinfectAreaList = disinfectAreaDao.select(
                new DisinfectArea()
                        .setIsDel(Boolean.FALSE)
                        .setAdminId(adminVO.getAdminId())
        );
        Map<String, Integer> disinfectAreaMap = disinfectAreaList.stream().collect(Collectors.toMap(DisinfectArea::getDisinfectAreaName, DisinfectArea::getId));


        for (int i = 0; i < datas.size(); i++) {
            try {
                DisinfectUserExcel excel = (DisinfectUserExcel) datas.get(i);
                if(StrUtil.isEmpty(excel.getName())){
                    throw new ServiceException("姓名不能为空");
                }
                if(StrUtil.isEmpty(excel.getIdNumber())){
                    throw new ServiceException("身份证不能为空");
                }
                if(StrUtil.isEmpty(excel.getDisinfectUserAuth())){
                    throw new ServiceException("人员权限不能为空");
                }
                DisinfectUserDTO disinfectUserDTO = BeanUtil.copyProperties(excel,DisinfectUserDTO.class);
                //人员权限
                List<Integer> disinfectUserAuth = new ArrayList<>();
                if(excel.getDisinfectUserAuth().contains("消杀")){
                    disinfectUserAuth.add(1);
                }
                if(excel.getDisinfectUserAuth().contains("检查")){
                    disinfectUserAuth.add(2);
                }
                if(disinfectUserAuth.size() == 0){
                    throw new ServiceException("人员权限有误");
                }
                disinfectUserDTO.setDisinfectAuthList(disinfectUserAuth);
                //人员区域
                List<Integer> disinfectAreaIdList = new ArrayList<>();
                if(StrUtil.isNotEmpty(excel.getDisinfectUserArea())){
                    String[] split = excel.getDisinfectUserArea().split("/");
                    for (String areaName : split) {
                        if(disinfectAreaMap.containsKey(areaName)){
                            disinfectAreaIdList.add(disinfectAreaMap.get(areaName));
                        }
                    }
                }
                disinfectUserDTO.setDisinfectAreaIdList(disinfectAreaIdList);
                try {
                    this.edit(disinfectUserDTO);
                } catch (Exception e) {
                    throw new ServiceException(e.getMessage());
                }
            } catch (ServiceException e) {
                list.add("第" + (i + 3) + "行," + e.getMessage());
            }
        }
        datas.clear();//结束后销毁不用的资源
        return RestResponse.ok(list);
    }

    private AdminVO adminCheck() {
        AdminVO info = adminService.getInfo();
        //todo 需要在这增加对于admin是否开启了消杀权限的判定
        if(!info.getDisinfectFlag().equals(1)){
            throw new ServiceException("账号未开启消杀");
        }
        return info;
    }
}
