package com.yx.face.service.disinfect.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.beust.jcommander.internal.Lists;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.AddressUtil;
import com.yx.face.boot.uitls.FileUtils;
import com.yx.face.dao.ProblemReportDao;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.model.dto.ProblemReportCountDTO;
import com.yx.face.model.dto.ProblemReportQueryDTO;
import com.yx.face.model.entity.ProblemReport;
import com.yx.face.model.entity.UserInfo;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.enums.ProblemReportStatusEnum;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.disinfect.*;
import com.yx.face.service.AdminService;
import com.yx.face.service.disinfect.ProblemReportService;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lejingjing
 * @version 1.0
 * @Description:
 * @date 2022/11/09
 **/
@Slf4j
@Service
public class ProblemReportServceImpl implements ProblemReportService {

    @Resource
    private ProblemReportDao problemReportDao;

    @Resource
    private AdminService adminService;

    @Resource
    private UserInfoDao userInfoDao;

   

    @Override
    public RestResult<ProblemReport> addProblemReport(ProblemReportAddVO problemReportAddVO) {
        ProblemReport problemReport = new ProblemReport();
        problemReport.setAdminId(problemReportAddVO.getAdminId());
        problemReport.setUserId(problemReportAddVO.getUserId());
        problemReport.setProblemDesc(problemReportAddVO.getDesc());
        problemReport.setAnonymityFlag(problemReportAddVO.getAnonymityFlag());
        problemReport.setCreateDate(new Date());
        problemReport.setCreateId(problemReportAddVO.getAdminId());
        problemReport.setUpdateDate(new Date());
        problemReport.setUpdateId(problemReportAddVO.getAdminId());
        problemReport.setAnonymityFlag(problemReportAddVO.getAnonymityFlag());
        problemReport.setReportTime(new Date());
        problemReport.setPictures(JSONUtil.toJsonStr(problemReportAddVO.getPictureVOList()));
        problemReport.setVideo(JSONUtil.toJsonStr(problemReportAddVO.getVideo()));
        problemReport.setStatus(ProblemReportStatusEnum.PREPARED.getStatus());
        problemReportDao.insert(problemReport);
        return RestResponse.ok(problemReport);
    }

    @Override
    public RestResult<List<ProblemReportListVO>> getProblemReortList(ProblemReportListQueryVO problemReportListQueryVO) {
        List<ProblemReport> problemReportList = problemReportDao.selectByExample(Example.builder(ProblemReport.class).where(WeekendSqls
                .custom().andEqualTo("adminId", problemReportListQueryVO.getAdminId())).orderByDesc("createDate").build());
        if(Objects.nonNull(problemReportListQueryVO)&&Objects.nonNull(problemReportListQueryVO.getUserId())){
            problemReportList = problemReportDao.selectByExample(Example.builder(ProblemReport.class).where(WeekendSqls
                    .custom().andEqualTo("adminId", problemReportListQueryVO.getAdminId())
                    .andEqualTo("userId",problemReportListQueryVO.getUserId())).orderByDesc("createDate").build());
        }

        List<ProblemReportListVO> problemReportListVOList = problemReportList.stream().map(e -> {
            ProblemReportListVO problemReportListVO = new ProblemReportListVO();
            problemReportListVO.setId(e.getId());
            problemReportListVO.setDesc(e.getProblemDesc());
            problemReportListVO.setStatus(e.getStatus());
            problemReportListVO.setReportTime(e.getReportTime());
            return problemReportListVO;
        }).collect(Collectors.toList());
        return RestResponse.ok(problemReportListVOList);
    }

    @Override
    public RestResult<ProblemReport> evaluate(ReportEvaluateVO reportEvaluateVO) {
        ProblemReport problemReport = problemReportDao.selectByPrimaryKey(reportEvaluateVO.getId());
        if (Objects.nonNull(problemReport)) {
            problemReport.setEvaluateContent(reportEvaluateVO.getEvaluateContent());
            problemReport.setEvaluateTime(new Date());
            problemReport.setEvaluateWorkpoint(reportEvaluateVO.getEvaluateWorkpoint());
            problemReportDao.updateByPrimaryKey(problemReport);
        }
        return RestResponse.ok(problemReport);
    }

    @Override
    public RestResult<PageInfo<ProblemReportPageVO>> getByPage(RestDTO<ProblemReportSearch> dto) {
        AdminVO adminVO = adminService.getInfo();
        List<AdminVO> adminVOList = adminService.getAdminlist();
        ProblemReportSearch problemReportSearch = dto.getData();
        log.info("获取下级账户id:{}", JSON.toJSONString(adminVOList));
        ProblemReportQueryDTO problemReportQueryDTO = initQueryParam(adminVO,problemReportSearch, adminVOList);
        Page page = PageHelper.startPage(dto.getPageNum(), dto.getPageSize(),true);
        List<ProblemReport> problemReportList = problemReportDao.getByPage(problemReportQueryDTO);
        List<ProblemReportPageVO> result = problemReportList.stream().map(e -> {
            UserInfo userInfo = userInfoDao.selectByPrimaryKey(e.getUserId());
            if(Objects.nonNull(userInfo)&&StrUtil.isNotBlank(userInfo.getUsername())){
                userInfo.setUsername(DesensitizedUtil.chineseName(userInfo.getUsername()));
            }
            if(Objects.nonNull(userInfo)&&StrUtil.isNotBlank(userInfo.getIdNumber())){
                userInfo.setIdNumber(DesensitizedUtil.idCardNum(userInfo.getIdNumber(),4,4));
            }
            if(Objects.nonNull(userInfo)&&StrUtil.isNotBlank(userInfo.getPhone())){
                userInfo.setPhone(DesensitizedUtil.mobilePhone(userInfo.getPhone()));
            }
            ProblemReportPageVO problemReportPageVO = new ProblemReportPageVO();
            problemReportPageVO.setId(e.getId());
            problemReportPageVO.setAnonymityFlag(e.getAnonymityFlag());
            problemReportPageVO.setUserInfo(userInfo);
            problemReportPageVO.setReportTime(e.getReportTime());
            problemReportPageVO.setDesc(e.getProblemDesc());
            if(StrUtil.isBlank(e.getPictures())){
                problemReportPageVO.setPictures(Lists.newArrayList());
            }else{
                problemReportPageVO.setPictures(JSON.parseObject(e.getPictures(), List.class));
            }
            if(StrUtil.isBlank(e.getVideo())){
                problemReportPageVO.setVideo(new FileVO());
            }else{
                problemReportPageVO.setVideo(JSON.parseObject(e.getVideo(), FileVO.class));
            }
            problemReportPageVO.setHandleTime(e.getHandleTime());
            problemReportPageVO.setHandleResult(e.getHandleResult());
            problemReportPageVO.setEvaluateTime(e.getEvaluateTime());
            problemReportPageVO.setEvaluateContent(e.getEvaluateContent());
            problemReportPageVO.setEvaluateWorkpoint(e.getEvaluateWorkpoint());
            problemReportPageVO.setStatus(e.getStatus());
            return problemReportPageVO;
        }).collect(Collectors.toList());
        PageInfo info=new PageInfo(result);
        info.setTotal(page.getTotal());
        return RestResponse.ok(info);
    }

    @Override
    public RestResult<ProblemReport> updateProblemReport(ProblemReportUpdateVO problemReportUpdateVO) {
        ProblemReport report = problemReportDao.selectByPrimaryKey(problemReportUpdateVO.getId());
        if (Objects.isNull(report)) {
            return RestResponse.error("上报问题不存在");
        }
        if(Objects.nonNull(problemReportUpdateVO.getStatus())){
            report.setStatus(problemReportUpdateVO.getStatus());
        }
        if(Objects.nonNull(problemReportUpdateVO.getHandleResult())){
            report.setHandleResult(problemReportUpdateVO.getHandleResult());
        }
        if (problemReportDao.updateByPrimaryKey(report) > 0) {
            return RestResponse.ok(report);
        }
        report.setHandleTime(new Date());
        return RestResponse.error("更新失败");
    }

    @Override
    public RestResult<ProblemReportData> getProblemReortData() {
        List<AdminVO> adminlist = adminService.getAdminlist();
        //计算总数
        List<Integer> adminIdList = adminlist.stream().map(e -> {
            return e.getAdminId();
        }).collect(Collectors.toList());
        Integer total = problemReportDao.selectCountByExample(Example.builder(ProblemReport.class)
                .where(WeekendSqls.custom().andIn("adminId", adminIdList)).build());
        List<ProblemReportCountDTO> statusCountMapList = problemReportDao.selectCountGroupByStatus(adminIdList);
        //已处理
        Integer finishedCount = 0;
        //待处理
        Integer preparedCount = 0;
        //处理中
        Integer handledCount = 0;
        //已关闭
        Integer closedCount = 0;
        if (CollUtil.isNotEmpty(statusCountMapList)) {
            for (ProblemReportCountDTO problemReportCountDTO : statusCountMapList) {
                Integer status = problemReportCountDTO.getProblemReportStatus();
                Integer count = problemReportCountDTO.getCount();
                if (status.equals(ProblemReportStatusEnum.FINISHED.getStatus())) {
                    finishedCount = count;
                }
                if (status.equals(ProblemReportStatusEnum.PREPARED.getStatus())) {
                    preparedCount = count;
                }
                if (status.equals(ProblemReportStatusEnum.HANDLED.getStatus())) {
                    handledCount = count;
                }
                if (status.equals(ProblemReportStatusEnum.CLOSED.getStatus())) {
                    closedCount = count;
                }
            }
        }
        //一星
        Integer oneStarCount = 0;
        //二星
        Integer twoStarCount = 0;
        //三星
        Integer threeStarCount = 0;
        //四星
        Integer fourStarCount = 0;
        //五星
        Integer fiveStarCount = 0;
        //一星
        Double oneStarPercentage = 0.00D;
        //二星
        Double twoStarPercentage = 0.00D;
        //三星
        Double threeStarPercentage = 0.00D;
        //四星
        Double fourStarPercentage = 0.00D;
        //五星
        Double fiveStarPercentage = 0.00D;
        List<ProblemReportCountDTO> evaluetMapList = problemReportDao.selectCountGroupByEvaluateWorkpoint(adminIdList);
        if (CollUtil.isNotEmpty(evaluetMapList)) {
            Integer maxStarCount = evaluetMapList.stream().filter(e->Objects.nonNull(e.getEvaluateWorkpoint())).map(ProblemReportCountDTO::getCount).max(Comparator.comparing(Integer::intValue)).get();
            for (ProblemReportCountDTO problemReportCountDTO : evaluetMapList) {
                Integer evaluateWorkpoint = problemReportCountDTO.getEvaluateWorkpoint();
                Integer count = problemReportCountDTO.getCount();
                if(Objects.nonNull(evaluateWorkpoint)){
                    if (evaluateWorkpoint.equals(1)) {
                        oneStarCount = count;
                        oneStarPercentage = (Double.parseDouble(String.valueOf(count))/maxStarCount)*100;
                    }
                    if (evaluateWorkpoint.equals(2)) {
                        twoStarCount = count;
                        twoStarPercentage = (Double.parseDouble(String.valueOf(count))/maxStarCount)*100;
                    }
                    if (evaluateWorkpoint.equals(3)) {
                        threeStarCount = count;
                        threeStarPercentage = (Double.parseDouble(String.valueOf(count))/maxStarCount)*100;
                    }
                    if (evaluateWorkpoint.equals(4)) {
                        fourStarCount = count;
                        fourStarPercentage = (Double.parseDouble(String.valueOf(count))/maxStarCount)*100;
                    }
                    if (evaluateWorkpoint.equals(5)) {
                        fiveStarCount = count;
                        fiveStarPercentage = (Double.parseDouble(String.valueOf(count))/maxStarCount)*100;
                    }
                }
            }
        }
        ProblemReportData problemReportData = new ProblemReportData();
        problemReportData.setTotal(total);
        problemReportData.setFinishedNum(finishedCount);
        problemReportData.setPreparedNum(preparedCount);
        problemReportData.setHandledNum(handledCount);
        problemReportData.setClosedNum(closedCount);
        problemReportData.setFiveStarNum(fiveStarCount);
        problemReportData.setFourStarNum(fourStarCount);
        problemReportData.setThreeStarNum(threeStarCount);
        problemReportData.setTwoStarNum(twoStarCount);
        problemReportData.setOneStarNum(oneStarCount);
        problemReportData.setFiveStarPercentage(fiveStarPercentage);
        problemReportData.setFourStarPercentage(fourStarPercentage);
        problemReportData.setThreeStarPercentage(threeStarPercentage);
        problemReportData.setTwoStarPercentage(twoStarPercentage);
        problemReportData.setOneStarPercentage(oneStarPercentage);
        return RestResponse.ok(problemReportData);
    }

    @Override
    public RestResult<ProblemReportDetailVO> getProblemReportDetailById(Integer reportId) {
        ProblemReport problemReport = problemReportDao.selectByPrimaryKey(reportId);
        ProblemReportDetailVO problemReportDetailVO = new ProblemReportDetailVO();
        problemReportDetailVO.setAdminId(problemReport.getAdminId());
        problemReportDetailVO.setAnonymityFlag(problemReport.getAnonymityFlag());
        problemReportDetailVO.setCreateDate(problemReport.getCreateDate());
        problemReportDetailVO.setCreateId(problemReport.getCreateId());
        problemReportDetailVO.setUpdateDate(problemReport.getUpdateDate());
        problemReportDetailVO.setUpdateId(problemReport.getUpdateId());
        problemReportDetailVO.setEvaluateContent(problemReport.getEvaluateContent());
        problemReportDetailVO.setEvaluateWorkpoint(problemReport.getEvaluateWorkpoint());
        problemReportDetailVO.setEvaluateTime(problemReport.getEvaluateTime());
        problemReportDetailVO.setHandleResult(problemReport.getHandleResult());
        problemReportDetailVO.setHandleTime(problemReport.getHandleTime());
        problemReportDetailVO.setProblemDesc(problemReport.getProblemDesc());
        problemReportDetailVO.setReportTime(problemReport.getReportTime());
        problemReportDetailVO.setStatus(problemReport.getStatus());
        problemReportDetailVO.setUserId(problemReport.getUserId());
        if(StrUtil.isNotBlank(problemReport.getPictures())){
            List<FileVO> pictures = JSON.parseObject(problemReport.getPictures(),List.class);
            problemReportDetailVO.setPictures(pictures);
        }
        if(StrUtil.isNotBlank(problemReport.getVideo())){
            FileVO video = JSON.parseObject(problemReport.getVideo(),FileVO.class);
            problemReportDetailVO.setVideo(video);
        }
        return RestResponse.ok(problemReportDetailVO);
    }

    @Override
    public String uploadVideo(InputStream inputStream, Integer userId) {
        String path = null;
        //按指定大小把图片进行缩和放（会遵循原图高宽比例）
        //此处把图片压成60*60的缩略图
        try {
            BufferedImage image = Thumbnails.of(inputStream).size(60, 60).asBufferedImage();//变为400*300,遵循原图比例缩或放到400*某个高度
            //输出流
            ByteArrayOutputStream stream = new ByteArrayOutputStream();
            ImageIO.write(image, "png", stream);
            String base64 = Base64.encode(stream.toByteArray());
            stream.flush();
            stream.close();
            String name = UUID.randomUUID().toString().concat(".jpg");
            String filePath = FileUtils.getDirectory(Constants.UPLOAD_VIDEO);
            path = AddressUtil.address + FileUtils.transformBase64Image(base64, filePath, name);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return path;
    }

    @Override
    public RestResult<AdminVO> getAuthFlagByAdminId(Integer adminId) {
        return RestResponse.ok(adminService.getInfo(adminId));
    }

    public ProblemReportQueryDTO initQueryParam(AdminVO adminVO, ProblemReportSearch problemReportSearch, List<AdminVO> adminVOList) {
        ProblemReportQueryDTO problemReportQueryDTO = new ProblemReportQueryDTO();
        if ((!adminVO.getType().equals(AdminTypeEnum.ADMINISTRATOR.getCode())) && CollUtil.isNotEmpty(adminVOList)) {
            List<Integer> adminIdList = adminVOList.stream().map(e -> {
                return e.getAdminId();
            }).collect(Collectors.toList());
            problemReportQueryDTO.setAdminIdList(adminIdList);
        }
        if (Objects.nonNull(problemReportSearch)&&StrUtil.isNotBlank(problemReportSearch.getUsername())) {
            problemReportQueryDTO.setUsername(problemReportSearch.getUsername());
            problemReportQueryDTO.setAnonymityFlag(Boolean.FALSE);
        }
        if (Objects.nonNull(problemReportSearch)&&StrUtil.isNotBlank(problemReportSearch.getMobile())) {
            problemReportQueryDTO.setMobile(problemReportSearch.getMobile());
            problemReportQueryDTO.setAnonymityFlag(Boolean.FALSE);
        }
        if (Objects.nonNull(problemReportSearch)&&StrUtil.isNotBlank(problemReportSearch.getIdNumber())) {
            problemReportQueryDTO.setIdNumber(problemReportSearch.getIdNumber());
            problemReportQueryDTO.setAnonymityFlag(Boolean.FALSE);
        }
        if (Objects.nonNull(problemReportSearch)&&StrUtil.isNotBlank(problemReportSearch.getDescKeywords())) {
            problemReportQueryDTO.setDescKeywords(problemReportSearch.getDescKeywords());
        }
        if (Objects.nonNull(problemReportSearch)&&Objects.nonNull(problemReportSearch.getReportStartDate())) {
            problemReportQueryDTO.setReportStartDate(problemReportSearch.getReportStartDate());
        }
        if (Objects.nonNull(problemReportSearch)&&Objects.nonNull(problemReportSearch.getReportEndDate())) {
            problemReportQueryDTO.setReportEndDate(problemReportSearch.getReportEndDate());
        }
        if (Objects.nonNull(problemReportSearch)&&Objects.nonNull(problemReportSearch.getStatus())) {
            problemReportQueryDTO.setStatus(problemReportSearch.getStatus());
        }
        return problemReportQueryDTO;
    }
}
