package com.yx.face.service.feign;

import com.yx.face.boot.restful.RestResult;
import com.yx.face.service.feign.domain.vo.RuiYingUserQueryResponseVO;
import com.yx.face.service.feign.domain.vo.RuiYingUserQueryVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

/**
 * 锐颖设备下发接口
 */
@FeignClient(name = "RemoteRYDeviceDownloadFaceService", url = "${rpc.device.ruiying.host}", path = "/ryprocessor/ruiying")
public interface RemoteRYDeviceDownloadFaceService {

    /**
     * 人员下发
     *
     * @param map
     * @return
     */
    @PostMapping("/addUser")
    public Map<String, Object> addUser(@RequestBody Map<String, Object> map);

    /**
     * 人员删除
     *
     * @param map
     * @return
     */
    @PostMapping("/delUser")
    public Map<String, Object> delUser(@RequestBody Map<String, Object> map);


    /**
     * 人员修改
     *
     * @param map
     * @return
     */
    @PostMapping("/updateUser")
    public Map<String, Object> updateUser(@RequestBody Map<String, Object> map);


    /**
     * 更改人员类型
     * @param map
     * @return
     */
    @PostMapping("/updateType")
    public Map<String, Object> updateType(@RequestBody Map<String, Object> map);

    /**
     * 人员查询
     *
     * @param ruiYingUserQueryVO
     * @return
     */
    @PostMapping("/queryUser")
    public RestResult<List<RuiYingUserQueryResponseVO>> queryUser(@RequestBody RuiYingUserQueryVO ruiYingUserQueryVO);

    /**
     * 数据上报-心跳检测接口
     *
     * @return
     */
    @GetMapping("/heartbeat")
    public RestResult heartbeat(@RequestParam("SerialNo") String serialNo);

    /**
     * 数据上报-图片推送接口
     *
     * @return
     */
    @PostMapping("/pushPicture")
    public RestResult pushPicture(@RequestParam("SerialNo") String serialNo,
                                  @RequestBody Map<String, Object> data);
}
