package com.yx.face.service.fkj;

import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.model.api.fkj.dto.FkjEthQrCodeDto;
import com.yx.face.model.api.fkj.dto.FkjQrCodeDto;
import com.yx.face.model.api.fkj.dto.FkjVguangQrCodeDto;
import com.yx.face.model.api.fkj.dto.VisiteeDto;
import com.yx.face.model.api.fkj.dto.VisiteePageDto;
import com.yx.face.model.api.fkj.vo.FkjAreaInfoVo;
import com.yx.face.model.api.fkj.vo.FkjEthQrCodeVo;
import com.yx.face.model.api.fkj.vo.FkjQrCodeVo;
import com.yx.face.model.api.fkj.vo.VisiteeVo;
import com.yx.face.model.dto.fkj.FkjFloorInitConfig;
import com.yx.face.model.dto.fkj.FkjFloorSearch;
import com.yx.face.model.entity.Visitee;

import java.util.Date;
import java.util.List;

/**
 * @author shisl
 * @package com.yx.face.service.fkj.impl
 * @class FkjService
 * @date 2023/7/28 下午9:32
 * @description
 */
public interface FkjService {


    /**
     * 根据sn获取账户区域配置信息
     *
     * @param sn
     * @return
     */
    FkjAreaInfoVo getAreaInfoBySn(String sn);


    /**
     * 添加或编辑受访人
     *
     * @param visiteeDto
     * @return
     */
    Boolean add(VisiteeDto visiteeDto);

    /**
     * 删除受访人
     *
     * @param ids
     * @return
     */
    Boolean del(List<Long> ids);

    /**
     * 获取分页列表
     *
     * @param pageDtoRestDTO
     * @return
     */
    PageInfo<Visitee> page(RestDTO<VisiteePageDto> pageDtoRestDTO);

    /**
     * 获取受访人列表
     *
     * @param sn
     * @return
     */
    List<VisiteeVo> list(String sn);

    /**
     * 设备心跳上报
     *
     * @param sn
     * @return
     */
    Boolean tracking(String sn);

    /**
     * 访客单机版二维码生成
     *
     * @param dto
     * @return
     */
    FkjEthQrCodeVo ethQrCode(FkjEthQrCodeDto dto);

    /**
     * 微光互联二维码生成
     *
     * @param dto
     * @return
     */
    FkjEthQrCodeVo vguangQrCode(FkjVguangQrCodeDto dto);


    /**
     * 访客二维码生成
     *
     * @param
     * @return
     */
    String qrCodeCreate(String floor,
                        Date timeStart, Date timeEnd,
                        Integer adminId, String adminName,
                        String idNumber, String name, String phone, String visiTeeXm);

    /**
     * 根据三要素获取访客二维码
     *
     * @param dto
     * @return
     */
    List<FkjQrCodeVo> visitorQrCode(FkjQrCodeDto dto);

    /**
     * 批量删除受访人
     *
     * @param pageDtoRestDTO
     * @return
     */
    Boolean batchDel(RestDTO<VisiteePageDto> pageDtoRestDTO);

    /**
     * 获取访客机楼层对应关系配置
     *
     * @param adminId
     * @return
     */
    List<FkjFloorInitConfig> floorConfigList(Integer adminId);

    /**
     * 添加访客机楼层对应关系配置
     *
     * @param dto
     * @param adminId
     */
    void floorConfigAdd(List<FkjFloorInitConfig> dto, Integer adminId);

    /**
     * 初始化访客机楼层关系
     *
     * @param adminId
     */
    void initFloorConfig(Integer adminId);

    /**
     * 访客机获取楼层列表
     *
     * @param search
     * @return
     */
    List<FkjFloorInitConfig> floorConfigList(FkjFloorSearch search);

//    /**
//     * 捷顺访客二维码生成
//     *
//     * @param dto
//     * @return
//     */
//    FkjEthQrCodeVo jslifeQrCode(FkjJshunQrCodeDto dto);

}
