package com.yx.face.service.fkj.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.uitls.Chinese2PinyinUtil;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.boot.uitls.IDCardEncryptionUtil;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.RandomUtils;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.AdminTagDao;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FkjFloorConfigDao;
import com.yx.face.dao.FkjHeartTrackingDao;
import com.yx.face.dao.FkjQrcodeDao;
import com.yx.face.dao.VisiteeDao;
import com.yx.face.dao.ZoneDeviceRDao;
import com.yx.face.model.api.fkj.dto.FkjEthQrCodeDto;
import com.yx.face.model.api.fkj.dto.FkjQrCodeDto;
import com.yx.face.model.api.fkj.dto.FkjVguangQrCodeDto;
import com.yx.face.model.api.fkj.dto.VisiteeDto;
import com.yx.face.model.api.fkj.dto.VisiteePageDto;
import com.yx.face.model.api.fkj.vo.FkjAreaInfoVo;
import com.yx.face.model.api.fkj.vo.FkjEthQrCodeVo;
import com.yx.face.model.api.fkj.vo.FkjQrCodeVo;
import com.yx.face.model.api.fkj.vo.VisiteeVo;
import com.yx.face.model.dto.fkj.FkjFloorInitConfig;
import com.yx.face.model.dto.fkj.FkjFloorSearch;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.AdminTag;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.FkjFloorConfig;
import com.yx.face.model.entity.FkjHeartTracking;
import com.yx.face.model.entity.FkjQrcode;
import com.yx.face.model.entity.Visitee;
import com.yx.face.model.entity.ZoneDeviceR;
import com.yx.face.model.enums.FkjQrcodeFloorEnum;
import com.yx.face.model.enums.FkjQrcodeFloorV2Enum;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.ZoneDeviceRService;
import com.yx.face.service.fkj.FkjService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;


/**
 * @author shisl
 * @package com.yx.face.service.fkj.impl
 * @class FkjServiceImpl
 * @date 2023/7/28 下午9:32
 * @description
 */
@Slf4j
@Service
public class FkjServiceImpl implements FkjService {

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private AdminDao adminDao;

    @Resource
    private AdminTagDao adminTagDao;

    @Resource
    private VisiteeDao visiteeDao;

    @Resource
    private FkjHeartTrackingDao fkjHeartTrackingDao;

    @Resource
    private FkjQrcodeDao fkjQrcodeDao;

    @Resource
    private Executor executor;

    @Resource
    private AdminService adminService;

    @Resource
    private ZoneDeviceRService zoneDeviceRService;

    @Resource
    private ZoneDeviceRDao zoneDeviceRDao;

    @Resource
    private FkjFloorConfigDao fkjFloorConfigDao;

//    @Value("${qrcode.jieshun.domain}")
//    private String domain;
//
//    @Value("${qrcode.jieshun.appId}")
//    private String appId;
//
//    @Value("${qrcode.jieshun.appId}")
//    private String secret;
//
//    @Value("${qrcode.jieshun.projectCode}")
//    private String projectCode;
//
//    @Value("${qrcode.jieshun.abilityCode}")
//    private String abilityCode;
//
//    @Value("${qrcode.jieshun.cid}")
//    private String cid;


    @Override
    public FkjAreaInfoVo getAreaInfoBySn(String sn) {
        FkjAreaInfoVo vo = new FkjAreaInfoVo();
        FaceDevice faceDevice = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, sn)).build());
        if (Preconditions.isBlank(faceDevice)) {
            throw new CommonException("设备数据不存在或已删除");
        }

        Admin admin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
        if (Preconditions.isBlank(admin)) {
            throw new CommonException("设备所在账号信息不存在或已删除");
        }

        vo.setProvinceName(admin.getProvinceName());
        vo.setAreaName(admin.getAreaName());
        vo.setCityName(admin.getCityName());
        vo.setFaceLogHoldDays(Preconditions.isNotBlank(admin.getFaceLogHoldDays()) ? admin.getFaceLogHoldDays().toString() : null);
        vo.setName(admin.getNickname());
        vo.setPhone(admin.getPhone());
        vo.setPoliceStationCode(admin.getPoliceStationCode());
        vo.setPoliceStationName(admin.getPoliceStationName());
        vo.setUsername(admin.getUsername());
        //获取场景名
        if (Preconditions.isNotBlank(admin.getTagId())) {
            AdminTag adminTag = adminTagDao.selectByPrimaryKey(admin.getTagId());
            vo.setTagName(Preconditions.isNotBlank(adminTag) ? adminTag.getName() : null);
        }
        return vo;
    }

    @Override
    public Boolean add(VisiteeDto visiteeDto) {
        if (Preconditions.isBlank(visiteeDto.getAdminId())) {
            throw new CommonException("adminId不能为空");
        }

        if (Preconditions.isBlank(visiteeDto.getFloor())) {
            throw new CommonException("楼层不能为空");
        }

        String zoneIds = visiteeDto.getZoneIds();
        if (Preconditions.isNotBlank(visiteeDto.getZoneNames())) {
            zoneIds = handleZoneIds(visiteeDto.getAdminId(), visiteeDto.getZoneNames());
        }

        FkjFloorConfig config = fkjFloorConfigDao.selectOneByExample(Example.builder(FkjFloorConfig.class).where(WeekendSqls.<FkjFloorConfig>custom().andEqualTo(FkjFloorConfig::getAdminId, visiteeDto.getAdminId())).build());
        if (Preconditions.isBlank(config) || Preconditions.isBlank(config.getFloorConfig())) {
            throw new CommonException("未配置【基本设置】-【支付宝小程序显示设置】-【梯控楼层关系表】");
        }

        List<FkjFloorInitConfig> configs = JSONObject.parseArray(config.getFloorConfig(), FkjFloorInitConfig.class);
        Map<String, List<FkjFloorInitConfig>> configMap = configs.stream().collect(Collectors.groupingBy(FkjFloorInitConfig::getActualityFloor));
        List<String> dtoFloors = Arrays.asList(visiteeDto.getFloor().split(","));

        checkFloors(dtoFloors, configMap);

        Visitee visitee = null;
        if (Preconditions.isNotBlank(visiteeDto.getId())) {
            visitee = visiteeDao.selectByPrimaryKey(visiteeDto.getId());
            if (Preconditions.isBlank(visitee)) {
                throw new CommonException("访客数据不存在或已删除");
            }

            visitee.setCompany(visiteeDto.getCompany());
            visitee.setFloor(visiteeDto.getFloor());
            visitee.setRoom(visiteeDto.getRoom());
            visitee.setShowTitle(visiteeDto.getShowTitle());
            visitee.setUnitName(visiteeDto.getUnitName());
            visitee.setZoneIds(zoneIds);
            if (Preconditions.isBlank(visitee.getIdNumber()) && Preconditions.isNotBlank(visiteeDto.getIdNumber())) {
//                if (!SMSOrIdCardUtils.isIdCard(visiteeDto.getIdNumber())) {
//                    throw new CommonException("身份证号格式有误");
//                }
                visitee.setIdNumber(visiteeDto.getIdNumber());
            }

            if (Preconditions.isBlank(visitee.getPhone()) && Preconditions.isNotBlank(visiteeDto.getPhone())) {
                if (!SMSOrIdCardUtils.isMobile(visiteeDto.getPhone())) {
                    throw new CommonException("手机号格式有误");
                }
                visitee.setPhone(visiteeDto.getPhone());
            }

            visitee.setName(visiteeDto.getName());
            visitee.setPinyinFirstLetter(Chinese2PinyinUtil.getPinyinFirstLetter(visiteeDto.getName()));
            visiteeDao.updateByPrimaryKey(visitee);
        } else {
//            if (Preconditions.isNotBlank(visiteeDto.getIdNumber()) && !SMSOrIdCardUtils.isIdCard(visiteeDto.getIdNumber())) {
//                throw new CommonException("身份证号格式有误");
//            }

            if (Preconditions.isNotBlank(visiteeDto.getPhone()) && !SMSOrIdCardUtils.isMobile(visiteeDto.getPhone())) {
                throw new CommonException("手机号格式有误");
            }
            visitee = BeanUtil.copyProperties(visiteeDto, Visitee.class);
            visitee.setPinyinFirstLetter(Chinese2PinyinUtil.getPinyinFirstLetter(visitee.getName()));
            visitee.setZoneIds(zoneIds);
            visitee.setCreateTime(new Date());
            visiteeDao.insert(visitee);
        }
        return Boolean.TRUE;
    }

    private void checkFloors(List<String> dtoFloors, Map<String, List<FkjFloorInitConfig>> configMap) {
        List<String> errorFloors = new ArrayList<>();
        for (String item : dtoFloors) {
            if (Preconditions.isBlank(configMap.get(item))) {
                errorFloors.add(item);
            }
        }
        if (Preconditions.isNotBlank(errorFloors)) {
            throw new CommonException("楼层【" + String.join(",", errorFloors) + "】在【梯控楼层关系表】中不存在");
        }
    }

    private String handleZoneIds(Integer adminId, String zoneNames) {
        if (Preconditions.isBlank(zoneNames)) {
            return null;
        }
        List<String> names = Arrays.asList(zoneNames.split(","));
        List<ZoneDeviceR> zoneDeviceRS = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom().andEqualTo(ZoneDeviceR::getAdminId, adminId).andIn(ZoneDeviceR::getName, names).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(zoneDeviceRS)) {
            return null;
        }
        List<Integer> zoneIds = zoneDeviceRS.stream().map(ZoneDeviceR::getId).collect(Collectors.toList());
        return JSONObject.toJSONString(zoneIds);
    }

    @Override
    public Boolean del(List<Long> ids) {
        List<Visitee> visitees = visiteeDao.selectByExample(Example.builder(Visitee.class).where(WeekendSqls.<Visitee>custom().andIn(Visitee::getId, ids)).build());
        if (Preconditions.isBlank(visitees)) {
            throw new CommonException("受访人数据不存在或已删除");
        }
        visitees.stream().forEach(item -> visiteeDao.delete(item));
        return Boolean.TRUE;
    }

    @Override
    public PageInfo<Visitee> page(RestDTO<VisiteePageDto> dto) {
        VisiteePageDto visiteePageDto = dto.getData();
        Admin loginAdmin = adminDao.selectByPrimaryKey(visiteePageDto.getAdminId());
        visiteePageDto.setAdminType(loginAdmin.getType());
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<Visitee> visitees = visiteeDao.list(visiteePageDto);
        if (Preconditions.isNotBlank(visitees)) {
            List<Integer> adminIds = visitees.stream().map(Visitee::getAdminId).collect(Collectors.toList());
            List<Admin> admins = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andIn(Admin::getId, adminIds)).build());
            Map<Integer, Admin> adminMap = admins.stream().collect(Collectors.toMap(Admin::getId, v -> v, (v1, v2) -> v1));
            visitees.stream().forEach(item -> {
                if (Preconditions.isNotBlank(item.getIdNumber()) && SMSOrIdCardUtils.isIdCard(item.getIdNumber())) {
                    item.setIdNumber(IDCardEncryptionUtil.idEncrypt(item.getIdNumber()));
                }

                if (Preconditions.isNotBlank(item.getName())) {
                    item.setName(SMSOrIdCardUtils.hideName(item.getName().trim()));
                }

                if (Preconditions.isNotBlank(item.getPhone()) && SMSOrIdCardUtils.isMobile(item.getPhone())) {
                    item.setPhone(SMSOrIdCardUtils.hidePhone(item.getPhone().trim()));
                }

                Admin admin = adminMap.get(item.getAdminId());
                if (Preconditions.isNotBlank(admin)) {
                    item.setUsername(admin.getUsername());
                }
                //补全区域
                completeZones(item);
            });
        }
        return new PageInfo<>(visitees);
    }

    private void completeZones(Visitee item) {
        String zoneIds = item.getZoneIds();
        if (Preconditions.isBlank(zoneIds)) {
            return;
        }
        List<Integer> ids = JSONArray.parseArray(zoneIds, Integer.class);
        if (Preconditions.isBlank(ids)) {
            return;
        }
        List<ZoneDeviceR> zoneDeviceRS = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom().andIn(ZoneDeviceR::getId, ids).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(zoneDeviceRS)) {
            return;
        }
        List<String> zoneNames = zoneDeviceRS.stream().map(ZoneDeviceR::getName).collect(Collectors.toList());
        item.setZoneNames(StringUtils.join(zoneNames, ","));

    }

    @Override
    public List<VisiteeVo> list(String sn) {
        FaceDevice faceDevice = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, sn)).build());
        if (Preconditions.isBlank(faceDevice)) {
            throw new CommonException("设备不存在或已删除");
        }
        Admin loginAdmin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
        //获取设备所在区域
        Integer zoneId = zoneDeviceRService.getZoneIdBySn(faceDevice.getAdminId(), sn);
        VisiteePageDto dto = new VisiteePageDto();
        dto.setAdminType(loginAdmin.getType());
        dto.setAdminId(faceDevice.getAdminId());
        dto.setZoneId(zoneId);
        dto.setSortByFloor(1);
        List<Visitee> visitees = visiteeDao.list(dto);
        return Preconditions.isNotBlank(visitees) ? BeanUtil.copyToList(visitees, VisiteeVo.class) : Collections.emptyList();
    }

    @Override
    public Boolean tracking(String sn) {
        executor.execute(() -> {
            FaceDevice faceDevice = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, sn)).build());
            if (Preconditions.isBlank(faceDevice)) {
                return;
            }
            if (Preconditions.isBlank(faceDevice.getOnline()) || faceDevice.getOnline().intValue() == 0) {
                faceDevice.setOnline(1);
                faceDeviceDao.updateDeviceOnline(faceDevice.getId(), 1);
            }

            try {
                FkjHeartTracking tracking = fkjHeartTrackingDao.selectOneByExample(Example.builder(FkjHeartTracking.class).where(WeekendSqls.<FkjHeartTracking>custom().andEqualTo(FkjHeartTracking::getSn, sn).andEqualTo(FkjHeartTracking::getAdminId, faceDevice.getAdminId())).build());
                if (Preconditions.isNotBlank(tracking)) {
                    tracking.setLastTrackingTime(new Date());
                    fkjHeartTrackingDao.updateByPrimaryKey(tracking);
                } else {
                    tracking = new FkjHeartTracking();
                    tracking.setSn(sn);
                    tracking.setCreateTime(new Date());
                    tracking.setLastTrackingTime(new Date());
                    tracking.setAdminId(faceDevice.getAdminId());
                    fkjHeartTrackingDao.insert(tracking);
                }
            } catch (Exception e) {
            }
        });
        return Boolean.TRUE;
    }

    @Override
    public String qrCodeCreate(String floor,
                               Date timeStart, Date timeEnd,
                               Integer adminId, String adminName,
                               String idNumber, String name, String phone, String visiTeeXm) {
        try {
            //单机版二维码
            String timeStartStr = DateUtil.format(timeStart, DateUtils.YYYYMMDDHHMMSS);
            String year_month_day_hour_start = " " + timeStartStr.substring(2, 4) + " " + timeStartStr.substring(4, 6) + " " + timeStartStr.substring(6, 8) + " " + timeStartStr.substring(8, 10);
            String timeEndStr = DateUtil.format(timeEnd, DateUtils.YYYYMMDDHHMMSS);
            String year_month_day_hour_end = " " + timeEndStr.substring(2, 4) + " " + timeEndStr.substring(4, 6) + " " + timeEndStr.substring(6, 8) + " " + timeEndStr.substring(8, 10);
            if (Preconditions.isNotBlank(floor)) {
                //TODO。。。获取数据库楼层配置
                List<Integer> dbFloor = getDbHexadecimalFloor(adminId, floor);
                String floorHexadecimal = Preconditions.isNotBlank(dbFloor) ? FkjQrcodeFloorV2Enum.getHexadecimalByFloor(dbFloor) : FkjQrcodeFloorEnum.getByFloor(Integer.valueOf(floor)).getHexadecimal();
                String hexadecimal = FkjEthQrCodeDto.hexadecimalPrefix
                        + FkjEthQrCodeDto.door
                        + FkjEthQrCodeDto.group
                        + FkjEthQrCodeDto.backups_start
                        + year_month_day_hour_start
                        + year_month_day_hour_end
                        + floorHexadecimal
                        + FkjEthQrCodeDto.backups_end;
                String ethQrCode = hexadecimal + " " + makeCheckSum(hexadecimal.replace(" ", ""));
                return handleEthQrCode(adminId, adminName, idNumber, name, visiTeeXm, phone, ethQrCode, timeStart, timeEnd);
            }
        } catch (Exception e) {
            log.error("楼层【{}】二维码生成失败", floor);
        }
        return null;
    }

    private List<Integer> getDbHexadecimalFloor(Integer adminId, String qFloor) {
        List<Integer> floors = new ArrayList<>();
        FkjFloorConfig floorConfig = fkjFloorConfigDao.selectOneByExample(Example.builder(FkjFloorConfig.class).where(WeekendSqls.<FkjFloorConfig>custom().andEqualTo(FkjFloorConfig::getAdminId, adminId)).build());
        if (Preconditions.isNotBlank(floorConfig) && Preconditions.isNotBlank(floorConfig.getFloorConfig())) {
            List<FkjFloorInitConfig> configs = JSONObject.parseArray(floorConfig.getFloorConfig(), FkjFloorInitConfig.class);
            Map<String, List<FkjFloorInitConfig>> configMap = configs.stream().collect(Collectors.groupingBy(FkjFloorInitConfig::getActualityFloor));
            List<String> floorsStr = Arrays.asList(qFloor.split(","));
            for (String item : floorsStr) {
                if (Preconditions.isNotBlank(configMap.get(item))) {
                    floors.add(configMap.get(item).get(0).getHexadecimalFloor());
                }
            }
        }
        return floors;
    }

    private String handleEthQrCode(Integer adminId, String adminName, String idNumber, String name, String visiTeeXm, String phone, String ethQrCode, Date timeStart, Date timeEnd) {
        try {
            FkjQrcode qrcode = fkjQrcodeDao.selectOneByExample(Example.builder(FkjQrcode.class).where(WeekendSqls.<FkjQrcode>custom().andEqualTo(FkjQrcode::getAdminId, adminId).andEqualTo(FkjQrcode::getIdNumber, idNumber)).build());
            if (Preconditions.isBlank(qrcode)) {
                qrcode = new FkjQrcode();
                qrcode.setAdminId(adminId);
                qrcode.setCreateTime(new Date());
            }
            qrcode.setEthQrcode(Preconditions.isNotBlank(ethQrCode) ? ethQrCode.replace(" ", "").trim() : null);
            qrcode.setAdminName(adminName);
            qrcode.setIdNumber(idNumber);
            qrcode.setName(name);
            qrcode.setVisiteeName(visiTeeXm);
            qrcode.setPhone(phone);
            qrcode.setStartTime(timeStart);
            qrcode.setEndTime(timeEnd);
            qrcode.setUpdateTime(new Date());
            if (Preconditions.isBlank(qrcode.getId())) {
                fkjQrcodeDao.insert(qrcode);
            } else {
                fkjQrcodeDao.updateByPrimaryKey(qrcode);
            }
        } catch (Exception e) {
            log.info("--->重复插入进入补偿逻辑");
            //补偿逻辑
            updateEthQrCode(adminId, adminName, idNumber, name, visiTeeXm, phone, ethQrCode, timeStart, timeEnd);
        }
        return Preconditions.isNotBlank(ethQrCode) ? ethQrCode.replace(" ", "").trim() : null;
    }

    private void updateEthQrCode(Integer adminId, String adminName, String idNumber, String name, String visiTeeXm, String phone, String ethQrCode, Date timeStart, Date timeEnd) {
        try {
            FkjQrcode qrcode = fkjQrcodeDao.selectOneByExample(Example.builder(FkjQrcode.class).where(WeekendSqls.<FkjQrcode>custom().andEqualTo(FkjQrcode::getAdminId, adminId).andEqualTo(FkjQrcode::getIdNumber, idNumber)).build());
            if (Preconditions.isBlank(qrcode)) {
                return;
            }
            qrcode.setEthQrcode(Preconditions.isNotBlank(ethQrCode) ? ethQrCode.replace(" ", "").trim() : null);
            qrcode.setAdminName(adminName);
            qrcode.setIdNumber(idNumber);
            qrcode.setName(name);
            qrcode.setVisiteeName(visiTeeXm);
            qrcode.setPhone(phone);
            qrcode.setStartTime(timeStart);
            qrcode.setEndTime(timeEnd);
            qrcode.setUpdateTime(new Date());
            fkjQrcodeDao.updateByPrimaryKey(qrcode);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void handleVguangQrCode(Integer adminId, String adminName, String idNumber, String name, String visiTeeXm, String phone, String vguangQrcode, Date timeStart, Date timeEnd) {
        try {
            FkjQrcode qrcode = fkjQrcodeDao.selectOneByExample(Example.builder(FkjQrcode.class).where(WeekendSqls.<FkjQrcode>custom().andEqualTo(FkjQrcode::getAdminId, adminId).andEqualTo(FkjQrcode::getIdNumber, idNumber)).build());
            if (Preconditions.isBlank(qrcode)) {
                qrcode = new FkjQrcode();
                qrcode.setAdminId(adminId);
                qrcode.setCreateTime(new Date());
            }
            qrcode.setVguangQrcode(vguangQrcode);
            qrcode.setAdminName(adminName);
            qrcode.setIdNumber(idNumber);
            qrcode.setName(name);
            qrcode.setVisiteeName(visiTeeXm);
            qrcode.setPhone(phone);
            qrcode.setStartTime(timeStart);
            qrcode.setEndTime(timeEnd);
            qrcode.setUpdateTime(new Date());
            if (Preconditions.isBlank(qrcode.getId())) {
                fkjQrcodeDao.insert(qrcode);
            } else {
                fkjQrcodeDao.updateByPrimaryKey(qrcode);
            }
        } catch (Exception e) {
            //补偿逻辑
            log.info("--->重复插入进入补偿逻辑");
            updateVguangQrCode(adminId, adminName, idNumber, name, visiTeeXm, phone, vguangQrcode, timeStart, timeEnd);
        }
    }

    private void updateVguangQrCode(Integer adminId, String adminName, String idNumber, String name, String visiTeeXm, String phone, String vguangQrcode, Date timeStart, Date timeEnd) {
        try {
            FkjQrcode qrcode = fkjQrcodeDao.selectOneByExample(Example.builder(FkjQrcode.class).where(WeekendSqls.<FkjQrcode>custom().andEqualTo(FkjQrcode::getAdminId, adminId).andEqualTo(FkjQrcode::getIdNumber, idNumber)).build());
            if (Preconditions.isBlank(qrcode)) {
                return;
            }
            qrcode.setAdminId(adminId);
            qrcode.setVguangQrcode(vguangQrcode);
            qrcode.setAdminName(adminName);
            qrcode.setIdNumber(idNumber);
            qrcode.setName(name);
            qrcode.setVisiteeName(visiTeeXm);
            qrcode.setPhone(phone);
            qrcode.setStartTime(timeStart);
            qrcode.setEndTime(timeEnd);
            qrcode.setUpdateTime(new Date());
            fkjQrcodeDao.updateByPrimaryKey(qrcode);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    @Override
    public List<FkjQrCodeVo> visitorQrCode(FkjQrCodeDto dto) {

        List<FkjQrcode> qrcodes = fkjQrcodeDao.selectByExample(Example.builder(FkjQrcode.class).where(WeekendSqls.<FkjQrcode>custom().andEqualTo(FkjQrcode::getIdNumber, dto.getIdNumber()).andGreaterThanOrEqualTo(FkjQrcode::getEndTime, new Date())).build());
        if (Preconditions.isBlank(qrcodes)) {
            return new ArrayList<>();
        }

        Map<Integer, Admin> adminMap = null;
        Admin admin = null;
        if (Preconditions.isNotBlank(dto.getAdminId())) {
            qrcodes = qrcodes.stream().filter(item -> item.getAdminId().intValue() == dto.getAdminId().intValue()).collect(Collectors.toList());
            admin = adminDao.selectByPrimaryKey(dto.getAdminId());
        } else {
            List<Integer> adminIds = qrcodes.stream().map(FkjQrcode::getAdminId).collect(Collectors.toList());
            List<Admin> admins = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andIn(Admin::getId, adminIds)).build());
            if (Preconditions.isNotBlank(admins)) {
                adminMap = admins.stream().collect(Collectors.toMap(Admin::getId, v -> v, (v1, v2) -> v1));
            }
        }

        List<FkjQrCodeVo> fkjQrCodeVos = BeanUtil.copyToList(qrcodes, FkjQrCodeVo.class);
        if (Preconditions.isNotBlank(fkjQrCodeVos)) {
            Admin finalAdmin = admin;
            Map<Integer, Admin> finalAdminMap = adminMap;
            fkjQrCodeVos.stream().forEach(item -> {
                item.setVguangQrcode(null);
                item.setZfbMiniEthTitle("梯控二维码");
                item.setZfbMiniVguangTitle("闸机二维码");
                item.setEthQrcode(Preconditions.isNotBlank(item.getEthQrcode()) ? item.getEthQrcode().replace(" ", "") : null);
                if (Preconditions.isNotBlank(dto.getAdminId())) {
                    if (Preconditions.isNotBlank(Preconditions.isNotBlank(finalAdmin))) {
                        item.setZfbMiniEthTitle(Preconditions.isNotBlank(finalAdmin.getZfbMiniEthTitle()) ? finalAdmin.getZfbMiniEthTitle() : "梯控二维码");
                        item.setZfbMiniVguangTitle(Preconditions.isNotBlank(finalAdmin.getZfbMiniVguangTitle()) ? finalAdmin.getZfbMiniVguangTitle() : "闸机二维码");
                    }
                } else if (Preconditions.isNotBlank(finalAdminMap) && Preconditions.isNotBlank(finalAdminMap.get(item.getAdminId()))) {
                    Admin admin1 = finalAdminMap.get(item.getAdminId());
                    item.setEthQrcode(Preconditions.isNotBlank(item.getEthQrcode()) ? item.getEthQrcode().replace(" ", "") : null);
                    item.setZfbMiniEthTitle(Preconditions.isNotBlank(admin1.getZfbMiniEthTitle()) ? admin1.getZfbMiniEthTitle() : "梯控二维码");
                    item.setZfbMiniVguangTitle(Preconditions.isNotBlank(admin1.getZfbMiniVguangTitle()) ? admin1.getZfbMiniVguangTitle() : "闸机二维码");
                }
            });
        }
        log.info("--->根据三要素获取访客二维码:{}", Preconditions.isNotBlank(fkjQrCodeVos) ? JSONObject.toJSONString(fkjQrCodeVos) : Collections.emptyList());
        return fkjQrCodeVos;
    }

    @Override
    public Boolean batchDel(RestDTO<VisiteePageDto> dto) {
        dto.setPageNum(0);
        dto.setPageSize(50000);
        PageInfo<Visitee> page = this.page(dto);
        List<Visitee> visitees = page.getList();
        if (Preconditions.isBlank(visitees)) {
            return true;
        }
        visitees.stream().forEach(item -> {
            visiteeDao.deleteByPrimaryKey(item.getId());
        });
        return Boolean.TRUE;
    }

    @Override
    public List<FkjFloorInitConfig> floorConfigList(Integer adminId) {
        FkjFloorConfig floorConfig = fkjFloorConfigDao.selectOneByExample(Example.builder(FkjFloorConfig.class).where(WeekendSqls.<FkjFloorConfig>custom().andEqualTo(FkjFloorConfig::getAdminId, adminId)).build());
        List<FkjFloorInitConfig> configs = (Preconditions.isBlank(floorConfig) || Preconditions.isBlank(floorConfig.getFloorConfig())) ? Collections.emptyList() : JSONObject.parseArray(floorConfig.getFloorConfig(), FkjFloorInitConfig.class);
        return configs;
    }

    @Override
    public void floorConfigAdd(List<FkjFloorInitConfig> dto, Integer adminId) {
        FkjFloorConfig floorConfig = fkjFloorConfigDao.selectOneByExample(Example.builder(FkjFloorConfig.class).where(WeekendSqls.<FkjFloorConfig>custom().andEqualTo(FkjFloorConfig::getAdminId, adminId)).build());
        if (Preconditions.isBlank(floorConfig)) {
            floorConfig = new FkjFloorConfig();
            floorConfig.setAdminId(adminId);
            floorConfig.setCreateTime(new Date());
            floorConfig.setUpdateTime(new Date());
            floorConfig.setFloorConfig(JSONObject.toJSONString(dto));
            fkjFloorConfigDao.insert(floorConfig);
        } else {
            floorConfig.setFloorConfig(JSONObject.toJSONString(dto));
            fkjFloorConfigDao.updateByPrimaryKey(floorConfig);
        }
    }

    @Override
    public void initFloorConfig(Integer adminId) {
        FkjFloorConfig floorConfig = fkjFloorConfigDao.selectOneByExample(Example.builder(FkjFloorConfig.class).where(WeekendSqls.<FkjFloorConfig>custom().andEqualTo(FkjFloorConfig::getAdminId, adminId)).build());
        if (Preconditions.isBlank(floorConfig)) {
            floorConfig = new FkjFloorConfig();
            floorConfig.setAdminId(adminId);
            floorConfig.setCreateTime(new Date());
            floorConfig.setUpdateTime(new Date());
            floorConfig.setFloorConfig(JSONObject.toJSONString(FkjFloorInitConfig.initFloorConfig()));
            fkjFloorConfigDao.insert(floorConfig);
        }
    }

    @Override
    public List<FkjFloorInitConfig> floorConfigList(FkjFloorSearch search) {
        FaceDevice device = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, search.getSn())).build());
        if (Preconditions.isBlank(device)) {
            throw new CommonException("设备不存在或已删除，请联系管理员");
        }
        FkjFloorConfig floorConfig = fkjFloorConfigDao.selectOneByExample(Example.builder(FkjFloorConfig.class).where(WeekendSqls.<FkjFloorConfig>custom().andEqualTo(FkjFloorConfig::getAdminId, device.getAdminId())).build());
        List<FkjFloorInitConfig> configs = (Preconditions.isBlank(floorConfig) || Preconditions.isBlank(floorConfig.getFloorConfig())) ? Collections.emptyList() : JSONObject.parseArray(floorConfig.getFloorConfig(), FkjFloorInitConfig.class);
        return configs;
    }

    @Override
    public FkjEthQrCodeVo ethQrCode(FkjEthQrCodeDto dto) {
        FaceDevice device = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, dto.getSn())).build());
        if (Preconditions.isBlank(device)) {
            throw new CommonException("设备不存在或已删除，请联系管理员");
        }

        Admin admin = adminDao.selectOneByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getId, device.getAdminId())).build());
        if (Preconditions.isBlank(admin)) {
            throw new CommonException("设备所属账户不存在或已停用，请联系管理员");
        }
        Date timeStart;
        Date timeEnd;
        BigDecimal userVisiterEffectiveTime = admin.getUserVisiterEffectiveTime();
        if (userVisiterEffectiveTime != null) {
            timeStart = DateUtil.date();
            int timelong = userVisiterEffectiveTime.multiply(new BigDecimal(1440)).intValue();
            timeEnd = DateUtils.addMinutes(new Date(), timelong);
        } else {
            timeStart = DateUtil.date();
            timeEnd = DateUtils.addDays(DateUtil.date(), 15);
        }

        String timeStartStr = DateUtil.format(timeStart, DateUtils.YYYYMMDDHHMMSS);
        String year_month_day_hour_start = " " + timeStartStr.substring(2, 4) + " " + timeStartStr.substring(4, 6) + " " + timeStartStr.substring(6, 8) + " " + timeStartStr.substring(8, 10);

        String timeEndStr = DateUtil.format(timeEnd, DateUtils.YYYYMMDDHHMMSS);
        String year_month_day_hour_end = " " + timeEndStr.substring(2, 4) + " " + timeEndStr.substring(4, 6) + " " + timeEndStr.substring(6, 8) + " " + timeEndStr.substring(8, 10);
        FkjEthQrCodeVo vo = new FkjEthQrCodeVo();
        try {
            //TODO。。。获取数据库楼层配置
            List<Integer> dbFloor = getDbHexadecimalFloor(admin.getId(), dto.getFloor());
            String floorHexadecimal = Preconditions.isNotBlank(dbFloor) ? FkjQrcodeFloorV2Enum.getHexadecimalByFloor(dbFloor) : FkjQrcodeFloorEnum.getByFloor(Integer.valueOf(dto.getFloor())).getHexadecimal();
            String hexadecimal = FkjEthQrCodeDto.hexadecimalPrefix
                    + FkjEthQrCodeDto.door
                    + FkjEthQrCodeDto.group
                    + FkjEthQrCodeDto.backups_start
                    + year_month_day_hour_start
                    + year_month_day_hour_end
                    + floorHexadecimal
                    + FkjEthQrCodeDto.backups_end;
            String qrCodeContent = hexadecimal + " " + makeCheckSum(hexadecimal.replace(" ", ""));
            vo.setQrCodeContent(Preconditions.isNotBlank(qrCodeContent) ? qrCodeContent.replace(" ", "") : null);
            //TODO 存储单机二维码
            handleNoStatusEthQrcode(vo.getQrCodeContent(), timeStart, timeEnd, admin, Boolean.TRUE);
        } catch (Exception e) {
            throw new CommonException("楼层【" + dto.getFloor() + "】二维码生成失败");
        }
        return vo;
    }

    private void handleNoStatusEthQrcode(String ethQrcode, Date timeStart, Date timeEnd, Admin admin, Boolean first) {
        try {
            List<FkjQrcode> qrcodes = fkjQrcodeDao.selectByExample(Example.builder(FkjQrcode.class).where(WeekendSqls.<FkjQrcode>custom().andEqualTo(FkjQrcode::getAdminId, admin.getId()).andEqualTo(FkjQrcode::getEthQrcode, ethQrcode).andIsNull(FkjQrcode::getVguangQrcode)).build());
            FkjQrcode qrcode = null;
            if (Preconditions.isBlank(qrcodes)) {
                qrcode = new FkjQrcode();
                qrcode.setAdminId(admin.getId());
                qrcode.setCreateTime(new Date());
            } else {
                qrcode = qrcodes.get(0);
            }
            qrcode.setEthQrcode(ethQrcode);
            qrcode.setAdminName(admin.getUsername());
            qrcode.setStartTime(timeStart);
            qrcode.setEndTime(timeEnd);
            qrcode.setUpdateTime(new Date());
            if (Preconditions.isBlank(qrcode.getId())) {
                fkjQrcodeDao.insert(qrcode);
            } else {
                fkjQrcodeDao.updateByPrimaryKey(qrcode);
            }
        } catch (Exception e) {
            e.printStackTrace();
            //TODO补偿一次
            if (first) {
                handleNoStatusEthQrcode(ethQrcode, timeStart, timeEnd, admin, Boolean.FALSE);
            }
        }
    }

    @Override
    public FkjEthQrCodeVo vguangQrCode(FkjVguangQrCodeDto dto) {
        FaceDevice device = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, dto.getSn())).build());
        if (Preconditions.isBlank(device)) {
            throw new CommonException("设备不存在或已删除，请联系管理员");
        }
        String idNumber = dto.getPersonCode();
        FkjEthQrCodeVo vo = new FkjEthQrCodeVo();
        try {
            AdminVO info = adminService.getInfo(device.getAdminId());
            Date timeStart;
            Date timeEnd;
            /*根据访客有效时间来进行下发*/
            if (info.getUserVisiterEffectiveTime() != null) {
                timeStart = DateUtil.date();
                int timelong = info.getUserVisiterEffectiveTime().multiply(new BigDecimal(1440)).intValue();
                timeEnd = DateUtils.addMinutes(new Date(), timelong);
            } else {
                timeStart = DateUtil.date();
                timeEnd = DateUtils.addDays(DateUtil.date(), 15);
            }
            String vguangQrCode = DigestUtil.md5Hex(RandomUtils.generateString(32));
            handleVguangQrCode(info.getAdminId(), info.getUsername(), idNumber, dto.getVisitorName(), null, dto.getVisitorTel(), vguangQrCode, timeStart, timeEnd);
            vo.setQrCodeContent(vguangQrCode);
        } catch (Exception e) {
            log.error("---->二维码生成失败");
        }
        return vo;
    }

//    @Override
//    public FkjEthQrCodeVo jslifeQrCode(FkjJshunQrCodeDto dto) {
//        FkjEthQrCodeVo vo = new FkjEthQrCodeVo();
//        FaceDevice device = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, dto.getSn())).build());
//        if (Preconditions.isBlank(device)) {
//            throw new CommonException("设备不存在或已删除，请联系管理员");
//        }
//
//        Admin admin = adminDao.selectOneByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getId, device.getAdminId())).build());
//        if (Preconditions.isBlank(admin)) {
//            throw new CommonException("设备所属账户不存在或已停用，请联系管理员");
//        }
//        Date timeStart;
//        Date timeEnd;
//        BigDecimal userVisiterEffectiveTime = admin.getUserVisiterEffectiveTime();
//        if (userVisiterEffectiveTime != null) {
//            timeStart = DateUtil.date();
//            int timelong = userVisiterEffectiveTime.multiply(new BigDecimal(1440)).intValue();
//            timeEnd = DateUtils.addMinutes(new Date(), timelong);
//        } else {
//            timeStart = DateUtil.date();
//            timeEnd = DateUtils.addDays(DateUtil.date(), 15);
//        }
//        Date date = new Date();
//        JSONObject req = new JSONObject();
//        req.put("app_id", appId);
//        req.put("method", "3c.visitor.invite");
//        req.put("sign_type", "md5");
//        req.put("timestamp", date.getTime());
//        req.put("charset", "utf-8");
//        req.put("format", "form");
//        req.put("projectCode", projectCode);
//        req.put("abilityCode", abilityCode);
//        JSONObject bizContent = new JSONObject();
//        bizContent.put("v", "2");
//        bizContent.put("cid", cid);
//        JSONObject p = new JSONObject();
//        p.put("serviceId", "3c.visitor.invite");
//        p.put("requestType", "DATA");
//        JSONObject attributes = new JSONObject();
//        attributes.put("personCode", SecureUtil.md5(dto.getPersonCode()));
//        attributes.put("areaCode", dto.getAreaCode());
//        attributes.put("visitorType", 3);
//        //商户编号
//        attributes.put("businesserCode", cid);
//        attributes.put("visitorContent", dto.getVisitorContent());
//        JSONObject timeDesc = new JSONObject();
//        //{\"sd\":\"2015-06-07 15:00:00\",\"ed\":\"2015-06-09 23:00:00\",\"st\":\"15:00\",\"et\":\"17:30\",\"w\":\"0000011\"}
//        timeDesc.put("sd", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, timeStart));
//        timeDesc.put("ed", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, timeEnd));
//        timeDesc.put("st", DateUtils.parseDateToStr(DateUtils.HH_MM, timeStart));
//        timeDesc.put("et", DateUtils.parseDateToStr(DateUtils.HH_MM, timeEnd));
//        String week = getWeek(DateUtils.parse(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, timeStart), DateUtils.YYYY_MM_DD), DateUtils.parse(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, timeEnd), DateUtils.YYYY_MM_DD));
//        timeDesc.put("w", week);
//        attributes.put("timeDesc", JSONObject.toJSONString(timeDesc));
//        p.put("attributes", attributes);
//        List<JSONObject> dataItems = new ArrayList<>();
//        JSONObject dataItem = new JSONObject();
//        dataItem.put("visitorName", dto.getVisitorName());
//        dataItem.put("cardNo", dto.getCarNo());
//        dataItem.put("visitorTel", Preconditions.isNotBlank(dto.getVisitorTel()) ? dto.getVisitorTel() : "13888888888");
//        dataItem.put("carNo", dto.getCarNo());
//        p.put("dataItems", dataItems);
//        bizContent.put("p", p);
//        req.put("biz_content", bizContent);
//        req.put("sign", getJieShunSign(req));
//        log.info("--->获取捷顺二维码请求入参:{}", JSONObject.toJSONString(req));
//        String sync = OkHttpUtils.builder().url(domain)
//                .addHeader("Content-Type", "application/json")
//                .postString(JSONObject.toJSONString(req))
//                .sync();
//        log.info("--->获取捷顺二维码服务返回参数:{}", JSONObject.toJSONString(sync));
//        FkjQrCodeJieShunVo shunVo = JSONObject.parseObject(sync, FkjQrCodeJieShunVo.class);
//        if (Preconditions.isNotBlank(shunVo.getResultCode()) && shunVo.getResultCode().intValue() == 0) {
//            if (Preconditions.isNotBlank(shunVo.getDataItems()) && Preconditions.isNotBlank(shunVo.getDataItems().get(0))) {
//                FkjQrCodeJieShunDataItemsVo itemsVo = shunVo.getDataItems().get(0);
//                if (Preconditions.isNotBlank(itemsVo) && Preconditions.isNotBlank(itemsVo.getAttributes())) {
//                    FkjQrCodeJieShunDataItemsAttributesVo attributesVo = itemsVo.getAttributes();
//                    vo.setQrCodeContent(attributesVo.getQrCodeContent());
//                    vo.setContent(attributesVo.getContent());
//                }
//            }
//        }
//        return vo;
//    }

//    /**
//     * 获取捷顺签名
//     *
//     * @return
//     */
//    private String getJieShunSign(JSONObject s) {
//        JSONObject object = JSONObject.parseObject(s.toJSONString(), Feature.OrderedField);
////        String sign = JieshunSigner.buildSignJson(object,secret);
//        return null;
//    }
//
//    private static String getWeek(Date startDate, Date endDate) {
//        StringBuffer wbuffer = new StringBuffer();
//        List<Integer> weeks = new ArrayList<>();
//        while (startDate.compareTo(endDate) <= 0) {
//            Calendar cal = Calendar.getInstance();
//            cal.setTime(startDate);
//            int dayOfWeek = cal.get(Calendar.DAY_OF_WEEK) - 1;
//            if (dayOfWeek == 0) {
//                dayOfWeek = 7;
//            }
//            if (Preconditions.isBlank(weeks) || !weeks.contains(dayOfWeek)) {
//                weeks.add(dayOfWeek);
//            }
//            cal.add(Calendar.DATE, 1);
//            startDate = cal.getTime();
//        }
//
//        wbuffer.append(weeks.contains(1) ? "1" : "0")
//                .append(weeks.contains(2) ? "1" : "0")
//                .append(weeks.contains(3) ? "1" : "0")
//                .append(weeks.contains(4) ? "1" : "0")
//                .append(weeks.contains(5) ? "1" : "0")
//                .append(weeks.contains(6) ? "1" : "0")
//                .append(weeks.contains(7) ? "1" : "0");
//        return wbuffer.toString();
//    }

//    public static void main(String[] args) {
//        //求和18
//        log.info("---->求和{}", makeCheckSum("A4 00 01 00 00 00 00 00 17 03 12 05 18 03 12 05 00 00 00 00 00 00 00 10 00 00 00 00 00 00 00".replace(" ", "")));
//    }

    /**
     * SUM 校验和 前面所有字节的和值(取低位)
     *
     * @param data
     * @return
     */
    public static String makeCheckSum(String data) {
        if (Preconditions.isBlank(data)) {
            return "00";
        }
        try {
            int total = 0;
            int len = data.length();
            int num = 0;
            while (num < len) {
                String s = data.substring(num, num + 2);
                total += Integer.parseInt(s, 16);
                num = num + 2;
            }
            //用256求余最大是255，即16进制的ff
            int mod = total % 256;
            String hex = Integer.toHexString(mod);
            len = hex.length();
            // 如果不够校验位的长度，补0,这里用的是两位校验
            if (len < 2) {
                hex = "0" + hex;
            }
            return hex;
        } catch (Exception e) {
        }
        return "00";
    }


}
