package com.yx.face.service.fkj.impl;

import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.dao.FkjQrcodeDao;
import com.yx.face.dao.ScanCodeBoxCodeDao;
import com.yx.face.dao.UserVisitorListDao;
import com.yx.face.model.entity.FkjQrcode;
import com.yx.face.model.entity.ScanCodeBoxCode;
import com.yx.face.service.fkj.ScanCodeBoxService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author shisl
 * @package com.yx.face.service
 * @class ScanCodeBoxServiceImpl
 * @date 2023/5/31 上午9:34
 * @description
 */
@Slf4j
@Service
public class ScanCodeBoxServiceImpl implements ScanCodeBoxService {

    @Resource
    private ScanCodeBoxCodeDao scanCodeBoxCodeDao;

    @Resource
    private FkjQrcodeDao fkjQrcodeDao;

    @Resource
    private UserVisitorListDao userVisitorListDao;


    @Override
    public Boolean dataCreate(String code) {
        try {
            List<FkjQrcode> qrcodes = fkjQrcodeDao.selectByExample(Example.builder(FkjQrcode.class).where(WeekendSqls.<FkjQrcode>custom().andEqualTo(FkjQrcode::getEthQrcode, code)).build());
            if (Preconditions.isNotBlank(qrcodes)) {
                updateScanCodeBoxCode(code, String.valueOf(qrcodes.get(0).getId()));
                return Boolean.TRUE;
            } else {
                delScanCodeBoxCode(code);
                return Boolean.FALSE;
            }
        } catch (Exception e) {
            delScanCodeBoxCode(code);
            log.error("--->扫码盒子创建数据服务异常");
            return Boolean.FALSE;
        }
    }

    private void delScanCodeBoxCode(String code) {
        try {
            List<ScanCodeBoxCode> scanCodeBoxCodes = scanCodeBoxCodeDao.selectByExample(Example.builder(ScanCodeBoxCode.class)
                    .where(WeekendSqls.<ScanCodeBoxCode>custom()
                            .andEqualTo(ScanCodeBoxCode::getCodeNum, code)).build());
            if (scanCodeBoxCodes != null && scanCodeBoxCodes.size() > 0) {
                ScanCodeBoxCode boxCode = scanCodeBoxCodes.get(0);
                scanCodeBoxCodeDao.delete(boxCode);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateScanCodeBoxCode(String code, String successId) {
        try {
            List<ScanCodeBoxCode> scanCodeBoxCodes = scanCodeBoxCodeDao.selectByExample(Example.builder(ScanCodeBoxCode.class)
                    .where(WeekendSqls.<ScanCodeBoxCode>custom()
                            .andEqualTo(ScanCodeBoxCode::getCodeNum, code)).build());
            if (scanCodeBoxCodes != null && scanCodeBoxCodes.size() > 0) {
                ScanCodeBoxCode boxCode = scanCodeBoxCodes.get(0);
                boxCode.setTransactionId(successId);
                boxCode.setCreateTime(new Date());
                scanCodeBoxCodeDao.updateByPrimaryKey(boxCode);
            } else {
                ScanCodeBoxCode box = new ScanCodeBoxCode();
                box.setTransactionId(successId);
                box.setCodeNum(code);
                box.setCreateTime(new Date());
                scanCodeBoxCodeDao.insert(box);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public Boolean addScanCodeBoxCode(String code) {
        try {
            ScanCodeBoxCode bCode = new ScanCodeBoxCode();
            bCode.setCodeNum(code);
            bCode.setCreateTime(new Date());
            scanCodeBoxCodeDao.insert(bCode);
        } catch (Exception e) {
            log.error("--->数据重复插入:{}", code);
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @Override
    public Boolean checkQrcodeExist(String code) {
        try {
            List<FkjQrcode> qrcodes = fkjQrcodeDao.selectByExample(Example.builder(FkjQrcode.class).where(WeekendSqls.<FkjQrcode>custom().andEqualTo(FkjQrcode::getEthQrcode, code)).build());
//            log.info("--->扫码盒子查询二维码数据接口响应参数:{}", JSONObject.toJSONString(qrcode));
            if (Preconditions.isBlank(qrcodes)) {
                log.error("--->扫码盒子查询二维码数据不存在:{}", code);
                return Boolean.FALSE;
            }
            FkjQrcode qrcode = qrcodes.get(0);
//            Integer adminId = qrcode.getAdminId();
//            String idNumber = qrcode.getIdNumber();
//            if (Preconditions.isNotBlank(adminId) && Preconditions.isNotBlank(idNumber)){
//                UserVisitorList visitor = userVisitorListDao.selectOneByExample(Example.builder(UserVisitorList.class).where(WeekendSqls.<UserVisitorList>custom().andEqualTo(UserVisitorList::getAdminId, adminId).andEqualTo(UserVisitorList::getIdNumber, idNumber)).build());
//                if (Preconditions.isBlank(visitor)) {
//                    log.error("--->扫码盒子查询二维码访客数据数据不存在:{}", idNumber);
//                    return Boolean.FALSE;
//                }
//            }

            Date startTime = qrcode.getStartTime();
            Date endTime = qrcode.getEndTime();
            Date now = new Date();
            if (now.before(startTime) || now.after(endTime)) {
                log.error("--->扫码盒子二维码数据已过期:{}", code);
                return Boolean.FALSE;
            }
            return Boolean.TRUE;
        } catch (Exception e) {
            delScanCodeBoxCode(code);
            log.error("--->扫码盒子查询二维码数据服务异常");
            return Boolean.FALSE;
        }

    }

    @Override
    public Boolean checkQrcodeUsed(String code) {
        List<ScanCodeBoxCode> scanCodeBoxCodes = scanCodeBoxCodeDao.selectByExample(Example.builder(ScanCodeBoxCode.class)
                .where(WeekendSqls.<ScanCodeBoxCode>custom()
                        .andEqualTo(ScanCodeBoxCode::getCodeNum, code)).build());
        if (scanCodeBoxCodes == null || scanCodeBoxCodes.size() == 0) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }
}
