package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.beust.jcommander.internal.Lists;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.easyexcel.CustomMergeStrategy;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.*;
import com.yx.face.model.dto.AdminDataDTO;
import com.yx.face.model.dto.AdminIDTypeDTO;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.enums.StatAgeTypeEnum;
import com.yx.face.model.enums.StatDataStatusEnum;
import com.yx.face.model.search.FaceServerLogSearch;
import com.yx.face.model.vo.AdminDataVO;
import com.yx.face.model.vo.AdminDeviceVO;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.FaceLogVO;
import com.yx.face.model.vo.stat.*;
import com.yx.face.service.AdminDataShowService;
import com.yx.face.service.AdminService;
import com.yx.face.service.server.CommonServer;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName AdminDataShowServiceImpl
 * @Author WXG
 * @Date 2021/11/12 13:45
 */
@Service
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AdminDataShowServiceImpl implements AdminDataShowService {

    @Resource
    private AdminDataShowDao adminDataShowDao;
    @Resource
    private FaceLogDao faceLogDao;
    @Autowired
    private AdminService adminService;
    @Autowired
    private RedisUtil redisUtil;
    @Resource
    private ShortTermFaceLogDao shortTermFaceLogDao;
    private final StatAgeDao statAgeDao;
    private final StatDataDao statDataDao;
    private final StatDataItemDao statDataItemDao;
    private final StatPeopleDao statPeopleDao;
    private final StatSexDao statSexDao;
    private final StatSourceDao statSourceDao;
    private final AreaCodeDao areaCodeDao;
    private final CommonServer commonServer;
    private String RName = "adminData" ;

    @Override
    public AdminDeviceVO getDeviceNumber() {
        Integer deviceAllNum =0;
        Integer deviceOnlineNum =0;
        Integer deviceOfflineNum =0;

        AdminVO adminVO = adminService.showDataGetInfo();
        Map<String,Object> maps = new HashMap<>();
        maps.put("adminVO",adminVO);
        List<FaceDevice> list = adminDataShowDao.getDeviceNumber(maps);
        if(!list.isEmpty()){
            deviceAllNum=list.size();
            for (FaceDevice faceDevice : list) {
                if(faceDevice.getOnline()==1) {
                    deviceOnlineNum++;
                }
            }
        }
        deviceOfflineNum = deviceAllNum -deviceOnlineNum;
        AdminDeviceVO adminDeviceVO = new AdminDeviceVO();
        adminDeviceVO.setDeviceAllNum(deviceAllNum);
        adminDeviceVO.setDeviceOnlineNum(deviceOnlineNum);
        adminDeviceVO.setDeviceOfflineNum(deviceOfflineNum);

        return adminDeviceVO;
    }

    @Override
    public AdminDataVO getAdminData() {
        AdminDataVO adminDataVO = new AdminDataVO();
        AdminVO adminVO = adminService.showDataGetInfo();
        Integer totalNum = 0;//通行总人数
        Integer todayNum = 0;//今日通行总人数
        Integer redNum = 0;//红码人数
        Integer yellowNum = 0;//黄码人数
        Integer greenNum = 0;//绿码人数
        Integer notKnowNum = 0;//健康码未知人数
        Integer twExceptionNum = 0;//温度异常人数
        Integer twNormal = 0;//温度正常人数
        Integer contactRedNum = 0;//密接红码人员人数
        Integer contactYellowNum = 0;//密接黄码人员人数
        Integer exceptionNum = 0;//异常人员人数
        Integer inoculationNum = 0;//已接种疫苗人数
        Integer noInoculationNum = 0;//未接种疫苗人数
        Integer nucleicNum = 0;//七日内完成核酸检测人数

        /*
        * 1.获取所有属于该账号的adminId(adminList)
        * 2.循环adminList根据每一条的adminId到redis库中查询数据
        * 3.查询到的数据查看有效时间
        *   1.  在有效期内
        *   2.  在有效期外 则重新查询数据，并存入到redis库中
        * 4.累加数据
        * 5.放入数据返回
        *
        * */
        //1.获取adminList
        Map<String,Object> maps = new HashMap<>();
        maps.put("adminVO",adminVO);
        List<Integer> adminList = adminDataShowDao.getAdminList(maps);
        Date date = new Date();
        for (Integer adminId : adminList) {
            AdminData adminData = (AdminData)redisUtil.get(RName + adminId);
            if(adminData==null){
                adminData = getAdminDataByAdminId(adminId,null);
            }else{
                Date effectiveTime = adminData.getEffectiveTime();
                if(effectiveTime.getTime()<date.getTime()){
                    adminData = getAdminDataByAdminId(adminId,adminData);
                }
            }
            totalNum+=adminData.getTotalNum();
            todayNum+=adminData.getTodayNum();
            redNum+=adminData.getRedNum();
            yellowNum+=adminData.getYellowNum();
            greenNum+=adminData.getGreenNum();
            notKnowNum+=adminData.getNotKnowNum();
            twExceptionNum+=adminData.getTwExceptionNum();
            twNormal+=adminData.getTwNormal();
            contactRedNum+=adminData.getContactRedNum();
            contactYellowNum+=adminData.getContactYellowNum();
            exceptionNum+=adminData.getExceptionNum();
            inoculationNum+=adminData.getInoculationNum();
            noInoculationNum+=adminData.getNoInoculationNum();
            nucleicNum+=adminData.getNucleicNum();
        }
        adminDataVO.setTotalNum(totalNum);
        adminDataVO.setTodayNum(todayNum);
        adminDataVO.setRedNum(redNum);
        adminDataVO.setYellowNum(yellowNum);
        adminDataVO.setGreenNum(greenNum);
        adminDataVO.setNotKnowNum(notKnowNum);
        adminDataVO.setTwExceptionNum(twExceptionNum);
        adminDataVO.setTwNormal(twNormal);
        adminDataVO.setContactRedNum(contactRedNum);
        adminDataVO.setContactYellowNum(contactYellowNum);
        adminDataVO.setExceptionNum(exceptionNum);
        adminDataVO.setInoculationNum(inoculationNum);
        adminDataVO.setNoInoculationNum(noInoculationNum);
        adminDataVO.setNucleicNum(nucleicNum);
        return adminDataVO;
    }
    private AdminData getAdminDataByAdminId(Integer adminId,AdminData adminData){
        AdminVO info = adminService.showDataGetInfo(adminId);

        Integer totalNum = 0;//通行总人数
        Integer todayNum = 0;//今日通行总人数
        Integer redNum = 0;//红码人数
        Integer yellowNum = 0;//黄码人数
        Integer greenNum = 0;//绿码人数
        Integer notKnowNum = 0;//健康码未知人数
        Integer twExceptionNum = 0;//温度异常人数
        Integer twNormal = 0;//温度正常人数
        Integer contactRedNum = 0;//密接红码人员人数
        Integer contactYellowNum = 0;//密接黄码人员人数
        Integer exceptionNum = 0;//异常人员人数
        Integer inoculationNum = 0;//已接种疫苗人数
        Integer noInoculationNum = 0;//未接种疫苗人数
        Integer nucleicNum = 0;//七日内完成核酸检测人数
        Integer biggerNum = 0;
        List<FaceLogVO> faceLogs = new ArrayList<>();

        FaceServerLogSearch data =null;
        Map<String, Object> map = new HashMap<>();
        map.put("adminVo", info);

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat sdfToday = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        long parse = 0;
        try {
            parse = sdfToday.parse(sdfToday.format(date)).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        String todayTime = sdf.format(new Date(parse));//当天0点
        map.put("todayTime", todayTime);

        if(adminData!=null){
            totalNum = adminData.getTotalNum();//通行总人数
            todayNum = adminData.getTodayNum();//今日通行总人数
            redNum = adminData.getRedNum();//红码人数
            yellowNum = adminData.getYellowNum();//黄码人数
            greenNum = adminData.getGreenNum();//绿码人数
            notKnowNum = adminData.getNotKnowNum();//健康码未知人数
            twExceptionNum = adminData.getTwExceptionNum();//温度异常人数
            twNormal = adminData.getTwNormal();//温度正常人数
            contactRedNum = adminData.getContactRedNum();//密接红码人员人数
            contactYellowNum = adminData.getContactYellowNum();//密接黄码人员人数
            exceptionNum = adminData.getExceptionNum();//异常人员人数
            inoculationNum = adminData.getInoculationNum();//已接种疫苗人数
            noInoculationNum = adminData.getNoInoculationNum();//未接种疫苗人数
            nucleicNum = adminData.getNucleicNum();//七日内完成核酸检测人数
            biggerNum = adminData.getBiggerNum();
            map.put("biggerNum", biggerNum);
            data = getFaceServerLogSearch();

        }else {
            Date d1 = new Date();
            totalNum = faceLogDao.getCount(map);
            data = new FaceServerLogSearch();
            adminData = new AdminData();
            Date d2 = new Date();
            log.info("所耗时长,{}",d2.getTime()- d1.getTime());
        }
        map.put("query", data);

        faceLogs = faceLogDao.getFaceLogsShowData(map);
        totalNum+=faceLogs.size();


        for (FaceLogVO faceLog : faceLogs) {
            if(faceLog.getLogId().intValue() > biggerNum){
                biggerNum = faceLog.getLogId().intValue();
            }
            Long parse1 = null;
            try {
                 parse1 = sdf.parse(faceLog.getFaceTime()).getTime();
            } catch (ParseException e) {
                e.printStackTrace();
            }
            if(parse1>parse){
                todayNum++;
                //红马及其接触人数
                if (faceLog.getJkmStatus().equals("10")) {
                    redNum++;
//                    //获取前后5分钟内的密切接触红马人员
//                    long createTime = parse1;
//                    String startTime = sdf.format(new Date(createTime - (5 * 60 * 1000)-(8*60*60*1000)));
//                    String endTime = sdf.format(new Date(createTime + (5 * 60 * 1000)-(8*60*60*1000)));
//                    List<Long> cbs = adminDataShowDao.getCountByTimeAndSn(startTime, endTime, faceLog.getDeviceSn(),faceLog.getLogId());
//                    for (Long cb : cbs) {
//                        red.add(cb);
//                    }
                }
                //黄马及其接触人数
                else if (faceLog.getJkmStatus().equals("01")){
                    yellowNum++;
//                    //获取前后5分钟内的密切接触黄马人员
//                    long createTime = parse1;
//                    String startTime = sdf.format(new Date(createTime - (5 * 60 * 1000)-(8*60*60*1000)));
//                    String endTime = sdf.format(new Date(createTime + (5 * 60 * 1000)-(8*60*60*1000)));
//                    List<Long> cbs = adminDataShowDao.getCountByTimeAndSn(startTime, endTime, faceLog.getDeviceSn(),faceLog.getLogId());
//                    for (Long cb : cbs) {
//                        yellow.add(cb);
//                    }
                }
                else if (faceLog.getJkmStatus().equals("00")) {
                    greenNum++;
                } else {
                    notKnowNum++;
                }
                if (faceLog.getTwStatus().equals("1")) {
                    twExceptionNum++;
                } else {
                    twNormal++;
                }
                if (faceLog.getYmStatus()!=null && !faceLog.getYmStatus().equals("")){
                    inoculationNum++;
                }else {
                    noInoculationNum++;
                }
                if (faceLog.getHsStatus()!=null && !faceLog.getHsStatus().equals("")){
                    nucleicNum++;
                }
            }

        }

        adminData.setAdminId(adminId);
        adminData.setTotalNum(totalNum);
        adminData.setTodayNum(todayNum);
        adminData.setRedNum(redNum);
        adminData.setYellowNum(yellowNum);
        adminData.setGreenNum(greenNum);
        adminData.setNotKnowNum(notKnowNum);
        adminData.setTwExceptionNum(twExceptionNum);
        adminData.setTwNormal(twNormal);
        adminData.setContactRedNum(contactRedNum);
        adminData.setContactYellowNum(contactYellowNum);
//        adminData.setContactRedNum(contactRedNum+red.size());
//        adminData.setContactYellowNum(contactYellowNum+yellow.size());
        adminData.setExceptionNum(exceptionNum);
        adminData.setInoculationNum(inoculationNum);
        adminData.setNoInoculationNum(noInoculationNum);
        adminData.setNucleicNum(nucleicNum);
        adminData.setEffectiveTime(new Date(date.getTime() + (60 * 1000)));
        adminData.setTime(date);
        adminData.setBiggerNum(biggerNum);

        /*
        * 如果红码和黄码人数不为0则需要重新统计密切接触人数
        * */
        if(yellowNum != 0){
            LinkedHashSet<Long> yellowredContact = new LinkedHashSet<>();
            /*获取黄码*/
            String zero = getZero();//获取当天的0点；
            List<FaceLogVO> faceLogVOList = faceLogDao.getYellow(adminId, zero, "01");
            for (FaceLogVO faceLogVO : faceLogVOList) {
                Long faceTime=null;
                try {
                     faceTime = sdf.parse(faceLogVO.getFaceTime()).getTime();
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                String startTime = sdf.format(new Date(faceTime - (5 * 60 * 1000)));
                String endTime = sdf.format(new Date(faceTime + (5 * 60 * 1000)));
                List<Long> cbs = adminDataShowDao.getCountByTimeAndSn(startTime, endTime, faceLogVO.getDeviceSn(),faceLogVO.getLogId());
                for (Long cb : cbs) {
                    yellowredContact.add(cb);
                }
            }
            adminData.setContactYellowNum(yellowredContact.size());
        }
        if(redNum != 0){
            LinkedHashSet<Long> redContact = new LinkedHashSet<>();
            /*获取黄码*/
            String zero = getZero();//获取当天的0点；
            List<FaceLogVO> faceLogVOList = faceLogDao.getYellow(adminId, zero, "10");
            for (FaceLogVO faceLogVO : faceLogVOList) {
                Long faceTime=null;
                try {
                    faceTime = sdf.parse(faceLogVO.getFaceTime()).getTime();
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                String startTime = sdf.format(new Date(faceTime - (5 * 60 * 1000)));
                String endTime = sdf.format(new Date(faceTime + (5 * 60 * 1000)));
                List<Long> cbs = adminDataShowDao.getCountByTimeAndSn(startTime, endTime, faceLogVO.getDeviceSn(),faceLogVO.getLogId());
                for (Long cb : cbs) {
                    redContact.add(cb);
                }
            }
            adminData.setContactRedNum(redContact.size());
        }
        //将数据存入redis
        redisUtil.set(RName + adminId,adminData);
        return adminData;
    }
    private String getZero(){
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        Date zero = calendar.getTime();
        return sdf.format(zero);
    }

    private FaceServerLogSearch getFaceServerLogSearch() {
        FaceServerLogSearch data = new FaceServerLogSearch();

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        Date zero = calendar.getTime();

        calendar.add(5,1);
        Date one = calendar.getTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        data.setStartTime(sdf.format(zero));
        data.setEndTime(sdf.format(one));

        return data;
    }

    @Override
    public AdminDataVO getGloablePassData() {
        AdminDataVO resultData = new AdminDataVO();
        AdminVO adminInfo = adminService.getInfo();
        Integer todayNum = 0;
        Integer redNum = 0;
        Integer yellowNum = 0;
        Integer greenNum = 0;
        Integer notKnowNum = 0;
        Integer twExceptionNum = 0;
        Integer twNormal = 0;
        Integer exceptionNum = 0;
        Integer inoculationNum = 0;
        Integer noInoculationNum = 0;
        Integer nucleicNum = 0;
        List<AdminIDTypeDTO> adminIDTypeDTOListList = getAdminIdList(adminInfo);
        if (CollectionUtil.isEmpty(adminIDTypeDTOListList)) {
            return resultData;
        }
        List<Integer> adminIdList = adminIDTypeDTOListList.stream().map(AdminIDTypeDTO::getAdminId).collect(Collectors.toList());
        String startDate = DateUtil.format(Convert.toDate(DateUtil.today()), DatePattern.NORM_DATETIME_PATTERN);
        String endDate = DateUtil.format(DateUtil.endOfDay(Convert.toDate(DateUtil.today())), DatePattern.NORM_DATETIME_PATTERN);
        List<FaceLogVO> faceLogList = faceLogDao.getFaceLogByAdminIdAndFaceTime(adminIdList, null, startDate, endDate);
        for (AdminIDTypeDTO adminIDTypeDTO : adminIDTypeDTOListList) {
            List<FaceLogVO> filterAdminFaceLogList = faceLogList.stream().filter(e -> e.getAdminId().equals(adminIDTypeDTO.getAdminId())).collect(Collectors.toList());
            for (FaceLogVO faceLog : filterAdminFaceLogList) {
                if (faceLog.getJkmStatus().equals("10")) {
                    redNum++;
                } else if (faceLog.getJkmStatus().equals("01")) {
                    yellowNum++;
                } else if (faceLog.getJkmStatus().equals("00")) {
                    greenNum++;
                } else {
                    notKnowNum++;
                }
                if (faceLog.getTwStatus().equals("1")) {
                    twExceptionNum++;
                } else {
                    twNormal++;
                }
                if (faceLog.getYmStatus() != null && !faceLog.getYmStatus().equals("")) {
                    inoculationNum++;
                } else {
                    noInoculationNum++;
                }
                if (faceLog.getHsStatus() != null && !faceLog.getHsStatus().equals("")) {
                    nucleicNum++;
                }
            }
            todayNum += filterAdminFaceLogList.size();
        }
//        Set<Long> concactYellowSet = shortTermFaceLogDao.selectConcactHealthCodeFaces(HealthCodeStatusEnum.YELLOW_CODE.getCode(),adminIdList,null,startDate,endDate);
//        Set<Long> concactRedSet = shortTermFaceLogDao.selectConcactHealthCodeFaces(HealthCodeStatusEnum.RED_CODE.getCode(),adminIdList,null,startDate,endDate);
        Integer totalNum = faceLogDao.count(adminIdList);

        resultData.setTotalNum(totalNum).setTodayNum(todayNum).setRedNum(redNum).setYellowNum(yellowNum)
                .setGreenNum(greenNum).setNotKnowNum(notKnowNum).setTwExceptionNum(twExceptionNum).setTwNormal(twNormal)
                .setContactRedNum(0).setContactYellowNum(0).setExceptionNum(exceptionNum)
                .setInoculationNum(inoculationNum).setNucleicNum(nucleicNum).setNoInoculationNum(noInoculationNum);
        return resultData;
    }



    public List<AdminIDTypeDTO> getAdminIdList(AdminVO adminInfo) {
        List<AdminIDTypeDTO> adminIDTypeLists = Lists.newArrayList();
        switch (AdminTypeEnum.getAdminTypeEnumByCode(adminInfo.getType())) {
            case ADMINISTRATOR:
                adminIDTypeLists = adminDataShowDao.selectIdAndTypeList(adminInfo);
                break;
            case PROVINCIAL_MASTER_ACCOUNT:
                adminIDTypeLists = adminDataShowDao.selectIdAndTypeList(adminInfo);
                break;
            case PROVINCIAL_LEVEL:
                adminIDTypeLists = adminDataShowDao.selectIdAndTypeList(adminInfo);
                break;
            case MUNICIPAL:
                adminIDTypeLists = adminDataShowDao.selectIdAndTypeList(adminInfo);
                break;
            case DISTRICT:
                AdminIDTypeDTO adminIDTypeDTO = new AdminIDTypeDTO();
                adminIDTypeDTO.setType(adminInfo.getType());
                adminIDTypeDTO.setAdminId(adminInfo.getAdminId());
                adminIDTypeLists.add(adminIDTypeDTO);
                break;
            case ENTERPRISE:
                return adminIDTypeLists;
            default:
                break;
        }
        return adminIDTypeLists;
    }


    @Override
    public StatPassVO statPass(AdminDataDTO dto) {
        StatPassVO statPassVO = new StatPassVO();
        LocalDate statDate = dto.getStatDate();

        List<Integer> belowAdminIdList = commonServer.getBelowAdminIdList();

        //1.进出人次
        for (Integer adminId : belowAdminIdList) {
            String allNumkey = Constants.STAT_ALL_NUM + adminId + "::" + statDate;
            if(redisUtil.hasKey(allNumkey)){
                statPassVO.setAllNum(statPassVO.getAllNum() + redisUtil.get(allNumkey,Integer.class));
            }
        }
        //2.进场人数
        for (Integer adminId : belowAdminIdList) {
            String inNumKey = Constants.STAT_IN_NUM + adminId + "::" + statDate;
            if(redisUtil.hasKey(inNumKey)){
                statPassVO.setInNum(statPassVO.getInNum() + redisUtil.get(inNumKey,Integer.class));
            }
        }

        //3.在场人数
        if(CollUtil.isEmpty(belowAdminIdList)){
            return statPassVO;
        }
        LocalDate now = LocalDate.now();
        if(now.isEqual(statDate)){
            DateTime date = DateUtil.date();
            Long count = statDataDao.selectCount(
                    Wrappers.<StatData>query()
                            .select("DISTINCT id_number")
                            .lambda()
                            .in(StatData::getAdminId, belowAdminIdList)
                            .eq(StatData::getStatus, StatDataStatusEnum.IN.getCode())

            );
            statPassVO.setAtNum(count);
        }
        return statPassVO;
    }

    @Override
    public StatAgeVO statAge(AdminDataDTO dto) {
        StatAgeVO statAgeVO = new StatAgeVO();
        LocalDate statDate = dto.getStatDate();

        List<Integer> belowAdminIdList = commonServer.getBelowAdminIdList();
        if(CollUtil.isEmpty(belowAdminIdList)){
            return statAgeVO;
        }

        List<StatAge> statAgeList = statAgeDao.selectList(
                Wrappers.<StatAge>lambdaQuery()
                        .in(StatAge::getAdminId, belowAdminIdList)
                        .eq(StatAge::getStatDate, statDate)

        );
        for (StatAge statAge : statAgeList) {
            StatAgeTypeEnum statAgeTypeEnum = StatAgeTypeEnum.getEnumByCode(statAge.getAgeType());
            if(statAgeTypeEnum == null){
                continue;
            }
            switch (statAgeTypeEnum){
                case AGE_20:statAgeVO.setAge20(statAgeVO.getAge20() + statAge.getStatNum());break;
                case AGE_35:statAgeVO.setAge35(statAgeVO.getAge35() + statAge.getStatNum());break;
                case AGE_50:statAgeVO.setAge50(statAgeVO.getAge50() + statAge.getStatNum());break;
                case AGE_65:statAgeVO.setAge65(statAgeVO.getAge65() + statAge.getStatNum());break;
                case AGE_OTHER:statAgeVO.setAgeOther(statAgeVO.getAgeOther() + statAge.getStatNum());break;
            }
        }
        return statAgeVO;
    }

    @Override
    public StatSexVO statSex(AdminDataDTO dto) {
        StatSexVO statSexVO = new StatSexVO();
        LocalDate statDate = dto.getStatDate();

        List<Integer> belowAdminIdList = commonServer.getBelowAdminIdList();
        if(CollUtil.isEmpty(belowAdminIdList)){
            return statSexVO;
        }

        List<StatSex> statSexList = statSexDao.selectList(
                Wrappers.<StatSex>lambdaQuery()
                        .in(StatSex::getAdminId, belowAdminIdList)
                        .eq(StatSex::getStatDate, statDate)
        );
        for (StatSex statSex : statSexList) {
            if(statSex.getSexType() == 0){
                statSexVO.setFemale(statSexVO.getFemale() + statSex.getStatNum());
            }else if(statSex.getSexType() == 1){
                statSexVO.setMale(statSexVO.getMale() + statSex.getStatNum());
            }
        }
        return statSexVO;
    }

    @Override
    public StatPeopleVO statPeople(AdminDataDTO dto) {
        StatPeopleVO statPeopleVO = new StatPeopleVO();
        LocalDate statDate = dto.getStatDate();

        List<Integer> belowAdminIdList = commonServer.getBelowAdminIdList();
        if(CollUtil.isEmpty(belowAdminIdList)){
            return statPeopleVO;
        }

        List<StatPeople> statPeopleList = statPeopleDao.selectList(
                Wrappers.<StatPeople>lambdaQuery()
                        .in(StatPeople::getAdminId, belowAdminIdList)
                        .eq(StatPeople::getStatDate, statDate)
                        .orderByAsc(StatPeople::getStatTime)
        );
        Map<LocalTime, List<StatPeople>> timeListMap = statPeopleList.stream().collect(Collectors.groupingBy(StatPeople::getStatTime));
        List<StatPeopleItemVO> list = new ArrayList<>(timeListMap.size());
        for (LocalTime localTime : timeListMap.keySet()) {
            list.add(new StatPeopleItemVO(timeListMap.get(localTime).stream().mapToInt(StatPeople::getStatNum).sum(),localTime));
        }
        List<StatPeopleItemVO> itemVOList = list.stream().sorted(Comparator.comparing(StatPeopleItemVO::getStatTime)).collect(Collectors.toList());
        statPeopleVO.setTimeList(itemVOList.stream().map(StatPeopleItemVO::getStatTime).collect(Collectors.toList()));
        statPeopleVO.setNumList(itemVOList.stream().map(StatPeopleItemVO::getStatNum).collect(Collectors.toList()));

        return statPeopleVO;
    }

    @Override
    public List<StatSourceVO> statSource(AdminDataDTO dto) {
        List<StatSourceVO> list = new ArrayList<>();

        LocalDate statDate = dto.getStatDate();

        List<Integer> belowAdminIdList = commonServer.getBelowAdminIdList();
        if(CollUtil.isEmpty(belowAdminIdList)){
            return new LinkedList<>();
        }

        List<StatSource> statPeopleList = statSourceDao.selectList(
                Wrappers.<StatSource>lambdaQuery()
                        .in(StatSource::getAdminId, belowAdminIdList)
                        .eq(StatSource::getStatDate, statDate)
                        .orderByDesc(StatSource::getStatNum)
        );
        Map<String,String> areaCodeMap = this.getAreaCodeMap(statPeopleList);

        Map<String, List<StatSource>> statPeopleMap = statPeopleList.stream().collect(Collectors.groupingBy(StatSource::getStatSourceCode));

        for (String statSourceCode : statPeopleMap.keySet()) {
            if(!areaCodeMap.containsKey(statSourceCode)){
                continue;
            }
            List<StatSource> statSourceList = statPeopleMap.get(statSourceCode);
            StatSourceVO statSourceVO = new StatSourceVO();
            statSourceVO.setProvinceName(areaCodeMap.get(statSourceCode));
            statSourceVO.setStatNum(statSourceList.stream().mapToInt(StatSource::getStatNum).sum());
            list.add(statSourceVO);
        }

        return list.stream().sorted(Comparator.comparing(StatSourceVO::getStatNum).reversed()).collect(Collectors.toList());
    }
    private Map<String, String> getAreaCodeMap(List<StatSource> statPeopleList) {
        if(CollUtil.isEmpty(statPeopleList)){
            return new HashMap<>();
        }
        List<String> statSourceCodeList = statPeopleList.stream().map(StatSource::getStatSourceCode).collect(Collectors.toList());
        if(CollUtil.isEmpty(statSourceCodeList)){
            return new HashMap<>();
        }
        List<AreaCode> areaCodeList = areaCodeDao.selectList(
                Wrappers.<AreaCode>lambdaQuery()
                        .in(AreaCode::getCode, statSourceCodeList)
        );
        return areaCodeList.stream().collect(Collectors.toMap(AreaCode::getCode, AreaCode::getName));
    }
    @Override
    public IPage<StatDataVO> pageList(RestDTO<StatDataSearch> dto) {
        AdminVO info = adminService.getInfo();

        LambdaQueryWrapper<StatData> wrapper = this.getPageWrapper(dto.getData(),info);
        Page<StatData> page = new Page<>(dto.getPageNum(), dto.getPageSize());
        IPage<StatData> pageList = statDataDao.selectPage(page, wrapper);
        Map<Long,List<StatDataItem>> map = this.getStatDataItemMao(pageList);


        boolean isShowName;
        boolean isShowPhone;
        boolean isShowNumber;
        if (CollUtil.isNotEmpty(info.getNoHideCloumn())) {
            isShowName = info.getNoHideCloumn().contains("1");
            isShowPhone = info.getNoHideCloumn().contains("2");
            isShowNumber = info.getNoHideCloumn().contains("3");
        } else {
            isShowNumber = false;
            isShowPhone = false;
            isShowName = false;
        }

        IPage<StatDataVO> convert = pageList.convert(object -> {
            StatDataVO item = new StatDataVO();
            BeanUtil.copyProperties(object, item);
            if (StrUtil.isNotBlank(item.getName()) && isShowName) {
                item.setName(SMSOrIdCardUtils.hideName(item.getName()));
            }
            if (StrUtil.isNotBlank(item.getPhone()) && isShowPhone) {
                item.setPhone(SMSOrIdCardUtils.hidePhone(item.getPhone()));
            }
            if (StrUtil.isNotBlank(item.getIdNumber()) && isShowNumber) {
                item.setIdNumber(SMSOrIdCardUtils.hideIdNumber(item.getIdNumber()));
            }
            //查询子项
            List<StatDataItem> statDataItemList = map.get(object.getId());
            item.putData(statDataItemList);
            return item;
        });
        return convert;
    }


    private Map<Long, List<StatDataItem>> getStatDataItemMao(IPage<StatData> pageList) {
        List<StatData> records = pageList.getRecords();
        if(CollUtil.isEmpty(records)){
            return new HashMap<>();
        }
        List<Long> idList = records.stream().map(StatData::getId).collect(Collectors.toList());
        List<StatDataItem> statDataItemList = statDataItemDao.selectList(Wrappers.<StatDataItem>lambdaQuery().in(StatDataItem::getStatDataId,idList).orderByDesc(StatDataItem::getCreateTime));
        return statDataItemList.stream().collect(Collectors.groupingBy(StatDataItem::getStatDataId));
    }

    private LambdaQueryWrapper<StatData> getPageWrapper(StatDataSearch data,AdminVO info) {
        LambdaQueryWrapper<StatData> wrapper = Wrappers.lambdaQuery();
        wrapper.like(StrUtil.isNotBlank(data.getName()),StatData::getName,data.getName());
        wrapper.like(StrUtil.isNotBlank(data.getPhone()),StatData::getPhone,data.getPhone());
        wrapper.like(StrUtil.isNotBlank(data.getIdNumber()),StatData::getIdNumber,data.getIdNumber());
        wrapper.like(StrUtil.isNotBlank(data.getCardIdEx()),StatData::getCardIdEx,data.getCardIdEx());
        wrapper.eq(ObjectUtil.isNotNull(data.getPeopleType()),StatData::getPeopleType,data.getPeopleType());
        wrapper.eq(ObjectUtil.isNotNull(data.getStatus()),StatData::getStatus,data.getStatus());
        wrapper.ge(StrUtil.isNotBlank(data.getStartTime()),StatData::getCreateTime, data.getStartTime());
        wrapper.le(StrUtil.isNotBlank(data.getEndTime()),StatData::getCreateTime, data.getEndTime());
        if(StrUtil.isNotBlank(data.getSn())){
            List<StatDataItem> statDataItemList = statDataItemDao.selectList(
                    Wrappers.<StatDataItem>query()
                            .select("DISTINCT stat_data_id")
                            .lambda()
                            .eq(StatDataItem::getAdminId, info.getAdminId())
                            .eq(StatDataItem::getSn, data.getSn())
            );
            List<Long> statDataIdList = statDataItemList.stream().map(StatDataItem::getStatDataId).collect(Collectors.toList());
            if(statDataIdList.isEmpty()){
                statDataIdList = CollUtil.toList(-1L);
            }
            wrapper.in(StatData::getId,statDataIdList);
        }
        wrapper.eq(StatData::getAdminId,info.getAdminId());
        return wrapper;
    }

    @Override
    public void excelOut(StatDataSearch data, HttpServletResponse response) {
        RestDTO<StatDataSearch> dto = new RestDTO<>();
        dto.setPageSize(50000);
        dto.setPageNum(0);
        dto.setData(data);

        AdminVO info = adminService.getInfo();

        LambdaQueryWrapper<StatData> wrapper = this.getPageWrapper(dto.getData(),info);
        Page<StatData> page = new Page<>(dto.getPageNum(), dto.getPageSize());
        IPage<StatData> pageList = statDataDao.selectPage(page, wrapper);
        Map<Long,List<StatDataItem>> map = this.getStatDataItemMao(pageList);

        List<StatDataExcel> list = new LinkedList<>();
        for (int i = 0; i < pageList.getRecords().size(); i++) {
           StatData statData = pageList.getRecords().get(i);
            if(map.containsKey(statData.getId())){
                List<StatDataItem> statDataItems = map.get(statData.getId());
                for (StatDataItem statDataItem : statDataItems){
                    StatDataExcel excel = this.getStatDataExcel(statData,statDataItem,info.getIsHideImportant());
                    excel.setStatDataId(i+1+"");
                    list.add(excel);
                }
            }
        }
        String fileName = "数据统计";
        String sheetName = "人员数据";
        // 这里注意 有同学反应使用swagger 会导致各种问题，请直接用浏览器或者用postman
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");

        try {
            fileName = URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(),StatDataExcel.class)
                    .registerWriteHandler(new CustomMergeStrategy(StatDataExcel.class))
                    .sheet(sheetName).doWrite(list);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private StatDataExcel getStatDataExcel(StatData statData, StatDataItem statDataItem, Integer isHideImportant) {
        StatDataExcel excel = BeanUtil.copyProperties(statData, StatDataExcel.class);
        if(1 == isHideImportant){
            excel.setName(SMSOrIdCardUtils.hideName(excel.getName()));
            excel.setPhone(SMSOrIdCardUtils.hidePhone(excel.getPhone()));
            excel.setIdNumber(SMSOrIdCardUtils.hideIdNumber(excel.getIdNumber()));
        }
        excel.setStatus(1 == statData.getStatus() ? "在场":"离场");
        excel.setPeopleType(1 == statData.getPeopleType() ? "常客":2 == statData.getPeopleType()?"访客":"其他");

        excel.setSn(statDataItem.getSn());
        excel.setSnName(statDataItem.getSnName());
        excel.setPass(1 == statDataItem.getPass() ?"进":"出");
        excel.setPassTime(statDataItem.getPassTime());
        excel.setPhoto(statDataItem.getPhoto());
        excel.setItemStatus(statDataItem.getStatus() ?"正常":"异常");

        return excel;
    }
}
