package com.yx.face.service.impl;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaCodeLineColor;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Lists;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.component.security.JWTUtils;
import com.yx.face.boot.component.wx.WxMaConfiguration;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.*;
import com.yx.face.boot.uitls.*;
import com.yx.face.dao.*;
import com.yx.face.model.dto.AdminDTO;
import com.yx.face.model.dto.AdminInoutWarnConfigDTO;
import com.yx.face.model.dto.EnterpriseAccountDTO;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.*;
import com.yx.face.model.excel.AdminInfoExcel;
import com.yx.face.model.search.AdminListNewSearch;
import com.yx.face.model.search.AdminSearch;
import com.yx.face.model.update.AdminUpdate;
import com.yx.face.model.vo.*;
import com.yx.face.model.vo.third.InterfAddrJsonVo;
import com.yx.face.service.*;
import com.yx.face.service.fkj.FkjService;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.Executor;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * @description: 管理人员信息表(Admin)表服务实现类 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-05-06 11:15:50 <br>
 */
@Slf4j
@Service
public class AdminServiceImpl implements AdminService {

    @Value("${weixin.buildingid}")
    private String appid;

   

    @Resource
    private AdminDao adminDao;

    @Resource
    private AdminAuthDao adminAuthDao;

    @Resource
    private AdminDetailDao adminDetailDao;

    @Resource
    private AreaCodeService areaCodeService;

    @Resource
    private AreaCodeDao areaCodeDao;

    @Resource
    private AdminTagDao adminTagDao;

    @Resource
    private UserInfoDao userInfoDao;

    @Resource
    private FaceLogDao faceLogDao;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private AdminService adminService;

    @Autowired
    private SmsService smsService;

    @Autowired
    private AlipayOpenApiService alipayOpenApiService;

    @Resource
    private SystemConfigDao systemConfigDao;

    @Resource
    private AdminPlaceInfoDao adminPlaceInfoDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private PrivateAreaDao privateAreaDao;

    @Resource
    private PrivateAreaDeviceDao privateAreaDeviceDao;

    @Resource
    private PrivateAreaWhitelistDao privateAreaWhitelistDao;

    @Resource
    private UserWhitelistDao userWhitelistDao;
    @Lazy
    @Resource
    private FaceService faceService;

    @Lazy
    @Autowired
    private OpenIssuedService openIssuedService;

    @Resource
    private Executor executor;
    @Resource
    private AdminInoutWarnConfigDao adminInoutWarnConfigDao;

    @Autowired
    private FkjService fkjService;

    private static final String ADMIN_LOGIN = "adminLogin";


    @Override
    public AdminVO login(String name, String password) {
        //账户名使用的昵称
        Admin adminInfo = adminDao.selectOneByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getUsername, name)).build());
        if (adminInfo == null) {
            throw new ServiceException("账号或密码输入错误");
        }
        if (adminInfo.getReviewStatus() != null && (adminInfo.getReviewStatus() == 0 || adminInfo.getReviewStatus() == 1)) {
            throw new ServiceException("您的账号无法登录，请联系管理员");
        }
        /*
         * redis校验
         * */
        String ipLogin = IPUtils.getIP() + ADMIN_LOGIN + adminInfo.getId();
        Integer adminLogin = (Integer) redisUtil.get(ipLogin);
        if (adminLogin != null && adminLogin >= 5) {
            redisUtil.incr(ipLogin, 1);
            redisUtil.expire(ipLogin, 300);
            throw new ServiceException("连续输错" + adminLogin + "次，账号锁定，请过5分钟后重试!");
        }

        password = PasswordUtils.buildPw(password);
        if (!adminInfo.getPassword().equals(password)) {
            //每次输错便加1，并设置过期时间为5分钟
            redisUtil.incr(ipLogin, 1);
            redisUtil.expire(ipLogin, 300);
            throw new ServiceException("账号或密码输入错误");
        } else {
            //登录成功
            boolean b = redisUtil.hasKey(ipLogin);
            if (b) {
                redisUtil.del(ipLogin);
            }
        }
        AdminVO adminLoginVO = new AdminVO();
        adminLoginVO.setAdminId(adminInfo.getId());
        adminLoginVO.setName(adminInfo.getNickname());
        adminLoginVO.setUsername(adminInfo.getUsername());
        adminLoginVO.setAvatar(adminInfo.getAvatar());
        /*adminLoginVO.setPhone();
        adminLoginVO.setEmail();
        adminLoginVO.setGender();*/
        adminLoginVO.setType(adminInfo.getType());
        adminLoginVO.setToken(JWTUtils.sign(adminInfo.getId().toString(), password, DateUtils.addDays(new Date(), 1)));
        /*wxg 2021-11-4 增加省市区详细地址 开始*/
        adminLoginVO.setProvinceId(adminInfo.getProvinceId());
        adminLoginVO.setProvinceName(adminInfo.getProvinceName());
        adminLoginVO.setCityId(adminInfo.getCityId());
        adminLoginVO.setCityName(adminInfo.getCityName());
        adminLoginVO.setAreaId(adminInfo.getAreaId());
        adminLoginVO.setAreaName(adminInfo.getAreaName());
        adminLoginVO.setStreetId(adminInfo.getStreetId());
        adminLoginVO.setStreetName(adminInfo.getStreetName());
        adminLoginVO.setAddress(adminInfo.getAddress());
        adminLoginVO.setPoliceStationCode(adminInfo.getPoliceStationCode());
        adminLoginVO.setPoliceStationName(adminInfo.getPoliceStationName());
        adminLoginVO.setIsControlEarlyWarning(adminInfo.getIsControlEarlyWarning());
        adminLoginVO.setTagId(adminInfo.getTagId());
        adminLoginVO.setIsOpenPrivateArea(adminInfo.getIsOpenPrivateArea());
        //场所类别
        adminLoginVO.setPlaceType(adminInfo.getPlaceType());
        if (StringUtils.hasText(adminInfo.getNoHideCloumn())) {
            adminLoginVO.setNoHideCloumn(Arrays.asList(adminInfo.getNoHideCloumn().split(",")));
        } else {
            adminLoginVO.setNoHideCloumn(new ArrayList<>());
        }


        if (StringUtils.hasText(adminInfo.getWhiteVisitorType())) {
            adminLoginVO.setWhiteVisitorType(Arrays.asList(adminInfo.getWhiteVisitorType().split(",")));
        } else {
            adminLoginVO.setWhiteVisitorType(new ArrayList<>());
        }
        if (adminInfo.getTagId() != null) {
            adminLoginVO.setTagName(adminTagDao.selectByPrimaryKey(adminInfo.getTagId()).getName());
        }
        SystemConfigVO vo = systemConfigDao.sysConfiguration(adminLoginVO.getAdminId());
        if (vo != null) {
            adminLoginVO.setSysQueryConfiguration(1);
            adminLoginVO.setSysQueryConfiguration(vo.getStatus());
        } else {
            adminLoginVO.setSysQueryConfiguration(0);
        }
        /*wxg 2021-11-4 增加省市区详细地址 结束*/
        /*wxg 2022-8-11 增加三码 及 核酸时长*/
        adminLoginVO.setThreeCodeList(JSON.parseArray(adminInfo.getThreeCode(), String.class));
        adminLoginVO.setHsHour(adminInfo.getHsHour());
        /*wxg 2022-8-17 增加常访客有效时长和风险区*/
        adminLoginVO.setRiskAreaList(adminInfo.getRiskArea() == null ? new ArrayList() : Arrays.stream(adminInfo.getRiskArea().split(",")).collect(Collectors.toList()));
        adminLoginVO.setUserWhitelistEffectiveTime(adminInfo.getUserWhitelistEffectiveTime());
        adminLoginVO.setUserVisiterEffectiveTime(adminInfo.getUserVisiterEffectiveTime());
        /*wxg 2022-8-29 增加是否开启物理卡号*/
        adminLoginVO.setCardPhysics(adminInfo.getCardPhysics());
        /*wxg 2022-9-2 增加实名认证年龄段 为空则表示都需要实名认证 格式：20-50*/
        if (StringUtils.hasText(adminInfo.getRealAuthenticationAge())) {
            String[] list = adminInfo.getRealAuthenticationAge().split("-");
            if (list.length == 2) {
                adminLoginVO.setRealAuthenticationAgeStart(Integer.parseInt(list[0]));
                adminLoginVO.setRealAuthenticationAgeEnd(Integer.parseInt(list[1]));
            }
        }

        return adminLoginVO;
    }


    @Override
    public Boolean logout() {
        return null;
    }

    @Override
    public AdminVO getInfo(Integer adminId) {
        Admin adminInfo = adminDao.selectByPrimaryKey(adminId);
        if (adminInfo == null) {
            throw new ServiceException("无此用户信息");
        }

        AdminVO adminLoginVO = BeanUtil.copyProperties(adminInfo, AdminVO.class);
        //查询账号附表信息
        AdminDetail adminDetail = adminDetailDao.selectOneByExample(Example.builder(AdminDetail.class).where(WeekendSqls.<AdminDetail>custom().andEqualTo(AdminDetail::getAdminId, adminId)).build());
        //预警配置
        if (adminInfo.getIsOpenInoutWarning()) {
            AdminInoutWarnConfig adminInoutWarnConfig = adminInoutWarnConfigDao.selectOne(
                    new AdminInoutWarnConfig().setAdminId(adminId)
            );
            adminLoginVO.setAdminInoutWarnConfigDTO(BeanUtil.copyProperties(adminInoutWarnConfig, AdminInoutWarnConfigDTO.class));
        }
        //二维码信息
        adminLoginVO.setAdminDetail(parseData(adminInfo, adminDetail));
        adminLoginVO.setAdminId(adminId);
        adminLoginVO.setName(adminInfo.getNickname());
        if (adminInfo.getTagId() != null) {
            adminLoginVO.setTagName(adminTagDao.selectByPrimaryKey(adminInfo.getTagId()).getName());
        }
        adminLoginVO.setTagId(adminInfo.getTagId());
        adminLoginVO.setIsOut(adminInfo.getIsOut());
        adminLoginVO.setIsHideImportant(adminInfo.getIsHideImportant());
        adminLoginVO.setIsOpenHealthCode(adminInfo.getIsOpenHealthCode());
        adminLoginVO.setShowQrcode(adminInfo.getShowQrcode());
        adminLoginVO.setQrcodeText(adminInfo.getQrcodeText());
        adminLoginVO.setAdvertise(adminInfo.getAdvertise());
        adminLoginVO.setTenantPlatformId(adminInfo.getTenantPlatformId());
        adminLoginVO.setTopTitle(adminInfo.getTopTitle());
        adminLoginVO.setClockWayType(adminInfo.getClockWayType());
        //账户添加配置信息2023-04-25
        adminLoginVO.setOpenThirdLinkPlatform(adminInfo.getOpenThirdLinkPlatform());
        adminLoginVO.setSubLineSuccess(adminInfo.getSubLineSuccess());
        adminLoginVO.setSubLineFail(adminInfo.getSubLineFail());
        adminLoginVO.setMainLineLeft(adminInfo.getMainLineLeft());
        adminLoginVO.setSubLineLeft(adminInfo.getSubLineLeft());
        adminLoginVO.setMainLineRight(adminInfo.getMainLineRight());
        adminLoginVO.setSubLineRight(adminInfo.getSubLineRight());
        adminLoginVO.setOpenVoiceMsgSuccess(adminInfo.getOpenVoiceMsgSuccess());
        adminLoginVO.setOpenVoiceMsgFail(adminInfo.getOpenVoiceMsgFail());
        adminLoginVO.setFaceLogHoldDays(adminInfo.getFaceLogHoldDays());
        adminLoginVO.setLatestResultPage(adminInfo.getLatestResultPage());
        adminLoginVO.setZfbMiniEthTitle(adminInfo.getZfbMiniEthTitle());
        adminLoginVO.setZfbMiniVguangTitle(adminInfo.getZfbMiniVguangTitle());
        //获取接口权限授权配置
        List<AdminAuth> adminAuths = adminAuthDao.selectByExample(Example.builder(AdminAuth.class)
                .where(WeekendSqls.<AdminAuth>custom()
                        .andEqualTo(AdminAuth::getAdminId, adminInfo.getId())
                        .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)
                ).build());
        if (Preconditions.isNotBlank(adminAuths)) {
            adminAuths.stream().forEach(item -> {
                //三方联动平台
                if (item.getType().intValue() == AdminAuthTypeEnum.M_PLATFORM.getCode()
                        && Preconditions.isNotBlank(adminInfo.getOpenThirdLinkPlatform())
                        && adminInfo.getOpenThirdLinkPlatform()) {
                    AdminAuthVO adminAuthVO = BeanUtil.copyProperties(item, AdminAuthVO.class);
                    adminAuthVO.setInterfaceAddresses(Preconditions.isNotBlank(item.getInterfAddrJson()) ? JSONObject.parseArray(item.getInterfAddrJson(), InterfAddrJsonVo.class) : null);
                    adminLoginVO.setMAdminAuthVO(adminAuthVO);
                    adminLoginVO.setPrivateKey(item.getPrivateKey());
                    adminLoginVO.setAppKey(item.getAppKey());
                    adminLoginVO.setAppSecret(item.getAppSecret());
                    adminLoginVO.setInterfaceAddress(item.getInterfaceAddress());
                    adminLoginVO.setInterfaceAddresses(Preconditions.isNotBlank(item.getInterfAddrJson()) ? JSONObject.parseArray(item.getInterfAddrJson(), InterfAddrJsonVo.class) : null);
                }

                //通用打卡
                if (item.getType().intValue() == AdminAuthTypeEnum.GENERAL_CLOCK_WAY.getCode()
                        && Preconditions.isNotBlank(adminInfo.getClockWayType())
                        && "1".equals(adminInfo.getClockWayType().trim())) {
                    adminLoginVO.setClockWayPushUrl(item.getInterfaceAddress());
                    AdminAuthVO adminAuthVO = BeanUtil.copyProperties(item, AdminAuthVO.class);
                    adminAuthVO.setInterfaceAddresses(Preconditions.isNotBlank(item.getInterfAddrJson()) ? JSONObject.parseArray(item.getInterfAddrJson(), InterfAddrJsonVo.class) : null);
                    adminLoginVO.setClockWayAdminAuthVO(adminAuthVO);
                    adminLoginVO.setClockWayPushInterfaceAddresses(Preconditions.isNotBlank(item.getInterfAddrJson()) ? JSONObject.parseArray(item.getInterfAddrJson(), InterfAddrJsonVo.class) : null);
                }

                if (item.getType().intValue() == AdminAuthTypeEnum.USER_DISTRICT_M.getCode()) {
                    AdminAuthVO adminAuthVO = BeanUtil.copyProperties(item, AdminAuthVO.class);
                    adminLoginVO.setUserDistrictAdminAuthVO(adminAuthVO);
                }


            });
        }

        adminLoginVO.setCodeType(Arrays.stream(adminInfo.getCodeType().split(",")).collect(Collectors.toList()));
        List<ScreenPhotoVO> screenPhotos = new ArrayList();
        if (StrUtil.isNotBlank(adminInfo.getScreenPhotos())) {
            screenPhotos = JSON.parseObject(adminInfo.getScreenPhotos(), List.class);
        }
        adminLoginVO.setScreenShowPhotos(screenPhotos);
        if (StringUtils.hasText(adminInfo.getNoHideCloumn())) {
            adminLoginVO.setNoHideCloumn(Arrays.asList(adminInfo.getNoHideCloumn().split(",")));
        } else {
            adminLoginVO.setNoHideCloumn(new ArrayList<>());
        }
        if (StringUtils.hasText(adminInfo.getWhiteVisitorType())) {
            adminLoginVO.setWhiteVisitorType(Arrays.asList(adminInfo.getWhiteVisitorType().split(",")));
        } else {
            adminLoginVO.setWhiteVisitorType(new ArrayList<>());
        }
        /*wxg 2021-11-4 增加省市区详细地址 结束*/
        SystemConfigVO vo = systemConfigDao.sysConfiguration(adminLoginVO.getAdminId());
        if (vo != null) {
            adminLoginVO.setSysQueryConfiguration(1);
            adminLoginVO.setSysQueryConfiguration(vo.getStatus());
        } else {
            adminLoginVO.setSysQueryConfiguration(0);
        }
        /*wxg 2022-8-11 增加三码 及 核酸时长*/
        if (StringUtils.hasText(adminInfo.getThreeCode())) {
            adminLoginVO.setThreeCodeList(JSON.parseArray(adminInfo.getThreeCode(), String.class));
        } else {
            adminLoginVO.setThreeCodeList(new ArrayList<>());
        }
        /*wxg 2022-8-17 增加常访客有效时长和风险区*/
        if (StringUtils.hasText(adminInfo.getRiskArea())) {
            adminLoginVO.setRiskAreaList(Arrays.stream(adminInfo.getRiskArea().split(",")).collect(Collectors.toList()));
        } else {
            adminLoginVO.setRiskAreaList(new ArrayList<>());
        }
        /*wxg 2022-9-2 增加实名认证年龄段 为空则表示都需要实名认证 格式：20-50*/
        if (StringUtils.hasText(adminInfo.getRealAuthenticationAge())) {
            String[] list = adminInfo.getRealAuthenticationAge().split("-");
            if (list.length == 2) {
                adminLoginVO.setRealAuthenticationAgeStart(Integer.parseInt(list[0]));
                adminLoginVO.setRealAuthenticationAgeEnd(Integer.parseInt(list[1]));
            }
        }
        /*wxg 2022-9-21 增加内宾不允许进入的年龄段 格式：20-50"*/
        adminLoginVO.setNotAllowAgeMsg(adminInfo.getNotAllowAgeMsg());
        adminLoginVO.setIsAlarm(adminInfo.getIsAlarm());
        if (StringUtils.hasText(adminInfo.getNotAllowAge())) {
            String[] list = adminInfo.getNotAllowAge().split("-");
            if (list.length == 2) {
                adminLoginVO.setNotAllowAgeStart(Integer.parseInt(list[0]));
                adminLoginVO.setNotAllowAgeEnd(Integer.parseInt(list[1]));
            }
        }
        /*wxg 2023-4-14 考试设备列表*/
        if (StrUtil.isNotEmpty(adminInfo.getExamDevices())) {
            List<String> list = Arrays.asList(adminInfo.getExamDevices().split(","));
            adminLoginVO.setExamDeviceList(list);
        } else {
            adminLoginVO.setExamDeviceList(new ArrayList<>());
        }
        return adminLoginVO;
    }

    /**
     * 解析转换二维码信息数据
     *
     * @param admin
     * @param adminDetail
     * @return
     */
    private AdminDetailVO parseData(Admin admin, AdminDetail adminDetail) {
        AdminDetailVO adminDetailVO = new AdminDetailVO();
        String cnPath = null;
        String enPath = null;
        String regularPath = null;
        String buildingPath = null;
        String buildingRegularPath = null;
        String zfbRegularPath = null;
        String zfbBuildingPath = null;
        String zfbBuildingRegularPath = null;
        String zfbFkjCodePath = null;

        try {
            if (adminDetail == null || StrUtil.isEmpty(adminDetail.getReservationCode())) {
                //预约内宾地址
                String cnContent = AddressUtil.address.substring(0, AddressUtil.address.length() - 1).concat("/appointment").concat("?orgId=") + admin.getId() + "&org=" + admin.getUsername() + "&type=" + AdminCodeEnum.RESERVATION_CODE.getCode() + "&language=ch";
                cnPath = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/admin_code_cn" + admin.getId() + ".png";
                //生成内宾二维码
                QRCodeUtil.encode(cnContent, "", cnPath, true);
            }

            if (adminDetail == null || StrUtil.isEmpty(adminDetail.getReservationEnCode())) {
                //预约外宾地址
                String enContent = AddressUtil.address.substring(0, AddressUtil.address.length() - 1).concat("/appointment").concat("?orgId=") + admin.getId() + "&org=" + admin.getUsername() + "&type=" + AdminCodeEnum.RESERVATION_CODE.getCode() + "&language=en";
                enPath = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/admin_code_en" + admin.getId() + ".png";
                //生成外宾二维码
                QRCodeUtil.encode(enContent, "", enPath, true);
            }

            if (adminDetail == null || StrUtil.isEmpty(adminDetail.getRegularCode())) {
                //常客码地址
                String regularContent = AddressUtil.address.substring(0, AddressUtil.address.length() - 1).concat("?orgId=") + admin.getId() + "&org=" + admin.getUsername() + "&type=" + AdminCodeEnum.REGULAR_CODE.getCode();
                regularPath = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/admin_code_regular" + admin.getId() + ".png";
                //生成常客二维码
                QRCodeUtil.encode(regularContent, "", regularPath, true);
            }

            if (adminDetail == null || StrUtil.isEmpty(adminDetail.getBuildingCode())) {
                //楼宇码地址
                buildingPath = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/admin_code_building" + admin.getId() + ".png";
                //生成楼宇小程序二维码
                final WxMaService wxService = WxMaConfiguration.getMaService(appid);
                byte[] wxaCodeBytes = wxService.getQrcodeService().createWxaCodeBytes("pages/resitor_scence/resitor_scence" + "?adminId=" + admin.getId() + "&type=" + AdminCodeEnum.GENERIC_CODE.getCode(), 430, false, new WxMaCodeLineColor("0", "0", "0"), false);
                FileUtil.writeBytes(wxaCodeBytes, new File(buildingPath));
            }
            if (adminDetail == null || StrUtil.isEmpty(adminDetail.getBuildingRegularCode())) {
                //楼宇常客码地址
                buildingRegularPath = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/admin_code_building_regular" + admin.getId() + ".png";
                //生成楼宇小程序二维码
                final WxMaService wxService = WxMaConfiguration.getMaService(appid);
                byte[] wxaCodeBytes = wxService.getQrcodeService().createWxaCodeBytes("pages/resitor_scence/resitor_scence" + "?adminId=" + admin.getId() + "&type=" + AdminCodeEnum.REGULAR_CODE.getCode(), 430, false, new WxMaCodeLineColor("0", "0", "0"), false);
                FileUtil.writeBytes(wxaCodeBytes, new File(buildingRegularPath));
            }

            //支付宝小程序二维码
            if (adminDetail == null || StrUtil.isEmpty(adminDetail.getZfbRegularCode())) {
                //生成通用常客二维码
                JSONObject obj = new JSONObject();
                obj.put("url_param", "pages/wx_airport/rlxf_index/rlxf_index");
                obj.put("query_param", "orgId=" + admin.getId() + "&org=" + admin.getUsername() + "&type=" + AdminCodeEnum.REGULAR_CODE.getCode());
                obj.put("describe", admin.getUsername() + "通用常客码");
                obj.put("size", "m");
                zfbRegularPath = alipayOpenApiService.createQrcode4701(obj.toJSONString());
            }
            if (adminDetail == null || StrUtil.isEmpty(adminDetail.getZfbBuildingCode())) {
                //楼宇码地址
                //生成楼宇小程序二维码
                JSONObject obj = new JSONObject();
                obj.put("url_param", "pages/resitor_scence/resitor_scence");
                obj.put("query_param", "adminId=" + admin.getId() + "&type=" + AdminCodeEnum.GENERIC_CODE.getCode());
                obj.put("describe", admin.getUsername() + "楼宇通用码");
                obj.put("size", "m");
                zfbBuildingPath = alipayOpenApiService.createQrcode(obj.toJSONString());
            }
            if (adminDetail == null || StrUtil.isEmpty(adminDetail.getZfbBuildingRegularCode())) {
                //楼宇常客码地址
                //生成楼宇小程序二维码
                JSONObject obj = new JSONObject();
                obj.put("url_param", "pages/resitor_scence/resitor_scence");
                obj.put("query_param", "adminId=" + admin.getId() + "&type=" + AdminCodeEnum.REGULAR_CODE.getCode());
                obj.put("describe", admin.getUsername() + "楼宇常客码");
                obj.put("size", "m");
                zfbBuildingRegularPath = alipayOpenApiService.createQrcode(obj.toJSONString());
            }

            //生成支付宝访客机二维码
            if (adminDetail == null || StrUtil.isEmpty(adminDetail.getZfbFkjCode())) {
                zfbFkjCodePath = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/zfb_fkj_qr_code" + admin.getId() + ".png";
                StringBuffer buffer = new StringBuffer("alipays://platformapi/startapp?")
                        .append("appId").append("=").append("2021003181682188").append("&")
                        .append("page").append("=").append("pages/visitor_qrcode/visitor_qrcode").append("&")
                        .append("query").append("=").append(URLEncoder.encode(("a=" + admin.getId()), "UTF-8"));
                String zfbFkjCodeContent = buffer.toString();
                QRCodeUtil.encode(zfbFkjCodeContent, "", zfbFkjCodePath, true);
            }
        } catch (Exception e) {
            throw new ServiceException("生成二维码失败");
        }

        if (adminDetail == null) {
            adminDetail = new AdminDetail().setAdminId(admin.getId()).setReservationCode(AddressUtil.address.concat(cnPath)).setReservationEnCode(AddressUtil.address.concat(enPath)).setRegularCode(AddressUtil.address.concat(regularPath)).setBuildingCode(AddressUtil.address.concat(buildingPath));
            adminDetail.setZfbBuildingCode(zfbBuildingPath);
            adminDetail.setZfbBuildingRegularCode(zfbBuildingRegularPath);
            adminDetail.setZfbRegularCode(zfbRegularPath);
            adminDetail.setZfbFkjCode(AddressUtil.address.concat(zfbFkjCodePath));
            //保存账号附表信息
            adminDetailDao.insert(adminDetail);
        } else {
            if (StrUtil.isNotEmpty(cnPath)) {
                adminDetail.setReservationCode(AddressUtil.address.concat(cnPath));
            }
            if (StrUtil.isNotEmpty(enPath)) {
                adminDetail.setReservationEnCode(AddressUtil.address.concat(enPath));
            }

            if (StrUtil.isNotEmpty(regularPath)) {
                adminDetail.setRegularCode(AddressUtil.address.concat(regularPath));
            }

            if (StrUtil.isNotEmpty(buildingPath)) {
                adminDetail.setBuildingCode(AddressUtil.address.concat(buildingPath));
            }
            if (StrUtil.isNotEmpty(buildingRegularPath)) {
                adminDetail.setBuildingRegularCode(AddressUtil.address.concat(buildingRegularPath));
            }
            if (StrUtil.isNotEmpty(zfbRegularPath)) {
                adminDetail.setZfbRegularCode(zfbRegularPath);
            }
            if (StrUtil.isNotEmpty(zfbBuildingPath)) {
                adminDetail.setZfbBuildingCode(zfbBuildingPath);
            }
            if (StrUtil.isNotEmpty(zfbBuildingRegularPath)) {
                adminDetail.setZfbBuildingRegularCode(zfbBuildingRegularPath);
            }

            if (StrUtil.isNotEmpty(zfbFkjCodePath)) {
                adminDetail.setZfbFkjCode(AddressUtil.address.concat(zfbFkjCodePath));
            }
            //更新账号附表信息
            adminDetailDao.updateByPrimaryKeySelective(adminDetail);
        }

        if (AdminPlaceTypeEnum.COMMON_PLACE.getCode().equals(admin.getPlaceType())) {
            if (StrUtil.isNotEmpty(admin.getCodeType()) && admin.getCodeType().contains(AdminCodeEnum.GENERIC_CODE.getCode())) {
                //通用码
                adminDetailVO.setGenericCode(admin.getEmail()).setGenericStatus(true);
                adminDetailVO.setZfbGenericCode(admin.getZfbEmail());
            }

            if (StrUtil.isNotEmpty(admin.getCodeType()) && admin.getCodeType().contains(AdminCodeEnum.REGULAR_CODE.getCode())) {
                //常客码
                adminDetailVO.setRegularCode(adminDetail.getRegularCode());
                adminDetailVO.setRegularStatus(true);
                //通用常客码
                adminDetailVO.setZfbRegularCode(adminDetail.getZfbRegularCode());
            }


        } else if (AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(admin.getPlaceType())
                || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(admin.getPlaceType())
        ) {
            adminDetailVO.setBuildingStatus(true);
            if (admin.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
                Admin admin1 = adminDao.selectByPrimaryKey(admin.getTenantId());
                if (admin1.getCodeType().contains("0")) {
                    adminDetailVO.setBuildingCode(adminDetail.getBuildingCode());
                    adminDetailVO.setZfbBuildingCode(adminDetail.getZfbBuildingCode());
                }
                if (admin1.getCodeType().contains("1")) {
                    adminDetailVO.setBuildingRegularCode(adminDetail.getBuildingRegularCode());
                    adminDetailVO.setZfbBuildingRegularCode(adminDetail.getZfbBuildingRegularCode());
                }
            } else {
                if (admin.getCodeType().contains("0")) {
                    adminDetailVO.setBuildingCode(adminDetail.getBuildingCode());
                    adminDetailVO.setZfbBuildingCode(adminDetail.getZfbBuildingCode());
                }
                if (admin.getCodeType().contains("1")) {
                    adminDetailVO.setBuildingRegularCode(adminDetail.getBuildingRegularCode());
                    adminDetailVO.setZfbBuildingRegularCode(adminDetail.getZfbBuildingRegularCode());
                }
            }
        } else {
            adminDetailVO.setReservationStatus(true);
            //预约内宾二维码
            adminDetailVO.setReservationCode(adminDetail.getReservationCode());
            //预约外宾二维码
            adminDetailVO.setReservationEnCode(adminDetail.getReservationEnCode());
        }
        adminDetailVO.setZfbFkjCode(adminDetail.getZfbFkjCode());
        return adminDetailVO;
    }

    private void checkEditAdminParam(AdminDTO dto) {
        if (StringUtils.hasText(dto.getIdNumber())) {
            if (!ConvertUtils.idCardJudge(dto.getIdNumber())) {
                throw new ServiceException("身份证号格式不正确");
            }
        }
        if (StringUtils.hasText(dto.getPhone())) {
            if (!ConvertUtils.phoneJudge(dto.getPhone())) {
                throw new ServiceException("手机号格式不正确");
            }
        }
        if (StringUtils.hasText(dto.getPassword())) {
            if (dto.getPassword().length() < 6) {
                throw new ServiceException("密码不能少于六位");
            }
        }
        if (dto.getPlaceCodeStatus() != null && dto.getPlaceCodeStatus() && CollectionUtil.isEmpty(dto.getPlaceInfoList())) {
            throw new ServiceException("开启场所码状态，对应信息不能为空");
        }
    }

    @Transactional
    @Override
    public Boolean editAdmin(AdminDTO dto) {
        checkEditAdminParam(dto);
        Date date = new Date();
        Admin adminInfo = BeanUtil.copyProperties(dto, Admin.class);
        List<String> punchReportFieldsConfig = dto.getPunchReportFieldsConfig();
        Optional.ofNullable(punchReportFieldsConfig).ifPresent(item -> adminInfo.setPunchReportFieldsConfig(String.join(",", item)));
        adminInfo.setOpenInOutStatistics(dto.getOpenInOutStatistics());
        adminInfo.setClockWayType(dto.getClockWayType());
        adminInfo.setClockWayPushUrl(dto.getClockWayPushUrl());
        adminInfo.setId(dto.getAdminId());
        adminInfo.setUsername(dto.getUsername());
        adminInfo.setAvatar(dto.getAvatar());
        adminInfo.setIdNumber(dto.getIdNumber());
        adminInfo.setPhone(dto.getPhone());
        adminInfo.setEmail(dto.getEmail());
        adminInfo.setZfbEmail(dto.getZfbEmail());
        adminInfo.setGender(dto.getGender());
        adminInfo.setUpdateTime(date);
        adminInfo.setNickname(dto.getName());
        adminInfo.setType(dto.getType());
        adminInfo.setIsOut(dto.getIsOut());
        adminInfo.setTagId(dto.getTagId());
        adminInfo.setIsOpenPrivateArea(dto.getIsOpenPrivateArea());
        adminInfo.setIsOpenHealthCode(Preconditions.isNotBlank(dto.getIsOpenHealthCode()) ? dto.getIsOpenHealthCode() : 1);
        adminInfo.setShowQrcode(dto.getShowQrcode());
        adminInfo.setQrcodeText(dto.getQrcodeText());
        adminInfo.setAdvertise(Preconditions.isNotBlank(dto.getAdvertise()) ? dto.getAdvertise() : "");
        adminInfo.setHomeImgUrl(dto.getHomeImgUrl());
        adminInfo.setTopTitle(dto.getTopTitle());
        adminInfo.setNotAllowAgeMsg(dto.getNotAllowAgeMsg());
        adminInfo.setIsAlarm(dto.getIsAlarm());
        adminInfo.setTenantPlatformId(dto.getTenantPlatformId());
        //账户添加配置信息2023-04-25
        adminInfo.setOpenThirdLinkPlatform(dto.getOpenThirdLinkPlatform());
        adminInfo.setSubLineSuccess(dto.getSubLineSuccess());
        adminInfo.setSubLineFail(dto.getSubLineFail());
        adminInfo.setMainLineLeft(dto.getMainLineLeft());
        adminInfo.setSubLineLeft(dto.getSubLineLeft());
        adminInfo.setMainLineRight(dto.getMainLineRight());
        adminInfo.setSubLineRight(dto.getSubLineRight());
        adminInfo.setOpenVoiceMsgSuccess(dto.getOpenVoiceMsgSuccess());
        adminInfo.setOpenVoiceMsgFail(dto.getOpenVoiceMsgFail());
        adminInfo.setFaceLogHoldDays(Preconditions.isBlank(dto.getFaceLogHoldDays()) ? 60 : dto.getFaceLogHoldDays());
        adminInfo.setLatestResultPage(dto.getLatestResultPage());
        adminInfo.setZfbMiniEthTitle(dto.getZfbMiniEthTitle());
        adminInfo.setZfbMiniVguangTitle(dto.getZfbMiniVguangTitle());
        //二维码类型记录标识
        adminInfo.setCodeType(CollectionUtil.isEmpty(dto.getCodeType()) ? AdminCodeEnum.GENERIC_CODE.getCode() : String.join(",", dto.getCodeType()));
        //场所类型
        adminInfo.setPlaceType(dto.getPlaceType());
        adminInfo.setIsHideImportant(dto.getIsHideImportant());
        //场所码
        adminInfo.setPlaceCodeStatus(dto.getPlaceCodeStatus());
//                .setAddressCode(dto.getAddressCode()).setUnitName(dto.getUnitName()).setUnitAddress(dto.getUnitAddress()).setManagerHealthCode(dto.getManagerHealthCode());
        //核酸有效期时间设置
        adminInfo.setNucleicExpirationTime(dto.getNucleicExpirationTime());
        adminInfo.setDingtalkAppKey(dto.getDingtalkAppKey());
        adminInfo.setDingtalkAppSecret(dto.getDingtalkAppSecret());

        adminInfo.setProblemReportFlag(dto.getProblemReportFlag());
        adminInfo.setDisinfectFlag(dto.getDisinfectFlag());
        adminInfo.setLimitGather(Preconditions.isBlank(dto.getLimitGather()) ? 0 : dto.getLimitGather());
        if (!CollectionUtils.isEmpty(dto.getAddCustomList())) {
            String s = null;
            if (dto.getAdminId() == null) {
                s = JSON.toJSONString(dto.getAddCustomList().stream().peek(v -> v.setId(IdUtil.fastSimpleUUID())).collect(Collectors.toList()));
            } else {
                //取id为空的自定义字段
                dto.getAddCustomList().stream().filter(v -> v != null && null == v.getId()).forEach(v -> v.setId(IdUtil.fastSimpleUUID()));
                s = JSON.toJSONString(dto.getAddCustomList());
            }

            adminInfo.setContent(s);
        } else {
            adminInfo.setContent(StrUtil.EMPTY);
        }
        adminInfo.setIsPush(dto.getIsPush());
        if (!dto.getNoHideCloumn().isEmpty()) {
            String join = String.join(",", dto.getNoHideCloumn());
            adminInfo.setNoHideCloumn(join);
        } else {
            adminInfo.setNoHideCloumn("");
        }
        if (!dto.getWhiteVisitorType().isEmpty()) {
            String join = String.join(",", dto.getWhiteVisitorType());
            adminInfo.setWhiteVisitorType(join);
        } else {
            adminInfo.setWhiteVisitorType("");
        }
        /*wxg 2021-11-4 增加省市区详细地址 开始*/
        adminInfo.setProvinceId(dto.getProvinceId() == null ? null : dto.getProvinceId());
        adminInfo.setProvinceName(dto.getProvinceId() == null ? null : areaCodeService.getById(dto.getProvinceId()).getName());
        adminInfo.setCityId(dto.getCityId() == null ? null : dto.getCityId());
        adminInfo.setCityName(dto.getCityId() == null ? null : areaCodeService.getById(dto.getCityId()).getName());
        adminInfo.setAreaId(dto.getAreaId() == null ? null : dto.getAreaId());
        adminInfo.setAreaName(dto.getAreaId() == null ? null : areaCodeService.getById(dto.getAreaId()).getName());
        adminInfo.setStreetId(dto.getStreetId() == null ? null : dto.getStreetId());
        adminInfo.setStreetName(dto.getStreetId() == null ? null : areaCodeService.getById(dto.getStreetId()).getName());
        adminInfo.setAddress(dto.getAddress());
//        if(dto.getPoliceStationCode()==null||dto.getPoliceStationCode().equals("")){
//            throw new ServiceException("派出所编码不能为空");
//        }
        adminInfo.setPoliceStationName(dto.getPoliceStationName() == null ? null : dto.getPoliceStationName());
        adminInfo.setPoliceStationCode(dto.getPoliceStationCode() == null ? null : dto.getPoliceStationCode());
        adminInfo.setIsControlEarlyWarning(dto.getIsControlEarlyWarning() == null ? null : dto.getIsControlEarlyWarning());
        /*wxg 2021-11-4 增加省市区详细地址 结束*/
        /*wxg 2022-8-11 增加三码 及 核酸时长*/
        if (!CollectionUtils.isEmpty(dto.getThreeCodeList())) {
            String threeCode = JSON.toJSONString(dto.getThreeCodeList());
            /*如果包含了核酸信息则需要填写核酸的时长，一般*/
            if (threeCode.contains(AdminThreeCodeEnum.HS.getCode().toString())) {
                if (dto.getHsHour() == null || dto.getHsHour() < 12) {
                    throw new ServiceException("核酸时长不能低于12小时");
                }
                adminInfo.setHsHour(dto.getHsHour());
            }
            adminInfo.setThreeCode(threeCode);
        } else {
            adminInfo.setThreeCode(StrUtil.EMPTY);
            adminInfo.setHsHour(null);
        }
        /*wxg 2022-8-17 增加常访客有效期和风险区*/
        adminInfo.setUserWhitelistEffectiveTime(dto.getUserWhitelistEffectiveTime() == null ? null : dto.getUserWhitelistEffectiveTime());
        adminInfo.setUserVisiterEffectiveTime(dto.getUserVisiterEffectiveTime() == null ? null : dto.getUserVisiterEffectiveTime());
        if (!CollectionUtils.isEmpty(dto.getRiskAreaList())) {
            adminInfo.setRiskArea(org.apache.commons.lang3.StringUtils.join(dto.getRiskAreaList(), ","));
        } else {
            adminInfo.setRiskArea(StrUtil.EMPTY);//因为行程码接口的原因 默认传0
        }
        /*wxg 2022-8-29 增加是否开启物理卡号*/
        adminInfo.setCardPhysics(dto.getCardPhysics());
        /*wxg 2022-9-2 增加实名认证年龄段 为空则表示都需要实名认证 格式：20-50*/
        if (dto.getRealAuthenticationAgeStart() != null) {
            if (dto.getRealAuthenticationAgeEnd() == null) {
                throw new ServiceException("实名年龄段请填写完整");
            }
            if (dto.getRealAuthenticationAgeEnd() <= dto.getRealAuthenticationAgeStart()) {
                throw new ServiceException("实名年龄区间有误");
            }
            adminInfo.setRealAuthenticationAge(dto.getRealAuthenticationAgeStart() + "-" + dto.getRealAuthenticationAgeEnd());
        }
        /*wxg 2022-9-21 增加内宾不允许进入的年龄段 格式：20-50"*/
        if (dto.getNotAllowAgeStart() != null) {
            if (dto.getNotAllowAgeStart() == null) {
                throw new ServiceException("内宾不允许进入年龄区间请填写完整");
            }
            if (dto.getNotAllowAgeEnd() < dto.getNotAllowAgeStart()) {
                throw new ServiceException("内宾不允许进入年龄区间有误");
            }
            adminInfo.setNotAllowAge(dto.getNotAllowAgeStart() + "-" + dto.getNotAllowAgeEnd());
        }
        /*wxg 2022-10-11 增加没有行程信息是否开门 1-开门（默认）0-关门*/
        adminInfo.setNoXcOpenDoor(dto.getNoXcOpenDoor());
        /*wxg 2022-10-11 增加24小时内核酸报告未出是否开门：1-开门（默认）0-关门*/
        adminInfo.setNoHsOpenDoor(dto.getNoHsOpenDoor());
        /*wxg 2022-10-14 增加去过中高风险地区是否开门 1-开门（默认）0-关门*/
        adminInfo.setRiskXcOpenDoor(dto.getRiskXcOpenDoor());
        /*wxg 2022-10-17 增加不允许通行的身份证地区代码，用/分隔*/
        adminInfo.setNoAllowAreaCode(dto.getNoAllowAreaCode());
        /*wxg 2022-10-17 增加行程码需检测是否开门 1-开门（默认）0-关门*/
        adminInfo.setXcNeedCheckOpenDoor(dto.getXcNeedCheckOpenDoor());
        /*wxg 2022-12-1
        增加微信小程序登录方式 1-身份证 2-卡号认证
        是否需要实名认证 1-需要 0-不需要
        * */
        adminInfo.setWxLoginType(dto.getWxLoginType());
        adminInfo.setIsNeedRealAuthentication(dto.getIsNeedRealAuthentication());
        /*wxg 2022-12-09 增加是否开启访客零点清除功能 1-开启 0-关闭*/
        adminInfo.setIsOpenVisitorClear(dto.getIsOpenVisitorClear());
        /*wxg 2022-12-19 迁移自定义手机号输入标志位 1输入 2获取*/
        adminInfo.setMobileAutoInputFlag(dto.getMobileAutoInputFlag());
        if (dto.getPlaceType() != null && (dto.getPlaceType() == 2 || dto.getPlaceType() == 3)) {
            adminInfo.setIsOpenPrivateArea(1);
        } else {
            adminInfo.setIsOpenPrivateArea(2);
        }
        /*wxg 2023-4-14 考试设备列表*/
        if (CollUtil.isNotEmpty(dto.getExamDeviceList())) {
            adminInfo.setExamDevices(StrUtil.join(",", dto.getExamDeviceList()));
        } else {
            adminInfo.setExamDevices(StrUtil.EMPTY);
        }
        //是否开启考试
        adminInfo.setIsOpenExam(dto.getIsOpenExam());
        if (!dto.getIsOpenExam()) {
            adminInfo.setExamDevices(StrUtil.EMPTY);
        }
        ClassUtil.throwBusinessException(AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(dto.getPlaceType()) && StrUtil.isBlank(dto.getOrgOutId()), "请填入机构代码");

        if (dto.getAdminId() == null) {
            if (!StringUtils.hasText(adminInfo.getNoHideCloumn())) {
                adminInfo.setNoHideCloumn("1,2,3");
            }
            if (!StringUtils.hasText(dto.getPassword())) {
                throw new ServiceException("请输入密码");
            }
            if (!StringUtils.hasText(dto.getName())) {
                throw new ServiceException("请输入账号名称");
            }
            Admin info = new Admin();
            info.setUsername(dto.getUsername());
            info = adminDao.selectOne(info);
            if (info != null) {
                throw new ServiceException("该账号名称已被注册");
            }
            adminInfo.setPassword(PasswordUtils.buildPw(dto.getPassword()));
            adminInfo.setCreateTime(date);
            adminDao.insertSelective(adminInfo);
            try {
                String content = AddressUtil.address.substring(0, AddressUtil.address.length() - 1).concat("?orgId=") + adminInfo.getId() + "&org=" + dto.getUsername();
                String path = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/admin_code_" + adminInfo.getId() + ".png";
                QRCodeUtil.encode(content, "", path, true);
                adminInfo.setEmail(AddressUtil.address + path);

                //生成通用常客二维码
                JSONObject obj = new JSONObject();
                obj.put("url_param", "pages/wx_airport/rlxf_index/rlxf_index");
                obj.put("query_param", "orgId=" + adminInfo.getId() + "&org=" + dto.getUsername());
                obj.put("describe", adminInfo.getUsername() + "通用常客码");
                obj.put("size", "m");
                String zfbEmail = alipayOpenApiService.createQrcode4701(obj.toJSONString());
                adminInfo.setZfbEmail(zfbEmail);

                adminDao.updateByPrimaryKeySelective(adminInfo);
            } catch (Exception e) {
                throw new ServiceException("生成二维码失败");
            }

            //查询adminDetail是否存在 不存在初始化adminDetail
            initAdminDetail(adminInfo.getId());

            //添加场所码记录
            this.addPlaceInfo(dto.getPlaceInfoList(), adminInfo.getId(), false);

            //三方接入配置
            addAdminAuth(null, null, adminInfo.getId(), AdminAuthTypeEnum.USER_DISTRICT_M);

            //三方平台生成appKey等信息
            if (Preconditions.isNotBlank(dto.getOpenThirdLinkPlatform()) && dto.getOpenThirdLinkPlatform()) {
                addAdminAuth(dto.getInterfaceAddress(), dto.getInterfaceAddresses(), adminInfo.getId(), AdminAuthTypeEnum.M_PLATFORM);
            }

            //开启通用打卡生成appKey等信息
            if (Preconditions.isNotBlank(dto.getClockWayType()) && "1".equals(dto.getClockWayType().trim())) {
                addAdminAuth(dto.getClockWayPushUrl(), dto.getClockWayPushInterfaceAddresses(), adminInfo.getId(), AdminAuthTypeEnum.GENERAL_CLOCK_WAY);
            }

            //初始化梯控关系表
            fkjService.initFloorConfig(adminInfo.getId());

            return true;
        } else {
            if (StringUtils.hasText(dto.getPassword())) {
                adminInfo.setPassword(PasswordUtils.buildPw(dto.getPassword()));
            }
            if (StringUtils.hasText(dto.getUsername())) {
                Admin info = new Admin();
                info.setUsername(dto.getUsername());
                info = adminDao.selectOne(info);
                if (info != null) {
                    if (!info.getId().equals(dto.getAdminId())) {
                        throw new ServiceException("该账号名称已被注册");
                    }
                }
            }
            Admin admin = adminDao.selectByPrimaryKey(dto.getAdminId());
            if (admin == null) {
                throw new ServiceException("未发现该账号");
            }
            try {
                String content;
                JSONObject obj = new JSONObject();
                if (StringUtils.hasText(dto.getUsername())) {
                    content = AddressUtil.address.substring(0, AddressUtil.address.length() - 1).concat("?orgId=") + admin.getId() + "&org=" + dto.getUsername();
                    obj.put("query_param", "orgId=" + admin.getId() + "&org=" + dto.getUsername());
                } else {
                    content = AddressUtil.address.substring(0, AddressUtil.address.length() - 1).concat("?orgId=") + admin.getId() + "&org=" + admin.getUsername();
                    obj.put("query_param", "orgId=" + admin.getId() + "&org=" + admin.getUsername());
                }
                String path = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/admin_code_" + adminInfo.getId() + ".png";
                QRCodeUtil.encode(content, "", path, true);
                adminInfo.setEmail(AddressUtil.address + path);

                //生成通用常客二维码
                obj.put("url_param", "pages/wx_airport/rlxf_index/rlxf_index");
                obj.put("describe", "通用常客码");
                obj.put("size", "m");
                String zfbEmail = alipayOpenApiService.createQrcode4701(obj.toJSONString());
                adminInfo.setZfbEmail(zfbEmail);
            } catch (Exception e) {
                throw new ServiceException("生成二维码失败");
            }
            Admin admin1 = adminDao.selectByPrimaryKey(adminInfo.getId());
            //下发屏保
            List<String> deviceList = faceDeviceDao.selectOnlineDevceSNsByAdminId(admin1.getId());
            if (CollectionUtil.isNotEmpty(dto.getScreenShowPhotos())) {
                List<String> screenPhotos = dto.getScreenShowPhotos().stream().map(ScreenPhotoVO::getUrl).collect(Collectors.toList());
                String json = JSON.toJSONString(dto.getScreenShowPhotos());
                adminInfo.setScreenPhotos(json);
                if (CollectionUtil.isNotEmpty(deviceList)) {
                    if (!faceService.downFile(deviceList, screenPhotos, "screen.jpg")) {
                        log.error("设备屏保照片下发失败");
                    }
                }
            } else {
                adminInfo.setScreenPhotos("");
            }

            //下发首页图片
            if (Preconditions.isNotBlank(dto.getHomeImgUrl())) {
                if (CollectionUtil.isNotEmpty(deviceList)) {
                    if (!faceService.downFile(deviceList, Arrays.asList(dto.getHomeImgUrl()), "index.jpg")) {
                        log.error("设备首页照片下发失败");
                    }
                }
            } else {
                adminInfo.setHomeImgUrl("");
            }
            /*如果是楼宇账号修改 保持相关属性和楼宇一致*/
            if (AdminTypeEnum.DISTRICT.getCode().equals(adminInfo.getType())
                    && AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(adminInfo.getPlaceType())) {
                adminDao.updateByTenantId(adminInfo);
            }
            adminDao.updateByPrimaryKeySelective(adminInfo);
            //添加场所码记录
            this.addPlaceInfo(dto.getPlaceInfoList(), adminInfo.getId(), true);

            //三方接入配置
            addAdminAuth(null, null, adminInfo.getId(), AdminAuthTypeEnum.USER_DISTRICT_M);

            //三方平台生成appKey等信息
            if (Preconditions.isNotBlank(dto.getOpenThirdLinkPlatform()) && dto.getOpenThirdLinkPlatform()) {
                addAdminAuth(dto.getInterfaceAddress(), dto.getInterfaceAddresses(), adminInfo.getId(), AdminAuthTypeEnum.M_PLATFORM);
            }

            //开启通用打卡信息推送生成appKey等信息
            if (Preconditions.isNotBlank(dto.getClockWayType()) && "1".equals(dto.getClockWayType().trim())) {
                addAdminAuth(dto.getClockWayPushUrl(), dto.getClockWayPushInterfaceAddresses(), adminInfo.getId(), AdminAuthTypeEnum.GENERAL_CLOCK_WAY);
            }

            //初始化梯控关系表
            fkjService.initFloorConfig(adminInfo.getId());

            return true;
        }
    }

    //初始化账户详情表
    private void initAdminDetail(Integer adminId) {
        AdminDetail adminDetail = adminDetailDao.selectOneByExample(Example.builder(AdminDetail.class)
                .where(WeekendSqls.<AdminDetail>custom()
                        .andEqualTo(AdminDetail::getAdminId, adminId)
                ).build());
        if (Preconditions.isNotBlank(adminDetail)) {
            return;
        }

        adminDetail = new AdminDetail();
        adminDetail.setAdminId(adminId);
        adminDetailDao.insert(adminDetail);

    }

    //更新接口授权
    private void updateAdminAuth(String interfaceAddress, List<InterfAddrJsonVo> interfaceAddresses, Integer adminId, AdminAuthTypeEnum adminAuthTypeEnum) {
        AdminAuth adminAuth = adminAuthDao.selectOneByExample(Example.builder(AdminAuth.class)
                .where(WeekendSqls.<AdminAuth>custom()
                        .andEqualTo(AdminAuth::getAdminId, adminId)
                        .andEqualTo(AdminAuth::getType, adminAuthTypeEnum.getCode())
                        .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)
                ).build());
        if (Preconditions.isNotBlank(adminAuth)) {
            adminAuth.setInterfaceAddress(interfaceAddress);
            adminAuth.setInterfAddrJson(Preconditions.isNotBlank(interfaceAddresses) ? JSONObject.toJSONString(interfaceAddresses) : null);
            adminAuthDao.updateByPrimaryKey(adminAuth);
        }

    }

    //接口授权
    private void addAdminAuth(String interfaceAddress, List<InterfAddrJsonVo> interfaceAddresses, Integer adminId, AdminAuthTypeEnum adminAuthTypeEnum) {

        try {
            AdminAuth adminAuth = adminAuthDao.selectOneByExample(Example.builder(AdminAuth.class)
                    .where(WeekendSqls.<AdminAuth>custom()
                            .andEqualTo(AdminAuth::getAdminId, adminId)
                            .andEqualTo(AdminAuth::getType, adminAuthTypeEnum.getCode())
                            .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)
                    ).build());
            if (Preconditions.isNotBlank(adminAuth)) {
                adminAuth.setInterfaceAddress(interfaceAddress);
                adminAuth.setInterfAddrJson(Preconditions.isNotBlank(interfaceAddresses) ? JSONObject.toJSONString(interfaceAddresses) : null);
                adminAuthDao.updateByPrimaryKey(adminAuth);
            } else {
                adminAuth = new AdminAuth();
                String appKey = getAppKey(adminAuthTypeEnum);
                String appId = DigestUtil.md5Hex(adminAuthTypeEnum.name() + appKey);
                adminAuth.setAppId(appId);
                adminAuth.setAppKey(appKey);
                adminAuth.setAppSecret(AppUtils.getAppSecret(appKey));
                adminAuth.setPrivateKey(getPrivateKey(adminAuthTypeEnum));
                adminAuth.setAdminId(adminId);
                adminAuth.setCreateTime(new Date());
                adminAuth.setType(adminAuthTypeEnum.getCode());
                adminAuth.setIsDeleted(Boolean.FALSE);
                adminAuth.setInterfaceAddress(interfaceAddress);
                adminAuth.setInterfAddrJson(Preconditions.isNotBlank(interfaceAddresses) ? JSONObject.toJSONString(interfaceAddresses) : null);
                adminAuthDao.insert(adminAuth);
            }
        } catch (Exception e) {
            log.error("--->三方授权API编辑失败");
        }
    }

    /**
     * 获取私钥
     *
     * @return
     */
    private String getPrivateKey(AdminAuthTypeEnum authType) {
        boolean existFlag = true;
        String privateKey = null;
        while (existFlag) {
            privateKey = AppUtils.getPrivateKey();
            AdminAuth adminAuth = adminAuthDao.selectOneByExample(Example.builder(AdminAuth.class)
                    .where(WeekendSqls.<AdminAuth>custom()
                            .andEqualTo(AdminAuth::getPrivateKey, privateKey)
                            .andEqualTo(AdminAuth::getType, authType.getCode())
                            .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)
                    ).build());
            if (adminAuth == null) {
                existFlag = false;
            }
        }
        return privateKey;
    }

    private String getAppKey(AdminAuthTypeEnum authType) {
        boolean existFlag = true;
        String appKey = null;
        while (existFlag) {
            appKey = AppUtils.getAppKey();
            AdminAuth adminAuth = adminAuthDao.selectOneByExample(Example.builder(AdminAuth.class)
                    .where(WeekendSqls.<AdminAuth>custom()
                            .andEqualTo(AdminAuth::getAppKey, appKey)
                            .andEqualTo(AdminAuth::getType, authType.getCode())
                            .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)
                    ).build());
            if (adminAuth == null) {
                existFlag = false;
            }
        }
        return appKey;
    }


    /**
     * 添加/修改场所码信息
     *
     * @param adminPlaceInfos
     */
    private void addPlaceInfo(List<AdminPlaceInfo> adminPlaceInfos, Integer adminId, boolean modifyType) {
        if (CollectionUtil.isEmpty(adminPlaceInfos)) {
            //场所码关闭状态，删除对应信息
            adminPlaceInfoDao.deleteByExample(Example.builder(AdminPlaceInfo.class).where(WeekendSqls.<AdminPlaceInfo>custom().andEqualTo(AdminPlaceInfo::getAdminId, adminId)).build());
            //TODO 取消所有设备注册
            faceDeviceDao.cancelAllFaceDeviceRegister(adminId, null);
            return;
        }

        //查询所有场所码信息
        List<AdminPlaceInfo> placeInfoList = adminPlaceInfoDao.selectByExample(Example.builder(AdminPlaceInfo.class).where(WeekendSqls.<AdminPlaceInfo>custom().andEqualTo(AdminPlaceInfo::getAdminId, adminId)).build());
        Map<Integer, AdminPlaceInfo> oldPlaceInfoMap = placeInfoList.stream().collect(Collectors.toMap(AdminPlaceInfo::getId, v -> v, (v1, v2) -> v1));

        //添加场所信息
        List<AdminPlaceInfo> addPlaceInfoList = adminPlaceInfos.stream().filter(v -> v != null && v.getId() == null).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(addPlaceInfoList)) {
            //新增场所码记录信息
            adminPlaceInfoDao.insertList(addPlaceInfoList.stream().peek(v -> v.setCreateTime(DateUtil.date()).setAdminId(adminId)).collect(Collectors.toList()));
        }

        Map<Integer, AdminPlaceInfo> newPlaceInfoMap = adminPlaceInfos.stream().filter(v -> v != null && v.getId() != null).collect(Collectors.toMap(AdminPlaceInfo::getId, v -> v, (v1, v2) -> v1));
        List<AdminPlaceInfo> delPlaceInfoList = placeInfoList.stream().filter(v -> newPlaceInfoMap.get(v.getId()) == null).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(delPlaceInfoList)) {
            //删除场所码信息
            adminPlaceInfoDao.deleteByExample(Example.builder(AdminPlaceInfo.class).where(WeekendSqls.<AdminPlaceInfo>custom().andIn(AdminPlaceInfo::getId, delPlaceInfoList.stream().map(AdminPlaceInfo::getId).collect(Collectors.toList()))).build());
            //TODO 取消所有绑定此场所码设备注册
            List<Integer> adminPlaceInfoIds = delPlaceInfoList.stream().map(AdminPlaceInfo::getId).collect(Collectors.toList());
            faceDeviceDao.cancelAllFaceDeviceRegister(adminId, adminPlaceInfoIds);
        }

        List<AdminPlaceInfo> modifyPlaceInfoList = adminPlaceInfos.stream().filter(v -> v != null && v.getId() != null && oldPlaceInfoMap.get(v.getId()) != null).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(modifyPlaceInfoList)) {
            //修改场所码信息
            modifyPlaceInfoList.forEach(v -> adminPlaceInfoDao.updateByPrimaryKeySelective(v));
            handleAllModifyFaceDeviceRegister(oldPlaceInfoMap, modifyPlaceInfoList);
        }

        if (modifyType && adminPlaceInfos.size() == 1) {
            //查询当前账号场所码信息
            AdminPlaceInfo adminPlaceInfo = adminPlaceInfoDao.selectOneByExample(Example.builder(AdminPlaceInfo.class).where(WeekendSqls.<AdminPlaceInfo>custom().andEqualTo(AdminPlaceInfo::getAdminId, adminId)).build());
            //设备注册并绑定
            handleOneModifyFaceDeviceRegister(oldPlaceInfoMap, adminPlaceInfo);
            //            //更新设备绑定场所码信息
//            faceDeviceDao.updateByExampleSelective(new FaceDevice().setAdminPlaceInfoId(adminPlaceInfo.getId()), Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getAdminId, adminId)).build());
        }
    }

    private void handleOneModifyFaceDeviceRegister(Map<Integer, AdminPlaceInfo> oldPlaceInfoMap, AdminPlaceInfo adminPlaceInfo) {
        executor.execute(() -> {
            AdminPlaceInfo oldAdminPlaceInfo = oldPlaceInfoMap.get(adminPlaceInfo.getId());
            if (Preconditions.isNotBlank(oldAdminPlaceInfo)
                    && (!oldAdminPlaceInfo.getAddressCode().equals(adminPlaceInfo.getAddressCode())
                    || !oldAdminPlaceInfo.getManagerHealthCode().equals(adminPlaceInfo.getManagerHealthCode()))) {
                List<FaceDevice> faceDevices = faceDeviceDao.getDeviceListByAdminPlaceInfoId(adminPlaceInfo.getAdminId(), null);
                faceDevices.stream().forEach(device -> {
                    Boolean register = openIssuedService.deviceRegister(device, adminPlaceInfo);
                    device.setAdminPlaceInfoId(adminPlaceInfo.getId());
                    device.setPlaceRegisterStatus(register);
                    //更新设备绑定场所码信息
                    faceDeviceDao.updateByPrimaryKeySelective(device);
                });
            }
        });
    }

    private void handleAllModifyFaceDeviceRegister(Map<Integer, AdminPlaceInfo> oldPlaceInfoMap, List<AdminPlaceInfo> modifyPlaceInfoList) {
        if (Preconditions.isBlank(modifyPlaceInfoList)) {
            return;
        }
        executor.execute(() -> {
            modifyPlaceInfoList.forEach(v -> {
                AdminPlaceInfo oldAdminPlaceInfo = oldPlaceInfoMap.get(v.getId());
                if (Preconditions.isNotBlank(oldAdminPlaceInfo)
                        && (!oldAdminPlaceInfo.getAddressCode().equals(v.getAddressCode())
                        || !oldAdminPlaceInfo.getManagerHealthCode().equals(v.getManagerHealthCode()))) {
                    List<FaceDevice> faceDevices = faceDeviceDao.getDeviceListByAdminPlaceInfoId(oldAdminPlaceInfo.getAdminId(), v.getId());
                    if (Preconditions.isNotBlank(faceDevices)) {
                        faceDevices.stream().forEach(device -> {
                            Boolean register = openIssuedService.deviceRegister(device, v);
                            device.setAdminPlaceInfoId(v.getId());
                            device.setPlaceRegisterStatus(register);
                            //更新设备绑定场所码信息
                            faceDeviceDao.updateByPrimaryKeySelective(device);
                        });
                    }
                }
            });
        });
    }

    @Override
    public PageInfo<AdminVO> getAdmins(RestDTO<AdminSearch> dto) {
        AdminVO info = getInfo();
        info.setAdminIds(Lists.newArrayList(info.getAdminId()));
        if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType()))
                && AdminTypeEnum.DISTRICT.getCode().equals(info.getType())) {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, info.getAdminId())).build());
            info.getAdminIds().addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
            //若是楼宇区级账号
            info.getAdminIds().remove(info.getAdminId());

            if (CollectionUtil.isEmpty(info.getAdminIds())) {
                return new PageInfo<>();
            }
        } else {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getType, AdminTypeEnum.ENTERPRISE.getCode())).build());
            info.setNotInAdminIds(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
        }
        AdminSearch data = dto.getData();
        Map<String, Object> map = new HashMap();
        map.put("type", info.getType());
        map.put("adminVo", info);
        map.put("ads", data);

        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<AdminVO> admins = adminDao.getAdmins(map);

        Map<Integer, AdminTag> adminTagMap = this.getAdminTag(admins);


        if (!admins.isEmpty()) {
            //查询账号附表信息
            List<AdminDetail> adminDetails = adminDetailDao.selectByExample(Example.builder(AdminDetail.class).where(WeekendSqls.<AdminDetail>custom().andIn(AdminDetail::getAdminId, admins.stream().map(AdminVO::getAdminId).collect(Collectors.toList()))).build());
            Map<Integer, AdminDetail> adminDetailMap = adminDetails.stream().collect(Collectors.toMap(AdminDetail::getAdminId, v -> v, (v1, v2) -> v1));
            //查询账号场所码信息
            List<AdminPlaceInfo> adminPlaceInfos = adminPlaceInfoDao.selectByExample(Example.builder(AdminPlaceInfo.class).where(WeekendSqls.<AdminPlaceInfo>custom().andIn(AdminPlaceInfo::getAdminId, admins.stream().map(AdminVO::getAdminId).collect(Collectors.toList()))).build());
            Map<Integer, List<AdminPlaceInfo>> placeInfoMap = adminPlaceInfos.stream().collect(Collectors.groupingBy(AdminPlaceInfo::getAdminId));
            //三方
            List<AdminAuth> adminAuths = adminAuthDao.selectByExample(Example.builder(AdminAuth.class).where(WeekendSqls.<AdminAuth>custom().andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE).andIn(AdminAuth::getAdminId, admins.stream().map(AdminVO::getAdminId).collect(Collectors.toList()))).build());
            Map<Integer, List<AdminAuth>> adminAuthMap = adminAuths.stream().collect(Collectors.groupingBy(AdminAuth::getAdminId));

            for (AdminVO admin : admins) {
                if (adminTagMap.containsKey(admin.getTagId())) {
                    admin.setTagName(adminTagMap.get(admin.getTagId()).getName());
                }

                // 转化字段
                List<ScreenPhotoVO> screenPhotos = new ArrayList();
                if (StrUtil.isNotBlank(admin.getScreenPhotoStr())) {
                    screenPhotos = JSON.parseObject(admin.getScreenPhotoStr(), List.class);
                }
                admin.setScreenShowPhotos(screenPhotos);
                String punchReportFieldsConfigStr = admin.getPunchReportFieldsConfigStr();
                admin.setPunchReportFieldsConfig(StrUtil.isEmpty(punchReportFieldsConfigStr) ? Lists.newArrayList() : Arrays.asList(punchReportFieldsConfigStr.split(",")));
                //二维码类型记录标识
                admin.setCodeType(StrUtil.isEmpty(admin.getCodeTypeStr()) ? Lists.newArrayList() : Arrays.asList(admin.getCodeTypeStr().split(",")));

                AdminDetail detail = adminDetailMap.get(admin.getAdminId());
                if (detail != null) {
                    //附表信息
                    admin.setAdminDetail(BeanUtil.copyProperties(detail, AdminDetailVO.class));
                }

                List<AdminPlaceInfo> placeInfoList = placeInfoMap.get(admin.getAdminId());
                if (CollectionUtil.isNotEmpty(placeInfoList)) {
                    //场所码信息
                    admin.setPlaceInfoList(placeInfoList);
                }

                List<AdminAuth> adminAuth = adminAuthMap.get(admin.getAdminId());
                if (Preconditions.isNotBlank(adminAuth)) {
                    adminAuth.stream().forEach(item -> {
                        //三方联动平台
                        if (item.getType().intValue() == AdminAuthTypeEnum.M_PLATFORM.getCode()
                                && Preconditions.isNotBlank(admin.getOpenThirdLinkPlatform())
                                && admin.getOpenThirdLinkPlatform()) {
                            AdminAuthVO adminAuthVO = BeanUtil.copyProperties(item, AdminAuthVO.class);
                            adminAuthVO.setInterfaceAddresses(Preconditions.isNotBlank(item.getInterfAddrJson()) ? JSONObject.parseArray(item.getInterfAddrJson(), InterfAddrJsonVo.class) : null);
                            admin.setMAdminAuthVO(adminAuthVO);
                            admin.setPrivateKey(item.getPrivateKey());
                            admin.setAppKey(item.getAppKey());
                            admin.setAppSecret(item.getAppSecret());
                            admin.setInterfaceAddress(item.getInterfaceAddress());
                            admin.setInterfaceAddresses(Preconditions.isNotBlank(item.getInterfAddrJson()) ? JSONObject.parseArray(item.getInterfAddrJson(), InterfAddrJsonVo.class) : null);
                        }

                        //通用打卡信息推送
                        if (item.getType().intValue() == AdminAuthTypeEnum.GENERAL_CLOCK_WAY.getCode()
                                && Preconditions.isNotBlank(admin.getClockWayType())
                                && "1".equals(admin.getClockWayType().trim())) {
                            AdminAuthVO adminAuthVO = BeanUtil.copyProperties(item, AdminAuthVO.class);
                            adminAuthVO.setInterfaceAddresses(Preconditions.isNotBlank(item.getInterfAddrJson()) ? JSONObject.parseArray(item.getInterfAddrJson(), InterfAddrJsonVo.class) : null);
                            admin.setClockWayAdminAuthVO(adminAuthVO);
                            admin.setClockWayPushInterfaceAddresses(Preconditions.isNotBlank(item.getInterfAddrJson()) ? JSONObject.parseArray(item.getInterfAddrJson(), InterfAddrJsonVo.class) : null);
                        }

                        if (item.getType().intValue() == AdminAuthTypeEnum.USER_DISTRICT_M.getCode()) {
                            AdminAuthVO adminAuthVO = BeanUtil.copyProperties(item, AdminAuthVO.class);
                            admin.setUserDistrictAdminAuthVO(adminAuthVO);
                        }
                    });
                }

                if (StringUtils.hasText(admin.getWvTypes())) {
                    admin.setWhiteVisitorType(Arrays.asList(admin.getWvTypes().split(",")));
                } else {
                    admin.setWhiteVisitorType(new ArrayList<>());
                }
                if (StringUtils.hasText(admin.getNoHideCloumnString())) {
                    admin.setNoHideCloumn(Arrays.asList(admin.getNoHideCloumnString().split(",")));
                } else {
                    admin.setNoHideCloumn(new ArrayList<>());
                }
                /*wxg 2022-8-11 增加三码 及 核酸时长*/
                if (StringUtils.hasText(admin.getThreeCode())) {
                    admin.setThreeCodeList(JSON.parseArray(admin.getThreeCode(), String.class));
                } else {
                    admin.setThreeCodeList(new ArrayList<>());
                }
                admin.setHsHour(admin.getHsHour());
                /*wxg 2022-8-17 增加常访客有效时长和风险区*/
                if (StringUtils.hasText(admin.getRiskArea())) {
                    admin.setRiskAreaList(Arrays.stream(admin.getRiskArea().split(",")).collect(Collectors.toList()));
                } else {
                    admin.setRiskAreaList(new ArrayList<>());
                }
                /*wxg 2022-9-2 增加实名认证年龄段 为空则表示都需要实名认证 格式：20-50*/
                if (StringUtils.hasText(admin.getRealAuthenticationAge())) {
                    String[] list = admin.getRealAuthenticationAge().split("-");
                    if (list.length == 2) {
                        admin.setRealAuthenticationAgeStart(Integer.parseInt(list[0]));
                        admin.setRealAuthenticationAgeEnd(Integer.parseInt(list[1]));
                    }
                }
                /*wxg 2022-9-21 增加内宾不允许进入的年龄段 格式：20-50*/
                if (StringUtils.hasText(admin.getNotAllowAge())) {
                    String[] list = admin.getNotAllowAge().split("-");
                    if (list.length == 2) {
                        admin.setNotAllowAgeStart(Integer.parseInt(list[0]));
                        admin.setNotAllowAgeEnd(Integer.parseInt(list[1]));
                    }
                }
                /*wxg 2023-4-14 考试设备列表*/
                if (StrUtil.isNotEmpty(admin.getExamDevices())) {
                    List<String> list = Arrays.asList(admin.getExamDevices().split(","));
                    admin.setExamDeviceList(list);
                } else {
                    admin.setExamDeviceList(new ArrayList<>());
                }
            }
        }

        return new PageInfo<>(admins);
    }

    private Map<Integer, AdminTag> getAdminTag(List<AdminVO> admins) {
        if (CollUtil.isEmpty(admins)) {
            return new HashMap<>();
        }
        List<Integer> tagIdList = admins.stream().map(AdminVO::getTagId).collect(Collectors.toList());
        if (CollUtil.isEmpty(tagIdList)) {
            return new HashMap<>();
        }
        List<AdminTag> adminTagList = adminTagDao.selectByExample(
                Example.builder(AdminTag.class)
                        .where(WeekendSqls.<AdminTag>custom()
                                .andIn(AdminTag::getId, tagIdList)
                        ).build());
        return adminTagList.stream().collect(Collectors.toMap(AdminTag::getId, Function.identity()));
    }

    @Override
    public Boolean deleteAdmin(Long id) {
        Admin admin = adminDao.selectByPrimaryKey(id);
        ClassUtil.checkData(admin,"账号不存在");
        int count = faceDeviceDao.selectCount(new FaceDevice().setAdminId(id.intValue()));
        ClassUtil.throwBusinessException(count > 0,"请先删除该账号下的设备");
        if (AdminTypeEnum.DISTRICT.getCode().equals(admin.getType()) && (AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(admin.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(admin.getPlaceType()))) {
            List<Admin> adminList = adminDao.selectByExample(
                    Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, admin.getId())).select("id").build());
            ClassUtil.throwBusinessException(!adminList.isEmpty(),"请先删除该账号下的子级账号");
        }
        return adminDao.deleteByPrimaryKey(id) == 1;
    }


    @Override
    public AdminVO getAuthInfo(Integer id) {
        return adminDao.getAuthInfo(id);
    }

    @Override
    public AdminVO getInfo() {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (requestAttributes == null) {
            throw new CommonException(RestCode.UNAUTHORIZED);
        }
        HttpServletRequest request = requestAttributes.getRequest();
        String token = request.getHeader("Authorization");
        if (token == null || token.equals("")) {
            throw new ServiceException(RestCode.UNAUTHORIZED.code, "缺少token");
        }
        token = JWTUtils.getUserId(token);
        if (org.apache.commons.lang3.StringUtils.isEmpty(token)) {
            throw new CommonException(RestCode.UNAUTHORIZED);
        }
        return getInfo(Integer.parseInt(token));
    }

    @Override
    public RestResult<List<String>> uploadAdminInfoExcel(List<Object> datas) {
        List<String> list = new ArrayList<>();
        for (int i = 0; i < datas.size(); i++) {
            try {
                AdminInfoExcel adminInfoExcel = (AdminInfoExcel) datas.get(i);
                if (adminInfoExcel.getUsername() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,账号名称不能为空");
                }
                if (adminInfoExcel.getType() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,账号级别不能为空");
                }
                if (adminInfoExcel.getName() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,管理员姓名不能为空");
                }
                if (adminInfoExcel.getPhone() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,管理员手机号不能为空");
                }
                if (adminInfoExcel.getProvinceName() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,省不能为空");
                }
                if (adminInfoExcel.getCityName() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,市不能为空");
                }
                if (adminInfoExcel.getAreaName() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,区/县不能为空");
                }
                if (adminInfoExcel.getStreetName() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,街道不能为空");
                }
                if (adminInfoExcel.getAddress() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,详细地址不能为空");
                }
                if (adminInfoExcel.getPoliceStationName() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,派出所名称不能为空");
                }
                if (adminInfoExcel.getPassword() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,密码不能为空");
                }
                if (adminInfoExcel.getTag() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,所属部门不能为空");
                }
                if (adminInfoExcel.getIsOut() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,是否可以导出不能为空");
                }
                if (adminInfoExcel.getIsHideImportant() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,是否脱敏不能为空");
                }
                if (adminInfoExcel.getIsControlEarlyWarning() == null) {
                    throw new ServiceException("第" + (i + 2) + "行,是否开启布控不能为空");
                }

                AdminDTO adminDTO = new AdminDTO();
                if (adminInfoExcel.getType().equals("市级")) {
                    adminDTO.setType((byte) 4);
                } else if (adminInfoExcel.getType().equals("区县级")) {
                    adminDTO.setType((byte) 5);
                } else if (adminInfoExcel.getType().equals("省级")) {
                    adminDTO.setType((byte) 3);
                } else {
                    throw new ServiceException("第" + (i + 2) + "行,账号级别有误");
                }
                adminDTO.setUsername(adminInfoExcel.getUsername());
                adminDTO.setName(adminInfoExcel.getName());
                adminDTO.setPhone(adminInfoExcel.getPhone());

                AreaCode province = areaCodeDao.getAreaCode(adminInfoExcel.getProvinceName(), 0, null);
                if (province == null) {
                    throw new ServiceException("第" + (i + 2) + "行,请核对省的信息");
                }
                adminDTO.setProvinceId(province.getId());

                AreaCode city = areaCodeDao.getAreaCode(adminInfoExcel.getCityName(), 1, province.getCode());
                if (city == null) {
                    throw new ServiceException("第" + (i + 2) + "行," + province.getName() + "没有找到" + adminInfoExcel.getCityName());
                }
                adminDTO.setCityId(city.getId());

                AreaCode area = areaCodeDao.getAreaCode(adminInfoExcel.getAreaName(), 2, city.getCode());
                if (area == null) {
                    throw new ServiceException("第" + (i + 2) + "行," + province.getName() + adminInfoExcel.getCityName() + "没有找到" + adminInfoExcel.getAreaName());
                }
                adminDTO.setAreaId(area.getId());

                AreaCode street = areaCodeDao.getAreaCode(adminInfoExcel.getStreetName(), 3, area.getCode());
                if (street == null) {
                    throw new ServiceException("第" + (i + 2) + "行," + province.getName() + adminInfoExcel.getCityName() + adminInfoExcel.getAreaName() + "没有找到" + adminInfoExcel.getStreetName());
                }
                adminDTO.setStreetId(street.getId());

                adminDTO.setAddress(adminInfoExcel.getAddress());
                adminDTO.setPoliceStationName(adminInfoExcel.getPoliceStationName());
                adminDTO.setPassword(adminInfoExcel.getPassword());

                AdminTag adminTag = new AdminTag();
                adminTag.setName(adminInfoExcel.getTag());
                adminTag.setProvinceId(province.getId());
                AdminTag at = adminTagDao.selectOne(adminTag);
                if (at == null) {
                    throw new ServiceException("第" + (i + 2) + "行,没有该所属部门");
                }
                adminDTO.setTagId(at.getId());
                adminDTO.setIsOut(adminInfoExcel.getIsOut().equals("可以") ? 1 : 2);
                adminDTO.setIsHideImportant(adminInfoExcel.getIsHideImportant().equals("脱敏") ? 1 : 2);
                adminDTO.setIsControlEarlyWarning(adminInfoExcel.getIsControlEarlyWarning().equals("开启") ? 1 : 2);
                adminDTO.setPoliceStationCode(adminInfoExcel.getPoliceStationCode());

                try {
                    this.editAdmin(adminDTO);
                } catch (Exception e) {
                    throw new ServiceException("第" + (i + 2) + "行," + e.getMessage());
                }
            } catch (ServiceException e) {
                list.add(e.getMessage());
                continue;
            }
        }
        datas.clear();//结束后销毁不用的资源
        return RestResponse.ok(list);
    }

    @Override
    public List<AdminVO> getAdminlist() {
        AdminVO info = this.getInfo();
        info.setAdminIds(com.google.common.collect.Lists.newArrayList(info.getAdminId()));
        if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())) && AdminTypeEnum.DISTRICT.getCode().equals(info.getType())) {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, info.getAdminId())).build());
            info.getAdminIds().addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
            List<Admin> admins = adminDao.selectByIds(StrUtil.join(",", info.getAdminIds()));
            return admins.stream().map(e -> {
                AdminVO adminVO = new AdminVO();
                adminVO.setAdminId(e.getId());
                adminVO.setUsername(e.getUsername());
                return adminVO;
            }).collect(Collectors.toList());
        }
        return adminDao.getAdminlist(info);
    }

    @Override
    public Boolean updateAdmin(AdminUpdate adminUpdate) {
        AdminVO info = this.getInfo();
        Admin admin = new Admin();
        admin.setId(info.getAdminId());
        admin.setNickname(adminUpdate.getName());
        admin.setPhone(adminUpdate.getPhone());
        admin.setIsPush(adminUpdate.getIsPush());
        admin.setCardPhysics(adminUpdate.getCardPhysics());
        admin.setUserWhitelistEffectiveTime(adminUpdate.getUserWhitelistEffectiveTime());
        admin.setUserVisiterEffectiveTime(adminUpdate.getUserVisiterEffectiveTime());
        admin.setTopTitle(adminUpdate.getTopTitle());
//        admin.setIsOpenHealthCode(Preconditions.isNotBlank(adminUpdate.getIsOpenHealthCode()) ? adminUpdate.getIsOpenHealthCode() : 1);
        admin.setShowQrcode(adminUpdate.getShowQrcode());
        admin.setQrcodeText(adminUpdate.getQrcodeText());
        admin.setAdvertise(adminUpdate.getAdvertise());
        admin.setTenantPlatformId(adminUpdate.getTenantPlatformId());
        //账户添加配置信息2023-04-25
        admin.setClockWayPushUrl(adminUpdate.getClockWayPushUrl());
        admin.setSubLineSuccess(adminUpdate.getSubLineSuccess());
        admin.setSubLineFail(adminUpdate.getSubLineFail());
        admin.setMainLineLeft(adminUpdate.getMainLineLeft());
        admin.setSubLineLeft(adminUpdate.getSubLineLeft());
        admin.setMainLineRight(adminUpdate.getMainLineRight());
        admin.setSubLineRight(adminUpdate.getSubLineRight());
        admin.setOpenVoiceMsgSuccess(adminUpdate.getOpenVoiceMsgSuccess());
        admin.setOpenVoiceMsgFail(adminUpdate.getOpenVoiceMsgFail());
//        admin.setFaceLogHoldDays(Preconditions.isBlank(adminUpdate.getFaceLogHoldDays()) ? 60 : adminUpdate.getFaceLogHoldDays());
        admin.setLatestResultPage(adminUpdate.getLatestResultPage());
        admin.setZfbMiniEthTitle(adminUpdate.getZfbMiniEthTitle());
        admin.setZfbMiniVguangTitle(adminUpdate.getZfbMiniVguangTitle());
        if (StringUtils.hasText(adminUpdate.getPassword())) {
            admin.setPassword(PasswordUtils.buildPw(adminUpdate.getPassword()));
        }
        if (!info.getType().equals(AdminTypeEnum.DISTRICT.getCode())) {
            admin.setRiskArea(null);
        } else {
            admin.setRiskArea(org.apache.commons.lang3.StringUtils.join(adminUpdate.getRiskAreaList(), ","));
        }
        /*常访客*/
        if (CollUtil.isNotEmpty(adminUpdate.getWhiteVisitorType())) {
            String join = String.join(",", adminUpdate.getWhiteVisitorType());
            admin.setWhiteVisitorType(join);
        } else {
            admin.setWhiteVisitorType("");
        }
        admin.setNucleicExpirationTime(adminUpdate.getNucleicExpirationTime());
        if (adminUpdate.getRealAuthenticationAgeStart() != null) {
            if (adminUpdate.getRealAuthenticationAgeEnd() == null) {
                throw new ServiceException("实名年龄段请填写完整");
            }
            if (adminUpdate.getRealAuthenticationAgeEnd() <= adminUpdate.getRealAuthenticationAgeStart()) {
                throw new ServiceException("实名年龄区间有误");
            }
            admin.setRealAuthenticationAge(adminUpdate.getRealAuthenticationAgeStart() + "-" + adminUpdate.getRealAuthenticationAgeEnd());
        }
        if (!CollectionUtils.isEmpty(adminUpdate.getThreeCodeList())) {
            String threeCode = JSON.toJSONString(adminUpdate.getThreeCodeList());
            /*如果包含了核酸信息则需要填写核酸的时长，一般*/
            if (threeCode.contains(AdminThreeCodeEnum.HS.getCode().toString())) {
                if (adminUpdate.getHsHour() == null || adminUpdate.getHsHour() < 12) {
                    throw new ServiceException("核酸时长不能低于12小时");
                }
                admin.setHsHour(adminUpdate.getHsHour());
            }
            admin.setThreeCode(threeCode);
        } else {
            admin.setThreeCode(null);
            admin.setHsHour(null);
        }
        if (adminUpdate.getNotAllowAgeStart() != null) {
            if (adminUpdate.getNotAllowAgeStart() == null) {
                throw new ServiceException("内宾不允许进入年龄区间请填写完整");
            }
            if (adminUpdate.getNotAllowAgeEnd() < adminUpdate.getNotAllowAgeStart()) {
                throw new ServiceException("内宾不允许进入年龄区间有误");
            }
            admin.setNotAllowAge(adminUpdate.getNotAllowAgeStart() + "-" + adminUpdate.getNotAllowAgeEnd());
        }
        /*wxg 2022-10-11 增加没有行程信息是否开门 1-开门（默认）0-关门*/
        admin.setNoXcOpenDoor(adminUpdate.getNoXcOpenDoor());
        /*wxg 2022-10-11 增加24小时内核酸报告未出是否开门：1-开门（默认）0-关门*/
        admin.setNoHsOpenDoor(adminUpdate.getNoHsOpenDoor());
        /*wxg 2022-10-14 增加去过中高风险地区是否开门 1-开门（默认）0-关门*/
        admin.setRiskXcOpenDoor(adminUpdate.getRiskXcOpenDoor());
        /*wxg 2022-10-17 增加不允许通行的身份证地区代码，用/分隔*/
        admin.setNoAllowAreaCode(adminUpdate.getNoAllowAreaCode());
        /*wxg 2022-10-17 增加行程码需检测是否开门 1-开门（默认）0-关门*/
        admin.setXcNeedCheckOpenDoor(adminUpdate.getXcNeedCheckOpenDoor());
        //三天三检
        admin.setLimitGather(Preconditions.isBlank(adminUpdate.getLimitGather()) ? 0 : adminUpdate.getLimitGather());
        /*wxg 2022-12-09 增加是否开启访客零点清除功能 1-开启 0-关闭*/
        admin.setIsOpenVisitorClear(adminUpdate.getIsOpenVisitorClear());
        /*wxg 2023-4-14 考试设备列表*/
        if (CollUtil.isNotEmpty(adminUpdate.getExamDeviceList())) {
            admin.setExamDevices(StrUtil.join(",", adminUpdate.getExamDeviceList()));
        } else {
            admin.setExamDevices(StrUtil.EMPTY);
        }
        //下发屏保
        List<String> deviceList = faceDeviceDao.selectOnlineDevceSNsByAdminId(admin.getId());
        if (CollectionUtil.isNotEmpty(adminUpdate.getScreenShowPhotos())) {
            List<String> screenPhotos = adminUpdate.getScreenShowPhotos().stream().map(ScreenPhotoVO::getUrl).collect(Collectors.toList());
            String json = JSON.toJSONString(adminUpdate.getScreenShowPhotos());
            admin.setScreenPhotos(json);
            if (CollectionUtil.isNotEmpty(deviceList)) {
                if (!faceService.downFile(deviceList, screenPhotos, "screen.jpg")) {
                    log.error("设备屏保照片下发失败");
                }
            }
        } else {
            admin.setScreenPhotos("");
        }
        //下发首页图片
        admin.setHomeImgUrl(adminUpdate.getHomeImgUrl());
        if (Preconditions.isNotBlank(adminUpdate.getHomeImgUrl())) {
            if (CollectionUtil.isNotEmpty(deviceList)) {
                if (!faceService.downFile(deviceList, Arrays.asList(adminUpdate.getHomeImgUrl()), "index.jpg")) {
                    log.error("设备首页照片下发失败");
                }
            }
        }
        admin.setNotAllowAgeMsg(adminUpdate.getNotAllowAgeMsg());
        admin.setIsAlarm(adminUpdate.getIsAlarm());
        this.updateAdminInoutWarnConfig(adminUpdate, admin);
        adminDao.updateByPrimaryKeySelective(admin);
        if (info.getPlaceType().equals(AdminPlaceTypeEnum.BUILDING_PLACE.getCode()) &&
                info.getType().equals(AdminTypeEnum.DISTRICT.getCode())
        ) {
            adminDao.updateByTenantId(adminDao.selectByPrimaryKey(info.getAdminId()));
        }

        updateAdminAuth(null, null, admin.getId(), AdminAuthTypeEnum.USER_DISTRICT_M);

        //三方平台生成appKey等信息
        if (Preconditions.isNotBlank(adminUpdate.getOpenThirdLinkPlatform()) && adminUpdate.getOpenThirdLinkPlatform()) {
            updateAdminAuth(adminUpdate.getInterfaceAddress(), adminUpdate.getInterfaceAddresses(), admin.getId(), AdminAuthTypeEnum.M_PLATFORM);
        }

        //开启通用打卡生成appKey等信息
        if (Preconditions.isNotBlank(adminUpdate.getClockWayType()) && "1".equals(adminUpdate.getClockWayType().trim())) {
            updateAdminAuth(adminUpdate.getClockWayPushUrl(), adminUpdate.getClockWayPushInterfaceAddresses(), admin.getId(), AdminAuthTypeEnum.GENERAL_CLOCK_WAY);
        }


        return true;
    }

    private void updateAdminInoutWarnConfig(AdminUpdate adminUpdate, Admin admin) {
        AdminInoutWarnConfigDTO dto = adminUpdate.getAdminInoutWarnConfigDTO();
        if (ObjectUtil.isNull(dto)) {
            return;
        }
        if (ObjectUtil.isNull(dto.getHour())) {
            return;
        }
        if (ObjectUtil.isNull(dto.getNum())) {
            return;
        }
        if (ObjectUtil.isNull(dto.getPass())) {
            return;
        }
        AdminInoutWarnConfig adminInoutWarnConfig = adminInoutWarnConfigDao.selectOne(new AdminInoutWarnConfig().setAdminId(admin.getId()));
        if (ObjectUtil.isNull(adminInoutWarnConfig)) {
            adminInoutWarnConfig = BeanUtil.copyProperties(dto, AdminInoutWarnConfig.class);
            adminInoutWarnConfig.setAdminId(admin.getId());
            adminInoutWarnConfigDao.insertSelective(adminInoutWarnConfig);
            return;
        }
        //数据没有发生变化则无需修改
        if (dto.getHour().equals(adminInoutWarnConfig.getHour())
                && dto.getPass().equals(adminInoutWarnConfig.getPass())
                && dto.getNum().equals(adminInoutWarnConfig.getNum())
        ) {
            return;
        }
        BeanUtil.copyProperties(dto, adminInoutWarnConfig);
        adminInoutWarnConfigDao.updateByPrimaryKey(adminInoutWarnConfig);
        //同时清除redis中的数据
        List<String> keyList = redisUtil.getAllByKey(Constants.IN_OUT_WARNING + admin.getId());
        if (CollUtil.isNotEmpty(keyList)) {
            for (String key : keyList) {
                redisUtil.del(key);
            }
        }
    }

    @Override
    public AdminVO showDataGetInfo(Integer userId) {
        Admin adminInfo = adminDao.selectByPrimaryKey(userId);
        if (adminInfo == null) {
            throw new ServiceException("无此用户信息");
        }
        AdminVO adminLoginVO = new AdminVO();
        adminLoginVO.setAdminId(userId);
        adminLoginVO.setUsername(adminInfo.getUsername());
        adminLoginVO.setType(adminInfo.getType());
        /*wxg 2021-11-4 增加省市区详细地址 开始*/
        adminLoginVO.setProvinceId(adminInfo.getProvinceId());
        adminLoginVO.setProvinceName(adminInfo.getProvinceName());
        adminLoginVO.setCityId(adminInfo.getCityId());
        adminLoginVO.setCityName(adminInfo.getCityName());
        adminLoginVO.setAreaId(adminInfo.getAreaId());
        adminLoginVO.setAreaName(adminInfo.getAreaName());
        adminLoginVO.setStreetId(adminInfo.getStreetId());
        adminLoginVO.setStreetName(adminInfo.getStreetName());
        adminLoginVO.setTagId(adminInfo.getTagId());
        return adminLoginVO;
    }

    @Override
    public AdminVO showDataGetInfo() {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (requestAttributes == null) {
            throw new CommonException(RestCode.UNAUTHORIZED);
        }
        HttpServletRequest request = requestAttributes.getRequest();
        String token = request.getHeader("Authorization");
        if (token == null || token.equals("")) {
            throw new ServiceException(RestCode.UNAUTHORIZED.code, "缺少token");
        }
        token = JWTUtils.getUserId(token);
        if (org.apache.commons.lang3.StringUtils.isEmpty(token)) {
            throw new CommonException(RestCode.UNAUTHORIZED);
        }
        return showDataGetInfo(Integer.parseInt(token));
    }

    @Override
    public List<Long> getAdminListSearch(Map<String, Object> map) {
        return adminDao.getAdminListSearch(map);
    }

    @Override
    public List<AdminVO> getByAdminlist(List<Integer> list) {
        return adminDao.getByAdminlist(list);
    }

    @Override
    public AdminVO getAdminInfoById(Integer adminId) {
        Admin admin = adminDao.selectByPrimaryKey(adminId);
        if (Objects.isNull(admin)) {
            return null;
        }
        AdminVO adminVO = BeanUtil.copyProperties(admin, AdminVO.class);
        adminVO.setAdminId(adminId);
        adminVO.setPunchReportFieldsConfigStr(admin.getPunchReportFieldsConfig());
        return adminVO;
    }


    @Override
    public List<AdminCodeVO> getCodeType() {
        List<AdminCodeVO> objectList = Arrays.stream(AdminCodeEnum.values()).map(v -> {
            AdminCodeVO adminCodeVO = new AdminCodeVO();
            adminCodeVO.setCode(v.name());
            adminCodeVO.setDesc(v.getDesc());
            return adminCodeVO;
        }).collect(Collectors.toList());

        return objectList;
    }

    @Override
    public Boolean saveEnterpriseAccount(EnterpriseAccountDTO dto) {
        if (dto.getId() == null) {
            this.addEnterpriseAccount(dto);
        } else {
            this.modifyEnterpriseAccount(dto);
        }

        return true;
    }

    @Override
    public Boolean addEnterpriseAccount(EnterpriseAccountDTO dto) {
        AdminVO loginVO = adminService.getInfo();
        Admin admin = new Admin();
        admin.setUsername(dto.getUsername()).setPhone(dto.getPhone()).setNickname(dto.getNickname()).setType(AdminTypeEnum.ENTERPRISE.getCode()).setCreateTime(DateUtil.date()).setUpdateTime(DateUtil.date()).setNoHideCloumn(loginVO.getNoHideCloumn().stream().collect(Collectors.joining(",")));
        admin.setProvinceId(loginVO.getProvinceId()).setProvinceName(loginVO.getProvinceName()).setCityId(loginVO.getCityId()).setCityName(loginVO.getCityName()).setAreaId(loginVO.getAreaId()).setAreaName(loginVO.getAreaName()).setStreetId(loginVO.getStreetId()).setStreetName(loginVO.getStreetName());
        admin.setAddress(dto.getAddress()).setReviewStatus(AdminReviewStatusEnum.HAS_AGREED_TO.getCode()).setTenantId(loginVO.getAdminId()).setPlaceType(loginVO.getPlaceType()).setWhiteVisitorType(loginVO.getWhiteVisitorType().stream().collect(Collectors.joining(",")));
        admin.setIsOpenPrivateArea(dto.getIsOpenPrivateArea());
        admin.setThreeCode(loginVO.getThreeCode() == null ? "" : loginVO.getThreeCode());
        admin.setHsHour(loginVO.getContent() == null ? null : loginVO.getHsHour());
        admin.setUserWhitelistEffectiveTime(loginVO.getUserWhitelistEffectiveTime() == null ? null : loginVO.getUserWhitelistEffectiveTime());
        admin.setUserVisiterEffectiveTime(loginVO.getUserVisiterEffectiveTime() == null ? null : loginVO.getUserVisiterEffectiveTime());
        admin.setRiskArea(loginVO.getRiskArea() == null ? "" : loginVO.getRiskArea());
        admin.setIsCanSearch(dto.getIsCanSearch());
        admin.setTenantEnterpriseName(loginVO.getTenantEnterpriseName());
        admin.setEnterpriseName(loginVO.getEnterpriseName());

        admin.setWhitelistAppRealname(loginVO.getWhitelistAppRealname());
        admin.setWhitelistAppPhone(loginVO.getWhitelistAppPhone());
        admin.setWhitelistAppLoginList(loginVO.getWhitelistAppLoginList());
        admin.setWhitelistAppPhoto(loginVO.getWhitelistAppPhoto());

        admin.setVisitorAppRealname(loginVO.getVisitorAppRealname());
        admin.setVisitorAppPhone(loginVO.getVisitorAppPhone());
        admin.setVisitorAppLoginList(loginVO.getVisitorAppLoginList());
        admin.setVisitorAppPhoto(loginVO.getVisitorAppPhoto());

        admin.setBuildingCheckType(loginVO.getBuildingCheckType());
        if (!StringUtils.hasText(dto.getPassword())) {
            throw new ServiceException("请输入密码");
        }
        if (!StringUtils.hasText(dto.getUsername())) {
            throw new ServiceException("请输入账号名称");
        }

        Admin admin1 = adminDao.selectOne(new Admin().setUsername(dto.getUsername()));
        if (admin1 != null) {
            throw new ServiceException("该账号名称已被注册");
        }

        admin.setPassword(PasswordUtils.buildPw(dto.getPassword()));
        adminDao.insertSelective(admin);

        AdminDetail detail = new AdminDetail();
        detail.setAdminId(admin.getId()).setEnterpriseFullName(dto.getEnterpriseFullName()).setEnterpriseSimpleName(dto.getEnterpriseSimpleName()).setMainBusinessScope(dto.getMainBusinessScope()).setLeaseFloor(dto.getLeaseFloor()).setRoomNumber(dto.getRoomNumber());
        detail.setForTheName(dto.getForTheName()).setActualName(dto.getActualName()).setActualPhone(dto.getActualPhone()).setEmployeesNumber(dto.getEmployeesNumber()).setLeaseBuilding(dto.getLeaseBuilding()).setInformationInputType(dto.getInformationInputType()).setCompanyCode(dto.getCompanyCode());
        detail.setLegalPerson(dto.getLegalPerson()).setCompanyType(dto.getCompanyType()).setRegisteredCapital(dto.getRegisteredCapital()).setRegistrationAddress(dto.getRegistrationAddress());
        //楼宇码地址
        String buildingPath = FileUtils.getDirectory(Constants.SYSTEM_FILE) + "/admin_code_building" + admin.getId() + ".png";
        //生成楼宇小程序二维码
        final WxMaService wxService = WxMaConfiguration.getMaService(appid);
        byte[] wxaCodeBytes = new byte[0];
        try {
            wxaCodeBytes = wxService.getQrcodeService().createWxaCodeBytes(new StringBuilder().append("pages/resitor_scence/resitor_scence").append("?adminId=").append(admin.getId()).toString(), 430, false, new WxMaCodeLineColor("0", "0", "0"), false);
        } catch (WxErrorException e) {
            e.printStackTrace();
        }
        FileUtil.writeBytes(wxaCodeBytes, new File(buildingPath));
        detail.setBuildingCode(AddressUtil.address.concat(buildingPath));

        //生成支付宝小程序二维码
        JSONObject obj = new JSONObject();
        obj.put("url_param", "pages/resitor_scence/resitor_scence");
        obj.put("query_param", "adminId=" + admin.getId() + "&type=" + AdminCodeEnum.GENERIC_CODE.getCode());
        obj.put("describe", admin.getUsername() + "楼宇通用码");
        obj.put("size", "m");
        String qrcode = alipayOpenApiService.createQrcode(obj.toJSONString());
        detail.setZfbBuildingCode(qrcode);
        if (StrUtil.isNotEmpty(dto.getLeaseTermStr())) {
            detail.setLeaseTerm(DateUtil.parse(dto.getLeaseTermStr(), DatePattern.NORM_DATE_PATTERN));
        }

        if (StrUtil.isNotEmpty(dto.getRegistrationTimeStr())) {
            detail.setRegistrationTime(DateUtil.parse(dto.getRegistrationTimeStr(), DatePattern.NORM_DATE_PATTERN));
        }

        adminDetailDao.insert(detail);
        //初始化梯控关系表
        fkjService.initFloorConfig(admin.getId());
        return true;
    }

    @Override
    public Boolean modifyEnterpriseAccount(EnterpriseAccountDTO dto) {
        Admin admin = new Admin();
        admin.setId(dto.getId()).setUsername(dto.getUsername()).setPhone(dto.getPhone()).setNickname(dto.getNickname()).setType(AdminTypeEnum.ENTERPRISE.getCode()).setCreateTime(DateUtil.date()).setUpdateTime(DateUtil.date()).setAddress(dto.getAddress());
        admin.setIsOpenPrivateArea(dto.getIsOpenPrivateArea());
        admin.setIsCanSearch(dto.getIsCanSearch());
        if (StrUtil.isNotEmpty(dto.getPassword())) {
            admin.setPassword(PasswordUtils.buildPw(dto.getPassword()));
        }

        if (StringUtils.hasText(dto.getUsername())) {
            Admin info = new Admin();
            info.setUsername(dto.getUsername());
            info = adminDao.selectOne(info);
            if (info != null) {
                if (!info.getId().equals(dto.getAdminId())) {
                    throw new ServiceException("该账号名称已被注册");
                }
            }
        }
        Admin admin1 = adminDao.selectByPrimaryKey(dto.getId());
        /*判断是否把私有区域关闭了*/
        if (!admin1.getIsOpenPrivateArea().equals(dto.getIsOpenPrivateArea())) {
            if (dto.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.CLOSE_PRIVATE_AREA.getCode())) {
                privateAreaDao.delete(new PrivateArea().setAdminId(dto.getId()));
                privateAreaDeviceDao.delete(new PrivateAreaDevice().setAdminId(dto.getId()));
                privateAreaWhitelistDao.delete(new PrivateAreaWhitelist().setAdminId(dto.getId()));

                faceDeviceDao.updateAreaType(dto.getId());
                userWhitelistDao.updateAdminAreaType(dto.getId());
            }
        }
        //更新账号信息
        adminDao.updateByPrimaryKeySelective(admin);
        AdminDetail detail = new AdminDetail();
        detail.setEnterpriseFullName(dto.getEnterpriseFullName()).setEnterpriseSimpleName(dto.getEnterpriseSimpleName()).setMainBusinessScope(dto.getMainBusinessScope()).setLeaseFloor(dto.getLeaseFloor()).setRoomNumber(dto.getRoomNumber());
        detail.setForTheName(dto.getForTheName()).setActualName(dto.getActualName()).setActualPhone(dto.getActualPhone()).setEmployeesNumber(dto.getEmployeesNumber()).setLeaseBuilding(dto.getLeaseBuilding()).setInformationInputType(dto.getInformationInputType()).setCompanyCode(dto.getCompanyCode());
        detail.setLegalPerson(dto.getLegalPerson()).setCompanyType(dto.getCompanyType()).setRegisteredCapital(dto.getRegisteredCapital()).setRegistrationAddress(dto.getRegistrationAddress());

        if (StrUtil.isNotEmpty(dto.getLeaseTermStr())) {
            detail.setLeaseTerm(DateUtil.parse(dto.getLeaseTermStr(), DatePattern.NORM_DATE_PATTERN));
        }

        if (StrUtil.isNotEmpty(dto.getRegistrationTimeStr())) {
            detail.setRegistrationTime(DateUtil.parse(dto.getRegistrationTimeStr(), DatePattern.NORM_DATE_PATTERN));
        }

        //更新账号附表信息
        adminDetailDao.updateByExampleSelective(detail, Example.builder(AdminDetail.class).where(WeekendSqls.<AdminDetail>custom().andEqualTo(AdminDetail::getAdminId, admin.getId())).build());
        //初始化梯控关系表
        fkjService.initFloorConfig(dto.getId());
        return true;
    }

    @Override
    public Boolean audit(Integer id, Integer reviewStatus) {
        adminDao.updateByPrimaryKeySelective(new Admin().setId(id).setReviewStatus(reviewStatus));
        AdminDetail detail = adminDetailDao.selectOne(new AdminDetail().setAdminId(id));
        if (detail != null && detail.getUserId() != null) {
            UserInfo userInfo = userInfoDao.selectByPrimaryKey(detail.getUserId());
            Admin admin = adminDao.selectByPrimaryKey(id);
            smsService.subscribeMessage(null, userInfo.getId(), WXMessageTemplateEnum.COMPANY_COMMIT, admin.getUsername(), admin.getReviewStatus() == 2 ? "审核通过" : "审核不通过", null, null, null);
            alipayOpenApiService.subscribeAlipayMessage(null, userInfo.getId(), WXMessageTemplateEnum.COMPANY_COMMIT, admin.getUsername(), admin.getReviewStatus() == 2 ? "审核通过" : "审核不通过", null, null, null);

        }
        return true;
    }

    @Override
    public EnterpriseAccountDTO queryAdminById(Integer id) {
        //查询账号信息
        Admin admin = adminDao.selectByPrimaryKey(id);
        //查询账号附表信息
        AdminDetail adminDetail = adminDetailDao.selectOneByExample(Example.builder(AdminDetail.class).where(WeekendSqls.<AdminDetail>custom().andEqualTo(AdminDetail::getAdminId, admin.getId())).build());
        EnterpriseAccountDTO dto = BeanUtil.copyProperties(adminDetail, EnterpriseAccountDTO.class);
        dto.setIsOpenPrivateArea(admin.getIsOpenPrivateArea());
        dto.setIsCanSearch(admin.getIsCanSearch());
        dto.setId(admin.getId()).setNickname(admin.getNickname()).setUsername(admin.getUsername()).setPhone(admin.getPhone()).setAddress(admin.getAddress());
        if (adminDetail.getRegistrationTime() != null) {
            dto.setRegistrationTimeStr(DateUtil.format(adminDetail.getRegistrationTime(), DatePattern.NORM_DATE_PATTERN));
        }

        if (adminDetail.getLeaseTerm() != null) {
            dto.setLeaseTermStr(DateUtil.format(adminDetail.getLeaseTerm(), DatePattern.NORM_DATE_PATTERN));
        }
        return dto;
    }

    @Override
    public List<EnterpriseAccountDTO> queryEnterpriseAccount(Integer userId) {
        List<EnterpriseAccountDTO> dtoList = Lists.newArrayList();
        //查询企业账号信息
        List<Admin> admins = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, userId)).build());
        if (CollectionUtil.isEmpty(admins)) {
            return dtoList;
        }

        List<AdminDetail> adminDetails = adminDetailDao.selectByExample(Example.builder(AdminDetail.class).where(WeekendSqls.<AdminDetail>custom().andIn(AdminDetail::getAdminId, admins.stream().map(Admin::getId).collect(Collectors.toList()))).build());
        Map<Integer, AdminDetail> detailMap = adminDetails.stream().collect(Collectors.toMap(AdminDetail::getAdminId, v -> v, (v1, v2) -> v1));
        admins.forEach(admin -> {
            EnterpriseAccountDTO dto = new EnterpriseAccountDTO();
            AdminDetail adminDetail = detailMap.get(admin.getId());
            if (adminDetail != null) {
                dto = BeanUtil.copyProperties(adminDetail, EnterpriseAccountDTO.class);
            }
            dto.setId(admin.getId())
                    .setNickname(admin.getNickname())
                    .setUsername(admin.getUsername())
                    .setPhone(admin.getPhone())
                    .setAddress(admin.getAddress())
                    .setUserWhitelistEffectiveTime(admin.getUserWhitelistEffectiveTime())
                    .setUserVisiterEffectiveTime(admin.getUserVisiterEffectiveTime())
                    .setIsCanSearch(admin.getIsCanSearch())
            ;
            dtoList.add(dto);
        });

        return dtoList;
    }

    @Override
    public List<AdminPlaceInfo> queryPlaceInfos(Integer adminId) {
        List<AdminPlaceInfo> adminPlaceInfos = adminPlaceInfoDao.selectByExample(Example.builder(AdminPlaceInfo.class).where(WeekendSqls.<AdminPlaceInfo>custom().andEqualTo(AdminPlaceInfo::getAdminId, adminId)).build());
        return adminPlaceInfos;
    }

    @Override
    public List<AdminVO> getAdminListNew(AdminListNewSearch adminListNewSearch) {
        AdminVO info = adminService.getInfo();
        info.setAdminIds(com.google.common.collect.Lists.newArrayList(info.getAdminId()));
        if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())) && AdminTypeEnum.DISTRICT.getCode().equals(info.getType())) {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, info.getAdminId())).build());
            info.getAdminIds().addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
        }
        Map<String, Object> map = new HashMap<>();
        map.put("info", info);
        map.put("adminListNewSearch", adminListNewSearch);
        return adminDao.getAdminListNew(map);
    }

    @Override
    public List<RiskArea> riskAreaList(RiskArea riskArea) {
        return adminDao.riskAreaList(riskArea);
    }

    @Override
    public String getUserWhitelistEffectiveTime(Integer adminId) {
        AdminVO adminVO = null;
        if (adminId == null) {
            adminVO = this.getInfo();
        } else {
            adminVO = this.getInfo(adminId);
        }
        BigDecimal userWhitelistEffectiveTime = null;
        //区级账号和企业账号取常客有效期，其他的默认一年
        if (adminVO.getType().equals(AdminTypeEnum.DISTRICT.getCode()) || adminVO.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            if (adminVO.getUserWhitelistEffectiveTime() != null) {
                userWhitelistEffectiveTime = adminVO.getUserWhitelistEffectiveTime();
            } else {
                userWhitelistEffectiveTime = new BigDecimal(365);
            }
        } else {
            userWhitelistEffectiveTime = new BigDecimal(365);
        }
        int timelong = userWhitelistEffectiveTime.multiply(new BigDecimal(1440)).intValue();
        Date date = DateUtils.addMinutes(new Date(), timelong);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(date);
    }
}

