package com.yx.face.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.parser.Feature;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.domain.AlipayOpenAppMiniTemplatemessageSendModel;
import com.alipay.api.internal.util.AlipayEncrypt;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipayOpenAppMiniTemplatemessageSendRequest;
import com.alipay.api.request.AlipayOpenAppQrcodeCreateRequest;
import com.alipay.api.response.AlipayOpenAppMiniTemplatemessageSendResponse;
import com.alipay.api.response.AlipayOpenAppQrcodeCreateResponse;
import com.yx.face.boot.component.alipay.AlipayOpenApiProperties;
import com.yx.face.boot.component.alipay.IhuyiMsgProperties;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.OkHttpUtils;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.boot.uitls.Sm4Util;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.dao.iot.IotvspUserInfoDao;
import com.yx.face.model.dto.iot.AlipayAmpeSignOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayGroupIdOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayOpenIotvspUserwithimageEncryptionRequest;
import com.yx.face.model.dto.iot.AlipayOpenIotvspUserwithimageRequest;
import com.yx.face.model.dto.iot.AlipayOperateRecordCreateWithVidOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayOperateRecordInfoWithVidApiRequest;
import com.yx.face.model.dto.iot.AlipayUserDeleteEncryptionRequest;
import com.yx.face.model.dto.iot.AlipayUserDeleteOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayUserStateOpenApiRequest;
import com.yx.face.model.entity.UserInfo;
import com.yx.face.model.entity.UserVisitorListDetail;
import com.yx.face.model.enums.AlipayCertTypeEnum;
import com.yx.face.model.enums.AlipayMessageTemplateEnum;
import com.yx.face.model.enums.WXMessageTemplateEnum;
import com.yx.face.service.AlipayOpenApiService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.aop.interceptor.AsyncExecutionAspectSupport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;

/**
 * @author shisl
 * @package com.yx.face.service.impl
 * @class AlipayOpenApiServiceImpl
 * @date 2023/5/22 上午11:01
 * @description
 */
@Slf4j
@Service
public class AlipayOpenApiServiceImpl extends BaseService implements AlipayOpenApiService {

    @Resource
    private AlipayClient alipayOpenApiClient;

    @Resource
    private AlipayClient alipayOpenApiClient4701;

    @Autowired
    private AlipayOpenApiProperties alipayOpenApiProperties;

    @Autowired
    private IhuyiMsgProperties ihuyiMsgProperties;

    @Resource
    private UserInfoDao userInfoDao;

    @Resource
    private Executor executor;

    @Resource
    private IotvspUserInfoDao iotvspUserInfoDao;

    @Override
    public String decryptContent(String encryptedResponse) {
        //1. 获取验签和解密所需要的参数
        Map<String, String> openapiResult = JSON.parseObject(encryptedResponse, new TypeReference<Map<String, String>>() {
        }, Feature.OrderedField);
        String signType = alipayOpenApiProperties.getSignType();
        String charset = alipayOpenApiProperties.getCharset();
        String encryptType = "AES";
        String sign = openapiResult.get("sign");
        String content = openapiResult.get("response");
        //判断是否为加密内容
        boolean isDataEncrypted = !content.startsWith("{");
        boolean signCheckPass = false;
        //2. 验签
        String signContent = content;
        String signVeriKey = alipayOpenApiProperties.getAlipayPublicKey();
        String decryptKey = alipayOpenApiProperties.getDecryptKey();
        if (isDataEncrypted) {
            signContent = "\"" + signContent + "\"";
        }
        try {
            signCheckPass = AlipaySignature.rsaCheck(signContent, sign, signVeriKey, charset, signType);
        } catch (AlipayApiException e) {
            // 验签异常, 日志
        }
        if (!signCheckPass) {
            //验签不通过（异常或者报文被篡改），终止流程（不需要做解密）
            throw new ServiceException("验签失败");
        }
        //3. 解密
        String plainData = null;
        if (isDataEncrypted) {
            try {
                plainData = AlipayEncrypt.decryptContent(content, encryptType, decryptKey, charset);
            } catch (AlipayApiException e) {
                e.printStackTrace();
                //解密异常, 记录日志
                throw new ServiceException("解密异常");
            }
        } else {
            plainData = content;
        }
        return plainData;
    }


    @Async(AsyncExecutionAspectSupport.DEFAULT_TASK_EXECUTOR_BEAN_NAME)
    @Override
    public void subscribeAlipayMessage(String url, Long userId, WXMessageTemplateEnum wxMessageTemplateEnum, String p1, String p2, String p3, String p4, String p5) {
        try {
            AlipayOpenAppMiniTemplatemessageSendRequest request = new AlipayOpenAppMiniTemplatemessageSendRequest();
            AlipayOpenAppMiniTemplatemessageSendModel model = new AlipayOpenAppMiniTemplatemessageSendModel();
            JSONObject data = new JSONObject();
            url = null;
            if (StringUtils.isEmpty(url)) url = "pages/resitor_scence/resitor_scence";
            String method = "【支付宝订阅消息发送】";
            log.info(method + "入参：【url：{}，userId：{}，template：{}，p1：{}，p2：{}，p3：{}，p4：{}，p5：{}】", url, userId, wxMessageTemplateEnum, p1, p2, p3, p4, p5);
            if (userId == null || wxMessageTemplateEnum == null) {
                log.error("支付宝订阅消息必填参数都不能为空！");
                return;
            }

            UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
            if (ObjectUtil.isNull(userInfo) || StrUtil.isEmpty(userInfo.getZfbUserId())) {
                log.error("支付宝订阅消息支付宝用户ID参数不存在！");
                return;
            }
            String templateId = null;
            switch (wxMessageTemplateEnum) {
                case REVICE_APPLICATION://常客收到申请
                    templateId = AlipayMessageTemplateEnum.REVICE_APPLICATION.getTemplate();
                    break;
                case COMMIT_APPLICATION://访客申请提交
                    if (!SMSOrIdCardUtils.isMobile(p2)) {
                        p2 = userInfo.getPhone();
                    }
                    templateId = AlipayMessageTemplateEnum.COMMIT_APPLICATION.getTemplate();
                    break;
                case COMMIT_RESULT://常客同意后受邀人收到的结果
                    templateId = AlipayMessageTemplateEnum.COMMIT_RESULT.getTemplate();
                    break;
                case INVITE_RESULT://受邀人接受邀请时 常客提醒
                    templateId = AlipayMessageTemplateEnum.INVITE_RESULT.getTemplate();
                    break;
                case REMIND_REVICE_APPLICATION://提醒受邀人收到访客邀请
                    p1 = StringUtils.isEmpty(p1) ? "常客邀约" : p1;
                    templateId = AlipayMessageTemplateEnum.REMIND_REVICE_APPLICATION.getTemplate();
                    break;
                case COMPANY_COMMIT://用于企业申请提交告知用户
                    templateId = AlipayMessageTemplateEnum.COMPANY_COMMIT.getTemplate();
                    break;
                case COMPANY_RESULT://用于企业申请提交告知用户
                    templateId = AlipayMessageTemplateEnum.COMPANY_RESULT.getTemplate();
                    break;
                case VISITORY_IN_FIELD://用于企业申请提交告知用户
                    templateId = AlipayMessageTemplateEnum.VISITORY_IN_FIELD.getTemplate();
                    break;
            }

            if (StringUtils.isEmpty(templateId)) {
                log.error(method + "失败,模板id不存在");
                return;
            }
            data.put("keyword1", initValue(p1));
            data.put("keyword2", initValue(p2));
            data.put("keyword3", initValue(p3));
            data.put("keyword4", initValue(p4));
            data.put("keyword5", initValue(p5));
            model.setData(data.toJSONString());
            model.setUserTemplateId(templateId);//运营中心配置的消息模板id
            model.setPage(url);// 模板消息"进入小程序查看"按钮跳转的小程序页面地址
            model.setToUserId(userInfo.getZfbUserId());//接收模板消息的用户
            request.setBizModel(model);
            AlipayOpenAppMiniTemplatemessageSendResponse response = alipayOpenApiClient.execute(request);
            log.info(method + "结果为【" + response.getBody() + "】");
            if (response.isSuccess()) {
                log.info(method + "成功");
            } else {
                log.error(method + "失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    @Override
    @Async(AsyncExecutionAspectSupport.DEFAULT_TASK_EXECUTOR_BEAN_NAME)
    public void createAlipayUrlAndSendMsg(UserVisitorListDetail userVisitorListDetail) {
        try {
            StringBuffer buffer = new StringBuffer("alipays://platformapi/startapp?")
                    .append("appId").append("=").append(alipayOpenApiProperties.getAppId()).append("&")
                    .append("page").append("=").append("pages/visitor_make/visitor_make").append("&")
                    .append("query").append("=").append(URLEncoder.encode(("userVisitorDetailId=" + userVisitorListDetail.getId()), "UTF-8"));
            String scheme = URLEncoder.encode(buffer.toString(), "UTF-8");
            String urlLink = "https://ds.alipay.com/?scheme=" + scheme;
            String content = "您好，" + "楼宇综合管理" + "提醒您，您的好友" + userVisitorListDetail.getVisiteeXm() + "向您发送了一个访客邀请，点击链接打开小程序，接受邀请并填写访客信息，可享受" + userVisitorListDetail.getVisiteeCompany() + "区域快速通行，支付宝小程序链接：" + urlLink + " 。";

            log.info("--->发送{}短信请求参数:{}", userVisitorListDetail.getPhone(), content);
            sendMsg(userVisitorListDetail.getPhone(), content);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public String createQrcode(String bizContent) {
        try {
            AlipayOpenAppQrcodeCreateRequest request = new AlipayOpenAppQrcodeCreateRequest();
            request.setBizContent(bizContent);
            AlipayOpenAppQrcodeCreateResponse response = alipayOpenApiClient.execute(request);
            log.info("--->生成支付宝小程序二维码响应参数：{}", JSONObject.toJSONString(response));
            if (response.isSuccess()) {
                return response.getQrCodeUrlCircleBlue();
            }
        } catch (Exception e) {
            log.error("生成支付宝小程序二维码服务异常");
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public String createQrcode4701(String bizContent) {
        try {
            AlipayOpenAppQrcodeCreateRequest request = new AlipayOpenAppQrcodeCreateRequest();
            request.setBizContent(bizContent);
            AlipayOpenAppQrcodeCreateResponse response = alipayOpenApiClient4701.execute(request);
            log.info("--->生成4701通用常客二维码响应参数：{}", JSONObject.toJSONString(response));
            if (response.isSuccess()) {
                return response.getQrCodeUrl();
            }
        } catch (Exception e) {
            log.error("生成4701通用常客二维码服务异常");
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public RestResult queryOrgGroupId(AlipayGroupIdOpenApiRequest openApiRequest) {
        IsvInfo isvInfo = getIsvInfo(openApiRequest.getAppId());
        openApiRequest.setIsvPid(isvInfo.getIsvPid());
        openApiRequest.setOrgOutId(isvInfo.getOrgOutId());
        log.info("--->查询封闭场景的机构用户库id传递参数：{}", JSONObject.toJSONString(openApiRequest));
        try {
            String respStr = HttpUtil.post(iotvspDomain + iotvspGroupIdQueryUri, JSONObject.toJSONString(openApiRequest), 5000);
            log.info("--->查询封闭场景的机构用户库id响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return result;
        } catch (Exception e) {
            log.error("查询封闭场景的机构平台异常.", e);
        }
        return RestResponse.error("查询封闭场景GroupId失败");
    }

    @Override
    public RestResult createOperateRecordWithVidV2(AlipayOperateRecordCreateWithVidOpenApiRequest openApiRequest) {
        log.info("--->自定义vid录入机构用户信息获取操作流水id传递参数：{}", JSONObject.toJSONString(openApiRequest));
        IsvInfo isvInfo = getIsvInfo(openApiRequest.getAppId());
        openApiRequest.setIsvPid(isvInfo.getIsvPid());
        openApiRequest.setOrgOutId(openApiRequest.getOrgOutId() == null ? isvInfo.getOrgOutId() : openApiRequest.getOrgOutId());
        openApiRequest.setComponentOutId(isvInfo.getComponentOutId());
        //初始化vid
        List<AlipayOperateRecordInfoWithVidApiRequest> userInfoList = openApiRequest.getUserInfoList();
        if (Preconditions.isNotBlank(userInfoList)) {
            userInfoList.stream().forEach(item -> {
                item.setVid(DigestUtil.md5Hex(AlipayCertTypeEnum.IDENTITY_CARD.getCode() + openApiRequest.getOrgOutId() + item.getCertNo()));
            });
        }
        openApiRequest.setUserInfoList(userInfoList);
        try {
            String respStr = HttpUtil.post(iotvspDomain + iotvspUserwithvidCreateUri, JSONObject.toJSONString(openApiRequest), 5000);
            log.info("--->自定义vid录入机构用户信息获取操作流水id响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return result;
        } catch (Exception e) {
            log.error("--->自定义vid录入机构用户信息机构平台异常.", e);
        }
        return RestResponse.ok();
    }

    @Override
    public RestResult createOperateRecordWithoutImage(AlipayOpenIotvspUserwithimageRequest openApiRequest) {
        log.info("--->录入机构用户信息并直接入库参数：{}", JSONObject.toJSONString(openApiRequest));
        IsvInfo isvInfo = getIsvInfo(openApiRequest.getAppId());
        openApiRequest.setIsvPid(isvInfo.getIsvPid());
        openApiRequest.setOrgOutId(isvInfo.getOrgOutId());
        openApiRequest.setComponentOutId(isvInfo.getComponentOutIdScenic());
        try {
            String respStr = HttpUtil.post(iotvspDomain + iotvspUserWithoutImageCreateUri, JSONObject.toJSONString(openApiRequest), 5000);
            log.info("--->录入机构用户信息并直接入库响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return result;
        } catch (Exception e) {
            log.error("--->录入机构用户信息并直接入库平台异常.", e);
        }
        return RestResponse.error("用户信息入库失败");
    }

    @Override
    public RestResult createOperateRecordWithoutImageEncryption(AlipayOpenIotvspUserwithimageEncryptionRequest req) {
        //验证签名 placeInfoVO.getAppKey(), placeInfoVO.getAppSecret()
        Boolean checkSign = Sm4Util.checkSign(appKeyScenic, appSecretScenic, req.getSign(), String.valueOf(req.getReqTimestamp()));
        if (!checkSign) {
            throw new CommonException("签名错误");
        }

        //placeInfoVO.getPrivateKey() req.getBizContent()
        String decrypt = Sm4Util.decrypt(privateKeyScenic, req.getBizContent());
        log.info("--->录入机构用户信息并直接入库入参内容解密：{}", decrypt);
        if (Preconditions.isBlank(decrypt)) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        AlipayOpenIotvspUserwithimageRequest openApiRequest = JSONObject.parseObject(decrypt, AlipayOpenIotvspUserwithimageRequest.class);
        if (!checkBizContent(openApiRequest)) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        openApiRequest.setCertType(Preconditions.isNotBlank(openApiRequest) ? openApiRequest.getCertType() : AlipayCertTypeEnum.IDENTITY_CARD.getCode());
        //补全组建参数
        IsvInfo isvInfo = getIsvInfo(appId2189);
        openApiRequest.setAppId(appId2189);
        openApiRequest.setVid(DigestUtil.md5Hex(AlipayCertTypeEnum.IDENTITY_CARD.getCode() + isvInfo.getOrgOutId() + openApiRequest.getCertNo()));
        openApiRequest.setOrgOutId(isvInfo.getOrgOutId());
        openApiRequest.setComponentOutId(isvInfo.getComponentOutIdScenic());
        openApiRequest.setIsvPid(isvInfo.getIsvPid());
        try {
            String respStr = HttpUtil.post(iotvspDomain + iotvspUserWithoutImageCreateUri, JSONObject.toJSONString(openApiRequest), 5000);
            log.info("--->录入机构用户信息并直接入库响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return result;
        } catch (Exception e) {
            log.error("--->录入机构用户信息并直接入库平台异常.", e);
        }
        return RestResponse.ok();
    }

    @Override
    public RestResult deleteUserEncryption(AlipayUserDeleteEncryptionRequest req) {
        //验证签名 placeInfoVO.getAppKey(), placeInfoVO.getAppSecret()
        Boolean checkSign = Sm4Util.checkSign(appKeyScenic, appSecretScenic, req.getSign(), String.valueOf(req.getReqTimestamp()));
        if (!checkSign) {
            throw new CommonException("签名错误");
        }

        //placeInfoVO.getPrivateKey() req.getBizContent()
        String decrypt = Sm4Util.decrypt(privateKeyScenic, req.getBizContent());
        log.info("--->机构用户出库（加密版）出库入参内容解密：{}", decrypt);
        if (Preconditions.isBlank(decrypt)) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        AlipayOpenIotvspUserwithimageRequest openApiUserRequest = JSONObject.parseObject(decrypt, AlipayOpenIotvspUserwithimageRequest.class);
        if (!checkBizContent(openApiUserRequest)) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        AlipayUserDeleteOpenApiRequest openApiRequest = new AlipayUserDeleteOpenApiRequest();
        //补全组建参数
        IsvInfo isvInfo = getIsvInfo(appId2189);
        openApiRequest.setAppId(appId2189);
        openApiRequest.setVidList(Arrays.asList(DigestUtil.md5Hex(AlipayCertTypeEnum.IDENTITY_CARD.getCode() + isvInfo.getOrgOutId() + openApiUserRequest.getCertNo())));
        openApiRequest.setOrgOutId(isvInfo.getOrgOutId());
        openApiRequest.setIsvPid(isvInfo.getIsvPid());
        try {
            String respStr = HttpUtil.post(iotvspDomain + iotvspUserDeleteUri, JSONObject.toJSONString(openApiRequest), 5000);
            log.info("--->机构用户出库（加密版）出库响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return result;
        } catch (Exception e) {
            log.error("--->机构用户出库（加密版）出库平台异常.", e);
        }
        return RestResponse.ok();
    }

    private Boolean checkBizContent(AlipayOpenIotvspUserwithimageRequest withimageReq) {
        if (Preconditions.isBlank(withimageReq.getCertNo())
                || Preconditions.isBlank(withimageReq.getCertName())) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @Override
    public void deleteUser(AlipayUserDeleteOpenApiRequest openApiRequest) {
        executor.execute(() -> {
            IsvInfo isvInfo = getIsvInfo(openApiRequest.getAppId());
            openApiRequest.setIsvPid(isvInfo.getIsvPid());
            openApiRequest.setOrgOutId(StrUtil.isBlank(openApiRequest.getOrgOutId()) ? isvInfo.getOrgOutId() : openApiRequest.getOrgOutId());
            try {
                String respStr = HttpUtil.post(iotvspDomain + iotvspUserDeleteUri, JSONObject.toJSONString(openApiRequest), 5000);
                log.info("--->机构用户出库响应参数：{}", respStr);
            } catch (Exception e) {
                log.error("机构用户出库（自定义vid）机构平台异常.", e);
            }
        });
    }

    @Override
    public RestResult queryKey8Sign(AlipayAmpeSignOpenApiRequest openApiRequest) {
        try {
            String respStr = HttpUtil.post(iotvspDomain + key8SignUri, JSONObject.toJSONString(openApiRequest), 5000);
            log.info("--->获取key8设备签名响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return result;
        } catch (Exception e) {
            log.error("获取key8设备签名异常.", e);
        }
        return RestResponse.error("获取key8设备签名异常");
    }

    @Override
    public RestResult queryUserState(AlipayUserStateOpenApiRequest openApiRequest) {
        IsvInfo isvInfo = getIsvInfo(openApiRequest.getAppId());
        openApiRequest.setIsvPid(isvInfo.getIsvPid());
        openApiRequest.setOrgOutId(isvInfo.getOrgOutId());
        try {
            String respStr = HttpUtil.post(iotvspDomain + iotvspUserstateQueryUri, JSONObject.toJSONString(openApiRequest), 5000);
            log.info("--->查询机构用户在库结果响应参数：{}", JSONObject.toJSONString(respStr));
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return result;
        } catch (Exception e) {
            log.error("查询机构用户在库结果机构平台异常.", e);
        }
        return RestResponse.error("查询机构用户在库结果失败");
    }

    @Override
    public void deleteIotUser(List<Long> userIds) {
        try {
            if (Preconditions.isBlank(userIds)) {
                return;
            }
            List<UserInfo> userInfos = userInfoDao.selectByExample(Example.builder(UserInfo.class).where(WeekendSqls.<UserInfo>custom().andIn(UserInfo::getId, userIds)).build());
            if (Preconditions.isBlank(userInfos)) {
                return;
            }
            List<String> vids = new ArrayList<>();
            userInfos.stream().forEach(item -> {
                if (Preconditions.isNotBlank(item.getIdNumber())) {
                    //目前支持出库2188
                    vids.add(DigestUtil.md5Hex(AlipayCertTypeEnum.IDENTITY_CARD.getCode() + getIsvInfo(null).getOrgOutId() + item.getIdNumber()));
                }
            });
            if (Preconditions.isBlank(vids)) {
                return;
            }
            AlipayUserDeleteOpenApiRequest request = new AlipayUserDeleteOpenApiRequest();
            request.setVidList(vids);
            deleteUser(request);
        } catch (Exception e) {
            log.error("--->用户出库失败");
        }
    }

    private void sendMsg(String phone, String content) {
        String resp = OkHttpUtils.builder().url(ihuyiMsgProperties.getDomain())
                .addParam("account", ihuyiMsgProperties.getAccount())
                .addParam("password", ihuyiMsgProperties.getPassword())
                .addParam("mobile", phone)
                .addParam("content", content)
                .addParam("format", "json")
                .addParam("method", "Submit")
                .addHeader("Content-Type", "application/x-www-form-urlencoded")
                .get()
                .sync();
        JSONObject obj = JSON.parseObject(resp);
        if (obj.get("smsid") == null) {
            log.info("发送短信邀请{}失败", phone);
        } else {
            log.info("短信发送成功：{}", resp);
        }
    }

    private static JSONObject initValue(String info) {
        JSONObject data = new JSONObject();
        data.put("value", info);
        return data;
    }


}
