package com.yx.face.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.dao.AdminDao;
import com.yx.face.model.dto.Barcode2HealthCodeDTO;
import com.yx.face.model.dto.JkmDTO;
import com.yx.face.model.dto.SendMessageDTO;
import com.yx.face.model.dto.XcInfoDTO;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.enums.AdminThreeCodeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.vo.health.Barcode2HealthCodeVO;
import com.yx.face.model.vo.health.HealthCodeVO;
import com.yx.face.model.vo.JkmHsVO;
import com.yx.face.model.vo.health.HsInfo;
import com.yx.face.model.vo.health.JkmInfo;
import com.yx.face.service.ApiHealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @description:
 * @ClassName ApiHealthCodeServiceImpl
 * @Author WXG
 * @Date 2022/8/11 11:32
 */
@Slf4j
@Service
public class ApiHealthCodeServiceImpl implements ApiHealthCodeService {
    @Value("${jkm.request.address}")
    private String JkmAddress;
    @Value("${xc.request.address.sendMessage}")
    private String sendMessageAddress;
    @Value("${xc.request.address.getXcInfo}")
    private String xcInfoAddress;
    @Autowired
    private RestTemplate restTemplate;
    @Resource
    private AdminDao adminDao;
    String appKey = "6f32f95d52c04eb6851b8b0e1a3155ad";
    String appSecret = "1b8f230d5904480ca3b6cc737c1211a2";

    @Value("${jkm.request.queryInfoByBarCodeUrl}")
    private String queryInfoByBarCodeUrl;


    @Override
    public RestResult<Boolean> sendMessage(String phone) {
        SendMessageDTO sendMessageDTO = new SendMessageDTO();
        sendMessageDTO.setPhone(phone);
        sendMessageDTO.setAppKey(appKey);
        sendMessageDTO.setAppSecret(appSecret);
        RestResult result = restTemplate.postForObject(sendMessageAddress,
                sendMessageDTO, RestResult.class
        );
        return result;
    }

    @Override
    public RestResult<String> getXcInfo(XcInfoDTO xcInfoDTO) {
        if (xcInfoDTO.getAdminId() != null) {
            Admin admin = adminDao.selectByPrimaryKey(xcInfoDTO.getAdminId());
            if (admin != null && admin.getType().equals(AdminTypeEnum.DISTRICT.getCode()) && StringUtils.hasText(admin.getRiskArea())) {
                xcInfoDTO.setCity(admin.getRiskArea());
            }
        }
        xcInfoDTO.setAppKey(appKey);
        xcInfoDTO.setAppSecret(appSecret);
        String result = restTemplate.postForObject(xcInfoAddress,
                xcInfoDTO, String.class
        );
        try {
            JSONObject jsonObject = JSON.parseObject(result);
            JSONObject datas = jsonObject.getJSONObject("datas");
            JSONObject data = datas.getJSONObject("data");
            JSONObject result1 = data.getJSONObject("result");
            return RestResponse.ok(result1.getString("value"));
        } catch (Exception exception) {
            try {
                JSONObject jsonObject = JSON.parseObject(result);
                JSONObject datas = jsonObject.getJSONObject("datas");
                JSONObject data = datas.getJSONObject("data");
                String errorDesc = data.getString("errorDesc");
                return RestResponse.error(errorDesc);
            } catch (Exception exception1) {
                return RestResponse.error("查询失败");
            }
        }
    }

    @Override
    public RestResult<JkmHsVO> getJkmAndHs(JkmDTO jkmDTO) {
        if (jkmDTO == null || !StringUtils.hasText(jkmDTO.getIdNumber())) {
            return RestResponse.ok(new JkmHsVO());
        }
        SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
        requestFactory.setConnectTimeout(3000);
        requestFactory.setReadTimeout(3000);
        RestTemplate restTemplate = new RestTemplate(requestFactory);
        RestResult restResult = null;
        try {
            restResult = restTemplate.postForObject(
                    JkmAddress,
                    jkmDTO, RestResult.class);
        } catch (Exception e) {
            return RestResponse.ok(new JkmHsVO());
        }

        if (restResult == null || restResult.getData() == null) {
            return RestResponse.ok(new JkmHsVO());
        }

        Boolean isHs = false;
        Boolean isJkm = false;

        /*查看账号是否需要核酸和健康码*/
        if (jkmDTO.getAdminId() != null) {
            Admin admin = adminDao.selectByPrimaryKey(jkmDTO.getAdminId());
            if (admin != null && StringUtils.hasText(admin.getThreeCode())) {
                if (admin.getThreeCode().contains(AdminThreeCodeEnum.JKM.getCode().toString())) isJkm = true;
                if (admin.getThreeCode().contains(AdminThreeCodeEnum.HS.getCode().toString())) isHs = true;
            }
        }

        HealthCodeVO healthCodeVO = JSON.parseObject(JSON.toJSONString(restResult.getData()), HealthCodeVO.class);
        if (healthCodeVO != null) {
            JkmHsVO jkmHsVO = new JkmHsVO();
            if (!CollectionUtils.isEmpty(healthCodeVO.getNuclein())) {
                HsInfo hsInfo = healthCodeVO.getNuclein().get(0);
                if (hsInfo.getChecktime() != null && isHs) {
                    jkmHsVO.setHsStatus(1);
                    String result = hsInfo.getResult();
                    jkmHsVO.setHsCheckTime(com.alibaba.excel.util.DateUtils.format(hsInfo.getReportTime(), DateUtils.YYYY_MM_DD_HH_MM_SS));
                    jkmHsVO.setHsResultTime(com.alibaba.excel.util.DateUtils.format(hsInfo.getChecktime(), DateUtils.YYYY_MM_DD_HH_MM_SS));
                    int hour = DateUtils.getDatePoorHour(hsInfo.getChecktime(), new Date());
                    if (hour < 24) jkmHsVO.setHs("24小时" + result);
                    else if (hour < 48) jkmHsVO.setHs("48小时" + result);
                    else if (hour < 72) jkmHsVO.setHs("72小时" + result);
                    else {
                        jkmHsVO.setHs((hour / 24 + 1) + "天" + result);
                    }
                }
                if (!isHs) jkmHsVO.setHsStatus(null);
            }
            if (!CollectionUtils.isEmpty(healthCodeVO.getData())) {
                JkmInfo jkmInfo = healthCodeVO.getData().get(0);
                if (StringUtils.hasText(jkmInfo.getMzt())) {
                    jkmHsVO.setJkmStatus(1);
                    jkmHsVO.setJkm(jkmInfo.getMzt());
                }
                if (!isJkm) jkmHsVO.setJkmStatus(null);
            }
            return RestResponse.ok(jkmHsVO);
        }
        return RestResponse.ok(new JkmHsVO());
    }

    @Override
    public RestResult<Barcode2HealthCodeVO> queryInfoByBarCode(Barcode2HealthCodeDTO dto) {
        SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
        requestFactory.setConnectTimeout(3000);
        requestFactory.setReadTimeout(3000);
        RestTemplate restTemplate = new RestTemplate(requestFactory);
        RestResult restResult = null;
        try {
            dto.setAppSecret(appSecret);
            dto.setAppKey(appKey);
            restResult = restTemplate.postForObject(queryInfoByBarCodeUrl, dto, RestResult.class);
            log.info("--->barcode获取用户信息返回参数：{}", JSONObject.toJSONString(restResult));
        } catch (Exception e) {
            e.printStackTrace();
            return RestResponse.ok(new Barcode2HealthCodeVO());
        }

        if (restResult == null || restResult.getData() == null) {
            return RestResponse.ok(new Barcode2HealthCodeVO());
        }

        Barcode2HealthCodeVO codeVO = JSON.parseObject(JSON.toJSONString(restResult.getData()), Barcode2HealthCodeVO.class);
        return RestResponse.ok(codeVO);
    }


}
