package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.Base64Util;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.*;
import com.yx.face.model.api.appointment.dto.*;
import com.yx.face.model.api.appointment.vo.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.UserRightNoEnum;
import com.yx.face.model.vo.ContextVO;
import com.yx.face.model.vo.FaceDeviceLessVO;
import com.yx.face.model.vo.UserInfoVO;
import com.yx.face.model.vo.eth.EthCodeDto;
import com.yx.face.model.vo.eth.EthCodeVO;
import com.yx.face.service.AppointmentApiService;
import com.yx.face.service.BaiDuFaceService;
import com.yx.face.service.FaceService;
import com.yx.face.service.FaceTBService;
import com.yx.face.service.SpiIotUserWhitelistService;
import com.yx.face.service.TaskInfoService;
import com.yx.face.service.UserBlackListService;
import com.yx.face.service.UserWhitelistService;
import com.yx.face.service.fkj.FkjService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.Weekend;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName AppointmentApiServiceImpl
 * @Author WXG
 * @Date 2022/5/24 14:08
 */
@Service
public class AppointmentApiServiceImpl implements AppointmentApiService {

    @Resource
    private AdminDao adminDao;

    @Resource
    private PlaceDetailDao placeDetailDao;

    @Resource
    private UserInfoDao userInfoDao;

    @Resource
    private AppointmentPersonDao appointmentPersonDao;

    @Resource
    private ReservationRecordDao reservationRecordDao;

    @Resource
    private PlaceDao placeDao;

    @Resource
    private UserControlEarlyWarningListDao userControlEarlyWarningListDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Autowired
    private FaceTBService faceTBService;

    @Resource
    private FaceLogDao faceLogDao;

    @Resource
    private TaskInfoService taskInfoService;

    @Resource
    private FaceService faceService;

    @Resource
    private SpiIotUserWhitelistService spiIotUserWhitelistService;

    @Resource
    private BaiDuFaceService baiDuFaceService;

    @Resource
    private UserBlackListService userBlackListService;

    @Resource
    private UserWhitelistService userWhitelistService;

    @Resource
    private VisiteeDao visiteeDao;

    @Resource
    private FkjService fkjService;

    @Override
    public AppointmentInfoVO getAppointmentInfo(AppointmentInfoDTO appointmentInfoDTO) {
        /*
         * 根据前台传进来的adminId和userId查询用户是否已经进行预约了
         * */
        Integer adminId = appointmentInfoDTO.getAdminId();
        Admin admin = adminDao.selectByPrimaryKey(adminId);
        if (admin.getPlaceType() != 1) {//如果不为1代表不是预约场所的账号
            return returnAppointment(1);
        }
        /*如果为预约场景需要查询账号是否有可以进行预约的时间段
         * 有几种情况
         * 1。该账号在当前时间之后的可预约时段为空 返回type=1
         * 2.该账号在当前时间之后的可预约时段都被预约满了 返回type=4
         * 3.该账号在当前时间之后存在可预约的时间段，并可以预约 返回type=2
         * 4.根据用户信息查询预约记录发现用户已经预约，返回type=3
         * */
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Integer userId = appointmentInfoDTO.getUserId();
        Weekend<PlaceDetail> weekend = new Weekend<>(PlaceDetail.class);
        weekend.weekendCriteria()
                .andGreaterThan(PlaceDetail::getEndTime, sdf.format(date))
                .andEqualTo(PlaceDetail::getAdminId, admin.getId())
                .andEqualTo(PlaceDetail::getDelFlag, 0)
                .andEqualTo(PlaceDetail::getStatus, 1)
        ;
        List<PlaceDetail> placeDetailList = placeDetailDao.selectByExample(weekend);

        if (placeDetailList == null || placeDetailList.size() == 0) {
            return returnAppointment(1);
        }

        /*查询出剩余可预约人数大于0的时间段*/
        placeDetailList = placeDetailList.stream().filter(p -> p.getRemainingNumber() > 0).collect(Collectors.toList());
        if (placeDetailList.size() == 0) {
            return returnAppointment(4);
        }

        if (userId == null) {
            return returnAppointment(2);
        } else {
            UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
            AppointmentPerson appointmentPerson = appointmentPersonDao.selectOne(new AppointmentPerson().setIdNumber(userInfo.getIdNumber()).setAdminId(adminId));
            if (appointmentPerson == null) {
                return returnAppointment(2);
            } else {
                Weekend<ReservationRecord> rrWeekend = new Weekend<>(ReservationRecord.class);
                rrWeekend.weekendCriteria()
                        .andGreaterThan(ReservationRecord::getEndTime, sdf.format(date))
                        .andEqualTo(ReservationRecord::getAdminId, admin.getId())
                        .andEqualTo(ReservationRecord::getAppointmentPersonId, appointmentPerson.getId());
                ReservationRecord reservationRecord = reservationRecordDao.selectOneByExample(rrWeekend);
                if (reservationRecord == null) {
                    return returnAppointment(2);
                } else {
                    return returnAppointment(3);
                }
            }
        }
    }

    @Override
    public List<DateVO> getTimeList(Integer adminId) {
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
        Weekend<PlaceDetail> weekend = new Weekend<>(PlaceDetail.class);
        weekend.weekendCriteria()
                .andGreaterThan(PlaceDetail::getEndTime, sdf.format(date))
                .andGreaterThan(PlaceDetail::getRemainingNumber, 0)
                .andEqualTo(PlaceDetail::getAdminId, adminId)
                .andEqualTo(PlaceDetail::getDelFlag, 0)
                .andEqualTo(PlaceDetail::getStatus, 1);
        List<PlaceDetail> placeDetailList = placeDetailDao.selectByExample(weekend);
        return placeDetailList.stream().sorted((p1, p2) -> {
            if (p1.getStartTime().getTime() < p2.getStartTime().getTime()) {
                return -1;
            }
            if (p1.getStartTime().getTime() > p2.getStartTime().getTime()) {
                return 1;
            }
            return 0;
        }).map(p -> sdf1.format(p.getStartTime())).distinct().map(p -> {
            DateVO dateVO = new DateVO();
            dateVO.setDateTime(p);
            return dateVO;
        }).collect(Collectors.toList());
    }

    @Override
    public List<TimeSlotVO> getTimeSlotList(TimeSlotDTO timeSlotDTO) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat("HH:mm");
        try {
            Date today = sdf1.parse(timeSlotDTO.getChooseDate());
//            Date tomorrow = DateUtils.addDays(today, 1);
            DateTime tomorrow = DateUtil.endOfDay(today);
            Weekend<PlaceDetail> weekend = new Weekend<>(PlaceDetail.class);
            weekend.weekendCriteria()
                    .andBetween(PlaceDetail::getStartTime, sdf.format(today), sdf.format(tomorrow))
                    .andGreaterThan(PlaceDetail::getRemainingNumber, 0)
                    .andEqualTo(PlaceDetail::getAdminId, timeSlotDTO.getAdminId())
                    .andEqualTo(PlaceDetail::getDelFlag, 0)
                    .andEqualTo(PlaceDetail::getStatus, 1);
            List<PlaceDetail> placeDetailList = placeDetailDao.selectByExample(weekend);
            Date now = new Date();
            if (timeSlotDTO.getChooseDate().equals(sdf1.format(now))) {
                placeDetailList = placeDetailList.stream().filter(p -> p.getEndTime().getTime() > now.getTime()).collect(Collectors.toList());
            }
            return placeDetailList.stream().map(p -> {
                TimeSlotVO timeSlotVO = new TimeSlotVO();
                timeSlotVO.setTimeSlot(sdf2.format(p.getStartTime()) + " - " + sdf2.format(p.getEndTime()));
                timeSlotVO.setPlaceDetailId(p.getId());
                timeSlotVO.setRemainingNumber(p.getRemainingNumber());
                return timeSlotVO;
            }).collect(Collectors.toList());
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public RestResult<Boolean> selectAuthenticationAuthority(AuthenticationAuthorityDTO authenticationAuthorityDTO) {
        Place place = placeDao.selectOne(new Place().setAdminId(authenticationAuthorityDTO.getAdminId()));
        if (place != null && StringUtils.isNotEmpty(place.getDisableIdCard())) {
            if (authenticationAuthorityDTO.getIdNumber().indexOf(place.getDisableIdCard()) == 0) {
                return RestResponse.error("您无法进行实名认证");
            }
        }

        /*根据身份证和手机号查询布控名单*/
        if (!StringUtils.isNotEmpty(authenticationAuthorityDTO.getIdNumber()) && !StringUtils.isNotEmpty(authenticationAuthorityDTO.getPhone())) {
            return RestResponse.error("请输入身份证");
        }

        UserControlEarlyWarningList userControlEarlyWarningList = userControlEarlyWarningListDao.selectAuthenticationAuthority(authenticationAuthorityDTO);
        if (userControlEarlyWarningList == null) {
            return RestResponse.ok(true);
        } else {
            return RestResponse.error("您无法进行实名认证");
        }
    }

    @Override
    @Transactional
    public RestResult<Boolean> appointment(AppointmentDTO appointmentDTO) {
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(appointmentDTO.getUserId());
        PlaceDetail placeDetail = placeDetailDao.selectByPrimaryKey(appointmentDTO.getPlaceDetailId());
        if (placeDetail.getDelFlag() || !placeDetail.getStatus()) {
            return RestResponse.error("该预约时段已经停用");
        }
        Admin admin = adminDao.selectByPrimaryKey(appointmentDTO.getAdminId());
        if (admin != null && IdcardUtil.isValidCard(userInfo.getIdNumber()) && StrUtil.isNotEmpty(admin.getNotAllowAge())) {
            int age = IdcardUtil.getAgeByIdCard(userInfo.getIdNumber());
            String[] list = admin.getNotAllowAge().split("-");
            if (list.length == 2) {
                int notAllowAgeStart = Integer.parseInt(list[0]);
                int notAllowAgeEnd = Integer.parseInt(list[1]);
                if (notAllowAgeStart <= age && age <= notAllowAgeEnd) {
                    return RestResponse.error("因年龄限制，你不可以预约");
                }
            }
        }

        /*1.数据填入预约人员列表（根据身份证查询）
         *  有-更新 无-添加
         * */
        AppointmentPerson aPerson = this.saveAppointmentPerson(userInfo, appointmentDTO.getType(), appointmentDTO.getAdminId());
        /*2.数据填入预约人员记录表*/
        this.getReservationRecord(aPerson, placeDetail, appointmentDTO);
        /*3.修改预约人数*/
        updatePlaceDetail(placeDetail);
        /*4.人脸下发*/
        downUser(userInfo, appointmentDTO.getAdminId(), placeDetail);

        return RestResponse.ok(true);
    }

    @Override
    public RestResult<Boolean> updateUserInfo(UpdateUserInfoDTO updateUserInfoDTO) {
        /*1.获取用户信息，并判断该用户是否为外宾*/
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(updateUserInfoDTO.getUserId());
        if (userInfo == null) {
            throw new ServiceException("数据有误");
        }
        if (userInfo.getType() != 1) {
            throw new ServiceException("您无法修改数据");
        }
        if (StringUtils.isNotEmpty(updateUserInfoDTO.getUsername())) {
            userInfo.setUsername(updateUserInfoDTO.getUsername());
        }
        if (StringUtils.isNotEmpty(updateUserInfoDTO.getPhone())) {
            userInfo.setPhone(updateUserInfoDTO.getPhone());
        }
        if (StringUtils.isNotEmpty(updateUserInfoDTO.getIdNumber())) {
            userInfo.setIdNumber(updateUserInfoDTO.getIdNumber());
            if (SMSOrIdCardUtils.isIdCard(updateUserInfoDTO.getIdNumber())) {
                userInfo.setBirthday(SMSOrIdCardUtils.getBirthDate(updateUserInfoDTO.getIdNumber()));
                userInfo.setGenderStr(Preconditions.isNotBlank(SMSOrIdCardUtils.getGender(updateUserInfoDTO.getIdNumber())) ? SMSOrIdCardUtils.getGender(updateUserInfoDTO.getIdNumber()).getDesc() : null);
            }
        }
        if (StringUtils.isNotEmpty(updateUserInfoDTO.getAvatar())) {
            userInfo.setAvatar(updateUserInfoDTO.getAvatar());
            //获取百度人脸特征值
            String facialFeature = null;
            if (Preconditions.isNotBlank(updateUserInfoDTO.getAvatar())) {
                facialFeature = baiDuFaceService.baiDuFaceCharacteristic(Base64Util.imgToBase64(updateUserInfoDTO.getAvatar()));
            }
            userInfo.setFacialFeature(facialFeature);
        }
        userInfoDao.updateByPrimaryKey(userInfo);

        return RestResponse.ok(true);
    }

    @Override
    public RestResult<LookAppointmentVO> lookAppointment(UserAdminDTO userAdminDTO) {

        /*1.获取用户信息*/
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userAdminDTO.getUserId());
        if (userInfo == null) {
            throw new ServiceException("请先认证");
        }
        /*2.通过adminId和用户信息是否存在该用户*/
        AppointmentPerson aPerson = new AppointmentPerson();
        aPerson.setIdNumber(userInfo.getIdNumber());
        aPerson.setAdminId(userAdminDTO.getAdminId());
        aPerson = appointmentPersonDao.selectOne(aPerson);
        if (aPerson == null) {
            return RestResponse.error("你没有预约");
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Weekend<ReservationRecord> rrWeekend = new Weekend<>(ReservationRecord.class);
        rrWeekend.weekendCriteria()
                .andGreaterThan(ReservationRecord::getEndTime, sdf.format(new Date()))
                .andEqualTo(ReservationRecord::getAdminId, userAdminDTO.getAdminId())
                .andEqualTo(ReservationRecord::getAppointmentPersonId, aPerson.getId());
        ReservationRecord reservationRecord = reservationRecordDao.selectOneByExample(rrWeekend);
        if (reservationRecord == null) {
            return RestResponse.error("你没有预约");
        }
        /*3.如果有则返回数据*/
        LookAppointmentVO lookAppointmentVO = new LookAppointmentVO();
        lookAppointmentVO.setReservationRecordId(reservationRecord.getId());
        lookAppointmentVO.setUsername(userInfo.getUsername());
        lookAppointmentVO.setPhone(userInfo.getPhone());
        lookAppointmentVO.setIdNumber(userInfo.getIdNumber());
        lookAppointmentVO.setAvatar(userInfo.getAvatar());
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat("HH:mm");
        lookAppointmentVO.setDateTime(sdf1.format(reservationRecord.getStartTime()));
        lookAppointmentVO.setTimeSlot(sdf2.format(reservationRecord.getStartTime()) + " - " + sdf2.format(reservationRecord.getEndTime()));
        lookAppointmentVO.setPlaceTitle(reservationRecord.getTitle());
        if (StringUtils.isNotEmpty(reservationRecord.getContent())) {
            lookAppointmentVO.setContents(JSON.parseArray(reservationRecord.getContent(), ContextVO.class));
        }
        return RestResponse.ok(lookAppointmentVO);
    }

    @Override
    public RestResult<List<LookAppointmentVO>> lookAppointmentRecord(UserAdminDTO userAdminDTO) {

        /*1.获取用户信息*/
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userAdminDTO.getUserId());
        /*2.通过adminId和用户信息是否存在该用户*/
        AppointmentPerson aPerson = new AppointmentPerson();
        aPerson.setIdNumber(userInfo.getIdNumber());
        aPerson.setAdminId(userAdminDTO.getAdminId());
        aPerson = appointmentPersonDao.selectOne(aPerson);
        if (aPerson == null) {
            return RestResponse.error("你没有预约记录");
        }
        Weekend<ReservationRecord> rrWeekend = new Weekend<>(ReservationRecord.class);
        rrWeekend.weekendCriteria()
                .andEqualTo(ReservationRecord::getAdminId, userAdminDTO.getAdminId())
                .andEqualTo(ReservationRecord::getAppointmentPersonId, aPerson.getId());
        List<ReservationRecord> recordList = reservationRecordDao.selectByExample(rrWeekend);
        if (recordList == null || recordList.size() == 0) {
            return RestResponse.error("你没有预约");
        }
        recordList = recordList.stream().sorted(Comparator.comparing(ReservationRecord::getId).reversed()).collect(Collectors.toList());
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat("HH:mm");
        List<LookAppointmentVO> list = recordList.stream().map(p -> {
            LookAppointmentVO lookAppointmentVO = new LookAppointmentVO();
            lookAppointmentVO.setUsername(userInfo.getUsername());
            lookAppointmentVO.setPhone(userInfo.getPhone());
            lookAppointmentVO.setIdNumber(userInfo.getIdNumber());
            lookAppointmentVO.setAvatar(userInfo.getAvatar());
            lookAppointmentVO.setDateTime(sdf1.format(p.getStartTime()));
            lookAppointmentVO.setTimeSlot(sdf2.format(p.getStartTime()) + " - " + sdf2.format(p.getEndTime()));
            lookAppointmentVO.setPlaceTitle(p.getTitle());
            if (StringUtils.isNotEmpty(p.getContent())) {
                lookAppointmentVO.setContents(JSON.parseArray(p.getContent(), ContextVO.class));
            }
            return lookAppointmentVO;
        }).collect(Collectors.toList());

        return RestResponse.ok(list);
    }

    @Override
    public RestResult<UserInfoVO> foreignAuthentication(ForeignAuthenticationDTO foreignAuthenticationDTO) {

        String name = foreignAuthenticationDTO.getUsername();
        String phone = foreignAuthenticationDTO.getPhone();
        String idNumber = foreignAuthenticationDTO.getIdNumber();
        String avatar = foreignAuthenticationDTO.getAvatar();
        if (!org.springframework.util.StringUtils.hasText(idNumber)) {
            return RestResponse.error("证件号为空");
        }
        if (SMSOrIdCardUtils.isIdCard(idNumber)) {
            return RestResponse.error("护照号不应该为身份证号");
        }

        //获取百度人脸特征值
        String facialFeature = null;
        if (Preconditions.isNotBlank(avatar)) {
            facialFeature = baiDuFaceService.baiDuFaceCharacteristic(Base64Util.imgToBase64(avatar));
        }

        Date now = new Date();
        UserInfo userInfo = new UserInfo();
        userInfo.setIdNumber(idNumber);
        List<UserInfo> userInfoList = userInfoDao.select(userInfo);
        if (userInfoList == null || userInfoList.isEmpty()) {
            userInfo = new UserInfo();
            userInfo.setUsername(name);
            userInfo.setPhone(phone);
            userInfo.setIdNumber(idNumber);
            userInfo.setType(1);
            userInfo.setAvatar(avatar);
            userInfo.setFacialFeature(facialFeature);
            userInfo.setCountryCode(foreignAuthenticationDTO.getCountryCode());
            userInfo.setGenderStr(foreignAuthenticationDTO.getGender());
            userInfo.setBirthday(foreignAuthenticationDTO.getBirthday());
            userInfo.setUpdateTime(now);
            userInfo.setCreateTime(now);
            userInfo.setHztx(foreignAuthenticationDTO.getHztx());
            userInfoDao.insertSelective(userInfo);
            spiIotUserWhitelistService.sendUserInfoAddMq(userInfo.getId());
        } else {
            userInfo = userInfoList.get(0);
            if (userInfo.getType() != null && userInfo.getType() != 1) {
                return RestResponse.error("不是外宾不可修改");
            }
            userInfo.setUsername(StrUtil.isNotEmpty(name) ? name : null);
            userInfo.setPhone(StrUtil.isNotEmpty(phone) ? phone : null);
            userInfo.setAvatar(StrUtil.isNotEmpty(avatar) ? avatar : null);
            userInfo.setIdNumber(idNumber);
            userInfo.setCountryCode(foreignAuthenticationDTO.getCountryCode());
            userInfo.setGenderStr(foreignAuthenticationDTO.getGender());
            userInfo.setBirthday(foreignAuthenticationDTO.getBirthday());
            userInfo.setHztx(foreignAuthenticationDTO.getHztx());
            userInfo.setFacialFeature(facialFeature);
            userInfoDao.updateByPrimaryKeySelective(userInfo);
            spiIotUserWhitelistService.sendUserInfoUpdateMq(userInfo.getId());
        }
        UserInfoVO userInfoVO = new UserInfoVO();
        BeanUtils.copyProperties(userInfo, userInfoVO);
        userInfoVO.setUserId(userInfo.getId());
        return RestResponse.ok(userInfoVO);
    }

    @Override
    public PlaceInfoVO getInfo(Integer adminId) {
        if (adminId == null) {
            throw new ServiceException("数据有误");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat("HH:mm");
        PlaceDetail placeDetail = placeDetailDao.getInfo(adminId, sdf.format(new Date()));
        PlaceInfoVO placeInfoVO = new PlaceInfoVO();
        Place place = placeDao.selectOne(new Place().setAdminId(adminId));
        if (place != null) {
            placeInfoVO.setPlaceTitle(place.getTitle());
            placeInfoVO.setPicture(place.getPicture());
            placeInfoVO.setInstructions(place.getInstructions());
        }
        if (placeDetail != null) {
            placeInfoVO.setLimitNumber(placeDetail.getLimitNumber());
            placeInfoVO.setDateTime(sdf1.format(placeDetail.getStartTime()));
            placeInfoVO.setTimeSlot(sdf2.format(placeDetail.getStartTime()) + " - " + sdf2.format(placeDetail.getEndTime()));
        }
        return placeInfoVO;
    }

    @Override
    public RestResult<Boolean> cancelAppointment(CancelAppointmentDTO cancelAppointmentDTO) {
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(cancelAppointmentDTO.getUserId());
        if (userInfo == null) {
            throw new ServiceException("数据有误");
        }
        ReservationRecord reservationRecord = reservationRecordDao.selectByPrimaryKey(cancelAppointmentDTO.getReservationRecordId());
        if (reservationRecord == null) {
            throw new ServiceException("数据有误");
        }
        Weekend<FaceLog> rrWeekend = new Weekend<>(FaceLog.class);
        rrWeekend.weekendCriteria()
                .andEqualTo(FaceLog::getAdminId, reservationRecord.getAdminId())
                .andBetween(FaceLog::getFaceTime, reservationRecord.getStartTime(), reservationRecord.getEndTime())
                .andEqualTo(FaceLog::getCardid, userInfo.getIdNumber());

        List<FaceLog> recordList = faceLogDao.selectByExample(rrWeekend);
        if (recordList != null && recordList.size() > 0) {
            throw new ServiceException("您已经进入场所无法取消");
        }
        /*1.修改剩余预约人数*/
        PlaceDetail placeDetail = placeDetailDao.selectByPrimaryKey(reservationRecord.getPlaceDetailId());
        synchronized (this) {
            placeDetail.setRemainingNumber(placeDetail.getRemainingNumber() + 1);
            placeDetailDao.updateByPrimaryKeySelective(placeDetail);
        }
        /*2.删除数据*/
        reservationRecordDao.deleteByPrimaryKey(cancelAppointmentDTO.getReservationRecordId());
        /*3.删除设备上的人脸*/
        List<Long> list = new ArrayList<>();
        list.add(userInfo.getId());
        faceTBService.delFace(list, placeDetail.getAdminId());
        return RestResponse.ok(true);
    }

    @Override
    public RestResult<UserInfoVO> getUserInfoVO(Integer userId) {
        if (userId == null) {
            throw new ServiceException("请传入参数");
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
        if (userInfo == null) {
            throw new ServiceException("数据有误");
        }
        UserInfoVO userInfoVO = BeanUtil.copyProperties(userInfo, UserInfoVO.class);
        userInfoVO.setUserId(userInfo.getId());
        return RestResponse.ok(userInfoVO);
    }

    @Override
    public RestResult<EthCodeVO> getUserEthCode(EthCodeDto dto) {
        Integer adminId = dto.getAdminId();
        Long userId = dto.getUserId();
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
        if (userInfo == null || Preconditions.isBlank(userInfo.getIdNumber())) {
            throw new ServiceException("数据有误");
        }
        Admin admin = adminDao.selectByPrimaryKey(adminId);
        ClassUtil.checkData(admin, "账号不存在");
        if (Preconditions.isBlank(admin.getIsOpenLadder()) || !admin.getIsOpenLadder()) {
            throw new CommonException("所属账号未开启梯控");
        }
        UserBlackList userBlackList = userBlackListService.getByPhoneAndIdNumber(userInfo.getIdNumber(), userInfo.getPhone(), adminId);
        ClassUtil.throwBusinessException(ObjectUtil.isNotNull(userBlackList), "该用户被填加入了黑名单");
        //用户是否为常客
        UserWhitelist userWhitelist = userWhitelistService.getUserWhitelist(userInfo.getIdNumber(), userInfo.getPhone(), null, adminId);
        ClassUtil.throwBusinessException(ObjectUtil.isNull(userWhitelist), "用户信息不存在");
        ClassUtil.throwBusinessException(ObjectUtil.isNull(userWhitelist.getIdNumber()), "您未完成实名认证");
        Date timeStart = userWhitelist.getStartTime();
        Date timeEnd = userWhitelist.getEndTime();
        if (Preconditions.isBlank(timeStart)) {
            timeStart = DateUtil.offsetMinute(DateUtil.date(), -5);
            timeEnd = DateUtil.offsetMinute(DateUtil.date(), admin.getUserVisiterEffectiveTime().multiply(new BigDecimal(1440)).intValue());
        }
        //根据常客查询受访人
        List<Visitee> visitees = visiteeDao.selectByExample(Example.builder(Visitee.class).where(WeekendSqls.<Visitee>custom().andEqualTo(Visitee::getAdminId, adminId).andEqualTo(Visitee::getIdNumber, userInfo.getIdNumber())).build());
        if (Preconditions.isBlank(visitees)) {
            throw new CommonException(4004, "未查询到您的楼层信息!");
        }
        Visitee visitee = visitees.get(0);
        if (Preconditions.isBlank(visitee.getFloor())) {
            throw new CommonException(4004, "未查询到您的楼层信息!");
        }
        EthCodeVO vo = BeanUtil.copyProperties(userWhitelist, EthCodeVO.class);
        vo.setUsername(userWhitelist.getName());
        //梯控二维码
        try {
            String ladderQrCode = fkjService.qrCodeCreate(Preconditions.isNotBlank(visitee.getFloor()) ? visitee.getFloor().replace("楼", "") : null,
                    timeStart, timeEnd,
                    admin.getId(),
                    admin.getUsername(),
                    userWhitelist.getIdNumber(), userWhitelist.getName(),
                    userWhitelist.getPhone(), userWhitelist.getName());
            vo.setEthCode(ladderQrCode);
        } catch (Exception e) {
            throw new CommonException("获取梯控二维码失败");
        }
        return RestResponse.ok(vo);
    }


    private AppointmentInfoVO returnAppointment(Integer type) {
        AppointmentInfoVO appointmentInfoVO = new AppointmentInfoVO();
        appointmentInfoVO.setType(type);
        return appointmentInfoVO;
    }

    private synchronized AppointmentPerson saveAppointmentPerson(UserInfo userInfo, Integer type, Integer adminId) {
        AppointmentPerson aPerson = new AppointmentPerson();
        aPerson.setIdNumber(userInfo.getIdNumber());
        aPerson.setAdminId(adminId);
        aPerson = appointmentPersonDao.selectOne(aPerson);
        Date date = new Date();
        if (aPerson == null) {
            aPerson = new AppointmentPerson();
            aPerson.setAdminId(adminId);
            aPerson.setType(userInfo.getType());
            aPerson.setCreateTime(date);
            aPerson.setUpdateTime(date);
            aPerson.setIdNumber(userInfo.getIdNumber());
            aPerson.setName(userInfo.getUsername());
            aPerson.setPhone(userInfo.getPhone());
            aPerson.setStatus(1);
            aPerson.setAvatar(userInfo.getAvatar());
            appointmentPersonDao.insertSelective(aPerson);
        } else {
            aPerson.setUpdateTime(date);
            aPerson.setName(userInfo.getUsername());
            aPerson.setPhone(userInfo.getPhone());
            aPerson.setAvatar(userInfo.getAvatar());
            appointmentPersonDao.updateByPrimaryKeySelective(aPerson);
        }
        return aPerson;
    }

    private ReservationRecord getReservationRecord(AppointmentPerson aPerson, PlaceDetail placeDetail, AppointmentDTO appointmentDTO) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Weekend<ReservationRecord> rrWeekend = new Weekend<>(ReservationRecord.class);
        rrWeekend.weekendCriteria()
                .andGreaterThan(ReservationRecord::getEndTime, sdf.format(new Date()))
                .andEqualTo(ReservationRecord::getAppointmentPersonId, aPerson.getId())
                .andEqualTo(ReservationRecord::getPlaceId, placeDetail.getPlaceId());
        ReservationRecord reservationRecord = reservationRecordDao.selectOneByExample(rrWeekend);
        if (reservationRecord != null) {
            throw new ServiceException("请不要重复预约");
        }
        Place place = placeDao.selectByPrimaryKey(placeDetail.getPlaceId());
        if (place == null) {
            throw new ServiceException("数据有误");
        }
        Date date = new Date();
        reservationRecord = new ReservationRecord();
        reservationRecord.setAdminId(appointmentDTO.getAdminId());
        reservationRecord.setAppointmentPersonId(aPerson.getId());
        reservationRecord.setCreateTime(date);
        reservationRecord.setUpdateTime(date);
        reservationRecord.setPlaceDetailId(placeDetail.getId());
        reservationRecord.setPlaceId(placeDetail.getPlaceId());
        reservationRecord.setStartTime(placeDetail.getStartTime());
        reservationRecord.setEndTime(placeDetail.getEndTime());
        reservationRecord.setTitle(place.getTitle());
        /*自定义字段*/
        if (!CollectionUtils.isEmpty(appointmentDTO.getAddCustomList())) {
            String content = JSON.toJSONString(appointmentDTO.getAddCustomList());
            reservationRecord.setContent(content);
        }
        reservationRecordDao.insertSelective(reservationRecord);
        return reservationRecord;
    }

    private void updatePlaceDetail(PlaceDetail placeDetail) {
        Object o = new Object();
        synchronized (o) {
            placeDetail = placeDetailDao.selectByPrimaryKey(placeDetail.getId());
            if (placeDetail.getRemainingNumber() <= 0) {
                throw new ServiceException("预约已满");
            }
            placeDetail.setRemainingNumber(placeDetail.getRemainingNumber() - 1);
            placeDetailDao.updateByPrimaryKeySelective(placeDetail);
        }
    }

    private void downUser(UserInfo userInfo, Integer adminId, PlaceDetail placeDetail) {
        List<FaceDeviceLessVO> snList = faceDeviceDao.getSnList(adminId);
        if (CollectionUtil.isEmpty(snList)) {
            return;
        }
        List<String> snStrList = snList.stream().map(FaceDeviceLessVO::getSn).collect(Collectors.toList());
        for (String sn : snStrList) {
            DownFaceNew downFaceNew = new DownFaceNew();
            downFaceNew.setCardid(userInfo.getIdNumber());
            downFaceNew.setRightno(UserRightNoEnum.APPOINTMENT.getCode());
            downFaceNew.setPhoto(userInfo.getAvatar());
            downFaceNew.setName(userInfo.getUsername());
            downFaceNew.setUserId(userInfo.getId());
            downFaceNew.setSn(sn);
            downFaceNew.setPhone(userInfo.getPhone());
            downFaceNew.setValidPeriodStart(placeDetail.getStartTime());
            downFaceNew.setValidPeriodEnd(placeDetail.getEndTime());
            downFaceNew.setFacialFeature(userInfo.getFacialFeature());
            downFaceNew.setAdminId(adminId);
            faceTBService.downUser(downFaceNew);
        }
    }
}
