package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.Base64Util;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.*;
import com.yx.face.model.dto.IssueAllDTO;
import com.yx.face.model.dto.AppointmentPersonModifyDTO;
import com.yx.face.model.dto.AppointmentPersonQueryDTO;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.*;
import com.yx.face.model.vo.*;
import com.yx.face.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 常客 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-05-24
 */
@Service
@Slf4j
public class AppointmentPersonServiceImpl implements AppointmentPersonService {

    @Resource
    private AppointmentPersonDao appointmentPersonDao;

    @Resource
    private ReservationRecordDao reservationRecordDao;

    @Resource
    private AdminService adminService;

    @Resource
    private UserInfoService userInfoService;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private FaceTBService faceTBService;

    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private AsyncTriggerRetryImpl asyncTriggerRetryImpl;

    @Resource
    private IssuedTaskInfoDao issuedTaskInfoDao;
    @Resource
    private UserWhitelistService userWhitelistService;

    @Resource
    private BaiDuFaceService baiDuFaceService;


    @Override
    public PageInfo<AppointmentPersonVO> query(RestDTO<AppointmentPersonQueryDTO> dto) {
        AppointmentPersonQueryDTO object = dto.getData();
        if (object == null) {
            object = new AppointmentPersonQueryDTO();
        }
        AdminVO info = adminService.getInfo();
        /*查询*/
        if (StringUtils.isNotEmpty(object.getStartTime()) ||
                StringUtils.isNotEmpty(object.getEndTime()) ||
                StringUtils.isNotEmpty(object.getApplyStartTime()) ||
                StringUtils.isNotEmpty(object.getApplyEndTime())) {
            List<Integer> appointmentPersonIdList = reservationRecordDao.selectByTime(object);
            if (CollUtil.isEmpty(appointmentPersonIdList)) {
                return new PageInfo<>();
            }
            object.setAppointmentPersonIdList(appointmentPersonIdList);
        }
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<AppointmentPersonVO> list = appointmentPersonDao.selectListBySearch(info, object);
        boolean isShowName = false;
        boolean isShowPhone = false;
        boolean isShowNumber = false;
        if (info.getNoHideCloumn() != null && info.getNoHideCloumn().size() > 0) {
            if (info.getNoHideCloumn().contains("1")) {
                isShowName = true;
            }
            if (info.getNoHideCloumn().contains("2")) {
                isShowPhone = true;
            }
            if (info.getNoHideCloumn().contains("3")) {
                isShowNumber = true;
            }
        }

        for (AppointmentPersonVO v : list) {
            if (v.getPhone() != null && isShowPhone) {
                v.setPhone(SMSOrIdCardUtils.hidePhone(v.getPhone()));
            }
            if (v.getIdNumber() != null && isShowNumber) {
                v.setIdNumber(SMSOrIdCardUtils.hideIdNumber(v.getIdNumber()));
            }
            if (v.getName() != null && isShowName) {
                v.setName(SMSOrIdCardUtils.hideName(v.getName().trim()));
            }

        }
        return new PageInfo<>(list);
    }

    @Override
    public Boolean remove(Integer id) {
        AppointmentPerson appointmentPerson = appointmentPersonDao.selectByPrimaryKey(id);
        if (appointmentPerson == null) {
            throw new ServiceException("该数据已不存在，请刷新页面再试哦~");
        }
        List<String> faceDeviceOne = faceDeviceDao.selectList(appointmentPerson.getAdminId());
        if (faceDeviceOne.isEmpty()) {
            throw new ServiceException("该设备不存在或离线，请刷新页面再试哦~");
        }
        Long userId = userInfoDao.getUserInfoPhoneAndIdNumber(appointmentPerson.getIdNumber(), appointmentPerson.getPhone());
        ArrayList<Long> userIds = CollUtil.toList(userId);
        for (String sn : faceDeviceOne) {
            faceTBService.delFaceBySn(sn, userIds);
        }
        appointmentPersonDao.deleteByPrimaryKey(id);
        return true;
    }

    @Override
    public Boolean modify(AppointmentPersonModifyDTO dto) {
        AppointmentPerson appointmentPerson = new AppointmentPerson();
        BeanUtil.copyProperties(dto, appointmentPerson);
        appointmentPersonDao.updateByPrimaryKey(appointmentPerson);
        return true;
    }

    @Override
    public AppointmentDetailVO detail(Integer id) {
        AppointmentDetailVO appointmentDetailVO = new AppointmentDetailVO();
        AppointmentPerson appointmentPerson = appointmentPersonDao.selectByPrimaryKey(id);
        BeanUtil.copyProperties(appointmentPerson, appointmentDetailVO);
        //查询预约记录信息
        List<ReservationRecord> reservationRecords = reservationRecordDao.selectByExample(Example.builder(ReservationRecord.class).where(WeekendSqls.<ReservationRecord>custom().andEqualTo(ReservationRecord::getAppointmentPersonId, appointmentPerson.getId())).orderByDesc("createTime").build());
        if (CollectionUtil.isEmpty(reservationRecords)) {
            return appointmentDetailVO;
        }
        List<ReservationRecordDTO> reservationRecordDTOS = Convert.toList(ReservationRecordDTO.class, reservationRecords);
        for (ReservationRecordDTO reservationRecordDTO : reservationRecordDTOS) {
            if (StringUtils.isNotEmpty(reservationRecordDTO.getContent())) {
                reservationRecordDTO.setContents(JSON.parseArray(reservationRecordDTO.getContent(), ContextVO.class));
            } else {
                reservationRecordDTO.setContents(new ArrayList<>());
            }
        }
        appointmentDetailVO.setReservationRecordList(reservationRecordDTOS);
        AdminVO info = adminService.getInfo();
        boolean isShowName = false;
        boolean isShowPhone = false;
        if (info.getNoHideCloumn() != null && info.getNoHideCloumn().size() > 0) {
            if (info.getNoHideCloumn().contains("1")) {
                isShowName = true;
            }
            if (info.getNoHideCloumn().contains("2")) {
                isShowPhone = true;
            }
        }
        if (StringUtils.isNotEmpty(appointmentDetailVO.getName()) && isShowName) {
            appointmentDetailVO.setName(SMSOrIdCardUtils.hideName(appointmentDetailVO.getName()));
        }
        if (StringUtils.isNotEmpty(appointmentDetailVO.getPhone()) && isShowPhone) {
            appointmentDetailVO.setPhone(SMSOrIdCardUtils.hidePhone(appointmentDetailVO.getPhone()));
        }
        return appointmentDetailVO;
    }

    @Override
    public Boolean issue(Integer id) {
        AdminVO info = adminService.getInfo();
        AppointmentPerson appointmentPerson = appointmentPersonDao.selectByPrimaryKey(id);
        Long userId = userInfoService.checkingIdNumberAndPhone(appointmentPerson.getIdNumber(), appointmentPerson.getPhone());
        List<String> faceDeviceList = faceDeviceDao.selectList(info.getAdminId());
        List<ReservationRecord> reservationRecords = reservationRecordDao.selectByExample(Example.builder(ReservationRecord.class).where(WeekendSqls.<ReservationRecord>custom().andEqualTo(ReservationRecord::getAppointmentPersonId, appointmentPerson.getId())).orderByDesc("endTime").build());
        if (CollectionUtil.isEmpty(reservationRecords)) {
            throw new ServiceException("暂无预约记录");
        }
        ReservationRecord reservationRecord = reservationRecords.stream().findFirst().get();
        if (DateUtil.compare(DateUtil.date(), reservationRecord.getEndTime()) > 0) {
            throw new ServiceException("预约人员有效期已过");
        }
        String facialFeature = null;
        if (Preconditions.isNotBlank(appointmentPerson.getAvatar())) {
            facialFeature = baiDuFaceService.baiDuFaceCharacteristic(Base64Util.imgToBase64(appointmentPerson.getAvatar()));
        }
        for (String sn : faceDeviceList) {
            DownFaceNew downFaceNew = new DownFaceNew();
            downFaceNew.setSn(sn);
            downFaceNew.setUserId(userId);
            downFaceNew.setName(appointmentPerson.getName());
            downFaceNew.setPhoto(appointmentPerson.getAvatar());
            downFaceNew.setPhone(appointmentPerson.getPhone());
            downFaceNew.setCardid(appointmentPerson.getIdNumber());
            downFaceNew.setValidPeriodStart(reservationRecord.getStartTime());
            downFaceNew.setValidPeriodEnd(reservationRecord.getEndTime());
            downFaceNew.setRightno(UserRightNoEnum.APPOINTMENT.getCode());
            downFaceNew.setFacialFeature(facialFeature);
            downFaceNew.setAdminId(info.getAdminId());
            faceTBService.downUser(downFaceNew);
        }
        return true;
    }

    @Override
    public Boolean issueAll(IssueAllDTO dto) {
        userWhitelistService.addSnIssued(dto);
        return true;
    }
}
