package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.core.SnowflakeIdWorker;
import com.yx.face.boot.uitls.*;
import com.yx.face.constant.AutoTaskConstant;
import com.yx.face.dao.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.*;
import com.yx.face.model.enums.issue.IssueTypeEnum;
import com.yx.face.model.excel.UserInfoMsgExcel;
import com.yx.face.model.vo.*;
import com.yx.face.server.HandheldTerminalsServer;
import com.yx.face.server.dto.AddPersonDTO;
import com.yx.face.service.SpiIotUserWhitelistService;
import com.yx.face.service.feign.RemoteRYDeviceDownloadFaceService;
import com.yx.face.service.feign.TBDeviceFaceService;
import com.yx.face.service.issue.IssueDeviceService;
import com.yx.face.service.issue.ZoneUserIssueService;
import com.yx.face.service.middleware.MiddlewareService;
import com.yx.face.service.smartlock.AdminSmartlockDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName AsyncTriggerRetry
 * @Author WXG
 * @Date 2021/11/30 13:16
 */
@Service
@Slf4j
public class
AsyncTriggerRetryImpl {

    @Autowired
    private TBDeviceFaceService tbDeviceFaceService;
    @Resource
    private HandheldTerminalsServer handheldTerminalsServer;
    @Resource
    private HandheldTerminalsImpl handheldTerminalsImpl;
    @Resource
    private FacePassDao facePassDao;
    @Resource
    private FaceTaskDao faceTaskDao;
    @Autowired
    private RedisUtil redisUtil;
    @Resource
    private UserWhitelistDao userWhitelistdao;
    @Resource
    private FaceRequestLogDao faceRequestLogDao;
    @Resource
    private FaceDeviceTypeDao faceDeviceTypeDao;
    @Resource
    private AppointmentPersonDao appointmentPersonDao;
    @Resource
    private ReservationRecordDao reservationRecordDao;
    @Resource
    private AdminDao adminDao;
    @Resource
    private IssuedTaskInfoDao issuedTaskInfoDao;
    @Resource
    private Executor executor;
    @Resource
    private RemoteRYDeviceDownloadFaceService remoteRYDeviceDownloadFaceService;
    @Resource
    private TaskInfoDao taskInfoDao;

    @Autowired
    private SpiIotUserWhitelistService spiIotUserWhitelistService;

    @Resource
    private AdminSmartlockDeviceService adminSmartlockDeviceService;

    @Resource
    private UserWhitelistDao userWhitelistDao;

   

    @Resource
    private MiddlewareService middlewareService;

    @Resource
    private IssueDeviceService issueDeviceService;

    @Resource
    private ZoneUserIssueService zoneUserIssueService;

    @Resource
    private UserInfoDao userInfoDao;

    @Async
    public void send(Map<String, Object> params, FacePass facePass) {
        try {
            Map<String, Object> result = tbDeviceFaceService.downUser(params);
            log.info("下发人脸downFace = {}", result);
            if (result.get("msg") != null) {
                String msg = result.get("msg").toString();
                if (msg.contains("不在线")) {
                    //设备不在线 重试 3次
//                    facePass.setStatus(3);
//                    facePassDao.updateByPrimaryKeySelective(facePass);
                }
            }
        } catch (Exception e) {
            log.info("下发失败 原因 {}", e.getMessage());
            //设备不在线 重试 3次
//            facePass.setStatus(3);
//            facePassDao.updateByPrimaryKeySelective(facePass);
        }
    }

    @Async
    public void faceInsert(FaceDevice sn, Long userId, Date startTime, Date endTime) {
        Date now = new Date();
        Long faceTaskId = faceTaskDao.getFaceTaskId(sn.getSn(), userId);
        FaceTask faceTask = new FaceTask();
        faceTask.setId(faceTaskId);
        faceTask.setTaskTime(endTime);
        faceTask.setStatus(0);
        faceTask.setUpdateTime(now);
        if (faceTaskId == null) {
            faceTask.setDeviceSn(sn.getSn());
            faceTask.setUserId(userId);
            faceTask.setCreateTime(now);
            faceTaskDao.insertSelective(faceTask);
        } else {
            faceTaskDao.updateByPrimaryKeySelective(faceTask);
        }
    }

    public void addSnIssued(FaceDevice faceDevice, AdminVO adminVO, TaskInfo taskInfo, String issueType) {
        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectByPrimaryKey(faceDevice.getFaceDeviceTypeId());
        if (Preconditions.isBlank(faceDeviceType)) {
            log.error("--->设备类型不存在");
            return;
        }
        String code = faceDeviceType.getCode();
        //如果是手持终端先去查下companyId
        String companyId = "";
        if (code.equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())) {
            companyId = handheldTerminalsImpl.getCompanyId(faceDevice.getId(), faceDevice.getSn());
        }
        /*1.根据账号不同组装数据
         *   差异有几点
         *   1.预约场景的的人员信息，是存在预约人员表中的
         *   2.楼宇场景
         *       ①：楼宇公有设备，还需要把企业的人员进行组装加载
         *       ②：企业私有设备全量下发，只下发企业的私有设备所属区域的人员
         *   3.通用场景 组装当前设备所属账号下的常客信息即可
         * */
        redisUtil.del(faceDevice.getSn() + "");
        List<DownFaceNew> list = null;
        Integer placeType = adminVO.getPlaceType();
        if (placeType.equals(AdminPlaceTypeEnum.APPOINTMENT_PLACE.getCode())) {
            list = this.appointmentPersonList(faceDevice);
        } else if (AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(placeType)
                || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(placeType)
        ) {
            list = this.buildingPersonList(faceDevice, adminVO);
        } else {
            list = this.commonPersonList(faceDevice, adminVO);
        }

        log.info("数据走向:{}", placeType);
        log.info("数据内容:{}", JSON.toJSONString(list));
        //TODO。。。 兼容IOT设备
        spiIotUserWhitelistService.batchAddUser2Iot(faceDevice.getSn(), list);

        if (code.equals(FaceDeviceTypeEnum.HHFACE.getCode())) {
            if (Preconditions.isBlank(issueType) || IssueTypeEnum.ISSUE_ALL.getCode().equals(issueType)) {
                issueDeviceService.issueUserAll(faceDevice, list, adminVO);
            } else {
                issueDeviceService.issueUserAllIncrement(faceDevice, list, adminVO);
            }
            return;
        }

        if (Preconditions.isNotBlank(taskInfo)
                && taskInfo.getOnlineFlag().equals(TaskInfoOnlineStatus.OffLine.getCode())
                && taskInfo.getStatus().equals(AutoTaskInfoStatusEnum.NOT_STATRT.getStatus())
                && taskInfo.getRetry().equals(AutoTaskConstant.DEFAULT_RETRY_TIME)) {
            return;
        }
        Long taskId =null;
        if (CollUtil.isNotEmpty(list)) {
            taskId = issueDeviceService.handleIssueDeviceAll(faceDevice, list, adminVO, issueType);
            FaceRequestLog requestLog = intoFaceRequestLog("4.9远程触发分批同步人员信息", "/sendControl");
            for (DownFaceNew downFaceNew : list) {
                /*2.根据不同设备走不同的下发逻辑
                 * 目前支持的下发只有 1.天波  2.锐颖  3.手持
                 * */
                downFaceNew.setSn(faceDevice.getSn());
                if (code.equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())) {
                    this.handheldDownFace(downFaceNew, companyId);
                } else if (code.equals(FaceDeviceTypeEnum.FOUR_INCH_SCREEN.getCode())) {
                    this.ruiYingDownFace(downFaceNew);
                } else if (code.equals(FaceDeviceTypeEnum.TB.getCode())) {
                    this.tbDownFace(downFaceNew);
                } else {
                    log.info("没有下发逻辑的设备不要点全量下发");
                    continue;
                }
                faceInsert(faceDevice, downFaceNew.getUserId(), downFaceNew.getValidPeriodStart(), downFaceNew.getValidPeriodEnd());
            }
            if (code.equals(FaceDeviceTypeEnum.TB.getCode())) {
                //TB批量下发命令发送
                if(Preconditions.isNotBlank(taskInfo)) {
                    taskInfo.setFaceRequestLogId(requestLog.getId());
                    taskInfoDao.updateByPrimaryKeySelective(taskInfo);
                }
                Map<String, Object> map = this.tbBatchDownFace(faceDevice.getSn(), taskId);
                Map<String, Object> sendControl = tbDeviceFaceService.sendControl(map);
                log.info("发送分批同步请求返回参数；{}", sendControl);
            } else {
                //TODO 这里是除了天波其他的应该走到这
                if(Preconditions.isNotBlank(taskInfo)) {
                    taskInfo.setStatus(AutoTaskInfoStatusEnum.COMPELETED.getStatus());
                    taskInfo.setExecuteResult("处理完成");
                    taskInfoDao.updateByPrimaryKeySelective(taskInfo);
                }
            }

            if (code.equals(FaceDeviceTypeEnum.SMART_LOCK.getCode())) {
                //智能门锁
                handleSmartlockFaceDown(faceDevice, list, taskInfo);
            }

            if (code.equals(FaceDeviceTypeEnum.BLUETOOTH_SMART_LOCK.getCode())) {
                //蓝牙智能门锁
                handleBluetoothSmartlockFaceDown(faceDevice, list, taskInfo);
            }

        }
    }

    private void handleBluetoothSmartlockFaceDown(FaceDevice faceDevice, List<DownFaceNew> list, TaskInfo taskInfo) {
        if (Preconditions.isNotBlank(list)) {
            List<UserInfoMsgExcel> userInfoMsgExcelList = new LinkedList<>();
            for (DownFaceNew item : list) {
                try {
                    middlewareService.addUser(item, item.getValidPeriodStart(), item.getValidPeriodEnd(), faceDevice.getAdminId());
                } catch (Exception e) {
                    log.error("--->蓝牙智能门锁人脸下发失败，记录错误列表 message:{}", e.getMessage());
                    UserInfoMsgExcel userInfoMsgExcel = new UserInfoMsgExcel();
                    UserWhitelist userWhitelist = userWhitelistDao.selectByUserId(item.getUserId(), taskInfo.getCreateId());
                    userInfoMsgExcel.setName(userWhitelist.getName());
                    userInfoMsgExcel.setPhone(userWhitelist.getPhone());
                    userInfoMsgExcel.setIdNumber(userWhitelist.getIdNumber());
                    userInfoMsgExcel.setAvatar(userWhitelist.getAvatar());
                    userInfoMsgExcel.setStatus((userWhitelist.getStatus() != null && userWhitelist.getStatus() == 1) ? "已认证" : "未认证");
                    userInfoMsgExcel.setMsg(e.getMessage());
                    userInfoMsgExcelList.add(userInfoMsgExcel);
                }
            }
            //记录错误列表
            taskInfo.setExecuteResult(this.msgAddress(userInfoMsgExcelList));
            taskInfoDao.updateByPrimaryKey(taskInfo);
        }
    }

    private void handleSmartlockFaceDown(FaceDevice faceDevice, List<DownFaceNew> list, TaskInfo taskInfo) {
        if (Preconditions.isNotBlank(list)) {
            List<UserInfoMsgExcel> userInfoMsgExcelList = new LinkedList<>();
            for (DownFaceNew item : list) {
                try {
                    adminSmartlockDeviceService.addSmartLockUser(item, item.getValidPeriodStart(), item.getValidPeriodEnd(), faceDevice.getAdminId());
                } catch (Exception e) {
                    log.error("--->智能门锁人脸下发失败，记录错误列表 message:{}", e.getMessage());
                    UserInfoMsgExcel userInfoMsgExcel = new UserInfoMsgExcel();
                    UserWhitelist userWhitelist = userWhitelistDao.selectByUserId(item.getUserId(), taskInfo.getCreateId());
                    userInfoMsgExcel.setName(userWhitelist.getName());
                    userInfoMsgExcel.setPhone(userWhitelist.getPhone());
                    userInfoMsgExcel.setIdNumber(userWhitelist.getIdNumber());
                    userInfoMsgExcel.setAvatar(userWhitelist.getAvatar());
                    userInfoMsgExcel.setStatus((userWhitelist.getStatus() != null && userWhitelist.getStatus() == 1) ? "已认证" : "未认证");
                    userInfoMsgExcel.setMsg(e.getMessage());
                    userInfoMsgExcelList.add(userInfoMsgExcel);
                }
            }
            //记录错误列表
            taskInfo.setExecuteResult(this.msgAddress(userInfoMsgExcelList));
            taskInfoDao.updateByPrimaryKey(taskInfo);
        }
    }

    private String msgAddress(List<UserInfoMsgExcel> list) {
        if (CollUtil.isEmpty(list)) {
            return "";
        }
        String uploadExcel = Constants.UPLOAD_EXCEL;
        String fileName = uploadExcel + "/智能门锁全量下发错误记录-" + System.currentTimeMillis() + ".xlsx";
        // 这里 需要指定写用哪个class去写
        ExcelWriter excelWriter = null;
        try {
            Path directory = Paths.get(uploadExcel);
            if (!Files.exists(directory)) {
                Files.createDirectories(directory);
            }
            excelWriter = EasyExcel.write(fileName, UserInfoMsgExcel.class).build();
            WriteSheet writeSheet = EasyExcel.writerSheet("错误记录").build();
            excelWriter.write(list, writeSheet);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 千万别忘记finish 会帮忙关闭流
            if (excelWriter != null) {
                excelWriter.finish();
            }
        }
        return AddressUtil.address.concat(fileName);
    }

    private Map<String, Object> tbBatchDownFace(String sn, Long id) {
        Map<String, Object> map = Maps.newHashMap();
        Map<String, Object> data = Maps.newHashMap();
        data.put("syncUsersType", "0");
        map.put("SN", sn);
        map.put("taskId", id);
        map.put("type", "syncUsers");
        map.put("Data", data);
        return map;
    }

    /*天波批量下发人员数据*/
    private void tbDownFace(DownFaceNew downFaceNew) {
        Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(9);
        dataItem.put("pin", downFaceNew.getUserId());
        dataItem.put("name", downFaceNew.getName());
        dataItem.put("validPeriodStart", downFaceNew.getValidPeriodStart());
        dataItem.put("validPeriodEnd", downFaceNew.getValidPeriodEnd());
        dataItem.put("rightno", RoleTypeEnum.FREQUENTER.getCode());
        dataItem.put("userType", downFaceNew.getUserType());
        dataItem.put("opt", 1);
        dataItem.put("photo", downFaceNew.getPhoto());
        dataItem.put("cardid", downFaceNew.getCardid());
        dataItem.put("cardIdEx", downFaceNew.getCardIdEx());
        dataItem.put("phone", downFaceNew.getPhone());
        dataItem.put("facialFeature", downFaceNew.getFacialFeature());
        redisUtil.lSet(downFaceNew.getSn() + "", dataItem);
    }

    /*锐颖批量下发人员数据*/
    private void ruiYingDownFace(DownFaceNew downFaceNew) {
        Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(9);
        dataItem.put("pin", downFaceNew.getUserId());
        dataItem.put("name", downFaceNew.getName());
        dataItem.put("validPeriodStart", downFaceNew.getValidPeriodStart());
        dataItem.put("validPeriodEnd", downFaceNew.getValidPeriodEnd());
        dataItem.put("rightno", RoleTypeEnum.FREQUENTER.getCode());
        dataItem.put("userType", downFaceNew.getUserType());
        dataItem.put("opt", 1);
        dataItem.put("photo", downFaceNew.getPhoto());
        dataItem.put("cardid", downFaceNew.getCardid());
        dataItem.put("cardIdEx", downFaceNew.getCardIdEx());
        dataItem.put("phone", downFaceNew.getPhone());
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(1);
        data.add(dataItem);
        params.put("SN", downFaceNew.getSn());
        params.put("Data", data);
        params.put("taskId", SnowflakeIdWorker.getSnowId());
        remoteRYDeviceDownloadFaceService.addUser(params);
    }

    /*手持下发人员数据*/
    private void handheldDownFace(DownFaceNew downFaceNew, String companyId) {
        String photo = Base64Util.imgToBase64(downFaceNew.getPhoto());
        AddPersonDTO addPersonDTO = new AddPersonDTO();
        addPersonDTO.setCompanyId(companyId);
        addPersonDTO.setCardId(downFaceNew.getCardid());
        addPersonDTO.setXm(downFaceNew.getName());
        addPersonDTO.setPhoto(photo);
        addPersonDTO.setValidStartTime(downFaceNew.getValidPeriodStart().getTime());
        addPersonDTO.setValidEndTime(downFaceNew.getValidPeriodEnd().getTime());
        handheldTerminalsServer.addPerson(addPersonDTO);
    }

    /*通用场景获取下发人员数据*/
    private List<DownFaceNew> commonPersonList(FaceDevice faceDevice, AdminVO adminVO) {
        List<Integer> adminIdList = CollUtil.toList(adminVO.getAdminId());
        //TODO。。。查询设备所在区域->区域内包含人员
        List<UserWhitelist> whitelist = null;
        if (!zoneUserIssueService.checkIsZoneSn(faceDevice.getSn(), faceDevice.getAdminId())) {
            //开放区域设备
            List<UserWhitelistVO> userWhitelistVOList = userWhitelistdao.selectUserWhiteList(adminIdList);
            whitelist = Preconditions.isNotBlank(userWhitelistVOList) ? BeanUtil.copyToList(userWhitelistVOList, UserWhitelist.class) : null;
        } else {
            whitelist = zoneUserIssueService.getUserWhitelistBySn(faceDevice.getSn(), adminIdList);
        }

        if (Preconditions.isBlank(whitelist)) {
            return new ArrayList<>();
        }

        List<Long> userIds = whitelist.stream().map(UserWhitelist::getUserId).collect(Collectors.toList());
        List<UserInfo> userInfos = userInfoDao.selectByExample(Example.builder(UserInfo.class).where(WeekendSqls.<UserInfo>custom().andIn(UserInfo::getId, userIds)).build());
        Map<Long, UserInfo> userInfoMap = userInfos.stream().collect(Collectors.toMap(UserInfo::getId, v -> v, (v1, v2) -> v1));
        List<DownFaceNew> list = new ArrayList<>(whitelist.size());
        for (UserWhitelist userWhitelist : whitelist) {
            Date startTime = null;
            Date endTime = null;
            if (Preconditions.isNotBlank(faceDevice.getValidityPeriodStatus()) && faceDevice.getValidityPeriodStatus()) {
                startTime = faceDevice.getStartTime();
                endTime = faceDevice.getEndTime();
            } else {
                startTime = userWhitelist.getStartTime() == null ? new Date() : userWhitelist.getStartTime();
                if (userWhitelist.getEndTime() == null) {
                    if (adminVO.getUserWhitelistEffectiveTime() != null) {
                        int timelong = adminVO.getUserWhitelistEffectiveTime().multiply(new BigDecimal(1440)).intValue();
                        endTime = DateUtils.addMinutes(new Date(), timelong);
                    } else {
                        endTime = DateUtils.addYears(DateUtil.date(), 1);
                    }
                } else {
                    endTime = userWhitelist.getEndTime();
                }
            }
            if (DateUtil.compare(DateUtil.date(), endTime) > 0) {
                continue;
            }
            UserInfo userInfo = userInfoMap.get(userWhitelist.getUserId());
            DownFaceNew downFaceNew = new DownFaceNew();
            if (Preconditions.isNotBlank(userInfo)) {
                downFaceNew.setFacialFeature(userInfo.getFacialFeature());
            }
            downFaceNew.setCardid(userWhitelist.getIdNumber());
            downFaceNew.setRightno(UserRightNoEnum.REGULAR.getCode());
            downFaceNew.setPhoto(userWhitelist.getAvatar());
            downFaceNew.setName(userWhitelist.getName());
            downFaceNew.setUserId(userWhitelist.getUserId());
            downFaceNew.setPhone(userWhitelist.getPhone());
            downFaceNew.setValidPeriodStart(startTime);
            downFaceNew.setValidPeriodEnd(endTime);
            downFaceNew.setCardIdEx(userWhitelist.getCardIdEx());
            list.add(downFaceNew);
        }
        if (Preconditions.isNotBlank(faceDevice.getValidityPeriodStatus()) && faceDevice.getValidityPeriodStatus()) {
            for (Integer adminId : adminIdList) {
                userWhitelistdao.updateEffective(faceDevice.getStartTime(), faceDevice.getEndTime(), adminId);
            }
        }
        return list;
    }

    /*楼宇场景获取下发人员数据*/
    private List<DownFaceNew> buildingPersonList(FaceDevice faceDevice, AdminVO adminVO) {
        List<Integer> adminIdList = CollUtil.toList(adminVO.getAdminId());
        /*区级账号要把下面的企业账号id都加入进去*/
        if (AdminTypeEnum.DISTRICT.getCode().equals(adminVO.getType())) {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, adminVO.getAdminId())).build());
            adminIdList.addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
        }
        List<UserWhitelistVO> userWhitelistVOList = null;
        if (adminVO.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())
                && adminVO.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.OPEN_PRIVATE_AREA.getCode())
                && faceDevice.getAreaType().equals(2)) {
            userWhitelistVOList = userWhitelistdao.selectUserWhiteListByDevice(adminIdList, faceDevice.getId());
        } else {
            userWhitelistVOList = userWhitelistdao.selectUserWhiteList(adminIdList);
        }
        if (CollUtil.isEmpty(userWhitelistVOList)) {
            return new ArrayList<>();
        }
        //TODO。。。查询设备所在区域->区域内包含人员

        List<Long> userIds = userWhitelistVOList.stream().map(UserWhitelistVO::getUserId).collect(Collectors.toList());
        List<UserInfo> userInfos = userInfoDao.selectByExample(Example.builder(UserInfo.class).where(WeekendSqls.<UserInfo>custom().andIn(UserInfo::getId, userIds)).build());
        Map<Long, UserInfo> userInfoMap = userInfos.stream().collect(Collectors.toMap(UserInfo::getId, v -> v, (v1, v2) -> v1));

        List<DownFaceNew> list = new ArrayList<>(userWhitelistVOList.size());
        for (UserWhitelistVO userWhitelistVO : userWhitelistVOList) {
            Date startTime = null;
            Date endTime = null;
            if (faceDevice.getValidityPeriodStatus()) {
                startTime = faceDevice.getStartTime();
                endTime = faceDevice.getEndTime();
            } else {
                startTime = userWhitelistVO.getStartTime() == null ? new Date() : userWhitelistVO.getStartTime();
                if (userWhitelistVO.getEndTime() == null) {
                    if (adminVO.getUserWhitelistEffectiveTime() != null) {
                        int timelong = adminVO.getUserWhitelistEffectiveTime().multiply(new BigDecimal(1440)).intValue();
                        endTime = DateUtils.addMinutes(new Date(), timelong);
                    } else {
                        endTime = DateUtils.addYears(DateUtil.date(), 1);
                    }
                } else {
                    endTime = userWhitelistVO.getEndTime();
                }
            }
            if (DateUtil.compare(DateUtil.date(), endTime) > 0) {
                continue;
            }
            DownFaceNew downFaceNew = new DownFaceNew();
            UserInfo userInfo = userInfoMap.get(userWhitelistVO.getUserId());
            if (Preconditions.isNotBlank(userInfo)) {
                downFaceNew.setFacialFeature(userInfo.getFacialFeature());
            }
            downFaceNew.setCardid(userWhitelistVO.getIdNumber());
            downFaceNew.setRightno(UserRightNoEnum.REGULAR.getCode());
            downFaceNew.setPhoto(userWhitelistVO.getAvatar());
            downFaceNew.setName(userWhitelistVO.getName());
            downFaceNew.setUserId(userWhitelistVO.getUserId());
            downFaceNew.setPhone(userWhitelistVO.getPhone());
            downFaceNew.setValidPeriodStart(startTime);
            downFaceNew.setValidPeriodEnd(endTime);
            list.add(downFaceNew);
        }
        if (faceDevice.getValidityPeriodStatus()) {
            for (Integer adminId : adminIdList) {
                userWhitelistdao.updateEffective(faceDevice.getStartTime(), faceDevice.getEndTime(), adminId);
            }
        }
        return list;
    }

    /*预约场景获取下发人员数据*/
    private List<DownFaceNew> appointmentPersonList(FaceDevice faceDevice) {
        List<AppointmentPersonVO> appointmentPeopleList = appointmentPersonDao.selectList(faceDevice.getAdminId());
        List<ReservationRecord> reservationRecords = reservationRecordDao.selectByExample(Example.builder(ReservationRecord.class).where(WeekendSqls.<ReservationRecord>custom().andEqualTo(ReservationRecord::getAdminId, faceDevice.getAdminId()).andGreaterThan(ReservationRecord::getEndTime, DateUtil.date())).orderByDesc("endTime").build());
        Map<Integer, List<ReservationRecord>> collect = reservationRecords.stream().collect(Collectors.groupingBy(ReservationRecord::getAppointmentPersonId));
        if (CollUtil.isEmpty(appointmentPeopleList)) {
            return new ArrayList<>();
        }
        List<Long> userIds = appointmentPeopleList.stream().map(AppointmentPersonVO::getUserId).collect(Collectors.toList());
        List<UserInfo> userInfos = userInfoDao.selectByExample(Example.builder(UserInfo.class).where(WeekendSqls.<UserInfo>custom().andIn(UserInfo::getId, userIds)).build());
        Map<Long, UserInfo> userInfoMap = userInfos.stream().collect(Collectors.toMap(UserInfo::getId, v -> v, (v1, v2) -> v1));

        List<DownFaceNew> list = new ArrayList<>(appointmentPeopleList.size());
        for (AppointmentPersonVO person : appointmentPeopleList) {
            List<ReservationRecord> records = collect.get(person.getId());
            if (person.getUserId() == null || CollectionUtil.isEmpty(records)) {
                continue;
            }
            Long userId = person.getUserId();
            UserInfo userInfo = userInfoMap.get(userId);
            ReservationRecord reservationRecord = records.stream().findFirst().get();
            Date startTime = null;
            Date endTime = null;
            if (faceDevice.getValidityPeriodStatus()) {
                startTime = faceDevice.getStartTime();
                endTime = faceDevice.getEndTime();
            } else {
                startTime = reservationRecord.getStartTime();
                endTime = reservationRecord.getEndTime();
            }
            if (DateUtil.compare(DateUtil.date(), endTime) > 0) {
                continue;
            }
            DownFaceNew downFaceNew = new DownFaceNew();
            if (Preconditions.isNotBlank(userInfo)) {
                downFaceNew.setFacialFeature(userInfo.getFacialFeature());
            }
            downFaceNew.setCardid(person.getIdNumber());
            downFaceNew.setRightno(UserRightNoEnum.APPOINTMENT.getCode());
            downFaceNew.setPhoto(person.getAvatar());
            downFaceNew.setName(person.getName());
            downFaceNew.setUserId(userId);
            downFaceNew.setPhone(person.getPhone());
            downFaceNew.setValidPeriodStart(startTime);
            downFaceNew.setValidPeriodEnd(endTime);
            list.add(downFaceNew);
        }
        return list;
    }

    private FaceRequestLog intoFaceRequestLog(String name, String context) {
        FaceRequestLog requestLog = new FaceRequestLog();
        requestLog.setName(name);
        requestLog.setContext(context);
        requestLog.setParams("");
        requestLog.setStatus(1);
        requestLog.setUsedTime(0L);
        Date now = new Date();
        requestLog.setUpdateTime(now);
        requestLog.setCreateTime(now);
        faceRequestLogDao.insertSelective(requestLog);
        return requestLog;
    }


}