package com.yx.face.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.HttpUtil;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.model.api.appointment.dto.BaiduFaceDTO;
import com.yx.face.model.dto.baidu.BaiduFeature;
import com.yx.face.model.dto.baidu.BaiduFeatureResultFaceList;
import com.yx.face.service.BaiDuFaceService;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName BaiDuFaceServiceImpl
 * @Author WXG
 * @Date 2022/5/25 20:11
 */
@Service
@Slf4j
public class BaiDuFaceServiceImpl implements BaiDuFaceService {


    @Autowired
    private RedisUtil redisUtil;

    /**
     * 获取token类
     */

    /**
     * 获取权限token
     *
     * @return 返回示例：
     * {
     * "access_token": "24.460da4889caad24cccdb1fea17221975.2592000.1491995545.282335-1234567",
     * "expires_in": 2592000
     * }
     */
    public static String getAuth() {
        // 官网获取的 API Key 更新为你注册的
        String clientId = "wq2C0tbQFDI2curLTOINtbRE";
        // 官网获取的 Secret Key 更新为你注册的
        String clientSecret = "7OV3kgKN2wyTwE5QQSO4FdkGtGAc5eGt";

        String token = getAuth1(clientId, clientSecret);

        return token;
    }

    /**
     * 获取API访问token
     * 该token有一定的有效期，需要自行管理，当失效时需重新获取.
     *
     * @param ak - 百度云官网获取的 API Key
     * @param sk - 百度云官网获取的 Secret Key
     * @return assess_token 示例：
     * "24.460da4889caad24cccdb1fea17221975.2592000.1491995545.282335-1234567"
     */
    public static String getAuth1(String ak, String sk) {
        // 获取token地址
        String authHost = "https://aip.baidubce.com/oauth/2.0/token?";
        String getAccessTokenUrl = authHost
                // 1. grant_type为固定参数
                + "grant_type=client_credentials"
                // 2. 官网获取的 API Key
                + "&client_id=" + ak
                // 3. 官网获取的 Secret Key
                + "&client_secret=" + sk;
        try {
            URL realUrl = new URL(getAccessTokenUrl);
            // 打开和URL之间的连接
            HttpURLConnection connection = (HttpURLConnection) realUrl.openConnection();
            connection.setRequestMethod("GET");
            connection.connect();
            // 获取所有响应头字段
            Map<String, List<String>> map = connection.getHeaderFields();
            // 遍历所有的响应头字段
            for (String key : map.keySet()) {
                System.err.println(key + "--->" + map.get(key));
            }
            // 定义 BufferedReader输入流来读取URL的响应
            BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            String result = "";
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
            /**
             * 返回结果示例
             */
            System.err.println("result:" + result);
            org.json.JSONObject jsonObject = new org.json.JSONObject(result);
            String access_token = jsonObject.getString("access_token");
            return access_token;
        } catch (Exception e) {
            System.err.printf("获取token失败！");
            e.printStackTrace(System.err);
        }
        return null;
    }


    @Override
    public RestResult<Boolean> BaiDuFaceComparison(BaiduFaceDTO baiduFaceDTO) {
        if (!redisUtil.hasKey("baidutoken")) {
            String token = BaiDuFaceServiceImpl.getAuth();
            redisUtil.set("baidutoken", token, 2592000);
        }
        String accessToken = (String) redisUtil.get("baidutoken");

        String url = "https://aip.baidubce.com/rest/2.0/face/v3/match";
        try {


            JSONObject map1 = new JSONObject();
            map1.put("image", baiduFaceDTO.getFaceImage());
            map1.put("image_type", "URL");
            map1.put("face_type", "LIVE");
            map1.put("quality_control", "LOW");
            map1.put("liveness_control", "HIGH");

            JSONObject map2 = new JSONObject();
            map2.put("image", baiduFaceDTO.getCardImage());//线上使用是这个
//            map2.put("image","https://www.xiaofeifeiwangyiping.com/4.jpg");
            map2.put("image_type", "URL");
            map2.put("face_type", "CERT");
            map2.put("quality_control", "NONE");
            map2.put("liveness_control", "NONE");

            JSONArray jsonArray = new JSONArray();
            jsonArray.add(map1);
            jsonArray.add(map2);
            String param = jsonArray.toJSONString();

            // 注意这里仅为了简化编码每一次请求都去获取access_token，线上环境access_token有过期时间， 客户端可自行缓存，过期后重新获取。
            String result = HttpUtil.post(url, accessToken, "application/json", param);
            System.out.println(result);
            JSONObject jsonObject = JSONObject.parseObject(result);
            Integer errorCode = jsonObject.getInteger("error_code");
            if (!errorCode.equals(0)) return RestResponse.error("人证比对未通过");
            JSONObject result1 = jsonObject.getJSONObject("result");
            if (result1 == null) return RestResponse.error("人证比对未通过");
            Double score = result1.getDouble("score");
            System.out.println(score);
            if (score > 65) return RestResponse.ok(true);
            else return RestResponse.error("人证比对未通过");
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    @Override
    public String baiDuFaceCharacteristic(String base64) {
        try {
            if (Preconditions.isBlank(base64)) {
                return null;
            }

            if (!redisUtil.hasKey("baiduFacetoken")) {
                String token = BaiDuFaceServiceImpl.getFaceAuth();
                redisUtil.set("baiduFacetoken", token, 2592000);
            }
            String accessToken = (String) redisUtil.get("baiduFacetoken");
            String url = "https://aip.baidubce.com/rest/2.0/face/v1/feature";
            JSONObject map = new JSONObject();
            map.put("image", base64);
            map.put("image_type", "BASE64");
            map.put("version", "Android_8001");
            String param = map.toJSONString();
            String result = HttpUtil.post(url, accessToken, "application/json", param);
            log.info("百度人脸特征值返回{}", result);
            BaiduFeature baiduFeature = JSONObject.parseObject(result, BaiduFeature.class);
            if (Preconditions.isNotBlank(baiduFeature)
                    && baiduFeature.getErrorCode().intValue() == 0
                    && Preconditions.isNotBlank(baiduFeature.getResult())
                    && Preconditions.isNotBlank(baiduFeature.getResult().getFaceList())) {
                List<BaiduFeatureResultFaceList> faceLists = baiduFeature.getResult().getFaceList().stream().sorted(Comparator.comparing(BaiduFeatureResultFaceList::getFaceProbability, Comparator.reverseOrder())).collect(Collectors.toList());
                return faceLists.get(0).getFeature();
            }
            if (baiduFeature.getErrorCode().intValue() == 18) {
                Thread.sleep(1500);
                log.info("--->提取百度人脸特征值达到qps上限休眠2s");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static String getFaceAuth() {
        // 官网获取的 API Key 更新为你注册的
        String apiKey = "mmzHCM25e3cTZpZ4tvU5eKvO";
        // 官网获取的 Secret Key 更新为你注册的
        String secretKey = "cfz8lfSLfTibYLOhPHQJvQCbWJqVNRea";

        String token = getAuth1(apiKey, secretKey);

        return token;
    }
}
