package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.TypeReference;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.base.Joiner;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.core.SnowflakeIdWorker;
import com.yx.face.boot.restful.*;
import com.yx.face.boot.uitls.*;
import com.yx.face.dao.*;
import com.yx.face.model.api.building.dto.*;
import com.yx.face.model.api.building.vo.*;
import com.yx.face.model.api.building.vo.AdminLessVO;
import com.yx.face.model.api.fkj.dto.VisiteePageDto;
import com.yx.face.model.dto.UserWhitelistDTO;
import com.yx.face.model.dto.VisitorQrcodeDto;
import com.yx.face.model.dto.VisitorQrcodeIdDto;
import com.yx.face.model.dto.VisitorUserInfoDto;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.*;
import com.yx.face.model.vo.*;
import com.yx.face.model.vo.fkj.FkjCommonCodeVo;
import com.yx.face.service.*;
import com.yx.face.service.fkj.FkjService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.Weekend;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.yx.face.model.enums.AdminPlaceTypeEnum.BUILDING_PLACE;
import static com.yx.face.model.enums.FkjBaseActionEnum.FKJ_COMMON_CODE;

/**
 * @description:
 * @ClassName BuildingApiServiceImpl
 * @Author WXG
 * @Date 2022/5/31 16:29
 */
@Service
@Slf4j
public class BuildingApiServiceImpl implements BuildingApiService {

    @Resource
    private AdminDao adminDao;

    @Resource
    private UserInfoDao userInfoDao;

    @Resource
    private UserWhitelistDao userWhitelistDao;

    @Resource
    private AdminDetailDao adminDetailDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private FaceTBService faceTBService;

    @Resource
    private UserVisitorListDetailDao userVisitorListDetailDao;
    @Resource
    private FkjService fkjService;

    @Resource
    private UserVisitorListDao userVisitorListDao;
    @Resource
    private UserVisitorListService userVisitorListService;
    @Resource
    private UserWhiteListDetailDao userWhiteListDetailDao;

    @Autowired
    private SmsService smsService;

    @Autowired
    private AdminService adminService;

    @Autowired
    private UserWhitelistService userWhitelistService;

    @Resource
    private UserWhitelistCheckDao userWhitelistCheckDao;


    @Value("${visitor.register-url}")
    private String visitorRegisterUrl;

    @Autowired
    private AlipayOpenApiService alipayOpenApiService;
    @Resource
    private VisiteeDao visiteeDao;

    @Override
    public List<BuildingLessVO> getBuildingList() {
        List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getType, AdminTypeEnum.DISTRICT.getCode()).andEqualTo(Admin::getPlaceType, BUILDING_PLACE.getCode())).build());
        return adminList.stream().map(p -> {
            BuildingLessVO buildingLessVO = new BuildingLessVO();
            buildingLessVO.setAdminId(p.getId());
            buildingLessVO.setName(p.getUsername());
            return buildingLessVO;
        }).collect(Collectors.toList());
    }

    @Override
    public RestResult<List<UserWhiteCheckVO>> userWhiteCheck(UserWhiteCheckDTO userWhiteCheckDTO) {
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userWhiteCheckDTO.getUserId());
        if (userInfo == null) throw new ServiceException("数据有误");
        List<AdminDetail> adminDetails = null;
        Map<Integer, UserWhitelist> userWhitelistMap = null;
        List<UserWhitelist> userWhiteList = new LinkedList<>();
        /*三码信息校验*/
        /*2022-08-12 三码数据校验*/
        this.adminThreeCodeCheck(userWhiteCheckDTO.getAdminId(), userWhiteCheckDTO);


        /*第一种情况：是常客码扫码进入*/
        if (userWhiteCheckDTO.getType().toString().equals(AdminCodeEnum.REGULAR_CODE.getCode())) {
            Admin admin = adminDao.selectByPrimaryKey(userWhiteCheckDTO.getCompanyAdminId());
            if (admin.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
                Admin admin1 = adminDao.selectByPrimaryKey(admin.getTenantId());
                if (!admin1.getCodeType().contains("1")) {
                    return RestResponse.error("该企业没有开启常客码功能！");
                }
            }


            UserWhitelistDTO userWhitelistDTO = new UserWhitelistDTO();
            if (admin.getUserWhitelistEffectiveTime() != null) {
                int timelong = admin.getUserWhitelistEffectiveTime().multiply(new BigDecimal(1440)).intValue();
                Date date = DateUtils.addMinutes(new Date(), timelong);
                userWhitelistDTO.setStartTime(DateUtils.getTime());
                userWhitelistDTO.setEndTime(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, date));
            } else {
                Date date = DateUtils.addDays(new Date(), 365);
                userWhitelistDTO.setStartTime(DateUtils.getTime());
                userWhitelistDTO.setEndTime(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, date));
            }

            userWhitelistDTO.setName(userInfo.getUsername());
            userWhitelistDTO.setPhone(userInfo.getPhone());
            userWhitelistDTO.setIdNumber(userInfo.getIdNumber());
            userWhitelistDTO.setCardIdEx(userInfo.getCardIdEx());
            userWhitelistDTO.setAdminId(userWhiteCheckDTO.getCompanyAdminId());
            userWhitelistDTO.setInputType(UserWhiteInputTypeEnum.USER_WHITE.getType());
            userWhitelistDTO.setAvatar(userInfo.getAvatar());
            userWhitelistDTO.setPushType(0);
            userWhitelistDTO.setIsCheckIdNumber(Boolean.FALSE);
            userWhitelistDTO.setAdminAreaType(AdminAreaTypeEnum.CompanyPublic.getCode());
            if (!CollectionUtils.isEmpty(userWhiteCheckDTO.getAddCustomList())) {
                String s = JSON.toJSONString(userWhiteCheckDTO.getAddCustomList());
                userWhitelistDTO.setContent(s);
            }
            try {
                userWhitelistService.insert(userWhitelistDTO, 0);
            } catch (Exception e) {
                log.error("错误:", e);
                e.printStackTrace();
            }
            adminDetails = adminDetailDao.select(new AdminDetail().setAdminId(userWhiteCheckDTO.getCompanyAdminId()));
            List<Integer> adminIdList = new ArrayList<>();
            adminIdList.add(userWhiteCheckDTO.getCompanyAdminId());
            log.info("常客认证校验:账号信息:{},用户身份证:{},用户手机号:{},物理卡号{}", JSON.toJSONString(adminIdList), userInfo.getIdNumber(), userInfo.getPhone(), userInfo.getCardIdEx());
            for (Integer adminId : adminIdList) {
                UserWhitelist userWhitelist = userWhitelistService.getUserWhitelist(userInfo.getIdNumber(), userInfo.getPhone(), userInfo.getCardIdEx(), adminId);
                if (ObjectUtil.isNotNull(userWhitelist)) {
                    userWhiteList.add(userWhitelist);
                }
            }
            if (CollUtil.isEmpty(userWhiteList)) {
                return RestResponse.error("添加失败");
            }
            userWhitelistMap = userWhiteList.stream().collect(
                    Collectors.toMap(
                            UserWhitelist::getAdminId,
                            Function.identity(),
                            (entity1, entity2) -> entity1
                    )
            );
        }
        /*第二种情况：不是扫常客码进入*/
        else {
            /*1.根据公司Id列表和用户信息查询 用户是否在常客列表中*/
            List<Integer> adminIdList = new ArrayList<>();
            adminIdList.add(userWhiteCheckDTO.getCompanyAdminId());
            for (Integer adminId : adminIdList) {
                UserWhitelist userWhitelist = userWhitelistService.getUserWhitelist(userInfo.getIdNumber(), userInfo.getPhone(), userInfo.getCardIdEx(), adminId);
                if (ObjectUtil.isNotNull(userWhitelist)) {
                    userWhiteList.add(userWhitelist);
                }
            }
            ClassUtil.throwBusinessException(CollUtil.isEmpty(userWhiteList), "您的信息还没有录入相应的公司中");
            /*2.查询公司详细信息*/
            userWhitelistMap = userWhiteList.stream().collect(Collectors.toMap(UserWhitelist::getAdminId, Function.identity()));
            Weekend<AdminDetail> detailWeekend = new Weekend<>(AdminDetail.class);
            detailWeekend.weekendCriteria().andIn(AdminDetail::getAdminId, userWhiteList.stream().map(UserWhitelist::getAdminId).collect(Collectors.toList()));
            adminDetails = adminDetailDao.selectByExample(detailWeekend);
            ClassUtil.throwBusinessException(CollUtil.isEmpty(adminDetails), "公司详情为空");
        }
        if (CollUtil.isNotEmpty(userWhiteCheckDTO.getAddCustomList())) {
            String s = JSON.toJSONString(userWhiteCheckDTO.getAddCustomList());
            for (UserWhitelist userWhitelist : userWhiteList) {
                userWhitelist.setContent(s);
            }
        }
        /*3.常客信息认证,并进行下发*/
        userWhiteListAuthentication(userWhiteList, userInfo, userWhiteCheckDTO.getAdminId(), userWhiteCheckDTO.getCompanyAdminId());
        /*4.三码信息存入附表*/
        for (int i = 0; i < userWhiteList.size(); i++) {
            UserWhitelistDetail userWhitelistDetail = BeanUtil.copyProperties(userWhiteCheckDTO, UserWhitelistDetail.class);
            userWhitelistDetail.setUserWhitelistId(userWhiteList.get(i).getId());
            userWhitelistDetail.setTime(DateUtils.getTime());
            userWhiteListDetailDao.insertSelective(userWhitelistDetail);
        }
        /*5.数据返回*/
        log.info("数据：{},admin:{}", JSON.toJSONString(userWhitelistMap), JSON.toJSONString(adminDetails));
        Map<Integer, UserWhitelist> finalUserWhitelistMap = userWhitelistMap;
        List<UserWhiteCheckVO> userWhiteCheckVOList = adminDetails.stream().map(p ->
                new UserWhiteCheckVO()
                        .setAdminId(p.getAdminId())
                        .setCompanyName(adminDao.selectByPrimaryKey(p.getAdminId()).getUsername())
                        .setUserWhitelistId(finalUserWhitelistMap.get(p.getAdminId()).getId())
        ).collect(Collectors.toList());
        return RestResponse.ok(userWhiteCheckVOList);
    }

    private void adminThreeCodeCheck(Integer companyAdminId, UserWhiteCheckDTO userWhiteCheckDTO) {
        Admin admin = adminDao.selectByPrimaryKey(companyAdminId);
        ClassUtil.checkData(admin, "账号不存在");
        String threeCode = admin.getThreeCode();
        if (StringUtils.isNotEmpty(threeCode)) {
            if (threeCode.contains(AdminThreeCodeEnum.JKM.getCode().toString())) {
                if (StringUtils.isEmpty(userWhiteCheckDTO.getJkm()) && StringUtils.isEmpty(userWhiteCheckDTO.getJkmPhoto())) {
                    throw new ServiceException("健康码数据不能为空");
                }
            }
            if (threeCode.contains(AdminThreeCodeEnum.HS.getCode().toString())) {
                if (StringUtils.isEmpty(userWhiteCheckDTO.getHs()) && StringUtils.isEmpty(userWhiteCheckDTO.getHsPhoto())) {
                    throw new ServiceException("核酸数据不能为空");
                }
            }
            if (threeCode.contains(AdminThreeCodeEnum.XCM.getCode().toString())) {
                if (StringUtils.isEmpty(userWhiteCheckDTO.getXc()) && StringUtils.isEmpty(userWhiteCheckDTO.getXcPhoto())) {
                    throw new ServiceException("行程码数据不能为空");
                }
            }
        }
    }

    private void userWhiteListAuthentication(List<UserWhitelist> userWhiteList, UserInfo userInfo, Integer adminId, Integer companyId) {
        /*1.常客信息认证回填*/
        for (UserWhitelist userWhitelist : userWhiteList) {
            userWhitelist.setStatus(1);
            userWhitelist.setUpdateTime(new Date());
            userWhitelist.setPhone(StrUtil.isNotEmpty(userInfo.getPhone()) ? userInfo.getPhone() : null);
            userWhitelist.setAvatar(StrUtil.isNotEmpty(userInfo.getAvatar()) ? userInfo.getAvatar() : null);
            userWhitelist.setIdNumber(StrUtil.isNotEmpty(userInfo.getIdNumber()) ? userInfo.getIdNumber() : null);
            userWhitelist.setCardIdEx(StrUtil.isNotEmpty(userInfo.getCardIdEx()) ? userInfo.getCardIdEx() : null);
            userWhitelist.setName(StrUtil.isNotEmpty(userInfo.getUsername()) ? userInfo.getUsername() : null);
            userWhitelist.setPushType(0);
            userWhitelistDao.updateByPrimaryKeySelective(userWhitelist);
            userWhitelistService.addSingle(userWhitelist.getId(), userWhitelist.getStartTime(), userWhitelist.getEndTime());
        }
    }

    @Override
    public RestResult<List<FaceDeviceVO>> userWhiteGetSnList(Integer adminId, Integer companyId) {
        log.info("楼宇Id：" + adminId);
        log.info("公司Id：" + companyId);
        List<FaceDevice> list = faceDeviceDao.select(new FaceDevice().setAdminId(adminId).setEnable(1).setAuth(1));
        if (companyId != null) {
            List<FaceDeviceLessVO> snList = faceDeviceDao.getSnList(companyId);
            List<FaceDevice> collect = snList.stream().map(p -> {
                FaceDevice faceDevice = new FaceDevice();
                BeanUtils.copyProperties(p, faceDevice);
                return faceDevice;
            }).collect(Collectors.toList());
            collect.stream().sequential().collect(Collectors.toCollection(() -> list));
        }
        List<FaceDeviceVO> collect = list.stream().map(p ->
        {
            FaceDeviceVO faceDeviceVO = new FaceDeviceVO();
            faceDeviceVO.setSn(p.getSn());
            faceDeviceVO.setName(p.getName());
            return faceDeviceVO;
        }).collect(Collectors.toList());
        log.info("返回数据：" + collect);
        return RestResponse.ok(collect);
    }

    @Override
    @Transactional
    public RestResult<List<UserWhiteInviteVO>> userWhiteInviteUserVisitor(UserWhiteInviteDTO userWhiteInviteDTO) {
        return RestResponse.ok(handleUserWhiteInviteUserVisitor(userWhiteInviteDTO, Boolean.FALSE));
    }

    private List<UserWhiteInviteVO> handleUserWhiteInviteUserVisitor(UserWhiteInviteDTO userWhiteInviteDTO, Boolean isAlipay) {
        /*时间校验*/
        if (StrUtil.isEmpty(userWhiteInviteDTO.getStartTime()) || StrUtil.isEmpty(userWhiteInviteDTO.getEndTime())) {
            throw new ServiceException("请输入访问的时间");
        }
        if (Convert.toDate(userWhiteInviteDTO.getStartTime()).getTime() >= Convert.toDate(userWhiteInviteDTO.getEndTime()).getTime()) {
            throw new ServiceException("开始时间必须大于结束时间");
        }
        if (Convert.toDate(userWhiteInviteDTO.getEndTime()).getTime() <= new Date().getTime()) {
            throw new ServiceException("结束时间必须大于当前时间");
        }

        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(userWhiteInviteDTO.getUserWhitelistId());
        ClassUtil.checkData(userWhitelist, "常客不存在");
        Integer count = userVisitorListDao.selectCountToday(userWhitelist.getId());
        if (userWhiteInviteDTO.getPhoneList() != null) {
            int size = userWhiteInviteDTO.getPhoneList().size();
            if (count + size > 10 && userWhiteInviteDTO.getInvitationWay() == 1) {
                throw new ServiceException("每天邀请的次数不可以超过10次,目前已使用" + count + "次");
            }
        }
        AdminDetail adminDetail = adminDetailDao.selectOne(new AdminDetail().setAdminId(userWhitelist.getAdminId()));
        ClassUtil.checkData(adminDetail, "adminDetail");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        UserVisitorListDetail userVisitorListDetail = new UserVisitorListDetail();
        BeanUtils.copyProperties(userWhiteInviteDTO, userVisitorListDetail);
        try {
            userVisitorListDetail.setStartTime(sdf.parse(userWhiteInviteDTO.getStartTime()));
            userVisitorListDetail.setEndTime(sdf.parse(userWhiteInviteDTO.getEndTime()));
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        String sn = CollUtil.isNotEmpty(userWhiteInviteDTO.getSnList()) ? Joiner.on(",").join(userWhiteInviteDTO.getSnList()) : null;
        userVisitorListDetail.setVisiteeXm(userWhitelist.getName());
        userVisitorListDetail.setVisiteePhone(userWhitelist.getPhone());
        userVisitorListDetail.setVisiteeCompany(adminDao.selectByPrimaryKey(adminDetail.getAdminId()).getUsername());
        userVisitorListDetail.setSn(sn);
        userVisitorListDetail.setVisitingType(VisitingTypeEnum.INVITE_RECORD.getCode());
        List<UserWhiteInviteVO> userWhiteInviteVOS = new ArrayList<>();
        for (int i = 0; i < userWhiteInviteDTO.getPhoneList().size(); i++) {
            userVisitorListDetail.setId(null);
            userVisitorListDetail.setPhone(userWhiteInviteDTO.getPhoneList().get(i));
            userVisitorListDetail.setAdminId(userWhitelist.getAdminId());
            userVisitorListDetail.setCreateTime(new Date());
            userVisitorListDetailDao.insertSelective(userVisitorListDetail);
            UserWhiteInviteVO userWhiteInviteVO = new UserWhiteInviteVO();
            userWhiteInviteVO.setPhone(userWhiteInviteDTO.getPhoneList().get(i));
            userWhiteInviteVO.setUserVisitorListDetailId(userVisitorListDetail.getId());
            userWhiteInviteVOS.add(userWhiteInviteVO);
            Integer personnelType = userWhiteInviteDTO.getPersonnelType();
            if (isAlipay) { //支付宝
                alipayOpenApiService.createAlipayUrlAndSendMsg(userVisitorListDetail);

                alipayOpenApiService.subscribeAlipayMessage(null, userInfoDao.selectZfbUserIdByPhone(userWhiteInviteDTO.getPhoneList().get(i)), WXMessageTemplateEnum.REMIND_REVICE_APPLICATION, userVisitorListDetail.getReason(), userWhitelist.getPhone(), personnelType == 0 ? "访客" : (personnelType == 1 ? "同事" : "亲属"), userVisitorListDetail.getVisiteeXm() + "对您邀约", null);
            } else {//微信
                if (userWhiteInviteDTO.getInvitationWay() == 1) {
                    smsService.createUrl(userVisitorListDetail);
                }
                Long userId = userInfoDao.selectByPhone(userWhiteInviteDTO.getPhoneList().get(i));
                smsService.subscribeMessage(null, userId, WXMessageTemplateEnum.REMIND_REVICE_APPLICATION, userVisitorListDetail.getReason(), userWhitelist.getPhone(), personnelType == 0 ? "访客" : (personnelType == 1 ? "同事" : "亲属"), userVisitorListDetail.getVisiteeXm() + "对您邀约", null);
            }
        }
        return userWhiteInviteVOS;
    }

    @Override
    public RestResult<UserVisitorListDetailVO> getVisitoryByUserVisitorListDetailId(Integer userVisitorListDetailId) {
        UserVisitorListDetail uvDetail = userVisitorListDetailDao.selectByPrimaryKey(userVisitorListDetailId);
        ClassUtil.checkData(uvDetail, "该信息不存在");
        UserVisitorListDetailVO userVisitorListDetailVO = new UserVisitorListDetailVO();
        BeanUtils.copyProperties(uvDetail, userVisitorListDetailVO);
        Admin companyAdmin = adminDao.selectByPrimaryKey(uvDetail.getAdminId());
        ClassUtil.checkData(companyAdmin, "所属公司不存在");
        Admin buildingAdmin = adminDao.selectByPrimaryKey(companyAdmin.getTenantId());
        ClassUtil.checkData(buildingAdmin, "所属楼宇不存在");
        userVisitorListDetailVO.setId(uvDetail.getId());
        userVisitorListDetailVO.setVisiteeCompanyId(companyAdmin.getId());
        userVisitorListDetailVO.setVisiteeCompany(companyAdmin.getUsername());
        userVisitorListDetailVO.setVisiteeBuilding(buildingAdmin.getUsername());
        userVisitorListDetailVO.setVisiteeBuildingId(buildingAdmin.getId());
        if (StringUtils.isNotEmpty(uvDetail.getContent())) {
            userVisitorListDetailVO.setContents(JSON.parseArray(uvDetail.getContent(), ContextVO.class));
        } else {
            userVisitorListDetailVO.setContents(new ArrayList<>());
        }

        return RestResponse.ok(userVisitorListDetailVO);
    }

    @Override
    public RestResult<List<UserWhiteIntervieweeBuildingVO>> getUserWhiteByphoneAndAdminId(UserWhiteIntervieweeDTO userWhiteIntervieweeDTO) {
        if (userWhiteIntervieweeDTO == null || StrUtil.isEmpty(userWhiteIntervieweeDTO.getPhone())) {
            return RestResponse.error("受访人手机号不能为空");
        }
        Admin admin = adminDao.selectByPrimaryKey(userWhiteIntervieweeDTO.getBuildingAdminId());
        ClassUtil.checkData(admin, "该楼宇不存在");
        ClassUtil.throwBusinessException(!AdminTypeEnum.DISTRICT.getCode().equals(admin.getType()), "请选择区级账号");
        ClassUtil.throwBusinessException(!BUILDING_PLACE.getCode().equals(admin.getPlaceType()), "请选择楼宇类型账号");
        List<UserWhitelistBuildingVO> userWhiteList = userWhitelistDao.selectByPhoneInCompany(userWhiteIntervieweeDTO.getPhone(), userWhiteIntervieweeDTO.getBuildingAdminId());
        if (userWhiteList == null) {
            return RestResponse.error("没有对应的常客");
        }
        userWhiteList = userWhiteList.stream().filter(p -> p.getStatus().equals(1)).collect(Collectors.toList());
        if (userWhiteList.size() == 0) {
            return RestResponse.error("没有实名认证完成的常客信息");
        }
        List<UserWhiteIntervieweeBuildingVO> buildingVOList = new ArrayList<>();
        Map<Integer, List<UserWhitelistBuildingVO>> map = userWhiteList.stream().filter(p -> p.getBuildingAdminId() != null).collect(Collectors.groupingBy(UserWhitelistBuildingVO::getBuildingAdminId));
        if (map.size() == 0) {
            return RestResponse.error("没有该常客信息");
        }
        for (Integer key : map.keySet()) {
            UserWhiteIntervieweeBuildingVO buildingVO = new UserWhiteIntervieweeBuildingVO();
            List<UserWhitelistBuildingVO> userWhitelistBuildingVOS = map.get(key);
            for (UserWhitelistBuildingVO userWhitelistBuildingVO : userWhitelistBuildingVOS) {
                buildingVO.setAdminId(userWhitelistBuildingVO.getBuildingAdminId());
                buildingVO.setUsername(userWhitelistBuildingVO.getBuildingName());
                List<UserWhiteIntervieweeVO> userWhiteIntervieweeVOList;
                if (CollectionUtil.isEmpty(buildingVO.getUserWhiteIntervieweeVOList())) {
                    userWhiteIntervieweeVOList = new ArrayList<>();
                } else {
                    userWhiteIntervieweeVOList = buildingVO.getUserWhiteIntervieweeVOList();
                }
                UserWhiteIntervieweeVO userWhiteIntervieweeVO = new UserWhiteIntervieweeVO();
                userWhiteIntervieweeVO.setUserWhitelistId(userWhitelistBuildingVO.getId());
                userWhiteIntervieweeVO.setAdminId(userWhitelistBuildingVO.getAdminId());
                userWhiteIntervieweeVO.setCompanyName(userWhitelistBuildingVO.getCompnyName());
                userWhiteIntervieweeVO.setName(userWhitelistBuildingVO.getName());
                userWhiteIntervieweeVO.setPhone(userWhitelistBuildingVO.getPhone());
                userWhiteIntervieweeVOList.add(userWhiteIntervieweeVO);
                buildingVO.setUserWhiteIntervieweeVOList(userWhiteIntervieweeVOList);
            }
            buildingVOList.add(buildingVO);
        }
        return RestResponse.ok(buildingVOList);
    }

    @Override
    public RestResult<Boolean> visitorySubmit(VisitorySubmitDTO visitorySubmitDTO) {
        handleVisitorySubmit(visitorySubmitDTO, Boolean.FALSE);
        return RestResponse.ok(true);
    }

    private void handleVisitorySubmit(VisitorySubmitDTO visitorySubmitDTO, Boolean isAlipay) {
        Admin buildingAdmin = adminDao.selectByPrimaryKey(visitorySubmitDTO.getVisiteeBuildingId());
        ClassUtil.checkData(buildingAdmin, "楼宇账号不存在");
        /*时间校验*/
        if (visitorySubmitDTO.getStartTime() == null || visitorySubmitDTO.getEndTime() == null) {
            throw new ServiceException("请输入访问的时间");
        }
        if (visitorySubmitDTO.getStartTime().getTime() >= visitorySubmitDTO.getEndTime().getTime()) {
            throw new ServiceException("开始时间必须大于结束时间");
        }
        if (visitorySubmitDTO.getEndTime().getTime() <= new Date().getTime()) {
            throw new ServiceException("结束时间必须大于当前时间");
        }

        UserVisitorListDetail userVisitorListDetail = BeanUtil.copyProperties(visitorySubmitDTO, UserVisitorListDetail.class);
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(visitorySubmitDTO.getUserId());
        /*访客信息填入*/
        userVisitorListDetail.setPhoto(userInfo.getAvatar());
        userVisitorListDetail.setName(userInfo.getUsername());
        userVisitorListDetail.setPhone(userInfo.getPhone());
        userVisitorListDetail.setIdNumber(userInfo.getIdNumber());
        if (!CollectionUtils.isEmpty(visitorySubmitDTO.getAddCustomList())) {
            String s = JSON.toJSONString(visitorySubmitDTO.getAddCustomList());
            userVisitorListDetail.setContent(s);
        }
        /*根据楼宇账号配置的不同，获取不同的企业信息*/
        Integer adminId;
        List<Long> userIdList = new LinkedList<>();
        if (AdminBuildingCheckTypeEnum.CKSH.getCode().equals(buildingAdmin.getBuildingCheckType())) {//常客审核
            UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(visitorySubmitDTO.getUserWhitelistId());
            ClassUtil.checkData(userWhitelist, "常客不存在");
            userVisitorListDetail.setVisiteeXm(userWhitelist.getName());
            userVisitorListDetail.setVisiteeCompany(adminDetailDao.selectOne(new AdminDetail().setAdminId(userWhitelist.getAdminId())).getEnterpriseFullName());
            userVisitorListDetail.setVisiteePhone(userWhitelist.getPhone());
            adminId = userWhitelist.getAdminId();
            userIdList.add(userWhitelist.getUserId());
            //TODO 补充受访人所属区域
            userVisitorListDetail.setZoneDevicePassIdList(userWhitelist.getZoneDevicePassIdList());
        } else {
            ClassUtil.throwBusinessException(visitorySubmitDTO.getVisiteeCompanyId() == null, "请选择要拜访的公司");
            adminId = visitorySubmitDTO.getVisiteeCompanyId();
            List<UserWhitelistCheck> userWhitelistCheckList = userWhitelistCheckDao.select(new UserWhitelistCheck().setAdminId(visitorySubmitDTO.getVisiteeBuildingId()));
            List<Integer> userWhitelistIdList = userWhitelistCheckList.stream().map(UserWhitelistCheck::getUserWhitelistId).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(userWhitelistIdList)) {
                List<UserWhitelist> userWhitelistList = userWhitelistDao.selectByIds(StrUtil.join(",", userWhitelistIdList));
                userIdList = userWhitelistList.stream().map(UserWhitelist::getUserId).filter(ObjectUtil::isNotNull).collect(Collectors.toList());
            }
        }



        /*2022-08-12 三码数据校验*/
        this.adminThreeCodeCheck(adminId, visitorySubmitDTO);
        userVisitorListDetail.setJkm(visitorySubmitDTO.getJkm());
        userVisitorListDetail.setJkmPhoto(visitorySubmitDTO.getJkmPhoto());
        userVisitorListDetail.setHs(visitorySubmitDTO.getHs());
        userVisitorListDetail.setHsPhoto(visitorySubmitDTO.getHsPhoto());
        userVisitorListDetail.setXc(visitorySubmitDTO.getXc());
        userVisitorListDetail.setXcPhoto(visitorySubmitDTO.getXcPhoto());
        userVisitorListDetail.setXcPhone(visitorySubmitDTO.getXcPhone());
        userVisitorListDetail.setHsCheckTime(visitorySubmitDTO.getHsCheckTime());
        userVisitorListDetail.setHsResultTime(visitorySubmitDTO.getHsResultTime());
        userVisitorListDetail.setTime(DateUtils.getTime());



        /*1.无邀请*/
        if (visitorySubmitDTO.getUserVisitorListDetailId() == null) {
            userVisitorListDetail.setReviewStatus(0);
            userVisitorListDetail.setVisitingType(VisitingTypeEnum.APPLY_RECORD.getCode());
            userVisitorListDetail.setAdminId(adminId);
            userVisitorListDetail.setCreateTime(new Date());
            userVisitorListDetailDao.insertSelective(userVisitorListDetail);
            if (isAlipay) {
                for (Long userId : userIdList) {
                    alipayOpenApiService.subscribeAlipayMessage(null, userId, WXMessageTemplateEnum.REVICE_APPLICATION, userInfo.getUsername(), userInfo.getPhone(), visitorySubmitDTO.getCompany(), userVisitorListDetail.getReason(), null);
                }
                alipayOpenApiService.subscribeAlipayMessage(null, userInfo.getId(), WXMessageTemplateEnum.COMMIT_APPLICATION, userInfo.getUsername(), userInfo.getPhone(), userVisitorListDetail.getCompany(), userVisitorListDetail.getReason(), "访客申请提交");
            } else {
                for (Long userId : userIdList) {
                    smsService.subscribeMessage(null, userId, WXMessageTemplateEnum.REVICE_APPLICATION, userInfo.getUsername(), userInfo.getPhone(), visitorySubmitDTO.getCompany(), userVisitorListDetail.getReason(), null);
                }
                smsService.subscribeMessage(null, userInfo.getId(), WXMessageTemplateEnum.COMMIT_APPLICATION, userInfo.getUsername(), userInfo.getPhone(), userVisitorListDetail.getCompany(), userVisitorListDetail.getReason(), "访客申请提交");
            }

        } else {
            UserVisitorListDetail userVisitorListDetail2 = userVisitorListDetailDao.selectByPrimaryKey(visitorySubmitDTO.getUserVisitorListDetailId());
            if (!userVisitorListDetail2.getPhone().equals(userInfo.getPhone())) {
                throw new ServiceException("您没有权限接收邀请");
            }
            if (userVisitorListDetail2.getReviewStatus() != null && userVisitorListDetail2.getReviewStatus() == 2) {
                throw new ServiceException("您已经接收邀请");
            }
            userVisitorListDetail.setId(visitorySubmitDTO.getUserVisitorListDetailId().longValue());
            userVisitorListDetail.setReviewStatus(2);
            /*1.访客信息填入*/
            String content = "";
            if (!CollectionUtils.isEmpty(visitorySubmitDTO.getAddCustomList())) {
                content = JSON.toJSONString(visitorySubmitDTO.getAddCustomList());
            }
            UserVisitorList userVisitorList = this.getUserVisitorList(userInfo, adminId, content);
            //将数据进行关联
            userVisitorListDetail.setUserVisitorListId(userVisitorList.getId());
            userVisitorListDetail.setPhoto(userInfo.getAvatar());
            userVisitorListDetailDao.updateByPrimaryKeySelective(userVisitorListDetail);
            /*下发到楼宇的设备中*/
            UserVisitorListDetail userVisitorListDetail1 = userVisitorListDetailDao.selectByPrimaryKey(userVisitorListDetail.getId());
            String snString = userVisitorListDetail1.getSn();
            List<String> snList = StrUtil.isBlank(snString) ? new ArrayList<>() : Arrays.stream(snString.split(",")).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(snList)) {
                for (String sn : snList) {
                    DownFaceNew downFaceNew = new DownFaceNew();
                    downFaceNew.setCardid(userInfo.getIdNumber());
                    downFaceNew.setRightno(UserRightNoEnum.VISITORS.getCode());
                    downFaceNew.setPhoto(userInfo.getAvatar());
                    downFaceNew.setName(userInfo.getUsername());
                    downFaceNew.setUserId(userInfo.getId());
                    downFaceNew.setSn(sn);
                    downFaceNew.setPhone(userInfo.getPhone());
                    downFaceNew.setValidPeriodStart(visitorySubmitDTO.getStartTime());
                    downFaceNew.setValidPeriodEnd(visitorySubmitDTO.getEndTime());
                    downFaceNew.setFacialFeature(userInfo.getFacialFeature());
                    downFaceNew.setAdminId(adminId);
                    faceTBService.downUser(downFaceNew);
                }
            }
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日 HH:mm");
            for (Long userId : userIdList) {
                UserInfo info = userInfoDao.selectByPrimaryKey(userId);
                if (ObjectUtil.isNull(info)) {
                    continue;
                }
                if (isAlipay) {
                    alipayOpenApiService.subscribeAlipayMessage(null, userId, WXMessageTemplateEnum.INVITE_RESULT, userInfo.getUsername(), info.getUsername(), userVisitorListDetail2.getReason(), sdf.format(userVisitorListDetail.getStartTime()) + "~" + sdf.format(userVisitorListDetail.getEndTime()), "您的邀约已完成");
                } else {
                    smsService.subscribeMessage(null, userId, WXMessageTemplateEnum.INVITE_RESULT, userInfo.getUsername(), info.getUsername(), userVisitorListDetail2.getReason(), sdf.format(userVisitorListDetail.getStartTime()) + "~" + sdf.format(userVisitorListDetail.getEndTime()), "您的邀约已完成");
                }
            }
        }
        //生成梯控码和通用二维码
        this.generateCode(userVisitorListDetail, visitorySubmitDTO.getUserWhitelistId());
    }

    private void generateCode(UserVisitorListDetail uf, Integer userWhitelistId) {
        //梯控二维码
        try {
            //根据常客获取楼层
            String ladderQrCode = this.getLadderQrCode(uf, userWhitelistId);

            uf.setLadderQrCode(ladderQrCode);
        } catch (Exception e) {
            log.error("楼层有误");
        }
        //通用二维码
        if (ObjectUtil.isNotNull(uf) && ObjectUtil.isNotNull(uf.getId())) {
            FkjCommonCodeVo fkjCommonCodeVo = new FkjCommonCodeVo();
            fkjCommonCodeVo.setAction(FKJ_COMMON_CODE.getCode());
            fkjCommonCodeVo.setUserVisitorListDetailId(uf.getId());
            String commonQrCode = AesUtils.encode(Constants.FKJ_KEY, JSON.toJSONString(fkjCommonCodeVo));
            //二维码存储起来
            uf.setCommonQrCode(commonQrCode);
            userVisitorListDetailDao.updateByPrimaryKeySelective(uf);
        }
    }

    private String getLadderQrCode(UserVisitorListDetail uf, Integer userWhitelistId) {
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(userWhitelistId);
        if (ObjectUtil.isNull(userWhitelist)) {
            return "";
        }
        Visitee visitee = this.getVisitee(userWhitelist.getIdNumber(), userWhitelist.getPhone(), userWhitelist.getAdminId());
        if (ObjectUtil.isNull(visitee)) {
            return "";
        }
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        return fkjService.qrCodeCreate(Preconditions.isNotBlank(visitee.getFloor()) ? visitee.getFloor() : null, uf.getStartTime(), uf.getEndTime(), userWhitelist.getAdminId(), ObjectUtil.isNotNull(admin) ? admin.getUsername() : "", uf.getIdNumber(), uf.getName(), uf.getPhone(), uf.getVisiteeXm());
    }

    public Visitee getVisitee(String idNumber, String phone, Integer adminId) {
        List<Visitee> visiteeList = null;
        if (StrUtil.isNotEmpty(idNumber)) {
            VisiteePageDto visiteePageDto = new VisiteePageDto();
            visiteePageDto.setIdNumber(idNumber);
            visiteePageDto.setAdminId(adminId);
            visiteeList = visiteeDao.list(visiteePageDto);
            if ((CollUtil.isNotEmpty(visiteeList))) {
                return visiteeList.get(0);
            }
        }
        if (StrUtil.isNotEmpty(phone)) {
            VisiteePageDto visiteePageDto = new VisiteePageDto();
            visiteePageDto.setPhone(phone);
            visiteePageDto.setAdminId(adminId);
            visiteeList = visiteeDao.list(visiteePageDto);
            if ((CollUtil.isNotEmpty(visiteeList))) {
                return visiteeList.get(0);
            }
        }
        return null;
    }

    private void adminThreeCodeCheck(Integer adminId, VisitorySubmitDTO visitorySubmitDTO) {
        Admin admin = adminDao.selectByPrimaryKey(adminId);
        ClassUtil.checkData(admin, "企业账号不存在");
        String threeCode = admin.getThreeCode();
        if (threeCode.contains(AdminThreeCodeEnum.JKM.getCode().toString())) {
            if (StringUtils.isEmpty(visitorySubmitDTO.getJkm()) && StringUtils.isEmpty(visitorySubmitDTO.getJkmPhoto())) {
                throw new ServiceException("健康码数据不能为空");
            }
        }
        if (threeCode.contains(AdminThreeCodeEnum.HS.getCode().toString())) {
            if (StringUtils.isEmpty(visitorySubmitDTO.getHs()) && StringUtils.isEmpty(visitorySubmitDTO.getHsPhoto())) {
                throw new ServiceException("核酸数据不能为空");
            }
        }
        if (threeCode.contains(AdminThreeCodeEnum.XCM.getCode().toString())) {
            if (StringUtils.isEmpty(visitorySubmitDTO.getXc()) && StringUtils.isEmpty(visitorySubmitDTO.getXcPhoto())) {
                throw new ServiceException("行程码数据不能为空");
            }
        }

    }

    @Override
    public RestResult<PageInfo<VisitoryVisitListVO>> visitoryVisitList(RestDTO<VisitoryVisitDTO> dto) {
        VisitoryVisitDTO data = dto.getData();
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(data.getUserId());
        ClassUtil.checkData(userInfo, "用户不存在");
        List<UserInfo> userInfoList = userInfoDao.select(new UserInfo().setBuildingOpenId(userInfo.getBuildingOpenId()));
        List<Long> userIdList = userInfoList.stream().map(UserInfo::getId).collect(Collectors.toList());
        if (CollUtil.isEmpty(userIdList)) {
            return RestResponse.ok(new PageInfo<>());
        }
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<VisitoryVisitListVO> visitoryVisitList = userVisitorListDetailDao.visitoryVisitList(userIdList, data.getUserVisitorListDetailId());
        for (VisitoryVisitListVO visitoryVisitListVO : visitoryVisitList) {
            if (StringUtils.isNotEmpty(visitoryVisitListVO.getContent())) {
                visitoryVisitListVO.setContents(JSON.parseArray(visitoryVisitListVO.getContent(), ContextVO.class));
            } else {
                visitoryVisitListVO.setContents(new ArrayList<>());
            }
        }
        return RestResponse.ok(new PageInfo<>(getSN(visitoryVisitList)));
    }

    @Override
    public RestResult<PageInfo<VisitoryVisitListVO>> userWhiteVisitList(RestDTO<UserWhiteVisitDTO> dto) {
        ClassUtil.throwBusinessException(CollUtil.isEmpty(dto.getData().getUserWhiteCheckVOList()), "传入常客数据");
        UserWhiteCheckVO userWhiteCheckVO = dto.getData().getUserWhiteCheckVOList().get(0);
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(userWhiteCheckVO.getUserWhitelistId());
        ClassUtil.checkData(userWhitelist, "常客不存在，请重新认证");
        Admin companyAdmin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        ClassUtil.checkData(companyAdmin, "公司不存在");
        Admin buildingAdmin = adminDao.selectByPrimaryKey(companyAdmin.getTenantId());
        ClassUtil.checkData(buildingAdmin, "楼宇不存在");

        List<VisitoryVisitListVO> visitoryVisitList;
        if (AdminBuildingCheckTypeEnum.CKSH.getCode().equals(buildingAdmin.getBuildingCheckType())
                || VisitingTypeEnum.INVITE_RECORD.getCode().equals(dto.getData().getVisitingType())
        ) {
            PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
            visitoryVisitList = userVisitorListDetailDao.userWhiteVisitList(dto.getData());
        } else {
            List<UserWhitelistCheck> userWhitelistCheckList = userWhitelistCheckDao.select(new UserWhitelistCheck().setUserWhitelistId(userWhitelist.getId()).setAdminId(buildingAdmin.getId()));
            List<Integer> userWhitelistIdList = userWhitelistCheckList.stream().map(UserWhitelistCheck::getUserWhitelistId).collect(Collectors.toList());
            if (CollUtil.isEmpty(userWhitelistIdList) || !userWhitelistIdList.contains(userWhitelist.getId())) {
                return RestResponse.ok(new PageInfo<>());
            }

            List<Admin> adminList = adminDao.select(new Admin().setTenantId(buildingAdmin.getId()));
            if (CollUtil.isNotEmpty(adminList)) {
                List<Integer> adminIdList = adminList.stream().map(Admin::getId).collect(Collectors.toList());
                Weekend<UserVisitorListDetail> weekend = new Weekend<>(UserVisitorListDetail.class);
                weekend.weekendCriteria().andIn(UserVisitorListDetail::getAdminId, adminIdList)
                        .andEqualTo(UserVisitorListDetail::getVisitingType, dto.getData().getVisitingType());
                weekend.orderBy("id").desc();
                PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
                List<UserVisitorListDetail> list = userVisitorListDetailDao.selectByExample(weekend);
                visitoryVisitList = list.stream().map(p -> {
                    VisitoryVisitListVO item = BeanUtil.copyProperties(p, VisitoryVisitListVO.class);
                    item.setUserVisitorListDetailId(p.getId());
                    return item;
                }).collect(Collectors.toList());
            } else {
                visitoryVisitList = new ArrayList<>();
            }
        }
        return RestResponse.ok(new PageInfo<>(getSN(visitoryVisitList)));
    }

    private List<VisitoryVisitListVO> getSN(List<VisitoryVisitListVO> visitoryVisitList) {
        Set<String> snList = new LinkedHashSet<>();
        for (int i = 0; i < visitoryVisitList.size(); i++) {
            VisitoryVisitListVO visitoryVisitListVO = visitoryVisitList.get(i);
            if (StringUtils.isNotEmpty(visitoryVisitListVO.getSn())) {
                String sn = visitoryVisitListVO.getSn();
                List<String> collect = Arrays.stream(sn.split(",")).collect(Collectors.toList());
                snList.addAll(collect);
            }
        }
        Map<String, FaceDeviceVO> map = new HashMap<>();
        if (CollUtil.isNotEmpty(snList)) {
            List<FaceDeviceVO> bySnList = faceDeviceDao.getBySnList(new ArrayList<>(snList));
            map = bySnList.stream().collect(Collectors.toMap(FaceDeviceVO::getSn, faceDeviceVO -> faceDeviceVO));
        }
        for (int i = 0; i < visitoryVisitList.size(); i++) {
            VisitoryVisitListVO visitoryVisitListVO = visitoryVisitList.get(i);
            if (StringUtils.isNotEmpty(visitoryVisitListVO.getSn())) {
                String sn = visitoryVisitListVO.getSn();
                List<String> throughList = new LinkedList<>();
                List<String> collect = Arrays.stream(sn.split(",")).collect(Collectors.toList());
                for (String s : collect) {
                    if (map.containsKey(s)) {
                        throughList.add(map.get(s).getName());
                    }
                }
                visitoryVisitListVO.setThroughList(throughList);
            }
            if (StringUtils.isNotEmpty(visitoryVisitListVO.getContent())) {
                List<ContextVO> contextVOList = JSONArray.parseArray(visitoryVisitList.get(i).getContent(), ContextVO.class);
                visitoryVisitList.get(i).setContents(contextVOList);
            }
        }
        return visitoryVisitList;
    }

    @Override
    @Transactional
    public RestResult<Boolean> userWhiteHandleVisitList(UserWhiteHandleDTO userWhiteHandleDTO) {
        handleUserWhiteHandleVisitList(userWhiteHandleDTO, Boolean.FALSE);
        return RestResponse.ok(true);
    }

    private void handleUserWhiteHandleVisitList(UserWhiteHandleDTO userWhiteHandleDTO, Boolean isAlipay) {
        UserVisitorListDetail userVisitorListDetail = userVisitorListDetailDao.selectByPrimaryKey(userWhiteHandleDTO.getUserVisitorListDetailId());
        if (userVisitorListDetail == null) {
            throw new ServiceException("数据有误");
        }
        if (userVisitorListDetail.getReviewStatus() == 2) {
            throw new ServiceException("已同意访客访问");
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userVisitorListDetail.getUserId());
        if (userInfo == null) {
            throw new ServiceException("数据有误");
        }

        boolean result = false;
        userVisitorListDetail.setReviewStatus(userWhiteHandleDTO.getReviewStatus());
        userVisitorListDetailDao.updateByPrimaryKeySelective(userVisitorListDetail);
        UserWhitelist userWhitelist = null;
        if (userWhiteHandleDTO.getReviewStatus() == 1) {
            userVisitorListDetail.setRemark(userWhiteHandleDTO.getRemark());
            userVisitorListDetailDao.updateByPrimaryKeySelective(userVisitorListDetail);
        } else if (userWhiteHandleDTO.getReviewStatus() == 2) {
            if (userWhiteHandleDTO.getSnList() == null || userWhiteHandleDTO.getSnList().size() == 0) {
                throw new ServiceException("设备列表不能为空");
            }
            result = true;
            /*1.访客信息填入*/
            userWhitelist = userWhitelistDao.selectByPrimaryKey(userVisitorListDetail.getUserWhitelistId());
            ClassUtil.checkData(userWhitelist, "常客不存在");
            UserVisitorList userVisitorList = getUserVisitorList(userInfo, userWhitelist.getAdminId(), userVisitorListDetail.getContent());
            //将数据进行关联
            userVisitorListDetail.setSn(String.join(",", userWhiteHandleDTO.getSnList()));
            userVisitorListDetail.setUserVisitorListId(userVisitorList.getId());
            userVisitorListDetail.setPersonnelType(userWhiteHandleDTO.getPersonnelType());
            userVisitorListDetail.setRemark(userWhiteHandleDTO.getRemark());
            userVisitorListDetailDao.updateByPrimaryKeySelective(userVisitorListDetail);
            /*2.人脸下发*/
            List<String> snList = userWhiteHandleDTO.getSnList();
            if (CollectionUtil.isNotEmpty(snList)) {
                for (String sn : snList) {
                    DownFaceNew downFaceNew = new DownFaceNew();
                    downFaceNew.setCardid(userInfo.getIdNumber());
                    downFaceNew.setRightno(UserRightNoEnum.VISITORS.getCode());
                    downFaceNew.setPhoto(userInfo.getAvatar());
                    downFaceNew.setName(userInfo.getUsername());
                    downFaceNew.setUserId(userInfo.getId());
                    downFaceNew.setSn(sn);
                    downFaceNew.setPhone(userInfo.getPhone());
                    downFaceNew.setValidPeriodStart(userVisitorListDetail.getStartTime());
                    downFaceNew.setValidPeriodEnd(userVisitorListDetail.getEndTime());
                    downFaceNew.setFacialFeature(userInfo.getFacialFeature());
                    downFaceNew.setAdminId(userWhitelist.getAdminId());
                    faceTBService.downUser(downFaceNew);
                }
            }
        } else {
            throw new ServiceException("无需修改为未处理");
        }
        if (isAlipay) {
            alipayOpenApiService.subscribeAlipayMessage(null, userInfo.getId(), WXMessageTemplateEnum.COMMIT_RESULT, userInfo.getUsername(), result ? "成功" : "失败", result ? "您的申请已通过，受访人为 " + userWhitelist.getName() : "申请失败", null, null);
        } else {
            smsService.subscribeMessage(null, userInfo.getId(), WXMessageTemplateEnum.COMMIT_RESULT, userInfo.getUsername(), result ? "成功" : "失败", result ? "您的申请已通过，受访人为 " + userWhitelist.getName() : "申请失败", null, null);
        }

    }

    private UserVisitorList getUserVisitorList(UserInfo userInfo, Integer adminId, String content) {
        AdminVO info = adminService.getInfo(adminId);
        String openId = "";
        switch (AdminPlaceTypeEnum.getEnum(info.getPlaceType())) {
            case COMMON_PLACE:
                openId = userInfo.getCommonOpenId();
                break;
            case APPOINTMENT_PLACE:
                openId = userInfo.getAppointmentOpenId();
                break;
            case BUILDING_PLACE:
                openId = userInfo.getBuildingOpenId();
                break;
        }
        UserVisitorList userVisitorList = userVisitorListService.getUserVisitorList(userInfo.getIdNumber(), userInfo.getPhone(), openId, adminId);
        if (userVisitorList != null) {
            userVisitorList.setAvatar(userInfo.getAvatar());
            userVisitorList.setName(userInfo.getUsername());
            userVisitorList.setPhone(userInfo.getPhone());
            userVisitorList.setIdNumber(userInfo.getIdNumber());
            userVisitorList.setContent(content);
            userVisitorList.setUserId(userInfo.getId());
            userVisitorListDao.updateByPrimaryKeySelective(userVisitorList);
            return userVisitorList;
        }
        userVisitorList = new UserVisitorList();
        userVisitorList.setAvatar(userInfo.getAvatar());
        userVisitorList.setName(userInfo.getUsername());
        userVisitorList.setPhone(userInfo.getPhone());
        userVisitorList.setIdNumber(userInfo.getIdNumber());
        userVisitorList.setPushType(0);
        userVisitorList.setAdminId(adminId);
        userVisitorList.setCreateTime(new Date());
        userVisitorList.setContent(content);
        userVisitorList.setUserId(userInfo.getId());
        userVisitorListDao.insertSelective(userVisitorList);
        return userVisitorList;
    }

    @Override
    public RestResult<List<CompanyVO>> userWhiteLookCompanyInfo(List<UserWhiteCheckVO> userWhiteCheckVOList) {
        /*1.通过常客列表查询到常客对应admin列表*/
        if (userWhiteCheckVOList == null | userWhiteCheckVOList.size() == 0) {
            return RestResponse.error("请传入常客列表");
        }
        List<Integer> userWhiteIdList = userWhiteCheckVOList.stream().map(UserWhiteCheckVO::getUserWhitelistId).collect(Collectors.toList());

        Weekend<UserWhitelist> weekend = new Weekend<>(UserWhitelist.class);
        weekend.weekendCriteria().andIn(UserWhitelist::getId, userWhiteIdList);
        List<UserWhitelist> userWhitelistList = userWhitelistDao.selectByExample(weekend);
        List<Integer> adminIdList = userWhitelistList.stream().map(UserWhitelist::getAdminId).collect(Collectors.toList());
        /*2.通过adminId列表查询到对应adminDetail列表*/
        Weekend<AdminDetail> adminWeekend = new Weekend<>(AdminDetail.class);
        adminWeekend.weekendCriteria().andIn(AdminDetail::getAdminId, adminIdList);
        List<AdminDetail> list = adminDetailDao.selectByExample(adminWeekend);
        /*3.组装数据返回*/
        list = list.stream().sorted(Comparator.comparing(AdminDetail::getId).reversed()).collect(Collectors.toList());
        List<CompanyVO> companyVOList = new ArrayList<>();
        for (AdminDetail detail : list) {
            CompanyVO companyVO = new CompanyVO();
            BeanUtils.copyProperties(detail, companyVO);
            Admin admin = adminDao.selectByPrimaryKey(detail.getAdminId());
            if (admin == null) {
                continue;
            }
            companyVO.setUsername(admin.getUsername());
            companyVO.setTenantId(admin.getTenantId());
            companyVO.setTenantName(admin.getUsername());
            companyVO.setReviewStatus(admin.getReviewStatus());
            companyVO.setAdminId(admin.getId());
            companyVO.setCreateTime(admin.getCreateTime());
            companyVO.setRoomNumber(StringUtils.isNotEmpty(detail.getRoomNumber()) ? Arrays.stream(detail.getRoomNumber().split(",")).collect(Collectors.toList()) : null);
            companyVOList.add(companyVO);
        }

        return RestResponse.ok(companyVOList);
    }

    @Override
    @Transactional
    public RestResult<Boolean> companyCommit(CompanyCommitDTO companyCommitDTO) {
        handleCompanyCommit(companyCommitDTO, Boolean.FALSE);
        return RestResponse.ok(true);
    }

    private void handleCompanyCommit(CompanyCommitDTO companyCommitDTO, Boolean isAlipay) {
        Admin admin = adminDao.selectByPrimaryKey(companyCommitDTO.getTenantId());
        if (admin == null || admin.getType() != 5) {
            throw new ServiceException("楼宇账号信息有误");
        }
        Admin a = new Admin();
        a.setUsername(companyCommitDTO.getUsername());
        a = adminDao.selectOne(a);
        if (a != null) {
            throw new ServiceException("账号名已经被注册");
        }


        admin.setUsername(companyCommitDTO.getUsername());
        admin.setPassword(PasswordUtils.buildPw(companyCommitDTO.getPassword()));
        admin.setType(AdminTypeEnum.ENTERPRISE.getCode());
        admin.setPlaceType(BUILDING_PLACE.getCode());
        admin.setTenantId(admin.getId());
        admin.setNickname(companyCommitDTO.getActualName());
        admin.setPhone(companyCommitDTO.getActualPhone());
        admin.setReviewStatus(0);
        admin.setContent(admin.getContent() == null ? "" : admin.getContent());
        if (companyCommitDTO.getAdminId() == null) {
            admin.setId(null);
            adminDao.insertSelective(admin);
            AdminDetail detail = new AdminDetail();
            BeanUtils.copyProperties(companyCommitDTO, detail);
            StringJoiner joiner = new StringJoiner(",", "", "");
            companyCommitDTO.getRoomNumber().forEach(joiner::add);
            detail.setRoomNumber(joiner.toString());
            detail.setInformationInputType(0);
            detail.setAdminId(admin.getId());
            adminDetailDao.insertSelective(detail);
        } else {
            admin.setId(companyCommitDTO.getAdminId());
            adminDao.updateByPrimaryKeySelective(admin);
            AdminDetail detail = new AdminDetail();
            detail.setAdminId(admin.getId());
            detail = adminDetailDao.selectOne(detail);
            if (detail == null) {
                throw new ServiceException("数据有误");
            }
            BeanUtils.copyProperties(companyCommitDTO, detail);
            StringJoiner joiner = new StringJoiner(",", "", "");
            companyCommitDTO.getRoomNumber().forEach(joiner::add);
            detail.setRoomNumber(joiner.toString());
            detail.setInformationInputType(0);
            detail.setAdminId(admin.getId());
            adminDetailDao.updateByPrimaryKeySelective(detail);
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(companyCommitDTO.getUserId());
        if (userInfo == null) {
            throw new ServiceException("请先实名认证");
        }
        if (isAlipay) {
            alipayOpenApiService.subscribeAlipayMessage(null, userInfo.getId(), WXMessageTemplateEnum.COMPANY_COMMIT, admin.getUsername(), "待审核", "您的申请已提交", null, null);
        } else {
            smsService.subscribeMessage(null, userInfo.getId(), WXMessageTemplateEnum.COMPANY_COMMIT, admin.getUsername(), "待审核", "您的申请已提交", null, null);
        }

    }

    @Override
    public RestResult<List<CompanyVO>> companyCommitInfoList(Integer userId) {
        List<AdminDetail> list = adminDetailDao.select(new AdminDetail().setUserId(userId));
        list = list.stream().sorted(Comparator.comparing(AdminDetail::getId).reversed()).collect(Collectors.toList());
        List<CompanyVO> companyVOList = new ArrayList<>();
        for (AdminDetail detail : list) {
            CompanyVO companyVO = new CompanyVO();
            BeanUtils.copyProperties(detail, companyVO);
            Admin admin = adminDao.selectByPrimaryKey(detail.getAdminId());
            if (admin == null) {
                continue;
            }
            companyVO.setUsername(admin.getUsername());
            companyVO.setTenantId(admin.getTenantId());
            companyVO.setTenantName(admin.getUsername());
            companyVO.setReviewStatus(admin.getReviewStatus());
            companyVO.setAdminId(admin.getId());
            companyVO.setCreateTime(admin.getCreateTime());
            companyVO.setRoomNumber(StringUtils.isNotEmpty(detail.getRoomNumber()) ? Arrays.stream(detail.getRoomNumber().split(",")).collect(Collectors.toList()) : null);
            companyVOList.add(companyVO);
        }

        return RestResponse.ok(companyVOList);
    }

    @Override
    public RestResult<List<FaceDeviceVO>> userWhiteSnList(Integer adminId) {
        Admin admin = adminDao.selectByPrimaryKey(adminId);
        if (admin == null || admin.getType() == null) {
            return RestResponse.error("数据有误");
        }
        List<FaceDevice> list = faceDeviceDao.select(new FaceDevice().setAdminId(adminId));
        if (admin.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            List<FaceDevice> list1 = faceDeviceDao.select(new FaceDevice().setAdminId(admin.getTenantId()));
            if (list == null || list.size() == 0) {
                list = list1;
            } else {
                list.addAll(list1);
            }
        }

        return RestResponse.ok(list.stream().map(p -> new FaceDeviceVO().setSn(p.getSn()).setName(p.getName())
        ).collect(Collectors.toList()));
    }

    @Override
    public List<BuildingLessVO> getCompanyList(Integer adminId) {
        Admin admin = new Admin();
        admin.setTenantId(adminId);
        List<Admin> adminList = adminDao.select(admin);
        return adminList.stream().map(p -> {
            BuildingLessVO buildingLessVO = new BuildingLessVO();
            buildingLessVO.setAdminId(p.getId());
            buildingLessVO.setName(p.getUsername());
            buildingLessVO.setIsCanSearch(p.getIsCanSearch());
            return buildingLessVO;
        }).collect(Collectors.toList());
    }

    @Override
    public RestResult<AdminLessVO> getBuildingAndCompanyInfo(Integer adminId) {
        Admin admin = adminDao.selectByPrimaryKey(adminId);
        ClassUtil.checkData(admin, "账号不存在");
        ClassUtil.throwBusinessException(!AdminTypeEnum.ENTERPRISE.getCode().equals(admin.getType()), "该账号不是企业账号");
        if (admin.getTenantId() != null) {
            Admin building = adminDao.selectByPrimaryKey(admin.getTenantId());
            if (building != null) {
                AdminLessVO adminLessVO = new AdminLessVO();
                adminLessVO.setVisiteeBuilding(building.getUsername());
                adminLessVO.setVisiteeBuildingId(building.getId());
                adminLessVO.setVisiteeCompany(admin.getUsername());
                adminLessVO.setVisiteeCompanyId(admin.getId());
                adminLessVO.setThreeCodeList(JSON.parseArray(admin.getThreeCode(), String.class));
                return RestResponse.ok(adminLessVO);
            }
        }
        return RestResponse.error("没有找到相关数据");
    }

    @Override
    public RestResult<Boolean> checkUserWhiteListId(Integer userWhiteListId) {
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(userWhiteListId);
        if (userWhitelist == null) {
            return RestResponse.error("常客不存在");
        } else {
            return RestResponse.ok(true);
        }
    }

    @Override
    public RestResult<Boolean> visitoryDownFaceAgain(Integer userVisitorListDetailId) {
        UserVisitorListDetail userVisitorListDetail = userVisitorListDetailDao.selectByPrimaryKey(userVisitorListDetailId);
        if (userVisitorListDetail == null || userVisitorListDetail.getUserId() == null) {
            return RestResponse.error("该条记录不存在");
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userVisitorListDetail.getUserId());
        if (StringUtils.isNotEmpty(userVisitorListDetail.getSn())) {
            List<String> snList = Arrays.stream(userVisitorListDetail.getSn().split(",")).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(snList)) {
                for (String sn : snList) {
                    DownFaceNew downFaceNew = new DownFaceNew();
                    downFaceNew.setCardid(userInfo.getIdNumber());
                    downFaceNew.setRightno(UserRightNoEnum.VISITORS.getCode());
                    downFaceNew.setPhoto(userInfo.getAvatar());
                    downFaceNew.setName(userInfo.getUsername());
                    downFaceNew.setUserId(userInfo.getId());
                    downFaceNew.setSn(sn);
                    downFaceNew.setPhone(userInfo.getPhone());
                    downFaceNew.setValidPeriodStart(userVisitorListDetail.getStartTime());
                    downFaceNew.setValidPeriodEnd(userVisitorListDetail.getEndTime());
                    downFaceNew.setFacialFeature(userInfo.getFacialFeature());
                    downFaceNew.setAdminId(userVisitorListDetail.getAdminId());
                    faceTBService.downUser(downFaceNew);
                }
            }
        }
        return RestResponse.ok(true);
    }

    @Override
    public RestResult<AdminVO> getThreeCodeByAdminId(Integer adminId) {
        AdminVO info = adminService.getInfo(adminId);
        if (info == null) {
            return RestResponse.error("数据为空");
        } else {
            return RestResponse.ok(info);
        }
    }

    /**
     * 获取访客QrcodeId
     *
     * @param visitorQrcodeDto VisitorQrcodeDto
     * @return QrcodeId
     */
    @Override
    public VisitorQrcodeIdDto getVisitorQrcodeId(VisitorQrcodeDto visitorQrcodeDto) {
        Date currTime = new Date();
        Example example = new Example(UserVisitorListDetail.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("userId", visitorQrcodeDto.getUserId());
        criteria.andLessThanOrEqualTo("startTime", currTime);
        criteria.andGreaterThanOrEqualTo("endTime", currTime);
        criteria.andEqualTo("reviewStatus", 2);
        List<UserVisitorListDetail> userVisitorListDetailList = userVisitorListDetailDao.selectByExample(example);
        if (ObjectUtil.isEmpty(userVisitorListDetailList)) {
            throw new ServiceException("当前时间无访问记录");
        }
        UserVisitorListDetail userVisitorListDetail = userVisitorListDetailList.get(userVisitorListDetailList.size() - 1);
        VisitorUserInfoDto visitorUserInfoDto = new VisitorUserInfoDto();
        visitorUserInfoDto.setTaskId(String.valueOf(SnowflakeIdWorker.getSnowId()));
        visitorUserInfoDto.setPhone(userVisitorListDetail.getPhone());
        visitorUserInfoDto.setIdNumber(userVisitorListDetail.getIdNumber());
        visitorUserInfoDto.setUsername(userVisitorListDetail.getName());
        visitorUserInfoDto.setAvatar(userVisitorListDetail.getPhoto());
        visitorUserInfoDto.setCreateTime(userVisitorListDetail.getCreateTime());
        visitorUserInfoDto.setWorkPlace(userVisitorListDetail.getVisiteeCompany());
        visitorUserInfoDto.setCarNumber(userVisitorListDetail.getCarno());
        String data = HttpUtil.post(visitorRegisterUrl, JSON.toJSONString(visitorUserInfoDto));
        RestResult<Long> result = JSON.parseObject(data, new TypeReference<RestResult<Long>>() {
        });
        if (RestCode.SUCCESS.getCode() != result.getCode()) {
            throw new ServiceException("访客注册查询服务调用失败");
        }
        VisitorQrcodeIdDto visitorQrcodeIdDto = new VisitorQrcodeIdDto();
        visitorQrcodeIdDto.setQrcodeId(result.getData());
        visitorQrcodeIdDto.setTaskId(visitorUserInfoDto.getTaskId());
        return visitorQrcodeIdDto;
    }

    /******************以下为支付宝小程序********************/
    @Override
    public RestResult<List<UserWhiteInviteVO>> alipayUserWhiteInviteUserVisitor(UserWhiteInviteDTO userWhiteInviteDTO) {
        return RestResponse.ok(handleUserWhiteInviteUserVisitor(userWhiteInviteDTO, Boolean.TRUE));
    }

    @Override
    public RestResult<Boolean> alipayVisitorySubmit(VisitorySubmitDTO visitorySubmitDTO) {
        handleVisitorySubmit(visitorySubmitDTO, Boolean.TRUE);
        return RestResult.ok(true);
    }

    @Override
    public RestResult<Boolean> alipayUserWhiteHandleVisitList(UserWhiteHandleDTO userWhiteHandleDTO) {
        handleUserWhiteHandleVisitList(userWhiteHandleDTO, Boolean.TRUE);
        return RestResult.ok(true);
    }

    @Override
    public RestResult<Boolean> alipayCompanyCommit(CompanyCommitDTO companyCommitDTO) {
        handleCompanyCommit(companyCommitDTO, Boolean.TRUE);
        return RestResult.ok(true);
    }


    public static void main(String[] args) {
        System.out.println(SnowflakeIdWorker.getSnowId());
    }

}
