package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.Sm4Util;
import com.yx.face.dao.DragonAuthDao;
import com.yx.face.model.dto.dragon.AllTypePersonVO;
import com.yx.face.model.dto.dragon.PdKeyPersonVO;
import com.yx.face.model.dto.dragon.PdPersonBasicVO;
import com.yx.face.model.dto.dragon.PdProvincePersonVO;
import com.yx.face.model.dto.dragon.RequestMsgEncryption;
import com.yx.face.model.dto.dragon.RequestMsgVO;
import com.yx.face.model.dto.dragon.ResponseMsgVO;
import com.yx.face.model.entity.DragonAuth;
import com.yx.face.service.DragonService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.TimeUnit;

/**
 * @author shisl
 * @package com.yx.face.service.impl
 * @class DragonServiceImpl
 * @date 2023/11/6 下午1:17
 * @description
 */
@Slf4j
@Service
public class DragonServiceImpl implements DragonService {

    @Resource
    private DragonAuthDao dragonAuthDao;

    @Value("${dragon.domain}")
    public String domain;
    @Value("${dragon.queryUserInfoUrl}")
    public String queryUserInfoUrl;

    @Value("${dragon.syncQueryUserInfoUrl}")
    public String syncQueryUserInfoUrl;
    @Value("${dragon.asyncQueryUserInfoUrl}")
    public String asyncQueryUserInfoUrl;


    private static final String LIMIT_REQUEST_NUMBER = "dragon:limit:appid:%s";

    @Autowired
    private RedisTemplate redisTemplate;

    @Resource
    private Executor executor;

    @Override
    public RestResult queryUserInfo(RequestMsgEncryption req) {
        log.info("--->巨龙接口原始入参:{}", JSONObject.toJSONString(req));
        if (Preconditions.isBlank(req)
                || Preconditions.isBlank(req.getBizContent())
                || Preconditions.isBlank(req.getAppId())
                || Preconditions.isBlank(req.getReqTimestamp())
                || Preconditions.isBlank(req.getSign())) {
            throw new CommonException("参数错误");
        }

        DragonAuth auth = dragonAuthDao.selectOneByExample(Example.builder(DragonAuth.class)
                .where(WeekendSqls.<DragonAuth>custom()
                        .andEqualTo(DragonAuth::getAppId, req.getAppId())
                        .andEqualTo(DragonAuth::getIsDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(auth)
                || (Preconditions.isNotBlank(auth.getSearchSwitchGongan())
                && !auth.getSearchSwitchGongan()
                && Preconditions.isNotBlank(auth.getSearchSwitchProvince())
                && !auth.getSearchSwitchProvince()
                && Preconditions.isNotBlank(auth.getBkSwitch())
                && !auth.getBkSwitch())
        ) {
            throw new CommonException("无权调用，请联系相关人员");
        }

        //校验调用次数和频次
        checkReqTimes(auth);

        String appKey = auth.getAppKey();
        String appSecret = auth.getAppSecret();
        String privateKey = auth.getPrivateKey();
        //验证签名
        Boolean checkSign = Sm4Util.checkSign(appKey, appSecret, req.getSign(), String.valueOf(req.getReqTimestamp()));
        if (!checkSign) {
            throw new CommonException("签名错误");
        }
        //内容解密
        String decrypt = Sm4Util.decrypt(privateKey, req.getBizContent());
        log.info("--->查询用户信息入参内容解密：{}", decrypt);
        if (Preconditions.isBlank(decrypt)) {
            throw new CommonException("内容解密失败，请检查后重新提交");
        }
        //数据转换
        RequestMsgVO vo = JSONObject.parseObject(decrypt, RequestMsgVO.class);
        if (!checkBizContent(vo)) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        RequestMsgVO reqDto = new RequestMsgVO();
        reqDto.setNumber(vo.getSfzh());
        RestResult<PdPersonBasicVO> result = null;
        //省常口开关是否开启 默认是0:关闭状态  先查询省接口用户基本信息
        if (Preconditions.isNotBlank(auth.getSearchSwitchProvince()) && auth.getSearchSwitchProvince()) {
            result = handleProvinceUser(reqDto);
        }

        //查询省常口用户基本信息为空--->>> 查询公安部用户基本信息
        if ((Preconditions.isBlank(result) || Preconditions.isBlank(result.getData()))
                && Preconditions.isNotBlank(auth.getSearchSwitchGongan()) && auth.getSearchSwitchGongan()) {
            result = handleGongAnUser(reqDto);
        }

        PdPersonBasicVO data = null;
        if (Preconditions.isBlank(result)
                || Preconditions.isBlank(result.getData())
                || (Preconditions.isNotBlank(result.getData().getQfjg())
                && "2".equals(result.getData().getQfjg()))) {
            data = new PdPersonBasicVO();
            data.setSfzh(vo.getSfzh());
        } else {
            data = result.getData();
        }

        //是否开启布控
        if (Preconditions.isNotBlank(auth.getBkSwitch()) && auth.getBkSwitch()) {
            //查询布控信息
            RestResult<PdKeyPersonVO> bk = handleBkUser(reqDto);
            if (Preconditions.isNotBlank(bk)
                    && Preconditions.isNotBlank(bk.getData())
                    && Preconditions.isNotBlank(bk.getData().getBkResult())
                    && "1".equals(bk.getData().getBkResult())) {
                data.setBkUser(Boolean.TRUE);
            }
        }
        //是否开启地址
        if (Preconditions.isBlank(auth.getDzSwitch()) || !auth.getDzSwitch()) {
            data.setDjzz(null);
            data.setHkszd(null);
            data.setJggj(null);
            data.setJgssx(null);
        }
        //是否开启照片
        if (Preconditions.isBlank(auth.getXpSwitch()) || !auth.getXpSwitch()) {
            data.setXp(null);
        }
        //签名+加密 组装返回参数
        long currentTimeMillis = System.currentTimeMillis();
        String sign = Sm4Util.getSign(appKey, appSecret, String.valueOf(currentTimeMillis));
        String encrypt = Sm4Util.encrypt(privateKey, JSONObject.toJSONString(data));
        ResponseMsgVO response = new ResponseMsgVO();
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(encrypt);
        response.setSign(sign);
        return RestResponse.ok(response);
    }

    //校验调用次数和频次
    private void checkReqTimes(DragonAuth auth) {
        //总次数是否调用完
        if (Preconditions.isNotBlank(auth.getLimitTimes()) && auth.getLimitTimes().intValue() == 0) {
            throw new CommonException("API调用次数已用完，请联系管理员");
        }

        //修改总次数
        if (Preconditions.isNotBlank(auth.getLimitTimes()) && auth.getLimitTimes().intValue() > 0) {
            auth.setLimitTimes(auth.getLimitTimes() - 1 > 0 ? auth.getLimitTimes() - 1 : 0);
        }
        auth.setLatestReqTime(new Date());
        auth.setHadReqTimes(auth.getHadReqTimes() + 1);
        dragonAuthDao.updateByPrimaryKey(auth);

        //校验频次单位时间内可调用次数
        if (Preconditions.isNotBlank(auth.getLimitFrequency())
                && auth.getLimitFrequency().intValue() != 0
                && Preconditions.isNotBlank(auth.getLimitFrequencyTime())
                && auth.getLimitFrequencyTime().intValue() != 0) {
            String cacheKey = String.format(LIMIT_REQUEST_NUMBER, auth.getAppId());
            if (!redisTemplate.hasKey(cacheKey) || redisTemplate.getExpire(cacheKey) <= 0) {
                //设备限制单位时间访问次数
                redisTemplate.opsForValue().set(cacheKey, 1, auth.getLimitFrequencyTime(), TimeUnit.SECONDS);
            } else {
                redisTemplate.opsForValue().increment(cacheKey, 1);
                Object timesStr = redisTemplate.opsForValue().get(cacheKey);
                if (Integer.valueOf(timesStr.toString()) > auth.getLimitFrequency()) {
                    throw new CommonException("每秒请求次数限制，请联系管理员");
                }
            }
        }

    }

    @Override
    public RestResult queryBk(RequestMsgEncryption req) {

        DragonAuth auth = dragonAuthDao.selectOneByExample(Example.builder(DragonAuth.class)
                .where(WeekendSqls.<DragonAuth>custom()
                        .andEqualTo(DragonAuth::getAppId, req.getAppId())
                        .andEqualTo(DragonAuth::getIsDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(auth) || !auth.getBkSwitch()) {
            throw new CommonException("无权调用，请联系相关人员");
        }

        String appKey = auth.getAppKey();
        String appSecret = auth.getAppSecret();
        String privateKey = auth.getPrivateKey();
        //验证签名
        Boolean checkSign = Sm4Util.checkSign(appKey, appSecret, req.getSign(), String.valueOf(req.getReqTimestamp()));
        if (!checkSign) {
            throw new CommonException("签名错误");
        }

        String decrypt = Sm4Util.decrypt(privateKey, req.getBizContent());
        log.info("--->布控入参内容解密：{}", decrypt);
        if (Preconditions.isBlank(decrypt)) {
            throw new CommonException("参数有误，请检查后重新提交");
        }

        RequestMsgVO vo = JSONObject.parseObject(decrypt, RequestMsgVO.class);
        if (!checkBizContent(vo)) {
            throw new CommonException("参数有误，请检查后重新提交");
        }

        RequestMsgVO reqDto = new RequestMsgVO();
        //查询用户布控信息
        reqDto.setNumber(vo.getNumber());
        RestResult<PdKeyPersonVO> result = handleBkUser(reqDto);
        if (Preconditions.isBlank(result) || Preconditions.isBlank(result.getData())) {
            return RestResponse.error("未查询到用户布控信息");
        }
        long currentTimeMillis = System.currentTimeMillis();
        String sign = Sm4Util.getSign(appKey, appSecret, String.valueOf(currentTimeMillis));
        String encrypt = Sm4Util.encrypt(privateKey, JSONObject.toJSONString(result.getData()));
        ResponseMsgVO response = new ResponseMsgVO();
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(encrypt);
        response.setSign(sign);
        return RestResponse.ok(response);
    }

    private RestResult<PdKeyPersonVO> handleBkUser(RequestMsgVO reqDto) {
        try {
            reqDto.setType(5);
            String respStr = HttpUtil.post(domain + queryUserInfoUrl, JSONObject.toJSONString(reqDto), 60000);
            log.info("--->查询用户布控信息响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            if (Preconditions.isBlank(result) || Preconditions.isBlank(result.getData())) {
                return RestResponse.ok(null);
            }
            AllTypePersonVO all = JSONObject.parseObject(result.getData().toString(), AllTypePersonVO.class);
            if (Preconditions.isBlank(all.getBasic())) {
                return RestResponse.ok(null);
            }
            List<PdKeyPersonVO> bk = all.getKey();
            PdKeyPersonVO bkVo = bk.get(0);
            return RestResponse.ok(bkVo);
        } catch (Exception e) {
            log.error("--->查询用户布控信息异常.", e);
        }
        return RestResponse.ok(null);
    }

    private RestResult<PdPersonBasicVO> handleProvinceUser(RequestMsgVO reqDto) {
        try {
            reqDto.setType(2);
            String respStr = HttpUtil.post(domain + queryUserInfoUrl, JSONObject.toJSONString(reqDto), 60000);
            log.info("--->查询省常口用户基本信息响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            if (Preconditions.isBlank(result) || Preconditions.isBlank(result.getData())) {
                return RestResponse.ok(null);
            }
            AllTypePersonVO all = JSONObject.parseObject(result.getData().toString(), AllTypePersonVO.class);
            if (Preconditions.isBlank(all.getBasic())) {
                return RestResponse.ok(null);
            }
            List<PdProvincePersonVO> province = all.getProvince();
            PdProvincePersonVO provinceVO = province.get(0);
            PdPersonBasicVO pdPersonBasicVO = BeanUtil.copyProperties(provinceVO, PdPersonBasicVO.class);
            return RestResponse.ok(pdPersonBasicVO);
        } catch (Exception e) {
            log.error("--->查询省常口用户基本信息异常.", e);
        }
        return RestResponse.ok(null);
    }

    private RestResult<PdPersonBasicVO> handleGongAnUser(RequestMsgVO reqDto) {
        reqDto.setType(1);
        try {
            String respStr = HttpUtil.post(domain + queryUserInfoUrl, JSONObject.toJSONString(reqDto), 60000);
            log.info("--->查询公安部用户基本信息响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            if (Preconditions.isBlank(result) || Preconditions.isBlank(result.getData())) {
                return RestResponse.ok(null);
            }
            AllTypePersonVO all = JSONObject.parseObject(result.getData().toString(), AllTypePersonVO.class);
            if (Preconditions.isBlank(all.getBasic())) {
                return RestResponse.ok(null);
            }
            List<PdPersonBasicVO> basic = all.getBasic();
            PdPersonBasicVO pdPersonBasicVO = basic.get(0);
            return RestResponse.ok(pdPersonBasicVO);
        } catch (Exception e) {
            log.error("--->查询公安部用户基本信息异常.", e);
        }
        return RestResponse.ok(null);
    }

    private boolean checkBizContent(RequestMsgVO vo) {
        if (Preconditions.isBlank(vo)
                || Preconditions.isBlank(vo.getSfzh())) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }


    @Override
    public RestResult<PdPersonBasicVO> testQueryUserInfo(RequestMsgVO vo) {
        log.info("--->测试巨龙接口原始入参:{}", JSONObject.toJSONString(vo));
        RequestMsgVO reqDto = new RequestMsgVO();
        //查询省常口用户基本信息
        reqDto.setNumber(vo.getNumber());
        RestResult<PdPersonBasicVO> result = handleProvinceUser(reqDto);
        //查询省常口用户基本信息为空--->>> 查询公安部用户基本信息
        if (Preconditions.isBlank(result) || Preconditions.isBlank(result.getData())) {
            result = handleGongAnUser(reqDto);
        }

        if (Preconditions.isBlank(result) || Preconditions.isBlank(result.getData())) {
            return RestResponse.error("未查询到用户相关信息");
        }
        PdPersonBasicVO data = result.getData();
        //查询布控信息
        RestResult<PdKeyPersonVO> bk = handleBkUser(reqDto);
        if (Preconditions.isNotBlank(bk)
                && Preconditions.isNotBlank(bk.getData())) {
            data.setBkUser(Boolean.TRUE);
        }
        return RestResponse.ok(data);
    }

    @Override
    public RestResult asyncQueryUserInfo(RequestMsgEncryption req) {
        log.info("--->(异步)巨龙接口原始入参:{}", JSONObject.toJSONString(req));
        if (Preconditions.isBlank(req)
                || Preconditions.isBlank(req.getBizContent())
                || Preconditions.isBlank(req.getAppId())
                || Preconditions.isBlank(req.getReqTimestamp())
                || Preconditions.isBlank(req.getSign())
                || Preconditions.isBlank(req.getCallbackUrl())) {
            throw new CommonException("参数错误");
        }

        DragonAuth auth = dragonAuthDao.selectOneByExample(Example.builder(DragonAuth.class)
                .where(WeekendSqls.<DragonAuth>custom()
                        .andEqualTo(DragonAuth::getAppId, req.getAppId())
                        .andEqualTo(DragonAuth::getIsDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(auth)
                || (Preconditions.isNotBlank(auth.getSearchSwitchGongan())
                && !auth.getSearchSwitchGongan()
                && Preconditions.isNotBlank(auth.getSearchSwitchProvince())
                && !auth.getSearchSwitchProvince()
                && Preconditions.isNotBlank(auth.getBkSwitch())
                && !auth.getBkSwitch())
        ) {
            throw new CommonException("无权调用，请联系相关人员");
        }

        //校验调用次数和频次
        checkReqTimes(auth);

        String appKey = auth.getAppKey();
        String appSecret = auth.getAppSecret();
        String privateKey = auth.getPrivateKey();
        //验证签名
        Boolean checkSign = Sm4Util.checkSign(appKey, appSecret, req.getSign(), String.valueOf(req.getReqTimestamp()));
        if (!checkSign) {
            throw new CommonException("签名错误");
        }
        //内容解密
        String decrypt = Sm4Util.decrypt(privateKey, req.getBizContent());
        log.info("--->查询用户信息入参内容解密：{}", decrypt);
        if (Preconditions.isBlank(decrypt)) {
            throw new CommonException("内容解密失败，请检查后重新提交");
        }
        //数据转换
        RequestMsgVO vo = JSONObject.parseObject(decrypt, RequestMsgVO.class);
        if (!checkBizContent(vo)) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        //异步推送
        executor.execute(() -> {
            String callbackUrl = req.getCallbackUrl();
            RequestMsgVO reqDto = new RequestMsgVO();
            reqDto.setNumber(vo.getSfzh());
            RestResult<PdPersonBasicVO> result = null;
            //省常口开关是否开启 默认是0:关闭状态  先查询省接口用户基本信息
            if (Preconditions.isNotBlank(auth.getSearchSwitchProvince()) && auth.getSearchSwitchProvince()) {
                result = handleProvinceUser(reqDto);
            }

            //查询省常口用户基本信息为空--->>> 查询公安部用户基本信息
            if ((Preconditions.isBlank(result) || Preconditions.isBlank(result.getData()))
                    && Preconditions.isNotBlank(auth.getSearchSwitchGongan()) && auth.getSearchSwitchGongan()) {
                result = handleGongAnUser(reqDto);
            }

            PdPersonBasicVO data = null;
            if (Preconditions.isBlank(result)
                    || Preconditions.isBlank(result.getData())
                    || (Preconditions.isNotBlank(result.getData().getQfjg())
                    && "2".equals(result.getData().getQfjg()))) {
                data = new PdPersonBasicVO();
                data.setSfzh(vo.getSfzh());
            } else {
                data = result.getData();
            }
            //是否开启布控
            if (Preconditions.isNotBlank(auth.getBkSwitch()) && auth.getBkSwitch()) {
                //查询布控信息
                RestResult<PdKeyPersonVO> bk = handleBkUser(reqDto);
                if (Preconditions.isNotBlank(bk)
                        && Preconditions.isNotBlank(bk.getData())
                        && Preconditions.isNotBlank(bk.getData().getBkResult())
                        && "1".equals(bk.getData().getBkResult())) {
                    data.setBkUser(Boolean.TRUE);
                }
            }
            //是否开启地址
            if (Preconditions.isBlank(auth.getDzSwitch()) || !auth.getDzSwitch()) {
                data.setDjzz(null);
                data.setHkszd(null);
                data.setJggj(null);
                data.setJgssx(null);
            }
            //是否开启照片
            if (Preconditions.isBlank(auth.getXpSwitch()) || !auth.getXpSwitch()) {
                data.setXp(null);
            }
            //签名+加密 组装返回参数
            long currentTimeMillis = System.currentTimeMillis();
            String sign = Sm4Util.getSign(appKey, appSecret, String.valueOf(currentTimeMillis));
            String encrypt = Sm4Util.encrypt(privateKey, JSONObject.toJSONString(data));
            ResponseMsgVO response = new ResponseMsgVO();
            response.setReqTimestamp(currentTimeMillis);
            response.setBizContent(encrypt);
            response.setSign(sign);
            RestResult<ResponseMsgVO> ok = RestResponse.ok(response);
            log.info("--->自助办证用户信息查询数据推送:{}", JSONObject.toJSONString(ok));
            HttpUtil.post(callbackUrl, JSONObject.toJSONString(ok), 10000);
        });
        return RestResponse.ok();
    }

    @Override
    public RestResult queryUserInfoV2(RequestMsgEncryption req) {
        try {
            log.info("--->巨龙接口原始入参:{}", JSONObject.toJSONString(req));
            String respStr = HttpUtil.post(domain + syncQueryUserInfoUrl, JSONObject.toJSONString(req), 60000);
            log.info("--->巨龙接口响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return RestResponse.ok(result);
        } catch (Exception e) {
            log.error("--->巨龙接口异常.", e);
        }
        return RestResponse.error("未查询到此用户信息");
    }

    @Override
    public RestResult asyncQueryUserInfoV2(RequestMsgEncryption req) {
        try {
            log.info("--->(异步)巨龙接口原始入参:{}", JSONObject.toJSONString(req));
            String respStr = HttpUtil.post(domain + asyncQueryUserInfoUrl, JSONObject.toJSONString(req), 60000);
            log.info("--->(异步)巨龙接口响应参数：{}", respStr);
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return RestResponse.ok(result);
        } catch (Exception e) {
            log.error("--->巨龙接口异常.", e);
        }
        return RestResponse.error("未查询到此用户信息");
    }

}
