package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.ExamUserWhitelistDao;
import com.yx.face.model.dto.ExamUserWhitelistDTO;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.excel.ExamUserWhitelistExcel;
import com.yx.face.model.excel.ExamUserWhitelistOutExcel;
import com.yx.face.model.search.ExamUserWhitelistSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.ExamUserWhitelistExcelVO;
import com.yx.face.model.vo.ExamUserWhitelistVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.ExamUserWhitelistService;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 党员考评 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2023-04-11
 */
@Service
public class ExamUserWhitelistServiceImpl implements ExamUserWhitelistService {

    @Resource
    private ExamUserWhitelistDao examUserWhitelistDao;
    @Resource
    private AdminService adminService;
    @Resource
    private AdminDao adminDao;

    @Override
    public RestResult<Boolean> insert(ExamUserWhitelistDTO examUserWhitelistDTO) {
        AdminVO info = null;
        if (examUserWhitelistDTO.getAdminId() == null) {
            info = adminService.getInfo();
            examUserWhitelistDTO.setAdminId(info.getAdminId());
        }else {
            info = adminService.getInfo(examUserWhitelistDTO.getAdminId());
        }
        if (!info.getType().equals(AdminTypeEnum.DISTRICT.getCode()) && info.getPlaceType().equals(AdminPlaceTypeEnum.COMMON_PLACE.getCode())) {
            throw new ServiceException("只能添加到通用场景的区级账号");
        }
        if(examUserWhitelistDTO.getEndTime().getTime() < examUserWhitelistDTO.getStartTime().getTime()){
            throw new ServiceException("结束时间不能早于开始时间");
        }
        if(!IdcardUtil.isValidCard(examUserWhitelistDTO.getIdNumber())){
            throw new ServiceException("身份证有误,请核对");
        }
        ExamUserWhitelist examUserWhitelist = BeanUtil.copyProperties(examUserWhitelistDTO,ExamUserWhitelist.class);
        examUserWhitelist.setCreateTime(new Date());
        examUserWhitelistDao.insertSelective(examUserWhitelist);
        return RestResult.ok(Boolean.TRUE);
    }

    @Override
    public PageInfo<ExamUserWhitelistVO> getPageList(RestDTO<ExamUserWhitelistSearch> dto) {
        ExamUserWhitelistSearch data = dto.getData();
        if (StrUtil.isNotBlank(data.getPhone()) && !SMSOrIdCardUtils.isMobile(data.getPhone())) {
            throw new ServiceException("手机号查询时须填写完整");
        }
        if(StrUtil.isNotBlank(data.getIdNumber()) && !IdcardUtil.isValidCard(data.getIdNumber())){
            throw new ServiceException("身份证号查询时须填写完整");
        }
        Map<String, Object> map = new HashMap<>();
        AdminVO info = adminService.getInfo();
        info.setAdminIds(Lists.newArrayList(info.getAdminId()));
        /*这段代码按照道理是不会用到的，保险起仍然放着了*/
        if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())) && AdminTypeEnum.DISTRICT.getCode().equals(info.getType())) {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, info.getAdminId())).build());
            info.getAdminIds().addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
        }
        /*这段代码按照道理是不会用到的，保险起仍然放着了*/
        map.put("adminVo", info);
        map.put("uws", dto.getData());
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<ExamUserWhitelistVO> list = examUserWhitelistDao.selectListBySearch(map);
        //根据账号配置进行数据脱敏
        if (data.getOutType().equals(1)){
            this.pageListDesensitization(list,info);
        }
        return new PageInfo<>(list);
    }
    private void pageListDesensitization(List<ExamUserWhitelistVO> list, AdminVO info) {
        boolean isShowName = false;
        boolean isShowPhone = false;
        boolean isShowNumber = false;
        if (info.getNoHideCloumn() != null && info.getNoHideCloumn().size() > 0) {
            if (info.getNoHideCloumn().contains("1")) {
                isShowName = true;
            }
            if (info.getNoHideCloumn().contains("2")) {
                isShowPhone = true;
            }
            if (info.getNoHideCloumn().contains("3")) {
                isShowNumber = true;
            }
        }
        for (int i = 0; i < list.size(); i++) {
            if (list.get(i).getName() != null && isShowName) {
                list.get(i).setName(SMSOrIdCardUtils.hideName(list.get(i).getName().trim()));
            }
            if (list.get(i).getPhone() != null && isShowPhone) {
                list.get(i).setPhone(SMSOrIdCardUtils.hidePhone(list.get(i).getPhone()));
            }
            if (list.get(i).getIdNumber() != null && isShowNumber) {
                list.get(i).setIdNumber(SMSOrIdCardUtils.hideIdNumber(list.get(i).getIdNumber()));
            }
        }
    }

    @Override
    public Integer delete(Integer examUserWhitelistId) {
        ExamUserWhitelist examUserWhitelist = examUserWhitelistDao.selectByPrimaryKey(examUserWhitelistId);
        if (examUserWhitelist == null || examUserWhitelist.getIsDeleted()) {
            throw new ServiceException("该数据已不存在，请刷新页面再试哦~");
        }
        examUserWhitelist.setIsDeleted(Boolean.TRUE);
        return examUserWhitelistDao.updateByPrimaryKeySelective(examUserWhitelist);
    }

    @Override
    public RestResult<List<ExamUserWhitelistExcelVO>> uploadExcel(List<Object> datas) {
        List<ExamUserWhitelistExcelVO> list = new ArrayList<>();
        AdminVO info = adminService.getInfo();
        for (int i = 0; i < datas.size(); i++) {
            ExamUserWhitelistExcel excel = (ExamUserWhitelistExcel) datas.get(i);
            try {
                ExamUserWhitelistDTO examUserWhitelistDTO = BeanUtil.copyProperties(excel,ExamUserWhitelistDTO.class);
                if (StrUtil.isBlank(excel.getIdNumber())) {
                    throw new ServiceException("第" + (i + 3) + "行,身份证号不能为空");
                }
                if (StrUtil.isBlank(excel.getStartTime())) {
                    throw new ServiceException("第" + (i + 3) + "行,考试开始时间不能为空");
                }
                if (StrUtil.isBlank(excel.getEndTime())) {
                    throw new ServiceException("第" + (i + 3) + "行,考试结束时间不能为空");
                }
                DateTime startTime = DateUtil.parse(excel.getStartTime());
                DateTime endTime = DateUtil.parse(excel.getEndTime());
                if(startTime == null){
                    throw new ServiceException("第" + (i + 3) + "行,考试开始时间格式有误");
                }
                if(endTime == null){
                    throw new ServiceException("第" + (i + 3) + "行,考试结束时间格式有误");
                }
                examUserWhitelistDTO.setStartTime(startTime);
                examUserWhitelistDTO.setEndTime(endTime);
                examUserWhitelistDTO.setAdminId(info.getAdminId());


                try {
                    this.insert(examUserWhitelistDTO);
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new ServiceException("第" + (i + 3) + "行," + e.getMessage());
                }
            } catch (ServiceException e) {
                ExamUserWhitelistExcelVO excelVO = BeanUtil.copyProperties(excel,ExamUserWhitelistExcelVO.class);
                excelVO.setMsg(e.getMessage());
                list.add(excelVO);
            }
        }
        return RestResponse.ok(list);
    }

    @Override
    public void excelOut(ExamUserWhitelistSearch dto, HttpServletResponse response) {
        AdminVO info = adminService.getInfo();
        if (info.getIsOut() != 1) {
            throw new ServiceException("你没有导出的权限");
        }
        RestDTO<ExamUserWhitelistSearch> restDTO = new RestDTO<>();
        restDTO.setPageNum(0);
        restDTO.setPageSize(50000);
        restDTO.setData(dto);
        PageInfo<ExamUserWhitelistVO> pageList = this.getPageList(restDTO);
        if(pageList == null || pageList.getList() == null || CollUtil.isEmpty(pageList.getList())){
            throw new ServiceException("没有可导出的记录");
        }

        List<ExamUserWhitelistVO> records = pageList.getList();
        this.pageListDesensitizationByOut(records,info);
        List<ExamUserWhitelistOutExcel> list = records.stream().map(object -> {
            ExamUserWhitelistOutExcel item = BeanUtil.copyProperties(object, ExamUserWhitelistOutExcel.class);
            item.setStartTime(DateUtil.format(object.getStartTime(), DatePattern.NORM_DATETIME_PATTERN));
            item.setEndTime(DateUtil.format(object.getEndTime(), DatePattern.NORM_DATETIME_PATTERN));
            if(object.getInTime() != null){
                item.setInTime(DateUtil.format(object.getInTime(), DatePattern.NORM_DATETIME_PATTERN));
            }
            String status= "";
            switch (object.getStatus()){
                case 0: status ="未入场";break;
                case 1: status ="已入场";break;
            }
            item.setStatus(status);

            return item;
        }).collect(Collectors.toList());

        // 这里注意 有同学反应使用swagger 会导致各种问题，请直接用浏览器或者用postman
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = null;
        try {
            fileName = URLEncoder.encode("考试人员", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), ExamUserWhitelistOutExcel.class).sheet("人员信息").doWrite(list);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private void pageListDesensitizationByOut(List<ExamUserWhitelistVO> list, AdminVO info) {
        for (int i = 0; i < list.size(); i++) {
            if (info.getIsHideImportant() == 1) {
                if (list.get(i).getPhone() != null) {
                    list.get(i).setPhone(SMSOrIdCardUtils.hidePhone(list.get(i).getPhone()));
                }
                if (list.get(i).getIdNumber() != null) {
                    list.get(i).setIdNumber(SMSOrIdCardUtils.hideIdNumber(list.get(i).getIdNumber()));
                }
                if (list.get(i).getName() != null) {
                    list.get(i).setName(list.get(i).getName().trim());
                }
            }
        }
    }
    @Override
    public RestResult<Boolean> deletedeleteAll() {
        AdminVO info = adminService.getInfo();
        if (!info.getType().equals(AdminTypeEnum.DISTRICT.getCode()) && info.getPlaceType().equals(AdminPlaceTypeEnum.COMMON_PLACE.getCode())) {
            throw new ServiceException("只有通用场景的区级账号可以使用该功能");
        }
        examUserWhitelistDao.updateByAdminId(info.getAdminId());
        return RestResponse.ok(true);
    }

}
