package com.yx.face.service.impl;


import com.github.pagehelper.PageHelper;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FaceDeviceTypeDao;
import com.yx.face.model.dto.FaceDeviceTypeDTO;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.FaceDeviceType;
import com.yx.face.model.enums.FaceDeviceTypeEnum;
import com.yx.face.model.search.FaceDeviceTypeSearch;
import com.yx.face.model.vo.DeviceCodeVO;
import com.yx.face.model.vo.FaceDeviceTypeVO;
import com.yx.face.service.FaceDeviceTypeService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 设备类型
 * </p>*
 */
@Service
public class FaceDeviceTypeServiceImpl implements FaceDeviceTypeService {

    @Resource
    private FaceDeviceTypeDao faceDeviceTypeDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Override
    public Integer addOrEdit(FaceDeviceTypeDTO faceDeviceTypeDTO) {
        FaceDeviceType faceDeviceType = new FaceDeviceType();
        Date date = new Date();
        faceDeviceType.setName(faceDeviceTypeDTO.getName());
        faceDeviceType.setCode(faceDeviceTypeDTO.getCode());
        faceDeviceType.setUpdateTime(date);
        if (faceDeviceTypeDTO.getFaceDeviceTypeId() == null || faceDeviceTypeDTO.getFaceDeviceTypeId() <= 0) {
            faceDeviceType.setCreateTime(date);
            String name = faceDeviceTypeDao.selectName(faceDeviceTypeDTO.getName(), null);
            if (StringUtils.isNotBlank(name)) {
                throw new ServiceException("设备类型名称已存在");
            }
            return faceDeviceTypeDao.insertSelective(faceDeviceType);
        } else {
            faceDeviceType.setId(faceDeviceTypeDTO.getFaceDeviceTypeId());
            faceDeviceType.setName(faceDeviceTypeDTO.getName());
            String name = faceDeviceTypeDao.selectName(faceDeviceTypeDTO.getName(), faceDeviceTypeDTO.getFaceDeviceTypeId());
            if (StringUtils.isNotBlank(name)) {
                throw new ServiceException("设备类型名称已存在");
            }
            return faceDeviceTypeDao.updateByPrimaryKeySelective(faceDeviceType);
        }
    }

    @Override
    public Integer delete(Integer thirdPartyId) {
        FaceDevice faceDevice = new FaceDevice();
        faceDevice.setFaceDeviceTypeId(thirdPartyId);
        int selectCountType = faceDeviceDao.selectCount(faceDevice);
        if (selectCountType>0){
            throw new ServiceException("删除失败该设备类型绑定在其他设备上请先解绑");
        }
        return faceDeviceTypeDao.deleteByPrimaryKey(thirdPartyId);
    }

    @Override
    public List<FaceDeviceTypeVO> getList() {
        List<FaceDeviceTypeVO> list = faceDeviceTypeDao.getList();
        return list;
    }

    @Override
    public List<FaceDeviceTypeVO> getPageList(RestDTO<FaceDeviceTypeSearch> dto) {
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<FaceDeviceTypeVO> list = faceDeviceTypeDao.getPageList(dto.getData());
        return list;
    }

    @Override
    public List<DeviceCodeVO> queryDeviceCode() {
        List<DeviceCodeVO> objectList = Arrays.stream(FaceDeviceTypeEnum.values()).map(v -> {
            DeviceCodeVO deviceCodeVO = new DeviceCodeVO();
            deviceCodeVO.setCode(v.name());
            deviceCodeVO.setDesc(v.getDesc());
            return deviceCodeVO;
        }).collect(Collectors.toList());

        return objectList;
    }

    @Override
    public FaceDeviceType queryById(Integer faceDeviceTypeId) {
        return faceDeviceTypeDao.selectByPrimaryKey(faceDeviceTypeId);
    }
}
