package com.yx.face.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.beust.jcommander.internal.Lists;
import com.beust.jcommander.internal.Sets;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.constant.ExportDocConstant;
import com.yx.face.dao.AdminDataShowDao;
import com.yx.face.dao.FaceLogDao;
import com.yx.face.dao.ShortTermFaceLogDao;
import com.yx.face.model.dto.AdminIDTypeDTO;
import com.yx.face.model.dto.MonthExportDTO;
import com.yx.face.model.dto.TodayExportDTO;
import com.yx.face.model.dto.WeekExportDTO;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.enums.DevicePassTypeEnum;
import com.yx.face.model.enums.HealthCodeStatusEnum;
import com.yx.face.model.vo.AdminDeviceVO;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.ExportWordVO;
import com.yx.face.model.vo.FaceLogVO;
import com.yx.face.model.vo.export.ExportRecordVO;
import com.yx.face.service.AdminDataShowService;
import com.yx.face.service.AdminService;
import com.yx.face.service.FaceLogExportService;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.Writer;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @author lejingjing
 * @version 1.0
 * @Description:
 * @date 2022/11/15
 **/
@Slf4j
@Service
public class FaceLogExportServiceImpl implements FaceLogExportService {

    @Resource
    private AdminService adminService;

    @Resource
    private ShortTermFaceLogDao shortTermFaceLogDao;

    @Resource
    private FreeMarkerConfigurer freeMarkerConfigurer;

    @Resource
    private AdminDataShowDao adminDataShowDao;

    @Resource
    private AdminDataShowService adminDataShowService;

    @Resource
    private FaceLogDao faceLogDao;

    public Template getTemplate(String name) throws IOException {
        return freeMarkerConfigurer.getConfiguration().getTemplate(name);
    }

    @Override
    public TodayExportDTO exportTodayWord(ExportRecordVO exportRecordVO) {
        checkDayParam(exportRecordVO);
        TodayExportDTO todayExportDTO = new TodayExportDTO();
        AdminVO adminInfo = adminService.getInfo();
        List<AdminIDTypeDTO> adminIDTypeDTOListList = getAdminIdList(adminInfo);
        if (CollectionUtil.isEmpty(adminIDTypeDTOListList)) {
            return todayExportDTO;
        }
        String currentDate = DateUtil.format(DateUtil.parse(exportRecordVO.getStartTime()), "yyyy年MM月dd日");
        List<Integer> adminIdList = null;
        if(adminInfo.getType().equals(Byte.valueOf("1"))||adminInfo.getType().equals(Byte.valueOf("2"))){
            adminIdList = null;
        }else{
            adminIdList = adminIDTypeDTOListList.stream().map(AdminIDTypeDTO::getAdminId).collect(Collectors.toList());
        }
        AdminDeviceVO adminDeviceVO = adminDataShowService.getDeviceNumber();
        ExportWordVO totalExportVO = getGloablePassData(adminIDTypeDTOListList, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime(), null);
        ExportWordVO outExportVO = getGloablePassData(adminIDTypeDTOListList, adminIdList,  exportRecordVO.getStartTime(), exportRecordVO.getEndTime(), DevicePassTypeEnum.PASS_OUT);
        ExportWordVO inExportVO = getGloablePassData(adminIDTypeDTOListList, adminIdList,  exportRecordVO.getStartTime(), exportRecordVO.getEndTime(), DevicePassTypeEnum.PASS_IN);
        initTotal(todayExportDTO, adminIdList,  exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initOut(todayExportDTO, adminIdList,  exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initIn(todayExportDTO, adminIdList,  exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initTodayExportDTO(adminInfo, todayExportDTO, currentDate, adminDeviceVO, totalExportVO, outExportVO, inExportVO);
        return todayExportDTO;
    }

    @Override
    public WeekExportDTO exportWeekWord(ExportRecordVO exportRecordVO) {
        checkWeekParam(exportRecordVO);
        WeekExportDTO weekExportDTO = new WeekExportDTO();
        AdminVO adminInfo = adminService.getInfo();
        weekExportDTO.setAdminName(adminInfo.getUsername());
        List<AdminIDTypeDTO> adminIDTypeDTOListList = getAdminIdList(adminInfo);
        if (CollectionUtil.isEmpty(adminIDTypeDTOListList)) {
            return weekExportDTO;
        }
        List<Integer> adminIdList = null;
        if(adminInfo.getType().equals(Byte.valueOf("1"))||adminInfo.getType().equals(Byte.valueOf("2"))){
            adminIdList = null;
        }else{
            adminIdList = adminIDTypeDTOListList.stream().map(AdminIDTypeDTO::getAdminId).collect(Collectors.toList());
        }
        String beginDateOfWeek = DateUtil.format(DateUtil.beginOfWeek(DateUtil.parse(exportRecordVO.getStartTime())), "yyyy年MM月dd日");
        String endDateOfWeek = DateUtil.format(DateUtil.parse(exportRecordVO.getEndTime()), "yyyy年MM月dd日");
        AdminDeviceVO adminDeviceVO = adminDataShowService.getDeviceNumber();
        ExportWordVO totalExportVO = getGloablePassData(adminIDTypeDTOListList, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime(), null);
        ExportWordVO outExportVO = getGloablePassData(adminIDTypeDTOListList, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime(), DevicePassTypeEnum.PASS_OUT);
        ExportWordVO inExportVO = getGloablePassData(adminIDTypeDTOListList, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime(), DevicePassTypeEnum.PASS_IN);
        initWeekDate(weekExportDTO, beginDateOfWeek, endDateOfWeek);
        initWeekTotal(weekExportDTO, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initWeekOut(weekExportDTO, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initWeekIn(weekExportDTO, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initWeekExportDTO(weekExportDTO, adminDeviceVO, totalExportVO, outExportVO, inExportVO);
        return weekExportDTO;
    }

    @Override
    public MonthExportDTO exportMonthWord(ExportRecordVO exportRecordVO) {
        checkMonthParam(exportRecordVO);
        MonthExportDTO monthExportDTO = new MonthExportDTO();
        AdminVO adminInfo = adminService.getInfo();
        monthExportDTO.setAdminName(adminInfo.getUsername());
        List<AdminIDTypeDTO> adminIDTypeDTOListList = getAdminIdList(adminInfo);
        if (CollectionUtil.isEmpty(adminIDTypeDTOListList)) {
            return monthExportDTO;
        }
        AdminDeviceVO adminDeviceVO = adminDataShowService.getDeviceNumber();
        String startMonthStr = DateUtil.format(DateUtil.parse(exportRecordVO.getStartTime()),"yyyy年MM月");
        List<Integer> adminIdList = null;
        if(adminInfo.getType().equals(Byte.valueOf("1"))||adminInfo.getType().equals(Byte.valueOf("2"))){
            adminIdList = null;
        }else{
            adminIdList = adminIDTypeDTOListList.stream().map(AdminIDTypeDTO::getAdminId).collect(Collectors.toList());
        }
        ExportWordVO totalExportVO = getFaceLogGloablePassData(adminIDTypeDTOListList, adminIdList, exportRecordVO.getStartTime(),
                                                             exportRecordVO.getEndTime(), null);
        ExportWordVO outExportVO = getFaceLogGloablePassData(adminIDTypeDTOListList, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime(), DevicePassTypeEnum.PASS_OUT);
        ExportWordVO inExportVO = getFaceLogGloablePassData(adminIDTypeDTOListList, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime(), DevicePassTypeEnum.PASS_IN);
        initMonthDate(monthExportDTO, exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initMonthTotal(monthExportDTO, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initMonthOutTotal(monthExportDTO, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initMonthInTotal(monthExportDTO, adminIdList, exportRecordVO.getStartTime(), exportRecordVO.getEndTime());
        initMonthExportDTO(startMonthStr, monthExportDTO, adminDeviceVO, totalExportVO, outExportVO, inExportVO);
        return monthExportDTO;
    }

    private void checkDayParam(ExportRecordVO exportRecordVO) {
        Date startDateTime = DateUtil.parse(exportRecordVO.getStartTime()).toJdkDate();
        Date endDateTime = DateUtil.parse(exportRecordVO.getEndTime()).toJdkDate();
        if(startDateTime.after(endDateTime)){
            throw new ServiceException("参数错误");
        }
        String startDate = DateUtil.format(DateUtil.parse(exportRecordVO.getStartTime()), "yyyy-MM-dd");
        String endDate = DateUtil.format(DateUtil.parse(exportRecordVO.getEndTime()), "yyyy-MM-dd");
        if(!startDate.equals(endDate)){
            throw new ServiceException("日期不是同一天");
        }
        Date lastMonth = DateUtil.lastMonth().toJdkDate();
        if(endDateTime.before(lastMonth)||startDateTime.after(new Date())){
            throw new ServiceException("日期不能超过本月");
        }
    }

    private void checkWeekParam(ExportRecordVO exportRecordVO) {
        Date startDateTime = DateUtil.parse(exportRecordVO.getStartTime()).toJdkDate();
        Date endDateTime = DateUtil.parse(exportRecordVO.getEndTime()).toJdkDate();
        if(startDateTime.after(endDateTime)){
            throw new ServiceException("参数错误");
        }
        Date endDate = DateUtil.offsetWeek(startDateTime,1).toJdkDate();
        if(endDate.before(endDateTime)){
            throw new ServiceException("最多导出一周内的数据");
        }
        Date lastMonth = DateUtil.lastMonth().toJdkDate();
        if(endDateTime.before(lastMonth)||startDateTime.after(new Date())){
            throw new ServiceException("日期不能超过本月");
        }
    }

    private void checkMonthParam(ExportRecordVO exportRecordVO) {
        Date startDateTime = DateUtil.parse(exportRecordVO.getStartTime()).toJdkDate();
        Date endDateTime = DateUtil.parse(exportRecordVO.getEndTime()).toJdkDate();
        if(startDateTime.after(endDateTime)){
            throw new ServiceException("参数错误");
        }
        Date endDate = DateUtil.offsetMonth(startDateTime,1).toJdkDate();
        if(endDate.before(endDateTime)){
            throw new ServiceException("最多导出一月内的数据");
        }
        Date lastMonth = DateUtil.lastMonth().toJdkDate();
        if(endDateTime.before(lastMonth)||startDateTime.after(new Date())){
            throw new ServiceException("日期不能超过本月");
        }
    }

    private MonthExportDTO initMonthDate(MonthExportDTO monthExportDTO, String beginDateOfMonth, String endDateOfMonth) {
        Date endDate = DateUtil.parseDate(endDateOfMonth);
        Date firstWeekDate = DateUtil.parseDate(beginDateOfMonth);
        Date secondWeekDate = getOffset7Day(firstWeekDate);
        Date thirdWeekDate = getOffset7Day(secondWeekDate);
        Date fourthWeekDate = getOffset7Day(thirdWeekDate);
        Date fifthWeekDate = getOffset7Day(fourthWeekDate);
        String firstWeekStr = DateUtil.format(firstWeekDate, "d日") + "-" + DateUtil.format(getOffset6Day(firstWeekDate), "d日") + "  第一周";
        String secondWeekStr = DateUtil.format(secondWeekDate, "d日") + "-" + DateUtil.format(getOffset6Day(secondWeekDate), "d日") + "  第二周";
        String thirdWeekStr = DateUtil.format(thirdWeekDate, "d日") + "-" + DateUtil.format(getOffset6Day(thirdWeekDate), "d日") + "  第三周";
        String fourthWeekStr = DateUtil.format(fourthWeekDate, "d日") + "-" + DateUtil.format(getOffset6Day(fourthWeekDate), "d日") + "  第四周";
        monthExportDTO.setFirstWeekStr(firstWeekStr);
        monthExportDTO.setSecondWeekStr(secondWeekStr);
        monthExportDTO.setThridWeekStr(thirdWeekStr);
        monthExportDTO.setFourthWeekStr(fourthWeekStr);
        if (endDate.after(fifthWeekDate)) {
            String fifthWeekStr = DateUtil.format(fifthWeekDate, "d日") + "-" + DateUtil.format(endDate, "d日") + "  第五周";
            monthExportDTO.setFifthWeekStr(fifthWeekStr);
        }
        if (endDate.equals(fifthWeekDate)) {
            String fifthWeekStr = DateUtil.format(fifthWeekDate, "d日") + "  第五周";
            monthExportDTO.setFifthWeekStr(fifthWeekStr);
        }
        return monthExportDTO;
    }

    private MonthExportDTO initMonthTotal(MonthExportDTO monthExportDTO, List<Integer> adminIdList, String beginDateOfMonth, String endDateOfMonth) {
        Integer total = figureUpByDateFromShortTermFaceLog(adminIdList, beginDateOfMonth, endDateOfMonth, null);
        if (Objects.nonNull(total) && (!total.equals(0))) {
            AtomicReference<Integer> firstWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> secondWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> thirdWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> forthWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> fifthWeekTotal = new AtomicReference<>(0);
            List<Map<String, Object>> weekGroupByDay = getMonthGroupByWeek(adminIdList, beginDateOfMonth, endDateOfMonth, null);
            if (CollectionUtil.isNotEmpty(weekGroupByDay)) {
                weekGroupByDay.stream().forEach(e -> {
                    String faceDay = String.valueOf(e.get("weekOfMonth"));
                    Integer dayCount = Integer.parseInt(String.valueOf(e.get("weekCount")));
                    if (ExportDocConstant.week1OfMonth.equals(faceDay)) {
                        firstWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week2OfMonth.equals(faceDay)) {
                        secondWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week3OfMonth.equals(faceDay)) {
                        thirdWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week4OfMonth.equals(faceDay)) {
                        forthWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week5OfMonth.equals(faceDay)) {
                        fifthWeekTotal.set(dayCount);
                    }
                });
            }
            Double firstWeekPercentage = getDouble(String.valueOf(firstWeekTotal.get()), String.valueOf(total));
            Double secondWeekPercentage = getDouble(String.valueOf(secondWeekTotal.get()), String.valueOf(total));
            Double thirdWeekPercentage = getDouble(String.valueOf(thirdWeekTotal.get()), String.valueOf(total));
            Double fourthWeekPercentage = getDouble(String.valueOf(forthWeekTotal.get()), String.valueOf(total));
            Double fifthWeekPercentage = getDouble(String.valueOf(fifthWeekTotal.get()), String.valueOf(total));
            monthExportDTO.setFirstWeekTotal(firstWeekTotal.get());
            monthExportDTO.setSecondWeekTotal(secondWeekTotal.get());
            monthExportDTO.setThridWeekTotal(thirdWeekTotal.get());
            monthExportDTO.setFourthWeekTotal(forthWeekTotal.get());
            monthExportDTO.setFifthWeekTotal(fifthWeekTotal.get());
            monthExportDTO.setFirstWeekPercentage(firstWeekPercentage);
            monthExportDTO.setSecondWeekPercentage(secondWeekPercentage);
            monthExportDTO.setThridWeekPercentage(thirdWeekPercentage);
            monthExportDTO.setFourthWeekPercentage(fourthWeekPercentage);
            monthExportDTO.setFifthWeekPercentage(fifthWeekPercentage);
        }
        return monthExportDTO;
    }

    private MonthExportDTO initMonthInTotal(MonthExportDTO monthExportDTO, List<Integer> adminIdList, String beginDateOfMonth, String endDateOfMonth) {
        Integer total = figureUpByDateFromShortTermFaceLog(adminIdList, beginDateOfMonth, endDateOfMonth, DevicePassTypeEnum.PASS_IN);
        if (Objects.nonNull(total) && (!total.equals(0))) {
            AtomicReference<Integer> firstWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> secondWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> thirdWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> forthWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> fifthWeekTotal = new AtomicReference<>(0);
            List<Map<String, Object>> weekGroupByDay = getMonthGroupByWeek(adminIdList, beginDateOfMonth, endDateOfMonth, DevicePassTypeEnum.PASS_IN);
            if (CollectionUtil.isNotEmpty(weekGroupByDay)) {
                weekGroupByDay.stream().forEach(e -> {
                    String faceDay = String.valueOf(e.get("weekOfMonth"));
                    Integer dayCount = Integer.parseInt(String.valueOf(e.get("weekCount")));
                    if (ExportDocConstant.week1OfMonth.equals(faceDay)) {
                        firstWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week2OfMonth.equals(faceDay)) {
                        secondWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week3OfMonth.equals(faceDay)) {
                        thirdWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week4OfMonth.equals(faceDay)) {
                        forthWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week5OfMonth.equals(faceDay)) {
                        fifthWeekTotal.set(dayCount);
                    }
                });
            }
            Double firstWeekPercentage = getDouble(String.valueOf(firstWeekTotal.get()), String.valueOf(total));
            Double secondWeekPercentage = getDouble(String.valueOf(secondWeekTotal.get()), String.valueOf(total));
            Double thirdWeekPercentage = getDouble(String.valueOf(thirdWeekTotal.get()), String.valueOf(total));
            Double fourthWeekPercentage = getDouble(String.valueOf(forthWeekTotal.get()), String.valueOf(total));
            Double fifthWeekPercentage = getDouble(String.valueOf(fifthWeekTotal.get()), String.valueOf(total));
            monthExportDTO.setFirstWeekInTotal(firstWeekTotal.get());
            monthExportDTO.setSecondWeekInTotal(secondWeekTotal.get());
            monthExportDTO.setThridWeekInTotal(thirdWeekTotal.get());
            monthExportDTO.setFourthWeekInTotal(forthWeekTotal.get());
            monthExportDTO.setFifthWeekInTotal(fifthWeekTotal.get());
            monthExportDTO.setFirstWeekInPercentage(firstWeekPercentage);
            monthExportDTO.setSecondWeekInPercentage(secondWeekPercentage);
            monthExportDTO.setThridWeekInPercentage(thirdWeekPercentage);
            monthExportDTO.setFourthWeekInPercentage(fourthWeekPercentage);
            monthExportDTO.setFifthWeekInPercentage(fifthWeekPercentage);
        }
        return monthExportDTO;
    }

    private MonthExportDTO initMonthOutTotal(MonthExportDTO monthExportDTO, List<Integer> adminIdList, String beginDateOfMonth, String endDateOfMonth) {
        Integer total = figureUpByDateFromShortTermFaceLog(adminIdList, beginDateOfMonth, endDateOfMonth, DevicePassTypeEnum.PASS_OUT);
        if (Objects.nonNull(total) && (!total.equals(0))) {
            AtomicReference<Integer> firstWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> secondWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> thirdWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> forthWeekTotal = new AtomicReference<>(0);
            AtomicReference<Integer> fifthWeekTotal = new AtomicReference<>(0);
            List<Map<String, Object>> weekGroupByDay = getMonthGroupByWeek(adminIdList, beginDateOfMonth, endDateOfMonth, DevicePassTypeEnum.PASS_OUT);
            if (CollectionUtil.isNotEmpty(weekGroupByDay)) {
                weekGroupByDay.stream().forEach(e -> {
                    String faceDay = String.valueOf(e.get("weekOfMonth"));
                    Integer dayCount = Integer.parseInt(String.valueOf(e.get("weekCount")));
                    if (ExportDocConstant.week1OfMonth.equals(faceDay)) {
                        firstWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week2OfMonth.equals(faceDay)) {
                        secondWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week3OfMonth.equals(faceDay)) {
                        thirdWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week4OfMonth.equals(faceDay)) {
                        forthWeekTotal.set(dayCount);
                    }
                    if (ExportDocConstant.week5OfMonth.equals(faceDay)) {
                        fifthWeekTotal.set(dayCount);
                    }
                });
            }
            Double firstWeekPercentage = getDouble(String.valueOf(firstWeekTotal.get()), String.valueOf(total));
            Double secondWeekPercentage = getDouble(String.valueOf(secondWeekTotal.get()), String.valueOf(total));
            Double thirdWeekPercentage = getDouble(String.valueOf(thirdWeekTotal.get()), String.valueOf(total));
            Double fourthWeekPercentage = getDouble(String.valueOf(forthWeekTotal.get()), String.valueOf(total));
            Double fifthWeekPercentage = getDouble(String.valueOf(fifthWeekTotal.get()), String.valueOf(total));
            monthExportDTO.setFirstWeekOutTotal(firstWeekTotal.get());
            monthExportDTO.setSecondWeekOutTotal(secondWeekTotal.get());
            monthExportDTO.setThridWeekOutTotal(thirdWeekTotal.get());
            monthExportDTO.setFourthWeekOutTotal(forthWeekTotal.get());
            monthExportDTO.setFifthWeekOutTotal(fifthWeekTotal.get());
            monthExportDTO.setFirstWeekOutPercentage(firstWeekPercentage);
            monthExportDTO.setSecondWeekOutPercentage(secondWeekPercentage);
            monthExportDTO.setThridWeekOutPercentage(thirdWeekPercentage);
            monthExportDTO.setFourthWeekOutPercentage(fourthWeekPercentage);
            monthExportDTO.setFifthWeekOutPercentage(fifthWeekPercentage);
        }
        return monthExportDTO;
    }

    private MonthExportDTO initMonthExportDTO(String currentDate, MonthExportDTO monthExportDTO, AdminDeviceVO adminDeviceVO, ExportWordVO totalExportVO,
                                              ExportWordVO outExportVO, ExportWordVO inExportVO) {
        monthExportDTO.setCurrentMonth(currentDate);
        monthExportDTO.setDeviceTotal(adminDeviceVO.getDeviceAllNum());
        monthExportDTO.setOnlineNum(adminDeviceVO.getDeviceOnlineNum());
        monthExportDTO.setOfflineNum(adminDeviceVO.getDeviceOfflineNum());
        monthExportDTO.setMonthTotal(totalExportVO.getTotalNum());
        monthExportDTO.setRedTotal(totalExportVO.getRedNum());
        monthExportDTO.setYellowTotal(totalExportVO.getYellowNum());
        monthExportDTO.setGreenTotal(totalExportVO.getGreenNum());
        monthExportDTO.setRedConcactNum(totalExportVO.getContactRedNum());
        monthExportDTO.setYellowConcactNum(totalExportVO.getContactYellowNum());
        monthExportDTO.setTemperatureExceptionNum(totalExportVO.getTwExceptionNum());
        monthExportDTO.setTemperatureNormalNum(totalExportVO.getTwNormal());
        monthExportDTO.setNoVaccinationNum(totalExportVO.getNoInoculationNum());
        monthExportDTO.setVaccinationNum(totalExportVO.getInoculationNum());
        monthExportDTO.setNucleicAcidDetectionNum(totalExportVO.getNucleicNum());
        monthExportDTO.setMonthOutTotal(outExportVO.getTotalNum());
        monthExportDTO.setRedOutTotal(outExportVO.getRedNum());
        monthExportDTO.setYellowOutTotal(outExportVO.getYellowNum());
        monthExportDTO.setGreenOutTotal(outExportVO.getGreenNum());
        monthExportDTO.setRedConcactOutNum(outExportVO.getContactRedNum());
        monthExportDTO.setYellowConcactOutNum(outExportVO.getContactYellowNum());
        monthExportDTO.setTemperatureExceptionOutNum(outExportVO.getTwExceptionNum());
        monthExportDTO.setTemperatureNormalOutNum(outExportVO.getTwNormal());
        monthExportDTO.setNoVaccinationOutNum(outExportVO.getNoInoculationNum());
        monthExportDTO.setVaccinationOutNum(outExportVO.getInoculationNum());
        monthExportDTO.setNucleicAcidDetectionOutNum(outExportVO.getNucleicNum());
        monthExportDTO.setMonthInTotal(inExportVO.getTotalNum());
        monthExportDTO.setRedInTotal(inExportVO.getRedNum());
        monthExportDTO.setYellowInTotal(inExportVO.getYellowNum());
        monthExportDTO.setGreenInTotal(inExportVO.getGreenNum());
        monthExportDTO.setRedConcactInNum(inExportVO.getContactRedNum());
        monthExportDTO.setYellowConcactInNum(inExportVO.getContactYellowNum());
        monthExportDTO.setTemperatureExceptionInNum(inExportVO.getTwExceptionNum());
        monthExportDTO.setTemperatureNormalInNum(inExportVO.getTwNormal());
        monthExportDTO.setNoVaccinationInNum(inExportVO.getNoInoculationNum());
        monthExportDTO.setVaccinationInNum(inExportVO.getInoculationNum());
        monthExportDTO.setNucleicAcidDetectionInNum(inExportVO.getNucleicNum());
        return monthExportDTO;
    }

    private List<Map<String, Object>> getMonthGroupByWeek(List<Integer> adminIdList, String beginDateOfMonth,
                                                          String endDateOfMonth, DevicePassTypeEnum devicePassTypeEnum) {
        if (Objects.nonNull(devicePassTypeEnum)) {
            return faceLogDao.getMonthGroupByWeek(adminIdList, beginDateOfMonth, endDateOfMonth, devicePassTypeEnum.getType());
        }
        return faceLogDao.getMonthGroupByWeek(adminIdList, beginDateOfMonth, endDateOfMonth, null);
    }

    private ExportWordVO getFaceLogGloablePassData(List<AdminIDTypeDTO> adminIDTypeDTOListList, List<Integer> adminIdList, String beginDateOfMonth,
                                                   String endDateOfMonth, DevicePassTypeEnum devicePassTypeEnum) {
        ExportWordVO resultData = new ExportWordVO();
        Integer totalNum = 0;
        Integer redNum = 0;
        Integer yellowNum = 0;
        Integer greenNum = 0;
        Integer notKnowNum = 0;
        Integer twExceptionNum = 0;
        Integer twNormal = 0;
        Integer exceptionNum = 0;
        Integer inoculationNum = 0;
        Integer noInoculationNum = 0;
        Integer nucleicNum = 0;
        List<FaceLogVO> faceLogList = Lists.newArrayList();
        if (Objects.nonNull(devicePassTypeEnum)) {
            faceLogList = faceLogDao.getFaceLogByAdminIdAndFaceTime(adminIdList, devicePassTypeEnum.getType(), beginDateOfMonth, endDateOfMonth);
        } else {
            faceLogList = faceLogDao.getFaceLogByAdminIdAndFaceTime(adminIdList, null, beginDateOfMonth, endDateOfMonth);
        }
        if (CollectionUtil.isEmpty(faceLogList)) {
            return resultData;
        }
        for (AdminIDTypeDTO adminIDTypeDTO : adminIDTypeDTOListList) {
            List<FaceLogVO> filterAdminFaceLogList = faceLogList.stream().filter(e -> e.getAdminId().equals(adminIDTypeDTO.getAdminId())).collect(Collectors.toList());
            for (FaceLogVO faceLog : filterAdminFaceLogList) {
                if (faceLog.getJkmStatus().equals("10")) {
                    redNum++;
                } else if (faceLog.getJkmStatus().equals("01")) {
                    yellowNum++;
                } else if (faceLog.getJkmStatus().equals("00")) {
                    greenNum++;
                } else {
                    notKnowNum++;
                }
                if (faceLog.getTwStatus().equals("1")) {
                    twExceptionNum++;
                } else {
                    twNormal++;
                }
                if (faceLog.getYmStatus() != null && !faceLog.getYmStatus().equals("")) {
                    inoculationNum++;
                } else {
                    noInoculationNum++;
                }
                if (faceLog.getHsStatus() != null && !faceLog.getHsStatus().equals("")) {
                    nucleicNum++;
                }
            }
            totalNum += filterAdminFaceLogList.size();
        }
        Set<Long> contactYellowNumSet = Sets.newHashSet();
        Set<Long> contactRedNumSet = Sets.newHashSet();
        if(Objects.nonNull(devicePassTypeEnum)){
            contactYellowNumSet = faceLogDao.getContactHealthCodeNum(HealthCodeStatusEnum.YELLOW_CODE.getCode(),adminIdList,devicePassTypeEnum.getType(),beginDateOfMonth,endDateOfMonth);
            contactRedNumSet = faceLogDao.getContactHealthCodeNum(HealthCodeStatusEnum.RED_CODE.getCode(),adminIdList,devicePassTypeEnum.getType(),beginDateOfMonth,endDateOfMonth);
        }else{
            contactYellowNumSet = faceLogDao.getContactHealthCodeNum(HealthCodeStatusEnum.YELLOW_CODE.getCode(),adminIdList,null,beginDateOfMonth,endDateOfMonth);
            contactRedNumSet = faceLogDao.getContactHealthCodeNum(HealthCodeStatusEnum.RED_CODE.getCode(),adminIdList,null,beginDateOfMonth,endDateOfMonth);
        }
        resultData.setTotalNum(totalNum);
        resultData.setRedNum(redNum);
        resultData.setYellowNum(yellowNum);
        resultData.setGreenNum(greenNum);
        resultData.setNotKnowNum(notKnowNum);
        resultData.setTwExceptionNum(twExceptionNum);
        resultData.setTwNormal(twNormal);
        resultData.setContactRedNum(contactRedNumSet.size());
        resultData.setContactYellowNum(contactYellowNumSet.size());
        resultData.setExceptionNum(exceptionNum);
        resultData.setInoculationNum(inoculationNum);
        resultData.setNoInoculationNum(noInoculationNum);
        resultData.setNucleicNum(nucleicNum);
        return resultData;
    }

    @Override
    public void exportDocToClient(HttpServletResponse response, String fileName, String tplName, Object data) throws
            IOException, TemplateException {
        response.reset();
        response.setCharacterEncoding("UTF-8");
        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Content-Type", "application/vnd.openxmlformats-officedocument.wordprocessingml.document");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        Writer writer = response.getWriter();
        Template template = getTemplate(tplName);
        template.process(data, writer);
        writer.close();
    }

    private TodayExportDTO initTodayExportDTO(AdminVO adminInfo, TodayExportDTO todayExportDTO, String currentDate, AdminDeviceVO adminDeviceVO, ExportWordVO totalExportVO,
                                              ExportWordVO outExportVO, ExportWordVO inExportVO) {
        todayExportDTO.setCurrentDate(currentDate);
        todayExportDTO.setAdminName(adminInfo.getUsername());
        todayExportDTO.setDeviceTotal(adminDeviceVO.getDeviceAllNum());
        todayExportDTO.setOnlineNum(adminDeviceVO.getDeviceOnlineNum());
        todayExportDTO.setOfflineNum(adminDeviceVO.getDeviceOfflineNum());
        todayExportDTO.setTodayTotal(totalExportVO.getTotalNum());
        todayExportDTO.setRedTotal(totalExportVO.getRedNum());
        todayExportDTO.setYellowTotal(totalExportVO.getYellowNum());
        todayExportDTO.setGreenTotal(totalExportVO.getGreenNum());
        todayExportDTO.setRedConcactNum(totalExportVO.getContactRedNum());
        todayExportDTO.setYellowConcactNum(totalExportVO.getContactYellowNum());
        todayExportDTO.setTemperatureExceptionNum(totalExportVO.getTwExceptionNum());
        todayExportDTO.setTemperatureNormalNum(totalExportVO.getTwNormal());
        todayExportDTO.setNoVaccinationNum(totalExportVO.getNoInoculationNum());
        todayExportDTO.setVaccinationNum(totalExportVO.getInoculationNum());
        todayExportDTO.setNucleicAcidDetectionNum(totalExportVO.getNucleicNum());
        todayExportDTO.setTodayOutTotal(outExportVO.getTotalNum());
        todayExportDTO.setRedOutTotal(outExportVO.getRedNum());
        todayExportDTO.setYellowOutTotal(outExportVO.getYellowNum());
        todayExportDTO.setGreenOutTotal(outExportVO.getGreenNum());
        todayExportDTO.setRedConcactOutNum(outExportVO.getContactRedNum());
        todayExportDTO.setYellowConcactOutNum(outExportVO.getContactYellowNum());
        todayExportDTO.setTemperatureExceptionOutNum(outExportVO.getTwExceptionNum());
        todayExportDTO.setTemperatureNormalOutNum(outExportVO.getTwNormal());
        todayExportDTO.setNoVaccinationOutNum(outExportVO.getNoInoculationNum());
        todayExportDTO.setVaccinationOutNum(outExportVO.getInoculationNum());
        todayExportDTO.setNucleicAcidDetectionOutNum(outExportVO.getNucleicNum());
        todayExportDTO.setTodayInTotal(inExportVO.getTotalNum());
        todayExportDTO.setRedInTotal(inExportVO.getRedNum());
        todayExportDTO.setYellowInTotal(inExportVO.getYellowNum());
        todayExportDTO.setGreenInTotal(inExportVO.getGreenNum());
        todayExportDTO.setRedConcactInNum(inExportVO.getContactRedNum());
        todayExportDTO.setYellowConcactInNum(inExportVO.getContactYellowNum());
        todayExportDTO.setTemperatureExceptionInNum(inExportVO.getTwExceptionNum());
        todayExportDTO.setTemperatureNormalInNum(inExportVO.getTwNormal());
        todayExportDTO.setNoVaccinationInNum(inExportVO.getNoInoculationNum());
        todayExportDTO.setVaccinationInNum(inExportVO.getInoculationNum());
        todayExportDTO.setNucleicAcidDetectionInNum(inExportVO.getNucleicNum());
        return todayExportDTO;
    }

    private TodayExportDTO initTotal(TodayExportDTO todayExportDTO, List<Integer> adminIdList, String startToday, String endToday) {
        Integer total = figureUpByDateFromShortTermFaceLog(adminIdList, startToday, endToday, null);
        if (Objects.nonNull(total) && (!total.equals(0))) {
            AtomicReference<Integer> todayThreeClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> todaySixClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> todayNineClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> todayTwelveClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> todayFifteenClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> todayEighteenClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> todayTwentyOneClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> todayTwentyFourClockNum = new AtomicReference<>(0);
            List<Map<String, Object>> groupByHour = getDayGroupByHour(adminIdList, startToday, endToday, null);
            if (CollectionUtil.isNotEmpty(groupByHour)) {
                groupByHour.stream().forEach(e -> {
                    String hour = String.valueOf(e.get("faceHour"));
                    Integer hourCount = Integer.parseInt(String.valueOf(e.get("hourCount")));
                    if (ExportDocConstant.groupByDayHour1.contains(hour)) {
                        todayThreeClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour2.contains(hour)) {
                        todaySixClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour3.contains(hour)) {
                        todayNineClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour4.contains(hour)) {
                        todayTwelveClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour5.contains(hour)) {
                        todayFifteenClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour6.contains(hour)) {
                        todayEighteenClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour7.contains(hour)) {
                        todayTwentyOneClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour8.contains(hour)) {
                        todayTwentyFourClockNum.updateAndGet(v -> v + hourCount);
                    }
                });
            }
            Double todayThreeClockPercentage = getDouble(String.valueOf(todayThreeClockNum.get()), String.valueOf(total));
            Double todaySixClockPercentage = getDouble(String.valueOf(todaySixClockNum.get()), String.valueOf(total));
            Double todayNineClockPercentage = getDouble(String.valueOf(todayNineClockNum.get()), String.valueOf(total));
            Double todayTwelveClockPercentage = getDouble(String.valueOf(todayTwelveClockNum.get()), String.valueOf(total));
            Double todayFifteenClockPercentage = getDouble(String.valueOf(todayFifteenClockNum.get()), String.valueOf(total));
            Double todayEighteenClockPercentage = getDouble(String.valueOf(todayEighteenClockNum.get()), String.valueOf(total));
            Double todayTwentyOneClockPercentage = getDouble(String.valueOf(todayTwentyOneClockNum.get()), String.valueOf(total));
            Double todayTwentyFourClockPercentage = getDouble(String.valueOf(todayTwentyFourClockNum.get()), String.valueOf(total));

            todayExportDTO.setTodayThreeClockNum(todayThreeClockNum.get());
            todayExportDTO.setTodaySixClockNum(todaySixClockNum.get());
            todayExportDTO.setTodayNineClockNum(todayNineClockNum.get());
            todayExportDTO.setTodayTwelveClockNum(todayTwelveClockNum.get());
            todayExportDTO.setTodayFifteenClockNum(todayFifteenClockNum.get());
            todayExportDTO.setTodayEighteenClockNum(todayEighteenClockNum.get());
            todayExportDTO.setTodayTwentyOneClockNum(todayTwentyOneClockNum.get());
            todayExportDTO.setTodayTwentyFourClockNum(todayTwentyFourClockNum.get());
            todayExportDTO.setTodayThreeClockPercentage(todayThreeClockPercentage);
            todayExportDTO.setTodaySixClockPercentage(todaySixClockPercentage);
            todayExportDTO.setTodayNineClockPercentage(todayNineClockPercentage);
            todayExportDTO.setTodayTwelveClockPercentage(todayTwelveClockPercentage);
            todayExportDTO.setTodayFifteenClockPercentage(todayFifteenClockPercentage);
            todayExportDTO.setTodayEighteenClockPercentage(todayEighteenClockPercentage);
            todayExportDTO.setTodayTwentyOneClockPercentage(todayTwentyOneClockPercentage);
            todayExportDTO.setTodayTwentyFourClockPercentage(todayTwentyFourClockPercentage);
        }
        return todayExportDTO;
    }

    public TodayExportDTO initOut(TodayExportDTO todayExportDTO, List<Integer> adminIdList, String startToday, String endToday) {
        Integer totalOut = figureUpByDateFromShortTermFaceLog(adminIdList, startToday, endToday, DevicePassTypeEnum.PASS_OUT);
        if (Objects.nonNull(totalOut) && (!totalOut.equals(0))) {
            if (Objects.nonNull(totalOut) && (!totalOut.equals(0))) {
                AtomicReference<Integer> outTodayThreeClockNum = new AtomicReference<>(0);
                AtomicReference<Integer> outTodaySixClockNum = new AtomicReference<>(0);
                AtomicReference<Integer> outTodayNineClockNum = new AtomicReference<>(0);
                AtomicReference<Integer> outTodayTwelveClockNum = new AtomicReference<>(0);
                AtomicReference<Integer> outTodayFifteenClockNum = new AtomicReference<>(0);
                AtomicReference<Integer> outTodayEighteenClockNum = new AtomicReference<>(0);
                AtomicReference<Integer> outTodayTwentyOneClockNum = new AtomicReference<>(0);
                AtomicReference<Integer> outTodayTwentyFourClockNum = new AtomicReference<>(0);
                List<Map<String, Object>> groupByHour = getDayGroupByHour(adminIdList, startToday, endToday, DevicePassTypeEnum.PASS_OUT);
                if (CollectionUtil.isNotEmpty(groupByHour)) {
                    groupByHour.stream().forEach(e -> {
                        String hour = String.valueOf(e.get("faceHour"));
                        Integer hourCount = Integer.parseInt(String.valueOf(e.get("hourCount")));
                        if (ExportDocConstant.groupByDayHour1.contains(hour)) {
                            outTodayThreeClockNum.updateAndGet(v -> v + hourCount);
                        }
                        if (ExportDocConstant.groupByDayHour2.contains(hour)) {
                            outTodaySixClockNum.updateAndGet(v -> v + hourCount);
                        }
                        if (ExportDocConstant.groupByDayHour3.contains(hour)) {
                            outTodayNineClockNum.updateAndGet(v -> v + hourCount);
                        }
                        if (ExportDocConstant.groupByDayHour4.contains(hour)) {
                            outTodayTwelveClockNum.updateAndGet(v -> v + hourCount);
                        }
                        if (ExportDocConstant.groupByDayHour5.contains(hour)) {
                            outTodayFifteenClockNum.updateAndGet(v -> v + hourCount);
                        }
                        if (ExportDocConstant.groupByDayHour6.contains(hour)) {
                            outTodayEighteenClockNum.updateAndGet(v -> v + hourCount);
                        }
                        if (ExportDocConstant.groupByDayHour7.contains(hour)) {
                            outTodayTwentyOneClockNum.updateAndGet(v -> v + hourCount);
                        }
                        if (ExportDocConstant.groupByDayHour8.contains(hour)) {
                            outTodayTwentyFourClockNum.updateAndGet(v -> v + hourCount);
                        }
                    });
                }
                Double todayThreeClockPercentage = getDouble(String.valueOf(outTodayThreeClockNum.get()), String.valueOf(totalOut));
                Double todaySixClockPercentage = getDouble(String.valueOf(outTodaySixClockNum.get()), String.valueOf(totalOut));
                Double todayNineClockPercentage = getDouble(String.valueOf(outTodayNineClockNum.get()), String.valueOf(totalOut));
                Double todayTwelveClockPercentage = getDouble(String.valueOf(outTodayTwelveClockNum.get()), String.valueOf(totalOut));
                Double todayFifteenClockPercentage = getDouble(String.valueOf(outTodayFifteenClockNum.get()), String.valueOf(totalOut));
                Double todayEighteenClockPercentage = getDouble(String.valueOf(outTodayEighteenClockNum.get()), String.valueOf(totalOut));
                Double todayTwentyOneClockPercentage = getDouble(String.valueOf(outTodayTwentyOneClockNum.get()), String.valueOf(totalOut));
                Double todayTwentyFourClockPercentage = getDouble(String.valueOf(outTodayTwentyFourClockNum.get()), String.valueOf(totalOut));

                todayExportDTO.setTodayThreeClockOutNum(outTodayThreeClockNum.get());
                todayExportDTO.setTodaySixClockOutNum(outTodaySixClockNum.get());
                todayExportDTO.setTodayNineClockOutNum(outTodayNineClockNum.get());
                todayExportDTO.setTodayTwelveClockOutNum(outTodayTwelveClockNum.get());
                todayExportDTO.setTodayFifteenClockOutNum(outTodayFifteenClockNum.get());
                todayExportDTO.setTodayEighteenClockOutNum(outTodayEighteenClockNum.get());
                todayExportDTO.setTodayTwentyOneClockOutNum(outTodayTwentyOneClockNum.get());
                todayExportDTO.setTodayTwentyFourClockOutNum(outTodayTwentyFourClockNum.get());
                todayExportDTO.setTodayThreeClockOutPercentage(todayThreeClockPercentage);
                todayExportDTO.setTodaySixClockOutPercentage(todaySixClockPercentage);
                todayExportDTO.setTodayNineClockOutPercentage(todayNineClockPercentage);
                todayExportDTO.setTodayTwelveClockOutPercentage(todayTwelveClockPercentage);
                todayExportDTO.setTodayFifteenClockOutPercentage(todayFifteenClockPercentage);
                todayExportDTO.setTodayEighteenClockOutPercentage(todayEighteenClockPercentage);
                todayExportDTO.setTodayTwentyOneClockOutPercentage(todayTwentyOneClockPercentage);
                todayExportDTO.setTodayTwentyFourClockOutPercentage(todayTwentyFourClockPercentage);
            }
        }
        return todayExportDTO;
    }

    public TodayExportDTO initIn(TodayExportDTO todayExportDTO, List<Integer> adminIdList, String startToday, String endToday) {
        Integer totalIn = figureUpByDateFromShortTermFaceLog(adminIdList, startToday, endToday, DevicePassTypeEnum.PASS_IN);
        if (Objects.nonNull(totalIn) && (!totalIn.equals(0))) {
            AtomicReference<Integer> inTodayThreeClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> inTodaySixClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> inTodayNineClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> inTodayTwelveClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> inTodayFifteenClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> inTodayEighteenClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> inTodayTwentyOneClockNum = new AtomicReference<>(0);
            AtomicReference<Integer> inTodayTwentyFourClockNum = new AtomicReference<>(0);
            List<Map<String, Object>> groupByHour = getDayGroupByHour(adminIdList, startToday, endToday, DevicePassTypeEnum.PASS_IN);
            if (CollectionUtil.isNotEmpty(groupByHour)) {
                groupByHour.stream().forEach(e -> {
                    String hour = String.valueOf(e.get("faceHour"));
                    Integer hourCount = Integer.parseInt(String.valueOf(e.get("hourCount")));
                    if (ExportDocConstant.groupByDayHour1.contains(hour)) {
                        inTodayThreeClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour2.contains(hour)) {
                        inTodaySixClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour3.contains(hour)) {
                        inTodayNineClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour4.contains(hour)) {
                        inTodayTwelveClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour5.contains(hour)) {
                        inTodayFifteenClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour6.contains(hour)) {
                        inTodayEighteenClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour7.contains(hour)) {
                        inTodayTwentyOneClockNum.updateAndGet(v -> v + hourCount);
                    }
                    if (ExportDocConstant.groupByDayHour8.contains(hour)) {
                        inTodayTwentyFourClockNum.updateAndGet(v -> v + hourCount);
                    }
                });
            }
            Double todayThreeClockPercentage = getDouble(String.valueOf(inTodayThreeClockNum.get()), String.valueOf(totalIn));
            Double todaySixClockPercentage = getDouble(String.valueOf(inTodaySixClockNum.get()), String.valueOf(totalIn));
            Double todayNineClockPercentage = getDouble(String.valueOf(inTodayNineClockNum.get()), String.valueOf(totalIn));
            Double todayTwelveClockPercentage = getDouble(String.valueOf(inTodayTwelveClockNum.get()), String.valueOf(totalIn));
            Double todayFifteenClockPercentage = getDouble(String.valueOf(inTodayFifteenClockNum.get()), String.valueOf(totalIn));
            Double todayEighteenClockPercentage = getDouble(String.valueOf(inTodayEighteenClockNum.get()), String.valueOf(totalIn));
            Double todayTwentyOneClockPercentage = getDouble(String.valueOf(inTodayTwentyOneClockNum.get()), String.valueOf(totalIn));
            Double todayTwentyFourClockPercentage = getDouble(String.valueOf(inTodayTwentyFourClockNum.get()), String.valueOf(totalIn));
            todayExportDTO.setTodayThreeClockInNum(inTodayThreeClockNum.get());
            todayExportDTO.setTodaySixClockInNum(inTodaySixClockNum.get());
            todayExportDTO.setTodayNineClockInNum(inTodayNineClockNum.get());
            todayExportDTO.setTodayTwelveClockInNum(inTodayTwelveClockNum.get());
            todayExportDTO.setTodayFifteenClockInNum(inTodayFifteenClockNum.get());
            todayExportDTO.setTodayEighteenClockInNum(inTodayEighteenClockNum.get());
            todayExportDTO.setTodayTwentyOneClockInNum(inTodayTwentyOneClockNum.get());
            todayExportDTO.setTodayTwentyFourClockInNum(inTodayTwentyFourClockNum.get());

            todayExportDTO.setTodayThreeClockInPercentage(todayThreeClockPercentage);
            todayExportDTO.setTodaySixClockInPercentage(todaySixClockPercentage);
            todayExportDTO.setTodayNineClockInPercentage(todayNineClockPercentage);
            todayExportDTO.setTodayTwelveClockInPercentage(todayTwelveClockPercentage);
            todayExportDTO.setTodayFifteenClockInPercentage(todayFifteenClockPercentage);
            todayExportDTO.setTodayEighteenClockInPercentage(todayEighteenClockPercentage);
            todayExportDTO.setTodayTwentyOneClockInPercentage(todayTwentyOneClockPercentage);
            todayExportDTO.setTodayTwentyFourClockInPercentage(todayTwentyFourClockPercentage);
        }
        return todayExportDTO;
    }


    public ExportWordVO getGloablePassData(List<AdminIDTypeDTO> adminIDTypeDTOListList, List<Integer> adminIdList,
                                           String startDate, String endDate, DevicePassTypeEnum devicePassTypeEnum) {
        ExportWordVO resultData = new ExportWordVO();
        Integer totalNum = 0;
        Integer redNum = 0;
        Integer yellowNum = 0;
        Integer greenNum = 0;
        Integer notKnowNum = 0;
        Integer twExceptionNum = 0;
        Integer twNormal = 0;
        Integer exceptionNum = 0;
        Integer inoculationNum = 0;
        Integer noInoculationNum = 0;
        Integer nucleicNum = 0;
        List<FaceLogVO> faceLogList = Lists.newArrayList();
        if (Objects.nonNull(devicePassTypeEnum)) {
            faceLogList = faceLogDao.getFaceLogByAdminIdAndFaceTime(adminIdList, devicePassTypeEnum.getType(), startDate, endDate);
        } else {
            faceLogList = faceLogDao.getFaceLogByAdminIdAndFaceTime(adminIdList, null, startDate, endDate);
        }
        if (CollectionUtil.isEmpty(faceLogList)) {
            return resultData;
        }
        for (AdminIDTypeDTO adminIDTypeDTO : adminIDTypeDTOListList) {
            List<FaceLogVO> filterAdminFaceLogList = faceLogList.stream().filter(e -> e.getAdminId().equals(adminIDTypeDTO.getAdminId())).collect(Collectors.toList());
            for (FaceLogVO faceLog : filterAdminFaceLogList) {
                if (faceLog.getJkmStatus().equals("10")) {
                    redNum++;
                } else if (faceLog.getJkmStatus().equals("01")) {
                    yellowNum++;
                } else if (faceLog.getJkmStatus().equals("00")) {
                    greenNum++;
                } else {
                    notKnowNum++;
                }
                if (faceLog.getTwStatus().equals("1")) {
                    twExceptionNum++;
                } else {
                    twNormal++;
                }
                if (faceLog.getYmStatus() != null && !faceLog.getYmStatus().equals("")) {
                    inoculationNum++;
                } else {
                    noInoculationNum++;
                }
                if (faceLog.getHsStatus() != null && !faceLog.getHsStatus().equals("")) {
                    nucleicNum++;
                }
            }
            totalNum += filterAdminFaceLogList.size();
        }
        Set<Long> concactYellow = Sets.newHashSet();
        Set<Long> concactRed = Sets.newHashSet();
        List<Integer> adminIds = adminIDTypeDTOListList.stream().map(e->{return e.getAdminId();}).collect(Collectors.toList());
        if(Objects.nonNull(devicePassTypeEnum)){
            concactYellow = shortTermFaceLogDao.selectConcactHealthCodeFaces(HealthCodeStatusEnum.YELLOW_CODE.getCode(),
                    adminIds, devicePassTypeEnum.getType(),startDate,endDate);
            concactRed = shortTermFaceLogDao.selectConcactHealthCodeFaces(HealthCodeStatusEnum.RED_CODE.getCode(),
                    adminIds,devicePassTypeEnum.getType(),startDate,endDate);
        }else{
            concactYellow = shortTermFaceLogDao.selectConcactHealthCodeFaces(HealthCodeStatusEnum.YELLOW_CODE.getCode(),
                    adminIds,null,startDate,endDate);
            concactRed = shortTermFaceLogDao.selectConcactHealthCodeFaces(HealthCodeStatusEnum.RED_CODE.getCode(),
                    adminIds,null,startDate,endDate);
        }
        resultData.setTotalNum(totalNum);
        resultData.setRedNum(redNum);
        resultData.setYellowNum(yellowNum);
        resultData.setGreenNum(greenNum);
        resultData.setNotKnowNum(notKnowNum);
        resultData.setTwExceptionNum(twExceptionNum);
        resultData.setTwNormal(twNormal);
        resultData.setContactRedNum(concactRed.size());
        resultData.setContactYellowNum(concactYellow.size());
        resultData.setExceptionNum(exceptionNum);
        resultData.setInoculationNum(inoculationNum);
        resultData.setNoInoculationNum(noInoculationNum);
        resultData.setNucleicNum(nucleicNum);
        return resultData;
    }

    public List<AdminIDTypeDTO> getAdminIdList(AdminVO adminInfo) {
        List<AdminIDTypeDTO> adminIDTypeLists = Lists.newArrayList();
        switch (AdminTypeEnum.getAdminTypeEnumByCode(adminInfo.getType())) {
            case ADMINISTRATOR:
                adminIDTypeLists = adminDataShowDao.selectIdAndTypeList(adminInfo);
                break;
            case PROVINCIAL_MASTER_ACCOUNT:
                adminIDTypeLists = adminDataShowDao.selectIdAndTypeList(adminInfo);
                break;
            case PROVINCIAL_LEVEL:
                adminIDTypeLists = adminDataShowDao.selectIdAndTypeList(adminInfo);
                break;
            case MUNICIPAL:
                adminIDTypeLists = adminDataShowDao.selectIdAndTypeList(adminInfo);
                break;
            case DISTRICT:
                AdminIDTypeDTO adminIDTypeDTO = new AdminIDTypeDTO();
                adminIDTypeDTO.setType(adminInfo.getType());
                adminIDTypeDTO.setAdminId(adminInfo.getAdminId());
                adminIDTypeLists.add(adminIDTypeDTO);
                break;
            case ENTERPRISE:
                return adminIDTypeLists;
            default:
                break;
        }
        return adminIDTypeLists;
    }

    /**
     * 热数据 基于日期统计总数
     *
     * @param startDate
     * @param endDate
     * @param devicePassTypeEnum
     * @return
     */
    public Integer figureUpByDateFromShortTermFaceLog(List<Integer> adminIdList, String startDate, String
            endDate, DevicePassTypeEnum devicePassTypeEnum) {
        if (Objects.nonNull(devicePassTypeEnum)) {
            return shortTermFaceLogDao.figureUpByDateFromShortTermFaceLog(adminIdList, startDate, endDate, devicePassTypeEnum.getType());
        }
        return shortTermFaceLogDao.figureUpByDateFromShortTermFaceLog(adminIdList, startDate, endDate, null);
    }

    /**
     * 全量数据 基于日期统计总数
     *
     * @param startDate
     * @param endDate
     * @param devicePassTypeEnum
     * @return
     */
    public Integer figureUpByDateFromFaceLog(List<Integer> adminIdList, String startDate, String
            endDate, DevicePassTypeEnum devicePassTypeEnum) {
        if (Objects.nonNull(devicePassTypeEnum)) {
            return faceLogDao.figureUpByDateFromFaceLog(adminIdList, startDate, endDate, devicePassTypeEnum.getType());
        }
        return faceLogDao.figureUpByDateFromFaceLog(adminIdList, startDate, endDate, null);
    }

    public List<Map<String, Object>> getDayGroupByHour(List<Integer> adminIdList, String startDate, String
            endDate, DevicePassTypeEnum devicePassTypeEnum) {
        if (Objects.nonNull(devicePassTypeEnum)) {
            return shortTermFaceLogDao.getDayGroupByHour(adminIdList, startDate, endDate, devicePassTypeEnum.getType());
        }
        return shortTermFaceLogDao.getDayGroupByHour(adminIdList, startDate, endDate, null);
    }

    public static Date getOffset1Day(Date date) {
        return DateUtil.offsetDay(date, 1).toJdkDate();
    }

    public static Date getOffset7Day(Date date) {
        return DateUtil.offsetDay(date, 7);
    }

    public static Date getOffset6Day(Date date) {
        return DateUtil.offsetDay(date, 6);
    }

    private WeekExportDTO initWeekDate(WeekExportDTO weekExportDTO, String beginWeekDate, String endWeekDate) {
        weekExportDTO.setStartWeekendDate(beginWeekDate);
        weekExportDTO.setEndWeekendDate(endWeekDate);
        Date today = DateUtils.getNowDate();
        Date monday = DateUtil.beginOfWeek(DateUtil.parse(DateUtil.today()));
        String mondayStr = DateUtil.format(monday, "MM月dd日");
        weekExportDTO.setMondayStr(mondayStr + "  周一");
        Date tuesday = getOffset1Day(monday);
        String tuesdayStr = DateUtil.format(tuesday, "MM月dd日");
        weekExportDTO.setTuesdayStr(tuesdayStr + "  周二");
        Date wednesDay = getOffset1Day(tuesday);
        String wednesDayStr = DateUtil.format(wednesDay, "MM月dd日");
        weekExportDTO.setWednesDayStr(wednesDayStr + "  周三");
        Date thursday = getOffset1Day(wednesDay);
        String thursdayStr = DateUtil.format(thursday, "MM月dd日");
        weekExportDTO.setThursdayStr(thursdayStr + "  周四");
        Date friday = getOffset1Day(thursday);
        String fridayStr = DateUtil.format(friday, "MM月dd日");
        weekExportDTO.setFridayStr(fridayStr + "  周五");
        Date saturday = getOffset1Day(friday);
        String saturdayStr = DateUtil.format(saturday, "MM月dd日");
        weekExportDTO.setSaturdayStr(saturdayStr + "  周六");
        Date sunday = getOffset1Day(saturday);
        String sundayStr = DateUtil.format(sunday, "MM月dd日");
        weekExportDTO.setSundayStr(sundayStr + "  周日");
        return weekExportDTO;
    }

    private WeekExportDTO initWeekTotal(WeekExportDTO weekExportDTO, List<Integer> adminIdList, String startWeek, String endWeek) {
        Integer total = figureUpByDateFromShortTermFaceLog(adminIdList, startWeek, endWeek, null);
        if (Objects.nonNull(total) && (!total.equals(0))) {
            AtomicReference<Integer> mondayTotal = new AtomicReference<>(0);
            AtomicReference<Integer> tuesdayTotal = new AtomicReference<>(0);
            AtomicReference<Integer> wednesDayTotal = new AtomicReference<>(0);
            AtomicReference<Integer> thursdayTotal = new AtomicReference<>(0);
            AtomicReference<Integer> fridayTotal = new AtomicReference<>(0);
            AtomicReference<Integer> saturdayTotal = new AtomicReference<>(0);
            AtomicReference<Integer> sundayTotal = new AtomicReference<>(0);
            List<Map<String, Object>> weekGroupByDay = getWeekGroupByDay(adminIdList, startWeek, endWeek, null);
            if (CollectionUtil.isNotEmpty(weekGroupByDay)) {
                weekGroupByDay.stream().forEach(e -> {
                    String faceDay = String.valueOf(e.get("faceDay"));
                    Integer dayCount = Integer.parseInt(String.valueOf(e.get("dayCount")));
                    if (ExportDocConstant.day1OfWeek.equals(faceDay)) {
                        mondayTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day2OfWeek.equals(faceDay)) {
                        tuesdayTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day3OfWeek.equals(faceDay)) {
                        wednesDayTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day4OfWeek.equals(faceDay)) {
                        thursdayTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day5OfWeek.equals(faceDay)) {
                        fridayTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day6OfWeek.equals(faceDay)) {
                        saturdayTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day7OfWeek.equals(faceDay)) {
                        sundayTotal.set(dayCount);
                    }
                });
            }
            Double mondayPercentage = getDouble(String.valueOf(mondayTotal.get()), String.valueOf(total));
            Double tuesdayPercentage = getDouble(String.valueOf(tuesdayTotal.get()), String.valueOf(total));
            Double wednesDayPercentage = getDouble(String.valueOf(wednesDayTotal.get()), String.valueOf(total));
            Double thursdayPercentage = getDouble(String.valueOf(thursdayTotal.get()), String.valueOf(total));
            Double fridayPercentage = getDouble(String.valueOf(fridayTotal.get()), String.valueOf(total));
            Double saturdayPercentage = getDouble(String.valueOf(saturdayTotal.get()), String.valueOf(total));
            Double sundayPercentage = getDouble(String.valueOf(sundayTotal.get()), String.valueOf(total));

            weekExportDTO.setMondayTotal(mondayTotal.get());
            weekExportDTO.setMondayPercentage(mondayPercentage);
            weekExportDTO.setTuesdayTotal(tuesdayTotal.get());
            weekExportDTO.setTuesdayPercentage(tuesdayPercentage);
            weekExportDTO.setWednesDayTotal(wednesDayTotal.get());
            weekExportDTO.setWednesDayPercentage(wednesDayPercentage);
            weekExportDTO.setThursdayTotal(thursdayTotal.get());
            weekExportDTO.setThursdayPercentage(thursdayPercentage);
            weekExportDTO.setFridayTotal(fridayTotal.get());
            weekExportDTO.setFridayPercentage(fridayPercentage);
            weekExportDTO.setSaturdayTotal(saturdayTotal.get());
            weekExportDTO.setSaturdayPercentage(saturdayPercentage);
            weekExportDTO.setSundayTotal(sundayTotal.get());
            weekExportDTO.setSundayPercentage(sundayPercentage);
        }
        return weekExportDTO;
    }


    private WeekExportDTO initWeekIn(WeekExportDTO weekExportDTO, List<Integer> adminIdList, String startWeek, String endWeek) {
        Integer inTotal = figureUpByDateFromShortTermFaceLog(adminIdList, startWeek, endWeek, DevicePassTypeEnum.PASS_IN);
        if (Objects.nonNull(inTotal) && (!inTotal.equals(0))) {
            AtomicReference<Integer> mondayInTotal = new AtomicReference<>(0);
            AtomicReference<Integer> tuesdayInTotal = new AtomicReference<>(0);
            AtomicReference<Integer> wednesDayInTotal = new AtomicReference<>(0);
            AtomicReference<Integer> thursdayInTotal = new AtomicReference<>(0);
            AtomicReference<Integer> fridayInTotal = new AtomicReference<>(0);
            AtomicReference<Integer> saturdayInTotal = new AtomicReference<>(0);
            AtomicReference<Integer> sundayInTotal = new AtomicReference<>(0);
            List<Map<String, Object>> weekGroupByDay = getWeekGroupByDay(adminIdList, startWeek, endWeek, DevicePassTypeEnum.PASS_IN);
            if (CollectionUtil.isNotEmpty(weekGroupByDay)) {
                weekGroupByDay.stream().forEach(e -> {
                    String faceDay = String.valueOf(e.get("faceDay"));
                    Integer dayCount = Integer.parseInt(String.valueOf(e.get("dayCount")));
                    if (ExportDocConstant.day1OfWeek.equals(faceDay)) {
                        mondayInTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day2OfWeek.equals(faceDay)) {
                        tuesdayInTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day3OfWeek.equals(faceDay)) {
                        wednesDayInTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day4OfWeek.equals(faceDay)) {
                        thursdayInTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day5OfWeek.equals(faceDay)) {
                        fridayInTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day6OfWeek.equals(faceDay)) {
                        saturdayInTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day7OfWeek.equals(faceDay)) {
                        sundayInTotal.set(dayCount);
                    }
                });
            }
            Double mondayPercentage = getDouble(String.valueOf(mondayInTotal.get()), String.valueOf(inTotal));
            Double tuesdayPercentage = getDouble(String.valueOf(tuesdayInTotal.get()), String.valueOf(inTotal));
            Double wednesDayPercentage = getDouble(String.valueOf(wednesDayInTotal.get()), String.valueOf(inTotal));
            Double thursdayPercentage = getDouble(String.valueOf(thursdayInTotal.get()), String.valueOf(inTotal));
            Double fridayPercentage = getDouble(String.valueOf(fridayInTotal.get()), String.valueOf(inTotal));
            Double saturdayPercentage = getDouble(String.valueOf(saturdayInTotal.get()), String.valueOf(inTotal));
            Double sundayPercentage = getDouble(String.valueOf(sundayInTotal.get()), String.valueOf(inTotal));
            weekExportDTO.setMondayInTotal(mondayInTotal.get());
            weekExportDTO.setMondayInPercentage(mondayPercentage);
            weekExportDTO.setTuesdayInTotal(tuesdayInTotal.get());
            weekExportDTO.setTuesdayInPercentage(tuesdayPercentage);
            weekExportDTO.setWednesDayInTotal(wednesDayInTotal.get());
            weekExportDTO.setWednesDayInPercentage(wednesDayPercentage);
            weekExportDTO.setThursdayInTotal(thursdayInTotal.get());
            weekExportDTO.setThursdayInPercentage(thursdayPercentage);
            weekExportDTO.setFridayInTotal(fridayInTotal.get());
            weekExportDTO.setFridayInPercentage(fridayPercentage);
            weekExportDTO.setSaturdayInTotal(saturdayInTotal.get());
            weekExportDTO.setSaturdayInPercentage(saturdayPercentage);
            weekExportDTO.setSundayInTotal(sundayInTotal.get());
            weekExportDTO.setSundayInPercentage(sundayPercentage);
        }
        return weekExportDTO;
    }

    private WeekExportDTO initWeekOut(WeekExportDTO weekExportDTO, List<Integer> adminIdList, String startWeek, String endWeek) {
        Integer outTotal = figureUpByDateFromShortTermFaceLog(adminIdList, startWeek, endWeek, DevicePassTypeEnum.PASS_OUT);
        if (Objects.nonNull(outTotal) && (!outTotal.equals(0))) {
            AtomicReference<Integer> mondayOutTotal = new AtomicReference<>(0);
            AtomicReference<Integer> tuesdayOutTotal = new AtomicReference<>(0);
            AtomicReference<Integer> wednesDayOutTotal = new AtomicReference<>(0);
            AtomicReference<Integer> thursdayOutTotal = new AtomicReference<>(0);
            AtomicReference<Integer> fridayOutTotal = new AtomicReference<>(0);
            AtomicReference<Integer> saturdayOutTotal = new AtomicReference<>(0);
            AtomicReference<Integer> sundayOutTotal = new AtomicReference<>(0);
            List<Map<String, Object>> weekGroupByDay = getWeekGroupByDay(adminIdList, startWeek, endWeek, DevicePassTypeEnum.PASS_OUT);
            if (CollectionUtil.isNotEmpty(weekGroupByDay)) {
                weekGroupByDay.stream().forEach(e -> {
                    String faceDay = String.valueOf(e.get("faceDay"));
                    Integer dayCount = Integer.parseInt(String.valueOf(e.get("dayCount")));
                    if (ExportDocConstant.day1OfWeek.equals(faceDay)) {
                        mondayOutTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day2OfWeek.equals(faceDay)) {
                        tuesdayOutTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day3OfWeek.equals(faceDay)) {
                        wednesDayOutTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day4OfWeek.equals(faceDay)) {
                        thursdayOutTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day5OfWeek.equals(faceDay)) {
                        fridayOutTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day6OfWeek.equals(faceDay)) {
                        saturdayOutTotal.set(dayCount);
                    }
                    if (ExportDocConstant.day7OfWeek.equals(faceDay)) {
                        sundayOutTotal.set(dayCount);
                    }
                });
            }
            Double mondayPercentage = getDouble(String.valueOf(mondayOutTotal.get()), String.valueOf(outTotal));
            Double tuesdayPercentage = getDouble(String.valueOf(tuesdayOutTotal.get()), String.valueOf(outTotal));
            Double wednesDayPercentage = getDouble(String.valueOf(wednesDayOutTotal.get()), String.valueOf(outTotal));
            Double thursdayPercentage = getDouble(String.valueOf(thursdayOutTotal.get()), String.valueOf(outTotal));
            Double fridayPercentage = getDouble(String.valueOf(fridayOutTotal.get()), String.valueOf(outTotal));
            Double saturdayPercentage = getDouble(String.valueOf(saturdayOutTotal.get()), String.valueOf(outTotal));
            Double sundayPercentage = getDouble(String.valueOf(sundayOutTotal.get()), String.valueOf(outTotal));

            weekExportDTO.setMondayOutTotal(mondayOutTotal.get());
            weekExportDTO.setMondayOutPercentage(mondayPercentage);
            weekExportDTO.setTuesdayOutTotal(tuesdayOutTotal.get());
            weekExportDTO.setTuesdayOutPercentage(tuesdayPercentage);
            weekExportDTO.setWednesDayOutTotal(wednesDayOutTotal.get());
            weekExportDTO.setWednesDayOutPercentage(wednesDayPercentage);
            weekExportDTO.setThursdayOutTotal(thursdayOutTotal.get());
            weekExportDTO.setThursdayOutPercentage(thursdayPercentage);
            weekExportDTO.setFridayOutTotal(fridayOutTotal.get());
            weekExportDTO.setFridayOutPercentage(fridayPercentage);
            weekExportDTO.setSaturdayOutTotal(saturdayOutTotal.get());
            weekExportDTO.setSaturdayOutPercentage(saturdayPercentage);
            weekExportDTO.setSundayOutTotal(sundayOutTotal.get());
            weekExportDTO.setSundayOutPercentage(sundayPercentage);
        }
        return weekExportDTO;
    }

    private List<Map<String, Object>> getWeekGroupByDay(List<Integer> adminIdList, String startWeek,
                                                        String endWeek, DevicePassTypeEnum devicePassTypeEnum) {
        if (Objects.nonNull(devicePassTypeEnum)) {
            return shortTermFaceLogDao.getWeekGroupByDay(adminIdList, startWeek, endWeek, devicePassTypeEnum.getType());
        }
        return shortTermFaceLogDao.getWeekGroupByDay(adminIdList, startWeek, endWeek, null);
    }

    private WeekExportDTO initWeekExportDTO(WeekExportDTO weekExportDTO, AdminDeviceVO adminDeviceVO, ExportWordVO totalExportVO, ExportWordVO outExportVO, ExportWordVO inExportVO) {
        weekExportDTO.setDeviceTotal(adminDeviceVO.getDeviceAllNum());
        weekExportDTO.setOnlineNum(adminDeviceVO.getDeviceOnlineNum());
        weekExportDTO.setOfflineNum(adminDeviceVO.getDeviceOfflineNum());
        weekExportDTO.setWeekTotal(totalExportVO.getTotalNum());
        weekExportDTO.setRedTotal(totalExportVO.getRedNum());
        weekExportDTO.setYellowTotal(totalExportVO.getYellowNum());
        weekExportDTO.setGreenTotal(totalExportVO.getGreenNum());
        weekExportDTO.setRedConcactNum(totalExportVO.getContactRedNum());
        weekExportDTO.setYellowConcactNum(totalExportVO.getContactYellowNum());
        weekExportDTO.setTemperatureExceptionNum(totalExportVO.getTwExceptionNum());
        weekExportDTO.setTemperatureNormalNum(totalExportVO.getTwNormal());
        weekExportDTO.setNoVaccinationNum(totalExportVO.getNoInoculationNum());
        weekExportDTO.setVaccinationNum(totalExportVO.getInoculationNum());
        weekExportDTO.setNucleicAcidDetectionNum(totalExportVO.getNucleicNum());
        weekExportDTO.setWeekOutTotal(outExportVO.getTotalNum());
        weekExportDTO.setRedOutTotal(outExportVO.getRedNum());
        weekExportDTO.setYellowOutTotal(outExportVO.getYellowNum());
        weekExportDTO.setGreenOutTotal(outExportVO.getGreenNum());
        weekExportDTO.setRedConcactOutNum(outExportVO.getContactRedNum());
        weekExportDTO.setYellowConcactOutNum(outExportVO.getContactYellowNum());
        weekExportDTO.setTemperatureExceptionOutNum(outExportVO.getTwExceptionNum());
        weekExportDTO.setTemperatureNormalOutNum(outExportVO.getTwNormal());
        weekExportDTO.setNoVaccinationOutNum(outExportVO.getNoInoculationNum());
        weekExportDTO.setVaccinationOutNum(outExportVO.getInoculationNum());
        weekExportDTO.setNucleicAcidDetectionOutNum(outExportVO.getNucleicNum());
        weekExportDTO.setWeekInTotal(inExportVO.getTotalNum());
        weekExportDTO.setRedInTotal(inExportVO.getRedNum());
        weekExportDTO.setYellowInTotal(inExportVO.getYellowNum());
        weekExportDTO.setGreenInTotal(inExportVO.getGreenNum());
        weekExportDTO.setRedConcactInNum(inExportVO.getContactRedNum());
        weekExportDTO.setYellowConcactInNum(inExportVO.getContactYellowNum());
        weekExportDTO.setTemperatureExceptionInNum(inExportVO.getTwExceptionNum());
        weekExportDTO.setTemperatureNormalInNum(inExportVO.getTwNormal());
        weekExportDTO.setNoVaccinationInNum(inExportVO.getNoInoculationNum());
        weekExportDTO.setVaccinationInNum(inExportVO.getInoculationNum());
        weekExportDTO.setNucleicAcidDetectionInNum(inExportVO.getNucleicNum());
        return weekExportDTO;
    }

    public Double getDouble(String a, String b) {
        Double c = Double.valueOf(a) / Double.valueOf(b) * 100;
        String str = String.format("%.2f", c);
        return Double.valueOf(str);
    }

    public static void main(String[] args) {
        Date currentDay = DateUtil.parse("2022-10-21").toJdkDate();
        Date lastMonth = DateUtil.lastMonth().toJdkDate();
        if(currentDay.before(lastMonth)){
            System.out.println("过期了");
        }
    }
}
