package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.codec.Base64;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.core.SnowflakeIdWorker;
import com.yx.face.boot.rabbitConfig.config.SnQuenueConfig;
import com.yx.face.boot.restful.RestCode;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.*;
import com.yx.face.dao.*;
import com.yx.face.model.dto.FaceDeviceDTO;
import com.yx.face.model.dto.SaveFaceLogDto;
import com.yx.face.model.dto.smartlock.SmartlockUserChangePwdReq;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.*;
import com.yx.face.model.excel.FaceLogVoExcel;
import com.yx.face.model.query.FaceServerLogQu;
import com.yx.face.model.query.QueryUser;
import com.yx.face.model.search.FaceDeviceSearch;
import com.yx.face.model.search.FaceServerLogSearch;
import com.yx.face.model.vo.*;
import com.yx.face.model.vo.visitor.UserVisitorListDetailHealth;
import com.yx.face.model.vo.visitor.UserVisitorListDetailInfo;
import com.yx.face.model.vo.visitor.UserVisitorListDetailVisitor;
import com.yx.face.service.*;
import com.yx.face.service.feign.TBDeviceFaceService;
import com.yx.face.service.smartlock.AdminSmartlockDeviceService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.thymeleaf.util.ListUtils;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description: FaceServiceImpl <br>
 * @date: 2021/6/2 14:49 <br>
 * @author: PWB <br>
 */

@Slf4j
@Service
public class FaceServiceImpl implements FaceService {

    @Value("${face.service.tb}")
    private String ip;

    @Resource
    private FacePassDao facePassDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private FaceLogDao faceLogDao;

    @Resource
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private TBDeviceFaceService tbDeviceFaceService;

    @Autowired
    private AdminService adminService;

    @Resource
    private AdminDao adminDao;

    @Resource
    private AdminAuthDao adminAuthDao;

    @Resource
    private AdminPlaceInfoDao adminPlaceInfoDao;

    @Resource
    private FaceDeviceLogoDao faceDeviceLogoDao;

    @Resource
    private SystemConfigDao systemConfigDao;

    @Resource
    private TbDeviceWhiteListService tbDeviceWhiteListService;

    @Resource
    private UserVisitorListDao userVisitorListDao;

    @Resource
    private UserWhitelistDao userWhitelistDao;

    @Resource
    private PrivateAreaDeviceDao privateAreaDeviceDao;

    @Resource
    private UserInfoDao userInfoDao;

    @Autowired
    private FaceTBService faceTBService;

    @Resource
    private FaceDeviceTypeDao faceDeviceTypeDao;

    @Resource
    private ShortTermFaceLogDao shortTermFaceLogDao;

   

    @Value("${iot.tenant.list.query.url}")
    private String iotTenantListQueryUrl;

    @Value("${yinchuan.gongan.dataPushUrl}")
    private String yinChuanGongAnDataPushUrl;

    @Resource
    private FaceLogDetailDao faceLogDetailDao;
    @Lazy
    @Autowired
    private OpenIssuedService openIssuedService;

    @Resource
    private AdminSmartlockDeviceService adminSmartlockDeviceService;

    @Resource
    private ZoneDeviceRService zoneDeviceRService;

    @Override
    public Boolean authFaceDevice(Integer deviceId) {
        FaceDevice faceDevice = faceDeviceDao.selectByPrimaryKey(deviceId);
        if (faceDevice == null) {
            throw new ServiceException("无此设备");
        }
        Admin admin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
        faceDevice.setAuth(1);
        faceDevice.setUpdateTime(new Date());
        Boolean aBoolean = tbDeviceWhiteListService.addOrDeleteTbDeviceWhiteList(faceDevice.getSn(), "00", admin.getTenantPlatformId());
        if (aBoolean) {
            return faceDeviceDao.updateByPrimaryKeySelective(faceDevice) == 1;
        }
        return aBoolean;
    }

    @Override
    public Boolean stopFaceDevice(Integer deviceId) {
        FaceDevice faceDevice = faceDeviceDao.selectByPrimaryKey(deviceId);
        if (faceDevice == null) {
            throw new ServiceException("无此设备");
        }
        faceDevice.setAuth(0);
        faceDevice.setUpdateTime(new Date());
        Admin admin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
        Boolean aBoolean = tbDeviceWhiteListService.addOrDeleteTbDeviceWhiteList(faceDevice.getSn(), "01", admin.getTenantPlatformId());
        if (aBoolean) {
            return faceDeviceDao.updateByPrimaryKeySelective(faceDevice) == 1;
        }
        return aBoolean;
    }

    @Override
    public Boolean addFaceDevice(FaceDeviceDTO deviceDTO) {
        FaceDevice device = faceDeviceDao.getFaceDeviceBySn(deviceDTO.getSn());
        if (device != null) {
            if (!device.getAdminId().equals(0)) {
                if (!device.getAdminId().equals(deviceDTO.getAdminId())) {
                    throw new ServiceException("该设备已绑定别的账号");
                }
            }
            device.setDevType(deviceDTO.getDevType());
            device.setName(deviceDTO.getName());
            device.setEnable(deviceDTO.getEnable());
            device.setAdminId(deviceDTO.getAdminId());
            device.setAdminPlaceInfoId(deviceDTO.getAdminPlaceInfoId());
            //设备注册场所码
            if (Preconditions.isNotBlank(device.getAdminPlaceInfoId())) {
                AdminPlaceInfo adminPlaceInfo = adminPlaceInfoDao.selectByPrimaryKey(device.getAdminPlaceInfoId());
                Boolean register = openIssuedService.deviceRegister(device, adminPlaceInfo);
                device.setPlaceRegisterStatus(register);
            }
            return faceDeviceDao.updateByPrimaryKeySelective(device) == 1;
        }
        AdminVO info = adminService.getInfo(deviceDTO.getAdminId());
        if (info.getType() != 5 && info.getType() != 99) {
            throw new ServiceException("请将设备添加到正确的账号中");
        }
        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectByPrimaryKey(deviceDTO.getFaceDeviceTypeId());
        ClassUtil.checkData(faceDeviceType, "设备类型不存在");
        //如果是访客机设备只能添加到通用场景的区级账号，或者企业级账号下
        ClassUtil.throwBusinessException(
                FaceDeviceTypeEnum.FKJ.getCode().equals(faceDeviceType.getCode()) &&
                        AdminTypeEnum.DISTRICT.getCode().equals(info.getType()) &&
                        !AdminPlaceTypeEnum.COMMON_PLACE.getCode().equals(info.getPlaceType())
                , "访客机设备只能添加到通用场景的区级账号，或者企业级账号下");

        FaceDevice faceDevice = new FaceDevice();
        faceDevice.setAdminPlaceInfoId(deviceDTO.getAdminPlaceInfoId());
        faceDevice.setSn(deviceDTO.getSn());
        faceDevice.setAreaType(deviceDTO.getAreaType());
        faceDevice.setAdminAreaType(deviceDTO.getAdminAreaType());
        faceDevice.setName(deviceDTO.getName());
        faceDevice.setEnable(deviceDTO.getEnable());
        faceDevice.setAdminId(deviceDTO.getAdminId());
        faceDevice.setPass(deviceDTO.getPass());
        faceDevice.setRemark(deviceDTO.getRemark());
        faceDevice.setThirdPartyId(deviceDTO.getThirdPartyId());
        faceDevice.setFaceDeviceTypeId(deviceDTO.getFaceDeviceTypeId());
        faceDevice.setDevType(deviceDTO.getDevType());
        Date now = new Date();
        faceDevice.setUpdateTime(now);
        faceDevice.setCreateTime(now);
        faceDeviceDao.insertSelective(faceDevice);
        //设备注册场所码
        if (Preconditions.isNotBlank(faceDevice.getAdminPlaceInfoId())) {
            AdminPlaceInfo adminPlaceInfo = adminPlaceInfoDao.selectByPrimaryKey(faceDevice.getAdminPlaceInfoId());
            Boolean register = openIssuedService.deviceRegister(faceDevice, adminPlaceInfo);
            faceDevice.setPlaceRegisterStatus(register);
        }
        if (deviceDTO.getPrivateAreaId() != null) {
            PrivateAreaDevice privateAreaDevice = new PrivateAreaDevice();
            privateAreaDevice.setDeviceId(faceDevice.getId());
            privateAreaDevice.setPrivateAreaId(deviceDTO.getPrivateAreaId());
            privateAreaDevice.setAdminId(deviceDTO.getAdminId());
            privateAreaDevice.setCreateTime(now);
            privateAreaDeviceDao.insertSelective(privateAreaDevice);
        }

        //TODO 处理智能门锁业务 添加管理员
        handleSmartlockInitBiz(deviceDTO);

        return true;
    }

    /**
     * 处理智能门锁业务 添加管理员
     *
     * @param deviceDTO
     */
    private void handleSmartlockInitBiz(FaceDeviceDTO deviceDTO) {
        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectByPrimaryKey(deviceDTO.getFaceDeviceTypeId());
        if (Preconditions.isBlank(faceDeviceType)
                || Preconditions.isBlank(faceDeviceType.getCode())
                || !FaceDeviceTypeEnum.SMART_LOCK.getCode().equals(faceDeviceType.getCode())) {
            return;
        }

        //初始化管理员
        SmartlockUserChangePwdReq req = new SmartlockUserChangePwdReq();
        req.setDeviceSn(deviceDTO.getSn());
        adminSmartlockDeviceService.userChangePwd(req);
    }

    @Override
    public Boolean modifyFaceDevice(FaceDeviceDTO deviceDTO) {
        FaceDevice device = faceDeviceDao.selectByPrimaryKey(deviceDTO.getDeviceId());
        if (device == null) {
            throw new ServiceException("没有发现该设备");
        }
        FaceDevice faceDevice = new FaceDevice();
        BeanUtils.copyProperties(device, faceDevice);
        //设备注册场所码
        if (Preconditions.isNotBlank(deviceDTO.getAdminPlaceInfoId())) {
            AdminPlaceInfo adminPlaceInfo = adminPlaceInfoDao.selectByPrimaryKey(deviceDTO.getAdminPlaceInfoId());
            Boolean register = openIssuedService.deviceRegister(faceDevice, adminPlaceInfo);
            faceDevice.setPlaceRegisterStatus(register);
        } else {
            faceDevice.setPlaceRegisterStatus(Boolean.FALSE);
        }
        AdminVO info = adminService.getInfo(deviceDTO.getAdminId());
        if (info.getType().equals(AdminTypeEnum.DISTRICT.getCode())) {
            deviceDTO.setAreaType(1);
        }
        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectByPrimaryKey(deviceDTO.getFaceDeviceTypeId());
        //如果是访客机设备只能添加到通用场景的区级账号，或者企业级账号下
        ClassUtil.throwBusinessException(
                FaceDeviceTypeEnum.FKJ.getCode().equals(faceDeviceType.getCode()) &&
                        AdminTypeEnum.DISTRICT.getCode().equals(info.getType()) &&
                        !AdminPlaceTypeEnum.COMMON_PLACE.getCode().equals(info.getPlaceType())
                , "访客机设备只能添加到通用场景的区级账号，或者企业级账号下");
        faceDevice.setId(deviceDTO.getDeviceId());
        faceDevice.setAdminPlaceInfoId(deviceDTO.getAdminPlaceInfoId());
        faceDevice.setSn(deviceDTO.getSn());
        faceDevice.setAreaType(deviceDTO.getAreaType());
        faceDevice.setAdminAreaType(deviceDTO.getAdminAreaType());
        faceDevice.setName(deviceDTO.getName());
        faceDevice.setEnable(deviceDTO.getEnable());
        faceDevice.setPass(deviceDTO.getPass());
        faceDevice.setRemark(deviceDTO.getRemark());
        faceDevice.setUpdateTime(new Date());
        faceDevice.setAdminId(deviceDTO.getAdminId());
        faceDevice.setThirdPartyId(deviceDTO.getThirdPartyId());
        faceDevice.setFaceDeviceTypeId(deviceDTO.getFaceDeviceTypeId());
        faceDevice.setDevType(deviceDTO.getDevType());
        faceDeviceDao.updateByPrimaryKeySelective(faceDevice);
        //TODO 编辑设备
        addAutoTask(deviceDTO, device);

        /*修改账号删除原有的设备绑定*/
        if (!device.getAdminId().equals(deviceDTO.getAdminId())) {
            privateAreaDeviceDao.delete(new PrivateAreaDevice().setDeviceId(device.getId()).setAdminId(device.getAdminId()));
            //TODO。。。修改商户平台id
            tbDeviceWhiteListService.addOrDeleteTbDeviceWhiteList(faceDevice.getSn(), "00", info.getTenantPlatformId());
        }
        /*1.如果公有区域转为私有区域的话，需要删除设备上原有的人脸*/
        if (!device.getAreaType().equals(deviceDTO.getAreaType()) && deviceDTO.getAreaType().equals(FaceDeviceAreatypeEnum.PRIVATE_AREA.getCode())) {
            /*清空设备上的人员信息*/
            faceTBService.deleteBySn(faceDevice.getSn());
            if (deviceDTO.getPrivateAreaId() != null) {
                PrivateAreaDevice privateAreaDevice = new PrivateAreaDevice();
                privateAreaDevice.setDeviceId(faceDevice.getId());
                privateAreaDevice.setPrivateAreaId(deviceDTO.getPrivateAreaId());
                privateAreaDevice.setAdminId(deviceDTO.getAdminId());
                privateAreaDevice.setCreateTime(new Date());
                privateAreaDeviceDao.insertSelective(privateAreaDevice);
            }
        } else if (!device.getAreaType().equals(deviceDTO.getAreaType()) && deviceDTO.getAreaType().equals(FaceDeviceAreatypeEnum.PUBLIC_AREA.getCode())) {
            //2.如果私有转公有的话，需要删除设备绑定
            privateAreaDeviceDao.delete(new PrivateAreaDevice().setDeviceId(device.getId()).setAdminId(device.getAdminId()));
        } else {
            //3.如果公私有性质不变，公有则无变化，私有则需要查看设备所属区域有无变化
            if (deviceDTO.getAreaType().equals(FaceDeviceAreatypeEnum.PRIVATE_AREA.getCode())) {
                if (deviceDTO.getPrivateAreaId() == null) {
                    throw new ServiceException("请选择私有区域");
                }
                PrivateAreaDevice privateAreaDevice1 = new PrivateAreaDevice();
                privateAreaDevice1.setDeviceId(faceDevice.getId());
                privateAreaDevice1.setAdminId(deviceDTO.getAdminId());
                privateAreaDevice1 = privateAreaDeviceDao.selectOne(privateAreaDevice1);

                PrivateAreaDevice privateAreaDevice = new PrivateAreaDevice();
                privateAreaDevice.setDeviceId(faceDevice.getId());
                privateAreaDevice.setPrivateAreaId(deviceDTO.getPrivateAreaId());
                privateAreaDevice.setAdminId(deviceDTO.getAdminId());
                privateAreaDevice.setCreateTime(new Date());
                if (privateAreaDevice1 != null) {
                    if (!deviceDTO.getPrivateAreaId().equals(privateAreaDevice1.getPrivateAreaId())) {//设备所属的私有区域发生了变化
                        /*
                         * 1.清空设备上的人员信息
                         * 2.删除私有区域信息设备信息
                         * 3.添加新的私有区域设备信息
                         * 4.重新下发
                         * */
                        /*1.清空设备上的人员信息*/
                        faceTBService.deleteBySn(faceDeviceDao.selectByPrimaryKey(privateAreaDevice1.getDeviceId()).getSn());
                        /*2.删除私有区域信息设备信息*/
                        privateAreaDeviceDao.deleteByPrimaryKey(privateAreaDevice1.getId());
                        /*3.添加新的私有区域设备信息*/
                        privateAreaDeviceDao.insertSelective(privateAreaDevice);
                    }
                } else {
                    privateAreaDeviceDao.insertSelective(privateAreaDevice);
                }
            }
        }
        return true;
    }

    public void addAutoTask(FaceDeviceDTO deviceDTO, FaceDevice device) {
        if (!deviceDTO.getAdminId().equals(device.getAdminId())) {
            faceTBService.deleteBySn(device.getSn());
        }
    }

    @Override
    public Boolean deleteDevice(Integer deviceId) {
        FaceDevice device = faceDeviceDao.selectByPrimaryKey(deviceId);
        if (device == null) {
            throw new ServiceException("无此数据");
        }
        Admin admin = adminDao.selectByPrimaryKey(device.getAdminId());
        Boolean aBoolean = tbDeviceWhiteListService.addOrDeleteTbDeviceWhiteList(device.getSn(), "01", admin.getTenantPlatformId());
        if (!aBoolean) {
            throw new ServiceException("删除失败");
        }
        Integer zoneId = zoneDeviceRService.getZoneIdBySn(device.getAdminId(), device.getSn());
        if (Preconditions.isNotBlank(zoneId)) {
            throw new ServiceException("删除失败，请先将设备从区域中移除");
        }
        faceTBService.deleteBySn(device.getSn());
        privateAreaDeviceDao.delete(new PrivateAreaDevice().setDeviceId(deviceId));

        rabbitTemplate.convertAndSend(SnQuenueConfig.QUEUE_NAME, device.getSn());
        faceDeviceDao.deleteByPrimaryKey(deviceId);
        return true;
    }

    @Override
    public PageInfo<FaceDeviceVO> getFaceDevices(RestDTO<FaceDeviceSearch> dto) {
        FaceDeviceSearch data = dto.getData();
        AdminVO info = adminService.getInfo();
        info.setAdminIds(Lists.newArrayList(info.getAdminId()));
        if (AdminTypeEnum.ENTERPRISE.getCode().equals(info.getType())) {
            Admin admin = adminDao.selectOneByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getId, info.getTenantId())).build());
            if (admin != null) {
                info.getAdminIds().add(admin.getId());
            }
        }
        if (AdminTypeEnum.DISTRICT.getCode().equals(info.getType()) && (AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType()))) {
            List<Admin> admin = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, info.getAdminId())).build());
            if (admin != null) {
                for (Admin admin1 : admin) {
                    info.getAdminIds().add(admin1.getId());
                }
            }
        }


        Map<String, Object> map = new HashMap<>();
        map.put("adminVo", info);
        map.put("fde", data);
//        data.setAdminId(info.getAdminId());
//        data.setType(info.getType().intValue());
//        data.setProvinceId(info.getProvinceId());
//        data.setCityId(info.getCityId());
//        data.setAreaId(info.getAreaId());


        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<FaceDeviceVO> faceDevices = faceDeviceDao.getFaceDevices(map);

        for (FaceDeviceVO faceDevice : faceDevices) {
            PrivateArea privateArea = privateAreaDeviceDao.selectByDeviceIdAndAdminId(faceDevice.getDeviceId(), faceDevice.getAdminId());
            if (privateArea != null) {
                faceDevice.setPrivateAreaId(privateArea.getId());
                faceDevice.setPrivateAreaName(privateArea.getPrivateAreaName());
            }
        }
        return new PageInfo<>(faceDevices);
    }

    @Override
    public PageInfo<FacePassVO> getFacePasses(RestDTO<QueryUser> dto) {
        AdminVO info = adminService.getInfo();
        Map<String, Object> map = new HashMap<>();
        map.put("adminVo", info);
        map.put("query", dto.getData());
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<FacePassVO> list = facePassDao.getFacePasses(map);

        boolean isShowName = false;
        boolean isShowPhone = false;
        boolean isShowNumber = false;
        if (info.getNoHideCloumn() != null && info.getNoHideCloumn().size() > 0) {
            if (info.getNoHideCloumn().contains("1")) {
                isShowName = true;
            }
            if (info.getNoHideCloumn().contains("2")) {
                isShowPhone = true;
            }
            if (info.getNoHideCloumn().contains("3")) {
                isShowNumber = true;
            }
        }

        for (int i = 0; i < list.size(); i++) {
            if (list.get(i).getPhone() != null && isShowPhone) {
                list.get(i).setPhone(SMSOrIdCardUtils.hidePhone(list.get(i).getPhone()));
            }
            if (list.get(i).getUsername() != null && isShowName) {
                list.get(i).setUsername(SMSOrIdCardUtils.hideName(list.get(i).getUsername()));
            }
        }
        return new PageInfo<>(list);
    }

    @SneakyThrows
    @Override
    public PageInfo<FaceLogVO> getFaceLogs(RestDTO<FaceServerLogSearch> dto) {
        FaceServerLogSearch data = dto.getData();
        AdminVO info = adminService.getInfo();
        info.setAdminIds(Lists.newArrayList(info.getAdminId()));
        if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())) && AdminTypeEnum.DISTRICT.getCode().equals(info.getType())) {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, info.getAdminId())).build());
            info.getAdminIds().addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
        }
        Map<String, Object> map = new HashMap<>();
        map.put("adminVo", info);
        map.put("query", data);
        /*1.获取所有满足条件的adminId*/
        List<Long> adminIds = adminService.getAdminListSearch(map);
        map.put("adminIds", adminIds);
        /*2.获取所有满足条件的设备编号
         * 注：因为没有设备的相关查询条件所以不需要查询 所有满足条件的设备编号
         * */
        List<String> deviceSns = null;
        if (data.getPass() != null || data.getFaceDeviceTypeId() != null || data.getThirdPartyId() != null) {
            deviceSns = faceDeviceDao.getDeviceListSearch(data.getPass(), data.getFaceDeviceTypeId(), data.getThirdPartyId());
            if (CollectionUtil.isEmpty(deviceSns)) {
                return new PageInfo<>(new ArrayList<>());
            }
        }
        map.put("deviceSns", deviceSns);
        if (CollectionUtil.isEmpty(adminIds)) {
            return new PageInfo<>(new ArrayList<>());
        }
        /*增加自定义字段查询的逻辑*/
        if (CollUtil.isNotEmpty(data.getContents())) {
            List<ContextVO> contextVOS = data.getContents().stream().filter(p -> StrUtil.isNotBlank(p.getCustomNameVal())).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(contextVOS)) {
                //todo 这里本来应该根据长期或者短期的标志去查询不同的附表，但是我考虑到长期的附表可能数据量太大，就算了
                List<Long> logIdList = faceLogDetailDao.selectByContext(contextVOS, contextVOS.size());
                if (CollUtil.isEmpty(logIdList)) {
                    return new PageInfo<>(new ArrayList<>());
                }
                map.put("logIdList", logIdList);
            }
        }
        /*3.通过查询回来的id值 插入map中重新查询face_log表 进行单表查询*/
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<FaceLogVO> list;
        if (data.getShortTermFlag()) {
            list = shortTermFaceLogDao.getFaceLogsNew(map);
        } else {
            list = faceLogDao.getFaceLogsNew(map);
        }
        Admin admin = adminDao.selectByPrimaryKey(info.getAdminId());
        String content = admin.getContent();
        if (info.getType() == 5 && CollUtil.isNotEmpty(list)) {
            if (StringUtils.isNotEmpty(content)) {
                List<Long> logIdList = list.stream().map(FaceLogVO::getLogId).collect(Collectors.toList());
                List<FaceLogDetail> faceLogDetails = faceLogDetailDao.selectByExample(Example.builder(FaceLogDetail.class).where(WeekendSqls.<FaceLogDetail>custom().andIn(FaceLogDetail::getFaceLogId, logIdList)).build());

                if (CollUtil.isNotEmpty(faceLogDetails)) {
                    List<ContextVO> contextVOList = JSONArray.parseArray(admin.getContent(), ContextVO.class);
                    for (int i = 0; i < list.size(); i++) {
                        List<ContextVO> contextList = BeanUtil.copyToList(contextVOList, ContextVO.class);
                        FaceLogVO faceLogVO = list.get(i);
                        for (FaceLogDetail faceLogDetail : faceLogDetails) {
                            if (faceLogVO.getLogId().equals(faceLogDetail.getFaceLogId())) {
                                for (ContextVO contextVO : contextList) {
                                    if (faceLogDetail.getContextId().equals(contextVO.getId())) {
                                        contextVO.setCustomNameVal(faceLogDetail.getContextCustomNameVal());
                                    }
                                }
                            }
                        }
                        faceLogVO.setContents(contextList);
                    }
                }

//                    FaceLogVO faceLogVO = list.get(i);
//                    Integer type = faceLogVO.getRightno();
//                    String cardId = faceLogVO.getCardid();
//                    if (type == 104) {
//                        List<ContextVO> visitorContent = faceService.getContextByFaceLogVisitor(admin.getId(), cardId);
//                        if (!ObjectUtils.isEmpty(visitorContent)) {
//                            faceLogVO.setContents(visitorContent);
//                        } else if (!StringUtils.isEmpty(content)) {
//                            faceLogVO.setContents(JSONArray.parseArray(admin.getContent(), ContextVO.class));
//                        }
//                    } else if (type == 105) {
//                        faceService.getContextByFaceLogWhite(admin.getId(), cardId,faceLogVO);
//                    } else if (!StringUtils.isEmpty(content)) {
//                        faceLogVO.setContents(JSONArray.parseArray(admin.getContent(), ContextVO.class));
//                    }
//                }
            }
        }
        /*4.查询完成之后应当对其中的数据包括进行填入
         *   1.对admin中的数据字段进行填入
         *   2.对face_device中的数据字段进行填入
         * */

        /*4.1 获取所有adminId并把获取相关的admin*/
        List<Integer> adminList = list.stream().collect(Collectors.groupingBy(FaceLogVO::getAdminId)).keySet().stream().map(key -> key).collect(Collectors.toList());
        List<AdminVO> adminVOList = adminService.getByAdminlist(adminList);
        Map<Integer, AdminVO> adminVOMap = adminVOList.stream().collect(Collectors.toMap(AdminVO::getAdminId, AdminVO -> AdminVO));


        List<String> snList = list.stream().collect(Collectors.groupingBy(FaceLogVO::getDeviceSn)).keySet().stream().map(key -> key).collect(Collectors.toList());
        List<FaceDeviceVO> faceDeviceVOList = faceDeviceDao.getBySnList(snList);
        Map<String, FaceDeviceVO> faceDeviceVOMap = faceDeviceVOList.stream().collect(Collectors.toMap(FaceDeviceVO::getSn, FaceDeviceVO -> FaceDeviceVO));

        boolean isShowName = false;
        boolean isShowPhone = false;
        boolean isShowNumber = false;
        if (info.getNoHideCloumn() != null && info.getNoHideCloumn().size() > 0 && !data.getOutflag()) {
            if (info.getNoHideCloumn().contains("1")) {
                isShowName = true;
            }
            if (info.getNoHideCloumn().contains("2")) {
                isShowPhone = true;
            }
            if (info.getNoHideCloumn().contains("3")) {
                isShowNumber = true;
            }
        }

        for (FaceLogVO faceLog : list) {
            AdminVO adminVO = adminVOMap.get(faceLog.getAdminId());
            if (Objects.nonNull(adminVO)) {
                faceLog.setUsername(adminVO.getUsername());
            }
            FaceDeviceVO faceDeviceVO = faceDeviceVOMap.get(faceLog.getDeviceSn());
            if (Objects.nonNull(faceDeviceVO)) {
                faceLog.setDeviceName(faceDeviceVO.getName());
                faceLog.setOutType(faceDeviceVO.getPass());
                faceLog.setThirdPartyName(faceDeviceVO.getThirdPartyName());
                faceLog.setFaceDeviceTypeName(faceDeviceVO.getFaceDeviceTypeName());
            }
            if (CollectionUtils.isEmpty(faceLog.getContents())) {
                faceLog.setContents(new ArrayList<>());
            }

            if (!faceLog.getXcInfo().startsWith("去过中高风险地区")
                    && !faceLog.getJkmStatus().equals("01")
                    && !faceLog.getJkmStatus().equals("10")) {
                if (faceLog.getPhone() != null && isShowPhone) {
                    faceLog.setPhone(SMSOrIdCardUtils.hidePhone(faceLog.getPhone()));
                }
                if (faceLog.getCardid() != null && isShowNumber) {
                    faceLog.setCardid(SMSOrIdCardUtils.hideIdNumber(faceLog.getCardid()));
                }
                if (faceLog.getName() != null && isShowName) {
                    faceLog.setName(SMSOrIdCardUtils.hideName(faceLog.getName().trim()));
                }

            }


            if (StringUtils.isNotBlank(faceLog.getYmStatus())) {
                String[] split = faceLog.getYmStatus().split(",");
                if (split.length > 1) {
                    faceLog.setYmStatus(split[1]);
                } else if (split.length == 1) {
                    faceLog.setYmStatus(split[0]);
                } else {
                    faceLog.setYmStatus("0");
                }
            }
            if (StringUtils.isNotBlank(faceLog.getHsStatus())) {
                String[] split = faceLog.getHsStatus().split(";");
                if (split.length >= 1) {
                    faceLog.setHsStatus(split[0]);
                } else {
                    faceLog.setHsStatus(null);
                }
            }
        }

        return new PageInfo<>(list);
    }

    @Override
    public FaceLogVO getDetails(FaceServerLogQu qu, Integer adminId) {
        String pwd = PasswordUtils.buildPw(qu.getPassword());
        if (StringUtils.equals(qu.getPassword(), pwd)) {
            throw new ServiceException("密码有误哦~");
        }
        SystemConfigVO vo = systemConfigDao.sysConfiguration(adminId);
        if (vo == null) {
            throw new ServiceException("未配置此权限哦~");
        }
        if (vo.getStatus() != 1) {
            throw new ServiceException("此权限未开启哦~");
        }
        return faceLogDao.getDetails(qu.getId());
    }

    @Override
    public Boolean saveFaceLog(SaveFaceLogDto saveFaceLogDto) {
        String encode = AesUtils.decode("85014cc5281fb803e4068643442191a5", saveFaceLogDto.getData());
        if (StringUtils.isBlank(encode)) {
            log.error("解密数据为空");
            return false;
        }
        SaveFaceLogVo saveFaceLogVo = JSONObject.parseObject(encode, SaveFaceLogVo.class);
        if (StringUtils.isBlank(saveFaceLogVo.getIdCard()) || !IdcardValidatorUtil.isValidatedAllIdcard(saveFaceLogVo.getIdCard())) {
            log.error("身份证为空，或身份证不合法");
            return false;
        }
        FaceLog faceLog = BeanUtil.copyProperties(saveFaceLogVo, FaceLog.class);
        faceLog.setAdminId(0);
        faceLog.setCreateTime(new Date());
        faceLogDao.insert(faceLog);
        return true;
    }

    @Override
    public List<ContextVO> getContext(Integer id) {
        //传值
        UserVisitorList userVisitorList = userVisitorListDao.selectByPrimaryKey(id);
        if (ObjectUtils.isEmpty(userVisitorList)) {
            return null;
        }
        //传值
        //查询admin信息
        Admin admin = adminDao.selectByPrimaryKey(userVisitorList.getAdminId());
        List<ContextVO> titleList = Lists.newArrayList();
        if (admin != null && StrUtil.isNotEmpty(admin.getContent())) {
            titleList = JSONArray.parseArray(admin.getContent(), ContextVO.class);
        }

        if (CollectionUtils.isEmpty(titleList)) {
            return titleList;
        }

        Map<String, String> stringMap = Maps.newHashMap();
        if (userVisitorList != null && StrUtil.isNotEmpty(userVisitorList.getContent())) {
            List<ContextVO> contextVOList = JSONArray.parseArray(userVisitorList.getContent(), ContextVO.class);
            stringMap = contextVOList.stream().filter(v -> v != null && StrUtil.isNotEmpty(v.getCustomNameVal())).collect(Collectors.toMap(ContextVO::getId, ContextVO::getCustomNameVal, (v1, v2) -> v1));
        }

        for (ContextVO context : titleList) {
            context.setCustomNameVal(stringMap.get(context.getId()));
        }
        return titleList;
    }


    /*
     * 根据id获取自定义字段信息
     * */
    @Override
    public List<ContextVO> getContextById(Integer adminId) {
        String contextById = faceLogDao.getContextById(adminId);
        List<ContextVO> ContextVO = new ArrayList<>();
        if (StringUtils.isEmpty(contextById) || contextById.equals(" ")) {
            return ContextVO;
        }

        List<ContextVO> contextVOS = JSONArray.parseArray(contextById, ContextVO.class);
        for (ContextVO contextVO : contextVOS) {
            String configurationOptions = contextVO.getConfigurationOptions();
            if (StringUtils.isNotEmpty(configurationOptions)) {
                String[] split = configurationOptions.split("/");

                List<Map> list = new ArrayList<>();
                for (String s : split) {
                    Map map = new HashMap();
                    map.put("name", s);
                    list.add(map);
                }
                contextVO.setConfigurationOption(list);
            }
        }

        return contextVOS;
    }

    @Override
    public List<ContextVO> getContextByWhiteId(Integer id) {
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(id);
        //查询admin信息
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        List<ContextVO> titleList = Lists.newArrayList();
        if (admin != null && StrUtil.isNotEmpty(admin.getContent())) {
            titleList = JSONArray.parseArray(admin.getContent(), ContextVO.class);
        }

        if (CollectionUtils.isEmpty(titleList)) {
            return titleList;
        }

        Map<String, String> stringMap = Maps.newHashMap();
        if (userWhitelist != null && StrUtil.isNotEmpty(userWhitelist.getContent())) {
            List<ContextVO> contextVOList = JSONArray.parseArray(userWhitelist.getContent(), ContextVO.class);
            stringMap = contextVOList.stream().
                    filter(v -> v != null && StrUtil.isNotEmpty(v.getCustomNameVal()))
                    .collect(Collectors.toMap(ContextVO::getId, ContextVO::getCustomNameVal, (v1, v2) -> v1));
        }

        for (ContextVO context : titleList) {
            context.setCustomNameVal(stringMap.get(context.getId()));
            String configurationOptions = context.getConfigurationOptions();
            if (StringUtils.isNotEmpty(configurationOptions)) {
                String[] split = configurationOptions.split("/");

                List<Map> optionList = new ArrayList<>();
                for (String s : split) {
                    Map optionMap = new HashMap();
                    optionMap.put("name", s);
                    optionList.add(optionMap);
                }
                context.setConfigurationOption(optionList);
            }
        }
        return titleList;
    }

    @Override
    public Boolean getContextByAdminId(Integer adminId) {
        String contextById = faceLogDao.getContextById(adminId);
        if (StringUtils.isNotEmpty(contextById)) {
            return true;
        }
        return false;
    }

    @Override
    public Integer getUserVisitorList(Integer adminId, String idNumber) {
        return faceLogDao.getVisitorCount(adminId, idNumber);
    }

    @Override
    public Boolean updateWhiteContent(Integer adminId, String idNumber, List<ContextVO> list) {
        if (!ListUtils.isEmpty(list)) {
            String content = JSON.toJSONString(list);
            faceLogDao.updateContentWhite(content, adminId, idNumber);
            return true;
        } else {
            throw new ServiceException("添加自定义字段失败!");

        }
    }

    @Override
    public Boolean updateVisitorContent(Integer adminId, String idNumber, List<ContextVO> list) {
        if (!ListUtils.isEmpty(list)) {
            String content = JSON.toJSONString(list);
            faceLogDao.updateVisitorContent(content, adminId, idNumber);
            return true;
        } else {
            throw new ServiceException("添加自定义字段失败!");
        }
    }

    @Override
    public List<ContextVO> getContextByFaceLogVisitor(Integer adminId, String cardId) {

      /*  Integer id = faceLogDao.getContextByFaceLogVisitor(adminId, cardId);
        if(id ==null){
            List<ContextVO> list = new ArrayList<>();
            return list;
        }*/
        UserVisitorList userVisitorList = userVisitorListDao.getVisitorList(adminId, cardId);
        if (ObjectUtils.isEmpty(userVisitorList)) {
            List<ContextVO> list = new ArrayList<>();
            return list;
        }
        Admin admin = adminDao.selectByPrimaryKey(userVisitorList.getAdminId());
        List<ContextVO> titleList = Lists.newArrayList();
        if (admin != null && StrUtil.isNotEmpty(admin.getContent())) {
            titleList = JSONArray.parseArray(admin.getContent(), ContextVO.class);
        }

        if (CollectionUtils.isEmpty(titleList)) {
            return titleList;
        }

        Map<String, String> stringMap = Maps.newHashMap();
        if (userVisitorList != null && StrUtil.isNotEmpty(userVisitorList.getContent())) {
            List<ContextVO> contextVOList = JSONArray.parseArray(userVisitorList.getContent(), ContextVO.class);
            stringMap = contextVOList.stream().filter(v -> v != null && StrUtil.isNotEmpty(v.getCustomNameVal())).collect(Collectors.toMap(ContextVO::getId, ContextVO::getCustomNameVal, (v1, v2) -> v1));
        }

        for (ContextVO context : titleList) {
            context.setCustomNameVal(stringMap.get(context.getId()));
        }
        return titleList;

    }

    @Override
    public List<ContextVO> getContextByFaceLogWhite(Integer adminId, String cardId, FaceLogVO faceLogVO) {

        Integer id = faceLogDao.getContextByFaceLogWhite(adminId, cardId);
        if (null == id) {
            faceLogVO.setContents(new ArrayList<>());
            return new ArrayList<>();
        }
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(id);
        if (ObjectUtils.isEmpty(userWhitelist)) {
            faceLogVO.setContents(new ArrayList<>());
            return new ArrayList<>();
        } else {
            if (StrUtil.isEmpty(faceLogVO.getCardidex())) {
                faceLogVO.setCardidex(userWhitelist.getCardIdEx());
            }
        }
        //查询admin信息
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());

        List<ContextVO> titleList = Lists.newArrayList();
        if (admin != null && StrUtil.isNotEmpty(admin.getContent())) {
            titleList = JSONArray.parseArray(admin.getContent(), ContextVO.class);
        }

        if (CollectionUtils.isEmpty(titleList)) {
            faceLogVO.setContents(titleList);
            return titleList;
        }

        Map<String, String> stringMap = Maps.newHashMap();
        if (StrUtil.isNotEmpty(userWhitelist.getContent())) {
            List<ContextVO> contextVOList = JSONArray.parseArray(userWhitelist.getContent(), ContextVO.class);
            stringMap = contextVOList.stream().filter(v -> v != null && StrUtil.isNotEmpty(v.getCustomNameVal())).collect(Collectors.toMap(ContextVO::getId, ContextVO::getCustomNameVal, (v1, v2) -> v1));
        }

        for (ContextVO context : titleList) {
            context.setCustomNameVal(stringMap.get(context.getId()));
        }
        faceLogVO.setContents(titleList);
        return titleList;

    }

    @Override
    public FaceLogVO getFaceLog(Integer logId) {
        return faceLogDao.getFaceLog(logId);
    }

    @Override
    public UserInfoVO getUser(Integer id) {
        return faceLogDao.getUser(id);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void modifyUserInfo(Integer userId, Integer adminId) {
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
        if (userInfo == null) {
            throw new ServiceException("数据不匹配");
        }

        //获取白名单
        UserWhitelist userWhitelist = userWhitelistDao.getByIdNumberAndId(userInfo.getIdNumber(), null, adminId);
        if (userWhitelist == null && StrUtil.isNotEmpty(userInfo.getPhone())) {
            userWhitelist = userWhitelistDao.getByPhoneAndId(userInfo.getPhone(), null, adminId);
        }

        /*判断用户是否在常客中
         * 1.如果在常客 更新常客列表
         * 2.如果用户不在常客则以访客进行操作
         * */
        if (userWhitelist != null) {
            /*1.更新常客列表*/
            UserWhitelist uwl = new UserWhitelist();
            uwl.setStatus(1);
            uwl.setUpdateTime(DateUtil.date());
            uwl.setId(userWhitelist.getId());
            uwl.setPhone(userInfo.getPhone());
            uwl.setAvatar(userInfo.getAvatar());
            uwl.setIdNumber(userInfo.getIdNumber());
            userWhitelistDao.updateByPrimaryKeySelective(uwl);
        } else {
            /*2.添加用户至访客中*/
            UserVisitorList userVisitorList = userVisitorListDao.selectByIdNumberAndAdminId(userInfo.getIdNumber(), adminId);
            if (userVisitorList == null && StrUtil.isNotEmpty(userInfo.getPhone())) {
                userVisitorList = userVisitorListDao.getByPhoneAndId(userInfo.getPhone(), adminId);
            }
            UserVisitorList userVisitorOne = new UserVisitorList();
            if (userVisitorList == null) {
                userVisitorOne.setAdminId(adminId);
                userVisitorOne.setCreateTime(DateUtil.date());
                userVisitorOne.setPhone(userInfo.getPhone());
                userVisitorOne.setAvatar(userInfo.getAvatar());
                userVisitorOne.setIdNumber(userInfo.getIdNumber());
                userVisitorOne.setName(userInfo.getUsername());
                userVisitorOne.setUserId(userInfo.getId());
                userVisitorListDao.insert(userVisitorOne);
            } else {
                userVisitorOne.setAdminId(adminId);
                userVisitorOne.setId(userVisitorList.getId());
                userVisitorOne.setUpdateTime(DateUtil.date());
                userVisitorOne.setAvatar(userInfo.getAvatar());
                userVisitorOne.setUserId(userInfo.getId());
                userVisitorListDao.updateByPrimaryKeySelective(userVisitorOne);
            }
        }
    }

    @Override
    public Boolean clear() {
        List<UserWhitelistVO> whitelistVOList = faceLogDao.queryWhiteList();
        List<UserVisitorListVO> voListVoLIST = faceLogDao.queryVisitorList();
        if (!ListUtils.isEmpty(whitelistVOList)) {
            for (int i = 0; i < whitelistVOList.size(); i++) {
                List<ContextVO> contextVOS = JSONArray.parseArray(whitelistVOList.get(i).getContent(), ContextVO.class);
                String s = JSON.toJSONString(contextVOS);
                Long id = whitelistVOList.get(i).getId();
                faceLogDao.clearWhiteContent(id, s);
            }

        }
        if (!ListUtils.isEmpty(voListVoLIST)) {
            for (int i = 0; i < voListVoLIST.size(); i++) {
                List<ContextVO> contextVOS = JSONArray.parseArray(whitelistVOList.get(i).getContent(), ContextVO.class);
                String s2 = JSON.toJSONString(contextVOS);
                Long id = voListVoLIST.get(i).getId();
                faceLogDao.clearVisitor(id, s2);
            }
        }
        return true;
    }

    public static void main(String[] args) {
        String screenPhoto = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/upload/imagesnew/20221101/16672700056202154.jpg";
        screenPhoto = screenPhoto.replace("https://noise.hz-hanghui.com:8088/", "http://127.0.0.1:9100/");
        System.out.println(screenPhoto);
    }

    @Override
    public Boolean downFile(List<String> deviceSNs, List<String> screenPhotos, String fileName) {
        log.info("--->下发图片：sn:{}, photo:{}, fileName:{}", JSONObject.toJSONString(deviceSNs), JSONObject.toJSONString(screenPhotos), fileName);
        if (CollectionUtil.isEmpty(deviceSNs) || CollectionUtil.isEmpty(screenPhotos)) {
            return Boolean.FALSE;
        }
        for (String screenPhoto : screenPhotos) {
            screenPhoto = screenPhoto.replace("https://tx.hz-hanghui.com:8088/", "http://127.0.0.1:9100/").replace("https://test.hz-hanghui.com:18890/", "http://127.0.0.1:9100/");
            Map<String, Object> paramMap = Maps.newHashMap();
            paramMap.put("SN", deviceSNs);
            paramMap.put("type", "downFile");
            Map<String, Object> itemMap = Maps.newHashMap();
            itemMap.put("fileName", fileName);
            itemMap.put("fileData", DownLoad.tranUrlToBase64String(screenPhoto));
            paramMap.put("Data", itemMap);
            paramMap.put("taskId", SnowflakeIdWorker.getSnowId());
            try {
                Map<String, Object> result = tbDeviceFaceService.sendControl(paramMap);
                log.info("下发屏保照片响应结果{}", result);
                if (!String.valueOf(result.get("errCode")).equals("00")) {
                    log.error("下发屏保照片失败");
                    return Boolean.FALSE;
                }
            } catch (Exception e) {
                log.error("下发屏保照片失败", e);
                return Boolean.FALSE;
            }
        }
        return Boolean.TRUE;
    }

    @Override
    public List<TenantVo> doGetTenants() {
        try {
            String respStr = HttpUtil.post(iotTenantListQueryUrl, new JSONObject());
            log.info("--->获取商户平台列表返回结果：{}", JSONObject.toJSONString(respStr));
            RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            if (result.getCode() == RestCode.SUCCESS.getCode()) {
                return JSONObject.parseArray(JSONObject.toJSONString(result.getData()), TenantVo.class);
            }
        } catch (Exception e) {
        }
        return Collections.emptyList();
    }

    @Override
    public void pushUserData2YinChuanGongAn(ClockWayPushVO pushVO) {
        String IDNumber = pushVO.getCardid();
        if (Preconditions.isBlank(IDNumber)) {
            throw new ServiceException("身份证号有误");
        }

        String Name = pushVO.getName();
        String MobilePhone = pushVO.getPhone();
        String LocalTime = DateUtil.format(new Date(), DateUtils.YYYYMMDDHHMMSS);
        JSONObject objReqFaceObject = new JSONObject();
        objReqFaceObject.put("IDType", "111");

        objReqFaceObject.put("IDNumber", IDNumber);
        objReqFaceObject.put("Name", Name);
        objReqFaceObject.put("MobilePhone", MobilePhone);
        try {
            AdminAuth adminAuth = null;
            if (Preconditions.isNotBlank(pushVO.getAdminId())) {
                adminAuth = adminAuthDao.selectOneByExample(Example.builder(AdminAuth.class)
                        .where(WeekendSqls.<AdminAuth>custom()
                                .andEqualTo(AdminAuth::getAdminId, pushVO.getAdminId())
                                .andEqualTo(AdminAuth::getType, AdminAuthTypeEnum.GENERAL_CLOCK_WAY.getCode())
                                .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)).build());
            }
            //解密
            if (Preconditions.isNotBlank(adminAuth) && Preconditions.isNotBlank(adminAuth.getPrivateKey())) {
                objReqFaceObject.put("IDNumber", Sm4Util.decrypt(adminAuth.getPrivateKey(), IDNumber));
                objReqFaceObject.put("Name", Sm4Util.decrypt(adminAuth.getPrivateKey(), Name));
                objReqFaceObject.put("MobilePhone", Sm4Util.decrypt(adminAuth.getPrivateKey(), MobilePhone));
            }
        } catch (Exception e) {
            //TODO。。。不做处理
        }

        objReqFaceObject.put("LocalTime", LocalTime);
        String photo = pushVO.getPhoto();
        if (Preconditions.isNotBlank(photo)) {
            objReqFaceObject.put("FaceData", photo);
//            try {
//                String FaceData = Base64Util.imgToBase64(photo);
//                objReqFaceObject.put("FaceData", FaceData);
//            } catch (Exception e) {
//                log.error("--->pushUserData2YinChuanGongAn现场照片解析2Base64失败");
//            }
        }

        JSONObject objReq = new JSONObject();
        objReq.put("FaceObject", objReqFaceObject);
        //header User-Identify 设备编码
        String deviceSn = pushVO.getDeviceSn();
//        //缓存中获取银川公安数据推送地址
//        String pushDate2YinChuanGongAnUrl = (String) redisUtil.get("yinchuan::gongan::dataPushUrl");
        log.info("--->银川公安数据推送参数:{}", objReq.toJSONString());
        String sync = OkHttpUtils.builder().url(yinChuanGongAnDataPushUrl).addHeader("User-Identify", deviceSn).addHeader("ContentEncoding", "UTF-8").localPost(objReq.toJSONString()).sync();
        log.info("--->银川公安数据响应参数:{}", sync);
    }

    @Override
    public UserVisitorListDetailInfo getWhiteListFrom(Integer id) {
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(id);
        if (userWhitelist == null) {
            return new UserVisitorListDetailInfo();
        }
        UserWhitelistDetail userWhitelistDetail = userWhitelistDao.getWhiteListFrom(id);
        return this.getWhiteListFrom(userWhitelistDetail, userWhitelist);
    }

    private UserVisitorListDetailInfo getWhiteListFrom(UserWhitelistDetail userWhitelistDetail, UserWhitelist userWhitelist) {
        if (userWhitelistDetail == null) {
            userWhitelistDetail = new UserWhitelistDetail();
        }
        UserVisitorListDetailInfo userVisitorListDetailInfo = new UserVisitorListDetailInfo();
        AdminVO info = adminService.getInfo(userWhitelist.getAdminId());
        /*2.数据填充*/
        //1.自定义字段
        if (StringUtils.isNotEmpty(info.getContent())) {
            userVisitorListDetailInfo.setContentStatus(1);
            if (StringUtils.isNotEmpty(userWhitelist.getContent())) {
                userVisitorListDetailInfo.setContents(JSON.parseArray(userWhitelist.getContent(), ContextVO.class));
            } else {
                userVisitorListDetailInfo.setContents(new ArrayList<>());
            }
        }
        //2.访客机字段
        if (userWhitelist.getPushType() != null && userWhitelist.getPushType() == 3) {
            userVisitorListDetailInfo.setVisitorStatus(1);
            UserVisitorListDetailVisitor userVisitorListDetailVisitor = new UserVisitorListDetailVisitor();
            BeanUtils.copyProperties(userWhitelistDetail, userVisitorListDetailVisitor);
            userVisitorListDetailInfo.setUserVisitorListDetailVisitor(userVisitorListDetailVisitor);
        }
        //3.健康码信息
        if (!CollectionUtils.isEmpty(info.getThreeCodeList())) {
            userVisitorListDetailInfo.setThreeCodeList(info.getThreeCodeList());
            UserVisitorListDetailHealth userVisitorListDetailHealth = new UserVisitorListDetailHealth();
            BeanUtils.copyProperties(userWhitelistDetail, userVisitorListDetailHealth);
            userVisitorListDetailInfo.setUserVisitorListDetailHealth(userVisitorListDetailHealth);
        } else {
            userVisitorListDetailInfo.setThreeCodeList(new ArrayList<>());
        }
        return userVisitorListDetailInfo;
    }

    @Override
    public UserVisitorListDetailInfo getVisitorListFrom(Integer id) {
        /*1.数据查询*/
        UserVisitorList userVisitorList = userVisitorListDao.selectByPrimaryKey(id);
        if (userVisitorList == null) {
            return new UserVisitorListDetailInfo();
        }
        UserVisitorListDetail userVisitorListDetail = userVisitorListDao.selectByVisitorId(id);
        return this.getVisitorListFrom(userVisitorListDetail, userVisitorList);
    }

    private UserVisitorListDetailInfo getVisitorListFrom(UserVisitorListDetail userVisitorListDetail, UserVisitorList userVisitorList) {
        if (userVisitorListDetail == null) {
            userVisitorListDetail = new UserVisitorListDetail();
        }
        UserVisitorListDetailInfo userVisitorListDetailInfo = new UserVisitorListDetailInfo();
        AdminVO info = adminService.getInfo(userVisitorList.getAdminId());
        if (StringUtils.isNotEmpty(info.getContent())) {
            userVisitorListDetailInfo.setContentStatus(1);
            if (userVisitorList != null) {
                if (StringUtils.isNotEmpty(userVisitorList.getContent())) {
                    userVisitorListDetailInfo.setContents(JSON.parseArray(userVisitorList.getContent(), ContextVO.class));
                } else {
                    userVisitorListDetailInfo.setContents(new ArrayList<>());
                }
            } else {
                if (StringUtils.isNotEmpty(userVisitorListDetail.getContent())) {
                    userVisitorListDetailInfo.setContents(JSON.parseArray(userVisitorListDetail.getContent(), ContextVO.class));
                } else {
                    userVisitorListDetailInfo.setContents(new ArrayList<>());
                }
            }

        }
        //2.访客机字段
        if (userVisitorList != null && userVisitorList.getPushType() != null && userVisitorList.getPushType() == 3) {
            userVisitorListDetailInfo.setVisitorStatus(1);
            UserVisitorListDetailVisitor userVisitorListDetailVisitor = new UserVisitorListDetailVisitor();
            BeanUtils.copyProperties(userVisitorListDetail, userVisitorListDetailVisitor);
            userVisitorListDetailInfo.setUserVisitorListDetailVisitor(userVisitorListDetailVisitor);
        }
        //3.健康码信息
        if (!CollectionUtils.isEmpty(info.getThreeCodeList())) {
            userVisitorListDetailInfo.setThreeCodeList(info.getThreeCodeList());
            UserVisitorListDetailHealth userVisitorListDetailHealth = new UserVisitorListDetailHealth();
            BeanUtils.copyProperties(userVisitorListDetail, userVisitorListDetailHealth);
            userVisitorListDetailInfo.setUserVisitorListDetailHealth(userVisitorListDetailHealth);
        } else {
            userVisitorListDetailInfo.setThreeCodeList(new ArrayList<>());
        }

        return userVisitorListDetailInfo;
    }

    @Override
    public UserVisitorListDetailInfo getUserListFrom(Integer adminId, String cardId, String time) {
        Integer id = faceLogDao.getContextByFaceLogVisitor(adminId, cardId);
        if (id == null) {
            return new UserVisitorListDetailInfo();
        }
        UserVisitorList userVisitorList = userVisitorListDao.selectByPrimaryKey(id);
        if (userVisitorList == null) {
            return new UserVisitorListDetailInfo();
        }
        UserVisitorListDetail userVisitorListDetail = userVisitorListDao.selectVisitorFromById(id, time);
        return this.getVisitorListFrom(userVisitorListDetail, userVisitorList);
    }

    @Override
    public UserVisitorListDetailInfo getUserListFromTwo(Integer adminId, String cardId, String time) {
        Integer id = faceLogDao.getContextByFaceLogWhite(adminId, cardId);
        if (id == null) {
            return new UserVisitorListDetailInfo();
        }
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(id);
        if (userWhitelist == null) {
            return new UserVisitorListDetailInfo();
        }
        UserWhitelistDetail userWhitelistDetail = userWhitelistDao.selectWhiteListFromById(id, time);
        return this.getWhiteListFrom(userWhitelistDetail, userWhitelist);
    }

    @Override
    public List<ContextVO> getContextTwo(Integer id, Admin admin, UserVisitorListVO userVisitorListVO) {
        if (ObjectUtils.isEmpty(userVisitorListVO)) {
            return null;
        }
        //传值
        //查询admin信息
        List<ContextVO> titleList = Lists.newArrayList();
        if (admin != null && StrUtil.isNotEmpty(admin.getContent())) {
            titleList = JSONArray.parseArray(admin.getContent(), ContextVO.class);
        }

        if (CollectionUtils.isEmpty(titleList)) {
            return titleList;
        }

        Map<String, String> stringMap = Maps.newHashMap();
        if (userVisitorListVO != null && StrUtil.isNotEmpty(userVisitorListVO.getContent())) {
            List<ContextVO> contextVOList = JSONArray.parseArray(userVisitorListVO.getContent(), ContextVO.class);
            stringMap = contextVOList.stream().filter(v -> v != null && StrUtil.isNotEmpty(v.getCustomNameVal())).collect(Collectors.toMap(ContextVO::getId, ContextVO::getCustomNameVal, (v1, v2) -> v1));
        }

        for (ContextVO context : titleList) {
            context.setCustomNameVal(stringMap.get(context.getId()));
        }
        return titleList;
    }

    @Override
    public List<ContextVO> getContextByWhiteIdTwo(Integer id, Admin admin, UserWhitelistVO whitelistVO) {
        List<ContextVO> titleList = Lists.newArrayList();
        if (admin != null && StrUtil.isNotEmpty(admin.getContent())) {
            titleList = JSONArray.parseArray(admin.getContent(), ContextVO.class);
        }

        if (CollectionUtils.isEmpty(titleList)) {
            return titleList;
        }

        Map<String, String> stringMap = Maps.newHashMap();
        if (whitelistVO != null && StrUtil.isNotEmpty(whitelistVO.getContent())) {
            List<ContextVO> contextVOList = JSONArray.parseArray(whitelistVO.getContent(), ContextVO.class);
            stringMap = contextVOList.stream().
                    filter(v -> v != null && StrUtil.isNotEmpty(v.getCustomNameVal()))
                    .collect(Collectors.toMap(ContextVO::getId, ContextVO::getCustomNameVal, (v1, v2) -> v1));
        }

        for (ContextVO context : titleList) {
            context.setCustomNameVal(stringMap.get(context.getId()));
        }
        return titleList;
    }

    @Override
    public UserInfoIdentificationVO queryFaceSuccess(String taskId) {
        Long userId = facePassDao.getFacePassByTaskId(taskId);
        UserInfoIdentificationVO vo = new UserInfoIdentificationVO();
        if (userId == null) {
            return vo;
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
        if (ObjectUtil.isNull(userInfo)) {
            return vo;
        }
        BeanUtil.copyProperties(userInfo, vo);
        vo.setUserId(userId);
        vo.setStatus(Boolean.TRUE);
        return vo;
    }

    @Override
    public List<FaceLogVoExcel> getFaceLogsExcel(FaceServerLogSearch data, List<FaceLogVO> faceLogVOList) {
        AdminVO info = adminService.getInfo();
        if (info.getIsOut() != 1) {
            throw new ServiceException("你没有导出的权限");
        }
        RestDTO<FaceServerLogSearch> dto = new RestDTO<>();
        dto.setPageNum(0);
        dto.setPageSize(50000);
        data.setOutflag(true);
        dto.setData(data);
        PageInfo<FaceLogVO> pageInfo = this.getFaceLogs(dto);
        List<FaceLogVO> list = pageInfo.getList();
        //拷贝对象传递使用
        faceLogVOList.addAll(JSON.parseArray(JSON.toJSONString(list), FaceLogVO.class));

        List<FaceLogVoExcel> faceLogs = new ArrayList<>();

        for (int i = 0; i < list.size(); i++) {
            FaceLogVoExcel fle = new FaceLogVoExcel();
            String phone = list.get(i).getPhone();
            String cardid = list.get(i).getCardid();
            String name = list.get(i).getName();
            if (info.getIsHideImportant() == 1) {
                if (list.get(i).getPhone() != null) {
                    list.get(i).setPhone(SMSOrIdCardUtils.hidePhone(phone));
                }
                if (list.get(i).getCardid() != null) {
                    list.get(i).setCardid(SMSOrIdCardUtils.hideIdNumber(cardid));
                }
                if (list.get(i).getName() != null) {
                    list.get(i).setName(SMSOrIdCardUtils.hideName(name.trim()));
                }
            }
            FaceLogVO fv = list.get(i);
            fle.setDeviceSn(fv.getDeviceSn() == null ? null : fv.getDeviceSn());
            fle.setDeviceName(fv.getDeviceName() == null ? null : fv.getDeviceName());
            fle.setName(fv.getName() == null ? null : fv.getName());
            fle.setPhone(fv.getPhone() == null ? null : fv.getPhone());
            fle.setCardid(fv.getCardid() == null ? null : fv.getCardid());
            fle.setPhoto(fv.getPhoto() == null ? null : fv.getPhoto());
            fle.setRightNoInt(fv.getRightno());
            fle.setShowPhone(phone);
            fle.setShowCardid(cardid);
            fle.setShowName(name);
            fle.setCardidex(fv.getCardidex());
            if (fv.getStatus() != null) {
                fle.setStatus(fv.getStatus() ? "有效" : "无效");
            }
            if (fv.getReasonType() != null) {
                FaceLogStatusEnum faceLogStatusEnum = FaceLogStatusEnum.getByCode(fv.getReasonType());
                fle.setReasonType(ObjectUtil.isNotNull(faceLogStatusEnum) ? faceLogStatusEnum.getDesc() : "");
            }
            if (fv.getOutType() != null) {
                int outType = fv.getOutType();
                if (outType == 0) {
                    fle.setOutType("通用");
                } else if (outType == 1) {
                    fle.setOutType("进门");
                } else if (outType == 2) {
                    fle.setOutType("出门");
                } else {
                    fle.setOutType("未知");
                }
            }

            if (fv.getVerifyType() != null) {
                Integer verifyType = fv.getVerifyType();
                if (verifyType == 0) {
                    fle.setVerifyType("人脸");
                } else if (verifyType == 1) {
                    fle.setVerifyType("卡");
                } else if (verifyType == 2) {
                    fle.setVerifyType("身份证");
                } else if (verifyType == 3) {
                    fle.setVerifyType("二维码");
                } else if (verifyType == 4) {
                    fle.setVerifyType("远程");
                } else if (verifyType == 6) {
                    fle.setVerifyType("IC卡+人脸");
                } else {
                    fle.setVerifyType("未知");
                }
            }

            if (fv.getRightno() != null) {
                Integer rightno = fv.getRightno();
                if (rightno == 106) {
                    fle.setRightno("管理员");
                } else if (rightno == 105) {
                    fle.setRightno("操作员");
                } else if (rightno == 104) {
                    fle.setRightno("普通员工");
                } else if (rightno == 103) {
                    fle.setRightno("访客");
                } else if (rightno == 102) {
                    fle.setRightno("禁止");
                } else if (rightno == 101) {
                    fle.setRightno("黑名单");
                } else {
                    fle.setRightno("未知");
                }
            }
            fle.setCreateTime(fv.getCreateTime());

            faceLogs.add(fle);
        }

        return faceLogs;
    }

    /**
     * @return
     */
    @Override
    public List<FaceDeviceVO> getFaceDeviceList() {
        AdminVO info = adminService.getInfo();
        info.setAdminIds(Lists.newArrayList(info.getAdminId()));
        if (info.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            info.getAdminIds().add(info.getTenantId());
        }
        Map<String, Object> map = new HashMap<>();
        map.put("adminVo", info);
        return faceDeviceDao.getFaceDeviceList(map);
    }

    @Override
    public List<FaceDeviceVO> getFaceDeviceListByAdminId(Admin admin) {
        AdminVO info = adminService.getInfo(admin.getId());
        info.setAdminIds(Lists.newArrayList(info.getAdminId()));
        if (info.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            info.getAdminIds().add(info.getTenantId());
        }
        Map<String, Object> map = new HashMap<>();
        map.put("adminVo", info);
        return faceDeviceDao.getFaceDeviceList(map);
    }

    /**
     * 设备 log 上传
     *
     * @param file
     * @param adminId
     * @return
     */
    @Override
    public String addFaceDeviceLogImg(InputStream file, Integer adminId) {
        String path = null;
        //按指定大小把图片进行缩和放（会遵循原图高宽比例）
        //此处把图片压成60*60的缩略图
        try {
            BufferedImage image = Thumbnails.of(file).size(60, 60).asBufferedImage();//变为400*300,遵循原图比例缩或放到400*某个高度
            //输出流
            ByteArrayOutputStream stream = new ByteArrayOutputStream();
            ImageIO.write(image, "png", stream);
            String base64 = Base64.encode(stream.toByteArray());
            stream.flush();
            stream.close();
            String name = UUID.randomUUID().toString().concat(".jpg");

            String filePath = FileUtils.getDirectory(Constants.UPLOAD_IMAGE);
            path = AddressUtil.address + FileUtils.transformBase64Image(base64, filePath, name);
            //判断logo 是否存在
            FaceDeviceLogo faceDeviceLogo = faceDeviceLogoDao.selectList(adminId);
            if (faceDeviceLogo == null) {
                faceDeviceLogo = new FaceDeviceLogo();
                faceDeviceLogo.setLogoImg(path);
                faceDeviceLogo.setAdminId(adminId);
                faceDeviceLogoDao.insert(faceDeviceLogo);
            } else {
                faceDeviceLogo.setLogoImg(path);
                faceDeviceLogoDao.updateByPrimaryKeySelective(faceDeviceLogo);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return path;
    }

    @Override
    public Boolean issuedAll(Integer type, Integer adminId, String sn) {
        List<String> list = new ArrayList<>();
        AdminVO info = adminService.getInfo();
        if (info == null) {
            list.add("数据匹配失败未找到此账号");
            throw new ServiceException(list.toString());
        }
        //设备列表
        List<FaceDevice> listSn = faceDeviceDao.selectSnList(type, sn, info.getProvinceId(), adminId);

        FaceDeviceLogo faceDeviceLogo = faceDeviceLogoDao.selectList(info.getAdminId());
        if (faceDeviceLogo == null) {
            list.add("LOGO失效下发失败请重新上传下发");
            throw new ServiceException(list.toString());
        }
        String logoImg = faceDeviceLogo.getLogoImg();

        log.info("【图片路径】：{}", logoImg);
        String base64 = null;
        base64 = Base64Util.encode(Base64Util.getByteArrayFromUrl(logoImg));
        if (StringUtils.isBlank(base64)) {
            list.add("LOGO失效下发失败请重新上传下发");
            throw new ServiceException(list.toString());
        }

        for (FaceDevice faceDevice : listSn) {
            //停用-0|启用-1
            Integer enable = faceDevice.getEnable();
            //离线-0|在线-1
            Integer online = faceDevice.getOnline();
            //未授权-0|已授权-1
            Integer auth = faceDevice.getAuth();
            if (enable == 0 || online == 0 || auth == 0) {
                list.add(faceDevice.getSn() + ":离线未授权停用状态LOGO下发失败请检查设备状态");
                continue;
            } else {
                List<String> stringList = new ArrayList<>();
                stringList.add(faceDevice.getSn());
                JSONObject img = new JSONObject();
                img.put("logo", base64);
                JSONObject json = new JSONObject();
                json.put("SN", stringList);
                json.put("type", "downLogo");
                json.put("taskId", faceDevice.getId());
                json.put("Data", img);
                log.info("log下发参数； {}", json);
                String sync = OkHttpUtils.builder().url(ip + "/Face/sendControl").localPost(json.toJSONString()).sync();
                log.info("log下发返回参数；{}", sync);
                JSONObject jsonObject = JSONObject.parseObject(sync);
                String msg = jsonObject.getString("msg");
                if ("命令下发成功".equals(msg)) {
                    log.info("logo命令下发成功" + jsonObject);
                } else {
                    list.add(faceDevice.getSn() + ":离线未授权停用状态LOGO下发失败请检查设备状态");
                }
            }
        }
        if (!list.isEmpty()) {
            throw new ServiceException(list.toString());
        }
        return true;
    }

    @Override
    public String getLogo(Integer adminId) {
        FaceDeviceLogo faceDeviceLogo = faceDeviceLogoDao.selectList(adminId);
        if (faceDeviceLogo == null) {
            return "";
        }
        return faceDeviceLogo.getLogoImg();
    }

    /**
     * 设备重启动
     *
     * @param
     * @return
     */
    @Override
    public Boolean doReboot(String sn) {
        FaceDevice faceDeviceBySn = faceDeviceDao.getFaceDeviceBySn(sn);
        if (faceDeviceBySn == null) {
            throw new ServiceException("此设备不存在");
        }
        List<String> sns = new ArrayList<>();
        sns.add(sn);
        Map<String, Object> map = Maps.newHashMap();
        Map<String, Object> data = Maps.newHashMap();
        data.put("rebootType", "2");
        data.put("time", "");
        map.put("SN", sns);
        map.put("taskId", System.currentTimeMillis());
        map.put("type", "reboot");
        map.put("Data", data);
        try {
            Map<String, Object> sendControl = tbDeviceFaceService.sendControl(map);
            log.info("重启设备返回参数；{}", sendControl);
            if (sendControl.isEmpty()) {
                throw new ServiceException("设备重启失败请检查是否离线");
            }
            if (!"00".equals((String) sendControl.get("errCode"))) {
                //设备不在线 重试 3次
//                    Boolean retry = triggerRetry(map);
                throw new ServiceException("设备重启失败请检查是否离线");
            }
        } catch (Exception e) {
            log.info("下发失败 原因 {}", e.getMessage());
            throw new ServiceException("设备重启失败请检查是否离线");
            //设备不在线 重试 3次
//            Boolean retry = triggerRetry(map);
        }
        return true;
    }

    public String getRandom() {
        Integer i = (int) (Math.random() * 1000 - 1);
        return String.valueOf(i);
    }
}

