package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.core.SnowflakeIdWorker;
import com.yx.face.boot.uitls.*;
import com.yx.face.dao.*;
import com.yx.face.model.dto.IssueAllDTO;
import com.yx.face.model.dto.RetryPushLogDto;
import com.yx.face.model.dto.iot.IotUserWhitelistDto;
import com.yx.face.model.dto.issue.IssueDeviceItemCallbackReq;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.*;
import com.yx.face.model.excel.UserInfoMsgExcel;
import com.yx.face.model.redis.InOutStatisticsBean;
import com.yx.face.model.vo.*;
import com.yx.face.model.vo.third.InterfAddrJsonVo;
import com.yx.face.model.vo.visitor.UserVisitorListDetailInfo;
import com.yx.face.model.vo.visitor.UserVisitorListDetailVisitor;
import com.yx.face.server.DingTalkServer;
import com.yx.face.server.dto.DingTalkRecordUpload;
import com.yx.face.service.*;
import com.yx.face.service.component.HandheldTerminalsService;
import com.yx.face.service.component.RuiYingService;
import com.yx.face.service.feign.RemoteRYDeviceDownloadFaceService;
import com.yx.face.service.feign.TBDeviceFaceService;
import com.yx.face.service.issue.HhfaceIssueService;
import com.yx.face.service.issue.IssueDeviceService;
import com.yx.face.service.middleware.MiddlewareService;
import com.yx.face.service.retry.RetryableService;
import com.yx.face.service.smartlock.AdminSmartlockDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

/**
 * @description: FaceTBServiceImpl <br>
 * @date: 2021/6/3 10:40 <br>
 * @author: PWB <br>
 */
@Slf4j
@Service
public class FaceTBServiceImpl implements FaceTBService {

    private final static Long SLEEP_MS = 60000L;

    @Resource
    private FacePassDao facePassDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private FaceRequestLogDao faceRequestLogDao;

    @Autowired
    @Lazy
    private AsyncTriggerRetryImpl asyncTriggerRetry;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private TBDeviceFaceService tbDeviceFaceService;

   

    @Autowired
    private AdminService adminService;

    @Resource
    private FaceTaskDao faceTaskDao;

    @Autowired
    @Lazy
    private HandheldTerminalsImpl handheldTerminalsImpl;

    @Autowired
    RabbitTemplate rabbitTemplate;

    @Autowired
    private HandheldTerminalsService handheldTerminalsService;
    @Autowired
    private RuiYingService ruiYingService;

    @Autowired
    private RemoteRYDeviceDownloadFaceService remoteRYDeviceDownloadFaceService;
    @Autowired
    private DingTalkServer dingTalkServer;
    @Resource
    private TaskInfoDao taskInfoDao;
    @Resource
    private Executor executor;
    @Resource
    private TaskInfoService taskInfoService;
    @Resource
    @Lazy
    private UserWhitelistService userWhitelistService;
    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private FaceDeviceTypeDao faceDeviceTypeDao;
    @Resource
    private UserWhitelistDao userWhitelistDao;
    @Autowired
    private SpiIotUserWhitelistService spiIotUserWhitelistService;

    @Resource
    private AdminAuthDao adminAuthDao;

    @Lazy
    @Autowired
    private FaceService faceService;

    @Resource
    private AdminSmartlockDeviceService adminSmartlockDeviceService;

    @Autowired
    private ZoneDeviceRService zoneDeviceRService;

    @Resource
    private MiddlewareService middlewareService;

    @Resource
    private IssueDeviceService issueDeviceService;

    @Resource
    private HhfaceIssueService hhfaceIssueService;

    @Resource
    private RetryableService retryableService;

    @Override
    public Boolean delFace(List<Long> userIds, Integer adminId) {
        List<FaceDeviceLessVO> snList = faceDeviceDao.getSnList(adminId);
        if (CollectionUtils.isEmpty(snList)) {
            return true;
        }
        for (FaceDeviceLessVO faceDeviceLessVO : snList) {
            this.delFaceBySn(faceDeviceLessVO.getSn(), userIds);
        }
        return true;
    }

    @Override
    public Map<String, Object> deleteBySn(String sn) {
        hhfaceIssueService.sendDeleteBySnMsg(sn, null);
//        return this.deleteBySn(sn, null);
        return null;
    }


    @Override
    public Map<String, Object> deleteBySn(String sn, TaskInfo taskInfo) {
        FaceDeviceLessVO bySn = faceDeviceDao.getBySn(sn);
        String faceDeviceTypeCode = bySn.getFaceDeviceTypeCode();
        //TODO。。。 兼容IOT设备
        spiIotUserWhitelistService.delBySn(sn);
        if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.HHFACE.getCode())) {
            //HHFACE人脸信息删除
            issueDeviceService.cleanUser(sn);
            return null;
        }
        //TODO 离线任务
        //增加离线任务  离线并且不是手持的设备需要增加为离线任务
        if (bySn != null && bySn.getOnline().equals(0)) {//设备离线
            //如果不是手持终端需要生成离线任务
            if (!bySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())
                    && !bySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.SMART_LOCK.getCode())
                    && !bySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.BLUETOOTH_SMART_LOCK.getCode())
                    && !bySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.ZFB.getCode())
                    && !bySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.HHFACE.getCode())) {
                if (taskInfo == null) {//生成离线任务
                    taskInfoService.saveDevice(bySn.getSn(), bySn.getFaceDeviceTypeCode());
                    log.info("生成离线任务：设备不在线，生成了全量删除离线任务");
                } else {
                    Integer retry = taskInfo.getRetry();
                    if (retry != null && retry >= 1) {
                        retry = retry - 1;
                        taskInfo.setRetry(retry);
                        taskInfoDao.updateByPrimaryKeySelective(taskInfo);
                        log.info("离线任务次数减1");
                    } else {
                        taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
                        log.info("删除离线任务");
                    }

                }
                return null;
            }
        }

        Map<String, Object> map = Maps.newHashMap();
        FaceRequestLog requestLog = intoFaceRequestLog("删除人员人脸信息", "/delUser");
        map.put("SN", sn);
        map.put("taskId", requestLog.getId());
        map.put("Data", "all");
        try {
            if (bySn == null) {
                faceDeviceTypeCode = taskInfo.getCode();
            }
            if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.TB.getCode())) {
                //天波人脸删除
                tbDeviceFaceService.delUser(map);
            } else if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())) {
                //手持终端人脸信息删除
                handheldTerminalsService.delPerson(sn);
            } else if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.FOUR_INCH_SCREEN.getCode())) {
                //锐颖人脸信息删除
                ruiYingService.delPerson(sn);
            } else if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.SMART_LOCK.getCode())) {
                //智能门锁人脸信息删除
                adminSmartlockDeviceService.cleanSmartlockUser(sn);
            } else if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.BLUETOOTH_SMART_LOCK.getCode())) {
                //蓝牙智能门锁人脸信息删除
                middlewareService.cleanUser(sn);
            }

        } catch (Exception e) {
            log.info("删除人脸异常 {}", e.getMessage());
        }
        if (taskInfo != null) {
            taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
        }
        return null;
    }

    @Override
    public Boolean delFaceBySn(String sn, List<Long> userIds) {
        hhfaceIssueService.sendDelFaceBySnMsg(sn, userIds, null);
//        return this.delFaceBySn(sn, userIds, null);
        return null;
    }

    @Override
    public Boolean delFaceBySn(String sn, List<Long> userIds, TaskInfo taskInfo) {
        FaceDeviceLessVO faceDevice = faceDeviceDao.getBySn(sn);
        if (faceDevice == null || CollUtil.isEmpty(userIds)) {
            return true;
        }

        //TODO。。。兼容IOT删除人员
        spiIotUserWhitelistService.delUserFromIot(sn, userIds);
        //TODO 离线任务
        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectByPrimaryKey(faceDevice.getFaceDeviceTypeId());
        //增加离线任务  离线并且不是手持的设备需要增加为离线任务
        if (faceDevice.getOnline().equals(0)) {//设备离线
            //如果不是手持终端/智能门锁需要生成离线任务
            if (!faceDeviceType.getCode().equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())
                    && !faceDeviceType.getCode().equals(FaceDeviceTypeEnum.SMART_LOCK.getCode())
                    && !faceDeviceType.getCode().equals(FaceDeviceTypeEnum.BLUETOOTH_SMART_LOCK.getCode())
                    && !faceDeviceType.getCode().equals(FaceDeviceTypeEnum.HHFACE.getCode())) {
                if (taskInfo == null) {//生成离线任务
                    taskInfo = taskInfoService.save(faceDevice.getSn(), userIds);
                } else {
                    Integer retry = taskInfo.getRetry();
                    if (retry != null && retry >= 1) {
                        retry = retry - 1;
                        taskInfo.setRetry(retry);
                        taskInfoDao.updateByPrimaryKeySelective(taskInfo);
                    } else {
                        taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
                    }
                }
                log.info("生成离线任务：设备不在线，生成了离线任务");
                return Boolean.TRUE;
            }
        }


        FaceRequestLog requestLog = intoFaceRequestLog("删除人员人脸信息", "/delUser");
        String faceDeviceTypeCode = faceDevice.getFaceDeviceTypeCode();
        Map<String, Object> result = null;
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(userIds.size());
        Map<String, Object> dataItem;
        for (Long userId : userIds) {
            dataItem = Maps.newHashMapWithExpectedSize(1);
            dataItem.put("pin", userId);
            data.add(dataItem);
        }
        params.put("SN", sn);
        params.put("data", data);
        params.put("taskId", requestLog.getId());

        if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.TB.getCode())) {
            //天波人脸删除
            result = tbDeviceFaceService.delUser(params);
        } else if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())) {
            //手持终端人脸信息删除
            handheldTerminalsService.delPerson(sn, userIds);
        } else if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.FOUR_INCH_SCREEN.getCode())) {
            //锐颖人脸信息删除
            result = remoteRYDeviceDownloadFaceService.delUser(params);
        } else if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.SMART_LOCK.getCode())) {
            //智能门锁人脸信息删除
            adminSmartlockDeviceService.delSmartlockUser(sn, userIds);
        } else if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.BLUETOOTH_SMART_LOCK.getCode())) {
            //蓝牙智能门锁人脸信息删除
            middlewareService.delUser(sn, userIds);
        } else if (faceDeviceTypeCode.equals(FaceDeviceTypeEnum.HHFACE.getCode())) {
            //HHFACE人脸信息删除
            issueDeviceService.delFaceDataFromDevice(userIds, sn);
        }
        log.info("删除人员人脸信息downFace = {}", result);
        if (taskInfo != null) {
            taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
        }

        return true;
    }

    /*=============================================【以下为推送】====================================================*/
    @Override
    public Map<String, Object> connect(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        if (dto == null) {
            return result;
        }
        String sn = null;
        String version = null;
        String devIP = null;
        Integer userCount = null;
        Integer swipeCardCount = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        if (dto.get("version") != null) {
            version = dto.get("version").toString();
        }
        if (dto.get("devIP") != null) {
            devIP = dto.get("devIP").toString();
        }
        if (dto.get("swipeCardCount") != null) {
            swipeCardCount = (Integer) dto.get("swipeCardCount");
        }
        if (dto.get("userCount") != null) {
            userCount = (Integer) dto.get("userCount");
        }
        log.info("==========================开始执行离线任务==============================");
        String finalSn = sn;
        executor.execute(() -> {
            executeOfflineTask(finalSn);
        });
        log.info("==========================结束执行离线任务==============================");
        Integer deviceId = faceDeviceDao.getIdBySn(sn);
        FaceDevice device = new FaceDevice();
        device.setId(deviceId);
        device.setOnline(1);
        device.setSn(sn);
        device.setPassCount(swipeCardCount);
        device.setUserCount(userCount);
        device.setAppVersion(version);
        device.setInterIp(devIP);
        Date now = new Date();
        device.setUpdateTime(now);
        if (deviceId == null) {
            return result;
        }
        faceDeviceDao.updateByPrimaryKeySelective(device);
        return result;
    }

    public void executeOfflineTask(String sn) {
        List<TaskInfo> onlineTaskList = taskInfoService.getOnlineTaskList(sn);
        if (CollectionUtil.isNotEmpty(onlineTaskList)) {
            try {
                onlineTaskList.forEach(e -> {
                    e.setStatus(AutoTaskInfoStatusEnum.HANDLED.getStatus());
                    e.setExecuteResult("离线任务正在执行");
                    taskInfoDao.updateByPrimaryKey(e);
                });
                log.info("延迟60秒执行....");
                Thread.sleep(SLEEP_MS);
                for (TaskInfo taskInfo : onlineTaskList) {
                    if (taskInfo.getType().equals(TaskTypeEnum.ALL_ISSUE.getType())) {
                        IssueAllDTO issueAllDTO = JSON.parseObject(taskInfo.getBody(), IssueAllDTO.class);
                        userWhitelistService.addSnIssued(issueAllDTO, taskInfo);
                    } else if (taskInfo.getType().equals(TaskTypeEnum.SINGLE_ISSUE.getType())) {
                        DownFaceNew downFace = JSON.parseObject(taskInfo.getBody(), DownFaceNew.class);
//                        this.downUser(downFace, taskInfo);
                        hhfaceIssueService.sendDownUserMsg(downFace, taskInfo);
                    } else if (taskInfo.getType().equals(TaskTypeEnum.SINGLE_REMOVE.getType())) {
                        List<Long> list = JSON.parseArray(taskInfo.getBody(), Long.class);
//                        this.delFaceBySn(taskInfo.getSn(), list, taskInfo);
                        hhfaceIssueService.sendDelFaceBySnMsg(sn, list, taskInfo);
                    } else if (taskInfo.getType().equals(TaskTypeEnum.ALL_REMOVE.getType())) {
//                        this.deleteBySn(taskInfo.getSn(), taskInfo);
                        hhfaceIssueService.sendDeleteBySnMsg(taskInfo.getSn(), taskInfo);
                    }
                }
            } catch (Exception e) {
                log.error("任务执行失败", e);
            }
        }
    }

    @Override
    public Map<String, Object> disconnect(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        String sn = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        if (!StringUtils.hasText(sn)) {
            return result;
        }
        Integer deviceId = faceDeviceDao.getIdBySn(sn);
        if (deviceId == null) {
            return result;
        }
        FaceDevice device = new FaceDevice();
        device.setId(deviceId);
        device.setSn(sn);
        device.setOnline(0);
        device.setUpdateTime(new Date());
        faceDeviceDao.updateByPrimaryKeySelective(device);
        return result;
    }

    @Override
    public Map<String, Object> report(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        if (dto == null) {
            return result;
        }
        String sn = null;
        String version = null;
        String devIP = null;
        Integer userCount = null;
        Integer swipeCardCount = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        if (!StringUtils.hasText(sn)) {
            return result;
        }
        Map<String, Object> data = (Map<String, Object>) dto.get("data");
        if (CollectionUtils.isEmpty(data)) {
            return result;
        }
        if (data.get("swipeCardCount") != null) {
            swipeCardCount = (Integer) data.get("swipeCardCount");
        }
        if (data.get("userCount") != null) {
            userCount = (Integer) data.get("userCount");
        }
        if (data.get("version") != null) {
            version = data.get("version").toString();
        }
        if (data.get("devIP") != null) {
            devIP = data.get("devIP").toString();
        }
        FaceDeviceLessVO faceDeviceLessVO = faceDeviceDao.getBySn(sn);
        if (Objects.isNull(faceDeviceLessVO)) {
            return result;
        }
        FaceDevice device = new FaceDevice();
        device.setId(faceDeviceLessVO.getId());
        device.setOnline(1);
        device.setSn(sn);
        device.setPassCount(swipeCardCount);
        device.setUserCount(userCount);
        device.setAppVersion(version);
        device.setInterIp(devIP);
        Date now = new Date();
        device.setUpdateTime(now);
        device.setReportTime(now);
        faceDeviceDao.updateByPrimaryKeySelective(device);
        return result;
    }

    @Override
    public Map<String, Object> eventRecords(Map<String, Object> dto) {
        rabbitTemplate.convertAndSend("faceLogQueue", dto);
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        String sn = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        if (org.apache.commons.lang3.StringUtils.isBlank(sn)) {
            return result;
        }
        FaceDevice faceDevice = faceDeviceDao.getFaceDeviceBySn(sn);
        if (ObjectUtil.isNull(faceDevice)) {
            return result;
        }

        Integer action = null;
        String orgid = "";
        if (dto.get("action") != null) {
            action = (Integer) dto.get("action");
        }
        if (dto.get("orgid") != null) {
            orgid = dto.get("orgid").toString();
        }
        List<Map<String, Object>> dataList = (List) dto.get("data");

        if (CollectionUtils.isEmpty(dataList)) {
            return result;
        }
        Integer recordid = 0;
        String pin = "0";
        String name = "";
        String photo = null;
        String cardid = "";
        String cardIdEx = "";
        Integer verifytype = 0;
        Integer rightno = 104;

        Integer outtype = 0;
        String idCard = "";
        String phone = "";
        //温度
        String extend3 = "";
        String extend1 = "";
        //健康码组装数据
        String extend5 = "";
        //体温是否异常 0 正常 1 异常
        String twStatus = "";
        //健康码状态00 绿码 01 黄码 10 红码
        String jkmStatus = "";
        //健康码状态 0 健康码 1 国康码
        String jkmType = "";
        //阴性 阳性
        String hsStatus = "";
        //疫苗状态 0针 等等依次累加
        String ymStatus = "";
        //行程信息
        String xcInfo = "";

        String verify_score = null;
        boolean status = true;

        FaceLog faceLog;
        Long userId;
        Date logTime;
        long nowL = System.currentTimeMillis();
        Date now = new Date(nowL);
        String faceTime = "";
        for (Map<String, Object> data : dataList) {
            if (data.get("recordid") != null) {
                recordid = (Integer) data.get("recordid");
            }
//            else {
//                continue;
//            }
            if (data.get("pin") != null) {
                pin = data.get("pin").toString();

            }
//            else{
//                continue;
//            }
            if (data.get("name") != null) {
                name = data.get("name").toString();
            }
            if (data.get("photo") != null) {
                photo = data.get("photo").toString();
            }
            if (data.get("cardid") != null) {
                cardid = data.get("cardid").toString();
            }
            if (data.get("cardIdEx") != null) {
                cardIdEx = data.get("cardIdEx").toString();
            }
            if (data.get("verifytype") != null) {
                verifytype = (Integer) data.get("verifytype");
            }
            if (data.get("rightno") != null) {
                rightno = (Integer) data.get("rightno");
            }
            if (data.get("time") != null) {
                faceTime = data.get("time").toString();
            }
            if (data.get("outtype") != null) {
                outtype = (Integer) data.get("outtype");
            }
            if (data.get("phone") != null) {
                phone = data.get("phone").toString();
            }
            if (data.get("extend3") != null) {
                extend3 = data.get("extend3").toString();
            }
            if (data.get("extend1") != null) {
                extend1 = data.get("extend1").toString();
            }
            if (data.get("verify_score") != null) {
                verify_score = data.get("verify_score").toString();
            }
            if (data.get("twStatus") != null) {
                twStatus = data.get("twStatus").toString();
            }
            if (data.get("jkmStatus") != null) {
                jkmStatus = data.get("jkmStatus").toString();
            }
            if (data.get("jkmType") != null) {
                jkmType = data.get("jkmType").toString();
            }
            if (data.get("hsStatus") != null) {
                hsStatus = data.get("hsStatus").toString();
                String[] split = hsStatus.split(",");
                hsStatus = split[0];
            }
            if (data.get("ymStatus") != null) {
                ymStatus = data.get("ymStatus").toString();
                String[] split = ymStatus.split(";");
                ymStatus = split[0];
            }
            if (data.get("xcInfo") != null) {
                xcInfo = data.get("xcInfo").toString();
            }
            /*无效的数据不用进行数据的处理只做存储*/
            if (data.get("status") != null) {
                status = (boolean) data.get("status");
            }
            if (!status) {
                return result;
            }

            if (verifytype == 0 && NumberUtil.isLong(pin)) {
                userId = Long.valueOf(pin);
            } else {
                userId = 0L;
            }
            faceLog = new FaceLog();
            faceLog.setRecordid(Long.valueOf(recordid));
            faceLog.setAction(action);
            faceLog.setDeviceSn(sn);
            faceLog.setUserId(userId);
            faceLog.setName(name);
            faceLog.setPhoto(photo);
            faceLog.setCardid(cardid);
            faceLog.setCardidex(cardIdEx);
            faceLog.setVerifyType(verifytype);
            faceLog.setRightno(rightno);
            faceLog.setOutType(faceDevice.getPass());
            faceLog.setFaceTime(faceTime);
            faceLog.setOrgid(orgid);
            faceLog.setIdCard(idCard);
            faceLog.setPhone(phone);
            faceLog.setExtend3(extend3);
            faceLog.setExtend1(extend1);
            faceLog.setExtend5(extend5);
            faceLog.setTwStatus(twStatus);
            faceLog.setJkmStatus(jkmStatus);
            faceLog.setJkmType(jkmType);
            faceLog.setHsStatus(hsStatus);
            faceLog.setYmStatus(ymStatus);
            faceLog.setXcInfo(xcInfo);
            faceLog.setAdminId(faceDevice.getAdminId());
            faceLog.setVerifyScore(verify_score);
            faceLog.setCreateTime(now);
            log.info("推送进出记录，姓名：{}, 身份证：{}, 健康码状态：{}, SN编码:{}",
                    faceLog.getName(), faceLog.getCardid(), faceLog.getJkmStatus(), sn);
            //推送出入记录
            Integer adminId = faceDeviceDao.getAdminIdBySn(sn);
            AdminVO adminInfo = adminService.getAdminInfoById(adminId);
            if ("1".equals(adminInfo.getClockWayType())) {
                pushGeneralClockWay(faceLog, adminInfo);
            }
            if ("2".equals(adminInfo.getClockWayType())) {
                // 钉钉预留
                log.info("钉钉打卡..........");
                dingTalkServer.attendanceRecordUpload(new DingTalkRecordUpload()
                        .setAppkey(adminInfo.getDingtalkAppKey())
                        .setAppsecret(adminInfo.getDingtalkAppSecret())
                        .setMobile(faceLog.getPhone())
                        .setDeviceId(faceLog.getDeviceSn())
                        .setDeviceName(faceLog.getName())
                        .setUserCheckTime(faceLog.getCreateTime().getTime()));
            }
            // 当天进出人数统计存放在redis中
            if (faceLog.getOutType() == null || faceLog.getOutType() == 0) {
                continue;
            }
            synchronized (this) {
                String redisKey = Constants.IN_OUT_STATISTICS.concat(String.valueOf(adminId));
                InOutStatisticsBean bean = (InOutStatisticsBean) redisUtil.get(redisKey);
                if (bean == null) {
                    bean = new InOutStatisticsBean();
                }
                if (FaceLog.IN.equals(faceLog.getOutType())) {
                    bean.setInNum(bean.getInNum() + 1);
                } else if (FaceLog.OUT.equals(faceLog.getOutType())) {
                    bean.setOutNum(bean.getOutNum() + 1);
                } else {
                    continue;
                }
                if (bean.getOutNum() > bean.getInNum()) {
                    bean.setStayNum(0);
                } else {
                    bean.setStayNum(bean.getInNum() - bean.getOutNum());
                }

                redisUtil.set(redisKey, bean);
            }

        }
        return result;
    }

    //通用打卡数据推送
    private void pushGeneralClockWay(FaceLog faceLog, AdminVO adminInfo) {
        String clockWayPushUrl = null;
        JSONObject logObj = null;
        try {
            ClockWayPushVO pushVO = BeanUtil.copyProperties(faceLog, ClockWayPushVO.class);
            FaceDevice device = faceDeviceDao.getFaceDeviceBySn(pushVO.getDeviceSn());
            if (Preconditions.isBlank(device)) {
                log.error("设备不存在，请联系管理人员");
                return;
            }
            //补全地址
            pushVO.setProvinceName(adminInfo.getProvinceName());
            pushVO.setCityName(adminInfo.getCityName());
            pushVO.setAreaName(adminInfo.getAreaName());
            pushVO.setStreetName(adminInfo.getStreetName());
            pushVO.setAddress(adminInfo.getAddress());
            //补全设备地址
            pushVO.setDeviceAddress(device.getName());
            //补全访客机用户字段
            completeVisitorData(pushVO);
            //查询授权配置文件 加密
            AdminAuth adminAuth = adminAuthDao.selectOneByExample(Example.builder(AdminAuth.class)
                    .where(WeekendSqls.<AdminAuth>custom()
                            .andEqualTo(AdminAuth::getAdminId, adminInfo.getAdminId())
                            .andEqualTo(AdminAuth::getType, AdminAuthTypeEnum.GENERAL_CLOCK_WAY.getCode())
                            .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)).build());
            if (Preconditions.isNotBlank(adminAuth) && Preconditions.isNotBlank(adminAuth.getPrivateKey())) {
                pushVO.setCardid(Preconditions.isNotBlank(pushVO.getCardid()) ? Sm4Util.encrypt(adminAuth.getPrivateKey(), pushVO.getCardid()) : null);
                pushVO.setName(Preconditions.isNotBlank(pushVO.getName()) ? Sm4Util.encrypt(adminAuth.getPrivateKey(), pushVO.getName()) : null);
                pushVO.setPhone(Preconditions.isNotBlank(pushVO.getPhone()) ? Sm4Util.encrypt(adminAuth.getPrivateKey(), pushVO.getPhone()) : null);
                pushVO.setIdCard(Preconditions.isNotBlank(pushVO.getIdCard()) ? Sm4Util.encrypt(adminAuth.getPrivateKey(), pushVO.getIdCard()) : null);
            } else {
                log.error("通用打卡未授权，请联系管理人员");
            }
            //获取设备所在区域
            Integer zoneId = zoneDeviceRService.getZoneIdBySn(adminInfo.getAdminId(), faceLog.getDeviceSn());
            InterfAddrJsonVo vo = null;
            if (Preconditions.isNotBlank(adminAuth) && Preconditions.isNotBlank(adminAuth.getInterfAddrJson())) {
                List<InterfAddrJsonVo> vos = com.alibaba.fastjson.JSONObject.parseArray(adminAuth.getInterfAddrJson(), InterfAddrJsonVo.class);
                List<InterfAddrJsonVo> filterZ = vos.stream().filter(z -> Preconditions.isNotBlank(z.getZoneId()) && z.getZoneId().intValue() != 0).collect(Collectors.toList());
                if (Preconditions.isNotBlank(zoneId) && Preconditions.isNotBlank(filterZ)) {
                    for (InterfAddrJsonVo item : filterZ) {
                        if (item.getZoneId().intValue() == zoneId.intValue()) {
                            vo = item;
                        }
                    }
                }

                List<InterfAddrJsonVo> filterNZ = vos.stream().filter(z -> Preconditions.isBlank(z.getZoneId()) || z.getZoneId().intValue() == 0).collect(Collectors.toList());
                if (Preconditions.isBlank(zoneId) && Preconditions.isNotBlank(filterNZ)) {
                    vo = filterNZ.get(0);
                }
            }
            clockWayPushUrl = Preconditions.isNotBlank(vo) ? vo.getInterfAddr() : Preconditions.isNotBlank(adminAuth) && Preconditions.isNotBlank(adminAuth.getInterfaceAddress()) ? adminAuth.getInterfaceAddress() : null;
            String punchReportFieldsConfigStr = Preconditions.isNotBlank(vo) ? vo.getPunchReportFields() : Preconditions.isNotBlank(adminInfo.getPunchReportFieldsConfigStr()) ? adminInfo.getPunchReportFieldsConfigStr() : null;
//          String clockWayPushUrl = Preconditions.isNotBlank(vo) ? vo.getInterfAddr() : null;
//          String punchReportFieldsConfigStr = Preconditions.isNotBlank(vo) ? vo.getPunchReportFields() : null;
            if (Preconditions.isBlank(clockWayPushUrl) || Preconditions.isBlank(punchReportFieldsConfigStr)) {
                log.error("上报参数缺失，请联系管理人员");
                return;
            }

            log.info("管理员配置的推送上报字段：{}", punchReportFieldsConfigStr);
            logObj = JSONUtil.parseObj(pushVO);
            // 通用回调
            JSONObject finalLogObj = logObj;
            Optional.ofNullable(punchReportFieldsConfigStr).ifPresent(item -> {
                List<String> allowFields = Arrays.asList(item.split(","));
                finalLogObj.forEach((key, value) -> {
                    if (!allowFields.contains(key)) {
                        finalLogObj.set(key, "");
                    }
                });
            });
            pushGeneralClockWayData(clockWayPushUrl, logObj.toString());
        } catch (Exception e) {
            this.retryPush(clockWayPushUrl, logObj);
            log.error("上报失败，请联系管理人员{} ", e);
            e.printStackTrace();
        }
    }

    private void retryPush(String clockWayPushUrl, JSONObject logObj) {
        if (StrUtil.isBlank(clockWayPushUrl)) {
            return;
        }
        if (ObjectUtil.isEmpty(logObj)) {
            return;
        }
        RetryPushLogDto dto = new RetryPushLogDto();
        dto.setClockWayPushUrl(clockWayPushUrl);
        dto.setLogObj(logObj.toString());
        rabbitTemplate.convertAndSend("topi-queue", dto);
        /*
         * 上面的操作是将信息推送到死信队列中
         * 死信队列设置了过期时间，过期之后会根据下面配置的规则，该交换机通过*.dead.*路由键转发到绑定该路由规则的队列中进行投递
         *         args.put("x-dead-letter-exchange", "dead-exchange"); // 与交换机产生关联
         *         args.put("x-dead-letter-routing-key","*.dead.*"); // 路由规则
         */
    }

    @RabbitListener(queues = "dead-queue")
    public void retryPush(RetryPushLogDto dto) {
        log.info("重试推送数据：{},重试次数：{}", dto.getClockWayPushUrl(), dto.getRetryCount() + 1);
        try {
            pushGeneralClockWayData(dto.getClockWayPushUrl(), dto.getLogObj());
        } catch (Exception e) {
            dto.setRetryCount(dto.getRetryCount() + 1);
            if (dto.getRetryCount() >= 12) {
                return;
            }
            rabbitTemplate.convertAndSend("topi-queue", dto);
        }
    }

    /**
     * 重试3次 重试间隔2秒
     *
     * @param clockWayPushUrl
     * @param logObj
     */
    public void pushGeneralClockWayData(String clockWayPushUrl, String logObj) {
        log.info("推送接口地址：{} 推送的参数：{}", clockWayPushUrl, logObj);
        String body = HttpUtil.post(clockWayPushUrl, logObj, 3000);
        log.info("推送接口响应内容：{}", body);
    }

    //补全访客机字段
    private void completeVisitorData(ClockWayPushVO pushVO) {
        try {
            log.info("--->补全访客机字段----");
            Integer type = 104;
            Integer adminId = pushVO.getAdminId();
            String cardId = pushVO.getCardid();
            List<UserWhitelist> userWhitelists = userWhitelistDao.selectByPhoneOrIdNumber(cardId, null, Arrays.asList(pushVO.getAdminId()));
            if (Preconditions.isNotBlank(userWhitelists)) {
                type = 105;
                try {
                    String content = userWhitelists.get(0).getContent();
                    if (Preconditions.isNotBlank(content)) {
                        List<ContextVO> contextVOList = JSONArray.parseArray(content, ContextVO.class);
                        pushVO.setCustomFieldList(contextVOList);
                    }
                } catch (Exception e) {
                    log.error("--->自定义字段解析出错");
                }
            }
            pushVO.setRightno(type);
            UserVisitorListDetailInfo userListFrom = null;
            String s = "yyyy-MM-dd HH:mm:ss";
            SimpleDateFormat f = new SimpleDateFormat(s);
            String time = f.format(pushVO.getCreateTime());
            if (type == 104) {
                userListFrom = faceService.getUserListFrom(adminId, cardId, time);
            } else if (type == 105) {
                userListFrom = faceService.getUserListFromTwo(adminId, cardId, time);
            }

            if (Preconditions.isBlank(userListFrom)) {
                return;
            }
            UserVisitorListDetailVisitor visitor = userListFrom.getUserVisitorListDetailVisitor();
            if (Preconditions.isBlank(visitor)) {
                return;
            }
            pushVO.setCompany(visitor.getCompany());
            pushVO.setTemperature(visitor.getTemperature());
            pushVO.setCarno(visitor.getCarno());
            pushVO.setReason(visitor.getReason());
            pushVO.setTime(visitor.getTime());
            pushVO.setOrg(visitor.getOrg());
            pushVO.setVisiteeCompany(visitor.getVisiteeCompany());
            pushVO.setVisiteeXm(visitor.getVisiteeXm());
            pushVO.setVisiteeBm(visitor.getVisiteeBm());
            pushVO.setVisiteeRoom(visitor.getVisiteeRoom());
            pushVO.setPcsdm(visitor.getPcsdm());
            pushVO.setPcsname(visitor.getPcsname());
            pushVO.setMachineId(visitor.getMachineId());
        } catch (Exception e) {
            log.error("--->获取访客机字段失败", e);
        }
    }

    @Override
    public Map<String, Object> pushUsers(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        return result;
    }

    @Override
    public Map<String, Object> delUser(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        if (dto == null) {
            return result;
        }
        String sn = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        if (!StringUtils.hasText(sn)) {
            return result;
        }
        Object Obj = dto.get("data");
        if (Obj == null) {
            return result;
        }
        List<Map<String, Object>> data = Obj instanceof List ? ((List<Map<String, Object>>) Obj) : null;
        String dataStr;
        if (data == null) {
            dataStr = Obj.toString();
            if ("all".equals(dataStr)) {
                facePassDao.removeFacePassBySn(sn, null);
            }
            return result;
        }
        if (data.isEmpty()) {
            return result;
        }
        for (Map<String, Object> map : data) {
            facePassDao.removeFacePassBySn(sn, (Long) map.get("pin"));
        }
        return result;
    }


    @Override
    public Map<String, Object> pushTaskResult(Map<String, Object> dto) {
        try {
            log.info("天波上报结果:{}", JSON.toJSONString(dto));
            Long taskId = Long.parseLong(String.valueOf(dto.get("taskId")));

            List<IssueDeviceItemCallbackReq> items = new ArrayList<>();
            Integer taskState = (Integer) dto.get("taskState");
            if (taskState.equals(0)) {
                if (dto.containsKey("data")) {
                    Map<String, Object> objectMapmap = (Map<String, Object>) dto.get("data");
                    List<Map<String, Object>> list = (List<Map<String, Object>>) objectMapmap.get("pin");
                    if (CollUtil.isNotEmpty(list)) {
                        for (Map<String, Object> map : list) {
                            if (map.containsKey("pin")) {
                                IssueDeviceItemCallbackReq req = new IssueDeviceItemCallbackReq();
                                req.setUserId((Integer.valueOf(map.get("pin").toString())));
                                req.setCode(100);
                                req.setMsg(map.containsKey("errmsg") ? String.valueOf(map.get("errmsg")) : "");
                                items.add(req);
                            }
                        }
                    }
                }
                if (CollUtil.isNotEmpty(items)) {
                    issueDeviceService.handleIssueDeviceItemCallback(taskId, items, true);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        if (dto == null) {
            return result;
        }
        return result;
    }

    private String msgAddress(List<UserInfoMsgExcel> list) {
        if (CollUtil.isEmpty(list)) {
            return "";
        }
        String uploadExcel = Constants.UPLOAD_EXCEL;
        String fileName = uploadExcel + "/全量下发错误记录-" + System.currentTimeMillis() + ".xlsx";
        // 这里 需要指定写用哪个class去写
        ExcelWriter excelWriter = null;
        try {
            Path directory = Paths.get(uploadExcel);
            if (!Files.exists(directory)) {
                Files.createDirectories(directory);
            }
            excelWriter = EasyExcel.write(fileName, UserInfoMsgExcel.class).build();
            WriteSheet writeSheet = EasyExcel.writerSheet("错误记录").build();
            excelWriter.write(list, writeSheet);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 千万别忘记finish 会帮忙关闭流
            if (excelWriter != null) {
                excelWriter.finish();
            }
        }
        return AddressUtil.address.concat(fileName);
    }

    @Override
    public Map<String, Object> getTaskId(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        result.put("taskId", SnowflakeIdWorker.getSnowId());
        if (dto == null) {
            return result;
        }
        return result;
    }

    @Override
    public void pushUsersOnline(List<Map<String, String>> list) {
        redisUtil.set("1", list);
    }

    @Override
    public Map<String, Object> syncUsers(Map<String, Object> param) {
        Map<String, Object> map = Maps.newHashMapWithExpectedSize(4);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        log.info("开始批量同步的数据，{}", sdf.format(new Date()));
        String SN = param.get("SN").toString();
        List<Object> shishi1 = redisUtil.lGet(SN, 0, -1);
        map.put("errCode", "00");
        map.put("totalNum", redisUtil.lGetListSize(SN));
        map.put("updateTime", "11");
        map.put("data", shishi1);
        log.info("结束批量同步的数据，{}", sdf.format(new Date()));
        return map;
    }

    @Override
    public Map<String, Object> downUser(DownFaceNew downFace) {

        hhfaceIssueService.sendDownUserMsg(downFace, null);
        return null;
//        return this.downUser(downFace, null);
    }

    @Override
    public Map<String, Object> downUser(DownFaceNew downFace, TaskInfo taskInfo) {
        //TODO。。。前置条件 校验数据完整性
        StringBuffer errorMsg = checkDataCompleteness(downFace);
        FaceDeviceLessVO faceDeviceBySn = faceDeviceDao.getBySn(downFace.getSn());
        if (Preconditions.isBlank(faceDeviceBySn)) {
            Map<String, Object> result = new HashMap<>();
            log.info("设备编号：{}不存在", downFace.getSn());
            errorMsg.append("设备编号不存在!");
            result.put("data", errorMsg.toString());
            result.put("code", 400);
            return result;
        }
        /*2.根据角色的不同设置不同的有效时间*/
        //当前时间
        Date now = new Date();
        Date startTime;
        Date endTime;
        Integer rightno = downFace.getRightno();//角色类型 104：访客 | 105：常客 | 101：黑名单
        //有效开始时间
        if (downFace.getValidPeriodStart() == null) {
            startTime = DateUtils.addMinutes(now, -10);//减10分钟 确保当前时间不会因为波动而没有存进去
        } else {
            startTime = downFace.getValidPeriodStart();
        }
        //有效结束时间
        if (downFace.getValidPeriodEnd() == null) {
            if (rightno == 104) {
                endTime = DateUtils.addDays(now, 1);
            } else if (rightno == 105) {
                endTime = DateUtils.addYears(now, 1);
            } else {
                endTime = DateUtils.addYears(now, 100);
            }
        } else {
            endTime = downFace.getValidPeriodEnd();
        }
        if (DateUtil.compare(DateUtil.date(), endTime) > 0) {
            errorMsg.append("该下发结束时间小于当前时间!");
        }

        Map<String, Object> result = new HashMap<>();
        if (Preconditions.isNotBlank(errorMsg.toString())) {
            handleFailedIssueDevice(downFace, startTime, endTime, faceDeviceBySn.getAdminId(), errorMsg.toString());
            log.error("--->下发失败：{}", errorMsg.toString());
            result.put("data", errorMsg.toString());
            result.put("code", 400);
            return result;
        }

        handleIotWhitelist(downFace, startTime, endTime);

        if (faceDeviceBySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.TB.getCode())
                || faceDeviceBySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())) {
            Long taskId = handleSuccessIssueDevice(downFace, startTime, endTime, faceDeviceBySn.getAdminId());
            handleTbOrHandhold(downFace, faceDeviceBySn, startTime, endTime, rightno, taskId);
        }

        //智能门锁设备
        try {
            if (faceDeviceBySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.SMART_LOCK.getCode())) {
                handleSuccessIssueDevice(downFace, startTime, endTime, faceDeviceBySn.getAdminId());
                adminSmartlockDeviceService.addSmartLockUser(downFace, startTime, endTime, faceDeviceBySn.getAdminId());
            }
        } catch (Exception e) {
            log.error("--->智能门锁人脸下发失败:{}", e.getMessage());
        }
        //蓝牙智能门锁设备
        try {
            if (faceDeviceBySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.BLUETOOTH_SMART_LOCK.getCode())) {
                handleSuccessIssueDevice(downFace, startTime, endTime, faceDeviceBySn.getAdminId());
                middlewareService.addUser(downFace, startTime, endTime, faceDeviceBySn.getAdminId());
            }
        } catch (Exception e) {
            log.error("--->蓝牙智能门锁人脸下发失败:{}", e.getMessage());
        }

        try {
            if (faceDeviceBySn.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.HHFACE.getCode())) {
                issueDeviceService.issueUserIncrement(downFace, startTime, endTime, faceDeviceBySn.getAdminId());
            }
        } catch (Exception e) {
            log.error("--->HHFACE人脸下发失败:{}", e.getMessage());
        }
        //人脸清除定时任务
        Long faceTaskId = faceTaskDao.getFaceTaskId(faceDeviceBySn.getSn(), downFace.getUserId());
        FaceTask faceTask = new FaceTask();
        faceTask.setId(faceTaskId);
        faceTask.setTaskTime(endTime);
        faceTask.setStatus(0);
        faceTask.setUpdateTime(now);
        if (faceTaskId == null) {
            faceTask.setDeviceSn(faceDeviceBySn.getSn());
            faceTask.setUserId(downFace.getUserId());
            faceTask.setCreateTime(now);
            faceTaskDao.insertSelective(faceTask);
        } else {
            faceTaskDao.updateByPrimaryKeySelective(faceTask);
        }
        result.put("data", "命令下发成功");
        return result;
    }

    private void handleFailedIssueDevice(DownFaceNew downFace, Date startTime, Date endTime, Integer adminId, String errorMsg) {
        issueDeviceService.handleIssueDevice(downFace, startTime, endTime, adminId, false, errorMsg);
    }

    private Long handleSuccessIssueDevice(DownFaceNew downFace, Date startTime, Date endTime, Integer adminId) {
        return issueDeviceService.handleIssueDevice(downFace, startTime, endTime, adminId, true, "下发成功");
    }

    //处理天波aiface和手持设备
    private void handleTbOrHandhold(DownFaceNew downFace, FaceDeviceLessVO faceDevice, Date startTime, Date endTime, Integer rightno, Long taskId) {
        //TODO 离线任务
        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectByPrimaryKey(faceDevice.getFaceDeviceTypeId());
        //增加离线任务  离线并且不是手持的设备需要增加为离线任务
        // 设备离线
//        if (faceDevice.getOnline().equals(0)
//                && !faceDeviceType.getCode().equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())
//                && !faceDeviceType.getCode().equals(FaceDeviceTypeEnum.SMART_LOCK.getCode())
//                && !faceDeviceType.getCode().equals(FaceDeviceTypeEnum.BLUETOOTH_SMART_LOCK.getCode())
//                && !faceDeviceType.getCode().equals(FaceDeviceTypeEnum.HHFACE.getCode())) {
//            if (taskInfo == null) {//生成离线任务
//                taskInfo = taskInfoService.save(faceDevice.getSn(), downFace);
//            } else {
//                Integer retry = taskInfo.getRetry();
//                if (retry != null && retry >= 1) {
//                    retry = retry - 1;
//                    taskInfo.setRetry(retry);
//                    taskInfoDao.updateByPrimaryKeySelective(taskInfo);
//                } else {
//                    taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
//                }
//            }
//            log.info("--->天波AiFace或手持设备生成离线任务");
//            return;
//        }

        //人脸下发到设备里面
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(1);
        Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(8);
        // 下发参数
        dataItem.put("pin", downFace.getUserId());
        dataItem.put("name", downFace.getName());
        dataItem.put("validPeriodStart", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, startTime));
        dataItem.put("validPeriodEnd", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, endTime));
        dataItem.put("rightno", rightno);
        dataItem.put("userType", rightno == 104 ? 0 : 3);
        if (org.apache.commons.lang3.StringUtils.isNotBlank(downFace.getPhoto())) {
            dataItem.put("photo", downFace.getPhoto());
        }
        dataItem.put("cardid", downFace.getCardid());
        dataItem.put("phone", downFace.getPhone());
        dataItem.put("cardIdEx", downFace.getCardIdEx());
        data.add(dataItem);
        params.put("SN", downFace.getSn());
        params.put("Data", data);
        params.put("taskId", taskId);
        /*人脸下发的几种情况*/
        if (faceDevice.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.TB.getCode())) {
            // 天波人脸下发
            asyncTriggerRetry.send(params, null);
        }
        if (faceDevice.getFaceDeviceTypeCode().equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())) {
            /*2.手持终端人脸下发*/
            handheldTerminalsImpl.send(params, null, faceDevice.getId());
        }
//        /*离线任务完成即删除*/
//        if (taskInfo != null) {
//            taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
//        }
        log.info("--->天波AiFace或手持设备人员下发成功");
    }

    private void handleIotWhitelist(DownFaceNew downFace, Date startTime, Date endTime) {
        IotUserWhitelistDto dto = new IotUserWhitelistDto();
        dto.setIdNumber(downFace.getCardid());
        dto.setSn(downFace.getSn());
        dto.setStartTime(startTime);
        dto.setEndTime(endTime);
        dto.setName(downFace.getName());
        dto.setPhone(downFace.getPhone());
        spiIotUserWhitelistService.addSingleUser2Iot(dto);
    }

    //校验数据完整性
    private StringBuffer checkDataCompleteness(DownFaceNew downFace) {
        /*1.先进行数据的校验*/
        StringBuffer errorMsg = new StringBuffer();//存放错误信息
        if (Preconditions.isBlank(downFace)) {
            errorMsg.append("请传入下发数据!");
        }
        if (Preconditions.isNotBlank(downFace) && Preconditions.isBlank(downFace.getSn())) {
            errorMsg.append("设备编号不能为空!");
        }
        if (Preconditions.isNotBlank(downFace) && Preconditions.isBlank(downFace.getUserId())) {
            errorMsg.append("唯一标识userId不能为空!");
        }
        if (Preconditions.isNotBlank(downFace) && Preconditions.isBlank(downFace.getRightno())) {
            errorMsg.append("权限级别不能为空!");
        }
        return errorMsg;
    }

    private FaceRequestLog intoFaceRequestLog(String name, String context) {
        FaceRequestLog requestLog = new FaceRequestLog();
        requestLog.setName(name);
        requestLog.setContext(context);
        requestLog.setParams("");
        requestLog.setStatus(1);
        requestLog.setUsedTime(0L);
        Date now = new Date();
        requestLog.setUpdateTime(now);
        requestLog.setCreateTime(now);
        faceRequestLogDao.insertSelective(requestLog);
        return requestLog;
    }
}
