package com.yx.face.service.impl;

import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.Base64Util;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.dao.ShortTermFaceLogDao;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.model.dto.Barcode2HealthCodeDTO;
import com.yx.face.model.dto.FixDataDto;
import com.yx.face.model.entity.ShortTermFaceLog;
import com.yx.face.model.entity.UserInfo;
import com.yx.face.boot.uitls.Chinese2PinyinUtil;
import com.yx.face.dao.UserVisitorListDetailDao;
import com.yx.face.dao.UserWhitelistDao;
import com.yx.face.dao.VisiteeDao;
import com.yx.face.dao.ZoneDeviceRDao;
import com.yx.face.model.entity.UserVisitorListDetail;
import com.yx.face.model.entity.UserWhitelist;
import com.yx.face.model.entity.Visitee;
import com.yx.face.model.entity.ZoneDeviceR;
import com.yx.face.model.vo.health.Barcode2HealthCodeVO;
import com.yx.face.service.ApiHealthCodeService;
import com.yx.face.service.BaiDuFaceService;
import com.yx.face.service.FixDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package com.yx.face.service.impl
 * @class FixDataServiceImpl
 * @date 2022/10/13 上午11:26
 * @description
 */
@Slf4j
@Service
public class FixDataServiceImpl implements FixDataService {

    @Resource
    private ShortTermFaceLogDao shortTermFaceLogDao;

    @Resource
    private Executor executor;

    @Autowired
    private ApiHealthCodeService apiHealthCodeService;

    @Resource
    private UserInfoDao userInfoDao;

    @Resource
    private BaiDuFaceService baiDuFaceService;

    @Resource
    private VisiteeDao visiteeDao;

    @Resource
    private UserVisitorListDetailDao userVisitorListDetailDao;

    @Resource
    private ZoneDeviceRDao zoneDeviceRDao;

    @Resource
    private UserWhitelistDao userWhitelistDao;

    @Override
    public void fixShortItemFaceLogData(FixDataDto dto) {
        List<ShortTermFaceLog> list = shortTermFaceLogDao.list(dto);
        if (list != null && list.size() > 0) {
            List<ShortTermFaceLog> handleFaceLogs = new ArrayList<>();
            int count = 1;
            for (ShortTermFaceLog item : list) {
                if (count % 500 == 0) {
                    handleFaceLogs.add(item);
                    handleShortTermFaceLogs(handleFaceLogs);
                    handleFaceLogs = null;
                    handleFaceLogs = new ArrayList<>();
                } else {
                    handleFaceLogs.add(item);
                }
                count++;
            }

            if (handleFaceLogs != null && handleFaceLogs.size() > 0) {
                handleShortTermFaceLogs(handleFaceLogs);
                handleFaceLogs = null;
            }
        }
    }

    @Override
    public void completeUserInfoFacialFeature(List<Long> userIds) {
//        String faceCharacteristic = baiDuFaceService.baiDuFaceCharacteristic(Base64Util.imgToBase64("https://tx.hz-hanghui.com:8088/yx-fyzd/file/upload/imagesnew/20240606/514961641499000832.jpg"));
//        log.info(faceCharacteristic);
        List<UserInfo> userInfos = userInfoDao.selectByExample(Example.builder(UserInfo.class).where(WeekendSqls.<UserInfo>custom().andIsNotNull(UserInfo::getAvatar).andIsNull(UserInfo::getFacialFeature)).orderByDesc("id").build());
        if (Preconditions.isNotBlank(userInfos)) {
            handleUserInfos(userInfos);
        }
    }

    private void handleUserInfos(List<UserInfo> userInfos) {
        for (UserInfo item : userInfos) {
            try {
                if (Preconditions.isBlank(item.getAvatar())) {
                    continue;
                }
                String faceCharacteristic = baiDuFaceService.baiDuFaceCharacteristic(Base64Util.imgToBase64(item.getAvatar()));
                if (Preconditions.isBlank(faceCharacteristic)) {
                    log.info("--->未获取到人脸特征值");
                    continue;
                }
                item.setFacialFeature(faceCharacteristic);
                userInfoDao.updateByPrimaryKey(item);
            } catch (Exception e) {
                log.error(e.getMessage());
                continue;
            }
        }
    }

    @Override
    public void fixVisiteeData() {
        List<Visitee> visitees = visiteeDao.selectByExample(Example.builder(Visitee.class).where(WeekendSqls.<Visitee>custom().andIsNotNull(Visitee::getName).andIsNull(Visitee::getPinyinFirstLetter)).build());
        if (Preconditions.isBlank(visitees)) {
            return;
        }
        visitees.stream().forEach(item -> {
            item.setPinyinFirstLetter(Chinese2PinyinUtil.getPinyinFirstLetter(item.getName()));
            visiteeDao.updateByPrimaryKey(item);
        });
    }


    @Override
    public void fixVisitorListDetailData() {
        log.info("--->补全访客记录所属区域开始");
        //查询的记录
        Example e = new Example(UserVisitorListDetail.class);
        Example.Criteria c = e.createCriteria();
        c.orIsNotNull("machineId").orIsNotNull("userWhitelistId");

        Example.Criteria criteria = e.createCriteria();
        criteria.andIsNull("zoneDevicePassIdList");
        e.and(criteria);

        List<UserVisitorListDetail> dataList = userVisitorListDetailDao.selectByExample(e);
        if (Preconditions.isNotBlank(dataList)) {
            List<UserVisitorListDetail> snVisitors = dataList.stream().filter(item -> Preconditions.isNotBlank(item.getMachineId())).collect(Collectors.toList());
            List<UserVisitorListDetail> miniVisitors = dataList.stream().filter(item -> Preconditions.isBlank(item.getMachineId()) && Preconditions.isNotBlank(item.getUserWhitelistId())).collect(Collectors.toList());
            handleVisitorZone(snVisitors, miniVisitors);
        }
        log.info("--->补全访客记录所属区域结束");
    }

    private void handleVisitorZone(List<UserVisitorListDetail> snVisitors, List<UserVisitorListDetail> miniVisitors) {
        if (Preconditions.isNotBlank(snVisitors)) {
            for (UserVisitorListDetail item : snVisitors) {
                List<ZoneDeviceR> dbAll = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class)
                        .where(WeekendSqls.<ZoneDeviceR>custom().andEqualTo(ZoneDeviceR::getAdminId, item.getAdminId())
                                .andIsNotNull(ZoneDeviceR::getSnJson)
                                .andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
                if (Preconditions.isBlank(dbAll)) {
                    continue;
                }
                forZone:
                for (ZoneDeviceR d : dbAll) {
                    List<String> sns = Arrays.asList(d.getSnJson().split(","));
                    if (sns.contains(item.getMachineId())) {
                        item.setZoneDevicePassIdList(Arrays.asList(d.getId()));
                        userVisitorListDetailDao.updateByPrimaryKey(item);
                        break forZone;
                    }
                }
            }
        }

        if (Preconditions.isNotBlank(miniVisitors)) {
            for (UserVisitorListDetail item : miniVisitors) {
                UserWhitelist whitelist = userWhitelistDao.selectByPrimaryKey(item.getUserWhitelistId());
                if (Preconditions.isNotBlank(whitelist) && Preconditions.isNotBlank(whitelist.getZoneDevicePassIdList())) {
                    item.setZoneDevicePassIdList(whitelist.getZoneDevicePassIdList());
                    userVisitorListDetailDao.updateByPrimaryKey(item);
                }
            }
        }
    }

    private void handleShortTermFaceLogs(List<ShortTermFaceLog> list) {
        executor.execute(() -> {
            Map<String, List<ShortTermFaceLog>> groupItem = list.stream().collect(Collectors.groupingBy(ShortTermFaceLog::getCardid));
            groupItem.forEach((key, value) -> {
                if (key != null) {
                    String barcode = key;

                    Barcode2HealthCodeDTO fixDataHealthCodeDTO = new Barcode2HealthCodeDTO();
                    fixDataHealthCodeDTO.setBarCode(barcode);
                    //获取场所码接口
                    RestResult<Barcode2HealthCodeVO> restResult = apiHealthCodeService.queryInfoByBarCode(fixDataHealthCodeDTO);
                    Barcode2HealthCodeVO healthCodeVO = restResult.getData();
                    if (healthCodeVO != null && healthCodeVO.getUserName() != null && healthCodeVO.getIdNum() != null) {
                        List<ShortTermFaceLog> fixData = value;
                        List<Long> ids = fixData.stream().map(ShortTermFaceLog::getId).collect(Collectors.toList());
                        shortTermFaceLogDao.updateLogs(healthCodeVO.getUserName(), healthCodeVO.getIdNum(), ids);
                    }
                }
            });
        });
    }

}
