package com.yx.face.service.impl;

import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.Base64Util;
import com.yx.face.dao.FaceDeviceDetailDao;
import com.yx.face.dao.FacePassDao;
import com.yx.face.model.entity.FaceDeviceDetail;
import com.yx.face.model.entity.FacePass;
import com.yx.face.server.HandheldTerminalsServer;
import com.yx.face.server.dto.AddPersonDTO;
import com.yx.face.server.dto.BindCompanyDTO;
import com.yx.face.server.vo.BindCompanyVO;
import com.yx.face.server.vo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Map;

/**
 * @description: 手持设备相关操作
 * @ClassName HandheldTerminalsImpl
 * @Author WXG
 * @Date 2022/4/19 13:53
 */
@Service
@Slf4j
public class HandheldTerminalsImpl {

    @Resource
    private FaceDeviceDetailDao faceDeviceDetailDao;
    @Resource
    private FacePassDao facePassDao;
    @Resource
    private HandheldTerminalsServer handheldTerminalsServer;

    private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    /*数据下发到手持终端*/
    @Async
    public void send(Map<String, Object> params, FacePass facePass,Integer DeviceId) {
        /*
        * params的数据格式如下
        * @param
        * map {
           "SN": "555555",
           "taskId": 1,
           "Data": [{
           "pin": "777",
           "name": "张三",
           "photo": "/9j/4AAQSklJpSm.........9k=",
           "cardid": "1234567890",
           "validPeriodStart": "2018-06-23 08:00:00",
           "validPeriodEnd": "2019-06-23 08:00:00",
           "ruleId": "1",
           "userType": 0
           }]
           }<br>
        * */

        String sn = (String)params.get("SN");
        /*1.获取设备companyId*/
        String companyId = getCompanyId(DeviceId,sn);


        if(!StringUtils.hasText(companyId)){
            log.info(sn+":没有对应的companyId");
            facePass.setStatus(3);
            facePassDao.updateByPrimaryKeySelective(facePass);
            return;
        }

        /*2.数据解析（params）*/
        List<Map<String, Object>> data = (List<Map<String, Object>>)params.get("Data");
        for (Map<String, Object> dataItem : data) {
            Long pin = (Long)dataItem.get("pin");
            String name = (String)dataItem.get("name");
            String startTime = (String)dataItem.get("validPeriodStart");
            String endTime = (String)dataItem.get("validPeriodEnd");
            Integer userType = (Integer)dataItem.get("userType");
            String photo = (String) dataItem.get("photo");
            Integer rightno = (Integer)dataItem.get("rightno");
            String cardid = (String)dataItem.get("cardid");

            try {
                log.info("图片地址:{}",photo);
                photo = Base64Util.imgToBase64(photo);

                AddPersonDTO addPersonDTO = new AddPersonDTO();
                addPersonDTO.setCompanyId(companyId);
                addPersonDTO.setCardId(cardid);
                addPersonDTO.setXm(name);
                addPersonDTO.setPhoto(photo);
                addPersonDTO.setValidStartTime(sdf.parse(startTime).getTime());
                addPersonDTO.setValidEndTime(sdf.parse(endTime).getTime());
                ResultVO ResultVO = handheldTerminalsServer.addPerson(addPersonDTO);

                if(ResultVO.getErrCode() != 1){
                    log.info(sn+":调用下发接口失败,返回的错误信息:{}",ResultVO);
                    facePass.setStatus(3);
                    facePassDao.updateByPrimaryKeySelective(facePass);
                    return;
                }else{
                    log.info("手持终端下发成功,返回数据{}",ResultVO);
                }

            } catch (ParseException e) {
                e.printStackTrace();
            }
        }

    }
    public String getCompanyId(Integer DeviceId , String sn){
        String companyId ="";
        FaceDeviceDetail faceDeviceDetail = faceDeviceDetailDao.selectOne(new FaceDeviceDetail().setFaceDeviceId(DeviceId));
        if(faceDeviceDetail == null) {
            BindCompanyDTO bindCompanyDTO = new BindCompanyDTO();
            bindCompanyDTO.setCondition(sn.replaceAll("hh_",""));
            ResultVO companyId1 = handheldTerminalsServer.bindCompany(bindCompanyDTO);
            if(companyId1.getContent() != null && companyId1.getContent().size()>0){
                List<BindCompanyVO> content = companyId1.getContent();
                BindCompanyVO bindCompanyVO = content.get(0);
                companyId = bindCompanyVO.getCompany_id();
                faceDeviceDetail = new FaceDeviceDetail();
                faceDeviceDetail.setCompanyId(companyId);
                faceDeviceDetail.setFaceDeviceId(DeviceId);
                faceDeviceDetailDao.insertSelective(faceDeviceDetail);
            }else{
                throw new ServiceException(sn+":没有对应的companyId,请配置辖区");
            }
        }else companyId = faceDeviceDetail.getCompanyId();
        if(!StringUtils.hasText(companyId)){
            throw new ServiceException(sn+":没有对应的companyId");
        }
        return companyId;
    }
}
