package com.yx.face.service.impl;

import cn.hutool.core.date.DateUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.dao.HealthCodeConfigDao;
import com.yx.face.model.dto.HealthCodeConfigPageDTO;
import com.yx.face.model.entity.HealthCodeConfig;
import com.yx.face.service.HealthCodeConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import java.util.List;

/**
 * 服务实现类
 *
 * @author lileilei
 * @since 2022-08-29
 */
@Service
public class HealthCodeConfigServiceImpl implements HealthCodeConfigService {
    @Autowired
    private HealthCodeConfigDao healthCodeConfigDao;

    @Override
    public PageInfo<HealthCodeConfig> getPageList(RestDTO<HealthCodeConfigPageDTO> dto) {
        //查询条件
        HealthCodeConfigPageDTO data = dto.getData();
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<HealthCodeConfig> pageVOList = healthCodeConfigDao.selectListBySearch(data);
        if (pageVOList != null && pageVOList.size() > 0) {
            pageVOList.stream().forEach(item -> item.setValidRequestTimes((item.getRequestNumberLimit() != null && item.getRequestNumberLimit().intValue() - item.getRequestedTimes().intValue() >= 0) ? (item.getRequestNumberLimit().intValue() - item.getRequestedTimes().intValue()) : 0));
        }
        return new PageInfo<>(pageVOList);
    }

    @Override
    public RestResult del(Integer id) {
        healthCodeConfigDao.deleteByPrimaryKey(id);
        return RestResponse.ok();
    }

    @Override
    public RestResult save(HealthCodeConfig config) {
        //根据sn查询健康码配置信息
        HealthCodeConfig healthCodeConfig = healthCodeConfigDao.selectOneByExample(Example.builder(HealthCodeConfig.class).where(WeekendSqls.<HealthCodeConfig>custom().andEqualTo(HealthCodeConfig::getSn, config.getSn())).build());

        if (config.getId() == null) {
            if (healthCodeConfig != null) {
                return RestResponse.error("设备编码已存在");
            }
            config.setCreateTime(DateUtil.date()).setUpdateTime(DateUtil.date());
            healthCodeConfigDao.insertSelective(config);
        } else {
            if (healthCodeConfig != null && !healthCodeConfig.getId().equals(config.getId())) {
                return RestResponse.error("设备编码已存在");
            }
            healthCodeConfigDao.updateById(config);
        }

        return RestResponse.ok();
    }
}
