package com.yx.face.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.google.common.collect.Lists;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.InOutStatisticsDao;
import com.yx.face.model.dto.InOutStatisticsDto;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.InOutStatistics;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.redis.InOutStatisticsBean;
import com.yx.face.model.search.FaceServerLogSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.InOutStatisticsService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
@AllArgsConstructor(onConstructor_ = @Autowired)
public class InOutStatisticsServiceImpl implements InOutStatisticsService {

    private final InOutStatisticsDao inOutStatisticsDao;

    private final RedisUtil redisUtil;
    private final AdminService adminService;
    private final AdminDao adminDao;
    /**
     * 将当天的进出记录统计从redis中刷新到数据库
     */
/*    @Override
    public void refreshStatisticsCache() {
        long yesterdayTimestamp = System.currentTimeMillis() - 1000 * 3600 * 24;
        Date yesterday = new Date(yesterdayTimestamp);
        InOutStatistics inOutStatistics = getInOutStatisticsFromCache();
        inOutStatistics.setStatisticDate(yesterday);
        inOutStatistics.setCreateTime(new Date());
        inOutStatisticsDao.insert(inOutStatistics);
        redisUtil.del(RedisCacheKey.IN_OUT_STATISTICS.key);
    }*/

    /**
     * 从缓存中获取进出统计信息
     *
     * @return InOutStatistics
     */
    private InOutStatistics getInOutStatisticsFromCache(List<Long> adminIds) {
        InOutStatistics inOutStatistics = new InOutStatistics();
        int inNum = 0;
        int outNum = 0;
        for (Long adminId : adminIds) {
            String redisKey =  Constants.IN_OUT_STATISTICS.concat(String.valueOf(adminId));
            InOutStatisticsBean bean = (InOutStatisticsBean)redisUtil.get(redisKey);
            if(bean == null){
                continue;
            }
            inNum += bean.getInNum();
            outNum += bean.getOutNum();
        }
        inOutStatistics.setInNum(inNum);
        inOutStatistics.setOutNum(outNum);
        inOutStatistics.setStayNum(inNum > outNum?inNum - outNum:0);
        return inOutStatistics;
    }

    /**
     * 进出记录统计列表
     *
     * @param inOutStatisticsDto InOutStatisticsDto
     * @return PageInfo<InOutStatistics>
     */
    @Override
    public InOutStatistics dataByTime(InOutStatisticsDto inOutStatisticsDto) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String currDateStr = sdf.format(new Date());
        long todayTimestamp = sdf.parse(currDateStr).getTime();
        InOutStatistics result;
        AdminVO info = adminService.getInfo();
        info.setAdminIds(Lists.newArrayList(info.getAdminId()));
        if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())) && AdminTypeEnum.DISTRICT.getCode().equals(info.getType())) {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, info.getAdminId())).build());
            info.getAdminIds().addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
        }
        Map<String, Object> map = new HashMap<>();
        map.put("adminVo", info);
        map.put("query", new FaceServerLogSearch());
        /*1.获取所有满足条件的adminId*/
        List<Long> adminIds = adminService.getAdminListSearch(map);

        if (inOutStatisticsDto.getStartDate().getTime() >= todayTimestamp) {
            result = getInOutStatisticsFromCache(adminIds);
        } else {
            Example example = new Example(InOutStatistics.class);
            Example.Criteria criteria = example.createCriteria();
            if (ObjectUtil.isNotEmpty(inOutStatisticsDto.getStartDate())) {
                criteria.andGreaterThanOrEqualTo("statisticDate", inOutStatisticsDto.getStartDate());
            }
            if (ObjectUtil.isNotEmpty(inOutStatisticsDto.getEndDate())) {
                criteria.andLessThanOrEqualTo("statisticDate", inOutStatisticsDto.getEndDate());
            }
            criteria.andIn("adminId",adminIds);

            List<InOutStatistics> inOutStatistics = inOutStatisticsDao.selectByExample(example);
            int inNum = 0;
            int outNum = 0;
            for (InOutStatistics data : inOutStatistics) {
                inNum += data.getInNum();
                outNum += data.getOutNum();
            }
            if(inOutStatisticsDto.getEndDate().getTime() >= todayTimestamp){
                result = getInOutStatisticsFromCache(adminIds);
                inNum += result.getInNum();
                outNum += result.getOutNum();
            }

            result = new InOutStatistics();
            result.setInNum(inNum);
            result.setOutNum(outNum);
            result.setStayNum(inNum >= outNum ? inNum - outNum : 0);
        }
        return result;
    }
}
