package com.yx.face.service.impl;

import cn.hutool.core.date.DateUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.dao.IssuedTaskInfoDao;
import com.yx.face.model.dto.TaskInfoPageDTO;
import com.yx.face.model.entity.IssuedTaskInfo;
import com.yx.face.model.enums.IssuedTaskStatusEnum;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.TaskInfoPageVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.IssuedTaskInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 下发任务列表 服务实现类
 *
 * @author lileilei
 * @since 2022-07-13
 */
@Service
public class IssuedTaskInfoServiceImpl implements IssuedTaskInfoService {
    @Autowired
    private IssuedTaskInfoDao issuedTaskInfoDao;
    @Autowired
    private AdminService adminService;

    @Override
    public PageInfo<TaskInfoPageVO> getPageList(RestDTO<TaskInfoPageDTO> dto) {
        //查询条件
        TaskInfoPageDTO data = dto.getData();
        //查询当前账户信息
        AdminVO info = adminService.getInfo();
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<TaskInfoPageVO> pageVOList = issuedTaskInfoDao.selectListBySearch(data, info);
        return new PageInfo<>(pageVOList);
    }

    @Override
    public Boolean cancel(Integer id) {
        IssuedTaskInfo issuedTaskInfo = issuedTaskInfoDao.selectByPrimaryKey(id);
        if(IssuedTaskStatusEnum.ISSUED.getCode().equals(issuedTaskInfo.getStatus())){
            throw new ServiceException("任务状态正在下发中，无法取消，请稍后");
        }

        if(IssuedTaskStatusEnum.COMPLETED.getCode().equals(issuedTaskInfo.getStatus())){
            throw new ServiceException("您的任务下发已完成，无法取消");
        }

        issuedTaskInfoDao.updateByPrimaryKeySelective(new IssuedTaskInfo().setId(id).setStatus(IssuedTaskStatusEnum.CANCELLED.getCode()).setUpdateTime(DateUtil.date()));
        return true;
    }
}
