package com.yx.face.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.*;
import com.yx.face.model.api.appointment.vo.OCRVO;
import com.yx.face.model.api.building.vo.BusinessLicenseVO;
import com.yx.face.model.enums.CountryCodeEnum;
import com.yx.face.model.enums.OCREnum;
import com.yx.face.service.OCRService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @description:
 * @ClassName OCRServiceImpl
 * @Author WXG
 * @Date 2021/10/20 9:55
 */
@Slf4j
@Service
public class OCRServiceImpl implements OCRService {

   
    @Autowired
    private RedisUtil redisUtil;


    @Override
    public OCRVO getPassportDetail(String passportUrl, Integer type) {
        JSONObject jsonData = getOCR(passportUrl, OCREnum.PASSPORT, 1);
        JSONObject body1 = jsonData.getJSONObject("body");
        //将数据存到map集合中
        OCRVO ocrvo = new OCRVO();
        if (body1 != null) {

            String acatar = null;//头像需要经过重新的处理返回
            ocrvo.setName(body1.getString("bgxm"));//姓名
            ocrvo.setPassportCode(body1.getString("hzhm"));//护照号码
            ocrvo.setEfficientTime(body1.getString("yxqz"));//护照有效时间
            acatar = body1.getString("tx");
            ocrvo.setCountryCode(body1.getString("czrgjdm"));//国家代码
            ocrvo.setCountryName(Preconditions.isNotBlank(CountryCodeEnum.getCountryCodeEnumByThreeCode(body1.getString("czrgjdm"))) ? CountryCodeEnum.getCountryCodeEnumByThreeCode(body1.getString("czrgjdm")).getCountryName() : null);//国家全称
            // csrq 出生日期
            ocrvo.setBirthday(body1.getString("csrq"));
            // xb 男女
            ocrvo.setGender(body1.getString("xb"));
            if (acatar != null) {
                String pathString = new Date().getTime() + String.valueOf((int) (Math.random() * 10000)) + ".jpg";
                String filePath = FileUtils.getDirectory(Constants.UPLOAD_IMAGE_NEW) + "/";
                acatar = AddressUtil.address + FileUtils.transformBase64Image(acatar, filePath, pathString);
                ocrvo.setAvatar(acatar);
                ocrvo.setHztx(acatar);
            }
        }
        return ocrvo;
    }

    @Override
    public BusinessLicenseVO getOCRDetail(String passportUrl) {
        JSONObject jsonData = getOCR(passportUrl, OCREnum.BUSINESS_LICENSE, 2008);

        DateFormat format = new SimpleDateFormat("yyyy年MM月dd日");
        JSONArray body1 = jsonData.getJSONArray("body");
        BusinessLicenseVO bl = new BusinessLicenseVO();
        if (body1 != null && body1.size() > 0) {
            for (int i = 0; i < body1.size(); i++) {
                JSONObject jsonObject1 = body1.getJSONObject(i);
                String desc = jsonObject1.getString("desc");
                String content = jsonObject1.getString("content");
                if (StringUtils.isEmpty(desc)) continue;
                try {
                    switch (desc) {
                        case "统一社会信用代码":
                            bl.setCompanyCode(content);
                            break;
                        case "组织机构代码":
                            bl.setOrganizationCode(content);
                            break;
                        case "税务登记证号":
                            bl.setTaxNo(content);
                            break;
                        case "社保登记号":
                            bl.setSocialNo(content);
                            break;
                        case "统计证证号":
                            bl.setStatisticalNo(content);
                            break;
                        case "名称":
                            bl.setEnterpriseFullName(content);
                            break;
                        case "类型":
                            bl.setCompanyType(content);
                            break;
                        case "住所":
                            bl.setRegisteredAddress(content);
                            break;
                        case "法定代表人":
                            bl.setLegalPerson(content);
                            break;
                        case "注册资本":
                            bl.setRegisteredCapital(content);
                            break;
                        case "成立日期":
                            bl.setRegistrationTime(StringUtils.isEmpty(content) ? null : format.parse(content));
                            break;
                        case "营业期限":
                            bl.setBusinessTerm(content);
                            break;
                        case "经营范围":
                            bl.setMainBusinessScope(content);
                            break;
                        case "登记机关":
                            bl.setRegistrationAuthority(content);
                            break;
                        case "登记日期":
                            bl.setDateOfRegistration(StringUtils.isEmpty(content) ? null : format.parse(content));
                            break;
                        case "二维码":
                            bl.setQrCode(content);
                            break;
                        case "副本":
                            bl.setCopy(content);
                            break;
                    }
                } catch (ParseException e) {
                    throw new RuntimeException(e);
                }
            }
        }
        return bl;
    }

    private JSONObject getOCR(String passportUrl, OCREnum ocrEnum, Integer type) {
        if (!redisUtil.hasKey(ocrEnum.getTokenName())) {
            this.getOCRCurrencyToken(ocrEnum);
        }
        String apptoken = (String) redisUtil.get(ocrEnum.getTokenName());

        String img = this.picForScale(passportUrl);
        String deskey = ocrEnum.getDeskey();
        JSONObject body = new JSONObject();
        body.put("type", type);
        String bodyString = body.toJSONString();
        JSONObject other = new JSONObject();
        other.put("idcardImg", img);
        Object otherString = other.clone();
        //加密
        String encrypt = DESUtil.encrypt(bodyString, deskey);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("body", encrypt);
        jsonObject.put("other", otherString);
        jsonObject.put("token", apptoken);

        JSONObject jsonData = this.getCurrencyJSONObject(jsonObject, ocrEnum.getRequsetAdress());
        JSONObject header = jsonData.getJSONObject("header");
        if (header.get("code").toString().equals("1000")) {
            getOCRCurrencyToken(ocrEnum);
            jsonData = getOCR(passportUrl, ocrEnum, type);
            header = jsonData.getJSONObject("header");
        }
        if (header.get("code").toString().equals("99")) {
            throw new ServiceException("通用请求错误");
        } else if (header.get("code").toString().equals("1001")) {
            throw new ServiceException("token的获取次数超过限制");
        }
        return jsonData;
    }

    public void getOCRCurrencyToken(OCREnum ocrEnum) {
        String appid = ocrEnum.getAppid();
        String appsecret = ocrEnum.getAppsecret();
        // post请求，分为两种，一种是普通表单提交，一种是json提交
        JSONObject jsonObject = new JSONObject();
        JSONObject jsonObject1 = new JSONObject();
        jsonObject1.put("appid", appid);
        jsonObject1.put("secret", appsecret);
        jsonObject1.put("token", "");
        jsonObject.put("body", jsonObject1);
        String s = jsonObject.toJSONString();
        String sync = OkHttpUtils.builder().url(ocrEnum.getTokenAddress())
                .postString(s)
                .sync();
        JSONObject jsonData = JSON.parseObject(sync);
        log.info("返回值；" + jsonData);
        JSONObject body = jsonData.getJSONObject("body");
        if (body != null) {
            redisUtil.set(ocrEnum.getTokenName(), body.getString("token"), 7200);
        }
    }

    private JSONObject getCurrencyJSONObject(JSONObject jsonObject, String url) {
        String data = jsonObject.toJSONString();
        String sync = OkHttpUtils.builder().url(url)
                .postString(data)
                .sync();
        log.info("ocr识别返回数据{}",sync);
        return JSON.parseObject(sync);
    }

    /*图片压缩*/
    private String picForScale(String passportUrl) {
        byte[] bytes = null;
        String img = null;
        try {
            bytes = Base64Util.getByteArrayFromUrl(passportUrl);
            if (bytes == null) {
                throw new ServiceException("网络异常识别失败请重新再试！");
            }
            bytes = ThumbnailsUtil.compressPicForScale(bytes, 300);// 图片小于300kb
            img = Base64.encodeBase64String(bytes);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return img;
    }
}
