package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.yx.face.boot.config.HealthCodeProperties;
import com.yx.face.boot.rabbitConfig.config.SnQuenueConfig;
import com.yx.face.boot.restful.RestCode;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.dao.*;
import com.yx.face.model.dto.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.AdminAuthTypeEnum;
import com.yx.face.model.enums.AdminCodeEnum;
import com.yx.face.model.search.QueryExamUserWhitelistSearch;
import com.yx.face.model.vo.PlaceInfoVO;
import com.yx.face.model.vo.QueryExamUserWhitelistVO;
import com.yx.face.model.vo.third.InterfAddrJsonVo;
import com.yx.face.server.PlaceHealthCodeServer;
import com.yx.face.server.dto.DeviceBindManagerDTO;
import com.yx.face.server.dto.DevicePunchCardDTO;
import com.yx.face.server.dto.DeviceRegisterDTO;
import com.yx.face.server.dto.TokenDataDTO;
import com.yx.face.server.vo.HealthResultVO;
import com.yx.face.service.*;
import com.yx.face.service.job.HealthCodeTask;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.support.ListenerExecutionFailedException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/6/4 19:52
 * @description:
 */
@Service
@Slf4j
public class OpenIssuedServiceImpl implements OpenIssuedService {
    /**
     * 场所信息缓存
     */
    private static final String PLACE_SESSION_ID = "admin:place:info:";

    private static final String OPEN_PLACE_SESSION_ID = "open:admin:place:info:";

    /**
     * 默认常访客数量大小
     */
    private static final int SIZE = 200;
    @Autowired
    private UserInfoService userInfoService;
    @Autowired
    private UserWhitelistService userWhitelistService;
    @Autowired
    private UserVisitorListService userVisitorListService;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Resource
    private AdminDao adminDao;
    @Resource
    private AdminAuthDao adminAuthDao;
    @Autowired
    private Executor executor;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;
    @Resource
    private PlaceRecordDao placeRecordDao;
    @Autowired
    private RedisUtil redisUtil;
    @Resource
    private HealthCodeConfigDao healthCodeConfigDao;

    @Resource
    private ExamUserWhitelistDao examUserWhitelistDao;

    @Autowired
    private ZoneDeviceRService zoneDeviceRService;
    @Autowired
    private HealthCodeProperties healthCodeProperties;


    @Async
    @Override
    public RestResult issued(OpenIssuedDTO openIssuedDTO) {
        //删除人员
        this.delData(openIssuedDTO);
        //新增常客人员
        this.addWhiteData(openIssuedDTO);
        //新增访客人员
        this.addVisitorData(openIssuedDTO);
        return RestResponse.ok();
    }

    @Override
    public RestResult delData(OpenIssuedDTO openIssuedDTO) {
        List<OpenDelDataDTO> delData = openIssuedDTO.getDelData();
        if (CollectionUtil.isEmpty(delData) || delData.size() > SIZE) {
            log.warn("删除人员数据大小：{}", delData == null ? 0 : delData.size());
            return RestResponse.ok();
        }

        delData.forEach(v -> {
            try {
                Integer adminId = Integer.valueOf(openIssuedDTO.getAdminId().toString());
                //查询常客信息
                UserWhitelist userWhitelist = userWhitelistService.getByIdNumberAndId(v.getIdNumber(), null, adminId);
                if (userWhitelist == null && StrUtil.isNotEmpty(v.getPhone())) {
                    userWhitelist = userWhitelistService.getByPhoneAndId(v.getPhone(), null, adminId);
                }

                //查询访客信息
                UserVisitorList userVisitorList = userVisitorListService.selectByIdNumberAndAdminId(v.getIdNumber(), adminId);
                if (userVisitorList == null && StrUtil.isNotEmpty(v.getPhone())) {
                    userVisitorList = userVisitorListService.getByPhoneAndId(v.getPhone(), adminId);
                }

                if (userVisitorList != null) {
                    userVisitorListService.delete(userVisitorList.getId());
                }

                if (userWhitelist != null) {
                    userWhitelistService.delete(userWhitelist.getId(), Integer.valueOf(openIssuedDTO.getAdminId().toString()));
                }
            } catch (Exception e) {
                log.error("错误日志信息：{}, 删除用户信息：{}", e.getMessage(), JSON.toJSONString(v));
                e.printStackTrace();
            }
        });
        return RestResponse.ok();
    }

    @Override
    public RestResult addWhiteData(OpenIssuedDTO openIssuedDTO) {
        //常客数据
        List<OpenWhiteDataDTO> whiteData = openIssuedDTO.getWhiteData();
        if (CollectionUtil.isEmpty(whiteData) || whiteData.size() > SIZE) {
            log.warn("添加常客数据大小：{}", whiteData == null ? 0 : whiteData.size());
            return RestResponse.ok();
        }

        whiteData.forEach(v -> {
            try {
                if (StrUtil.isNotEmpty(v.getIdNumber())) {
                    v.setIdNumber(v.getIdNumber().trim());
                }

                if (StrUtil.isNotEmpty(v.getPhone())) {
                    v.setPhone(v.getPhone().trim());
                }

                String taskId = new StringBuilder().append(openIssuedDTO.getAdminId()).append(StrUtil.DASHED).append(System.currentTimeMillis()).toString();
                ConfirmFaceNotifyDTO confirmFaceNotifyDTO = new ConfirmFaceNotifyDTO();
                confirmFaceNotifyDTO.setTaskId(taskId).setName(v.getName()).setPhone(v.getPhone()).setIdNumber(v.getIdNumber()).setImg(v.getImg());
                Long userId = userInfoService.confirmFaceNotify(confirmFaceNotifyDTO);
                if (userId == null) {
                    log.error("创建用户人脸库失败，姓名：{}, 身份证：{}", v.getName(), v.getIdNumber());
                    return;
                }
                UserInfoInsertDTO userInfoInsertDTO = new UserInfoInsertDTO();
                userInfoInsertDTO.setUserId(userId);
                String admin = taskId.split("-")[0];
                userInfoInsertDTO.setAdminId(Integer.valueOf(admin));
                userInfoInsertDTO.setAction(2);
                userInfoInsertDTO.setType(AdminCodeEnum.REGULAR_CODE.getCode());
                //下发常客人脸信息
                userInfoService.userInfoInsert(userInfoInsertDTO);
            } catch (Exception e) {
                log.error("错误日志信息：{}, 添加常客用户信息：{}", e.getMessage(), JSON.toJSONString(v));
                e.printStackTrace();
            }
        });

        return RestResponse.ok();
    }

    @Override
    public RestResult addVisitorData(OpenIssuedDTO openIssuedDTO) {
        //常客数据
        List<OpenVisitorDataDTO> visitorData = openIssuedDTO.getVisitorData();
        if (CollectionUtil.isEmpty(visitorData) || visitorData.size() > SIZE) {
            log.warn("添加访客数据大小：{}", visitorData == null ? 0 : visitorData.size());
            return RestResponse.ok();
        }

        visitorData.forEach(v -> {
            try {
                if (StrUtil.isNotEmpty(v.getIdNumber())) {
                    v.setIdNumber(v.getIdNumber().trim());
                }

                if (StrUtil.isNotEmpty(v.getPhone())) {
                    v.setPhone(v.getPhone().trim());
                }

                String taskId = new StringBuilder().append(openIssuedDTO.getAdminId()).append(StrUtil.DASHED).append(System.currentTimeMillis()).toString();
                ConfirmFaceNotifyDTO confirmFaceNotifyDTO = new ConfirmFaceNotifyDTO();
                confirmFaceNotifyDTO.setTaskId(taskId).setName(v.getName()).setPhone(v.getPhone()).setIdNumber(v.getIdNumber()).setImg(v.getImg());
                Long userId = userInfoService.confirmFaceNotify(confirmFaceNotifyDTO);
                if (userId == null) {
                    log.error("创建用户人脸库失败，姓名：{}, 身份证：{}", v.getName(), v.getIdNumber());
                    return;
                }
                UserInfoInsertDTO userInfoInsertDTO = new UserInfoInsertDTO();
                userInfoInsertDTO.setUserId(userId);
                String admin = taskId.split("-")[0];
                userInfoInsertDTO.setAdminId(Integer.valueOf(admin));
                userInfoInsertDTO.setAction(2);
                //下发常客人脸信息
                userInfoService.userInfoInsert(userInfoInsertDTO);
            } catch (Exception e) {
                log.error("错误日志信息：{}, 添加访客用户信息：{}", e.getMessage(), JSON.toJSONString(v));
                e.printStackTrace();
            }
        });

        return RestResponse.ok();
    }

    @RabbitListener(queues = SnQuenueConfig.QUEUE_NAME)
    public void toClearUserInfo(String sn) {
        log.info("接收到的sn:{}",sn);
        /*
        * 1.查询设备对应配置
        * 2.传给联动平台
        * 3.联动平台成功之后，告诉rabbitmq消费成功，反之消费失败
        * */
        RestResult<PlaceInfoVO> restResult = this.queryPlaceInfoBySn(sn);
        if(restResult.code != RestCode.SUCCESS.code){
            restResult.setMsg(sn);
        }
        for (String url : healthCodeProperties.getUrlList()) {
            //放入
            String key  = sn+url;
            HealthCodeTask.hashMap.put(key,new HealthCodeTaskDto(sn,url,12,DateUtil.offsetMinute(new Date(),5)));
            try {
                HttpResponse response = HttpRequest.post(url)
                        .body(JSON.toJSONString(restResult)).timeout(3000).execute();
                String result = response.body();
                RestResult resp = JSON.parseObject(result, new TypeReference<RestResult>() {});
                log.info("sn配置联动平台缓存{}", JSON.toJSONString(resp));
                HealthCodeTask.hashMap.remove(key);
            }catch (ListenerExecutionFailedException exception){//重试用完
                log.info("重试次数用完");
            }

        }

    }

    @Override
    public RestResult<PlaceInfoVO> queryPlaceInfoBySn(String sn) {
        PlaceInfoVO placeInfoVO = new PlaceInfoVO();
        //查询设备信息
        FaceDevice faceDevice = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, sn)).build());
        if (Preconditions.isBlank(faceDevice)) {
            return RestResponse.error(RestCode.DEVICE_NOT_FOUND, placeInfoVO, RestCode.DEVICE_NOT_FOUND.getMsg());
        }
        if (faceDevice.getAuth() == null || faceDevice.getAuth() == 0) {
            return RestResponse.error(RestCode.UNAUTHORIZED, placeInfoVO, RestCode.UNAUTHORIZED.getMsg());
        }
        placeInfoVO.setSn(sn);
        //查询账户信息
        Admin admin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
        placeInfoVO.setScenicFlag(admin.getScenicFlag());
        placeInfoVO.setNoXcOpenDoor(admin.getNoXcOpenDoor());
        placeInfoVO.setNoHsOpenDoor(admin.getNoHsOpenDoor());
        placeInfoVO.setRiskXcOpenDoor(admin.getRiskXcOpenDoor());
        placeInfoVO.setNoAllowAreaCode(admin.getNoAllowAreaCode());
        placeInfoVO.setXcNeedCheckOpenDoor(admin.getXcNeedCheckOpenDoor());
        placeInfoVO.setLimitGather(admin.getLimitGather());
        placeInfoVO.setAdminId(admin.getId());
        placeInfoVO.setCityCode(String.valueOf(admin.getCityId()));
        placeInfoVO.setIsOpenHealthCode(admin.getIsOpenHealthCode());
        placeInfoVO.setShowQrcode(admin.getShowQrcode());
        placeInfoVO.setQrcodeText(admin.getQrcodeText());
        placeInfoVO.setAdvertise(admin.getAdvertise());
        placeInfoVO.setTopTitle(admin.getTopTitle());
        placeInfoVO.setNotAllowAge(admin.getNotAllowAge());
        placeInfoVO.setNotAllowAgeMsg(admin.getNotAllowAgeMsg());
        placeInfoVO.setIsAlarm(admin.getIsAlarm());
        placeInfoVO.setPlaceType(admin.getPlaceType());
        //核酸有效期时间
        placeInfoVO.setNucleicExpirationTime(admin.getNucleicExpirationTime());
        /*wxg 2023-4-14 考试设备列表*/
        placeInfoVO.setIsOpenExam(admin.getIsOpenExam());
        placeInfoVO.setExamDevices(admin.getExamDevices());

        /**
         * ------ 账户添加配置信息2023-04-25 ------
         **/
        placeInfoVO.setOpenThirdLinkPlatform(admin.getOpenThirdLinkPlatform());
        placeInfoVO.setSubLineSuccess(admin.getSubLineSuccess());
        placeInfoVO.setSubLineFail(admin.getSubLineFail());
        placeInfoVO.setMainLineLeft(admin.getMainLineLeft());
        placeInfoVO.setSubLineLeft(admin.getSubLineLeft());
        placeInfoVO.setMainLineRight(admin.getMainLineRight());
        placeInfoVO.setSubLineRight(admin.getSubLineRight());
        placeInfoVO.setOpenVoiceMsgSuccess(admin.getOpenVoiceMsgSuccess());
        placeInfoVO.setOpenVoiceMsgFail(admin.getOpenVoiceMsgFail());
        placeInfoVO.setLatestResultPage(Preconditions.isBlank(admin.getLatestResultPage()) ? Boolean.FALSE : admin.getLatestResultPage());
        //获取三方api参数和接口地址
        completeAdminAuthInfo(placeInfoVO, admin, sn);

        return RestResponse.ok(placeInfoVO);
    }

    private void completeAdminAuthInfo(PlaceInfoVO placeInfoVO, Admin admin, String sn) {
        try {
            if (Preconditions.isNotBlank(admin.getOpenThirdLinkPlatform()) && admin.getOpenThirdLinkPlatform()) {
                AdminAuth adminAuth = adminAuthDao.selectOneByExample(Example.builder(AdminAuth.class)
                        .where(WeekendSqls.<AdminAuth>custom()
                                .andEqualTo(AdminAuth::getAdminId, admin.getId())
                                .andEqualTo(AdminAuth::getType, AdminAuthTypeEnum.M_PLATFORM.getCode())
                                .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)
                        ).build());
                if (Preconditions.isNotBlank(adminAuth)) {
                    //获取设备所在区域
                    Integer zoneId = zoneDeviceRService.getZoneIdBySn(admin.getId(), sn);
                    //获取设备接口配置
                    List<InterfAddrJsonVo> vos = null;
                    InterfAddrJsonVo vo = null;
                    if (Preconditions.isNotBlank(adminAuth.getInterfAddrJson())) {
                        vos = JSONObject.parseArray(adminAuth.getInterfAddrJson(), InterfAddrJsonVo.class);

                        List<InterfAddrJsonVo> filterZ = vos.stream().filter(z -> Preconditions.isNotBlank(z.getZoneId()) && z.getZoneId().intValue() != 0).collect(Collectors.toList());
                        if (Preconditions.isNotBlank(zoneId) && Preconditions.isNotBlank(filterZ)) {
                            for (InterfAddrJsonVo item : filterZ) {
                                if (item.getZoneId().intValue() == zoneId.intValue()) {
                                    vo = item;
                                }
                            }
                        }

                        List<InterfAddrJsonVo> filterNZ = vos.stream().filter(z -> Preconditions.isBlank(z.getZoneId()) || z.getZoneId().intValue() == 0).collect(Collectors.toList());
                        if (Preconditions.isBlank(zoneId) && Preconditions.isNotBlank(filterNZ)) {
                            vo = filterNZ.get(0);
                        }
                    }

                    placeInfoVO.setAppKey(adminAuth.getAppKey());
                    placeInfoVO.setAppSecret(adminAuth.getAppSecret());
                    placeInfoVO.setPrivateKey(adminAuth.getPrivateKey());
                    placeInfoVO.setInterfaceAddress(Preconditions.isNotBlank(vo) ? vo.getInterfAddr() : Preconditions.isNotBlank(adminAuth.getInterfaceAddress()) ? adminAuth.getInterfaceAddress() : null);
                }
            }
        } catch (Exception e) {
            log.error("--->获取授权三方API参数和接口地址失败");
        }

    }

    /**
     * 设备注册
     *
     * @param faceDevice
     * @param adminPlaceInfo
     * @return
     */
    @Override
    public Boolean deviceRegister(FaceDevice faceDevice, AdminPlaceInfo adminPlaceInfo) {
        return true;
    }

    @Override
    public QueryExamUserWhitelistVO getExamUserWhitelist(QueryExamUserWhitelistSearch dto) {
        log.info("查询考试人员信息入参:{}", JSON.toJSONString(dto));
        if (dto == null || StrUtil.isBlank(dto.getIdNumber()) || StrUtil.isBlank(dto.getSn())) {
            throw new ServiceException("数据有误，请传参");
        }
        //查询设备信息
        FaceDevice faceDevice = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, dto.getSn())).build());
        if (Preconditions.isBlank(faceDevice)) {
            throw new ServiceException("设备不存在");
        }
        Date date = new Date();
        String nowTime = DateUtil.format(date, DatePattern.NORM_DATETIME_PATTERN);
        ExamUserWhitelist examUserWhitelist = examUserWhitelistDao.
                selectByAdminIdAndIdNumber(
                        faceDevice.getAdminId(),
                        dto.getIdNumber(),
                        nowTime);
        QueryExamUserWhitelistVO queryExamUserWhitelistVO = null;
        if (examUserWhitelist != null) {
            queryExamUserWhitelistVO = BeanUtil.copyProperties(examUserWhitelist, QueryExamUserWhitelistVO.class);
            queryExamUserWhitelistVO.setIsCanIn(Boolean.TRUE);
   /*         if(examUserWhitelist.getStatus().equals(0)){
                examUserWhitelist.setStatus(1);
                examUserWhitelist.setInTime(date);
                examUserWhitelistDao.updateByPrimaryKeySelective(examUserWhitelist);
            }*/
        } else {
            return null;
        }

        return queryExamUserWhitelistVO;
    }

    @Override
    public RestResult<PlaceInfoVO> queryOpenPlaceInfoBySn(String sn) {
        PlaceInfoVO placeInfoVO = new PlaceInfoVO();
        String sessionSnPlaceKey = new StringBuilder().append(OPEN_PLACE_SESSION_ID).append(sn).toString();
        //查询设备绑定场所码信息
        HealthCodeConfig config = healthCodeConfigDao.selectOneByExample(Example.builder(HealthCodeConfig.class).where(WeekendSqls.<HealthCodeConfig>custom().andEqualTo(HealthCodeConfig::getSn, sn)).build());
        if (config == null) {
            return RestResponse.error("设备暂无调用权限，请联系管理员");
        }

        if (config.getRequestNumberLimit() != null
                && config.getRequestNumberLimit().intValue() != 0
                && config.getRequestNumberLimit().intValue() - config.getRequestedTimes().intValue() > 0) {
            healthCodeConfigDao.increaseRequestedTimesById(config.getId());
        }

        if (config != null && !config.getEnableStatus()) {
            //若无开启场所码，不走以下流程
            return RestResponse.ok(placeInfoVO.setEnableStatus(config.getEnableStatus())
                    .setRequestNumber(config.getRequestNumber())
                    .setRequestNumberLimit(config.getRequestNumberLimit())
                    .setRequestedTimes(config.getRequestedTimes()));
        }

        executor.execute(() -> {
            if (!config.getRegisterStatus()) {
                //设备注册
                HealthResultVO registerResultVO = placeHealthCodeServer.deviceRegister(new DeviceRegisterDTO().setDeviceId(config.getSn()).setDeviceName(config.getSn()).setAddressCode(config.getAddressCode()));
                if (registerResultVO.getSuccess() && registerResultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                    //更新设备注册状态
                    healthCodeConfigDao.updateByPrimaryKeySelective(config.setRegisterStatus(true));
                }
            }

            Object sessionPlaceInfo = redisUtil.get(sessionSnPlaceKey);
            //判断设备是否绑定管理人员 是否绑定场所码
            if (sessionPlaceInfo == null) {
                //设备绑定管理人员
                HealthResultVO managerResultVO = placeHealthCodeServer.deviceBindManager(new DeviceBindManagerDTO().setDeviceId(config.getSn()).setHealthCodeQrCode(config.getManagerHealthCode()));
                //绑定地址码
                HealthResultVO punchResultVO = placeHealthCodeServer.devicePunchCard(new DevicePunchCardDTO().setDeviceId(config.getSn()).setDeviceName(config.getSn()).setAddressCode(config.getAddressCode()));

                if (managerResultVO.getSuccess() && managerResultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode())) && punchResultVO.getSuccess() && punchResultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                    //缓存设备sn历史场所码信息
                    redisUtil.set(sessionSnPlaceKey, JSON.toJSONString(config));
                }
            } else {
                //绑定管理员、场所码标识
                boolean successFlag = true;
                //比对历史缓存场所码信息和最新比较处理
                HealthCodeConfig placeInfo = JSON.parseObject(sessionPlaceInfo.toString(), HealthCodeConfig.class);
                if (placeInfo.getManagerHealthCode() != null && !placeInfo.getManagerHealthCode().equals(config.getManagerHealthCode())) {
                    //设备绑定管理人员
                    HealthResultVO managerResultVO = placeHealthCodeServer.deviceBindManager(new DeviceBindManagerDTO().setDeviceId(config.getSn()).setHealthCodeQrCode(config.getManagerHealthCode()));
                    if (!managerResultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                        successFlag = false;
                    }
                }

                if (placeInfo.getAddressCode() != null && !placeInfo.getAddressCode().equals(config.getAddressCode())) {
                    //绑定地址码
                    HealthResultVO punchResultVO = placeHealthCodeServer.devicePunchCard(new DevicePunchCardDTO().setDeviceId(config.getSn()).setDeviceName(config.getSn()).setAddressCode(config.getAddressCode()));
                    if (!punchResultVO.getCode().equals(Integer.valueOf(RestCode.SUCCESS.getCode()))) {
                        successFlag = false;
                    }
                }

                if (successFlag) {
                    //缓存设备sn历史场所码信息
                    redisUtil.set(sessionSnPlaceKey, JSON.toJSONString(config));
                }
            }
        });

        return RestResponse.ok(placeInfoVO.setEnableStatus(config.getEnableStatus())
                .setAddressCode(config.getAddressCode())
                .setManagerHealthCode(config.getManagerHealthCode())
                .setUnitName(config.getUnitName())
                .setUnitAddress(config.getUnitAddress())
                .setRequestNumber(config.getRequestNumber())
                .setRequestNumberLimit(config.getRequestNumberLimit())
                .setRequestedTimes(config.getRequestedTimes()));
    }

    @Override
    public Boolean savePlaceRecord(PlaceRecordDTO dto) {
        placeRecordDao.insert(dto);
        return true;
    }

    @Override
    public HealthResultVO<TokenDataDTO> getToken(Boolean cleanSession, Integer placeAreaCode) {
        return placeHealthCodeServer.getToken(cleanSession, placeAreaCode);
    }

    @Override
    public List<FaceDevice> queryFaceDevices(Long adminId) {
        return faceDeviceDao.selectByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getAdminId, adminId)).build());
    }

    @Override
    public RestResult saveHealthCodeConfig(HealthCodeConfigDTO configDTO) {
        //查询设备绑定场所码信息
        HealthCodeConfig config = healthCodeConfigDao.selectOneByExample(Example.builder(HealthCodeConfig.class).where(WeekendSqls.<HealthCodeConfig>custom().andEqualTo(HealthCodeConfig::getSn, configDTO.getSn())).build());
        if (config == null) {
            return RestResponse.error("设备不存在，请联系管理员");
        }

        if (!config.getEnableStatus()) {
            return RestResponse.error("设备未开启场所码，请联系管理员");
        }

        healthCodeConfigDao.updateByPrimaryKeySelective(config.setAddressCode(configDTO.getAddressCode()).setUnitName(configDTO.getUnitName()).setUnitAddress(configDTO.getUnitAddress()).setManagerHealthCode(configDTO.getManagerHealthCode()));
        /*在设备绑定场所码的时候就去注册到场所码中*/
        this.queryOpenPlaceInfoBySn(config.getSn());
        return RestResponse.ok();
    }

}
