package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.google.common.collect.Lists;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.dao.PlaceDao;
import com.yx.face.dao.PlaceDetailDao;
import com.yx.face.model.dto.PlaceDTO;
import com.yx.face.model.dto.PlaceDetailDTO;
import com.yx.face.model.entity.Place;
import com.yx.face.model.entity.PlaceDetail;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.PlaceVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.PlaceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-05-24
 */
@Service
public class PlaceServiceImpl implements PlaceService {
    @Resource
    private PlaceDao placeDao;
    @Resource
    private PlaceDetailDao placeDetailDao;
    @Autowired
    private AdminService adminService;

    @Override
    public PlaceVO query() {
        PlaceVO placeVO = new PlaceVO();
        //当前账号信息
        AdminVO info = adminService.getInfo();
        //场所信息
        Place place = placeDao.selectOneByExample(Example.builder(Place.class).where(WeekendSqls.<Place>custom().andEqualTo(Place::getAdminId, info.getAdminId())).build());

        if(place == null){
            return placeVO;
        }

        BeanUtil.copyProperties(place, placeVO);
        //场所预约信息
        List<PlaceDetail> placeDetails = placeDetailDao.selectByExample(Example.builder(PlaceDetail.class).where(WeekendSqls.<PlaceDetail>custom().andEqualTo(PlaceDetail::getPlaceId, place.getId()).andEqualTo(PlaceDetail::getDelFlag, false)).build());
        if(CollectionUtil.isEmpty(placeDetails)){
            placeVO.setPlaceDetailDTOList(Lists.newArrayList(new PlaceDetailDTO()));
            return placeVO;
        }
        List<PlaceDetailDTO> list = new LinkedList<>();
        for (PlaceDetail placeDetail : placeDetails) {
            PlaceDetailDTO placeDetailDTO = BeanUtil.copyProperties(placeDetail, PlaceDetailDTO.class);
            placeDetailDTO.setPlaceDetailId(placeDetail.getId());
            placeDetailDTO.setStartTime(placeDetail.getStartTime());
            placeDetailDTO.setEndTime(placeDetail.getEndTime());
            placeDetailDTO.setPlaceDetailId(placeDetail.getId());
            placeDetailDTO.setCreateTime(DateUtil.formatDateTime(placeDetail.getCreateTime()));
            if(placeDetail.getLimitNumber() != null && placeDetail.getRemainingNumber() != null){
                placeDetailDTO.setAppointmentNumber(placeDetail.getLimitNumber() - placeDetail.getRemainingNumber());
            }
            list.add(placeDetailDTO);
        }
        placeVO.setPlaceDetailDTOList(list);

//        Map<String, List<PlaceDetail>> collect = placeDetails.stream().collect(Collectors.groupingBy(PlaceDetail::getDateTimeStr));
//        PlaceDetail placeDetail = placeDetails.stream().findFirst().get();
//        placeVO.setInstructions(placeDetail.getInstructions());
//        placeVO.setDateTimeRange(StrUtil.isEmpty(placeDetail.getDateTimeRange()) ? null : Arrays.stream(placeDetail.getDateTimeRange().split(",")).collect(Collectors.toList()));
//        placeVO.setPlaceDetailDTOList(collect.get(placeDetail.getDateTimeStr()) == null ? Lists.newArrayList(new PlaceDetailDTO()) : Convert.toList(PlaceDetailDTO.class, collect.get(placeDetail.getDateTimeStr())));
        return placeVO;
    }

    @Override
    public Boolean save(PlaceDTO dto) {

        //当前账号信息
        AdminVO info = adminService.getInfo();

        //场所信息
        Place place = placeDao.selectOneByExample(Example.builder(Place.class).where(WeekendSqls.<Place>custom().andEqualTo(Place::getAdminId, info.getAdminId())).build());
        if(place == null){
            place = new Place();
            place.setDisableIdCard(dto.getDisableIdCard());
            place.setInstructions(dto.getInstructions());
            place.setTitle(dto.getTitle());
            place.setPicture(dto.getPicture());
            place.setAdminId(info.getAdminId()).setCreateTime(DateUtil.date());
            this.insert(place);
        }else {
            place.setDisableIdCard(dto.getDisableIdCard());
            place.setInstructions(dto.getInstructions());
            place.setTitle(dto.getTitle());
            place.setPicture(dto.getPicture());
            this.update(place);
        }
/*        if(CollectionUtil.isEmpty(dto.getDateTimeRange())){
            throw new ServiceException("场所预约日期时间段不允许为空");
        }

        dto.setPlaceDetailDTOList(dto.getPlaceDetailDTOList() == null ? Lists.newArrayList(new PlaceDetailDTO()) : dto.getPlaceDetailDTOList());
        dto.getPlaceDetailDTOList().forEach(v -> {
            if(StrUtil.isEmpty(v.getStartTimeStr()) || StrUtil.isEmpty(v.getEndTimeStr()) || v.getLimitNumber() == null){
                throw new ServiceException("场所预约开始或者结束时段或者限定人数不允许为空");
            }
        });

        //当前账号信息
        AdminVO info = adminService.getInfo();
        Place place = new Place();
        BeanUtil.copyProperties(dto, place);

        if(dto.getId() == null){
            place.setAdminId(info.getAdminId()).setCreateTime(DateUtil.date());
            this.insert(place);
            dto.setId(place.getId());
        }else{
            this.update(place);
        }

        //删除场所历史预约记录信息
        placeDetailDao.updateByExampleSelective(new PlaceDetail().setDelFlag(true), Example.builder(PlaceDetail.class).where(WeekendSqls.<PlaceDetail>custom().andEqualTo(PlaceDetail::getPlaceId, place.getId())).build());

        if(CollectionUtil.isEmpty(dto.getDateTimeRange())){
            return true;
        }

        List<PlaceDetail> placeDetails = Lists.newArrayList();
        dto.getDateTimeRange().stream().forEach(v -> {
            dto.getPlaceDetailDTOList().forEach(detail -> {
                //开始时间
                String startTimeStr = new StringBuilder().append(v).append(StrUtil.SPACE).append(detail.getStartTimeStr()).toString();
                //结束时间
                String endTimeStr = new StringBuilder().append(v).append(StrUtil.SPACE).append(detail.getEndTimeStr()).toString();
                PlaceDetail placeDetail = new PlaceDetail().setFullTimeStr(new StringBuilder().append(startTimeStr).append(StrUtil.COMMA).append(endTimeStr).toString()).setStartTime(DateUtil.parse(startTimeStr, DatePattern.NORM_DATETIME_MINUTE_PATTERN)).setEndTime(DateUtil.parse(endTimeStr, DatePattern.NORM_DATETIME_MINUTE_PATTERN)).setLimitNumber(detail.getLimitNumber())
                        .setRemainingNumber(detail.getLimitNumber()).setPlaceId(dto.getId()).setAdminId(info.getAdminId()).setCreateTime(DateUtil.date()).setDelFlag(false).setInstructions(dto.getInstructions()).setDateTimeStr(v).setDateTimeRange(dto.getDateTimeRange().stream().collect(Collectors.joining(","))).setStartTimeStr(detail.getStartTimeStr()).setEndTimeStr(detail.getEndTimeStr());
                placeDetails.add(placeDetail);
            });
        });

        //新增场所预约记录信息
        placeDetailDao.insertList(placeDetails);*/
        return true;
    }

    @Override
    public Boolean insert(Place place) {
        placeDao.insertUseGeneratedKeys(place);
        return true;
    }

    @Override
    public Boolean update(Place place) {
        placeDao.updateByPrimaryKey(place);
        return true;
    }

    @Override
    public Boolean appointmentSave(PlaceDetailDTO placeDetailDTO) {
        if(placeDetailDTO.getStartTime() != null && placeDetailDTO.getEndTime() != null){
            String substring = DateUtil.format(placeDetailDTO.getStartTime(), DatePattern.NORM_DATE_PATTERN);
            String substring1 = DateUtil.format(placeDetailDTO.getEndTime(), DatePattern.NORM_DATE_PATTERN);

            if(!substring.equals(substring1)){
                throw new ServiceException("开始时间-结束时间需在同一天");
            }
        }else {
            throw new ServiceException("开始时间-结束时间不能为空");
        }
        PlaceDetail placeDetail = BeanUtil.copyProperties(placeDetailDTO, PlaceDetail.class);
        //当前账号信息
        AdminVO info = adminService.getInfo();
        Place place = placeDao.selectOneByExample(Example.builder(Place.class).where(WeekendSqls.<Place>custom().andEqualTo(Place::getAdminId, info.getAdminId())).build());
        if(place == null){
            throw new ServiceException("请先添加基础数据");
        }
        placeDetail.setLimitNumber(placeDetailDTO.getLimitNumber());
        placeDetail.setRemainingNumber(placeDetailDTO.getLimitNumber());

        placeDetail.setPlaceId(place.getId());
        placeDetail.setCreateTime(new Date());
        placeDetail.setAdminId(info.getAdminId());
        placeDetailDao.insertSelective(placeDetail);
        return true;
    }

    @Override
    public Boolean appointmentDelete(Integer placeDetailId) {
        PlaceDetail placeDetail = placeDetailDao.selectByPrimaryKey(placeDetailId);
        if(placeDetail == null){
            throw new ServiceException("数据不存在");
        }
        placeDetail.setDelFlag(true);
        placeDetailDao.updateByPrimaryKeySelective(placeDetail);
        return true;
    }

    @Override
    public Boolean appointmentUpdate(PlaceDetailDTO placeDetailDTO) {
        if(placeDetailDTO.getPlaceDetailId() == null){
            throw new ServiceException("请选择要修改的数据");
        }
        PlaceDetail placeDetail = new PlaceDetail();
        placeDetail.setId(placeDetailDTO.getPlaceDetailId());
        placeDetail.setStatus(placeDetailDTO.getStatus());
        placeDetailDao.updateByPrimaryKeySelective(placeDetail);
        return true;
    }
}
