package com.yx.face.service.impl;

import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.dao.PrivateAreaDao;
import com.yx.face.dao.PrivateAreaDeviceDao;
import com.yx.face.model.dto.PrivateAreaDeviceDTO;
import com.yx.face.model.entity.PrivateArea;
import com.yx.face.model.entity.PrivateAreaDevice;
import com.yx.face.model.search.PrivateAreaDeviceSearch;
import com.yx.face.model.vo.FaceDeviceVO;
import com.yx.face.model.vo.PrivateAreaDeviceVO;
import com.yx.face.service.PrivateAreaDeviceService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 私有区域设备关联表 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-06-29
 */
@Service
public class PrivateAreaDeviceServiceImpl implements PrivateAreaDeviceService {
    @Resource
    private PrivateAreaDeviceDao privateAreaDeviceDao;
    @Resource
    private PrivateAreaDao privateAreaDao;

    @Override
    public RestResult<PageInfo<PrivateAreaDeviceVO>> privateAreaDevicePageList(RestDTO<PrivateAreaDeviceSearch> dto) {
        PrivateAreaDeviceSearch data = dto.getData();
        if(data.getPrivateAreaId() == null)throw new ServiceException("请传入私有区域id");
        List<PrivateAreaDeviceVO> list = privateAreaDeviceDao.privateAreaDevicePageList(data);
        return RestResponse.ok(new PageInfo<>(list));
    }

    @Override
    public RestResult<List<FaceDeviceVO>> getPrivateAreaDeviceList(Integer privateAreaId) {
        List<FaceDeviceVO> list = privateAreaDeviceDao.getPrivateAreaDeviceList(privateAreaId);
        return RestResponse.ok(list);
    }

    @Override
    @Transactional
    public Integer addPrivateAreaDevice(PrivateAreaDeviceDTO privateAreaDeviceDTO) {
        if (privateAreaDeviceDTO.getDeviceIdList() == null || privateAreaDeviceDTO.getDeviceIdList().size() ==0 || privateAreaDeviceDTO.getPrivateAreaId() == null){
            throw new ServiceException("数据不能空");
        }
        PrivateArea privateArea = privateAreaDao.selectByPrimaryKey(privateAreaDeviceDTO.getPrivateAreaId());
        List<Integer> deviceIdList = privateAreaDeviceDTO.getDeviceIdList();
        for (Integer deviceId : deviceIdList) {
            PrivateAreaDevice privateAreaDevice = new PrivateAreaDevice();
            privateAreaDevice.setPrivateAreaId(privateAreaDeviceDTO.getPrivateAreaId());
            privateAreaDevice.setDeviceId(deviceId);
            PrivateAreaDevice privateAreaDevice1 = privateAreaDeviceDao.selectOne(privateAreaDevice);
            if(privateAreaDevice1 != null){
                throw new ServiceException("该设备已被添加到该私有区域中");
            }
            privateAreaDevice.setAdminId(privateArea.getAdminId());
            privateAreaDevice.setCreateTime(new Date());
            privateAreaDeviceDao.insertSelective(privateAreaDevice);
        }


        return 1;
    }

    @Override
    public Integer deletePrivateAreaDevice(Integer privateAreaDeviceId) {
        return privateAreaDeviceDao.deleteByPrimaryKey(privateAreaDeviceId);
    }
}
