package com.yx.face.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.PrivateAreaDao;
import com.yx.face.dao.PrivateAreaDeviceDao;
import com.yx.face.dao.PrivateAreaWhitelistDao;
import com.yx.face.model.dto.PrivateAreaDTO;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.PrivateArea;
import com.yx.face.model.entity.PrivateAreaDevice;
import com.yx.face.model.entity.PrivateAreaWhitelist;
import com.yx.face.model.enums.AdminIsOpenPrivateAreaEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.search.PrivateAreaSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.PrivateAreaVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.PrivateAreaService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 私有区域表 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-06-29
 */
@Service
public class PrivateAreaServiceImpl implements PrivateAreaService {

    @Autowired
    private AdminService adminService;

    @Resource
    private PrivateAreaDao privateAreaDao;
    @Resource
    private AdminDao adminDao;
    @Resource
    private PrivateAreaWhitelistDao privateAreaWhitelistDao;
    @Resource
    private PrivateAreaDeviceDao privateAreaDeviceDao;

    @Override
    public RestResult<PageInfo<PrivateAreaVO>> privateAreaPageList(RestDTO<PrivateAreaSearch> dto) {
        AdminVO info = adminService.getInfo();
        /*1.根据不同账号查询不同数据
            ①如果是楼宇级别的账号，需要查询出他的子级账号
            ②企业账号，则只有他本身
        */
        List<Integer> adminList = null;
        if(info.getType().equals(AdminTypeEnum.DISTRICT.getCode())){
            Admin admin = new Admin();
            admin.setTenantId(info.getAdminId());
            List<Admin> admins = adminDao.select(admin);
            if(admins == null ||admins.size() == 0)return RestResponse.ok(new PageInfo<>(new ArrayList<>()));
            adminList = admins.stream().map(Admin::getId).collect(Collectors.toList());
        }else {//企业账号
            adminList = new ArrayList<>();
            adminList.add(info.getAdminId());
        }
        /*2.分页查询*/
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<PrivateAreaVO> list = privateAreaDao.privateAreaPageList(dto.getData(),adminList);
        return RestResponse.ok(new PageInfo<>(list));
    }

    @Override
    public Integer addOrUpdatePrivateArea(PrivateAreaDTO privateAreaDTO) {
        AdminVO info2 = adminService.getInfo();
        if(info2.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())){
            privateAreaDTO.setAdminId(info2.getAdminId());
        }else {
            if(privateAreaDTO.getAdminId() == null){
                throw new ServiceException("请选择账号");
            }
        }
        Integer adminId = privateAreaDTO.getAdminId();
        AdminVO info1 = adminService.getInfo(adminId);
        if(info1.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.CLOSE_PRIVATE_AREA.getCode())){
            throw new ServiceException("绑定的账号需要开启私有区域才能添加私有区域");
        }

        AdminVO info = adminService.getInfo();
        PrivateArea pa = new PrivateArea();
        pa.setAdminId(privateAreaDTO.getAdminId());
        pa.setPrivateAreaName(privateAreaDTO.getPrivateAreaName());
        PrivateArea privateArea1 = privateAreaDao.selectOne(pa);
        if(privateArea1 != null)throw new ServiceException("私有区域名称不能重复");
        Date date = new Date();
        PrivateArea privateArea = new PrivateArea();
        BeanUtils.copyProperties(privateAreaDTO,privateArea);
        if (privateAreaDTO.getPrivateAreaId() == null){
            if(privateAreaDTO.getAdminId() == null){
                if(info.getType().equals(AdminTypeEnum.DISTRICT.getCode())){
                    throw new ServiceException("楼宇账号需要填入adminId");
                }else {
                    privateArea.setAdminId(info.getAdminId());
                }
            }
            privateArea.setCreateTime(date);
            privateAreaDao.insertSelective(privateArea);
        }else {
            privateArea.setId(privateAreaDTO.getPrivateAreaId());
            privateAreaDao.updateByPrimaryKeySelective(privateArea);
        }
        return 1;
    }

    @Override
    @Transactional
    public Integer deletePrivateArea(Integer privateAreaId) {
        PrivateArea privateArea = privateAreaDao.selectByPrimaryKey(privateAreaId);
        if(privateArea == null)throw new ServiceException("数据不存在");
        /*删除私有区域的同时，删除私有区域的设备，和常客信息*/
        privateAreaDao.deleteByPrimaryKey(privateAreaId);
        privateAreaWhitelistDao.delete(new PrivateAreaWhitelist().setPrivateAreaId(privateAreaId));
        privateAreaDeviceDao.delete(new PrivateAreaDevice().setPrivateAreaId(privateAreaId));
        return 1;
    }

    @Override
    public RestResult<List<PrivateAreaVO>> getPrivateAreaList(Integer adminId) {
        AdminVO info = adminService.getInfo(adminId);
        if(!info.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())){
            throw new ServiceException("请输入企业的账号");
        }
        if(info.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.CLOSE_PRIVATE_AREA.getCode())){
            throw new ServiceException("该账号未开启私有区域");
        }
        List<PrivateArea> list = privateAreaDao.select(new PrivateArea().setAdminId(adminId));
        List<PrivateAreaVO> voList = new ArrayList<>();
        for (PrivateArea privateArea : list) {
            PrivateAreaVO privateAreaVO = new PrivateAreaVO();
            privateAreaVO.setPrivateAreaId(privateArea.getId());
            privateAreaVO.setPrivateAreaName(privateArea.getPrivateAreaName());
            voList.add(privateAreaVO);
        }
        return RestResponse.ok(voList);
    }


}
