package com.yx.face.service.impl;

import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.PrivateAreaDao;
import com.yx.face.dao.PrivateAreaWhitelistDao;
import com.yx.face.model.dto.PrivateAreaWhitelistDTO;
import com.yx.face.model.dto.PrivateAreaWhitelistListDTO;
import com.yx.face.model.entity.PrivateArea;
import com.yx.face.model.entity.PrivateAreaWhitelist;
import com.yx.face.model.search.PrivateAreaWhiteListSearch;
import com.yx.face.model.search.WhitelistPrivateAreaWhiteListSearch;
import com.yx.face.model.vo.*;
import com.yx.face.service.AdminService;
import com.yx.face.service.PrivateAreaWhitelistService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 私有区域人员表 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-06-29
 */
@Service
public class PrivateAreaWhitelistServiceImpl implements PrivateAreaWhitelistService {

    @Autowired
    private AdminService adminService;

    @Resource
    private PrivateAreaWhitelistDao privateAreaWhitelistDao;
    @Resource
    private PrivateAreaDao privateAreaDao;

    @Override
    public RestResult<PageInfo<PrivateAreaWhitelistVO>> privateAreaWhitelistPageList(RestDTO<PrivateAreaWhiteListSearch> dto) {

        PrivateAreaWhiteListSearch data = dto.getData();
        if(data.getPrivateAreaId() == null)throw new ServiceException("请传入私有区域id");
        List<PrivateAreaWhitelistVO> list = privateAreaWhitelistDao.privateAreaWhitelistPageList(data);
        AdminVO info = adminService.getInfo();
        boolean isShowName = false;
        boolean isShowPhone = false;
        boolean isShowNumber = false;
        if (info.getNoHideCloumn() !=null && info.getNoHideCloumn().size()>0){
            if(info.getNoHideCloumn().contains("1"))isShowName = true ;
            if(info.getNoHideCloumn().contains("2"))isShowPhone = true ;
            if(info.getNoHideCloumn().contains("3"))isShowNumber = true ;
        }
        for (PrivateAreaWhitelistVO privateAreaWhitelistVO : list) {
            if (StringUtils.isNotBlank(privateAreaWhitelistVO.getName()) && isShowName)
                privateAreaWhitelistVO.setName(SMSOrIdCardUtils.hideName(privateAreaWhitelistVO.getName()));
            if (StringUtils.isNotBlank(privateAreaWhitelistVO.getPhone()) && isShowPhone)
                privateAreaWhitelistVO.setPhone(SMSOrIdCardUtils.hidePhone(privateAreaWhitelistVO.getPhone()));
            if (StringUtils.isNotBlank(privateAreaWhitelistVO.getIdNumber()) && isShowNumber)
                privateAreaWhitelistVO.setIdNumber(SMSOrIdCardUtils.hideIdNumber(privateAreaWhitelistVO.getIdNumber()));
        }
        return RestResponse.ok(new PageInfo<>(list));
    }

    @Override
    public RestResult<List<UserWhitelistVO>> getPrivateAreaWhitelistList(PrivateAreaWhitelistListDTO privateAreaWhitelistListDTO) {
        List<UserWhitelistVO> list = privateAreaWhitelistDao.getPrivateAreaWhitelistList(privateAreaWhitelistListDTO);
        return RestResponse.ok(list);
    }

    @Override
    @Transactional
    public Integer addPrivateAreaWhitelist(PrivateAreaWhitelistDTO privateAreaWhitelistDTO) {
        if (privateAreaWhitelistDTO.getUserWhitelistIdList() == null || privateAreaWhitelistDTO.getUserWhitelistIdList().size() ==0  || privateAreaWhitelistDTO.getPrivateAreaId() == null){
            throw new ServiceException("数据不能空");
        }
        PrivateArea privateArea = privateAreaDao.selectByPrimaryKey(privateAreaWhitelistDTO.getPrivateAreaId());
        List<Integer> userWhitelistIdList = privateAreaWhitelistDTO.getUserWhitelistIdList();
        for (Integer userWhitelistId : userWhitelistIdList) {
            PrivateAreaWhitelist privateAreaWhitelist = new PrivateAreaWhitelist();
            privateAreaWhitelist.setPrivateAreaId(privateAreaWhitelistDTO.getPrivateAreaId());
            privateAreaWhitelist.setUserWhitelistId(userWhitelistId);
            PrivateAreaWhitelist privateAreaWhitelist1 = privateAreaWhitelistDao.selectOne(privateAreaWhitelist);
            if(privateAreaWhitelist1 != null){
                throw new ServiceException("该人员已被添加到该私有区域中");
            }
            privateAreaWhitelist.setAdminId(privateArea.getAdminId());
            privateAreaWhitelist.setCreateTime(new Date());
            privateAreaWhitelistDao.insertSelective(privateAreaWhitelist);
        }
        return 1;
    }

    @Override
    public Integer deletePrivateAreaWhitelist(Integer privateAreaDeviceId) {
        return privateAreaWhitelistDao.deleteByPrimaryKey(privateAreaDeviceId);
    }

    @Override
    public RestResult<PageInfo<WhitelistPrivateAreaVO>> privateAreaPageListByWhitelistId(RestDTO<WhitelistPrivateAreaWhiteListSearch> dto) {
        WhitelistPrivateAreaWhiteListSearch data = dto.getData();
        if(data.getUserWhitelistId() == null)throw new ServiceException("常客id不能为空");
        List<WhitelistPrivateAreaVO> list = privateAreaWhitelistDao.privateAreaPageListByWhitelistId(data);
        return RestResponse.ok(new PageInfo<>(list));
    }
}
