package com.yx.face.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.constant.RabbitMqConstants;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.dao.iot.SpiIotUserWhitelistDao;
import com.yx.face.model.dto.UserOpenJudgeDto;
import com.yx.face.model.dto.iot.BatchAddUser2IotDto;
import com.yx.face.model.dto.iot.DelUserFromIotDto;
import com.yx.face.model.dto.iot.IotUserWhitelistBizContentDto;
import com.yx.face.model.dto.iot.IotUserWhitelistDto;
import com.yx.face.model.dto.iot.UserInfoDto;
import com.yx.face.model.entity.DownFaceNew;
import com.yx.face.model.entity.SpiIotUserWhitelist;
import com.yx.face.model.entity.UserWhitelist;
import com.yx.face.service.AlipayOpenApiService;
import com.yx.face.service.SpiIotUserWhitelistService;
import com.yx.face.service.UserWhitelistService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.aop.interceptor.AsyncExecutionAspectSupport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author shisl
 * @package com.yx.face.service.impl
 * @class SpiIotUserWhitelistServiceImpl
 * @date 2023/4/19 上午11:24
 * @description
 */
@Slf4j
@Service
public class SpiIotUserWhitelistServiceImpl implements SpiIotUserWhitelistService {

    @Resource
    private SpiIotUserWhitelistDao spiIotUserWhitelistDao;
    @Resource
    private RabbitTemplate rabbitTemplate;

    @Autowired
    @Lazy
    private UserWhitelistService userWhitelistService;



//    @Async(AsyncExecutionAspectSupport.DEFAULT_TASK_EXECUTOR_BEAN_NAME)
    public void sendIotMqMessage(String message) {
        //消息压入队列
        rabbitTemplate.convertAndSend(RabbitMqConstants.IOT_USER_QUEUE_NAME, message);
    }

    @Override
    public void addSingleUser2Iot(IotUserWhitelistDto dto) {
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.IOT_MESSAGE_TYPE_addSingleUser2iot);
        object.put("data", JSONObject.toJSONString(dto));
        sendIotMqMessage(JSONObject.toJSONString(object));
        log.info("---->addSingleUser2Iot send mq success");

    }

    @Override
    public void delUserFromIot(String sn, List<Long> userIds) {
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.IOT_MESSAGE_TYPE_delUserFromIot);
        DelUserFromIotDto dto = new DelUserFromIotDto();
        dto.setSn(sn);
        dto.setUserIds(userIds);
        object.put("data", JSONObject.toJSONString(dto));
        sendIotMqMessage(JSONObject.toJSONString(object));
        log.info("---->delUserFromIot send mq success");

        //用户IOT出库
//        if (Preconditions.isNotBlank(dto.getUserIds())) {
//            alipayOpenApiService.deleteIotUser(dto.getUserIds());
//        }

    }

    @Override
    public void batchAddUser2Iot(String sn, List<DownFaceNew> users) {
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.IOT_MESSAGE_TYPE_batchAddUser2Iot);
        BatchAddUser2IotDto dto = new BatchAddUser2IotDto();
        dto.setSn(sn);
        dto.setUsers(users);
        object.put("data", JSONObject.toJSONString(dto));
        sendIotMqMessage(JSONObject.toJSONString(object));
        log.info("---->checkIotUserPass send mq success");

    }

    @Override
    public void delBySn(String sn) {
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.IOT_MESSAGE_TYPE_delBySn);
        object.put("data", sn);
        sendIotMqMessage(JSONObject.toJSONString(object));
        log.info("---->delBySn send mq success");
    }

    @Override
    public RestResult checkIotUserPass(IotUserWhitelistDto dto) {
        log.info("---->checkIotUserPass:{}", JSONObject.toJSONString(dto));
        SpiIotUserWhitelist user = spiIotUserWhitelistDao.selectOneByExample(Example.builder(SpiIotUserWhitelist.class)
                .where(WeekendSqls.<SpiIotUserWhitelist>custom()
                        .andEqualTo(SpiIotUserWhitelist::getIdNumber, dto.getIdNumber())
                        .andEqualTo(SpiIotUserWhitelist::getIsDeleted, Boolean.FALSE)).build());
        List<IotUserWhitelistBizContentDto> dbBizContents = new ArrayList<>();
        if (Preconditions.isNotBlank(user)) {
            dbBizContents = JSONObject.parseArray(user.getBizContent(), IotUserWhitelistBizContentDto.class);
        }
        Boolean flag = Boolean.FALSE;
        long currentTimeMillis = System.currentTimeMillis();
        if (Preconditions.isNotBlank(dbBizContents)) {
            for (IotUserWhitelistBizContentDto item : dbBizContents) {
                if (item.getSn().equals(dto.getSn())
                        && Preconditions.isNotBlank(item.getStartTime())
                        && Preconditions.isNotBlank(item.getEndTime())
                        && currentTimeMillis >= item.getStartTime().getTime()
                        && currentTimeMillis <= item.getEndTime().getTime()) {
                    flag = true;
                }
            }
        }
        if (flag) {
            return RestResponse.ok("成功");
        } else {
   /*         //用户不存在 查询是否是通用场景 是否开启访客功能
            FaceDevice faceDevice = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, dto.getSn())).build());
            if (Preconditions.isBlank(faceDevice)) {
                return RestResponse.error(RestCode.DEVICE_NOT_FOUND, RestCode.DEVICE_NOT_FOUND.getMsg());
            }
            if (Preconditions.isBlank(faceDevice.getAuth()) || faceDevice.getAuth().intValue() == 0) {
                return RestResponse.error(RestCode.UNAUTHORIZED, RestCode.UNAUTHORIZED.getMsg());
            }
            //whiteVisitorType 常访客类型1常客 2访客 场所类型，0：通用场所，1：预约场所 2:楼宇
            Admin admin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
            if (Preconditions.isNotBlank(admin)
                    && Preconditions.isNotBlank(admin.getPlaceType())
                    && admin.getPlaceType().intValue() == 0
                    && Preconditions.isNotBlank(admin.getWhiteVisitorType())) {
                if (admin.getWhiteVisitorType().contains("2")) {
                    return RestResponse.ok("成功");
                } else {
                    return RestResponse.error("暂未开启访客功能，禁止通行！");
                }
            }*/
            return RestResponse.error("禁止通行！");
        }
    }

    @Override
    @Async(AsyncExecutionAspectSupport.DEFAULT_TASK_EXECUTOR_BEAN_NAME)
    public void sendUserInfoAddMq(Long userInfoId) {
        UserInfoDto dto = new UserInfoDto();
        dto.setUserInfoId(userInfoId);
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.USER_MESSAGE_TYPE_ADD);
        object.put("data", JSONObject.toJSONString(dto));
        sendIotMqMessage(JSONObject.toJSONString(object));
        //消息压入队列
        rabbitTemplate.convertAndSend(RabbitMqConstants.USER_INFO_QUEUE_NAME, JSONObject.toJSONString(object));
        log.info("---->sendUserInfoAddMq send mq success");
    }

    @Override
    @Async(AsyncExecutionAspectSupport.DEFAULT_TASK_EXECUTOR_BEAN_NAME)
    public void sendUserInfoUpdateMq(Long userInfoId) {
        UserInfoDto dto = new UserInfoDto();
        dto.setUserInfoId(userInfoId);
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.USER_MESSAGE_TYPE_UPDATE);
        object.put("data", JSONObject.toJSONString(dto));
        sendIotMqMessage(JSONObject.toJSONString(object));
        //消息压入队列
        rabbitTemplate.convertAndSend(RabbitMqConstants.USER_INFO_QUEUE_NAME, JSONObject.toJSONString(object));
        log.info("---->sendUserInfoUpdateMq send mq success");
    }

    @Override
    @Async(AsyncExecutionAspectSupport.DEFAULT_TASK_EXECUTOR_BEAN_NAME)
    public void sendUserInfoDelMq(String phone) {
        UserInfoDto dto = new UserInfoDto();
        dto.setPhone(phone);
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.USER_MESSAGE_TYPE_DEL);
        object.put("data", JSONObject.toJSONString(dto));
        sendIotMqMessage(JSONObject.toJSONString(object));
        //消息压入队列
        rabbitTemplate.convertAndSend(RabbitMqConstants.USER_INFO_QUEUE_NAME, JSONObject.toJSONString(object));
        log.info("---->sendUserInfoDelMq send mq success");
    }

    @Override
    public RestResult<Boolean> userOepnJudgeUrl(UserOpenJudgeDto dto) {
        if (StrUtil.isBlank(dto.getIdNumber())){
            return RestResponse.error("身份证不能为空");
        }
        if(ObjectUtil.isNull(dto.getAdminId())){
            return RestResponse.error("账号不能为空");
        }
        UserWhitelist userWhitelist = userWhitelistService.getUserWhitelist(dto.getIdNumber(), null, dto.getCardIdEx(), dto.getAdminId());
        if(ObjectUtil.isNull(userWhitelist)){
            return RestResponse.error("后台没有查询到该用户");
        }
        DateTime now = DateUtil.date();
        if(DateUtil.compare(userWhitelist.getStartTime(),now) > 0 || DateUtil.compare(userWhitelist.getEndTime(),now) < 0){
            return RestResponse.error("用户不在有效期");
        }
        return RestResponse.ok(true);
    }
}
