package com.yx.face.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.alipay.api.AlipayApiException;
import com.alipay.api.internal.util.AlipayEncrypt;
import com.alipay.api.internal.util.AlipaySignature;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.*;
import com.yx.face.model.dto.ShortTermDTO;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.ColorEnum;
import com.yx.face.model.enums.FaceDeviceTypeEnum;
import com.yx.face.model.enums.ZfbSpiServiceEnum;
import com.yx.face.model.vo.health.HealtCodeNucleinVO;
import com.yx.face.model.vo.health.HealtCodeUserVO;
import com.yx.face.service.FaceTBService;
import com.yx.face.service.SpiServerService;
import com.yx.face.service.UserInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.Weekend;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;


/**
 * @author shisl
 * @package com.yx.face.service.impl
 * @class SpiHealthCodeServiceImpl
 * @date 2022/11/13 下午1:59
 * <p>
 * SPI开发文档地址：https://opendocs.alipay.com/pre-open/03m3rc
 * </P>
 * @description
 */
@Slf4j
@Service
public class SpiServerServiceImpl implements SpiServerService {

    private static String appKey = "6f32f95d52c04eb6851b8b0e1a3155ad";
    private static String appSecret = "1b8f230d5904480ca3b6cc737c1211a2";

    private static String SPI_BIZ_PREFIX = "ZFB::SPI::BIZ::";
    private static String SPI_HEALTH_PREFIX = "ZFB::SPI::HEALTH::";
    private static String ENCRYPT_TYPE = "AES";
    private static String SIGN_TYPE = "RSA2";
    private static String CHARSET = "UTF-8";
    private static String SUCCESS_CODE = "10000";
    private static String ERROR_CODE = "40000";


    // 用于对支付宝行业云传入的报文进行验签
    @Value("${zfb.spi.publicKey}")
    private String publicKey;

    //私钥
    @Value("${zfb.spi.privateKey}")
    private String privateKey;

    //密钥
    @Value("${zfb.spi.secretKey}")
    private String secretKey;

    @Value("${jkm.request.queryHealthCodeUrl}")
    private String queryHealthCodeUrl;

    @Autowired
    private RestTemplate restTemplate;

    @Resource
    private RedisUtil redisUtil;

    @Autowired
    private FaceTBService faceTBService;

    @Autowired
    private UserInfoService userInfoService;
    @Resource
    private UserWhitelistSchoolDao userWhitelistSchoolDao;
    @Resource
    private UserWhitelistDao userWhitelistDao;
    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private AdminDao adminDao;
    @Resource
    private ZoneDeviceRDao zoneDeviceRDao;
    /**
     * //errorCode 40000 代表异常
     *
     * @param params
     * @return
     * @throws AlipayApiException
     */
    @Override
    public String spiGatewayService(Map<String, String> params) throws AlipayApiException {
        log.info("--->SPI服务查询入参:{}", JSON.toJSONString(params));
        // 业务处理结果载体
        JSONObject response = new JSONObject();
        //获得当前调用的service
        String serviceStr = (String) params.get("service");
        if (Preconditions.isBlank(ZfbSpiServiceEnum.getByName(serviceStr))) {
            response.put("code", ERROR_CODE);
            response.put("sub_code", "SERVICE-IS-NULL");
            response.put("sub_msg", "service参数错误");
            return handleResponseResult(response);
        }

        // 1、 验签、请求报文
        boolean isPass = checkSign(params);
        if (!isPass) {
            // 验签失败：构造错误
            response.put("code", ERROR_CODE);
            response.put("sub_code", "ISV-VERIFICATION-FAILED");
            response.put("sub_msg", "验签失败");
            return handleResponseResult(response);
        }

        //获得当前调用的业务参数
        String bizContent = (String) params.get("biz_content");
        // 2、验签成功：处理业务逻辑，并构造业务处理结果
        String bizContentPlainText = AlipayEncrypt.decryptContent(bizContent, ENCRYPT_TYPE, secretKey, CHARSET);
        log.info("--->解密后的入参：{}", bizContentPlainText);
        JSONObject jsonObject = JSONObject.parseObject(bizContentPlainText);
        switch (Objects.requireNonNull(ZfbSpiServiceEnum.getByName(serviceStr))) {
            case QUERY_HEALTH_INFO:
                response = queryHealthInfo(jsonObject);
                break;
            case UPLOAD_HEALTH_INFO:
                response = uploadHealthInfo(jsonObject);
                break;
            case QUERY_BIZ_INFO:
                response = queryBizInfo(jsonObject);
                break;
            case VERIFY_BIZ_INFO:
                response = verifyBizInfo(jsonObject);
                break;
            case QUERY_DYNAMIC_CONTENT:
                response = queryDynamicContent(jsonObject);
                break;
            default:
                log.info("--->接口名称 service: {}", serviceStr);
                break;
        }
        return handleResponseResult(response);
    }

    private JSONObject queryDynamicContent(JSONObject jsonObject) {
        Map<String,String> map = new HashMap<>();
        JSONObject response = new JSONObject();
        response.put("needNotify",false);
        this.getUidByVid(response,jsonObject,map);
        response.put("businessMsgCode","k12-in-out");
        response.put("entry",map);
        response.put("code",10000);
        return response;
    }

    private void getUidByVid(JSONObject response , JSONObject jsonObject ,Map<String,String> map) {
        String vid = jsonObject.getString("vid");
        if (StrUtil.isBlank(vid)){
            return;
        }
        //格式账户名称-区域名称 例子：公司测试-测试区域
        String pointName = jsonObject.getString("pointName");
        if(StrUtil.isBlank(pointName) || !pointName.contains("-")){
            log.info("点位配置错误");return;
        }
        String[] split = pointName.split("-");
        if(split.length != 2){
            log.info("点位配置错误【-】");return;
        }
        Admin admin = adminDao.selectOne(new Admin().setUsername(split[0]));
        if(ObjectUtil.isNull(admin)){
            log.info("账号不存在");return;
        }
        List<Admin> adminList = adminDao.select(new Admin().setTenantId(admin.getId()));
        List<Integer> adminIdList = adminList.stream().map(Admin::getId).collect(Collectors.toList());
        adminIdList.add(admin.getId());
        List<ZoneDeviceR> zoneDeviceRList = zoneDeviceRDao.selectByExample(
                Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom()
                        .andIn(ZoneDeviceR::getAdminId, adminIdList)
                        .andEqualTo(ZoneDeviceR::getDeleted, false)
                        .andEqualTo(ZoneDeviceR::getName, split[1])
                ).build());
        if(CollUtil.isEmpty(zoneDeviceRList)){
            log.info("区域不存在");return;
        }
        ZoneDeviceR zoneDeviceR = zoneDeviceRList.get(0);
        List<UserWhitelistSchool> userWhitelistSchoolList = userWhitelistSchoolDao.select(new UserWhitelistSchool().setVid(vid));
        if(CollUtil.isEmpty(userWhitelistSchoolList)){
            log.info("学校信息不存在");return;
        }
        List<Integer> userWhitelistIdList = userWhitelistSchoolList.stream().map(UserWhitelistSchool::getUserWhitelistId).collect(Collectors.toList());
        List<UserWhitelist> userWhitelistList = userWhitelistDao.selectByIds(StrUtil.join(",", userWhitelistIdList));
        userWhitelistList = userWhitelistList.stream().filter(p->adminIdList.contains(p.getAdminId())).collect(Collectors.toList());
        if(CollUtil.isEmpty(userWhitelistList)){
            log.info("用户不存在");return;
        }
        UserWhitelist userWhitelist = CollUtil.isEmpty(userWhitelistList) ? null : userWhitelistList.get(0);
        map.put("bizAction",ObjectUtil.isNull(userWhitelist) ? null : userWhitelist.getName() + "-" + zoneDeviceR.getSpiPushMsg());
        List<Long> userIdList = userWhitelistSchoolList.stream().map(UserWhitelistSchool::getUserId).collect(Collectors.toList());
        Weekend<UserInfo> weekend = new Weekend<>(UserInfo.class);
        weekend.weekendCriteria().andIn(UserInfo::getId,userIdList)
                        .andIsNotNull(UserInfo::getZfbUserId);
        List<UserInfo> userInfoList = userInfoDao.selectByExample(weekend);
        UserInfo userInfo = CollUtil.isEmpty(userInfoList) ? null : userInfoList.get(0);
        map.put("otherUid", ObjectUtil.isNull(userInfo) ? null : userInfo.getZfbUserId());
        response.put("needNotify", ObjectUtil.isNotNull(userInfo) && ObjectUtil.isNotNull(userWhitelist));
    }


    private String handleResponseResult(JSONObject response) throws AlipayApiException {
        // 3、业务处理结果加签，业务返回结果需要进行排序
        String contentToSign = JSON.toJSONString(response, SerializerFeature.MapSortField);
        log.info("--->响应待加密加签的内容:" + contentToSign);
        String responseSecretText = AlipayEncrypt.encryptContent(contentToSign, ENCRYPT_TYPE, secretKey, CHARSET);
        String sign1 = AlipaySignature.rsaSign(responseSecretText, privateKey, CHARSET, SIGN_TYPE);
        // 4、构造http响应结果
        // http响应结果载体
        JSONObject result = new JSONObject();
        result.put("sign", sign1);
        result.put("response", responseSecretText);
        log.info("--->响应结果：{}", result.toJSONString());
        return result.toJSONString();
    }


    //健康码服务
    private JSONObject queryHealthInfo(JSONObject bizContent) throws AlipayApiException {
        log.info("--->ENTER健康码查询");
        // 业务处理结果载体
        JSONObject response = new JSONObject();
        ShortTermDTO shortTermDTO = new ShortTermDTO();
        shortTermDTO.setSn(bizContent.getString("deviceSn"));
        shortTermDTO.setDeviceTypeCode(FaceDeviceTypeEnum.ZFB.getCode());
        //身份证/扫脸
        if ("CARD".equals(bizContent.getString("type"))) {
            shortTermDTO.setIdNumber(bizContent.getString("certNo"));
            shortTermDTO.setName(bizContent.getString("userName"));
            shortTermDTO.setMobile(bizContent.getString("phone"));
            response.put("phone", bizContent.getString("phone"));
            response.put("certNo", bizContent.getString("certNo"));
            response.put("username", bizContent.getString("userName"));
        }
        //二维码
        if ("QRCODE".equals(bizContent.getString("type"))) {
            shortTermDTO.setHealthCode(bizContent.getString("qrcode"));
        }

        String transactionId = bizContent.getString("transactionId");
        try {
            shortTermDTO.setToken("hh_" + appKey);
            RestResult shortTerm = restTemplate.postForObject(queryHealthCodeUrl, shortTermDTO, RestResult.class);
            log.info("--->健康码返回结果:{}", JSONObject.toJSONString(shortTerm));
            if (shortTerm != null && shortTerm.getData() != null) {
                response = handleHealthCodeInfo(response, shortTerm, shortTermDTO);
            } else {
                //40000 代表异常
                response.put("errorCode", ERROR_CODE);
                response.put("code", ERROR_CODE);
                response.put("sub_code", "HH-HEALTH-CODE-FAILED");
                response.put("sub_msg", shortTerm.getMsg());
            }
        } catch (Exception e) {
            log.error("--->查询健康码异常:{}", e);
            response.put("errorCode", ERROR_CODE);
            response.put("code", ERROR_CODE);
            response.put("sub_code", "HH-HEALTH-CODE-FAILED");
            response.put("sub_msg", "查询健康码异常");
        }
        redisUtil.set(SPI_HEALTH_PREFIX + transactionId, JSONObject.toJSONString(response), 6 * 60 * 60);
        return response;
    }

    private JSONObject uploadHealthInfo(JSONObject jsonObject) throws AlipayApiException {
        log.info("--->ENTER健康码结果上报");
        // 业务处理结果载体
        JSONObject response = new JSONObject();

        ShortTermDTO shortTermDTO = new ShortTermDTO();
        //身份证/扫脸
        if ("CARD".equals(jsonObject.getString("type"))) {
            shortTermDTO.setIdNumber(jsonObject.getString("certNo"));
            shortTermDTO.setName(jsonObject.getString("userName"));
            shortTermDTO.setMobile(jsonObject.getString("phone"));
        }

        //二维码
        if ("QRCODE".equals(jsonObject.getString("type"))) {
            shortTermDTO.setHealthCode(jsonObject.getString("qrcode"));
        }

        try {
            String transactionId = jsonObject.getString("transactionId");
            Object extBiz = redisUtil.get(SPI_HEALTH_PREFIX + transactionId);
            if (extBiz != null) {
                log.info("--->结果上报SPI:{}", extBiz.toString());
                //TODO 处理上报结果
                handleUploadHealthInfo(response, extBiz, transactionId, shortTermDTO);
            } else {
                log.error("--->信息上报失败");
                response.put("code", ERROR_CODE);
                response.put("success", false);
                response.put("data", "上报失败");
                response.put("sub_code", "SPI-UPLOAD-HEALTH-INFO-FAIL");
                response.put("sub_msg", "结果上报失败");
            }
        } catch (Exception e) {
            log.error("--->查询健康码异常:{}", e);
            response.put("code", ERROR_CODE);
            // 验签失败：构造错误
            response.put("sub_code", "HH-HEALTH-CODE-FAILED");
            response.put("sub_msg", "查询健康码异常");
        }
        return response;
    }

    // 结果上报
    private void handleUploadHealthInfo(JSONObject response, Object extBiz, String transactionId, ShortTermDTO shortTermDTO) {
        try {
            JSONObject extBizObj = JSONObject.parseObject(extBiz.toString());
            if (Preconditions.isNotBlank(extBizObj.get("code")) && "10000".equals(extBizObj.getString("code"))) {
                Map<String, Object> dto = new HashMap<>();
                dto.put("SN", extBizObj.get("SN"));
                dto.put("action", 1);
                dto.put("orgid", 0);
                List<Map<String, Object>> data = new ArrayList<>();
                Map<String, Object> map = new HashMap<>();
                map.put("recordid", 0);
                Long pin = userInfoService.checkingIdNumberAndPhone(extBizObj.getString("certNo"), extBizObj.getString("phone"));
                map.put("pin", pin);
                map.put("name", extBizObj.get("username"));
                map.put("photo", "");
                map.put("cardid", extBizObj.get("certNo"));
                map.put("cardIdEx", "");
                map.put("verifytype", shortTermDTO.getHealthCode() != null ? 3 : 2);
                map.put("rightno", 104);
                map.put("time", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, new Date()));
                map.put("outtype", 0);
                map.put("phone", extBizObj.get("phone"));
                map.put("extend3", 0);
                map.put("extend1", 2);
                map.put("verify_score", 0);
                map.put("twStatus", "");
                map.put("jkmStatus", ColorEnum.getByCode(extBizObj.getString("color")).getStatus());
                map.put("jkmType", 0);
                String hsStatus = "未知";
                if (extBizObj.get("hesuan") != null) {
                    JSONObject jsonObject = JSONObject.parseObject(extBizObj.get("hesuan").toString());
                    if (jsonObject != null && jsonObject.get("reportResult") != null) {
                        if ("NEGATIVE".equals(jsonObject.getString("reportResult"))) {
                            hsStatus = "阴性";
                        }
                        if (Preconditions.isNotBlank(jsonObject.get("content"))) {
                            hsStatus = jsonObject.getString("content");
                        }

                        if ("POSITIVE".equals(jsonObject.getString("reportResult"))) {
                            hsStatus = "阳性";
                        }
                    }
                }
                map.put("hsStatus", hsStatus);
                //疫苗
                map.put("ymStatus", "0");
                if (extBizObj.get("yimiao") != null) {
                    JSONObject jsonObject = JSONObject.parseObject(extBizObj.get("yimiao").toString());
                    if (jsonObject != null && jsonObject.get("vaccinateCount") != null) {
                        map.put("ymStatus", jsonObject.getString("vaccinateCount"));
                    }
                }
                map.put("xcInfo", extBizObj.get("xcInfo"));
                data.add(map);
                dto.put("data", data);
                faceTBService.eventRecords(dto);
            }

            response.put("code", SUCCESS_CODE);
            response.put("success", true);
            response.put("data", "上报成功");

            //业务处理完毕删除redis
            redisUtil.del(SPI_HEALTH_PREFIX + transactionId);
        } catch (Exception e) {
            log.error("--->信息上报失败", e);
            response.put("code", ERROR_CODE);
            response.put("success", false);
            response.put("data", "上报失败");
            response.put("sub_code", "SPI-UPLOAD-HEALTH-INFO-FAIL");
            response.put("sub_msg", "结果上报失败");
        }
    }

    private JSONObject handleHealthCodeInfo(JSONObject response, RestResult shortTerm, ShortTermDTO shortTermDTO) {

        HealtCodeUserVO data = JSON.parseObject(JSON.toJSONString(shortTerm.getData()), HealtCodeUserVO.class);
        response.put("SN", shortTermDTO.getSn());
        //用户基本信息
        response.put("certNo", data.getIdNumber());
        response.put("username", Preconditions.isNotBlank(data.getUsername()) ? data.getUsername() : shortTermDTO.getName());
        response.put("phone", data.getPhone());

        response.put("certDeseNo", data.getShowIdNumber());
        response.put("userDeseName", Preconditions.isNotBlank(data.getShowUsername()) ? data.getShowUsername() : SMSOrIdCardUtils.hideName(shortTermDTO.getName()));
        //健康码颜色：GREEN：绿色 YELLOW：黄色 RED：红色 GRAY：灰色 "GREEN"
        response.put("color", ColorEnum.getByValue(data.getBackgroundColor()));

        //行程
        response.put("xingcheng", "NOPASSBY");
        String travelValidationDesc = data.getTravelValidationDesc();
        if (travelValidationDesc != null) {
            response.put("xingcheng", "去过中高风险地区".equals(travelValidationDesc) ? "PASSBY" : "NOPASSBY");
            response.put("xcInfo", travelValidationDesc);

        }
        //核酸信息
        JSONObject hesuan = new JSONObject();
        String reportResult = "NORESULT";
        hesuan.put("reportResult", reportResult);
        hesuan.put("checkTime", "");
        hesuan.put("reportTime", "");
        if (data.getNuclein() != null) {
            HealtCodeNucleinVO nuclein = data.getNuclein().get(0);
            if (nuclein.getChecktime() != null) {
                hesuan.put("checkTime", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, nuclein.getChecktime()));
            }
            if (nuclein.getReportTime() != null) {
                hesuan.put("reportTime", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, nuclein.getReportTime()));
            }
            if ("阴性".equals(nuclein.getResult())) {
                reportResult = "NEGATIVE";
            }
            if ("阳性".equals(nuclein.getResult())) {
                reportResult = "POSITIVE";
            }
            hesuan.put("content", nuclein.getContent());
            hesuan.put("reportResult", reportResult);
        }
        response.put("hesuan", hesuan);
        //疫苗信息
        JSONObject yimiao = new JSONObject();
        yimiao.put("vaccinateCount", "0");
        if (data.getVaccineNum() != null && NumberUtil.isNumber(data.getVaccineNum())) {
            yimiao.put("vaccinateCount", data.getVaccineNum());
        }
        response.put("yimiao", yimiao);
        //额外参数 开门信号 语音播报
        Map<String, String> extInfo = new HashMap<>();
        extInfo.put("isOpen", data.getIsOpen().intValue() == 0 ? "0" : "1");
        extInfo.put("openMsg", data.getOpenMsg());
        extInfo.put("openVoiceMsg", data.getOpenVoiceMsg());
        response.put("extInfo", extInfo);
        response.put("errorCode", SUCCESS_CODE);
        if (data.getIsOpen() == 1) {
            response.put("sub_code", data.getOpenMsg());
            response.put("sub_msg", data.getOpenMsg());
        }
        response.put("code", SUCCESS_CODE);
        log.info("--->健康码信息组装返回结果:{}", response.toJSONString());
        return response;
    }

    private JSONObject queryBizInfo(JSONObject jsonObject) throws AlipayApiException {
        log.info("--->ENTER业务卡片SPI服务（根据支付宝那边反馈，SPI业务接口暂时不使用，直接返回10000）");
        // 业务处理结果载体
        JSONObject response = new JSONObject();
        response.put("flag", "1");
        response.put("flagMsg", "");
        response.put("title", "");
        response.put("showName", "");
        response.put("showContent", "");
        response.put("description", "");
        response.put("imageUrl", "");
        response.put("extBizId", "");
        //用户基本信息
        response.put("idcard", "");
        response.put("name", "");
        response.put("code", SUCCESS_CODE);
        return response;
    }

    // TODO 处理并组装业务参数 （根据支付宝那边反馈，SPI业务接口暂时不使用，直接返回10000）
    private void handleBizInfo(JSONObject response, RestResult shortTerm, String transactionId) {
        HealtCodeUserVO data = JSON.parseObject(JSON.toJSONString(shortTerm.getData()), HealtCodeUserVO.class);
        response.put("flag", data.getIsOpen().intValue() == 0 ? "1" : "0");
        response.put("flagMsg", data.getOpenMsg());
        response.put("title", data.getTitle());
        response.put("showName", data.getTitle());
        response.put("showContent", data.getOpenMsg());
        response.put("description", data.getOpenMsg());
        response.put("imageUrl", data.getVaccineIconUrl());
        response.put("extBizId", transactionId);

        //用户基本信息
        response.put("idcard", data.getIdNumber());
        response.put("name", data.getUsername());
        response.put("code", SUCCESS_CODE);
        log.info("--->业务信息组装返回结果:{}", JSON.toJSONString(response));

    }

    private JSONObject verifyBizInfo(JSONObject jsonObject) throws AlipayApiException {
        log.info("--->ENTER业务开闸接口");
        // 业务处理结果载体
        JSONObject response = new JSONObject();
        //获得当前调用的业务参数
        Object extBiz = redisUtil.get(SPI_HEALTH_PREFIX + jsonObject.getString("transactionId"));
        if (extBiz != null) {
            handleVerifyBizInfo(response, extBiz);
        } else {
            response.put("code", ERROR_CODE);
            response.put("sub_code", "SPI-BIZ-FAILED");
            response.put("sub_msg", "未查询到业务信息");
            response.put("flag", "0");
            response.put("flagMsg", "未查询到业务信息");
            response.put("title", "闸机通行");
            response.put("showName", "禁止通行");
            response.put("description", "未查询到业务信息");
        }
        return response;
    }

    private void handleVerifyBizInfo(JSONObject response, Object extBiz) {

        JSONObject extBizObj = JSONObject.parseObject(extBiz.toString());
        if (Preconditions.isNotBlank(extBizObj.get("errorCode"))
                && extBizObj.get("errorCode").equals(ERROR_CODE)) {
            response.put("flag", "0");
            response.put("flagMsg", extBizObj.get("sub_msg"));
            response.put("title", "闸机通行");
            response.put("showName", "禁止通行");
            response.put("description", extBizObj.get("sub_msg"));
        } else {
            String flag = "1";
            String flagMsg = "请通行";
            if (extBizObj.get("extInfo") != null) {
                JSONObject jsonObject = JSONObject.parseObject(extBizObj.get("extInfo").toString());
                if (jsonObject != null && jsonObject.get("isOpen") != null) {
                    flag = jsonObject.getString("isOpen");
                    flagMsg = jsonObject.getString("openMsg");
                }
            }
            response.put("flag", flag.equals("0") ? "1" : "0");
            response.put("flagMsg", flagMsg);
            response.put("title", "闸机通行");
            response.put("showName", extBizObj.get("username"));
            response.put("showContent", "");
            response.put("description", extBizObj.get("xcInfo"));
            response.put("imageUrl", extBizObj.get("imageUrl"));
        }
        //用户基本信息
        response.put("idcard", extBizObj.get("certNo"));
        response.put("name", extBizObj.get("username"));
        response.put("code", SUCCESS_CODE);
        log.info("--->业务闸机组装返回结果：{}", response.toJSONString());
    }

    private boolean checkSign(Map<String, String> params) {
        // 1、验签、请求报文
        boolean isPass = false;
        String sign = (String) params.get("sign");
        log.info("-->请求参数中的sign:{}", sign);
        sign = sign.replaceAll(" ", "+");
//        log.info("--->替换后的sign:{}", sign);
        params.put("sign", sign);
//        log.info("--->准备验签,参数是：{}", JsonUtils.toJson(params));
        try {
            isPass = AlipaySignature.rsaCheckV1(params, publicKey, CHARSET, SIGN_TYPE);
        } catch (AlipayApiException e) {
            log.error("--->验签失败", e);
        }
//        isPass = true;
        return isPass;
    }

}
