package com.yx.face.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.constant.AutoTaskConstant;
import com.yx.face.dao.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.*;
import com.yx.face.model.vo.*;
import com.yx.face.service.AdminService;
import com.yx.face.service.TaskInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.Weekend;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lejingjing
 * @version 1.0
 * @Description: 自动下发任务处理服务
 * @date 2022/10/10
 **/
@Service
@Slf4j
public class TaskInfoServiceImpl implements TaskInfoService {

    @Resource
    private AdminService adminService;

    @Resource
    private TaskInfoDao taskInfoDao;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Override
    public Integer deleteBySN(String sn) {
        int count = taskInfoDao.deleteByExample(Example.builder(TaskInfo.class)
                .where(WeekendSqls.<TaskInfo>custom().andEqualTo(TaskInfo::getSn, sn)).build());
        return count;
    }

    @Override
    public RestResult<PageInfo<AutoTaskVO>> getByPage(RestDTO dto) {
        AdminVO adminVO = adminService.getInfo();
        List<AdminVO> adminVOList = adminService.getAdminlist();
        List<Integer> adminIds = Lists.newArrayList();
        if(CollectionUtil.isNotEmpty(adminVOList)){
            adminIds = adminVOList.stream().map(AdminVO::getAdminId).collect(Collectors.toList());
        }
         if (adminVO.getType().equals(AdminTypeEnum.ADMINISTRATOR.getCode())) {
            adminIds.add(adminVO.getAdminId());
        }
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<AutoTaskVO> autoTaskVOList = taskInfoDao.selectTaskListByAdminId(adminIds);
        for (AutoTaskVO autoTaskVO : autoTaskVOList) {
            autoTaskVO.setTaskTypeStr(TaskTypeEnum.getTaskTypeByType(autoTaskVO.getTaskType()).getDesc());
            autoTaskVO.setCreateTimeStr(DateUtil.format(autoTaskVO.getCreateTime(), AutoTaskConstant.DATE_FORMAT_PATTERN));
        }
        return RestResponse.ok(new PageInfo<>(autoTaskVOList));
    }

    @Override
    public RestResult<AutoTaskVO> getDetailById(Integer taskId) {
        AutoTaskVO autoTaskVO = taskInfoDao.selectById(taskId);
        return RestResponse.ok(autoTaskVO);
    }

    @Override
    public RestResult<Boolean> cancelTask(Integer taskId) {
        TaskInfo taskInfo = taskInfoDao.selectByPrimaryKey(taskId);
        if (Objects.nonNull(taskInfo)) {
            taskInfo.setStatus(AutoTaskInfoStatusEnum.CANNELED.getStatus());
            taskInfoDao.updateByPrimaryKey(taskInfo);
            return RestResponse.ok(Boolean.TRUE);
        }
        return RestResponse.error("任务不存在");
    }

    @Override
    public List<TaskInfo> getOnlineTaskList(String sn) {
        return taskInfoDao.getOnlineTaskList(sn);
    }

    @Override
    public RestResult<Boolean> initIssueTaskId() {
        List<TaskInfo> taskInfoList = taskInfoDao.selectByExample(Example.builder(TaskInfo.class)
                .where(WeekendSqls.custom().andEqualTo("onlineFlag","0")).build());
        taskInfoList.forEach(e->{
            taskInfoDao.updateByPrimaryKey(e);
        });
        return RestResponse.ok(Boolean.TRUE);
    }

    @Override
    public TaskInfo save(String sn, String body) {
        List<TaskInfo> select = taskInfoDao.select(
                new TaskInfo()
                        .setSn(sn)
                        .setOnlineFlag(TaskInfoOnlineStatus.OffLine.getCode())
                        .setStatus(AutoTaskInfoStatusEnum.NOT_STATRT.getStatus()));
        if(CollUtil.isNotEmpty(select)){
            for (TaskInfo taskInfo : select) {
                taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
            }
        }
        FaceDeviceLessVO bySn = faceDeviceDao.getBySn(sn);
        List<String> codeTypeList = this.getCodeTypeList();
        if(bySn != null && !codeTypeList.contains(bySn.getFaceDeviceTypeCode())){
            return null;
        }
        TaskInfo taskInfo = new TaskInfo();
        taskInfo.setSn(sn);
        if (bySn != null) {
            taskInfo.setCreateId(bySn.getAdminId());
        }
        taskInfo.setCreateTime(new Date());
        taskInfo.setUpdateTime(new Date());
        taskInfo.setBody(body);
        taskInfo.setType(TaskTypeEnum.ALL_ISSUE.getType());
        taskInfo.setOnlineFlag(TaskInfoOnlineStatus.OffLine.getCode());
        taskInfo.setRetry(AutoTaskConstant.DEFAULT_RETRY_TIME);
        taskInfo.setExecuteResult("离线任务,等待执行");
        taskInfo.setStatus(AutoTaskInfoStatusEnum.NOT_STATRT.getStatus());
        taskInfoDao.insertSelective(taskInfo);
        return taskInfo;
    }

    @Override
    public TaskInfo save(String sn, DownFaceNew downFace) {
        List<Integer> taskTypeList = CollUtil.toList(TaskTypeEnum.ALL_ISSUE.getType(),TaskTypeEnum.ALL_REMOVE.getType());
        Weekend<TaskInfo> weekend = new Weekend<>(TaskInfo.class);
        weekend.weekendCriteria()
                .andEqualTo(TaskInfo::getSn,sn)
                .andEqualTo(TaskInfo::getOnlineFlag,TaskInfoOnlineStatus.OffLine.getCode())
                .andEqualTo(TaskInfo::getStatus,AutoTaskInfoStatusEnum.NOT_STATRT.getStatus())
                .andIn(TaskInfo::getType,taskTypeList);
        List<TaskInfo> taskInfoList =taskInfoDao.selectByExample(weekend);
        if(CollUtil.isNotEmpty(taskInfoList)){
            return null;
        }

        List<TaskInfo> select = taskInfoDao.select(
                new TaskInfo()
                        .setSn(sn)
                        .setOnlineFlag(TaskInfoOnlineStatus.OffLine.getCode())
                        .setStatus(AutoTaskInfoStatusEnum.NOT_STATRT.getStatus())
                        .setType(TaskTypeEnum.SINGLE_ISSUE.getType())
                        .setUserId(downFace.getUserId())
        );
        if(CollUtil.isNotEmpty(select)){
            for (TaskInfo taskInfo : select) {
                taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
            }
        }
        FaceDeviceLessVO bySn = faceDeviceDao.getBySn(sn);
        List<String> codeTypeList = this.getCodeTypeList();
        if(bySn != null && !codeTypeList.contains(bySn.getFaceDeviceTypeCode())){
            return null;
        }
        TaskInfo taskInfo = new TaskInfo();
        taskInfo.setSn(sn);
        if (bySn != null) {
            taskInfo.setCreateId(bySn.getAdminId());
        }
        taskInfo.setCreateTime(new Date());
        taskInfo.setUpdateTime(new Date());
        taskInfo.setBody(JSON.toJSONString(downFace));
        taskInfo.setType(TaskTypeEnum.SINGLE_ISSUE.getType());
        taskInfo.setOnlineFlag(TaskInfoOnlineStatus.OffLine.getCode());
        taskInfo.setRetry(AutoTaskConstant.DEFAULT_RETRY_TIME);
        taskInfo.setExecuteResult("离线任务,等待执行");
        taskInfo.setStatus(AutoTaskInfoStatusEnum.NOT_STATRT.getStatus());
        taskInfo.setUserId(downFace.getUserId());
        taskInfo.setUserType(downFace.getRightno());
        taskInfoDao.insertSelective(taskInfo);
        return taskInfo;
    }

    @Override
    public TaskInfo save(String sn, List<Long> userIds) {
        List<Integer> taskTypeList = CollUtil.toList(TaskTypeEnum.ALL_ISSUE.getType(),TaskTypeEnum.ALL_REMOVE.getType());
        Weekend<TaskInfo> weekend = new Weekend<>(TaskInfo.class);
        weekend.weekendCriteria()
                .andEqualTo(TaskInfo::getSn,sn)
                .andEqualTo(TaskInfo::getOnlineFlag,TaskInfoOnlineStatus.OffLine.getCode())
                .andEqualTo(TaskInfo::getStatus,AutoTaskInfoStatusEnum.NOT_STATRT.getStatus())
                .andIn(TaskInfo::getType,taskTypeList);
        List<TaskInfo> taskInfoList =taskInfoDao.selectByExample(weekend);
        if(CollUtil.isNotEmpty(taskInfoList)){
            return null;
        }

        Long userId = userIds.stream().findFirst().get();
        List<TaskInfo> select = taskInfoDao.select(
                new TaskInfo()
                        .setSn(sn)
                        .setOnlineFlag(TaskInfoOnlineStatus.OffLine.getCode())
                        .setStatus(AutoTaskInfoStatusEnum.NOT_STATRT.getStatus())
                        .setType(TaskTypeEnum.SINGLE_REMOVE.getType())
                        .setUserId(userId)
        );
        if(CollUtil.isNotEmpty(select)){
            for (TaskInfo taskInfo : select) {
                taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
            }
        }
        FaceDeviceLessVO bySn = faceDeviceDao.getBySn(sn);
        List<String> codeTypeList = this.getCodeTypeList();
        if(bySn != null && !codeTypeList.contains(bySn.getFaceDeviceTypeCode())){
            return null;
        }
        TaskInfo taskInfo = new TaskInfo();
        taskInfo.setSn(sn);
        if (bySn != null) {
            taskInfo.setCreateId(bySn.getAdminId());
        }
        taskInfo.setCreateTime(new Date());
        taskInfo.setUpdateTime(new Date());
        taskInfo.setBody(JSON.toJSONString(userIds));
        taskInfo.setType(TaskTypeEnum.SINGLE_REMOVE.getType());
        taskInfo.setOnlineFlag(TaskInfoOnlineStatus.OffLine.getCode());
        taskInfo.setRetry(AutoTaskConstant.DEFAULT_RETRY_TIME);
        taskInfo.setExecuteResult("离线任务,等待执行");
        taskInfo.setStatus(AutoTaskInfoStatusEnum.NOT_STATRT.getStatus());
        taskInfo.setUserId(userId);
        taskInfoDao.insertSelective(taskInfo);
        return taskInfo;
    }

    @Override
    public TaskInfo saveDevice(String sn, String code) {
        List<TaskInfo> select = taskInfoDao.select(
                new TaskInfo()
                        .setSn(sn)
                        .setOnlineFlag(TaskInfoOnlineStatus.OffLine.getCode())
                        .setStatus(AutoTaskInfoStatusEnum.NOT_STATRT.getStatus())
        );
        if(CollUtil.isNotEmpty(select)){
            for (TaskInfo taskInfo : select) {
                taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
            }
        }
        FaceDeviceLessVO bySn = faceDeviceDao.getBySn(sn);
        List<String> codeTypeList = this.getCodeTypeList();
        if(bySn != null && !codeTypeList.contains(bySn.getFaceDeviceTypeCode())){
            return null;
        }
        TaskInfo taskInfo = new TaskInfo();
        taskInfo.setSn(sn);
        if (bySn != null) {
            taskInfo.setCreateId(bySn.getAdminId());
        }
        taskInfo.setCode(code);
        taskInfo.setCreateTime(new Date());
        taskInfo.setUpdateTime(new Date());
        taskInfo.setType(TaskTypeEnum.ALL_REMOVE.getType());
        taskInfo.setOnlineFlag(TaskInfoOnlineStatus.OffLine.getCode());
        taskInfo.setRetry(AutoTaskConstant.DEFAULT_RETRY_TIME);
        taskInfo.setExecuteResult("离线任务,等待执行");
        taskInfo.setStatus(AutoTaskInfoStatusEnum.NOT_STATRT.getStatus());
        taskInfoDao.insertSelective(taskInfo);
        return taskInfo;
    }

    private List<String> getCodeTypeList(){
        return CollUtil.toList(
                FaceDeviceTypeEnum.TB.getCode(),
                FaceDeviceTypeEnum.FOUR_INCH_SCREEN.getCode());
    }

}