package com.yx.face.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdcardUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.*;
import com.yx.face.model.dto.UserBlackListDTO;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.search.UserBlacklistSearch;
import com.yx.face.model.search.UserWhitelistSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.UserBlackListVO;
import com.yx.face.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName UserBlackListServiceImpl
 * @Author cui
 * @Date 2021/11/5 14:16
 */
@Slf4j
@Service
public class UserBlackListServiceImpl implements UserBlackListService {
    @Resource
    private UserBlackListDao userBlackListDao;

    @Resource
    private AdminService adminService;

    @Resource
    private AdminDao adminDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private UserInfoService userInfoService;

    @Resource
    private UserWhitelistService userWhitelistService;

    @Resource
    private UserVisitorListService userVisitorListService;

    @Resource
    private FaceTBService faceTBService;

    @Override
    public PageInfo<UserBlackListVO> getPageList(RestDTO<UserWhitelistSearch> dto) {
        UserWhitelistSearch data = dto.getData();

        if (data.getPhone() != null && !data.getPhone().equals("") && (!SMSOrIdCardUtils.isMobile(data.getPhone()))) {
            throw new ServiceException("手机号查询时须填写完整");
        }
        AdminVO info = adminService.getInfo();
        info.setAdminIds(Lists.newArrayList(info.getAdminId()));
        if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())) && AdminTypeEnum.DISTRICT.getCode().equals(info.getType())) {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, info.getAdminId())).build());
            info.getAdminIds().addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
        }

        Map<String, Object> map = new HashMap<>();
        map.put("adminVo", info);
        map.put("uws", dto.getData());
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<UserBlackListVO> list = userBlackListDao.selectListBySearch(map);

        boolean isShowName = false;
        boolean isShowPhone = false;
        boolean isShowNumber = false;
        if (info.getNoHideCloumn() != null && info.getNoHideCloumn().size() > 0) {
            if (info.getNoHideCloumn().contains("1")) {
                isShowName = true;
            }
            if (info.getNoHideCloumn().contains("2")) {
                isShowPhone = true;
            }
            if (info.getNoHideCloumn().contains("3")) {
                isShowNumber = true;
            }
        }

        for (UserBlackListVO userBlackListVO : list) {
            if (StringUtils.isNotBlank(userBlackListVO.getPhone()) && isShowPhone) {
                userBlackListVO.setPhone(SMSOrIdCardUtils.hidePhone(userBlackListVO.getPhone()));
            }
            if (StringUtils.isNotBlank(userBlackListVO.getIdNumber()) && isShowNumber) {
                userBlackListVO.setIdNumber(SMSOrIdCardUtils.hideIdNumber(userBlackListVO.getIdNumber()));
            }
        }
        return new PageInfo<>(list);
    }


    @Override
    public Integer delete(Integer id, Integer adminId) throws Exception {
        UserBlackList userBlackList = userBlackListDao.selectByPrimaryKey(id);
        //判断白名单是否存在
        if (userBlackList == null) {
            throw new ServiceException("该数据已不存在，请刷新页面再试哦~");
        }
        if (userBlackList.getType() == 2) {
            throw new ServiceException("你不能删除该黑名单");
        }

        List<String> faceDeviceOne = faceDeviceDao.selectList(userBlackList.getAdminId());
        Long userId = userInfoService.checkingIdNumberAndPhone(userBlackList.getIdNumber(), userBlackList.getPhone());
        ArrayList<Long> userIds = CollUtil.toList(userId);
        for (String sn : faceDeviceOne) {
            faceTBService.delFaceBySn(sn,userIds);
        }

        return userBlackListDao.deleteByPrimaryKey(id);
    }

    @Override
    public PageInfo<UserBlackListVO> getCattlePageList(RestDTO<UserBlacklistSearch> dto) {
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<UserBlackListVO> list = userBlackListDao.getCattlePageList(dto.getData());
        return new PageInfo<>(list);
    }

    @Override
    public Integer deleteCattle(Integer id) {
        UserBlackList userBlackList = userBlackListDao.selectByPrimaryKey(id);

        //判断黄牛黑名单是否存在
        if (userBlackList == null) {
            throw new ServiceException("该数据已不存在，请刷新页面再试哦~");
        }
        if (userBlackList.getType() == 1) {
            throw new ServiceException("你不能删除该黑名单");
        }

        List<String> faceDeviceOne = faceDeviceDao.selectGuiSnList();
        if (faceDeviceOne.isEmpty()) {
            throw new ServiceException("该设备不存在或离线，请刷新页面再试哦~");
        }

        //TODO 删除黑名单 离线任务
        if (CollectionUtil.isNotEmpty(faceDeviceOne)) {
            ArrayList<Long> userIds = CollUtil.toList(userBlackList.getPinid());
            for (String sn : faceDeviceOne) {
                faceTBService.delFaceBySn(sn,userIds);
            }
        }
        return userBlackListDao.deleteByPrimaryKey(id);
    }

    @Override
    public UserBlackList getByPhoneAndIdNumber(String idNumber, String phone, Integer adminId) {
        UserBlackList userBlackList = null;
        if (StringUtils.isNotEmpty(idNumber)) {
            userBlackList = userBlackListDao.getByPhoneAndIdNumber(idNumber, null, adminId);
            if (userBlackList == null && StringUtils.isNotEmpty(phone)) {
                userBlackList = userBlackListDao.getByPhoneAndIdNumber(null, phone, adminId);
            }
        } else {
            if (StringUtils.isNotEmpty(phone)) {
                userBlackList = userBlackListDao.getByPhoneAndIdNumber(null, phone, adminId);
            }
        }
        return userBlackList;
    }

    @Override
    public Integer insert(UserBlackListDTO userBlackListDTO) {
        if (userBlackListDTO.getIdNumber() == null) {
            throw new ServiceException("身份证不能为空");
        }
        if (!IdcardUtil.isValidCard(userBlackListDTO.getIdNumber())) {
            throw new ServiceException("身份证不合法");
        }

        UserWhitelist userWhitelist = userWhitelistService.getUserWhitelist(userBlackListDTO.getIdNumber(), userBlackListDTO.getPhone(),null, userBlackListDTO.getAdminId());
        if (userWhitelist != null) {
            throw new ServiceException("该用户已经被添加到常客中");
        }
        UserVisitorList userVisitorList = userVisitorListService.getByPhoneAndIdNumber(userBlackListDTO.getIdNumber(), userBlackListDTO.getPhone(), userBlackListDTO.getAdminId());
        if (userVisitorList != null) {
            throw new ServiceException("该用户已经被添加到访客中");
        }
        UserBlackList userBlackList = this.getByPhoneAndIdNumber(userBlackListDTO.getIdNumber(), userBlackListDTO.getPhone(), userBlackListDTO.getAdminId());
        Integer insert = null;
        if (userBlackList == null) {
            userBlackList = new UserBlackList();
            userBlackList.setIdNumber(userBlackListDTO.getIdNumber());
            userBlackList.setPhone(userBlackListDTO.getPhone());
            userBlackList.setName(userBlackListDTO.getName());
            userBlackList.setAdminId(userBlackListDTO.getAdminId());
            userBlackList.setType(1);
            userBlackList.setCreateTime(new Date());
            insert = userBlackListDao.insertSelective(userBlackList);
        } else {
            throw new ServiceException("黑名单已经存在");
        }
        return insert;
    }


}
