package com.yx.face.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.UserRightNoEnum;
import com.yx.face.model.search.UserCattleDetail;
import com.yx.face.model.search.UserCattleSearch;
import com.yx.face.model.vo.*;
import com.yx.face.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 黄牛 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-01-11
 */
@Service
@Slf4j
public class UserCattleServiceImpl implements UserCattleService {

    @Resource
    private UserCattleDao userCattleDao;

    @Resource
    private UserBlackListDao userBlackListDao;

    @Resource
    private FaceLogDao faceLogDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private FaceTBService faceTBService;

    @Resource
    private AdminService adminService;

    @Resource
    private TaskInfoService taskInfoService;

    @Resource
    private FaceService faceService;

    @Override
    public PageInfo<UserCattleVO> getPageList(RestDTO<UserCattleSearch> dto) {
        AdminVO info = adminService.getInfo();
        Map<String, Object> map = new HashMap<>();
        map.put("adminVO", info);
        map.put("uc", dto.getData());
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<UserCattleVO> list = userCattleDao.selectListBySearch(map);
        for (UserCattleVO userCattleVO : list) {
            if (userCattleVO.getPhone() != null) {
                userCattleVO.setPhone(SMSOrIdCardUtils.hidePhone(userCattleVO.getPhone()));
            }
            if (userCattleVO.getIdNumber() != null) {
                userCattleVO.setIdNumber(SMSOrIdCardUtils.hideIdNumber(userCattleVO.getIdNumber()));
            }
        }
        return new PageInfo<>(list);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Integer addBlack(Integer userCattleId) {
        Date now = new Date();
        UserCattle userCattle = userCattleDao.selectByPrimaryKey(userCattleId);
        UserBlackList userBlackListOne = new UserBlackList();
        userBlackListOne.setAdminId(userCattle.getAdminId());
        userBlackListOne.setPhone(userCattle.getPhone());
        userBlackListOne.setType(2);
        UserBlackList userBlackOne = userBlackListDao.selectOne(userBlackListOne);
        UserBlackList userBlackList = new UserBlackList();
        if (userBlackOne != null) {
            userBlackList.setId(userBlackOne.getId());
            userBlackList.setAvatar(userCattle.getAvatar());
            userBlackList.setUpdateTime(now);
            userBlackListDao.updateByPrimaryKeySelective(userBlackList);
        } else {
            userBlackList.setAvatar(userCattle.getAvatar());
            userBlackList.setName(userCattle.getName());
            userBlackList.setPhone(userCattle.getPhone());
            userBlackList.setCreateTime(now);
            userBlackList.setAdminId(userCattle.getAdminId());
            userBlackList.setIdNumber(userCattle.getIdNumber());
            userBlackList.setType(2);
            userBlackList.setPinid(now.getTime());
            userBlackListDao.insert(userBlackList);
        }
        List<String> faceList = faceDeviceDao.selectGuiSnList();
        if (faceList.isEmpty()) {
            throw new ServiceException("该账号下没绑定设备，添加失败哦~");
        }
        UserBlackList userBlackList1 = userBlackListDao.selectByPrimaryKey(userBlackList.getId());
        for (String sn : faceList) {
            DownFaceNew downFaceNew = new DownFaceNew();
            downFaceNew.setSn(sn);
            downFaceNew.setUserId(userBlackList1.getPinid());
            downFaceNew.setName(userBlackList1.getName());
            downFaceNew.setPhoto(userBlackList1.getAvatar());
            downFaceNew.setPhone(userBlackList1.getPhone());
            downFaceNew.setCardid(userBlackList1.getIdNumber());
            downFaceNew.setRightno(UserRightNoEnum.BLACKLIST.getCode());
            downFaceNew.setAdminId(userBlackList.getAdminId());
            faceTBService.downUser(downFaceNew);
        }
        userCattle.setStatus(1);
        userCattleDao.updateByPrimaryKeySelective(userCattle);
        return 1;
    }

    @Override
    public List<FaceLogVO> getFaceLogList(UserCattleDetail userCattleDetail) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE, -10);
        String faceTime = sdf.format(calendar.getTime());
        UserCattle userCattle = userCattleDao.selectByPrimaryKey(userCattleDetail.getUserCattleId());
        userCattleDetail.setAdminId(userCattle.getAdminId());
        userCattleDetail.setCardid(userCattle.getIdNumber());
        userCattleDetail.setFaceTime(faceTime);
        List<FaceLogVO> faceLogs = faceLogDao.getByAdminIdAndIdNumber(userCattleDetail);
        for (FaceLogVO faceLog : faceLogs) {
            if (faceLog.getPhone() != null) {
                faceLog.setPhone(SMSOrIdCardUtils.hidePhone(faceLog.getPhone()));
            }
            if (faceLog.getCardid() != null) {
                faceLog.setCardid(SMSOrIdCardUtils.hideIdNumber(faceLog.getCardid()));
            }
        }
        return faceLogs;
    }
}
