package com.yx.face.service.impl;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.beust.jcommander.internal.Lists;
import com.beust.jcommander.internal.Maps;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.component.wx.WxMaConfiguration;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.*;
import com.yx.face.dao.*;
import com.yx.face.model.api.building.dto.OpenIdSetDTO;
import com.yx.face.model.api.common.dto.CardIdExSetDTO;
import com.yx.face.model.dto.*;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.*;
import com.yx.face.model.query.QueryUser;
import com.yx.face.model.vo.*;
import com.yx.face.service.*;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.Weekend;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: 用户基础信息表(UserInfo)表服务实现类 <br>
 * @author: PWB <br>
 * @since: 1.0 <br>
 * @date: 2021-06-04 14:25:50 <br>
 */
@Slf4j
@Service
public class UserInfoServiceImpl implements UserInfoService {

    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private UserFaceRecordsDao userFaceRecordsDao;

    @Resource
    private UserWhitelistDao userWhitelistdao;

    @Resource
    private AdminDao adminDao;

    @Resource
    private UserVisitorListDao userVisitorListDao;
    @Resource
    @Lazy
    private UserInfoOpenDao userInfoOpenDao;
    @Value("${weixin.appid}")
    private String appid;

    @Value("${weixin.appointmentid}")
    private String appointmentid;

    @Value("${weixin.buildingid}")
    private String buildingid;

   

    @Autowired
    private UserBlackListService userBlackListService;

    @Autowired
    private UserWhitelistService userWhitelistService;

    @Resource
    private UserVisitorListService userVisitorListService;

    @Resource
    private SpiIotUserWhitelistService spiIotUserWhitelistService;

    @Resource
    private BaiDuFaceService baiDuFaceService;

    @Override
    public String getWxPhone(WxInfoDTO dto) {
        String app = appid;
        if (dto.getType().equals(WxInfoEnum.APPIONtment.getCode())) {
            app = appointmentid;
        } else if (dto.getType().equals(WxInfoEnum.BUILDING.getCode())) {
            app = buildingid;
        }
        final WxMaService wxService = WxMaConfiguration.getMaService(app);
        WxMaJscode2SessionResult session;

        try {
            session = wxService.getUserService().getSessionInfo(dto.getCode());
        } catch (WxErrorException e) {
            throw new ServiceException("获取登录后的session信息失败");
        }
        WxMaPhoneNumberInfo phoneNoInfo = wxService.getUserService().getPhoneNoInfo(session.getSessionKey(), dto.getEncryptedData(), dto.getIv());
        return phoneNoInfo.getPhoneNumber();
    }

    @Override
    public Long confirmFaceNotify(ConfirmFaceNotifyDTO dto) {
        Date now = new Date();
        String pathString = now.getTime() + String.valueOf((int) (Math.random() * 10000)) + ".jpg";
        String filePath = FileUtils.getDirectory(Constants.UPLOAD_IMAGE_NEW) + "/";
        String img = FileUtils.transformBase64Image(dto.getImg(), filePath, pathString);
        if (StrUtil.isEmpty(img)) {
            return null;
        }
        String image = AddressUtil.address + img;
        UserInfo userInfo = this.userInfoDataInsert(dto.getName(), null, dto.getPhone(), dto.getIdNumber(),dto.getCardIdEx(), image,null,null);
        UserFaceRecords faceRecords = BeanUtil.copyProperties(dto,UserFaceRecords.class);
        faceRecords.setUserId(userInfo.getId());
        faceRecords.setUsername(dto.getName());
        faceRecords.setPhoto(image);
        faceRecords.setCreateTime(now);
        userFaceRecordsDao.insertSelective(faceRecords);
        return userInfo.getId();
    }

    private UserInfo userInfoDataInsert(String name, String userId, String phone, String idNumber,String cardIdEx, String image,Integer type,String code) {
        //如果身份证/手机号/物理卡号都为空的情况下，通过open获取UserInfo返回
        if(StrUtil.isEmpty(idNumber) && StrUtil.isEmpty(phone) && StrUtil.isEmpty(cardIdEx)){
            ClassUtil.throwBusinessException(type == null,"请传入当前的小程序");
            ClassUtil.throwBusinessException(StrUtil.isEmpty(code),"请传入当前用户code值");
            return this.getOpenUserInfo(name,userId,image,type,code);
        }
        Long userInfoId = this.getUserId(idNumber, phone, cardIdEx);
        UserInfo userInfo = null;
        if(userInfoId == null){
            userInfo = new UserInfo();
            userInfo.setCreateTime(new Date());
        }else {
            userInfo = userInfoDao.selectByPrimaryKey(userInfoId);
        }
        userInfo.setUsername(StrUtil.isNotEmpty(name) ? name : null);
        userInfo.setPhone(StrUtil.isNotEmpty(phone) ? phone : null);
        userInfo.setIdNumber(StrUtil.isNotEmpty(idNumber) ? idNumber : null);
        userInfo.setCardIdEx(StrUtil.isNotEmpty(cardIdEx) ? cardIdEx : null);
        userInfo.setAvatar(StrUtil.isNotEmpty(image) ? image : null);
        userInfo.setZfbUserId(userId);
        //获取百度人脸特征值
        String facialFeature = null;
        if (Preconditions.isNotBlank(image)) {
            facialFeature = baiDuFaceService.baiDuFaceCharacteristic(Base64Util.imgToBase64(image));
        }
        userInfo.setFacialFeature(facialFeature);
        if (SMSOrIdCardUtils.isIdCard(idNumber)) {
            userInfo.setBirthday(SMSOrIdCardUtils.getBirthDate(idNumber));
            userInfo.setGenderStr(Preconditions.isNotBlank(SMSOrIdCardUtils.getGender(idNumber)) ? SMSOrIdCardUtils.getGender(idNumber).getDesc() : null);
        }
        if(userInfoId == null){
            userInfoDao.insertSelective(userInfo);
            spiIotUserWhitelistService.sendUserInfoAddMq(userInfo.getId());
        }else {
            userInfoDao.updateByPrimaryKeySelective(userInfo);
            spiIotUserWhitelistService.sendUserInfoUpdateMq(userInfo.getId());
        }
        return userInfo;
    }

    private UserInfo getOpenUserInfo(String name, String userId, String image, Integer type, String code) {
        String app = "";
        switch (WxInfoEnum.getEnum(type)){
            case ZHEJIANG : app = appid;break;
            case APPIONtment : app = appointmentid;break;
            case BUILDING : app = buildingid;break;
        }
        final WxMaService wxService = WxMaConfiguration.getMaService(app);
        WxMaJscode2SessionResult session;
        try {
            session = wxService.getUserService().getSessionInfo(code);
        } catch (WxErrorException e) {
            log.info("输出异常",e);
            log.info("数据打印：{}",code);
            throw new ServiceException("获取登录后的session信息失败");
        }
        String openid = session.getOpenid();
        UserInfo userInfo = new UserInfo();
        switch (WxInfoEnum.getEnum(type)){
            case ZHEJIANG : userInfo.setCommonOpenId(openid);break;
            case APPIONtment : userInfo.setAppointmentOpenId(openid);break;
            case BUILDING : userInfo.setBuildingOpenId(openid);break;
        }
        //获取百度人脸特征值
        String facialFeature = null;
        if (Preconditions.isNotBlank(image)) {
            facialFeature = baiDuFaceService.baiDuFaceCharacteristic(Base64Util.imgToBase64(image));
        }
        List<UserInfo> userInfoList = userInfoDao.select(userInfo);
        if(CollUtil.isNotEmpty(userInfoList)){
            UserInfo info = userInfoList.get(0);
            info.setUsername(StrUtil.isNotEmpty(name) ? name : null);
            info.setZfbUserId(StrUtil.isNotEmpty(userId) ? userId : null);
            info.setAvatar(StrUtil.isNotEmpty(image) ? image : null);
            info.setFacialFeature(facialFeature);
            userInfoDao.updateByPrimaryKeySelective(info);
            return info;
        }
        userInfo.setCreateTime(new Date());
        userInfo.setUsername(StrUtil.isNotEmpty(name) ? name : null);
        userInfo.setAvatar(StrUtil.isNotEmpty(image) ? image : null);
        userInfo.setZfbUserId(StrUtil.isNotEmpty(userId) ? userId : null);
        userInfo.setFacialFeature(facialFeature);
        userInfoDao.insertSelective(userInfo);
        return userInfo;
    }

    @Override
    public RestResult<ConfirmFaceNotifyNewVO> doConfirmFaceNotifyNew(ConfirmFaceNotifyNewDTO dto) {
        UserInfo userInfo = this.userInfoDataInsert(
                dto.getUsername()
                , dto.getUserId()
                , dto.getPhone()
                , dto.getIdNumber()
                , dto.getCardIdEx()
                , dto.getAvatar()
                , dto.getType()
                , dto.getCode()
        );
        ConfirmFaceNotifyNewVO confirmFaceNotifyNewVO = new ConfirmFaceNotifyNewVO();
        BeanUtils.copyProperties(dto, confirmFaceNotifyNewVO);
        confirmFaceNotifyNewVO.setUserId(userInfo.getId());
        confirmFaceNotifyNewVO.setStatus(true);
        confirmFaceNotifyNewVO.setCreateTime(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, new Date()));
        return RestResponse.ok(confirmFaceNotifyNewVO);
    }

    @Override
    public UserInfoInsertVO userInfoInsert(UserInfoInsertDTO dto) {
        if (!StringUtils.hasText(dto.getType())) {
            dto.setType("0");
        }
        log.info("小程序认证入参:{}", JSON.toJSONString(dto));
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(dto.getUserId());
        ClassUtil.checkData(userInfo,"用户不存在");
        Admin admin = adminDao.selectByPrimaryKey(dto.getAdminId());
        ClassUtil.checkData(admin,"账号不存在");
        if (admin.getPlaceType().equals(AdminPlaceTypeEnum.APPOINTMENT_PLACE.getCode())) {
            return new UserInfoInsertVO();
        }
        String whiteVisitorType = admin.getWhiteVisitorType();
        ClassUtil.throwBusinessException(StrUtil.isBlank(whiteVisitorType),"没有常访客权限哦~");
        //查询黑名单
        UserBlackList userBlackList = userBlackListService.getByPhoneAndIdNumber(userInfo.getIdNumber(), userInfo.getPhone(), dto.getAdminId());
        ClassUtil.throwBusinessException(ObjectUtil.isNotNull(userBlackList),"你暂无权限，请联系管理员");

        //是否只开启了常客模式
        boolean isOnlyRegular = whiteVisitorType.contains("1") && !whiteVisitorType.contains("2");

        //查询常客白名单
        UserWhitelist userWhitelist = userWhitelistService.getUserWhitelist(userInfo.getIdNumber(), userInfo.getPhone(),userInfo.getCardIdEx(), dto.getAdminId());
        if (dto.getType().equals(AdminCodeEnum.REGULAR_CODE.getCode())) {//如果是扫常客码的话，需要判断是否包含常客码权限
            String codeType = admin.getCodeType();
            if (StrUtil.isBlank(codeType) || !codeType.contains("1")) {
                throw new ServiceException("请联系管理员开通常客码权限");
            }
        } else {
            if (isOnlyRegular && userWhitelist == null) {//只开启了常客白名单，并且不是扫常客码进入，则需要先在常客中添加数据
                throw new ServiceException("您不在白名单中哦~");
            }
        }
        //查询访客
        UserVisitorList userVisitorList = userVisitorListService.getByPhoneAndIdNumber(userInfo.getIdNumber(), userInfo.getPhone(), dto.getAdminId());
        log.info("小程序认证常客数据,{}", userWhitelist);
        log.info("小程序认证访客数据,{}", userVisitorList);
        String content = null;
        if (dto.getAction() == 1 && CollUtil.isNotEmpty(dto.getContents())) {
            content = JSON.toJSONString(dto.getContents());
        }
        Date now = new Date();
        Date timeStart;
        Date timeEnd = null;
        //如果是常客码认证
        if (dto.getType().equals(AdminCodeEnum.REGULAR_CODE.getCode())) {
            List<Integer> whitelistAppLoginList = admin.getWhitelistAppLoginList();
            ClassUtil.throwBusinessException(whitelistAppLoginList.contains(AdminAppLoginEnum.XM.getCode()) && StrUtil.isEmpty(userInfo.getUsername()),"请输入姓名");
            ClassUtil.throwBusinessException(whitelistAppLoginList.contains(AdminAppLoginEnum.SFZ.getCode()) && StrUtil.isEmpty(userInfo.getIdNumber()),"请输入身份证");
            ClassUtil.throwBusinessException(whitelistAppLoginList.contains(AdminAppLoginEnum.SJH.getCode()) && StrUtil.isEmpty(userInfo.getPhone()),"请输入手机号");
            ClassUtil.throwBusinessException(whitelistAppLoginList.contains(AdminAppLoginEnum.WLKH.getCode()) && StrUtil.isEmpty(userInfo.getCardIdEx()),"请输入物理卡号");
            if (userWhitelist == null) {
                /*根据常访客有效时间来进行下发*/
                if (admin.getUserWhitelistEffectiveTime() != null) {
                    timeStart = DateUtil.date();
                    int timelong = admin.getUserWhitelistEffectiveTime().multiply(new BigDecimal(1440)).intValue();
                    timeEnd = DateUtils.addMinutes(new Date(), timelong);
                } else {
                    timeStart = DateUtil.date();
                    timeEnd = DateUtils.addYears(DateUtil.date(), 1);
                }
                userWhitelist = new UserWhitelist()
                        .setAdminId(admin.getId())
                        .setCreateTime(now)
                        .setStartTime(timeStart)
                        .setEndTime(timeEnd);
                this.userWhitelistDataInsert(userWhitelist, userInfo, content);
                userWhitelistdao.insert(userWhitelist);
            } else {
                userWhitelist.setUpdateTime(now);
                this.userWhitelistDataInsert(userWhitelist, userInfo, content);
                userWhitelistdao.updateByPrimaryKeySelective(userWhitelist);
            }
        }
        //访客码认证
        else {
            //访客码认证，但是这个人是常客
            if (userWhitelist != null) {
                userWhitelist.setUpdateTime(now);
                this.userWhitelistDataInsert(userWhitelist, userInfo, content);
                /*根据常访客有效时间来进行下发*/
                if (userWhitelist.getStartTime() != null && userWhitelist.getEndTime() != null) {
                    timeStart = userWhitelist.getStartTime();
                    timeEnd = userWhitelist.getEndTime();
                } else {
                    if (admin.getUserWhitelistEffectiveTime() != null) {
                        timeStart = DateUtil.date();
                        int timelong = admin.getUserWhitelistEffectiveTime().multiply(new BigDecimal(1440)).intValue();
                        timeEnd = DateUtils.addMinutes(new Date(), timelong);
                    } else {
                        timeStart = DateUtil.date();
                        timeEnd = DateUtils.addYears(DateUtil.date(), 1);
                    }
                }
                userWhitelist.setStartTime(timeStart);
                userWhitelist.setEndTime(timeEnd);
                userWhitelistdao.updateByPrimaryKeySelective(userWhitelist);
            } else {
                if (admin.getUserVisiterEffectiveTime() != null) {
                    int timelong = admin.getUserVisiterEffectiveTime().multiply(new BigDecimal(1440)).intValue();
                    timeEnd = DateUtils.addMinutes(new Date(), timelong);
                } else {
                    timeEnd = DateUtils.addDays(DateUtil.date(), 15);
                }
                //访客这个人没有身份证号和手机号的时候，使用openId查询
                String openid = "";
                if(StrUtil.isEmpty(userInfo.getIdNumber()) && StrUtil.isEmpty(userInfo.getPhone())){
                    switch (AdminPlaceTypeEnum.getEnum(admin.getPlaceType())){
                        case COMMON_PLACE : openid = userInfo.getCommonOpenId();break;
                        case APPOINTMENT_PLACE : openid = userInfo.getAppointmentOpenId();break;
                        case BUILDING_PLACE : openid = userInfo.getBuildingOpenId();break;
                    }
                    userVisitorList = getUserVisitorList(openid,admin.getId());
                }

                if (userVisitorList == null) {
                    userVisitorList = new UserVisitorList();
                    userVisitorList.setAdminId(dto.getAdminId());
                    userVisitorList.setCreateTime(now);
                    userVisitorList.setOpenId(openid);
                    this.userVisitoristDataInsert(userVisitorList, userInfo, content);
                    userVisitorListDao.insert(userVisitorList);
                } else {
                    userVisitorList.setUpdateTime(now);
                    this.userVisitoristDataInsert(userVisitorList, userInfo, content);
                    userVisitorListDao.updateByPrimaryKeySelective(userVisitorList);
                }
            }
        }
        //认证为常客 访客为空
        if (userWhitelist != null && userVisitorList != null) {
            userVisitorListDao.deleteByPrimaryKey(userVisitorList.getId());
        }

        //数据整理返回
        UserInfoInsertVO userInfoInsertVO = new UserInfoInsertVO();
        if (dto.getAction() == 2) {
            if (userWhitelist != null) {
                userInfoInsertVO.setDate(userWhitelist.getEndTime());
                try {
                    userWhitelistService.addSingle(userWhitelist.getId(), userWhitelist.getStartTime(), userWhitelist.getEndTime());
                } catch (Exception e) {
                    log.info("常客下发失败:", e);
                }
                if (userVisitorList != null) {
                    userVisitorListDao.deleteByPrimaryKey(userVisitorList.getId());
                }
            } else {
                userInfoInsertVO.setDate(timeEnd);
                try {
                    userVisitorListService.addSingle(null, userVisitorList.getId());
                } catch (Exception e) {
                    log.info("访客下发失败:", e);
                }
            }
        }
        log.info("小程序人脸下发接口完成");
        return userInfoInsertVO;
    }

    private UserVisitorList getUserVisitorList(String openid, Integer adminId) {
        UserVisitorList userVisitorList = new UserVisitorList();
        userVisitorList.setOpenId(openid);
        userVisitorList.setAdminId(adminId);
        List<UserVisitorList> list = userVisitorListDao.select(userVisitorList);
        if(CollUtil.isNotEmpty(list)){
            return list.get(0);
        }
        return null;
    }

    private void userWhitelistDataInsert(UserWhitelist userWhitelist, UserInfo userInfo, String content) {
        userWhitelist.setName(userInfo.getUsername());
        userWhitelist.setPhone(userInfo.getPhone());
        userWhitelist.setAvatar(userInfo.getAvatar());
        userWhitelist.setIdNumber(userInfo.getIdNumber());
        userWhitelist.setCardIdEx(userInfo.getCardIdEx());
        userWhitelist.setPushType(0);
        userWhitelist.setStatus(1);
        userWhitelist.setContent(content);
        userWhitelist.setUserId(userInfo.getId());
    }

    private void userVisitoristDataInsert(UserVisitorList userVisitorList, UserInfo userInfo, String content) {
        userVisitorList.setName(userInfo.getUsername());
        userVisitorList.setPhone(userInfo.getPhone());
        userVisitorList.setAvatar(userInfo.getAvatar());
        userVisitorList.setIdNumber(userInfo.getIdNumber());
        userVisitorList.setPushType(0);
        userVisitorList.setContent(content);
        userVisitorList.setUserId(userInfo.getId());
    }

    @Override
    public PageInfo<UserInfoVO> getUsers(RestDTO<QueryUser> dto) {
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return new PageInfo<>(userInfoDao.getUsersByFacePass(dto.getData()));
    }


    @Override
    public List<UserInfoVO> getUserInfoByUserIds(List<Long> userIds) {
        return userInfoDao.getUserInfoByUserIds(userIds);
    }

    @Override
    public RestResult<Boolean> openIdSet(OpenIdSetDTO dto) {
        if (dto.getUserId() == null) {
            return RestResponse.error("用户Id不存在请重新登录");
        }
        String app = appid;
        if (dto.getType().equals(WxInfoEnum.APPIONtment.getCode())) {
            app = appointmentid;
        } else if (dto.getType().equals(WxInfoEnum.BUILDING.getCode())) {
            app = buildingid;
        }
        final WxMaService wxService = WxMaConfiguration.getMaService(app);
        WxMaJscode2SessionResult session;

        try {
            session = wxService.getUserService().getSessionInfo(dto.getCode());
        } catch (WxErrorException e) {
            throw new ServiceException("获取登录后的session信息失败");
        }
        UserInfo userInfo = new UserInfo();
        userInfo.setId(dto.getUserId());
        switch (dto.getType()){
            case 1 :userInfo.setCommonOpenId(session.getOpenid());break;
            case 2 :userInfo.setAppointmentOpenId(session.getOpenid());break;
            case 3 :userInfo.setBuildingOpenId(session.getOpenid());break;
        }
        userInfoDao.updateByPrimaryKeySelective(userInfo);
        return RestResponse.ok(true);
    }
    @Override
    public Integer realAuthenticationType(RealAuthenticationTypeDTO realAuthenticationTypeDTO) {
        if (Preconditions.isBlank(realAuthenticationTypeDTO.getIdNumber())) {
            throw new ServiceException("参数有误，检查后提交");
        }

        Admin admin = adminDao.selectByPrimaryKey(realAuthenticationTypeDTO.getAdminId());
        if (admin == null) {
            return 1;
        }

        Integer age = null;
        if (SMSOrIdCardUtils.isIdCard(realAuthenticationTypeDTO.getIdNumber())) {
            //身份证
            age = SMSOrIdCardUtils.getAge(realAuthenticationTypeDTO.getIdNumber());
            if (Preconditions.isBlank(age)) {
                throw new ServiceException("身份证有误");
            }
        } else {
            //护照号
            List<UserInfo> userInfos = userInfoDao.selectByExample(Example.builder(UserInfo.class)
                    .where(WeekendSqls.<UserInfo>custom()
                            .andEqualTo(UserInfo::getIdNumber, realAuthenticationTypeDTO.getIdNumber())).build());
            if (Preconditions.isBlank(userInfos)) {
                throw new ServiceException("用户信息不存在或已删除");
            }
            String birthday = userInfos.get(0).getBirthday();
            if (Preconditions.isBlank(birthday)) {
                return 1;
            }

            try {
                age = SMSOrIdCardUtils.calAgeByBirthday(birthday);
            } catch (Exception e) {
                return 1;
            }
        }

        if (StringUtils.hasText(admin.getRealAuthenticationAge())) {
            String[] list = admin.getRealAuthenticationAge().split("-");
            if (list.length == 2) {
                if (Integer.parseInt(list[0]) <= age && age <= Integer.parseInt(list[1])) {
                    return 1;
                } else {
                    return 2;
                }
            }
        }
        return 1;
    }

    /**
     * 更新或修改 user_info
     *
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long addOrUpdateUserInfo(UserWhitelist userWhitelist) {
        Long userId = this.getUserId(userWhitelist.getIdNumber(),userWhitelist.getPhone(),userWhitelist.getCardIdEx());

        //获取百度人脸特征值
        String facialFeature = null;
        if (Preconditions.isNotBlank(userWhitelist.getAvatar())) {
            facialFeature = baiDuFaceService.baiDuFaceCharacteristic(Base64Util.imgToBase64(userWhitelist.getAvatar()));
        }
        if (userId == null) {
            Date now = new Date();
            UserInfo userInfo = BeanUtil.copyProperties(userWhitelist, UserInfo.class,"id","");
            if (SMSOrIdCardUtils.isIdCard(userWhitelist.getIdNumber())) {
                userInfo.setBirthday(SMSOrIdCardUtils.getBirthDate(userWhitelist.getIdNumber()));
                userInfo.setGenderStr(Preconditions.isNotBlank(SMSOrIdCardUtils.getGender(userWhitelist.getIdNumber())) ? SMSOrIdCardUtils.getGender(userWhitelist.getIdNumber()).getDesc() : null);
            }
            userInfo.setUsername(userWhitelist.getName());
            userInfo.setPhone(userWhitelist.getPhone());
            userInfo.setCardIdEx(userWhitelist.getCardIdEx());
            userInfo.setIdNumber(userWhitelist.getIdNumber());
            userInfo.setFacialFeature(facialFeature);
            userInfo.setUpdateTime(now);
            userInfo.setCreateTime(now);
            log.info("添加user信息 userInfo" + userInfo);
            userInfoDao.insertSelective(userInfo);
            userId = userInfo.getId();
            spiIotUserWhitelistService.sendUserInfoAddMq(userInfo.getId());
        } else {
            UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
            userInfo.setUsername(userWhitelist.getName());
            userInfo.setPhone(userWhitelist.getPhone());
            userInfo.setCardIdEx(userWhitelist.getCardIdEx());
            userInfo.setIdNumber(userWhitelist.getIdNumber());
            userInfo.setAvatar(userWhitelist.getAvatar());
            userInfo.setFacialFeature(facialFeature);
            userInfoDao.updateByPrimaryKey(userInfo);
        }
        return userId;
    }

    public UserInfo getUserInfo(String idNumber, String phone, String cardIdEx){
        if(StrUtil.isNotEmpty(idNumber)){
            Weekend<UserInfo> weekend = new Weekend<>(UserInfo.class);
            weekend.weekendCriteria().andEqualTo(UserInfo::getIdNumber,idNumber);
            List<UserInfo> userInfoList = userInfoDao.selectByExample(weekend);
            if(CollUtil.isNotEmpty(userInfoList)){
                return userInfoList.get(0);
            }
        }
        if(StrUtil.isNotEmpty(phone)){
            Weekend<UserInfo> weekend = new Weekend<>(UserInfo.class);
            weekend.weekendCriteria().andEqualTo(UserInfo::getPhone,phone);
            List<UserInfo> userInfoList = userInfoDao.selectByExample(weekend);
            if(CollUtil.isNotEmpty(userInfoList)){
                return userInfoList.get(0);
            }
        }
        if(StrUtil.isNotEmpty(cardIdEx)){
            Weekend<UserInfo> weekend = new Weekend<>(UserInfo.class);
            weekend.weekendCriteria().andEqualTo(UserInfo::getCardIdEx,cardIdEx);
            List<UserInfo> userInfoList = userInfoDao.selectByExample(weekend);
            if(CollUtil.isNotEmpty(userInfoList)){
                return userInfoList.get(0);
            }
        }
        return null;
    }
    private Long getUserId(String idNumber, String phone, String cardIdEx) {
        UserInfo userInfo = this.getUserInfo(idNumber, phone, cardIdEx);
        if(ObjectUtil.isNotNull(userInfo)){
            return userInfo.getId();
        }
        return null;
    }

    /**
     * 检查身份证手机是否存在userInfo 身份证优先
     *
     * @param idNumber 身份证
     * @param phone    手机号
     * @return userId
     */
    @Override
    public Long checkingIdNumberAndPhone(String idNumber, String phone) {
        Long userId = null;
        if (StringUtils.hasText(idNumber)) {
            userId = userInfoDao.getUserInfoPhoneAndIdNumber(idNumber, null);
            if (userId == null) {
                if (StringUtils.hasText(phone)) {
                    userId = userInfoDao.getUserInfoPhoneAndIdNumber(null, phone);
                }
            }
        } else {
            if (StringUtils.hasText(phone)) {
                userId = userInfoDao.getUserInfoPhoneAndIdNumber(null, phone);
            }
        }
        return userId;
    }
    @Override
    public RestResult<Boolean> cardIdExSet(CardIdExSetDTO dto) {
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(dto.getUserId());
        if(ObjectUtil.isNotNull(userInfo) && StrUtil.isNotEmpty(dto.getCardIdEx())){
            userInfo.setCardIdEx(dto.getCardIdEx());
            userInfoDao.updateByPrimaryKeySelective(userInfo);
        }
        return RestResponse.ok(Boolean.TRUE);
    }

    @Override
    public List<ContextVO> getContextByCardId(Integer userId, Integer adminId) {
        //查询admin信息
        Admin admin = adminDao.selectByPrimaryKey(adminId);
        if(ObjectUtil.isNull(admin) || StrUtil.isEmpty(admin.getContent())){
            return Lists.newArrayList();
        }
        List<ContextVO> titleList = JSONArray.parseArray(admin.getContent(), ContextVO.class);
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
        if(ObjectUtil.isNull(userInfo)){
            return Lists.newArrayList();
        }
        String content = "";
        UserWhitelist userWhitelist = userWhitelistService.getUserWhitelist(userInfo.getIdNumber(), userInfo.getPhone(), userInfo.getCardIdEx(), adminId);
        if(ObjectUtil.isNotNull(userWhitelist)){
            content = userWhitelist.getContent();
        }
        if(ObjectUtil.isNotNull(userWhitelist) && StrUtil.isBlank(content)){
            String openId = "";
            switch (AdminPlaceTypeEnum.getEnum(admin.getPlaceType())){
                case COMMON_PLACE:openId = userInfo.getCommonOpenId();break;
                case APPOINTMENT_PLACE:openId = userInfo.getAppointmentOpenId();break;
                case BUILDING_PLACE:openId = userInfo.getBuildingOpenId();break;
            }
            UserVisitorList userVisitorList = userVisitorListService.getUserVisitorList(userInfo.getIdNumber(), userInfo.getPhone(), openId, adminId);
            if(ObjectUtil.isNotNull(userVisitorList)){
                content = userVisitorList.getContent();
            }
        }
        Map<String, String> stringMap = Maps.newHashMap();
        if (!StrUtil.isEmpty(content)) {
            List<ContextVO> voList = JSON.parseArray(content, ContextVO.class);
            stringMap = voList.stream().filter(v -> v != null && StrUtil.isNotEmpty(v.getCustomNameVal())).collect(Collectors.toMap(ContextVO::getId, ContextVO::getCustomNameVal, (v1, v2) -> v1));
        }

        for (ContextVO contextVO : titleList) {
            contextVO.setCustomNameVal(stringMap.get(contextVO.getId()));

            String configurationOptions = contextVO.getConfigurationOptions();
            if (StrUtil.isNotEmpty(configurationOptions)) {
                String[] split = configurationOptions.split("/");

                List<Map> list = new ArrayList<>();
                for (String s : split) {
                    Map map = new HashMap();
                    map.put("name", s);
                    list.add(map);
                }
                contextVO.setConfigurationOption(list);
            }
        }

        return titleList;
    }
}

