package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.UserWhitelistCheckDao;
import com.yx.face.dao.UserWhitelistDao;
import com.yx.face.model.dto.UserWhitelistCheckDto;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.UserWhitelist;
import com.yx.face.model.entity.UserWhitelistCheck;
import com.yx.face.model.enums.AdminBuildingCheckTypeEnum;
import com.yx.face.model.search.UserWhitelistCheckSearch;
import com.yx.face.model.vo.*;
import com.yx.face.service.AdminService;
import com.yx.face.service.UserWhitelistCheckService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.weekend.Weekend;
import tk.mybatis.mapper.weekend.WeekendCriteria;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 楼宇审核人员 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2023-08-23
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserWhitelistCheckServiceImpl implements UserWhitelistCheckService {

    private final UserWhitelistCheckDao userWhitelistCheckDao;
    private final UserWhitelistDao userWhitelistDao;
    private final AdminService adminService;
    private final AdminDao adminDao;

    @Override
    public PageInfo<UserWhitelistCheckVO> getPageList(RestDTO<UserWhitelistCheckSearch> dto) {
        AdminVO adminVO = this.getAdminVo(false);
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<UserWhitelistCheck> userWhitelistCheckList = userWhitelistCheckDao.selectByExample(this.getExample(adminVO,dto.getData()));
        List<UserWhitelistCheckVO> userWhitelistCheckVOList =this.getuserWhitelistCheckVOList(userWhitelistCheckList);
        return new PageInfo<>(userWhitelistCheckVOList);
    }

    @Override
    public List<AdminVO> getAdminList() {
        AdminVO info = adminService.getInfo();
        List<Admin> adminList = adminDao.select(new Admin().setTenantId(info.getAdminId()));
        return adminList.stream().map(p -> {
            AdminVO adminVO = new AdminVO();
            adminVO.setAdminId(p.getId());
            adminVO.setUsername(p.getUsername());
            return adminVO;
        }).collect(Collectors.toList());
    }


    private List<UserWhitelistCheckVO> getuserWhitelistCheckVOList(List<UserWhitelistCheck> userWhitelistCheckList) {
        if(CollUtil.isEmpty(userWhitelistCheckList)){
            return new LinkedList<>();
        }
        List<Integer> userWhitelistIdList = userWhitelistCheckList.stream().map(UserWhitelistCheck::getUserWhitelistId).collect(Collectors.toList());
        List<UserWhitelist> userWhitelistList = userWhitelistDao.selectByIds(StrUtil.join(",", userWhitelistIdList));
        Map<Integer, UserWhitelist> userWhitelistMap = userWhitelistList.stream().collect(Collectors.toMap(UserWhitelist::getId, Function.identity()));
        List<Integer> adminIdList = userWhitelistList.stream().map(UserWhitelist::getAdminId).collect(Collectors.toList());
        Map<Integer, Admin> adminMap;
        if(CollUtil.isNotEmpty(adminIdList)){
            List<Admin> adminList = adminDao.selectByIds(StrUtil.join(",", adminIdList));
            adminMap = adminList.stream().collect(Collectors.toMap(Admin::getId, Function.identity()));
        }else {
            adminMap = new HashMap<>();
        }
        return userWhitelistCheckList.stream().map(p -> {
            UserWhitelistCheckVO item = new UserWhitelistCheckVO();
            item.setUserWhitelistCheckId(p.getId());
            item.setCreateTime(p.getCreateTime());
            if(userWhitelistMap.containsKey(p.getUserWhitelistId())){
                UserWhitelist userWhitelist = userWhitelistMap.get(p.getUserWhitelistId());
                UserWhitelistLessVO lessVO = BeanUtil.copyProperties(userWhitelist,UserWhitelistLessVO.class);
                if(adminMap.containsKey(userWhitelist.getAdminId())){
                    lessVO.setUsername(adminMap.get(userWhitelist.getAdminId()).getUsername());
                }
                item.setUserWhitelistLessVO(lessVO);
            }
            return item;
        }).collect(Collectors.toList());
    }

    private Weekend<UserWhitelistCheck> getExample(AdminVO adminVO, UserWhitelistCheckSearch data) {
        Weekend<UserWhitelistCheck> weekend = new Weekend(UserWhitelistCheck.class);
        WeekendCriteria<UserWhitelistCheck, Object> criteria = weekend.weekendCriteria();
        criteria.andEqualTo(UserWhitelistCheck::getAdminId,adminVO.getAdminId());
        weekend.orderBy("id").desc();
        return weekend;
    }

    private AdminVO getAdminVo(boolean isCheck) {//是否校验有没有开启楼宇审核方式的指定人员审核
        AdminVO info = adminService.getInfo();
        ClassUtil.throwBusinessException(isCheck && !AdminBuildingCheckTypeEnum.ZDRYSH.getCode().equals(info.getBuildingCheckType()),"没有开启指定人员审核");
        return info;
    }

    @Override
    public List<UserWhitelistLittleVO> getUserWhitelist(Integer adminId) {
        Weekend<UserWhitelist> weekend = new Weekend(UserWhitelist.class);
        WeekendCriteria<UserWhitelist, Object> criteria = weekend.weekendCriteria();
        criteria.andEqualTo(UserWhitelist::getAdminId,adminId);
        weekend.selectProperties("id","name");
        List<UserWhitelist> userWhitelistList = userWhitelistDao.selectByExample(weekend);
        return userWhitelistList.stream().map(p -> {
            UserWhitelistLittleVO item = new UserWhitelistLittleVO();
            item.setUserWhitelistId(p.getId());
            item.setName(p.getName());
            return item;
        }).collect(Collectors.toList());
    }

    @Override
    public Boolean insert(UserWhitelistCheckDto dto) {
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(dto.getUserWhitelistId());
        ClassUtil.checkData(userWhitelist,"常客不存在");
        AdminVO adminVo = this.getAdminVo(true);
        List<UserWhitelistCheck> userWhitelistCheckList = userWhitelistCheckDao.select(new UserWhitelistCheck().setUserWhitelistId(dto.getUserWhitelistId()));
        ClassUtil.throwBusinessException(CollUtil.isNotEmpty(userWhitelistCheckList),"该人员已经拥有审核权限");
        UserWhitelistCheck userWhitelistCheck = new UserWhitelistCheck();
        userWhitelistCheck.setUserWhitelistId(dto.getUserWhitelistId());
        userWhitelistCheck.setAdminId(adminVo.getAdminId());
        userWhitelistCheckDao.insertSelective(userWhitelistCheck);
        return Boolean.TRUE;
    }

    @Override
    public Boolean delete(Long userWhitelistCheckId) {
        UserWhitelistCheck userWhitelistCheck = userWhitelistCheckDao.selectByPrimaryKey(userWhitelistCheckId);
        ClassUtil.checkData(userWhitelistCheck,"该记录不存在，请刷新");
        userWhitelistCheckDao.deleteByPrimaryKey(userWhitelistCheckId);
        return Boolean.TRUE;
    }

}
