package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.alibaba.fastjson.JSON;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.dao.UserWhitelistDao;
import com.yx.face.dao.UserWhitelistSchoolDao;
import com.yx.face.model.dto.iot.AlipayOperateRecordCreateWithVidOpenApiRequest;
import com.yx.face.model.dto.iot.AlipayOperateRecordInfoWithVidApiRequest;
import com.yx.face.model.dto.iot.AlipayUserDeleteOpenApiRequest;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.enums.AlipayCertTypeEnum;
import com.yx.face.model.school.dto.UserWhitelistSchoolCallBackDTO;
import com.yx.face.model.school.dto.UserWhitelistSchoolDTO;
import com.yx.face.model.school.vo.UserWhitelistSchoolDetailVO;
import com.yx.face.service.AlipayOpenApiService;
import com.yx.face.service.UserWhitelistSchoolService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 学校场景入库 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2023-09-07
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Slf4j
public class UserWhitelistSchoolServiceImpl implements UserWhitelistSchoolService {

    private final UserWhitelistSchoolDao userWhitelistSchoolDao;
    private final UserWhitelistDao userWhitelistDao;
    private final AdminDao adminDao;
    private final UserInfoDao userInfoDao;
    private final AlipayOpenApiService alipayOpenApiService;
    @Value("${alipay.open.api.appId5069}")
    private String appId;

    @Override
    public List<UserWhitelistSchoolDetailVO> getUserWhitelist(Long userId) {
        List<UserWhitelistSchool> userWhitelistSchoolList = userWhitelistSchoolDao.select(new UserWhitelistSchool().setUserId(userId));
        if(CollUtil.isEmpty(userWhitelistSchoolList)){
            return new LinkedList<>();
        }
        List<Integer> userWhitelistIdList = userWhitelistSchoolList.stream().map(UserWhitelistSchool::getUserWhitelistId).collect(Collectors.toList());
        List<UserWhitelist> userWhitelistList = userWhitelistDao.selectByIds(StrUtil.join(",", userWhitelistIdList));
        Map<Integer,String> usernameMap = this.getUsernameMap(userWhitelistList);
        return userWhitelistList.stream().map(p->{
            UserWhitelistSchoolDetailVO item = BeanUtil.copyProperties(p,UserWhitelistSchoolDetailVO.class);
            item.setUserWhitelistId(p.getId());
            if(usernameMap.containsKey(p.getAdminId())){
                item.setUsername(usernameMap.get(p.getAdminId()));
            }
            return item;
        }).collect(Collectors.toList());
    }


    private Map<Integer, String> getUsernameMap(List<UserWhitelist> userWhitelistList) {
        if(CollUtil.isEmpty(userWhitelistList)){
            return new HashMap<>();
        }
        List<Integer> adminIdList = userWhitelistList.stream().map(UserWhitelist::getAdminId).collect(Collectors.toList());
        List<Admin> adminList = adminDao.selectByIds(StrUtil.join(",", adminIdList));
        return adminList.stream().collect(Collectors.toMap(Admin::getId,Admin::getUsername));
    }
    @Override
    public List<UserWhitelistSchoolDetailVO> getUserWhitelistByIdNumber(String idNumber, Integer personnelType) {
        //获取所有是学校场景的学校级别账号
        List<Admin> adminList = adminDao.select(
                new Admin()
                        .setType(AdminTypeEnum.ENTERPRISE.getCode())
                        .setPlaceType(AdminPlaceTypeEnum.SCHOOL_PLACE.getCode())
        );
        if(CollUtil.isEmpty(adminList)){
            return new LinkedList<>();
        }
        List<Integer> adminIdList = adminList.stream().map(Admin::getId).collect(Collectors.toList());
        List<UserWhitelist> userWhitelistList = userWhitelistDao.selectByExample(
                Example.builder(UserWhitelist.class)
                        .where(WeekendSqls.<UserWhitelist>custom()
                                .andEqualTo(UserWhitelist::getIdNumber, idNumber)
                                .andEqualTo(UserWhitelist::getPersonnelType, personnelType)
                                .andIn(UserWhitelist::getAdminId, adminIdList)).build()
        );
        Map<Integer,String> usernameMap = adminList.stream().collect(Collectors.toMap(Admin::getId,Admin::getUsername));
        return userWhitelistList.stream().map(p->{
            UserWhitelistSchoolDetailVO item = BeanUtil.copyProperties(p,UserWhitelistSchoolDetailVO.class);
            item.setUserWhitelistId(p.getId());
            item.setUsername(usernameMap.get(p.getAdminId()));
            return item;
        }).collect(Collectors.toList());
    }

    @Override
    public RestResult userInsertData(UserWhitelistSchoolDTO dto) {
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(dto.getUserWhitelistId());
        ClassUtil.checkData(userWhitelist,"该常客不存在");
        ClassUtil.throwBusinessException(StrUtil.isBlank(userWhitelist.getIdNumber()),"该常客没有身份证号");
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(dto.getUserId());
        ClassUtil.checkData(userInfo,"该用户不存在");
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        ClassUtil.checkData(admin,"账号不存在");
        Admin tenantAdmin = adminDao.selectByPrimaryKey(admin.getTenantId());
        ClassUtil.checkData(tenantAdmin,"上级账号不存在");
        ClassUtil.throwBusinessException(StrUtil.isBlank(tenantAdmin.getOrgOutId()),"机构代码为空");
        userWhitelist.setAvatar(StrUtil.isBlank(dto.getAvatar()) ? null : dto.getAvatar());
        userWhitelistDao.updateByPrimaryKeySelective(userWhitelist);
        //todo 调用支付宝相关
        AlipayOperateRecordCreateWithVidOpenApiRequest req = new AlipayOperateRecordCreateWithVidOpenApiRequest();
        List<AlipayOperateRecordInfoWithVidApiRequest> userInfoList = new LinkedList<>();
        AlipayOperateRecordInfoWithVidApiRequest request = new AlipayOperateRecordInfoWithVidApiRequest();
        request.setCertType(AlipayCertTypeEnum.IDENTITY_CARD.getCode());
        request.setAvatar(dto.getAvatar());
        request.setCertName(userWhitelist.getName());
        request.setCertNo(userWhitelist.getIdNumber());
        request.setVid(DigestUtil.md5Hex(AlipayCertTypeEnum.IDENTITY_CARD.getCode() + tenantAdmin.getOrgOutId() + userWhitelist.getIdNumber()));
        userInfoList.add(request);
        req.setUserInfoList(userInfoList);
        req.setAppId(appId);
        req.setOrgOutId(tenantAdmin.getOrgOutId());
        log.info("调用支付宝生成流水号接口【createOperateRecordWithVidV2】");
        RestResult restResult = alipayOpenApiService.createOperateRecordWithVidV2(req);
        log.info("调用支付宝生成流水号接口【createOperateRecordWithVidV2】返回值:{}", JSON.toJSONString(restResult));
        return restResult;
    }

    @Override
    public RestResult<Boolean> userInsertCallback(UserWhitelistSchoolCallBackDTO dto) {
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(dto.getUserWhitelistId());
        ClassUtil.checkData(userWhitelist,"该常客不存在");
        ClassUtil.throwBusinessException(StrUtil.isBlank(userWhitelist.getIdNumber()),"该常客没有身份证号");
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(dto.getUserId());
        ClassUtil.checkData(userInfo,"该用户不存在");
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        ClassUtil.checkData(admin,"账号不存在");
        Admin tenantAdmin = adminDao.selectByPrimaryKey(admin.getTenantId());
        ClassUtil.checkData(tenantAdmin,"上级账号不存在");
        ClassUtil.throwBusinessException(StrUtil.isBlank(tenantAdmin.getOrgOutId()),"机构代码为空");

        UserWhitelistSchool userWhitelistSchool = userWhitelistSchoolDao.selectOne(new UserWhitelistSchool().setUserWhitelistId(userWhitelist.getId()));
        String vid = DigestUtil.md5Hex(AlipayCertTypeEnum.IDENTITY_CARD.getCode() + tenantAdmin.getOrgOutId() + userWhitelist.getIdNumber());
        userWhitelistSchool = ObjectUtil.isNull(userWhitelistSchool) ? new UserWhitelistSchool() : userWhitelistSchool;
        userWhitelistSchool.setUserId(dto.getUserId());
        userWhitelistSchool.setVid(vid);
        if(ObjectUtil.isNull(userWhitelistSchool.getId())){
            userWhitelistSchool.setUserWhitelistId(userWhitelist.getId());
            userWhitelistSchoolDao.insertSelective(userWhitelistSchool);
        }else {
            userWhitelistSchoolDao.updateByPrimaryKeySelective(userWhitelistSchool);
        }
        userWhitelist.setWarehousingStatus(Boolean.TRUE);
        userWhitelistDao.updateByPrimaryKeySelective(userWhitelist);
        return RestResult.ok(Boolean.TRUE);
    }

    @Override
    public RestResult<Boolean> closeFace(UserWhitelistSchoolCallBackDTO dto) {
        UserWhitelist userWhitelist = userWhitelistDao.selectByPrimaryKey(dto.getUserWhitelistId());
        ClassUtil.checkData(userWhitelist,"该常客不存在");
        ClassUtil.throwBusinessException(StrUtil.isBlank(userWhitelist.getIdNumber()),"该常客没有身份证号");
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(dto.getUserId());
        ClassUtil.checkData(userInfo,"该用户不存在");
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        ClassUtil.checkData(admin,"账号不存在");
        Admin tenantAdmin = adminDao.selectByPrimaryKey(admin.getTenantId());
        ClassUtil.checkData(tenantAdmin,"上级账号不存在");
        ClassUtil.throwBusinessException(StrUtil.isBlank(tenantAdmin.getOrgOutId()),"机构代码为空");
        UserWhitelistSchool userWhitelistSchool = userWhitelistSchoolDao.selectOne(new UserWhitelistSchool().setUserWhitelistId(userWhitelist.getId()));
        ClassUtil.checkData(userWhitelistSchool,"常客不存在vid");
        //调用支付宝出库
        AlipayUserDeleteOpenApiRequest req = new AlipayUserDeleteOpenApiRequest();
        req.setAppId(appId);
        req.setOrgOutId(tenantAdmin.getOrgOutId());
        List<String> vidList = CollUtil.toList(userWhitelistSchool.getVid());
        req.setVidList(vidList);
        log.info("调用支付宝生成流水号接口【createOperateRecordWithVidV2】");
        alipayOpenApiService.deleteUser(req);
        userWhitelist.setWarehousingStatus(Boolean.FALSE);
        userWhitelistDao.updateByPrimaryKeySelective(userWhitelist);
        log.info("删除常客school数据{}",JSON.toJSONString(userWhitelistSchool));
        userWhitelistSchoolDao.deleteByPrimaryKey(userWhitelistSchool.getId());
        return RestResult.ok(Boolean.TRUE);
    }
}
