package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.yx.face.boot.restful.*;
import com.yx.face.boot.uitls.*;
import com.yx.face.dao.*;
import com.yx.face.model.dto.IssueAllDTO;
import com.yx.face.model.dto.UserWhitelistBatchDTO;
import com.yx.face.model.dto.UserWhitelistBatchDelDTO;
import com.yx.face.model.dto.UserWhitelistBatchDistributionDTO;
import com.yx.face.model.dto.UserWhitelistBatchUpdateDTO;
import com.yx.face.model.dto.UserWhitelistDTO;
import com.yx.face.model.dto.iot.AlipayUserDeleteOpenApiRequest;
import com.yx.face.model.entity.*;
import com.yx.face.model.enums.*;
import com.yx.face.model.search.UserWhitelistSearch;
import com.yx.face.model.vo.*;
import com.yx.face.service.*;
import com.yx.face.service.issue.HhfaceIssueService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.thymeleaf.util.ListUtils;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.Executor;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName UserWhitelistServiceImpl
 * @Author WXG
 * @Date 2021/11/5 14:16
 */
@Slf4j
@Service
public class UserWhitelistServiceImpl implements UserWhitelistService {

    @Resource
    private UserWhitelistDao userWhitelistdao;
    @Resource
    private UserWhitelistSchoolDao userWhitelistSchoolDao;
    @Resource
    private UserWhiteListFromDao userWhiteListFromDao;

    @Resource
    private AdminService adminService;

    @Resource
    private FaceService faceService;

    @Resource
    private AdminDao adminDao;

    @Resource
    private AdminDetailDao adminDetailDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private UserVisitorListDao userVisitorListDao;

    @Autowired
    @Lazy
    private AsyncTriggerRetryImpl asyncTriggerRetryImpl;

    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private AreaCodeDao areaCodeDao;
    @Autowired
    @Lazy
    private UserInfoService userInfoService;

    @Resource
    private PrivateAreaWhitelistDao privateAreaWhitelistDao;

    @Resource
    private PrivateAreaDeviceDao privateAreaDeviceDao;

    @Autowired
    @Lazy
    private FaceTBService faceTBService;

    @Resource
    private PrivateAreaDao privateAreaDao;

    @Resource
    private UserWhitelistCheckDao userWhitelistCheckDao;


    @Resource
    private UserBlackListService userBlackListService;

    @Resource
    private TaskInfoService taskInfoService;

   

    @Resource
    private TaskInfoDao taskInfoDao;

    @Resource
    private ZoneDeviceRDao zoneDeviceRDao;

    @Resource
    private FaceDeviceTypeDao faceDeviceTypeDao;

    @Resource
    private AlipayOpenApiService alipayOpenApiService;
    @Resource
    private Executor executor;
    @Value("alipay.open.api.appId5069")
    private String appId;

    @Resource
    private HhfaceIssueService hhfaceIssueService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public UserWhitelist insert(UserWhitelistDTO userWhitelistDTO, Integer type) throws Exception {
        AdminVO info = adminService.getInfo(userWhitelistDTO.getAdminId());
        //数据校验
        UserWhitelist userWhitelist = getUserWhitelist(userWhitelistDTO, info);
        //自定义字段校验
        this.checkContent(info, userWhitelistDTO.getAddCustomList());
        userWhitelistDTO.setId(null);
        UserVisitorList userVisitorListByPhone = null;
        if (StringUtils.isNotBlank(userWhitelist.getIdNumber())) {
            userVisitorListByPhone = userVisitorListDao.selectByIdNumberAndAdminId(userWhitelistDTO.getIdNumber(), userWhitelistDTO.getAdminId());
        } else if (StringUtils.isNotBlank(userWhitelist.getPhone())) {
            userVisitorListByPhone = userVisitorListDao.selectByPhoneAndAdminId(userWhitelistDTO.getPhone(), userWhitelistDTO.getAdminId());
        }
        if (null != userVisitorListByPhone) {
            userWhitelist.setAvatar(userVisitorListByPhone.getAvatar());
            userWhitelist.setIdNumber(userVisitorListByPhone.getIdNumber());
            userWhitelist.setStatus(1);
            userVisitorListDao.deleteByPrimaryKey(userVisitorListByPhone.getId());
        }
        userWhitelist.setHid(userWhitelistDTO.getHid());
        userWhitelist.setContent(userWhitelistDTO.getContent());
        userWhitelist.setCreateTime(new Date());
        userWhitelist.setType(1);
        userWhitelist.setPushType(userWhitelistDTO.getInputType());
        userWhitelist.setZoneDevicePassIdList(userWhitelistDTO.getZoneDevicePassIdList());
        if (Preconditions.isBlank(userWhitelistDTO.getStartTime()) || Preconditions.isBlank(userWhitelistDTO.getEndTime()) || userWhitelistDTO.getStartTime().equals("null") || userWhitelistDTO.getEndTime().equals("null")) {
            userWhitelist.setStartTime(new Date());
            userWhitelist.setEndTime(DateUtils.getNextYearPreDay());
        } else {
            try {
                userWhitelist.setStartTime(DateUtils.parseDate(userWhitelistDTO.getStartTime()));
                userWhitelist.setEndTime(DateUtils.parseDate(userWhitelistDTO.getEndTime()));
            } catch (Exception e) {
                log.error("日期转换异常", e);
                throw new ServiceException("时间有误");
            }
        }
        if (null == userWhitelist.getStatus()) {
            userWhitelist.setStatus(2);
        }
        /*人工录入的话默认为已认证*/
        if (userWhitelistDTO.getInputType() == 2) {
            userWhitelist.setStatus(1);
        }
        Long userId = userInfoService.addOrUpdateUserInfo(userWhitelist);
        userWhitelist.setUserId(userId);
        userWhitelistdao.insert(userWhitelist);
        log.info("添加之后常客数据:{}", userWhitelist);
        /*只有开启了私有区域之后才会将数据填入*/
        if (info.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.OPEN_PRIVATE_AREA.getCode())) {
            if (userWhitelistDTO.getAdminAreaType().equals(AdminAreaTypeEnum.CompanyPrivate.getCode())) {
                if (userWhitelistDTO.getPrivateAreaIdList() == null || userWhitelistDTO.getPrivateAreaIdList().size() == 0) {
                    throw new ServiceException("请填入私有区域");
                }
            }
            /*私有区域数据填入*/
            if (userWhitelistDTO.getPrivateAreaIdList() != null) {
                List<Integer> privateAreaIdList = userWhitelistDTO.getPrivateAreaIdList();
                for (int i = 0; i < privateAreaIdList.size(); i++) {
                    PrivateAreaWhitelist privateAreaWhitelist = new PrivateAreaWhitelist();
                    privateAreaWhitelist.setUserWhitelistId(userWhitelist.getId());
                    privateAreaWhitelist.setPrivateAreaId(privateAreaIdList.get(i));
                    privateAreaWhitelist.setAdminId(userWhitelist.getAdminId());
                    privateAreaWhitelist.setCreateTime(new Date());
                    privateAreaWhitelistDao.insertSelective(privateAreaWhitelist);
                }
            }
        }
        /*下发*/
//        addSingleWhite(userWhitelist, 1, type);
        addSingle(userWhitelist.getId(), userWhitelist.getStartTime(), userWhitelist.getEndTime());

        return userWhitelist;
    }

    private void checkContent(AdminVO info, List<ContextVO> contextVOList) {
        List<ContextVO> adminContextList = JSONArray.parseArray(info.getContent(), ContextVO.class);
        if (CollUtil.isNotEmpty(contextVOList) && CollUtil.isNotEmpty(adminContextList)) {
            Map<String, ContextVO> adminMap = adminContextList.stream().collect(Collectors.toMap(ContextVO::getId, Function.identity()));
            for (ContextVO contextVO : contextVOList) {
                if (StrUtil.isEmpty(contextVO.getCustomNameVal())) {
                    continue;
                }
                if (contextVO.getCustomType().equals("下拉选择")) {
                    if (adminMap.containsKey(contextVO.getId())) {
                        String configurationOptions = adminMap.get(contextVO.getId()).getConfigurationOptions();
                        List<String> configList = Arrays.stream(configurationOptions.split("/")).collect(Collectors.toList());
                        if (!configList.contains(contextVO.getCustomNameVal())) {
                            throw new ServiceException(contextVO.getCustomName() + ":下拉列表选择不存在");
                        }
                    }
                } else if (contextVO.getCustomType().equals("省市区")) {
                    List<String> stringList = Arrays.stream(contextVO.getCustomNameVal().split("-")).collect(Collectors.toList());
                    if (stringList.size() == 3) {
                        AreaCode areaCode = areaCodeDao.selectByConetent(stringList.get(0), stringList.get(1), stringList.get(2), null);
                        if (areaCode == null) {
                            throw new ServiceException(contextVO.getCustomName() + "：没有找到相应的地理位置");
                        }
                    } else {
                        throw new ServiceException(contextVO.getCustomName() + "：格式有误，需要到区为止");
                    }
                } else if (contextVO.getCustomType().equals("省市区街道")) {
                    List<String> stringList = Arrays.stream(contextVO.getCustomNameVal().split("-")).collect(Collectors.toList());
                    if (stringList.size() == 4) {
                        AreaCode areaCode = areaCodeDao.selectByConetent(stringList.get(0), stringList.get(1), stringList.get(2), stringList.get(3));
                        if (areaCode == null) {
                            throw new ServiceException(contextVO.getCustomName() + "：没有找到相应的地理位置");
                        }
                    } else {
                        throw new ServiceException(contextVO.getCustomName() + "：格式有误，需要到街道为止");
                    }
                } else if (contextVO.getCustomType().equals("日期")) {
                    try {
                        DateUtil.parse(contextVO.getCustomNameVal());
                    } catch (Exception e) {
                        throw new ServiceException(contextVO.getCustomName() + "：日期格式有误");
                    }
                } else if (contextVO.getCustomType().equals("时间")) {
                    try {
                        DateUtil.parse(contextVO.getCustomNameVal());
                    } catch (Exception e) {
                        throw new ServiceException(contextVO.getCustomName() + ":时间格式有误");
                    }
                }
            }
        }
    }


    @Override
    public UserWhitelist update(UserWhitelistDTO userWhitelistDTO) {
        ClassUtil.throwBusinessException(userWhitelistDTO.getId() == null, "修改时常客主键不能为空");
        UserWhitelist userWhitelist = userWhitelistdao.selectByPrimaryKey(userWhitelistDTO.getId());
        ClassUtil.checkData(userWhitelist, "该常客不存在");
        ClassUtil.throwBusinessException(!whiteVisitorType(userWhitelist.getAdminId()), "没有开启常客功能");
        if (StringUtils.isNotBlank(userWhitelistDTO.getCardIdEx())) {
            UserWhitelist userWhitelistByCardIdEx = userWhitelistdao.getByCardIdExAndId(userWhitelistDTO.getCardIdEx(), userWhitelistDTO.getId(), userWhitelistDTO.getAdminId());
            if (userWhitelistByCardIdEx != null) {
                throw new ServiceException("物理卡号已经存在");
            }
        }
        userWhitelist.setUpdateTime(new Date());
        userWhitelist.setCardIdEx(StrUtil.isNotBlank(userWhitelistDTO.getCardIdEx()) ? userWhitelistDTO.getCardIdEx() : userWhitelist.getCardIdEx());
        userWhitelist.setAdminAreaType(userWhitelistDTO.getAdminAreaType() == null ? null : userWhitelistDTO.getAdminAreaType());
        userWhitelist.setContent(userWhitelistDTO.getContent());
        userWhitelist.setAdminId(userWhitelistDTO.getAdminId());
        userWhitelist.setZoneDevicePassIdList(userWhitelistDTO.getZoneDevicePassIdList());
        if (userWhitelist.getPushType() != null) {
            switch (userWhitelist.getPushType()) {
                case 2:
                    ;
                case 4:
                    ;
                case 5:
                    userWhitelist.setAvatar(userWhitelistDTO.getAvatar());
                    break;
            }
        }

        if (StrUtil.isNotEmpty(userWhitelistDTO.getStartTime())) {
            userWhitelist.setStartTime(DateUtil.parseDateTime(userWhitelistDTO.getStartTime()));
        }
        if (StrUtil.isNotEmpty(userWhitelistDTO.getEndTime())) {
            userWhitelist.setEndTime(DateUtil.parseDateTime(userWhitelistDTO.getEndTime()));
        }
        userWhitelist.setDeptName(userWhitelistDTO.getDeptName() == null ? "" : userWhitelistDTO.getDeptName());
        userWhitelist.setDutyName(userWhitelistDTO.getDutyName() == null ? "" : userWhitelistDTO.getDutyName());
        userWhitelist.setWorkName(userWhitelistDTO.getWorkName() == null ? "" : userWhitelistDTO.getWorkName());
        userWhitelist.setHomeAddress(userWhitelistDTO.getHomeAddress() == null ? "" : userWhitelistDTO.getHomeAddress());
        userWhitelist.setJobNumber(userWhitelistDTO.getJobNumber() == null ? "" : userWhitelistDTO.getJobNumber());

        /*
        私有区域相关数据处理
         1.查询出原本数据库中的
        */
        AdminVO info = adminService.getInfo(userWhitelistDTO.getAdminId());
        //私有区域相关
        this.extracted(userWhitelistDTO.getPrivateAreaIdList(), userWhitelist, info);

        Long userId = userInfoService.addOrUpdateUserInfo(userWhitelist);
        userWhitelist.setUserId(userId);
        userWhitelistdao.updateByPrimaryKeySelective(userWhitelist);
        this.addSingle(userWhitelist.getId(), userWhitelist.getStartTime(), userWhitelist.getEndTime());
        return userWhitelist;
    }

    private void extracted(List<Integer> privateAreaIdList, UserWhitelist userWhitelist, AdminVO info) {
        if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())) //楼宇场景
                && AdminTypeEnum.ENTERPRISE.getCode().equals(info.getType())//企业账号
                && AdminIsOpenPrivateAreaEnum.OPEN_PRIVATE_AREA.getCode().equals(info.getIsOpenPrivateArea())//开启私有区域
        ) {
            /*1.查询数据库中该常客原有的私有区域列表*/
            PrivateAreaWhitelist privateAreaWhitelist = new PrivateAreaWhitelist();
            privateAreaWhitelist.setUserWhitelistId(userWhitelist.getId());
            privateAreaWhitelist.setAdminId(userWhitelist.getAdminId());
            List<PrivateAreaWhitelist> list = privateAreaWhitelistDao.select(privateAreaWhitelist);
            /*2.将私有区域id提取出来*/
            List<Integer> sqlList = list.stream().map(PrivateAreaWhitelist::getPrivateAreaId).collect(Collectors.toList());

            privateAreaIdList = privateAreaIdList == null ? new ArrayList<>() : privateAreaIdList;
            List<Integer> finalPrivateAreaIdList = privateAreaIdList;
            /*3.将数据库中取出来的 和 传进来进行取交集*/
            List<Integer> intersection = sqlList.stream().filter(finalPrivateAreaIdList::contains).collect(Collectors.toList());
            /*4.剔除交集的私有区域，剩下的就是需要删除人脸的私有区域*/
            List<Integer> reduce1 = sqlList.stream().filter(item -> !intersection.contains(item)).collect(Collectors.toList());

            Long userId = userInfoService.checkingIdNumberAndPhone(userWhitelist.getIdNumber(), userWhitelist.getPhone());
            List<Long> longList = new ArrayList<>();
            longList.add(userId);
            /*5.删除多的私有区域中设备的人脸信息，然后对人进行重新下发*/
            if (reduce1.size() > 0) {
                /*6.根据私有区域ListId获取设备列表*/
                List<String> snList = privateAreaDeviceDao.selectByPrivateAreaIdList(reduce1);

                for (int i = 0; i < snList.size(); i++) {
                    faceTBService.delFaceBySn(snList.get(i), longList);
                }
                /*7删除原有的私有区域信息*/
                for (int i = 0; i < reduce1.size(); i++) {
                    PrivateAreaWhitelist privateAreaWhitelist1 = new PrivateAreaWhitelist();
                    privateAreaWhitelist1.setUserWhitelistId(userWhitelist.getId());
                    privateAreaWhitelist1.setAdminId(userWhitelist.getAdminId());
                    privateAreaWhitelist1.setPrivateAreaId(reduce1.get(i));
                    privateAreaWhitelistDao.delete(privateAreaWhitelist1);
                }
            }
            /*8.添加新的私有区域*/
            List<Integer> reduce2 = finalPrivateAreaIdList.stream().filter(item -> !intersection.contains(item)).collect(Collectors.toList());
            for (int i = 0; i < reduce2.size(); i++) {
                PrivateAreaWhitelist privateAreaWhitelist1 = new PrivateAreaWhitelist();
                privateAreaWhitelist1.setUserWhitelistId(userWhitelist.getId());
                privateAreaWhitelist1.setAdminId(userWhitelist.getAdminId());
                privateAreaWhitelist1.setPrivateAreaId(reduce2.get(i));
                privateAreaWhitelist1.setCreateTime(new Date());
                privateAreaWhitelistDao.insertSelective(privateAreaWhitelist1);
            }
        }
    }


    //数据校验
    private UserWhitelist getUserWhitelist(UserWhitelistDTO userWhitelistDTO, AdminVO info) {
        UserBlackList userBlackList = userBlackListService.getByPhoneAndIdNumber(userWhitelistDTO.getIdNumber(), userWhitelistDTO.getPhone(), userWhitelistDTO.getAdminId());
        if (userBlackList != null) {
            throw new ServiceException("该用户被填加入了黑名单");
        }
        UserWhitelist userWhitelist = new UserWhitelist();
        userWhitelist.setAdminId(userWhitelistDTO.getAdminId());
        boolean b = (StringUtils.isBlank(userWhitelistDTO.getIdNumber())
                && StringUtils.isBlank(userWhitelistDTO.getPhone())
                && StringUtils.isBlank(userWhitelistDTO.getCardIdEx()));
        if (b) {
            throw new ServiceException("身份证-手机号-物理卡号其中不能都为空");
        }
        if (StringUtils.isBlank(userWhitelistDTO.getName())) {
            throw new ServiceException("姓名不能为空哦");
        }

        //校验身份证
        if (StringUtils.isNotBlank(userWhitelistDTO.getIdNumber()) && userWhitelistDTO.getInputType() == 1) {
            if (!SMSOrIdCardUtils.isIdCard(userWhitelistDTO.getIdNumber()) && userWhitelistDTO.getIsCheckIdNumber()) {
                throw new ServiceException("身份证有误哦");
            } else {
                UserWhitelist byPhoneAndId = userWhitelistdao.getByIdNumberAndId(userWhitelistDTO.getIdNumber(), userWhitelistDTO.getId(), userWhitelistDTO.getAdminId());
                if (byPhoneAndId != null) {
                    throw new ServiceException("身份证已存在哦");
                }
                userWhitelist.setBirthDay(IdcardUtil.getBirthDate(userWhitelistDTO.getIdNumber()));
                userWhitelist.setSex(IdcardUtil.getGenderByIdCard(userWhitelistDTO.getIdNumber()));
            }
        } else if (userWhitelistDTO.getInputType() == 2) {
            if (StringUtils.isBlank(userWhitelistDTO.getIdNumber())) {
                throw new ServiceException("人工录入证件号不能为空");
            }
            UserWhitelist byPhoneAndId = userWhitelistdao.getByIdNumberAndId(userWhitelistDTO.getIdNumber(), userWhitelistDTO.getId(), userWhitelistDTO.getAdminId());
            if (byPhoneAndId != null) {
                throw new ServiceException("身份证已存在哦");
            }
        }
        //校验手机号
        if (StringUtils.isNotBlank(userWhitelistDTO.getPhone())) {
            UserWhitelist byPhoneAndId = userWhitelistdao.getByPhoneAndId(userWhitelistDTO.getPhone(), userWhitelistDTO.getId(), userWhitelistDTO.getAdminId());
            if (byPhoneAndId != null) {
                throw new ServiceException("手机号已经存在");
            }
        }
        //校验物理卡号
        if (StringUtils.isNotBlank(userWhitelistDTO.getCardIdEx())) {
            UserWhitelist userWhitelistByCardIdEx = userWhitelistdao.getByCardIdExAndId(userWhitelistDTO.getCardIdEx(), userWhitelistDTO.getId(), null);
            if (userWhitelistByCardIdEx != null) {
                throw new ServiceException("物理卡号已经存在");
            }
        }
        ClassUtil.throwBusinessException(AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType()) && StrUtil.isBlank(userWhitelistDTO.getIdNumber()), "身份证不能为空");
        ClassUtil.throwBusinessException(AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType()) && StrUtil.isBlank(userWhitelistDTO.getName()), "姓名不能为空");
        ClassUtil.throwBusinessException(AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType()) && !IdcardUtil.isValidCard(userWhitelistDTO.getIdNumber()), "身份证号有误");


        userWhitelist.setIdNumber(userWhitelistDTO.getIdNumber() == null ? null : userWhitelistDTO.getIdNumber());
        userWhitelist.setCardIdEx(userWhitelistDTO.getCardIdEx() == null ? null : userWhitelistDTO.getCardIdEx());
        userWhitelist.setName(userWhitelistDTO.getName() == null ? "" : userWhitelistDTO.getName());
        userWhitelist.setPhone(userWhitelistDTO.getPhone() == null ? "" : userWhitelistDTO.getPhone());
        userWhitelist.setAvatar(userWhitelistDTO.getAvatar() == null ? "" : userWhitelistDTO.getAvatar());
        userWhitelist.setInputType(userWhitelistDTO.getInputType() == null ? null : userWhitelistDTO.getInputType());
        userWhitelist.setAdminAreaType(userWhitelistDTO.getAdminAreaType() == null ? null : userWhitelistDTO.getAdminAreaType());
        userWhitelist.setPersonnelType(userWhitelistDTO.getPersonnelType());
        userWhitelist.setWarehousingStatus(AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType()) ? Boolean.FALSE : null);

        userWhitelist.setDeptName(userWhitelistDTO.getDeptName() == null ? "" : userWhitelistDTO.getDeptName());
        userWhitelist.setDutyName(userWhitelistDTO.getDutyName() == null ? "" : userWhitelistDTO.getDutyName());
        userWhitelist.setWorkName(userWhitelistDTO.getWorkName() == null ? "" : userWhitelistDTO.getWorkName());
        userWhitelist.setHomeAddress(userWhitelistDTO.getHomeAddress() == null ? "" : userWhitelistDTO.getHomeAddress());
        userWhitelist.setJobNumber(userWhitelistDTO.getJobNumber() == null ? "" : userWhitelistDTO.getJobNumber());


        return userWhitelist;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Integer delete(Integer id, Integer adminId) {
        UserWhitelist userWhitelist = userWhitelistdao.selectByPrimaryKey(id);
        ClassUtil.checkData(userWhitelist, "常客不存在");
        PrivateAreaWhitelist privateAreaWhitelist = new PrivateAreaWhitelist();
        privateAreaWhitelist.setUserWhitelistId(id);
        privateAreaWhitelistDao.delete(privateAreaWhitelist);
        List<String> faceDeviceOne = faceDeviceDao.selectListAll(userWhitelist.getAdminId());
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        ClassUtil.checkData(admin, "账号不存在");
        if (admin.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            faceDeviceOne.addAll(faceDeviceDao.selectList(admin.getTenantId()));
        }
        Long userId = userWhitelist.getUserId();

        //删除2188 iot用户出库
        if (Preconditions.isNotBlank(userId)) {
            alipayOpenApiService.deleteIotUser(Arrays.asList(userId));
        }
        if (userId != null) {
            ArrayList<Long> userIds = CollUtil.toList(userId);
            for (String sn : faceDeviceOne) {
                faceTBService.delFaceBySn(sn, userIds);
            }
        }
        userWhiteListFromDao.deleteById(id);
        userWhitelistdao.deleteByPrimaryKey(id);
        //删除常客相关信息
        executor.execute(() -> {
            userWhitelistCheckDao.delete(new UserWhitelistCheck().setUserWhitelistId(id));
            if (!AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(admin.getPlaceType())) {
                return;
            }
            //调用支付宝出库
            UserWhitelistSchool userWhitelistSchool = userWhitelistSchoolDao.selectOne(new UserWhitelistSchool().setUserWhitelistId(userWhitelist.getId()));
            if (ObjectUtil.isNull(userWhitelistSchool)) {
                return;
            }
            Admin tenantAdmin = adminDao.selectByPrimaryKey(admin.getTenantId());
            if (ObjectUtil.isNull(tenantAdmin) || StrUtil.isBlank(tenantAdmin.getOrgOutId())) {
                return;
            }
            AlipayUserDeleteOpenApiRequest req = new AlipayUserDeleteOpenApiRequest();
            req.setAppId(appId);
            req.setOrgOutId(tenantAdmin.getOrgOutId());
            List<String> vidList = CollUtil.toList(userWhitelistSchool.getVid());
            req.setVidList(vidList);
            log.info("调用支付宝生成流水号接口【createOperateRecordWithVidV2】");
            alipayOpenApiService.deleteUser(req);
            userWhitelistSchoolDao.deleteByPrimaryKey(userWhitelistSchool.getId());
        });
        return 1;
    }


    @Override
    public PageInfo<UserWhitelistVO> getPageList(RestDTO<UserWhitelistSearch> dto) {
        UserWhitelistSearch data = dto.getData();

        if (data.getPhone() != null && !data.getPhone().equals("") && (!SMSOrIdCardUtils.isMobile(data.getPhone()))) {
            throw new ServiceException("手机号查询时须填写完整");
        }
        AdminVO info = adminService.getInfo();
        info.setAdminIds(Lists.newArrayList(info.getAdminId()));
        if ((AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType())) && AdminTypeEnum.DISTRICT.getCode().equals(info.getType())) {
            List<Admin> adminList = adminDao.selectByExample(Example.builder(Admin.class).where(WeekendSqls.<Admin>custom().andEqualTo(Admin::getTenantId, info.getAdminId())).build());
            info.getAdminIds().addAll(adminList.stream().map(Admin::getId).collect(Collectors.toList()));
        }

        //查询账号附表信息
        List<AdminDetail> adminDetails = adminDetailDao.selectByExample(Example.builder(AdminDetail.class).where(WeekendSqls.<AdminDetail>custom().andIn(AdminDetail::getAdminId, info.getAdminIds())).build());
        Map<Integer, AdminDetail> detailMap = adminDetails.stream().collect(Collectors.toMap(AdminDetail::getAdminId, v -> v, (v1, v2) -> v1));

        Map<String, Object> map = new HashMap<>();
        List<ContextVO> contents = dto.getData().getContents();
        if (!ListUtils.isEmpty(contents)) {
            for (int i = 0; i < contents.size(); i++) {
                if (i == 0 && StringUtils.isNotEmpty(contents.get(0).getCustomNameVal())) {
                    String id = contents.get(0).getId();
                    String customNameVal = contents.get(0).getCustomNameVal();
                    String context = "\"id\":" + "\"" + id + "\"," + "\"customNameVal\":" + "\"" + customNameVal;
                    dto.getData().setContent(context);
                } else if (i == 1 && StringUtils.isNotEmpty(contents.get(1).getCustomNameVal())) {
                    String id = contents.get(1).getId();
                    String customNameVal = contents.get(1).getCustomNameVal();
                    String context = "\"id\":" + "\"" + id + "\"," + "\"customNameVal\":" + "\"" + customNameVal;
                    dto.getData().setContent1(context);
                } else if (i == 2 && StringUtils.isNotEmpty(contents.get(2).getCustomNameVal())) {
                    String id = contents.get(2).getId();
                    String customNameVal = contents.get(2).getCustomNameVal();
                    String context = "\"id\":" + "\"" + id + "\"," + "\"customNameVal\":" + "\"" + customNameVal;
                    dto.getData().setContent2(context);
                } else if (i == 3 && StringUtils.isNotEmpty(contents.get(3).getCustomNameVal())) {
                    String id = contents.get(3).getId();
                    String customNameVal = contents.get(3).getCustomNameVal();
                    String context = "\"id\":" + "\"" + id + "\"," + "\"customNameVal\":" + "\"" + customNameVal;
                    dto.getData().setContent3(context);
                } else if (i == 4 && StringUtils.isNotEmpty(contents.get(4).getCustomNameVal())) {
                    String id = contents.get(4).getId();
                    String customNameVal = contents.get(4).getCustomNameVal();
                    String context = "\"id\":" + "\"" + id + "\"," + "\"customNameVal\":" + "\"" + customNameVal;
                    dto.getData().setContent4(context);
                }
            }
        } else {
            dto.getData().setContents(null);
        }
        map.put("adminVo", info);
        map.put("uws", dto.getData());

        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<UserWhitelistVO> list = userWhitelistdao.selectListBySearch(map);
        Admin admin = adminDao.selectByPrimaryKey(info.getAdminId());

        List<Long> userWhitelistIdList = list.stream().map(UserWhitelistVO::getId).collect(Collectors.toList());
        /*如果账号开启了私有区域，则将私有区域列表返回*/
        if (info.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.OPEN_PRIVATE_AREA.getCode())) {
            if (userWhitelistIdList.size() > 0) {
                List<PrivateAreaUserWhiteVO> privateAreaUserWhiteVOList = privateAreaDao.selectByUserWhitelistIdList(userWhitelistIdList);
                Map<Long, List<PrivateAreaUserWhiteVO>> listMap = getByPrivateAreaUserWhiteVOList(privateAreaUserWhiteVOList);
                for (UserWhitelistVO userWhitelistVO : list) {
                    if (listMap.get(userWhitelistVO.getId()) != null) {
                        List<PrivateAreaUserWhiteVO> voList = listMap.get(userWhitelistVO.getId());
                        List<Integer> areaIdList = voList.stream().map(PrivateAreaUserWhiteVO::getPrivateAreaId).collect(Collectors.toList());
                        List<String> areaNameList = voList.stream().map(PrivateAreaUserWhiteVO::getPrivateAreaName).collect(Collectors.toList());
                        userWhitelistVO.setPrivateAreaIdList(areaIdList);
                        userWhitelistVO.setPrivateAreaNameList(areaNameList);
                    }
                }
            }
        }
        if (info.getType() == 5 || info.getType() == 99) {
            for (Integer i = 0; i < list.size(); i++) {
                if (StringUtils.isEmpty(list.get(i).getContent())) {
                    list.get(i).setContent(info.getContent());
                }
                Integer id = list.get(i).getId().intValue();
                List<ContextVO> context = faceService.getContextByWhiteIdTwo(id, admin, list.get(i));
                list.get(i).setContents(context);
            }
        }

        boolean isShowName = false;
        boolean isShowPhone = false;
        boolean isShowNumber = false;
        if (info.getNoHideCloumn() != null && info.getNoHideCloumn().size() > 0) {
            if (info.getNoHideCloumn().contains("1")) {
                isShowName = true;
            }
            if (info.getNoHideCloumn().contains("2")) {
                isShowPhone = true;
            }
            if (info.getNoHideCloumn().contains("3")) {
                isShowNumber = true;
            }
        }

        List<UserWhitelist> whitelists = new ArrayList<>();
        Map<Integer, UserWhitelist> whitelistsMap = new HashMap<>();
        if (Preconditions.isNotBlank(userWhitelistIdList)) {
            whitelists = userWhitelistdao.selectByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andIn(UserWhitelist::getId, userWhitelistIdList)).build());
            whitelistsMap = whitelists.stream().collect(Collectors.toMap(UserWhitelist::getId, a -> a, (k1, k2) -> k1));
        }

        Map<Integer, ZoneDeviceR> zoneDeviceRVoMap = getZoneDeviceRVoMap(whitelists);
        for (int i = 0; i < list.size(); i++) {
            UserWhitelistVO userWhitelistVO = list.get(i);
            userWhitelistVO.setZoneDevicePassIdList(whitelistsMap.get(userWhitelistVO.getId().intValue()).getZoneDevicePassIdList());
            if (CollUtil.isNotEmpty(userWhitelistVO.getZoneDevicePassIdList())) {
                for (Integer zoneDevicePassId : userWhitelistVO.getZoneDevicePassIdList()) {
                    if (zoneDeviceRVoMap.containsKey(zoneDevicePassId)) {
                        ZoneDeviceR zoneDeviceR = zoneDeviceRVoMap.get(zoneDevicePassId);
                        userWhitelistVO.getZoneDeviceRLessVOList().add(new ZoneDeviceRLessVO().setZoneDeviceRId(zoneDeviceR.getId()).setName(zoneDeviceR.getName()));
                    }
                }
            }

            if (userWhitelistVO.getName() != null && isShowName) {
                userWhitelistVO.setName(SMSOrIdCardUtils.hideName(userWhitelistVO.getName().trim()));
            }
            if (userWhitelistVO.getPhone() != null && isShowPhone) {
                userWhitelistVO.setPhone(SMSOrIdCardUtils.hidePhone(userWhitelistVO.getPhone()));
            }
            if (userWhitelistVO.getIdNumber() != null && isShowNumber) {
                userWhitelistVO.setIdNumber(SMSOrIdCardUtils.hideIdNumber(userWhitelistVO.getIdNumber()));
            }

            AdminDetail detail = detailMap.get(userWhitelistVO.getAdminId());
            if (detail != null) {
                //企业简称
                userWhitelistVO.setEnterpriseSimpleName(detail.getEnterpriseSimpleName());
                //企业全称
                userWhitelistVO.setEnterpriseFullName(detail.getEnterpriseFullName());
            }
        }

        return new PageInfo<>(list);
    }

    private Map<Integer, ZoneDeviceR> getZoneDeviceRVoMap(List<UserWhitelist> list) {
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>();
        }
        List<Integer> zoneDeviceRIdList = list.stream().filter(p -> CollUtil.isNotEmpty(p.getZoneDevicePassIdList())).flatMap(p -> p.getZoneDevicePassIdList().stream()).distinct().collect(Collectors.toList());
        if (CollUtil.isEmpty(zoneDeviceRIdList)) {
            return new HashMap<>();
        }
        return zoneDeviceRDao.selectByIds(StrUtil.join(",", zoneDeviceRIdList)).stream().collect(Collectors.toMap(ZoneDeviceR::getId, Function.identity()));
    }

    private Map<Long, List<PrivateAreaUserWhiteVO>> getByPrivateAreaUserWhiteVOList(List<PrivateAreaUserWhiteVO> privateAreaUserWhiteVOList) {
        Map<Long, List<PrivateAreaUserWhiteVO>> map = new HashMap<>();
        for (PrivateAreaUserWhiteVO privateAreaUserWhiteVO : privateAreaUserWhiteVOList) {
            if (map.get(privateAreaUserWhiteVO.getUserWhitelistId()) == null) {
                List<PrivateAreaUserWhiteVO> list = new ArrayList<>();
                list.add(privateAreaUserWhiteVO);
                map.put(privateAreaUserWhiteVO.getUserWhitelistId(), list);
            } else {
                List<PrivateAreaUserWhiteVO> list = map.get(privateAreaUserWhiteVO.getUserWhitelistId());
                list.add(privateAreaUserWhiteVO);
                map.put(privateAreaUserWhiteVO.getUserWhitelistId(), list);
            }
        }
        return map;
    }

    @Override
    public RestResult<String> addSnIssued(IssueAllDTO dto) {
        return this.addSnIssued(dto, null);
    }

    /**
     * 全量白名单下发
     */
    public RestResult<String> addSnIssued(IssueAllDTO dto, TaskInfo taskInfo) {
        Integer deviceId = dto.getDeviceId();
        String startTime = dto.getStartTime();
        String endTime = dto.getEndTime();
        Boolean validityPeriodStatus = dto.getValidityPeriodStatus();

        FaceDevice faceDevice = faceDeviceDao.selectOne(
                new FaceDevice().setId(deviceId)
                        .setEnable(1)//启用状态
                        .setAuth(1));//已授权状态
        if (ObjectUtil.isNull(faceDevice)) {
            return RestResponse.error(RestCode.UNAUTHORIZED, "设备不存在或设备未授权");
        }
        taskInfo = this.taskExcute(dto, faceDevice, taskInfo);
        AdminVO adminVO = adminService.getInfo(faceDevice.getAdminId());
        AdminEffectiveVO adminEffectiveVO = EffectiveUtil.dataCover(adminVO, startTime, endTime, 1);
        faceDevice.setValidityPeriodStatus(validityPeriodStatus);
        faceDevice.setStartTime(adminEffectiveVO.getWhiteStartTime());
        faceDevice.setEndTime(adminEffectiveVO.getWhiteEndTime());
        faceDeviceDao.updateByPrimaryKeySelective(faceDevice);//更新设备的下发时间
        hhfaceIssueService.sendAddSnIssuedMsg(faceDevice, adminVO, taskInfo, dto.getIssueType());
//        asyncTriggerRetryImpl.addSnIssued(faceDevice, adminVO, taskInfo, dto.getIssueType());//异步下发
        return RestResponse.ok("下发成功");
    }

    private TaskInfo taskExcute(IssueAllDTO dto, FaceDevice faceDevice, TaskInfo taskInfo) {
        FaceDeviceType faceDeviceType = faceDeviceTypeDao.selectByPrimaryKey(faceDevice.getFaceDeviceTypeId());
        //增加离线任务  离线并且不是手持的设备需要增加为离线任务
        if (faceDevice.getOnline().equals(0)) {//设备离线
            //如果不是手持终端需要生成离线任务
            if (!faceDeviceType.getCode().equals(FaceDeviceTypeEnum.HANDHELD_TERMINALS.getCode())
                    && !faceDeviceType.getCode().equals(FaceDeviceTypeEnum.SMART_LOCK.getCode())
                    && !faceDeviceType.getCode().equals(FaceDeviceTypeEnum.BLUETOOTH_SMART_LOCK.getCode())) {
                if (taskInfo == null) {//生成离线任务
                    taskInfo = taskInfoService.save(faceDevice.getSn(), JSON.toJSONString(dto));
                } else {
                    Integer retry = taskInfo.getRetry();
                    if (retry != null && retry >= 1) {
                        retry = retry - 1;
                        taskInfo.setRetry(retry);
                        taskInfoDao.updateByPrimaryKeySelective(taskInfo);
                    } else {
                        taskInfoDao.deleteByPrimaryKey(taskInfo.getId());
                    }
                }
            }
        } else {
            if (taskInfo == null) {//生成在线任务
                taskInfo = new TaskInfo();
                taskInfo.setSn(faceDevice.getSn());
                taskInfo.setCreateTime(new Date());
                taskInfo.setUpdateTime(new Date());
                taskInfo.setType(TaskTypeEnum.ALL_ISSUE.getType());
                taskInfo.setOnlineFlag(TaskInfoOnlineStatus.OnLine.getCode());
                taskInfo.setExecuteResult("在线任务,正在执行");
                taskInfo.setCreateId(faceDevice.getAdminId());
                taskInfo.setStatus(AutoTaskInfoStatusEnum.HANDLED.getStatus());
                taskInfoDao.insertSelective(taskInfo);
            } else {
                taskInfo.setStatus(AutoTaskInfoStatusEnum.HANDLED.getStatus());
                taskInfo.setExecuteResult("离线任务,正在执行");
                taskInfoDao.updateByPrimaryKeySelective(taskInfo);
            }
        }
        return taskInfo;
    }

    @Override
    public RestResult<String> addSingle(Integer userWhitelistId, Date startTime, Date endTime) {
        RestResult<String> objectRestResult = new RestResult<>();
        UserWhitelist userWhitelist = userWhitelistdao.selectByPrimaryKey(userWhitelistId);
        if (userWhitelist == null || userWhitelist.getUserId() == null) {
            objectRestResult.setCode(405);
            objectRestResult.setMsg("下发失败");
            return objectRestResult;
        }
        //TODO。。。查询当前账号下所有设备
        //查询当前账号信息
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        List<String> allSns = new ArrayList<>();
        List<Integer> adminIds = new ArrayList<>();
        List<String> zoneSns = new ArrayList<>();
        List<String> userZoneSns = new ArrayList<>();
        adminIds.add(admin.getId());
        if (admin.getType().equals(AdminTypeEnum.ENTERPRISE.getCode()) && Preconditions.isNotBlank(admin.getTenantId())) { //企业账号设备
            adminIds.add(admin.getTenantId());
        }

        if (admin.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.OPEN_PRIVATE_AREA.getCode()) && Preconditions.isNotBlank(userWhitelist)) { //私有区域设备
            List<FaceDeviceLessVO> snListByUserWhitelistId = faceDeviceDao.getSnListByUserWhitelistId(userWhitelist.getId());
            List<String> sns = snListByUserWhitelistId.stream().map(FaceDeviceLessVO::getSn).collect(Collectors.toList());
            if (Preconditions.isNotBlank(sns)) {
                allSns.addAll(sns);
            }
            List<Integer> privateAdminIds = snListByUserWhitelistId.stream().map(FaceDeviceLessVO::getAdminId).collect(Collectors.toList());
            if (Preconditions.isNotBlank(privateAdminIds)) {
                adminIds.addAll(privateAdminIds);
            }
        }

        //获取账号下所有设备
        if (Preconditions.isNotBlank(adminIds)) {
            List<FaceDevice> devices = faceDeviceDao.selectByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andIn(FaceDevice::getAdminId, adminIds).andEqualTo(FaceDevice::getAuth, 1)
                    .andEqualTo(FaceDevice::getEnable, 1)).build());
            if (Preconditions.isNotBlank(devices)) {
                allSns.addAll(devices.stream().map(FaceDevice::getSn).collect(Collectors.toList()));
            }

            //获取所有区域设备 和 当前用户所在区域设备
            List<ZoneDeviceR> zones = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom().andIsNotNull(ZoneDeviceR::getSnJson).andIn(ZoneDeviceR::getAdminId, adminIds).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
            if (Preconditions.isNotBlank(zones)) {
                for (ZoneDeviceR item : zones) {
                    zoneSns.addAll(Arrays.asList(item.getSnJson().split(",")));
                    if (Preconditions.isNotBlank(userWhitelist.getZoneDevicePassIdList()) && userWhitelist.getZoneDevicePassIdList().contains(item.getId())) {
                        userZoneSns.addAll(Arrays.asList(item.getSnJson().split(",")));
                    }
                }
            }
        }

        if (Preconditions.isBlank(allSns)) {
            log.error("--->该账号未绑定设备");
            objectRestResult.setCode(400);
            objectRestResult.setMsg("该账号未绑定设备");
            return objectRestResult;
        }

        if (Preconditions.isNotBlank(zoneSns)) {
            zoneSns = zoneSns.stream().filter(item -> allSns.contains(item)).collect(Collectors.toList());
        }

        if (Preconditions.isNotBlank(userZoneSns)) {
            userZoneSns = userZoneSns.stream().filter(item -> allSns.contains(item)).collect(Collectors.toList());
        }

        //下发用户信息到设备
        issueDeviceUser(userWhitelist, allSns, zoneSns, userZoneSns);

        objectRestResult.setCode(200);
        objectRestResult.setMsg("下发成功");
        return objectRestResult;
    }

    private void issueDeviceUser(UserWhitelist userWhitelist, List<String> allSns, List<String> zoneSns, List<String> userZoneSns) {

        //删除所有设备上此用户的数据
//        deleteWhiteListUserFromDevice(userWhitelist, allSns);

        List<String> issueSns = new ArrayList<>();

        if (Preconditions.isNotBlank(zoneSns)) {
            //过滤掉区域设备  共有区域设备
            allSns = allSns.stream().filter(item -> !zoneSns.contains(item)).collect(Collectors.toList());
            if (Preconditions.isNotBlank(allSns)) {
                issueSns.addAll(allSns);
            }
            //用户所在区域可执行的设备sn
            if (Preconditions.isNotBlank(userZoneSns)) {
                issueSns.addAll(userZoneSns);
            }
        } else {
            issueSns = allSns;
        }

        if (Preconditions.isBlank(issueSns)) {
            return;
        }

        String facialFeature = null;
        if (Preconditions.isNotBlank(userWhitelist.getUserId())) {
            UserInfo userInfo = userInfoDao.selectByPrimaryKey(userWhitelist.getUserId());
            facialFeature = userInfo.getFacialFeature();
        }
        //下发用户信息到设备
        for (String sn : issueSns) {
            DownFaceNew downFaceNew = new DownFaceNew();
            downFaceNew.setCardid(userWhitelist.getIdNumber());
            downFaceNew.setCardIdEx(userWhitelist.getCardIdEx());
            downFaceNew.setRightno(UserRightNoEnum.REGULAR.getCode());
            downFaceNew.setPhoto(userWhitelist.getAvatar());
            downFaceNew.setName(userWhitelist.getName());
            downFaceNew.setUserId(userWhitelist.getUserId());
            downFaceNew.setSn(sn);
            downFaceNew.setPhone(userWhitelist.getPhone());
            downFaceNew.setValidPeriodStart(userWhitelist.getStartTime());
            downFaceNew.setValidPeriodEnd(userWhitelist.getEndTime());
            downFaceNew.setFacialFeature(facialFeature);
            downFaceNew.setAdminId(userWhitelist.getAdminId());
            faceTBService.downUser(downFaceNew);
        }
    }

    public void deleteWhiteListUserFromDevice(UserWhitelist userWhitelist, List<String> sns) {
        if (Preconditions.isBlank(sns)) {
            return;
        }
        for (String sn : sns) {
            faceTBService.delFaceBySn(sn, Arrays.asList(userWhitelist.getUserId()));
        }
    }


    /**
     * 删除
     *
     * @param adminId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Integer deleteAll(Integer adminId) {
        List<Long> userIds = userWhitelistdao.getUserIdsByAdminId(adminId);
        List<FaceDevice> deviceList = faceDeviceDao.getFaceDeviceByAdminId(adminId);
        deleteALlWhiteListUsersFromDevice(userIds, deviceList);
        userWhitelistdao.deleteAdminIdAll(adminId);
        //删除2188 iot用户出库
        if (Preconditions.isNotBlank(userIds)) {
            alipayOpenApiService.deleteIotUser(userIds);
        }
        return 1;
    }

    private void deleteALlWhiteListUsersFromDevice(List<Long> userIds, List<FaceDevice> deviceList) {
        for (FaceDevice faceDevice : deviceList) {
            faceTBService.delFaceBySn(faceDevice.getSn(), userIds);
        }
    }

    /**
     * @param userWhitelist
     * @param type          1   学校场景第一次下发 不携带人脸 2 补全信息下发人脸
     * @param source        1 批量导入
     */
    @Override
    public void addSingleWhite(UserWhitelist userWhitelist, int type, Integer source) {
        if (StringUtils.isBlank(userWhitelist.getIdNumber())
                && StringUtils.isBlank(userWhitelist.getPhone())
                && StringUtils.isBlank(userWhitelist.getCardIdEx())) {
            throw new ServiceException("此数据手机号-身份证-卡号均为空");
        }
        if (userWhitelist.getUserId() == null) {
            throw new ServiceException("用户数据主键为空");
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userWhitelist.getUserId());
        if (userInfo == null) {
            throw new ServiceException("用户数据为空");
        }
        List<String> list = faceDeviceDao.selectList(userWhitelist.getAdminId());
        Admin admin = adminDao.selectByPrimaryKey(userWhitelist.getAdminId());
        if (admin.getIsOpenPrivateArea().equals(AdminIsOpenPrivateAreaEnum.OPEN_PRIVATE_AREA.getCode())) {
            List<FaceDeviceLessVO> snListByUserWhitelistId = faceDeviceDao.getSnListByUserWhitelistId(userWhitelist.getId());
            List<String> collect = snListByUserWhitelistId.stream().map(FaceDeviceLessVO::getSn).collect(Collectors.toList());
            if (list == null) {
                list = new ArrayList<>();
            }
            list.addAll(collect);
        }
        if (admin.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            list.addAll(faceDeviceDao.selectList(admin.getTenantId()));
        }
        log.info("设备列表1：{}", JSON.toJSONString(list));
        if (CollectionUtil.isEmpty(list)) {
            return;
        }

        for (String sn : list) {
            DownFaceNew downFaceNew = new DownFaceNew();
            downFaceNew.setCardid(userWhitelist.getIdNumber());
            downFaceNew.setCardIdEx(userWhitelist.getCardIdEx());
            downFaceNew.setRightno(UserRightNoEnum.REGULAR.getCode());
            downFaceNew.setPhoto(userWhitelist.getAvatar());
            downFaceNew.setName(userWhitelist.getName());
            downFaceNew.setUserId(userInfo.getId());
            downFaceNew.setSn(sn);
            downFaceNew.setPhone(userWhitelist.getPhone());
            downFaceNew.setValidPeriodStart(userWhitelist.getStartTime());
            downFaceNew.setValidPeriodEnd(userWhitelist.getEndTime());
            downFaceNew.setFacialFeature(userInfo.getFacialFeature());
            downFaceNew.setAdminId(userWhitelist.getAdminId());
            faceTBService.downUser(downFaceNew);
        }
    }

    /**
     * 校验是否常客设置
     *
     * @param adminId
     * @return
     */
    @Override
    public Boolean whiteVisitorType(Integer adminId) {
        String whiteVisitorType = userInfoDao.selectWhiteVisitorType(adminId.toString());
//      Boolean b = (StringUtils.isNotBlank(whiteVisitorType) && whiteVisitorType.contains("1") && !whiteVisitorType.contains("2"));
        Boolean b = (StringUtils.isNotBlank(whiteVisitorType) && whiteVisitorType.contains("1"));
        return b;
    }

    @Override
    public List<UserWhitelistVO> userWhitelistExcelOut(UserWhitelistSearch userWhitelistSearch) {
        RestDTO<UserWhitelistSearch> dto = new RestDTO<>();
        dto.setPageNum(0);
        dto.setPageSize(50000);
        dto.setData(userWhitelistSearch);
        PageInfo<UserWhitelistVO> pageList = getPageList(dto);
        return pageList.getList();
    }

    @Override
    public UserWhitelist getByIdNumberAndId(String idNumber, Object o, Integer adminId) {
        //查询常客信息
        UserWhitelist userWhitelist = userWhitelistdao.getByIdNumberAndId(idNumber, null, adminId);
        return userWhitelist;
    }

    @Override
    public UserWhitelist getByPhoneAndId(String phone, Object o, Integer adminId) {
        UserWhitelist userWhitelist = userWhitelistdao.getByPhoneAndId(phone, null, adminId);
        return userWhitelist;
    }

    @Override
    public UserWhitelist getOne(Integer id) {
        return userWhitelistdao.selectByPrimaryKey(id);
    }

    @Override
    public UserWhitelist getUserWhitelist(String idNumber, String phone, String cardIdEx, Integer adminId) {
        UserWhitelist userWhitelist = null;
        if (StrUtil.isNotEmpty(idNumber)) {
            userWhitelist = userWhitelistdao.getByIdNumberAndId(idNumber, null, adminId);
            if ((ObjectUtil.isNotNull(userWhitelist))) {
                return userWhitelist;
            }
        }
        if (StrUtil.isNotEmpty(phone)) {
            userWhitelist = userWhitelistdao.getByPhoneAndId(phone, null, adminId);
            if ((ObjectUtil.isNotNull(userWhitelist))) {
                return userWhitelist;
            }
        }
        if (StrUtil.isNotEmpty(cardIdEx)) {
            userWhitelist = userWhitelistdao.getByCardIdExAndId(cardIdEx, null, adminId);
            if ((ObjectUtil.isNotNull(userWhitelist))) {
                return userWhitelist;
            }
        }
        return userWhitelist;
    }

    @Override
    public RestResult<UserWhitelistParentVO> getParentInfo(Integer userWhitelistId) {
        UserWhitelist userWhitelist = userWhitelistdao.selectByPrimaryKey(userWhitelistId);
        ClassUtil.checkData(userWhitelist, "常客不存在");
        UserWhitelistSchool userWhitelistSchool = userWhitelistSchoolDao.selectOne(new UserWhitelistSchool().setUserWhitelistId(userWhitelist.getId()));
        if (ObjectUtil.isNull(userWhitelistSchool)) {
            return RestResult.ok(new UserWhitelistParentVO());
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userWhitelistSchool.getUserId());
        UserWhitelistParentVO userWhitelistParentVO = BeanUtil.copyProperties(userInfo, UserWhitelistParentVO.class);
        return RestResult.ok(userWhitelistParentVO);

    }

    @Override
    public RestResult<String> batchEditUserWhitelist(UserWhitelistBatchDTO dto) {
        AdminVO info = adminService.getInfo();
        this.checkDto(dto, info);
        //根据唯一标识获取数据
        UserWhitelist userWhitelist = this.getUniqueCode(dto);
        userWhitelist.setName(StrUtil.isNotBlank(dto.getName()) ? dto.getName() : userWhitelist.getName());
        userWhitelist.setIdNumber(StrUtil.isNotBlank(dto.getIdNumber()) ? dto.getIdNumber() : userWhitelist.getIdNumber());
        userWhitelist.setPhone(StrUtil.isNotBlank(dto.getPhone()) ? dto.getPhone() : userWhitelist.getPhone());
        userWhitelist.setCardIdEx(StrUtil.isNotBlank(dto.getCardIdEx()) ? dto.getCardIdEx() : userWhitelist.getCardIdEx());
        userWhitelist.setContent(CollUtil.isNotEmpty(dto.getAddCustomList()) ? JSON.toJSONString(dto.getAddCustomList()) : userWhitelist.getContent());
        userWhitelist.setStartTime(StrUtil.isNotBlank(dto.getStartTime()) ? DateUtils.parseDate(dto.getStartTime()) : userWhitelist.getStartTime());
        userWhitelist.setEndTime(StrUtil.isNotBlank(dto.getEndTime()) ? DateUtils.parseDate(dto.getEndTime()) : userWhitelist.getEndTime());
        userWhitelist.setPersonnelType(dto.getPersonnelType() != null ? dto.getPersonnelType() : userWhitelist.getPersonnelType());
        userWhitelist.setZoneDevicePassIdList(CollUtil.isNotEmpty(dto.getZoneDevicePassIdList()) ? dto.getZoneDevicePassIdList() : userWhitelist.getZoneDevicePassIdList());
        Long userId = userInfoService.addOrUpdateUserInfo(userWhitelist);
        userWhitelist.setUserId(userId);
        userWhitelistdao.updateByPrimaryKeySelective(userWhitelist);
        return RestResult.ok("修改成功");
    }

    @Override
    public Integer batchUpdate(UserWhitelistBatchUpdateDTO userWhitelistDTO) {
        ClassUtil.throwBusinessException(userWhitelistDTO.getIds() == null, "修改时常客主键不能为空");
        ClassUtil.throwBusinessException(!whiteVisitorType(userWhitelistDTO.getAdminId()), "没有开启常客功能");
        List<UserWhitelist> whitelists = userWhitelistdao.selectByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andIn(UserWhitelist::getId, userWhitelistDTO.getIds()).andEqualTo(UserWhitelist::getAdminId, userWhitelistDTO.getAdminId())).build());
        ClassUtil.throwBusinessException(Preconditions.isBlank(whitelists), "常客数据不存在");
        List<Integer> zoneIdList = userWhitelistDTO.getZoneDevicePassIdList();
        if (Preconditions.isNotBlank(zoneIdList)) {
            List<ZoneDeviceR> zoneDeviceRList = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom().andIn(ZoneDeviceR::getId, zoneIdList).andEqualTo(ZoneDeviceR::getAdminId, userWhitelistDTO.getAdminId()).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
            ClassUtil.throwBusinessException(Preconditions.isBlank(zoneDeviceRList), "所属区域数据不存在");
        }
        List<Integer> ids = whitelists.stream().map(UserWhitelist::getId).collect(Collectors.toList());

        UserWhitelist item = new UserWhitelist();
        item.setContent(userWhitelistDTO.getContent());
        item.setStartTime(userWhitelistDTO.getStartTime());
        item.setEndTime(userWhitelistDTO.getEndTime());
        item.setZoneDevicePassIdList(userWhitelistDTO.getZoneDevicePassIdList());
        userWhitelistdao.batchUpdateByIds(ids, item);
        List<FaceDevice> deviceList = faceDeviceDao.selectByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getAdminId, userWhitelistDTO.getAdminId()).andEqualTo(FaceDevice::getEnable, 1).andEqualTo(FaceDevice::getAuth, 1)).build());
        if (Preconditions.isNotBlank(deviceList)) {
            //TODO。。。删除并下发用户数据
//            List<Long> userIds = whitelists.stream().map(UserWhitelist::getUserId).collect(Collectors.toList());
            List<String> sns = deviceList.stream().map(FaceDevice::getSn).collect(Collectors.toList());
            //删除
//            log.info("--->用户id:{}, 设备：{} ", JSONObject.toJSONString(userIds), JSONObject.toJSONString(sns));
//            deleteALlWhiteListUsersFromDevice(userIds, deviceList);

            //下发
            whitelists.stream().forEach(oldItem -> {
                oldItem.setContent(Preconditions.isNotBlank(userWhitelistDTO.getContent()) ? userWhitelistDTO.getContent() : oldItem.getContent());
                oldItem.setStartTime(Preconditions.isNotBlank(userWhitelistDTO.getStartTime()) ? userWhitelistDTO.getStartTime() : oldItem.getStartTime());
                oldItem.setEndTime(Preconditions.isNotBlank(userWhitelistDTO.getEndTime()) ? userWhitelistDTO.getEndTime() : oldItem.getEndTime());
                oldItem.setZoneDevicePassIdList(Preconditions.isNotBlank(userWhitelistDTO.getZoneDevicePassIdList()) ? userWhitelistDTO.getZoneDevicePassIdList() : oldItem.getZoneDevicePassIdList());
            });
            AdminVO adminVo = adminService.getInfo(userWhitelistDTO.getAdminId());
            batchIssued(adminVo, whitelists, sns);
        } else {
            log.error("--->设备数据未授权或已停用");
        }
        return ids.size();
    }

    @Override
    public Integer batchDelete(UserWhitelistBatchDelDTO dto) {
        ClassUtil.throwBusinessException(dto.getIds() == null, "批量删除常客主键不能为空");
        ClassUtil.throwBusinessException(!whiteVisitorType(dto.getAdminId()), "没有开启常客功能");
        List<UserWhitelist> whitelists = userWhitelistdao.selectByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andIn(UserWhitelist::getId, dto.getIds()).andEqualTo(UserWhitelist::getAdminId, dto.getAdminId())).build());
        ClassUtil.throwBusinessException(Preconditions.isBlank(whitelists), "常客数据不存在");
        List<Long> userIds = whitelists.stream().map(UserWhitelist::getUserId).collect(Collectors.toList());
        userWhitelistdao.deleteByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andIn(UserWhitelist::getId, dto.getIds()).andEqualTo(UserWhitelist::getAdminId, dto.getAdminId())).build());
        List<FaceDevice> deviceList = faceDeviceDao.selectByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getAdminId, dto.getAdminId()).andEqualTo(FaceDevice::getEnable, 1).andEqualTo(FaceDevice::getAuth, 1)).build());
        if (Preconditions.isNotBlank(deviceList)) {
            deleteALlWhiteListUsersFromDevice(userIds, deviceList);
        } else {
            log.error("--->设备数据未授权或已停用");
        }
        return 1;
    }

    @Override
    public RestResult batchDistribution(UserWhitelistBatchDistributionDTO dto) {
        ClassUtil.throwBusinessException(dto.getIds() == null, "批量下发常客主键不能为空");
        ClassUtil.throwBusinessException(!whiteVisitorType(dto.getAdminId()), "没有开启常客功能");
        List<UserWhitelist> whitelists = userWhitelistdao.selectByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andIn(UserWhitelist::getId, dto.getIds()).andEqualTo(UserWhitelist::getAdminId, dto.getAdminId())).build());
        ClassUtil.throwBusinessException(Preconditions.isBlank(whitelists), "常客数据不存在");
        List<FaceDevice> deviceList = faceDeviceDao.selectByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getAdminId, dto.getAdminId()).andEqualTo(FaceDevice::getEnable, 1).andEqualTo(FaceDevice::getAuth, 1)).build());
        if (Preconditions.isNotBlank(deviceList)) {
            //清空设备用户信息
//            List<Long> userIds = whitelists.stream().map(UserWhitelist::getUserId).collect(Collectors.toList());
//            deleteALlWhiteListUsersFromDevice(userIds, deviceList);
            AdminVO adminVo = adminService.getInfo(dto.getAdminId());
            //下发用户数据
            batchIssued(adminVo, whitelists, deviceList.stream().map(FaceDevice::getSn).collect(Collectors.toList()));
        } else {
            log.error("--->设备数据未授权或已停用");
        }
        return RestResult.ok("success");
    }

    private void batchIssued(AdminVO adminVo, List<UserWhitelist> whitelists, List<String> sns) {
        List<Long> userIds = whitelists.stream().map(UserWhitelist::getUserId).collect(Collectors.toList());
        List<UserInfo> userInfos = userInfoDao.selectByExample(Example.builder(UserInfo.class).where(WeekendSqls.<UserInfo>custom().andIn(UserInfo::getId, userIds)).build());
        Map<Long, UserInfo> userInfoMap = userInfos.stream().collect(Collectors.toMap(UserInfo::getId, v -> v, (v1, v2) -> v1));

        //查询所有区域
        List<ZoneDeviceR> zoneDeviceRList = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom().andEqualTo(ZoneDeviceR::getAdminId, adminVo.getAdminId()).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        for (String sn : sns) {
            for (UserWhitelist userWhitelist : whitelists) {
                List<String> userZoneDeviceSns = new ArrayList<>();
                if (Preconditions.isNotBlank(zoneDeviceRList) && Preconditions.isNotBlank(userWhitelist.getZoneDevicePassIdList())) {
                    List<ZoneDeviceR> collect = zoneDeviceRList.stream().filter(item -> Preconditions.isNotBlank(item.getSnJson()) && userWhitelist.getZoneDevicePassIdList().contains(item.getId())).collect(Collectors.toList());
                    collect.stream().forEach(item -> {
                        userZoneDeviceSns.addAll(Arrays.asList(item.getSnJson().split(",")));
                    });
                }
                if (Preconditions.isNotBlank(userZoneDeviceSns) && !userZoneDeviceSns.contains(sn)) {
                    continue;
                }
                Date startTime = userWhitelist.getStartTime() == null ? new Date() : userWhitelist.getStartTime();
                Date endTime = userWhitelist.getEndTime();
                if (Preconditions.isBlank(userWhitelist.getEndTime())) {
                    if (adminVo.getUserWhitelistEffectiveTime() != null) {
                        int timelong = adminVo.getUserWhitelistEffectiveTime().multiply(new BigDecimal(1440)).intValue();
                        endTime = DateUtils.addMinutes(new Date(), timelong);
                    } else {
                        endTime = DateUtil.offset(DateUtil.date(), DateField.YEAR, 1);
                    }
                }
                UserInfo userInfo = userInfoMap.get(userWhitelist.getUserId());
                String facialFeature = null;
                if (Preconditions.isNotBlank(userInfo)) {
                    facialFeature = userInfo.getFacialFeature();
                }
                DownFaceNew downFaceNew = new DownFaceNew();
                downFaceNew.setCardid(userWhitelist.getIdNumber());
                downFaceNew.setCardIdEx(userWhitelist.getCardIdEx());
                downFaceNew.setRightno(UserRightNoEnum.REGULAR.getCode());
                downFaceNew.setPhoto(userWhitelist.getAvatar());
                downFaceNew.setName(userWhitelist.getName());
                downFaceNew.setUserId(userWhitelist.getUserId());
                downFaceNew.setSn(sn);
                downFaceNew.setPhone(userWhitelist.getPhone());
                downFaceNew.setValidPeriodStart(startTime);
                downFaceNew.setValidPeriodEnd(endTime);
                downFaceNew.setFacialFeature(facialFeature);
                downFaceNew.setAdminId(userWhitelist.getAdminId());
                faceTBService.downUser(downFaceNew);
            }
        }
    }

    private UserWhitelist getUniqueCode(UserWhitelistBatchDTO dto) {
        UserWhitelist userWhitelist = null;
        String msg = "";
        switch (dto.getUniqueCode()) {
            case 1:
                ClassUtil.throwBusinessException(StrUtil.isBlank(dto.getIdNumber()), "身份证号不能为空");
                userWhitelist = userWhitelistdao.getByIdNumberAndId(dto.getIdNumber(), null, dto.getAdminId());
                msg = "身份证号";
                break;
            case 2:
                ClassUtil.throwBusinessException(StrUtil.isBlank(dto.getPhone()), "手机号不能为空");
                userWhitelist = userWhitelistdao.getByPhoneAndId(dto.getPhone(), null, dto.getAdminId());
                msg = "手机号";
                break;
            case 3:
                ClassUtil.throwBusinessException(StrUtil.isBlank(dto.getCardIdEx()), "物理卡号不能为空");
                userWhitelist = userWhitelistdao.getByCardIdExAndId(dto.getCardIdEx(), null, dto.getAdminId());
                msg = "物理卡号";
                break;
        }
        ClassUtil.checkData(userWhitelist, "【" + msg + "】" + "查询，常客不存在");
        if (StrUtil.isNotBlank(dto.getIdNumber())) {
            UserWhitelist object = userWhitelistdao.getByIdNumberAndId(dto.getIdNumber(), userWhitelist.getId(), dto.getAdminId());
            ClassUtil.throwBusinessException(ObjectUtil.isNotNull(object), "身份证号重复");
        }
        if (StrUtil.isNotBlank(dto.getPhone())) {
            UserWhitelist object = userWhitelistdao.getByPhoneAndId(dto.getPhone(), userWhitelist.getId(), dto.getAdminId());
            ClassUtil.throwBusinessException(ObjectUtil.isNotNull(object), "手机号重复");
        }
        if (StrUtil.isNotBlank(dto.getCardIdEx())) {
            UserWhitelist object = userWhitelistdao.getByCardIdExAndId(dto.getCardIdEx(), userWhitelist.getId(), dto.getAdminId());
            ClassUtil.throwBusinessException(ObjectUtil.isNotNull(object), "物理卡号重复");
        }
        return userWhitelist;
    }

    private void checkDto(UserWhitelistBatchDTO dto, AdminVO info) {
        if (info.getPlaceType().equals(AdminPlaceTypeEnum.BUILDING_PLACE.getCode())
                && info.getType().equals(AdminTypeEnum.DISTRICT.getCode())) {
            ClassUtil.throwBusinessException(dto.getAdminId() == null, "楼宇的区级账号，请传入要修改的企业账号");
        } else {
            dto.setAdminId(info.getAdminId());
        }
    }
}
