package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.AddressUtil;
import com.yx.face.boot.uitls.ClassUtil;
import com.yx.face.boot.uitls.PageInfoUtil;
import com.yx.face.dao.WarningLogDao;
import com.yx.face.model.dto.UserBlackListDTO;
import com.yx.face.model.dto.WaringLogSureDTO;
import com.yx.face.model.dto.WaringLogDTO;
import com.yx.face.model.entity.UserBlackList;
import com.yx.face.model.entity.WarningLog;
import com.yx.face.model.enums.AdminNoHideCloumnEnum;
import com.yx.face.model.search.WaringLogSearch;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.WaringLogVo;
import com.yx.face.service.AdminService;
import com.yx.face.service.UserBlackListService;
import com.yx.face.service.WarningLogService;
import com.yx.face.service.excel.ExcelOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.weekend.Weekend;
import tk.mybatis.mapper.weekend.WeekendCriteria;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName WarningServiceImpl
 * @Author WXG
 * @Date 2023/12/5 16:39
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Service
@Slf4j
public class WarningLogServiceImpl implements WarningLogService {
    private final WarningLogDao warningLogDao;
    private final AdminService adminService;
    private final UserBlackListService userBlackListService;

   

    @Override
    public PageInfo<WaringLogVo> getWaringLogs(RestDTO<WaringLogSearch> dto) {
        AdminVO info = adminService.getInfo();
        Weekend<WarningLog> weekend = this.getPageWrapper(dto.getData());
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        List<WarningLog> list = warningLogDao.selectByExample(weekend);
        List<String> noHideCloumn = info.getNoHideCloumn();
        PageInfo<WarningLog> pageInfo = new PageInfo<>(list);
        boolean isHideName = noHideCloumn.contains(AdminNoHideCloumnEnum.XS.getCode() + "");
        boolean isHidePhone = noHideCloumn.contains(AdminNoHideCloumnEnum.SJH.getCode() + "");
        boolean isHideNumber = noHideCloumn.contains(AdminNoHideCloumnEnum.SFZH.getCode() + "");

        return PageInfoUtil.convertPageInfo(pageInfo, (object) -> {
            WaringLogVo item = BeanUtil.copyProperties(object, WaringLogVo.class);
            item.setWaringLogId(object.getId());
            if (isHideName) {
                item.setName(DesensitizedUtil.chineseName(object.getName()));
            }
            if (isHidePhone) {
                item.setPhone(DesensitizedUtil.fixedPhone(object.getPhone()));
            }
            if (isHideNumber) {
                item.setCardid(DesensitizedUtil.idCardNum(object.getCardid(), 4, 4));
            }
            return item;
        });
    }

    private Weekend<WarningLog> getPageWrapper(WaringLogSearch data) {
        List<AdminVO> adminlist = adminService.getAdminlist();
        List<Integer> adminIdList = adminlist.stream().map(AdminVO::getAdminId).collect(Collectors.toList());
        Weekend<WarningLog> weekend = new Weekend<>(WarningLog.class);
        WeekendCriteria<WarningLog, Object> weekendCriteria = weekend.weekendCriteria();
        weekendCriteria.andIn(WarningLog::getAdminId, adminIdList);
        if (StrUtil.isNotBlank(data.getDeviceSn())) {
            weekendCriteria.andEqualTo(WarningLog::getDeviceSn, data.getDeviceSn());
        }
        if (data.getAdminId() != null) {
            weekendCriteria.andEqualTo(WarningLog::getAdminId, data.getAdminId());
        }
        if (StrUtil.isNotBlank(data.getName())) {
            weekendCriteria.andLike(WarningLog::getName, data.getName());
        }
        if (StrUtil.isNotBlank(data.getPhone())) {
            weekendCriteria.andEqualTo(WarningLog::getPhone, data.getPhone());
        }
        if (StrUtil.isNotBlank(data.getIdNumber())) {
            weekendCriteria.andEqualTo(WarningLog::getCardid, data.getIdNumber());
        }
        if (StrUtil.isNotBlank(data.getCardIdEx())) {
            weekendCriteria.andEqualTo(WarningLog::getCardIdEx, data.getCardIdEx());
        }
        if (data.getPass() != null) {
            weekendCriteria.andEqualTo(WarningLog::getOutType, data.getPass());
        }
        if (data.getVerifyType() != null) {
            weekendCriteria.andEqualTo(WarningLog::getVerifyType, data.getVerifyType());
        }
        if (data.getRightno() != null) {
            weekendCriteria.andEqualTo(WarningLog::getRightno, data.getRightno());
        }
        if (StrUtil.isNotBlank(data.getStartTime())) {
            weekendCriteria.andGreaterThanOrEqualTo(WarningLog::getFaceTime, data.getStartTime());
        }
        if (StrUtil.isNotBlank(data.getEndTime())) {
            weekendCriteria.andLessThanOrEqualTo(WarningLog::getFaceTime, data.getEndTime());
        }
        weekend.orderBy("id").desc();
        return weekend;
    }

    @Override
    public String doGetWaringServerLogExcel(WaringLogSearch dto, HttpServletResponse response) {
        AdminVO info = adminService.getInfo();
        ClassUtil.throwBusinessException(info.getIsOut() != 1, "没有导出权限");
        Weekend<WarningLog> weekend = this.getPageWrapper(dto);
        PageHelper.startPage(0, 50000);
        List<WarningLog> list = warningLogDao.selectByExample(weekend);

        String uploadExcel = Constants.UPLOAD_EXCEL;
        String fileName = uploadExcel + "/预警记录-" + System.currentTimeMillis() + ".xlsx";
        try {
            Path directory = Paths.get(uploadExcel);
            if (!Files.exists(directory)) {
                Files.createDirectories(directory);
            }
            OutputStream out = Files.newOutputStream(Paths.get(fileName));
            ExcelOperation.InoutLogExcelOut(list, out);
            out.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AddressUtil.address.concat(fileName);
    }

    @Override
    public RestResult<Boolean> warningSure(WaringLogSureDTO dto) {
        WarningLog warningLog = warningLogDao.selectByPrimaryKey(dto.getWaringLogId());
        ClassUtil.checkData(warningLog, "预警记录不存在");
        ClassUtil.throwBusinessException(warningLog.getIsSure(), "预警记录已确认");
        warningLog.setIsSure(true);
        warningLog.setRemark(dto.getRemark());
        warningLogDao.updateByPrimaryKeySelective(warningLog);
        log.info("预警记录id:{}，确认", warningLog.getId());
        return RestResult.ok(true);
    }

    @Override
    public RestResult<Boolean> warningDelete(WaringLogDTO dto) {
        WarningLog warningLog = warningLogDao.selectByPrimaryKey(dto.getWaringLogId());
        ClassUtil.checkData(warningLog, "预警记录不存在");
        warningLogDao.deleteByPrimaryKey(warningLog.getId());
        log.info("预警记录删除:{}", JSON.toJSONString(warningLog));
        return RestResult.ok(true);
    }

    @Override
    public RestResult<Boolean> warningBlack(WaringLogDTO dto) {
        WarningLog warningLog = warningLogDao.selectByPrimaryKey(dto.getWaringLogId());
        ClassUtil.checkData(warningLog, "预警记录不存在");
        ClassUtil.throwBusinessException(warningLog.getIsBlack(), "预警记录已拉黑");
        UserBlackList userBlackList = userBlackListService.getByPhoneAndIdNumber(warningLog.getCardid(), null, warningLog.getAdminId());
        if (ObjectUtil.isNull(userBlackList)) {
            UserBlackListDTO userBlackListDTO = new UserBlackListDTO();
            userBlackListDTO.setName(warningLog.getName())
                    .setPhone(warningLog.getPhone())
                    .setIdNumber(warningLog.getCardid())
                    .setAdminId(warningLog.getAdminId());
            userBlackListService.insert(userBlackListDTO);
        }
        warningLog.setIsBlack(true);
        warningLogDao.updateByPrimaryKeySelective(warningLog);
        log.info("预警记录id:{}，拉黑", warningLog.getId());
        return RestResult.ok(true);
    }
}

