package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.rabbitConfig.config.SnQuenueConfig;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.dao.AdminAuthDao;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.UserWhitelistDao;
import com.yx.face.dao.ZoneDeviceRDao;
import com.yx.face.model.api.district.OpenDistrictDeleteResponse;
import com.yx.face.model.entity.AdminAuth;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.UserWhitelist;
import com.yx.face.model.entity.ZoneDeviceR;
import com.yx.face.model.enums.AdminAuthTypeEnum;
import com.yx.face.model.vo.FaceDeviceVO;
import com.yx.face.model.vo.third.InterfAddrJsonVo;
import com.yx.face.model.vo.third.ZoneDeviceRDto;
import com.yx.face.model.vo.third.ZoneDeviceRPageReq;
import com.yx.face.model.vo.third.ZoneSnReq;
import com.yx.face.service.ZoneDeviceRService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package com.yx.face.service.impl
 * @class ZoneDeviceRServiceImpl
 * @date 2023/7/5 上午11:14
 * @description
 */
@Slf4j
@Service
public class ZoneDeviceRServiceImpl implements ZoneDeviceRService {

    @Resource
    private ZoneDeviceRDao zoneDeviceRDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private AdminAuthDao adminAuthDao;
    @Resource
    private UserWhitelistDao userWhitelistDao;
    @Resource
    private RabbitTemplate rabbitTemplate;

    @Override
    public Integer create(ZoneDeviceRDto zoneDeviceRDto) {
        if (Preconditions.isNotBlank(zoneDeviceRDto.getId())) {
            return update(zoneDeviceRDto);
        } else {
            return add(zoneDeviceRDto);
        }
    }

    @Override
    public Integer getZoneIdBySn(Integer adminId, String sn) {
        List<ZoneDeviceR> dbAll = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class)
                .where(WeekendSqls.<ZoneDeviceR>custom().andEqualTo(ZoneDeviceR::getAdminId, adminId)
                        .andIsNotNull(ZoneDeviceR::getSnJson)
                        .andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(dbAll)) {
            return null;
        }
        for (ZoneDeviceR d : dbAll) {
            List<String> sns = Arrays.asList(d.getSnJson().split(","));
            if (sns.contains(sn)) {
                return d.getId();
            }
        }
        return null;
    }

    @Override
    public List<String> snList(ZoneSnReq dto) {
        WeekendSqls<ZoneDeviceR> sqls = WeekendSqls.<ZoneDeviceR>custom();
        //进行动态sql拼接
        sqls = sqls.andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE);
        if (Preconditions.isNotBlank(dto.getAdminId())) {
            sqls.andEqualTo(ZoneDeviceR::getAdminId, dto.getAdminId());
        }
        //所有被选中的设备
        List<String> allSelectedSn = new ArrayList<>();
        //具体的单个区域被选中的设备
        List<String> singleSelectedSn = new ArrayList<>();
        //未被选中的设备
        List<String> unselectedSn = new ArrayList<>();
        //查询
        List<ZoneDeviceR> list = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(sqls).build());
        if (Preconditions.isNotBlank(list)) {
            list.stream().forEach(item -> {//所有被选中的设备
                if (Preconditions.isNotBlank(item.getSnJson())) {
                    allSelectedSn.addAll(Arrays.asList(item.getSnJson().split(",")));
                }
            });

            if (Preconditions.isNotBlank(dto.getZoneId())) { //具体某个区域选中的设备
                ZoneDeviceR zoneDeviceR = list.stream().filter(item -> item.getId().intValue() == dto.getZoneId().intValue()).collect(Collectors.toList()).get(0);
                if (Preconditions.isNotBlank(zoneDeviceR.getSnJson())) {
                    singleSelectedSn.addAll(Arrays.asList(zoneDeviceR.getSnJson().split(",")));
                }
            }
        }

        //获取未选中设备
        WeekendSqls<FaceDevice> deviceSqls = WeekendSqls.<FaceDevice>custom();
        if (Preconditions.isNotBlank(dto.getAdminId())) {
            deviceSqls.andEqualTo(FaceDevice::getAdminId, dto.getAdminId());
        }
        if (Preconditions.isNotBlank(allSelectedSn)) {
            deviceSqls.andNotIn(FaceDevice::getSn, allSelectedSn);
        }
        List<FaceDevice> devices = faceDeviceDao.selectByExample(Example.builder(FaceDevice.class).where(deviceSqls).build());
        if (Preconditions.isNotBlank(devices)) {
            devices.stream().forEach(item -> unselectedSn.add(item.getSn()));
        }

        List<String> sns = new ArrayList<>();
        sns.addAll(unselectedSn);
        if (Preconditions.isNotBlank(dto.getZoneId()) && Preconditions.isNotBlank(singleSelectedSn)) {
            sns.addAll(singleSelectedSn);
        }
        return sns;
    }

    @Override
    public List<OpenDistrictDeleteResponse> delete(List<Integer> idList, Integer adminId) {

        List<OpenDistrictDeleteResponse> responses = new ArrayList<>();
        List<ZoneDeviceR> zones = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom().andIn(ZoneDeviceR::getId, idList).andEqualTo(ZoneDeviceR::getAdminId, adminId).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        List<Integer> dbZoneIds = null;
        if (Preconditions.isNotBlank(zones)) {
            dbZoneIds = zones.stream().map(ZoneDeviceR::getId).collect(Collectors.toList());
        }

        for (Integer item : idList) {
            OpenDistrictDeleteResponse response = new OpenDistrictDeleteResponse();
            response.setId(item);
            response.setMsg("删除成功");
            response.setResultsCode(1);
            if (Preconditions.isBlank(dbZoneIds) || !dbZoneIds.contains(item)) {
                response.setMsg("区域数据不存在或已删除");
                response.setResultsCode(0);
                responses.add(response);
                continue;
            }

            //校验区域绑定
            try {
                checkAdminAuthZoneBind(item, adminId);
            } catch (Exception e) {
                response.setMsg("区域数据不存在或已删除");
                response.setResultsCode(0);
                responses.add(response);
                continue;
            }
            try {
                checkUserWhitelistBind(item, adminId);
            } catch (Exception e) {
                response.setMsg("区域数据不存在或已删除");
                response.setResultsCode(0);
                responses.add(response);
                continue;
            }
            zoneDeviceRDao.deleteByPrimaryKey(item);
            responses.add(response);
        }
        return responses;
    }

    @Override
    public List<ZoneDeviceR> query(ZoneDeviceRDto dto) {
        WeekendSqls<ZoneDeviceR> sqls = WeekendSqls.<ZoneDeviceR>custom();
        //进行动态sql拼接
        sqls = sqls.andEqualTo(ZoneDeviceR::getAdminId, dto.getAdminId()).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE);
        if (Preconditions.isNotBlank(dto.getId())) {
            sqls.andEqualTo(ZoneDeviceR::getId, dto.getId());
        }

        if (Preconditions.isNotBlank(dto.getName())) {
            sqls.andLike(ZoneDeviceR::getName, "%" + dto.getName() + "%");
        }
        //查询
        List<ZoneDeviceR> list = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(sqls).orderByDesc("id").build());
        return list;
    }


    @Override
    public void delete(Integer id) {
        ZoneDeviceR zone = zoneDeviceRDao.selectByPrimaryKey(id);
        if (Preconditions.isBlank(zone) || zone.getDeleted()) {
            throw new CommonException("数据不存在或已删除");
        }
        //校验区域绑定
        checkAdminAuthZoneBind(id, zone.getAdminId());
        checkUserWhitelistBind(id, zone.getAdminId());

        zone.setDeleted(Boolean.TRUE);
        zone.setUpdateTime(new Date());
        zoneDeviceRDao.updateByPrimaryKey(zone);
        if(StrUtil.isBlank(zone.getSnJson())){
          return;
        }
        List<String> snsReq = Arrays.asList(zone.getSnJson().split(","));
        if(CollUtil.isEmpty(snsReq)){
            return;
        }
        for (String sn : snsReq) {
            rabbitTemplate.convertAndSend(SnQuenueConfig.QUEUE_NAME, sn);
        }
    }

    private void checkUserWhitelistBind(Integer id, Integer adminId) {
        List<UserWhitelist> userWhitelists = userWhitelistDao.selectByZoneIds(Arrays.asList(id), adminId);
        if (Preconditions.isBlank(userWhitelists)) {
            return;
        }
        throw new CommonException("区域有人员绑定，不可删除");
    }

    @Override
    public void deleteSn(Integer zoneId, String sn) {
        ZoneDeviceR zone = zoneDeviceRDao.selectByPrimaryKey(zoneId);
        if (Preconditions.isBlank(zone) || zone.getDeleted() || Preconditions.isBlank(zone.getSnJson())) {
            throw new CommonException("数据不存在或已删除");
        }

        List<String> sns = Arrays.asList(zone.getSnJson().split(","));
        if (!sns.contains(sn)) {
            throw new CommonException("sn数据不存在或已删除");
        }

        List<String> handleSns = new ArrayList<>();
        sns.stream().forEach(s -> {
            if (!s.equals(sn)) {
                handleSns.add(s);
            }
        });
        String snJson = null;
        if (Preconditions.isNotBlank(handleSns)) {
            snJson = handleSns.stream().collect(Collectors.joining(","));
        }
        zone.setSnJson(snJson);
        zoneDeviceRDao.updateByPrimaryKey(zone);
        rabbitTemplate.convertAndSend(SnQuenueConfig.QUEUE_NAME, sn);
    }

    @Override
    public PageInfo<ZoneDeviceRDto> page(RestDTO<ZoneDeviceRPageReq> dto) {
        reBuildQuery(dto);
        PageInfo<ZoneDeviceRDto> pageInfo = new PageInfo<ZoneDeviceRDto>();
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        WeekendSqls<ZoneDeviceR> sqls = WeekendSqls.<ZoneDeviceR>custom();
        //进行动态sql拼接
        sqls = sqls.andEqualTo(ZoneDeviceR::getAdminId, dto.getData().getAdminId()).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE);
        if (Preconditions.isNotBlank(dto.getData().getZoneId())) {
            sqls.andEqualTo(ZoneDeviceR::getId, dto.getData().getZoneId());
        }
        if (Preconditions.isNotBlank(dto.getData().getSn())) {
            sqls.andLike(ZoneDeviceR::getSnJson, "%" + dto.getData().getSn() + "%");
        }
        if (Preconditions.isNotBlank(dto.getData().getName())) {
            sqls.andLike(ZoneDeviceR::getName, "%" + dto.getData().getName() + "%");
        }
        //查询
        List<ZoneDeviceR> list = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(sqls).orderByDesc("id").build());
        if (Preconditions.isNotBlank(list)) {
            PageInfo<ZoneDeviceR> zoneDeviceRPageInfo = new PageInfo<>(list);
            BeanUtil.copyProperties(zoneDeviceRPageInfo, pageInfo);

            List<ZoneDeviceRDto> dtos = BeanUtil.copyToList(list, ZoneDeviceRDto.class);
            //补全设备参数
            completeDeviceInfo(dtos, dto.getData().getSn());
            pageInfo.setList(dtos);
        }
        return pageInfo;
    }

    private void completeDeviceInfo(List<ZoneDeviceRDto> dtos, String sn) {
        List<ZoneDeviceRDto> filterZones = dtos.stream().filter(item -> Preconditions.isNotBlank(item.getSnJson())).collect(Collectors.toList());
        if (Preconditions.isBlank(filterZones)) {
            return;
        }

        List<String> sns = new ArrayList<>();
        for (ZoneDeviceRDto item : filterZones) {
            sns.addAll(Arrays.asList(item.getSnJson().split(",")));
        }
        List<String> filterSns = sns.stream().filter(item -> Preconditions.isNotBlank(item) && Preconditions.isNotBlank(item.trim())).collect(Collectors.toList());
        if (Preconditions.isBlank(filterSns)) {
            return;
        }
        List<FaceDeviceVO> dbDevices = faceDeviceDao.getBySnList(filterSns);
        if (Preconditions.isBlank(dbDevices)) {
            return;
        }
        Map<String, FaceDeviceVO> snMap = dbDevices.stream().collect(Collectors.toMap(FaceDeviceVO::getSn, v -> v, (v1, v2) -> v1));
        dtos.stream().forEach(d -> {
            List<FaceDeviceVO> vos = new ArrayList<>();
            if (Preconditions.isNotBlank(d.getSnJson())) {
                List<String> fsns = Arrays.asList(d.getSnJson().split(","));
                for (String fsn : fsns) {
                    FaceDeviceVO vo = snMap.get(fsn);
                    if (Preconditions.isNotBlank(sn)) {
                        if (Preconditions.isNotBlank(vo) && sn.equals(vo.getSn())) {
                            vos.add(vo);
                        }
                    } else {
                        if (Preconditions.isNotBlank(vo)) {
                            vos.add(vo);
                        }
                    }
                }
            }
            d.setDevices(vos);
            vos = null;
        });
    }

    @Override
    public List<ZoneDeviceRDto> list(Integer adminId) {
        WeekendSqls<ZoneDeviceR> sqls = WeekendSqls.<ZoneDeviceR>custom();
        //进行动态sql拼接
        sqls = sqls.andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE);
        if (Preconditions.isNotBlank(adminId)) {
            sqls.andEqualTo(ZoneDeviceR::getAdminId, adminId);
        }
        //查询
        List<ZoneDeviceR> zoneDeviceRS = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(sqls).orderByDesc("id").build());
        if (Preconditions.isBlank(zoneDeviceRS)) {
            return null;
        }
        List<ZoneDeviceRDto> list = BeanUtil.copyToList(zoneDeviceRS, ZoneDeviceRDto.class);
        return list;
    }

    private Integer update(ZoneDeviceRDto zoneDeviceRDto) {

        ZoneDeviceR zone = zoneDeviceRDao.selectByPrimaryKey(zoneDeviceRDto.getId());
        if (Preconditions.isBlank(zone) || zone.getDeleted()) {
            throw new CommonException("数据不存在或已删除");
        }

        //校验区域名称
        checkNameInOtherZone(zoneDeviceRDto);
        //校验设备是否存在于其他区域
        checkSnInOtherZone(zoneDeviceRDto);
        String oldJson = zone.getSnJson();//原先的设备列表
        zone.setUpdateTime(new Date());
        zone.setName(zoneDeviceRDto.getName());
        zone.setSnJson(zoneDeviceRDto.getSnJson());
        zone.setSpiPushMsg(zoneDeviceRDto.getSpiPushMsg());
        zoneDeviceRDao.updateByPrimaryKey(zone);
        //设备缓存
        this.snCach(oldJson,zoneDeviceRDto.getSnJson());
        return zone.getId();
    }

    private void snCach(String oldSnJson, String newSnJson) {
        oldSnJson = StrUtil.isBlank(oldSnJson) ? "" : oldSnJson;
        newSnJson = StrUtil.isBlank(newSnJson) ? "" : newSnJson;
        List<String> oldSnsReq = Arrays.asList(oldSnJson.split(","));
        List<String> newSnsReq= Arrays.asList(newSnJson.split(","));
        List<String> oldDifference = oldSnsReq.stream()
                .filter(e -> !newSnsReq.contains(e))
                .collect(Collectors.toList());
        List<String> newDifference = newSnsReq.stream()
                .filter(e -> !oldSnsReq.contains(e))
                .collect(Collectors.toList());
        oldDifference.addAll(newDifference);
        for (String sn : oldDifference) {
            rabbitTemplate.convertAndSend(SnQuenueConfig.QUEUE_NAME, sn);
        }
    }

    public static void main(String[] args) {
        String oldSnJson = "119D002000800029,224F010T01400503,224F010T01400502,224F010T01400504,224F010T01400501,224F010T01400465,224F010T01400843,224F010T01400124,224F010T01400754,224F010T01400618,224F010T01400123,224F010T01400566,224F010T01400623,224F010T01400841,224F010T01400561,224F010T01400624,224F010T01400842,224F010T01400619,224F010T01400121,224F010T01400122,224F010T01400567,224F010T01400859";
        String newSnJson = "119D002000800029,224F010T01400503,224F010T01400502,224F010T01400504,224F010T01400501,224F010T01400465,224F010T01400843,224F010T01400124,224F010T01400754,224F010T01400618,224F010T01400123,224F010T01400566,224F010T01400623,224F010T01400841,224F010T01400561,224F010T01400624,224F010T01400842,224F010T01400619,224F010T01400121,224F010T01400122,224F010T01400567";
        List<String> oldSnsReq = Arrays.asList(oldSnJson.split(","));
        List<String> newSnsReq= Arrays.asList(newSnJson.split(","));
        List<String> oldDifference = oldSnsReq.stream()
                .filter(e -> !newSnsReq.contains(e))
                .collect(Collectors.toList());
        List<String> newDifference = newSnsReq.stream()
                .filter(e -> !oldSnsReq.contains(e))
                .collect(Collectors.toList());
        oldDifference.addAll(newDifference);
        System.out.println(oldDifference);
    }

    private Integer add(ZoneDeviceRDto zoneDeviceRDto) {
        //校验区域名称
        checkNameInOtherZone(zoneDeviceRDto);
        //校验设备是否存在于其他区域
        checkSnInOtherZone(zoneDeviceRDto);
        ZoneDeviceR zoneDeviceR = BeanUtil.copyProperties(zoneDeviceRDto, ZoneDeviceR.class);
        zoneDeviceR.setDeleted(Boolean.FALSE);
        zoneDeviceR.setCreateTime(new Date());
        zoneDeviceR.setUpdateTime(new Date());
        zoneDeviceRDao.insert(zoneDeviceR);
        /*设备缓存*/
        if(StrUtil.isNotBlank(zoneDeviceRDto.getSnJson())){
            List<String> snsReq = Arrays.asList(zoneDeviceRDto.getSnJson().split(","));
            if(CollUtil.isNotEmpty(snsReq)){
                for (String sn : snsReq) {
                    rabbitTemplate.convertAndSend(SnQuenueConfig.QUEUE_NAME, sn);
                }
            }
        }
        return zoneDeviceR.getId();
    }

    private void checkNameInOtherZone(ZoneDeviceRDto zoneDeviceRDto) {

        if (Preconditions.isBlank(zoneDeviceRDto.getName())) {
            return;
        }

        ZoneDeviceR dbName = zoneDeviceRDao.selectOneByExample(Example.builder(ZoneDeviceR.class)
                .where(WeekendSqls.<ZoneDeviceR>custom().andEqualTo(ZoneDeviceR::getAdminId, zoneDeviceRDto.getAdminId())
                        .andEqualTo(ZoneDeviceR::getName, zoneDeviceRDto.getName())
                        .andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(dbName)) {
            return;
        }

        StringBuffer errorMsg = new StringBuffer();
        if (Preconditions.isNotBlank(zoneDeviceRDto.getId())) {
            if (dbName.getName().trim().equals(zoneDeviceRDto.getName().trim())
                    && dbName.getId().intValue() != zoneDeviceRDto.getId().intValue()) {
                errorMsg.append("区域名称【" + zoneDeviceRDto.getName() + "】已存在；");
            }
        } else {
            if (dbName.getName().trim().equals(zoneDeviceRDto.getName().trim())) {
                errorMsg.append("区域名称【" + zoneDeviceRDto.getName() + "】已存在；");
            }
        }

        String msg = errorMsg.toString();
        if (Preconditions.isNotBlank(msg)) {
            throw new CommonException(msg);
        }
    }

    //校验设备是否存在于其他区域
    private void checkSnInOtherZone(ZoneDeviceRDto zoneDeviceRDto) {
        if (Preconditions.isBlank(zoneDeviceRDto.getSnJson())) {
            return;
        }

        List<ZoneDeviceR> dbSns = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class)
                .where(WeekendSqls.<ZoneDeviceR>custom().andEqualTo(ZoneDeviceR::getAdminId, zoneDeviceRDto.getAdminId())
                        .andIsNotNull(ZoneDeviceR::getSnJson)
                        .andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(dbSns)) {
            return;
        }

        StringBuffer errorMsg = new StringBuffer();
        List<String> snsReq = Arrays.asList(zoneDeviceRDto.getSnJson().split(","));
        for (String req : snsReq) {
            if (Preconditions.isNotBlank(zoneDeviceRDto.getId())) {
                for (ZoneDeviceR item : dbSns) {
                    List<String> dbSn = Arrays.asList(item.getSnJson().split(","));
                    if (dbSn.contains(req) && item.getId().intValue() != zoneDeviceRDto.getId()) {
                        errorMsg.append("设备【" + req + "】在【" + item.getName() + "】已存在；");
                    }
                }
            } else {
                for (ZoneDeviceR item : dbSns) {
                    List<String> dbSn = Arrays.asList(item.getSnJson().split(","));
                    if (dbSn.contains(req)) {
                        errorMsg.append("设备【" + req + "】在【" + item.getName() + "】已存在；");
                    }
                }
            }
        }

        String msg = errorMsg.toString();
        if (Preconditions.isNotBlank(msg)) {
            throw new CommonException(msg);
        }

    }

    /**
     * 校验区域绑定
     *
     * @param zoneId
     * @param adminId
     */
    private void checkAdminAuthZoneBind(Integer zoneId, Integer adminId) {
        List<AdminAuth> adminAuths = adminAuthDao.selectByExample(Example.builder(AdminAuth.class)
                .where(WeekendSqls.<AdminAuth>custom().andEqualTo(AdminAuth::getAdminId, adminId)
                        .andEqualTo(AdminAuth::getType, AdminAuthTypeEnum.M_PLATFORM.getCode())
                        .andIsNotNull(AdminAuth::getInterfAddrJson)
                        .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(adminAuths)) {
            return;
        }

        List<Integer> zoneIds = new ArrayList<>();
        adminAuths.stream().forEach(a -> {
            List<InterfAddrJsonVo> vos = JSONObject.parseArray(a.getInterfAddrJson(), InterfAddrJsonVo.class);
            zoneIds.addAll(vos.stream().filter(z -> Preconditions.isNotBlank(z.getZoneId()) && z.getZoneId().intValue() != 0).map(InterfAddrJsonVo::getZoneId).collect(Collectors.toList()));
        });

        if (Preconditions.isBlank(zoneIds)) {
            return;
        }

        if (zoneIds.contains(zoneId)) {
            throw new CommonException("区域有三方API绑定，不可删除");
        }
    }

    private void reBuildQuery(RestDTO<ZoneDeviceRPageReq> dto) {
        ZoneDeviceRPageReq object = dto.getData();
        if (object == null) {
            object = new ZoneDeviceRPageReq();
        }
        dto.setData(object);
    }

}
