package com.yx.face.service.issue;

import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.dao.UserWhitelistDao;
import com.yx.face.dao.ZoneDeviceRDao;
import com.yx.face.model.entity.UserWhitelist;
import com.yx.face.model.entity.ZoneDeviceR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

/**
 * @author shisl
 * @package com.yx.face.service.issue
 * @class ZoneUserIssueServiceImpl
 * @date 2024/4/12 上午11:40
 * @description
 */
@Slf4j
@Service
public class ZoneUserIssueServiceImpl implements ZoneUserIssueService {

    @Resource
    private ZoneDeviceRDao zoneDeviceRDao;

    @Resource
    private UserWhitelistDao userWhitelistDao;

    @Override
    public Boolean checkIsZoneSn(String sn, Integer adminId) {
        List<ZoneDeviceR> zones = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom().andIsNotNull(ZoneDeviceR::getSnJson).andEqualTo(ZoneDeviceR::getAdminId, adminId).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(zones)) {
            return Boolean.FALSE;
        }

        Integer zoneId = null;
        for (ZoneDeviceR item : zones) {
            List<String> sns = Arrays.asList(item.getSnJson().split(","));
            if (!sns.contains(sn)) {
                continue;
            }
            zoneId = item.getId();
        }
        if (Preconditions.isBlank(zoneId)) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @Override
    public Boolean checkIsZoneSn(String sn, List<Integer> adminIds) {
        List<ZoneDeviceR> zones = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom().andIsNotNull(ZoneDeviceR::getSnJson).andIn(ZoneDeviceR::getAdminId, adminIds).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(zones)) {
            return Boolean.FALSE;
        }

        Integer zoneId = null;
        for (ZoneDeviceR item : zones) {
            List<String> sns = Arrays.asList(item.getSnJson().split(","));
            if (!sns.contains(sn)) {
                continue;
            }
            zoneId = item.getId();
        }
        if (Preconditions.isBlank(zoneId)) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }


    @Override
    public List<UserWhitelist> getUserWhitelistBySn(String sn,  List<Integer> adminIds) {
        if (Preconditions.isBlank(sn)) {
            return null;
        }
        List<ZoneDeviceR> zones = zoneDeviceRDao.selectByExample(Example.builder(ZoneDeviceR.class).where(WeekendSqls.<ZoneDeviceR>custom().andIsNotNull(ZoneDeviceR::getSnJson).andIn(ZoneDeviceR::getAdminId, adminIds).andEqualTo(ZoneDeviceR::getDeleted, Boolean.FALSE)).build());
        if (Preconditions.isBlank(zones)) {
            return null;
        }

        Integer zoneId = null;
        for (ZoneDeviceR item : zones) {
            List<String> sns = Arrays.asList(item.getSnJson().split(","));
            if (!sns.contains(sn)) {
                continue;
            }
            zoneId = item.getId();
        }

        if (Preconditions.isBlank(zoneId)) {
            return null;
        }

        List<UserWhitelist> users = userWhitelistDao.selectByZoneId(zoneId, adminIds);
        return users;
    }
}
