package com.yx.face.service.issue.impl;

import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.RabbitMqConstants;
import com.yx.face.model.dto.hhface.AddSnIssuedMsg;
import com.yx.face.model.dto.hhface.DelFaceBySnMsg;
import com.yx.face.model.dto.hhface.DeleteBySnMsg;
import com.yx.face.model.dto.hhface.DownUserMsg;
import com.yx.face.model.dto.issue.WebsocketMsgReq;
import com.yx.face.model.entity.DownFaceNew;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.TaskInfo;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.service.issue.HhfaceIssueService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author shisl
 * @package com.yx.face.service.issue.impl
 * @class HhfaceIssueServiceImpl
 * @date 2024/9/10 下午2:03
 * @description
 */
@Slf4j
@Service
public class HhfaceIssueServiceImpl implements HhfaceIssueService {

    @Autowired
    RabbitTemplate rabbitTemplate;

    /**
     * 发送mq消息
     *
     * @param message
     */
    public void sendGeneralMqMessage(String message) {
        //消息压入队列
        rabbitTemplate.convertAndSend(RabbitMqConstants.GENERAL_USER_QUEUE_NAME, message);
    }

    /**
     * 回调mq消息
     *
     * @param message
     */
    public void sendGeneralCallbackMqMessage(String message) {
        //消息压入队列
        rabbitTemplate.convertAndSend(RabbitMqConstants.GENERAL_USER_CALLBACK_QUEUE_NAME, message);
    }

    @Override
    public void sendGeneralCallbackMessage2Mq(WebsocketMsgReq req) {
        sendGeneralCallbackMqMessage(JSONObject.toJSONString(req));
        log.info("---->sendGeneralCallbackMessage2Mq send mq success");
    }

    @Override
    public void sendDeleteBySnMsg(String sn, TaskInfo taskInfo) {
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.GENERAL_MESSAGE_TYPE_delUserBySn);
        DeleteBySnMsg msg = new DeleteBySnMsg();
        msg.setSn(sn);
        msg.setTaskInfo(taskInfo);
        object.put("data", JSONObject.toJSONString(msg));
        sendGeneralMqMessage(JSONObject.toJSONString(object));
        log.info("---->sendDeleteBySnMsg send mq success");
    }

    @Override
    public void sendDownUserMsg(DownFaceNew downFace, TaskInfo taskInfo) {
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.GENERAL_MESSAGE_TYPE_addSingleUser2Device);
        DownUserMsg msg = new DownUserMsg();
        msg.setDownFace(downFace);
        msg.setTaskInfo(taskInfo);
        object.put("data", JSONObject.toJSONString(msg));
        sendGeneralMqMessage(JSONObject.toJSONString(object));
        log.info("---->sendDownUserMsg send mq success");
    }

    @Override
    public void sendDelFaceBySnMsg(String sn, List<Long> userIds, TaskInfo taskInfo) {
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.GENERA_MESSAGE_TYPE_delUserFromDevice);
        DelFaceBySnMsg msg = new DelFaceBySnMsg();
        msg.setSn(sn);
        msg.setUserIds(userIds);
        msg.setTaskInfo(taskInfo);
        object.put("data", JSONObject.toJSONString(msg));
        sendGeneralMqMessage(JSONObject.toJSONString(object));
        log.info("---->sendDelFaceBySnMsg send mq success");
    }

    @Override
    public void sendAddSnIssuedMsg(FaceDevice faceDevice, AdminVO adminVO, TaskInfo taskInfo, String issueType) {
        JSONObject object = new JSONObject();
        object.put("bizType", RabbitMqConstants.GENERAL_MESSAGE_TYPE_batchAddUser2Device);
        AddSnIssuedMsg msg = new AddSnIssuedMsg();
        msg.setFaceDevice(faceDevice);
        msg.setAdminVO(adminVO);
        msg.setTaskInfo(taskInfo);
        msg.setIssueType(issueType);
        object.put("data", JSONObject.toJSONString(msg));
        sendGeneralMqMessage(JSONObject.toJSONString(object));
        log.info("---->sendAddSnIssuedMsg send mq success");

    }
}
