package com.yx.face.service.issue.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.uitls.AppUtils;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.dao.AdminDao;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.FaceDeviceTypeDao;
import com.yx.face.dao.FkjHeartTrackingDao;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.dao.issue.IssueDeviceDao;
import com.yx.face.dao.issue.IssueDeviceItemDao;
import com.yx.face.dao.issue.IssueDeviceUserDao;
import com.yx.face.model.dto.issue.DelDeviceFaceAllCallbackReq;
import com.yx.face.model.dto.issue.DelDeviceFaceAllReq;
import com.yx.face.model.dto.issue.DelDeviceFaceCallbackReq;
import com.yx.face.model.dto.issue.DelDeviceFaceReq;
import com.yx.face.model.dto.issue.IssueCancelReq;
import com.yx.face.model.dto.issue.IssueDeviceCallbackReq;
import com.yx.face.model.dto.issue.IssueDeviceDetailVo;
import com.yx.face.model.dto.issue.IssueDeviceFetchReq;
import com.yx.face.model.dto.issue.IssueDeviceItemCallbackReq;
import com.yx.face.model.dto.issue.IssueDeviceItemReq;
import com.yx.face.model.dto.issue.IssueDeviceItemSearch;
import com.yx.face.model.dto.issue.IssueDeviceItemVo;
import com.yx.face.model.dto.issue.IssueDeviceReq;
import com.yx.face.model.dto.issue.IssueDeviceSearch;
import com.yx.face.model.dto.issue.IssueDeviceUserReportReq;
import com.yx.face.model.dto.issue.IssueDeviceVo;
import com.yx.face.model.dto.issue.WebsocketMsgReq;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.entity.DownFaceNew;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.FaceDeviceType;
import com.yx.face.model.entity.FkjHeartTracking;
import com.yx.face.model.entity.issue.IssueDevice;
import com.yx.face.model.entity.issue.IssueDeviceItem;
import com.yx.face.model.entity.issue.IssueDeviceUser;
import com.yx.face.model.enums.issue.IssueStatusEnum;
import com.yx.face.model.enums.issue.IssueTypeEnum;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.UserInfoVO;
import com.yx.face.service.issue.IssueDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.java_websocket.client.WebSocketClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package com.yx.face.service.issue.impl
 * @class IssueDeviceServiceImpl
 * @date 2024/4/15 下午5:06
 * @description 设备人员下发（websocket）
 */
@Slf4j
@Service
public class IssueDeviceServiceImpl implements IssueDeviceService {

    @Resource
    private IssueDeviceDao issueDeviceDao;

    @Resource
    private IssueDeviceItemDao issueDeviceItemDao;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private IssueDeviceUserDao issueDeviceUserDao;

    @Resource
    private FaceDeviceTypeDao faceDeviceTypeDao;

    @Resource
    private AdminDao adminDao;

    @Resource
    private UserInfoDao userInfoDao;

    @Resource
    private WebSocketClient otaWebSocketClient;

    @Resource
    private FkjHeartTrackingDao fkjHeartTrackingDao;


    @Value("${websocket.httpApiSwitch}")
    private Boolean httpApiSwitch;

    @Value("${websocket.httpApi}")
    private String httpApiUrl;

    @Resource
    private Executor taskExecutor;


    //查询下发设备分页列表
    @Override
    public PageInfo<IssueDeviceVo> page(RestDTO<IssueDeviceSearch> dto) {
        IssueDeviceSearch data = dto.getData();
        PageInfo<IssueDeviceVo> pageInfo = new PageInfo<IssueDeviceVo>();
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        WeekendSqls<IssueDevice> sqls = WeekendSqls.<IssueDevice>custom();
        //进行动态sql拼接
        sqls = sqls.andEqualTo(IssueDevice::getIsDeleted, Boolean.FALSE);
        if (Preconditions.isNotBlank(data.getAdminId())) {
            sqls.andEqualTo(IssueDevice::getAdminId, data.getAdminId());
        }
        if (Preconditions.isNotBlank(data.getDeviceSn())) {
            sqls.andLike(IssueDevice::getDeviceSn, "%" + data.getDeviceSn() + "%");
        }
        if (Preconditions.isNotBlank(data.getIssueStatus())) {
            sqls.andEqualTo(IssueDevice::getIssueStatus, data.getIssueStatus());
        }
        if (Preconditions.isNotBlank(data.getIssueType())) {
            sqls.andEqualTo(IssueDevice::getIssueType, data.getIssueType());
        }
        List<IssueDevice> list = issueDeviceDao.selectByExample(Example.builder(IssueDevice.class).where(sqls).orderByDesc("id").build());
        if (Preconditions.isNotBlank(list)) {
            PageInfo<IssueDevice> issuePageInfo = new PageInfo<>(list);
            BeanUtil.copyProperties(issuePageInfo, pageInfo);
            List<IssueDeviceVo> vos = BeanUtil.copyToList(list, IssueDeviceVo.class);
            //补全设备参数
            completeIssueDeviceVo(vos);
            pageInfo.setList(vos);
        }
        return pageInfo;
    }

    private void completeIssueDeviceVo(List<IssueDeviceVo> vos) {
        if (Preconditions.isBlank(vos)) {
            return;
        }
        List<String> deviceSns = vos.stream().map(IssueDeviceVo::getDeviceSn).collect(Collectors.toList());
        List<FaceDevice> devices = faceDeviceDao.selectByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andIn(FaceDevice::getSn, deviceSns)).build());
        Map<String, FaceDevice> deviceMap = devices.stream().collect(Collectors.toMap(FaceDevice::getSn, v -> v, (v1, v2) -> v1));
        List<Integer> deviceTypeIds = devices.stream().map(FaceDevice::getFaceDeviceTypeId).collect(Collectors.toList());
        Map<Integer, FaceDeviceType> deviceTypeMap = null;
        if (Preconditions.isNotBlank(deviceTypeIds)) {
            List<FaceDeviceType> faceDeviceTypes = faceDeviceTypeDao.selectByExample(Example.builder(FaceDeviceType.class).where(WeekendSqls.<FaceDeviceType>custom().andIn(FaceDeviceType::getId, deviceTypeIds)).build());
            deviceTypeMap = faceDeviceTypes.stream().collect(Collectors.toMap(FaceDeviceType::getId, v -> v, (v1, v2) -> v1));
        }
        for (IssueDeviceVo item : vos) {
            FaceDevice faceDevice = deviceMap.get(item.getDeviceSn());
            if (Preconditions.isBlank(faceDevice)) {
                continue;
            }
            item.setName(faceDevice.getName());
            item.setOnline(faceDevice.getOnline());
            item.setFaceDeviceTypeId(faceDevice.getFaceDeviceTypeId());
            if (Preconditions.isNotBlank(deviceTypeMap)) {
                FaceDeviceType faceDeviceType = deviceTypeMap.get(faceDevice.getFaceDeviceTypeId());
                if (Preconditions.isNotBlank(faceDeviceType)) {
                    item.setFaceDeviceTypeName(faceDeviceType.getName());
                }
            }
        }
    }

    @Override
    public IssueDeviceDetailVo detail(Long id) {
        IssueDevice issueDevice = issueDeviceDao.selectOneByExample(Example.builder(IssueDevice.class).where(WeekendSqls.<IssueDevice>custom().andEqualTo(IssueDevice::getId, id)).build());
        IssueDeviceDetailVo detailVo = BeanUtil.copyProperties(issueDevice, IssueDeviceDetailVo.class);
        FaceDevice device = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, issueDevice.getDeviceSn())).build());
        if (Preconditions.isNotBlank(device)) {
            detailVo.setFaceDeviceTypeId(device.getFaceDeviceTypeId());
            detailVo.setName(device.getName());
            detailVo.setOnline(device.getOnline());
            FaceDeviceType deviceType = faceDeviceTypeDao.selectOneByExample(Example.builder(FaceDeviceType.class).where(WeekendSqls.<FaceDeviceType>custom().andEqualTo(FaceDeviceType::getId, device.getFaceDeviceTypeId())).build());
            if (Preconditions.isNotBlank(deviceType)) {
                detailVo.setFaceDeviceTypeName(deviceType.getName());
            }
        }
        List<IssueDeviceItem> issueDeviceItems = issueDeviceItemDao.selectByExample(Example.builder(IssueDeviceItem.class).where(WeekendSqls.<IssueDeviceItem>custom().andEqualTo(IssueDeviceItem::getIssueDeviceId, issueDevice.getId())).select("id", "issueStatus").build());
        long hadSuccessCount = issueDeviceItems.stream().filter(item -> "success".equals(item.getIssueStatus())).count();
        long hadFailedCount = issueDeviceItems.stream().filter(item -> "failed".equals(item.getIssueStatus())).count();
        detailVo.setHadIssueFaceCount(Integer.valueOf(String.valueOf(hadSuccessCount)));
        detailVo.setUnIssueFaceCount(Integer.valueOf(String.valueOf(hadFailedCount)));
        return detailVo;
    }

    @Override
    public PageInfo<IssueDeviceItemVo> pageItem(RestDTO<IssueDeviceItemSearch> dto) {
        IssueDeviceItemSearch data = dto.getData();
        PageInfo<IssueDeviceItemVo> pageInfo = new PageInfo<IssueDeviceItemVo>();
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        WeekendSqls<IssueDeviceItem> sqls = WeekendSqls.<IssueDeviceItem>custom();
        //进行动态sql拼接
        sqls = sqls.andEqualTo(IssueDeviceItem::getIsDeleted, Boolean.FALSE);
        if (Preconditions.isNotBlank(data.getIssueDeviceId())) {
            sqls.andEqualTo(IssueDeviceItem::getIssueDeviceId, data.getIssueDeviceId());
        }
        if (Preconditions.isNotBlank(data.getCardIdEx())) {
            sqls.andEqualTo(IssueDeviceItem::getCardIdEx, data.getCardIdEx());
        }
        if (Preconditions.isNotBlank(data.getStartTime()) && Preconditions.isNotBlank(data.getEndTime())) {
            sqls.andGreaterThanOrEqualTo(IssueDeviceItem::getIssueTime, data.getStartTime())
                    .andLessThanOrEqualTo(IssueDeviceItem::getIssueTime, data.getEndTime());
        }
        if (Preconditions.isNotBlank(data.getIdNumber())) {
            sqls.andEqualTo(IssueDeviceItem::getIdNumber, data.getIdNumber());
        }
        if (Preconditions.isNotBlank(data.getIssueStatus())) {
            sqls.andEqualTo(IssueDeviceItem::getIssueStatus, data.getIssueStatus());
        }
        if (Preconditions.isNotBlank(data.getPhone())) {
            sqls.andEqualTo(IssueDeviceItem::getPhone, data.getPhone());
        }
        if (Preconditions.isNotBlank(data.getUserName())) {
            sqls.andLike(IssueDeviceItem::getUserName, "%" + data.getUserName() + "%");
        }
        if (Preconditions.isNotBlank(data.getUserType())) {
            sqls.andEqualTo(IssueDeviceItem::getUserType, data.getUserType());
        }

        List<IssueDeviceItem> list = issueDeviceItemDao.selectByExample(Example.builder(IssueDeviceItem.class).where(sqls).orderByDesc("id").build());
        if (Preconditions.isNotBlank(list)) {
            PageInfo<IssueDeviceItem> issuePageInfo = new PageInfo<>(list);
            BeanUtil.copyProperties(issuePageInfo, pageInfo);
            List<IssueDeviceItemVo> vos = BeanUtil.copyToList(list, IssueDeviceItemVo.class);
            pageInfo.setList(vos);
        }
        return pageInfo;
    }

    @Override
    public List<IssueDeviceItemVo> issueDeviceItems(IssueDeviceItemSearch data) {

        WeekendSqls<IssueDeviceItem> sqls = WeekendSqls.<IssueDeviceItem>custom();
        //进行动态sql拼接
        sqls = sqls.andEqualTo(IssueDeviceItem::getIsDeleted, Boolean.FALSE);
        if (Preconditions.isNotBlank(data.getIssueDeviceId())) {
            sqls.andEqualTo(IssueDeviceItem::getIssueDeviceId, data.getIssueDeviceId());
        }
        if (Preconditions.isNotBlank(data.getCardIdEx())) {
            sqls.andEqualTo(IssueDeviceItem::getCardIdEx, data.getCardIdEx());
        }
        if (Preconditions.isNotBlank(data.getStartTime()) && Preconditions.isNotBlank(data.getEndTime())) {
            sqls.andGreaterThanOrEqualTo(IssueDeviceItem::getIssueTime, data.getStartTime())
                    .andLessThanOrEqualTo(IssueDeviceItem::getIssueTime, data.getEndTime());
        }
        if (Preconditions.isNotBlank(data.getIdNumber())) {
            sqls.andEqualTo(IssueDeviceItem::getIdNumber, data.getIdNumber());
        }
        if (Preconditions.isNotBlank(data.getIssueStatus())) {
            sqls.andEqualTo(IssueDeviceItem::getIssueStatus, data.getIssueStatus());
        }
        if (Preconditions.isNotBlank(data.getPhone())) {
            sqls.andEqualTo(IssueDeviceItem::getPhone, data.getPhone());
        }
        if (Preconditions.isNotBlank(data.getUserName())) {
            sqls.andEqualTo(IssueDeviceItem::getUserName, data.getUserName());
        }
        List<IssueDeviceItem> list = issueDeviceItemDao.selectByExample(Example.builder(IssueDeviceItem.class).where(sqls).build());
        List<IssueDeviceItemVo> vos = BeanUtil.copyToList(list, IssueDeviceItemVo.class);
        return vos;
    }

    @Override
    public RestResult<Boolean> cancel(Long issueDeviceId) {
        IssueDevice dbIssueDevice = issueDeviceDao.selectOneByExample(Example.builder(IssueDevice.class).where(WeekendSqls.<IssueDevice>custom()
                .andEqualTo(IssueDevice::getId, issueDeviceId)).build());
        if (Preconditions.isBlank(dbIssueDevice)) {
            String errorMsg = "任务不存在，无法取消任务";
            throw new CommonException(400, errorMsg);
        }

        if (!Arrays.asList(IssueStatusEnum.PENDING.getCode(), IssueStatusEnum.PROCESSING.getCode(),
                IssueStatusEnum.INTERRUPTION.getCode()).contains(dbIssueDevice.getIssueStatus())) {
            String errorMsg = String.format("设备【%s】状态为【%s】的任务，无法取消任务", dbIssueDevice.getDeviceSn(), IssueStatusEnum.getByCode(dbIssueDevice.getIssueStatus()).getDesc());
            throw new CommonException(400, errorMsg);
        }

        dbIssueDevice.setIssueStatus(IssueStatusEnum.CANCEL.getCode());
        dbIssueDevice.setRemarks(String.format("%s取消", DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss")));
        issueDeviceDao.updateByPrimaryKey(dbIssueDevice);
        handleIssueCancel(dbIssueDevice);
        return RestResponse.ok(Boolean.TRUE);
    }

    private void handleIssueCancel(IssueDevice dbIssueDevice) {
        try {
            if (Preconditions.isBlank(dbIssueDevice)
                    || !Arrays.asList(IssueStatusEnum.PROCESSING.getCode(), IssueStatusEnum.PENDING.getCode(), IssueStatusEnum.INTERRUPTION.getCode()).contains(dbIssueDevice.getIssueStatus())) {
                log.error("{}非等待或下发中状态，禁止取消", dbIssueDevice.getTraceId());
                return;
            }
            WebsocketMsgReq req = new WebsocketMsgReq();
            req.setSn(dbIssueDevice.getDeviceSn());
            req.setSource(Constants.SOURCE_TQSERVER);
            req.setOp(Constants.OP_ISSUE_CANCEL);
            req.setMsg(IssueTypeEnum.ISSUE_CANCEL.getDesc());
            IssueCancelReq faceReq = new IssueCancelReq();
            faceReq.setDeviceSn(dbIssueDevice.getDeviceSn());
            faceReq.setTraceId(dbIssueDevice.getTraceId());
            req.setData(faceReq);
            log.info("--->发送websocket消息最终参数:{}", JSONObject.toJSONString(req));
            hhFaceMiddlewareIssueApi(req);
        } catch (Exception e) {
            log.error("websocket连接中断");
        }
    }

    @Override
    public RestResult<Boolean> retryIssue(Long issueDeviceId) {
        IssueDevice dbIssueDevice = issueDeviceDao.selectOneByExample(Example.builder(IssueDevice.class).where(WeekendSqls.<IssueDevice>custom()
                .andEqualTo(IssueDevice::getId, issueDeviceId)).build());
        if (Preconditions.isBlank(dbIssueDevice)) {
            String errorMsg = "任务不存在，无法执行下发任务";
            throw new CommonException(400, errorMsg);
        }

        if (!Arrays.asList(IssueStatusEnum.INTERRUPTION.getCode(), IssueStatusEnum.FAILED.getCode()).contains(dbIssueDevice.getIssueStatus())) {
            String errorMsg = String.format("设备【%s】状态为【%s】的任务，无法执行下发任务", dbIssueDevice.getDeviceSn(), IssueStatusEnum.getByCode(dbIssueDevice.getIssueStatus()).getDesc());
            throw new CommonException(400, errorMsg);
        }

        dbIssueDevice.setIssueStatus(IssueStatusEnum.PENDING.getCode());
        issueDeviceDao.updateByPrimaryKey(dbIssueDevice);

        List<IssueDeviceItem> items = issueDeviceItemDao.selectByExample(Example.builder(IssueDeviceItem.class).where(WeekendSqls.<IssueDeviceItem>custom().andEqualTo(IssueDeviceItem::getIssueDeviceId, issueDeviceId)).build());
        handleIssue(dbIssueDevice, items, IssueTypeEnum.getByCode(dbIssueDevice.getIssueType()).getOpCode(), IssueTypeEnum.getByCode(dbIssueDevice.getIssueType()).getDesc());
        return RestResponse.ok(Boolean.TRUE);
    }

    /**
     * --------------websocket相关-----------------
     **/
    @Override
    public Long handleIssueDevice(DownFaceNew downFace, Date startTime, Date endTime, Integer adminId, boolean issueStatus, String errorMsg) {
        Admin admin = adminDao.selectByPrimaryKey(adminId);
        IssueDevice issueDevice = new IssueDevice();
        issueDevice.setAdminId(adminId);
        issueDevice.setAdminUserName(admin.getUsername());
        issueDevice.setDeviceSn(downFace.getSn());
        issueDevice.setHadIssueFaceCount(1);
        issueDevice.setIssueFaceCount(1);
        issueDevice.setIssueType(IssueTypeEnum.ISSUE_INCREMENT.getCode());
        issueDevice.setTraceId(AppUtils.getAppSecret(AppUtils.getAppKey()));
        issueDevice.setIssueStatus(issueStatus ? IssueStatusEnum.SUCCESS.getCode() : IssueStatusEnum.FAILED.getCode());
        issueDevice.setRemarks(errorMsg);
        issueDevice.setCreateTime(new Date());
        issueDevice.setUpdateTime(new Date());
        issueDevice.setIsDeleted(Boolean.FALSE);
        issueDevice.setVersion(1);
        issueDeviceDao.insert(issueDevice);

        IssueDeviceItem item = new IssueDeviceItem();
        item.setCardIdEx(downFace.getCardIdEx());
        item.setIdNumber(downFace.getCardid());
        item.setIssueDeviceId(issueDevice.getId());
        item.setIssueStatus(issueStatus ? IssueStatusEnum.SUCCESS.getCode() : IssueStatusEnum.FAILED.getCode());
        item.setRemarks(errorMsg);
        item.setIssueTime(new Date());
        item.setOrgId(DigestUtil.md5Hex(admin.getId() + downFace.getUserId() + issueDevice.getDeviceSn()));
        item.setPhone(downFace.getPhone());
        item.setStartTime(downFace.getValidPeriodStart());
        item.setEndTime(downFace.getValidPeriodEnd());
        item.setUserId(Integer.valueOf(String.valueOf(downFace.getUserId())));
        item.setUserName(downFace.getName());
        item.setUserType(downFace.getRightno());
        item.setPhoto(downFace.getPhoto());
        item.setFacialFeature(downFace.getFacialFeature());
        item.setCreateTime(new Date());
        item.setUpdateTime(new Date());
        item.setIsDeleted(Boolean.FALSE);
        item.setVersion(1);
        issueDeviceItemDao.insert(item);
        return issueDevice.getId();
    }

    @Override
    public Long handleIssueDeviceAll(FaceDevice faceDevice, List<DownFaceNew> list, AdminVO adminVO, String issueType) {
        Admin admin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
        IssueDevice issueDevice = new IssueDevice();
        issueDevice.setAdminId(admin.getId());
        issueDevice.setAdminUserName(admin.getUsername());
        issueDevice.setDeviceSn(faceDevice.getSn());
        issueDevice.setHadIssueFaceCount(list.size());
        issueDevice.setIssueFaceCount(list.size());
        issueDevice.setIssueType(Preconditions.isNotBlank(issueType) ? issueType : IssueTypeEnum.ISSUE_ALL.getCode());
        issueDevice.setTraceId(AppUtils.getAppSecret(AppUtils.getAppKey()));
        issueDevice.setIssueStatus(IssueStatusEnum.SUCCESS.getCode());
        issueDevice.setCreateTime(new Date());
        issueDevice.setUpdateTime(new Date());
        issueDevice.setIsDeleted(Boolean.FALSE);
        issueDevice.setVersion(1);
        issueDeviceDao.insert(issueDevice);
        List<IssueDeviceItem> items = new ArrayList<>();
        for (DownFaceNew whitelist : list) {
            IssueDeviceItem item = new IssueDeviceItem();
            item.setCardIdEx(whitelist.getCardIdEx());
            item.setIdNumber(whitelist.getCardid());
            item.setIssueDeviceId(issueDevice.getId());
            item.setIssueStatus(IssueStatusEnum.SUCCESS.getCode());
            item.setIssueTime(new Date());
            item.setOrgId(DigestUtil.md5Hex(admin.getId() + whitelist.getUserId() + issueDevice.getDeviceSn()));
            item.setPhone(whitelist.getPhone());
            item.setStartTime(whitelist.getValidPeriodStart());
            item.setEndTime(whitelist.getValidPeriodEnd());
            item.setUserId(Integer.valueOf(String.valueOf(whitelist.getUserId())));
            item.setUserName(whitelist.getName());
            item.setUserType(whitelist.getRightno());
            item.setPhoto(whitelist.getPhoto());
            item.setCreateTime(new Date());
            item.setUpdateTime(new Date());
            item.setIsDeleted(Boolean.FALSE);
            item.setVersion(1);
            items.add(item);
        }
        issueDeviceItemDao.insertList(items);

        return issueDevice.getId();
    }

    @Override
    public void issueUserAllIncrement(FaceDevice faceDevice, List<DownFaceNew> list, AdminVO adminVO) {
        if (Preconditions.isBlank(list)) {
            return;
        }
//        IssueDeviceUser issueDeviceUser = issueDeviceUserDao.selectOneByExample(Example.builder(IssueDeviceUser.class).where(WeekendSqls.<IssueDeviceUser>custom().andEqualTo(IssueDeviceUser::getDeviceSn, faceDevice.getSn())).build());
        Admin admin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
        //issue device 创建增量任务
        IssueDevice issueDevice = createIssueDevice(faceDevice.getSn(), admin, IssueTypeEnum.ISSUE_INCREMENT, list.size());
        //创建任务项
        List<IssueDeviceItem> issueDeviceItems = createIssueDeviceItems(list, issueDevice, admin);
        //全量下发用户数据
        handleIssue(issueDevice, issueDeviceItems, Constants.OP_ISSUE_INCREMENT, "增量下发");
    }

    @Override
    public void cleanUser(String sn) {
        FaceDevice device = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom()
                .andEqualTo(FaceDevice::getSn, sn)).build());
        Admin admin = adminDao.selectByPrimaryKey(device.getAdminId());
        IssueDevice issueDevice = handleIssueDeviceDelAll(device, admin);
        sendCleanUserFromDeviceMsg(issueDevice, sn);
    }

    private void sendCleanUserFromDeviceMsg(IssueDevice issueDevice, String sn) {
        try {
            WebsocketMsgReq req = new WebsocketMsgReq();
            req.setSn(sn);
            req.setSource(Constants.SOURCE_TQSERVER);
            req.setOp(Constants.OP_FACE_DELETED_ALL);
            req.setMsg("人员全量删除");
            DelDeviceFaceAllReq faceReq = new DelDeviceFaceAllReq();
            faceReq.setDeviceSn(sn);
            faceReq.setTraceId(issueDevice.getTraceId());
            req.setData(faceReq);
            log.info("--->发送websocket消息最终参数:{}", JSONObject.toJSONString(req));
//            otaWebSocketClient.send(JSONObject.toJSONString(req));
            hhFaceMiddlewareIssueApi(req);
        } catch (Exception e) {
            log.error("websocket连接中断");
        }
    }

    private IssueDevice handleIssueDeviceDelAll(FaceDevice device, Admin admin) {
        return createIssueDevice(device.getSn(), admin, IssueTypeEnum.FACE_DELETED_ALL, device.getUserCount());
    }

    @Override
    public void issueUserIncrement(DownFaceNew downFace, Date startTime, Date endTime, Integer adminId) {
        Admin admin = adminDao.selectByPrimaryKey(adminId);
        downFace.setValidPeriodStart(startTime);
        downFace.setValidPeriodEnd(endTime);
        //issue device 创建增量任务
        IssueDevice issueDevice = createIssueDevice(downFace.getSn(), admin, IssueTypeEnum.ISSUE_INCREMENT, 1);
        //创建任务项
        IssueDeviceItem issueDeviceItem = createIssueDeviceItem(downFace, issueDevice, admin);
        //全量下发用户数据
        handleIssue(issueDevice, Arrays.asList(issueDeviceItem), Constants.OP_ISSUE_INCREMENT, "增量下发");
    }

    @Override
    public void issueUserAll(FaceDevice faceDevice, List<DownFaceNew> list, AdminVO adminVO) {
        if (Preconditions.isBlank(list)) {
            return;
        }
        Admin admin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
        //issue device 创建全量任务
        checkAndHandleIssueDeviceAllConflictTask(faceDevice.getSn());

        IssueDevice issueDevice = createIssueDevice(faceDevice.getSn(), admin, IssueTypeEnum.ISSUE_ALL, list.size());
        //创建任务项
        List<IssueDeviceItem> issueDeviceItems = createIssueDeviceItems(list, issueDevice, admin);
        //全量下发用户数据
        handleIssue(issueDevice, issueDeviceItems, Constants.OP_ISSUE_ALL, "全量下发");

    }

    private void checkAndHandleIssueDeviceAllConflictTask(String sn) {
        List<IssueDevice> issueDevices = issueDeviceDao.selectByExample(Example.builder(IssueDevice.class).where(WeekendSqls.<IssueDevice>custom()
                .andEqualTo(IssueDevice::getDeviceSn, sn)
                .andIn(IssueDevice::getIssueStatus, Arrays.asList(IssueStatusEnum.PENDING.getCode(),
                        IssueStatusEnum.PROCESSING.getCode(),
                        IssueStatusEnum.INTERRUPTION.getCode()))).build());
        if (Preconditions.isBlank(issueDevices)) {
            return;
        }
        List<Long> issueDeviceIds = issueDevices.stream().map(IssueDevice::getId).collect(Collectors.toList());
        issueDeviceDao.updateStatusAndRemark(issueDeviceIds, IssueStatusEnum.CANCEL.getCode(), "全量任务下发取消");
        //向设备发送取消指令
        for (IssueDevice item : issueDevices) {
            handleIssueCancel(item);
        }
    }

    private List<IssueDeviceItem> createIssueDeviceItems(List<DownFaceNew> list, IssueDevice issueDevice, Admin admin) {
        log.info("--->插入详情列表:{}", list.size());
        if (Preconditions.isBlank(list)) {
            return null;
        }
        List<IssueDeviceItem> items = new ArrayList<>();
        for (DownFaceNew whitelist : list) {
            IssueDeviceItem item = new IssueDeviceItem();
            item.setCardIdEx(whitelist.getCardIdEx());
            item.setIdNumber(whitelist.getCardid());
            item.setIssueDeviceId(issueDevice.getId());
            item.setIssueStatus(IssueStatusEnum.PROCESSING.getCode());
            item.setIssueTime(new Date());
            item.setOrgId(DigestUtil.md5Hex(admin.getId() + whitelist.getUserId() + issueDevice.getDeviceSn()));
            item.setPhone(whitelist.getPhone());
            item.setStartTime(whitelist.getValidPeriodStart());
            item.setEndTime(whitelist.getValidPeriodEnd());
            item.setUserId(Integer.valueOf(String.valueOf(whitelist.getUserId())));
            item.setUserName(whitelist.getName());
            item.setUserType(whitelist.getRightno());
            item.setPhoto(whitelist.getPhoto());
            //TODO。。。人脸特征值
            item.setFacialFeature(whitelist.getFacialFeature());
            item.setCreateTime(new Date());
            item.setUpdateTime(new Date());
            item.setIsDeleted(Boolean.FALSE);
            item.setVersion(1);
            items.add(item);
        }
        createIssueDeviceItems(items);
        return items;
    }

    private void createIssueDeviceItems(List<IssueDeviceItem> items) {
        taskExecutor.execute(() -> {
            List<List<IssueDeviceItem>> split = ListUtil.split(items, 800);
            for (List<IssueDeviceItem> splitItems : split) {
                issueDeviceItemDao.insertList(splitItems);
            }
        });
    }

    private IssueDevice createIssueDevice(String sn, Admin admin, IssueTypeEnum issueType, Integer issueFaceCount) {
        IssueDevice issueDevice = new IssueDevice();
        issueDevice.setAdminId(admin.getId());
        issueDevice.setAdminUserName(admin.getUsername());
        issueDevice.setDeviceSn(sn);
        issueDevice.setHadIssueFaceCount(0);
        issueDevice.setIssueFaceCount(issueFaceCount);
        issueDevice.setIssueType(issueType.getCode());
        issueDevice.setTraceId(AppUtils.getAppSecret(AppUtils.getAppKey()));
        issueDevice.setIssueStatus(IssueStatusEnum.PENDING.getCode());
        issueDevice.setCreateTime(new Date());
        issueDevice.setUpdateTime(new Date());
        issueDevice.setIsDeleted(Boolean.FALSE);
        issueDevice.setVersion(1);
        issueDeviceDao.insert(issueDevice);
        return issueDevice;
    }

    private IssueDeviceItem createIssueDeviceItem(DownFaceNew downFace, IssueDevice issueDevice, Admin admin) {
        IssueDeviceItem item = new IssueDeviceItem();
        item.setCardIdEx(downFace.getCardIdEx());
        item.setIdNumber(downFace.getCardid());
        item.setIssueDeviceId(issueDevice.getId());
        item.setIssueStatus(IssueStatusEnum.PROCESSING.getCode());
        item.setIssueTime(new Date());
        item.setOrgId(DigestUtil.md5Hex(admin.getId() + downFace.getUserId() + issueDevice.getDeviceSn()));
        item.setPhone(downFace.getPhone());
        item.setStartTime(downFace.getValidPeriodStart());
        item.setEndTime(downFace.getValidPeriodEnd());
        item.setUserId(Integer.valueOf(String.valueOf(downFace.getUserId())));
        item.setUserName(downFace.getName());
        item.setUserType(downFace.getRightno());
        item.setPhoto(downFace.getPhoto());
        //TODO。。。人脸特征值
        item.setFacialFeature(downFace.getFacialFeature());
        item.setCreateTime(new Date());
        item.setUpdateTime(new Date());
        item.setIsDeleted(Boolean.FALSE);
        item.setVersion(1);
        issueDeviceItemDao.insert(item);
        return item;
    }

    @Override
    public void delFaceDataFromDevice(List<Long> userIds, String sn) {
        IssueDevice issueDevice = handleIssueDeviceDel(userIds, sn);
        sendDelFaceDataFromDeviceMsg(issueDevice, sn, userIds);
    }

    private void sendDelFaceDataFromDeviceMsg(IssueDevice issueDevice, String sn, List<Long> userIds) {
        try {
            WebsocketMsgReq req = new WebsocketMsgReq();
            req.setSn(sn);
            req.setSource(Constants.SOURCE_TQSERVER);
            req.setOp(Constants.OP_FACE_DELETED);
            req.setMsg("人员删除");
            DelDeviceFaceReq faceReq = new DelDeviceFaceReq();
            faceReq.setDeviceSn(sn);
            faceReq.setUserIds(userIds);
            faceReq.setTraceId(issueDevice.getTraceId());
            req.setData(faceReq);
            log.info("--->发送websocket消息最终参数:{}", JSONObject.toJSONString(req));
//            otaWebSocketClient.send(JSONObject.toJSONString(req));
            hhFaceMiddlewareIssueApi(req);
        } catch (Exception e) {
            log.error("websocket连接中断");
        }

    }


    //删除
    private IssueDevice handleIssueDeviceDel(List<Long> userIds, String sn) {
        FaceDevice device = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom()
                .andEqualTo(FaceDevice::getSn, sn)
                .andEqualTo(FaceDevice::getAuth, 1)
                .andEqualTo(FaceDevice::getEnable, 1)).build());
        if (Preconditions.isBlank(device)) {
            throw new CommonException(String.format("设备数据【%s】不存在或已删除", sn));
        }
        Admin admin = adminDao.selectByPrimaryKey(device.getAdminId());
        IssueDevice issueDevice = createIssueDevice(sn, admin, IssueTypeEnum.FACE_DELETED, userIds.size());
        List<UserInfoVO> userInfos = userInfoDao.getUserInfoByUserIds(userIds);
        List<DownFaceNew> downFaceNews = new ArrayList<>();
        for (UserInfoVO item : userInfos) {
            DownFaceNew downFaceNew = BeanUtil.copyProperties(item, DownFaceNew.class);
            downFaceNew.setName(item.getUsername());
            downFaceNew.setPhoto(item.getAvatar());
            downFaceNew.setCardid(item.getIdNumber());
            downFaceNew.setCardIdEx(item.getCardIdEx());
            downFaceNew.setRightno(105);
            downFaceNews.add(downFaceNew);
        }
        createIssueDeviceItems(downFaceNews, issueDevice, admin);
        return issueDevice;
    }

    @Override
    public void handleCallback(WebsocketMsgReq callbackReq) {
        String op = callbackReq.getOp();
        switch (op) {
            case "ISSUE_CALLBACK": //人脸下发
                IssueDeviceCallbackReq issueCallbackData = JSONObject.parseObject(JSONObject.toJSONString(callbackReq.getData()), IssueDeviceCallbackReq.class);
                handleIssueDeviceCallback(issueCallbackData);
                break;
            case "FACE_DELETED_CALLBACK": //人脸删除
                DelDeviceFaceCallbackReq delFaceCallbackData = JSONObject.parseObject(JSONObject.toJSONString(callbackReq.getData()), DelDeviceFaceCallbackReq.class);
                handleDelFaceCallback(delFaceCallbackData);
                break;
            case "FACE_DELETED_ALL_CALLBACK": //全量人脸删除
                DelDeviceFaceAllCallbackReq delFaceAllCallbackData = JSONObject.parseObject(JSONObject.toJSONString(callbackReq.getData()), DelDeviceFaceAllCallbackReq.class);
                handleDelFaceAllCallback(delFaceAllCallbackData);
                break;
            case "ISSUE_DEVICE_USER_REPORT": //设备人员数据上报
                IssueDeviceUserReportReq reportReq = JSONObject.parseObject(JSONObject.toJSONString(callbackReq.getData()), IssueDeviceUserReportReq.class);
                handleIssueDeviceUserReport(reportReq);
                break;
            case "ISSUE_DEVICE_FETCH": //设备人员获取
                IssueDeviceFetchReq fetchReq = JSONObject.parseObject(JSONObject.toJSONString(callbackReq.getData()), IssueDeviceFetchReq.class);
                handleIssueDeviceFetchReq(fetchReq);
                break;
            default:
                //TODO。。。默认
        }
    }

    private void handleIssueDeviceFetchReq(IssueDeviceFetchReq fetchReq) {
        List<IssueDevice> issueDevices = issueDeviceDao.selectByExample(Example.builder(IssueDevice.class).where(WeekendSqls.<IssueDevice>custom()
                .andEqualTo(IssueDevice::getDeviceSn, fetchReq.getDeviceSn())
                .andIn(IssueDevice::getIssueStatus, Arrays.asList(IssueStatusEnum.PENDING.getCode(),
                        IssueStatusEnum.PROCESSING.getCode(),
                        IssueStatusEnum.INTERRUPTION.getCode()))).orderByAsc("id").build());
        if (Preconditions.isBlank(issueDevices)) {
            log.error(String.format("--->设备【%s】不存在待下发的任务", fetchReq.getDeviceSn()));
            return;
        }
        List<Long> issueDeviceIds = issueDevices.stream().map(IssueDevice::getId).collect(Collectors.toList());
        List<IssueDeviceItem> issueDeviceItems = issueDeviceItemDao.selectByExample(Example.builder(IssueDeviceItem.class).where(WeekendSqls.<IssueDeviceItem>custom()
                .andIn(IssueDeviceItem::getIssueDeviceId, issueDeviceIds)).orderByAsc("id").build());
        Map<Long, List<IssueDeviceItem>> issueDeviceIdMap = new HashMap<>();
        if (Preconditions.isNotBlank(issueDeviceItems)) {
            issueDeviceIdMap = issueDeviceItems.stream().collect(Collectors.groupingBy(IssueDeviceItem::getIssueDeviceId));
        }

        for (IssueDevice item : issueDevices) {
            List<IssueDeviceItem> items = issueDeviceIdMap.get(item.getId());
            String issueType = item.getIssueType();
            switch (issueType) {
                case "face_delete":
                    List<Integer> iUserIds = items.stream().map(IssueDeviceItem::getUserId).collect(Collectors.toList());
                    sendDelFaceDataFromDeviceMsg(item, item.getDeviceSn(), JSONArray.parseArray(iUserIds.toString(), Long.class));
                    break;
                case "face_delete_all":
                    sendCleanUserFromDeviceMsg(item, item.getDeviceSn());
                    break;
                case "all":
                    handleIssue(item, items, IssueTypeEnum.getByCode(issueType).getOpCode(), IssueTypeEnum.getByCode(issueType).getDesc());
                    break;
                case "increment":
                    handleIssue(item, items, IssueTypeEnum.getByCode(issueType).getOpCode(), IssueTypeEnum.getByCode(issueType).getDesc());
                    break;
                default:
                    //TODO。。。默认
            }
        }

    }

    private void handleDelFaceAllCallback(DelDeviceFaceAllCallbackReq delFaceAllCallbackData) {
        String traceId = delFaceAllCallbackData.getTraceId();
        String deviceSn = delFaceAllCallbackData.getDeviceSn();
        IssueDevice issueDevice = issueDeviceDao.selectOneByExample(Example.builder(IssueDevice.class).where(WeekendSqls.<IssueDevice>custom()
                .andIn(IssueDevice::getIssueStatus, Arrays.asList(IssueStatusEnum.PROCESSING.getCode(), IssueStatusEnum.PENDING.getCode(), IssueStatusEnum.INTERRUPTION.getCode()))
                .andEqualTo(IssueDevice::getDeviceSn, deviceSn)
                .andEqualTo(IssueDevice::getTraceId, traceId)).build());
        if (Preconditions.isBlank(issueDevice)) {
            log.error("--->通过TraceId：【{}】未查询到相关主数据", traceId);
            return;
        }
        if (Preconditions.isNotBlank(issueDevice.getIssueStatus())
                && IssueStatusEnum.CANCEL.getCode().equals(issueDevice.getIssueStatus())) {
            log.error("--->当前TraceId：【{}】任务已取消", traceId);
            return;
        }
        issueDevice.setIssueStatus(delFaceAllCallbackData.getIssueStatus());
        issueDevice.setRemarks(delFaceAllCallbackData.getMsg());
        issueDeviceDao.updateByPrimaryKey(issueDevice);
    }

    //TODO。。。处理删除人脸数据的回调数据
    private void handleDelFaceCallback(DelDeviceFaceCallbackReq delFaceCallbackData) {
        String traceId = delFaceCallbackData.getTraceId();
        String deviceSn = delFaceCallbackData.getDeviceSn();
        List<IssueDeviceItemCallbackReq> items = delFaceCallbackData.getItems();
        IssueDevice issueDevice = issueDeviceDao.selectOneByExample(Example.builder(IssueDevice.class).where(WeekendSqls.<IssueDevice>custom()
                .andIn(IssueDevice::getIssueStatus, Arrays.asList(IssueStatusEnum.PROCESSING.getCode(), IssueStatusEnum.PENDING.getCode(), IssueStatusEnum.INTERRUPTION.getCode()))
                .andEqualTo(IssueDevice::getDeviceSn, deviceSn)
                .andEqualTo(IssueDevice::getTraceId, traceId)).build());
        if (Preconditions.isBlank(issueDevice)) {
            log.error("--->通过TraceId：【{}】未查询到相关主数据", traceId);
            return;
        }
        if (Preconditions.isNotBlank(issueDevice.getIssueStatus())
                && IssueStatusEnum.CANCEL.getCode().equals(issueDevice.getIssueStatus())) {
            log.error("--->当前TraceId：【{}】任务已取消", traceId);
            return;
        }
        issueDevice.setIssueStatus(Preconditions.isBlank(delFaceCallbackData.getIssueStatus()) ? IssueStatusEnum.SUCCESS.getCode() : delFaceCallbackData.getIssueStatus());
        issueDevice.setRemarks(Preconditions.isBlank(delFaceCallbackData.getMsg()) ? IssueStatusEnum.getByCode(issueDevice.getIssueStatus()).getDesc() : delFaceCallbackData.getMsg());
        issueDeviceDao.updateByPrimaryKey(issueDevice);
        //处理回调用户数据
        handleIssueDeviceItemCallback(issueDevice.getId(), items);
    }

    //处理下发人员回调数据
    private void handleIssueDeviceCallback(IssueDeviceCallbackReq callbackReq) {
        String traceId = callbackReq.getTraceId();
        String deviceSn = callbackReq.getDeviceSn();
        String issueStatus = callbackReq.getIssueStatus();
        List<IssueDeviceItemCallbackReq> items = callbackReq.getItems();
        IssueDevice issueDevice = issueDeviceDao.selectOneByExample(Example.builder(IssueDevice.class).where(WeekendSqls.<IssueDevice>custom()
                .andIn(IssueDevice::getIssueStatus, Arrays.asList(IssueStatusEnum.PROCESSING.getCode(), IssueStatusEnum.PENDING.getCode(), IssueStatusEnum.INTERRUPTION.getCode()))
                .andEqualTo(IssueDevice::getDeviceSn, deviceSn)
                .andEqualTo(IssueDevice::getTraceId, traceId)).build());
        if (Preconditions.isBlank(issueDevice)) {
            log.error("--->通过TraceId：【{}】未查询到相关主数据", traceId);
            return;
        }
        if (Preconditions.isNotBlank(issueDevice.getIssueStatus())
                && IssueStatusEnum.CANCEL.getCode().equals(issueDevice.getIssueStatus())) {
            log.error("--->当前TraceId：【{}】任务已取消", traceId);
            return;
        }

        issueDevice.setIssueStatus(issueStatus);
        issueDevice.setHadIssueFaceCount(callbackReq.getHadIssuedFaceCount());
        issueDevice.setRemarks(callbackReq.getMsg());
        issueDeviceDao.updateByPrimaryKey(issueDevice);
        if ((!IssueStatusEnum.SUCCESS.getCode().equals(issueStatus) && !IssueStatusEnum.FAILED.getCode().equals(issueStatus)) || Preconditions.isBlank(items)) {
            log.error("--->通过TraceId：【{}】非成功或失败状态暂不做处理", traceId);
            return;
        }
        //处理回调用户数据
        handleIssueDeviceItemCallback(issueDevice.getId(), items);
    }

    private void handleIssueDeviceItemCallback(Long issueDeviceId, List<IssueDeviceItemCallbackReq> items) {
        this.handleIssueDeviceItemCallback(issueDeviceId, items, false);
    }

    public void handleIssueDeviceItemCallback(Long issueDeviceId, List<IssueDeviceItemCallbackReq> items, Boolean isAiFace) {
        if (Preconditions.isBlank(items)) {
            log.error("--->通过IssueDeviceId：【{}】回调下发数据为空", issueDeviceId);
            return;
        }
        List<Integer> userIds = items.stream().map(IssueDeviceItemCallbackReq::getUserId).collect(Collectors.toList());
        Example build = Example.builder(IssueDeviceItem.class).where(WeekendSqls.<IssueDeviceItem>custom()
                .andEqualTo(IssueDeviceItem::getIssueDeviceId, issueDeviceId)
                .andIn(IssueDeviceItem::getUserId, userIds)
        ).build();

        List<IssueDeviceItem> dbIssueDeviceItems = issueDeviceItemDao.selectByExample(build);
        if (Preconditions.isBlank(dbIssueDeviceItems)) {
            log.error("--->通过IssueDeviceId：【{}】未查询到相关主数据", issueDeviceId);
            return;
        }
        Map<Integer, IssueDeviceItemCallbackReq> idiCallbackMap = items.stream().collect(Collectors.toMap(IssueDeviceItemCallbackReq::getUserId, v -> v, (v1, v2) -> v1));
        dbIssueDeviceItems.stream().forEach(item -> {
            IssueDeviceItemCallbackReq itemCallbackReq = idiCallbackMap.get(item.getUserId());
            if (Preconditions.isNotBlank(itemCallbackReq)) {
                item.setIssueStatus(itemCallbackReq.getCode().intValue() == 200 ? IssueStatusEnum.SUCCESS.getCode() : IssueStatusEnum.FAILED.getCode());
                item.setRemarks(itemCallbackReq.getMsg());
                issueDeviceItemDao.updateByPrimaryKey(item);
            }
        });
    }

    //处理设备定时上报设备人员数量消息
    private void handleIssueDeviceUserReport(IssueDeviceUserReportReq reportReq) {

        IssueDeviceUser issueDeviceUser = issueDeviceUserDao.selectOneByExample(Example.builder(IssueDeviceUser.class).where(WeekendSqls.<IssueDeviceUser>custom().andEqualTo(IssueDeviceUser::getDeviceSn, reportReq.getDeviceSn())).build());
        if (Preconditions.isBlank(issueDeviceUser)) {
            issueDeviceUser = new IssueDeviceUser();
        }
        issueDeviceUser.setDeviceSn(reportReq.getDeviceSn());
        issueDeviceUser.setHhFaceVersion(reportReq.getHhFaceVersion());
        issueDeviceUser.setInternetIp(reportReq.getInternetIp());
        issueDeviceUser.setUserIds(reportReq.getUserIds());
        issueDeviceUser.setUserCount(reportReq.getUserCount());
        issueDeviceUser.setLastTrackingTime(new Date());
        issueDeviceUser.setCreateTime(new Date());
        issueDeviceUser.setUpdateTime(new Date());
        issueDeviceUser.setIsDeleted(Boolean.FALSE);
        issueDeviceUser.setVersion(1);
        if (Preconditions.isBlank(issueDeviceUser.getId())) {
            issueDeviceUserDao.insert(issueDeviceUser);
        } else {
            issueDeviceUserDao.updateByPrimaryKey(issueDeviceUser);
        }

        try {
            FaceDevice device = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom()
                    .andEqualTo(FaceDevice::getSn, reportReq.getDeviceSn())
                    .andEqualTo(FaceDevice::getAuth, 1)
                    .andEqualTo(FaceDevice::getEnable, 1)).build());
            if (Preconditions.isNotBlank(device)) {
                device.setOnline(1);
                device.setInterIp(reportReq.getInternetIp());
                device.setUserCount(reportReq.getUserCount());
                device.setAppVersion(reportReq.getHhFaceVersion());
                faceDeviceDao.updateByPrimaryKey(device);
                FkjHeartTracking tracking = fkjHeartTrackingDao.selectOneByExample(Example.builder(FkjHeartTracking.class).where(WeekendSqls.<FkjHeartTracking>custom().andEqualTo(FkjHeartTracking::getSn, device.getSn()).andEqualTo(FkjHeartTracking::getAdminId, device.getAdminId())).build());
                if (Preconditions.isNotBlank(tracking)) {
                    tracking.setLastTrackingTime(new Date());
                    fkjHeartTrackingDao.updateByPrimaryKey(tracking);
                } else {
                    tracking = new FkjHeartTracking();
                    tracking.setSn(device.getSn());
                    tracking.setCreateTime(new Date());
                    tracking.setLastTrackingTime(new Date());
                    tracking.setAdminId(device.getAdminId());
                    fkjHeartTrackingDao.insert(tracking);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private void handleIssue(IssueDevice issueDevice, List<IssueDeviceItem> issueDeviceItems, String op, String msg) {
        try {
            WebsocketMsgReq req = new WebsocketMsgReq();
            IssueDeviceReq deviceReq = BeanUtil.copyProperties(issueDevice, IssueDeviceReq.class);
            List<IssueDeviceItemReq> items = BeanUtil.copyToList(issueDeviceItems, IssueDeviceItemReq.class);
            deviceReq.setItems(items);
            req.setSn(issueDevice.getDeviceSn());
            req.setSource(Constants.SOURCE_TQSERVER);
            req.setOp(op);
            req.setData(deviceReq);
            req.setMsg(msg);
            log.info("--->发送websocket消息最终参数:{}", JSONObject.toJSONString(req));
            hhFaceMiddlewareIssueApi(req);
        } catch (Exception e) {
            log.error("websocket连接中断");
        }
    }

    public void hhFaceMiddlewareIssueApi(WebsocketMsgReq req) {
        if (httpApiSwitch) {
            String respStr = HttpUtil.post(httpApiUrl, JSONObject.toJSONString(req), 60000);
            log.info("--->http请求api接口：{}", respStr);
//        RestResult result = JSONObject.parseObject(respStr, RestResult.class);
            return;
        }
        otaWebSocketClient.send(JSONObject.toJSONString(req));
    }


}
