package com.yx.face.service.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.model.dto.HealthCodeTaskDto;
import com.yx.face.model.vo.PlaceInfoVO;
import com.yx.face.service.OpenIssuedService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @description:
 * @ClassName AdminDataShowTask
 * @Author WXG
 * @Date 2021/11/12 20:52
 */
@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
@EnableAsync
@Slf4j
public class HealthCodeTask {
    /*
    * 建立一个全局的Map
    * */
    public static ConcurrentHashMap<String, HealthCodeTaskDto> hashMap = new ConcurrentHashMap();

    @Resource
    private OpenIssuedService openIssuedService;

    @Scheduled(cron = "0 1/5 * * * ?", zone = "Asia/Shanghai")
    @Async
    public void syncHealthCodeRedis() {
        /*
        * 兜底逻辑
        * 在同步联动平台设备的缓存时候，如果重试用完的时候，会把设备丢到这里
        * */
        if(hashMap.isEmpty()){
            return;
        }
        log.info("----------开始------同步联动平台的设备缓存配置-------------");
        Date now = new Date();
        for (String key : hashMap.keySet()) {
            HealthCodeTaskDto dto = hashMap.get(key);
            if(DateUtil.compare(now,dto.getStartRetryTime()) <= 0){
                continue;
            }
            try {
                RestResult<PlaceInfoVO> restResult = openIssuedService.queryPlaceInfoBySn(dto.getSn());
                HttpResponse response = HttpRequest.post(dto.getUrl())
                        .body(JSON.toJSONString(restResult)).timeout(3000).execute();
                String result = response.body();
                RestResult resp = JSON.parseObject(result, new TypeReference<RestResult>() {});
                log.info("sn配置联动平台缓存{}", JSON.toJSONString(resp));
                hashMap.remove(key);
            }catch (Exception exception){
                Integer retryNumber = dto.getRetryNumber();
                retryNumber--;
                if(retryNumber <= 0){
                    hashMap.remove(key);
                }
                dto.setRetryNumber(retryNumber);
            }
        }
        log.info("----------结束------同步联动平台的设备缓存配置-------------");

    }
}
