package com.yx.face.service.middleware.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.uitls.Base64Util;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.boot.uitls.OkHttpUtils;
import com.yx.face.boot.uitls.Preconditions;
import com.yx.face.constant.CommonConstants;
import com.yx.face.dao.FacePassDao;
import com.yx.face.dao.smartlock.MiddlewareDeviceUserDao;
import com.yx.face.model.api.middleware.DeleteUserInfoCallbackDto;
import com.yx.face.model.api.middleware.DeleteUserInfoDto;
import com.yx.face.model.api.middleware.PushUserInfoCallbackDto;
import com.yx.face.model.api.middleware.PushUserInfoDto;
import com.yx.face.model.api.middleware.SendMsg2AppDto;
import com.yx.face.model.api.middleware.SendMsg2ServerDto;
import com.yx.face.model.entity.FacePass;
import com.yx.face.model.entity.MiddlewareDeviceUser;
import com.yx.face.model.entity.DownFaceNew;
import com.yx.face.service.fkj.FkjService;
import com.yx.face.service.middleware.MiddlewareService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package com.yx.face.service.middleware
 * @class MiddlewareServiceImpl
 * @date 2024/1/21 下午4:39
 * @description
 */
@Service
@Slf4j
public class MiddlewareServiceImpl implements MiddlewareService {

    @Value("${middleware.domain}")
    private String middleDomain;

    @Value("${middleware.uri}")
    private String middleUri;

    @Resource
    private FkjService fkjService;

    @Resource
    private MiddlewareDeviceUserDao middlewareDeviceUserDao;


    @Resource
    private FacePassDao facePassDao;

    @Override
    public void sendMsg2Server(SendMsg2ServerDto req) {
        log.info("--->收到设备发送消息:{}", JSONObject.toJSONString(req));
        String op = req.getOp();
        Object data = req.getData();
        if (CommonConstants.WEBSOCKET_OP_DEVICE_HEART_TRACKING.equals(op)) { //设备发送心跳(15秒发送一次心跳 60秒未发送心跳视为设备离线状态)
            //TODO。。。
            fkjService.tracking(req.getSn());
            return;
        }

        if (CommonConstants.WEBSOCKET_OP_PUSH_USER_INFO_CALLBACK.equals(op)) { //下发用户信息回调
            //TODO。。。
            List<PushUserInfoCallbackDto> pushUsers = JSONObject.parseArray(JSONObject.toJSONString(data), PushUserInfoCallbackDto.class);
            handlePushUserInfoCallback(req.getSn(), pushUsers);
            return;
        }

        if (CommonConstants.WEBSOCKET_OP_DELETE_USER_INFO_CALLBACK.equals(op)) { //删除用户回调
            //TODO。。。
            List<DeleteUserInfoCallbackDto> delUsers = JSONObject.parseArray(JSONObject.toJSONString(data), DeleteUserInfoCallbackDto.class);
            handleDeleteUserInfoCallback(req.getSn(), delUsers);
            return;
        }
    }

    private void handlePushUserInfoCallback(String sn, List<PushUserInfoCallbackDto> pushUsers) {
        if (Preconditions.isBlank(pushUsers) || Preconditions.isBlank(sn)) {
            return;
        }

        List<String> pins = pushUsers.stream().map(PushUserInfoCallbackDto::getPin).collect(Collectors.toList());
        if (Preconditions.isBlank(pins)) {
            return;
        }
        Map<String, PushUserInfoCallbackDto> pushMap = pushUsers.stream().collect(Collectors.toMap(PushUserInfoCallbackDto::getPin, a -> a, (k1, k2) -> k1));


        List<MiddlewareDeviceUser> users = middlewareDeviceUserDao.selectByExample(Example.builder(MiddlewareDeviceUser.class)
                .where(WeekendSqls.<MiddlewareDeviceUser>custom()
                        .andEqualTo(MiddlewareDeviceUser::getDeviceSn, sn).andIn(MiddlewareDeviceUser::getPin, pins)).build());
        if (Preconditions.isBlank(users)) {
            return;
        }

        Map<Long, MiddlewareDeviceUser> usersMap = users.stream().collect(Collectors.toMap(MiddlewareDeviceUser::getUserId, a -> a, (k1, k2) -> k1));


        List<Long> userIds = users.stream().map(MiddlewareDeviceUser::getUserId).collect(Collectors.toList());
        List<FacePass> facePasses = facePassDao.selectByExample(Example.builder(FacePass.class)
                .where(WeekendSqls.<FacePass>custom()
                        .andEqualTo(FacePass::getDeviceSn, sn).andIn(FacePass::getUserId, userIds)).build());
        if (Preconditions.isBlank(facePasses)) {
            return;
        }

        facePasses.stream().forEach(item -> {
            if (usersMap.containsKey(item.getUserId())) {
                PushUserInfoCallbackDto callbackDto = pushMap.get(usersMap.get(item.getUserId()).getPin());
                item.setRemark(callbackDto.getMsg());
                item.setStatus("200".equals(callbackDto.getStatus()) ? 2 : 3);
                item.setUpdateTime(new Date());
                facePassDao.updateByPrimaryKey(item);
            }
        });
    }

    private void handleDeleteUserInfoCallback(String sn, List<DeleteUserInfoCallbackDto> delUsers) {
        if (Preconditions.isBlank(delUsers) || Preconditions.isBlank(sn)) {
            return;
        }
        List<String> pins = delUsers.stream().filter(item -> "200".equals(item.getStatus())).map(DeleteUserInfoCallbackDto::getPin).collect(Collectors.toList());
        if (Preconditions.isBlank(pins)) {
            return;
        }
        middlewareDeviceUserDao.deleteByExample(Example.builder(MiddlewareDeviceUser.class)
                .where(WeekendSqls.<MiddlewareDeviceUser>custom()
                        .andEqualTo(MiddlewareDeviceUser::getDeviceSn, sn)
                        .andIn(MiddlewareDeviceUser::getPin, pins)).build());

    }


    public void sendMsg2App(SendMsg2AppDto dto) {
        try {
            log.info("--->服务端发送消息给中间件入参:{}", JSONObject.toJSONString(dto));
            String resp = OkHttpUtils.builder().url(middleDomain + middleUri).localPost(JSONObject.toJSONString(dto)).sync();
            log.info("--->服务端发送消息给中间件响应参数:{}", resp);
        } catch (Exception e) {
            log.error("--->{}服务端发送消息给中间件服务异常", dto.getSn(), e);
        }

    }

    @Override
    public void addUser(DownFaceNew downFace, Date startTime, Date endTime, Integer adminId) {
        if (Preconditions.isBlank(downFace.getPhoto())) {
            throw new CommonException("人脸照片不存在");
        }
        SendMsg2AppDto app = new SendMsg2AppDto();
        app.setSn(downFace.getSn());
        app.setSource(CommonConstants.WEBSOCKET_SOURCE_SERVER);
        app.setOp(CommonConstants.WEBSOCKET_OP_PUSH_USER_INFO);
        app.setMsg("推送(下发)用户信息");
        String pin = DigestUtils.md5DigestAsHex(downFace.getSn().concat(downFace.getUserId() + "").getBytes());
        PushUserInfoDto dto = new PushUserInfoDto();
        dto.setCardid(downFace.getCardid());
        dto.setPin(pin);
        dto.setCardIdEx(downFace.getCardIdEx());
        dto.setName(downFace.getName());
        dto.setPhone(downFace.getPhone());
        dto.setValidPeriodStart(DateUtil.format(startTime, DateUtils.YYYY_MM_DD_HH_MM_SS));
        dto.setValidPeriodEnd(DateUtil.format(endTime, DateUtils.YYYY_MM_DD_HH_MM_SS));
        dto.setPhoto(Base64Util.imgToBase64(downFace.getPhoto()));
        app.setData(Arrays.asList(dto));
        sendMsg2App(app);
        handleMiddlewareDeviceUser(downFace.getSn(), downFace.getUserId(), adminId, pin);
    }

    private void handleMiddlewareDeviceUser(String sn, Long userId, Integer adminId, String pin) {
        MiddlewareDeviceUser user = middlewareDeviceUserDao.selectOneByExample(Example.builder(MiddlewareDeviceUser.class)
                .where(WeekendSqls.<MiddlewareDeviceUser>custom()
                        .andEqualTo(MiddlewareDeviceUser::getDeviceSn, sn)
                        .andEqualTo(MiddlewareDeviceUser::getUserId, userId)).build());
        if (Preconditions.isNotBlank(user)) {
            user.setAdminId(adminId);
            user.setUserId(userId);
            user.setPin(pin);
            user.setDeviceSn(sn);
            middlewareDeviceUserDao.updateByPrimaryKey(user);
        } else {
            user = new MiddlewareDeviceUser();
            user.setAdminId(adminId);
            user.setUserId(userId);
            user.setPin(pin);
            user.setCreateTime(new Date());
            user.setDeviceSn(sn);
            middlewareDeviceUserDao.insert(user);
        }

    }

    @Override
    public void cleanUser(String sn) {
        List<MiddlewareDeviceUser> users = middlewareDeviceUserDao.selectByExample(Example.builder(MiddlewareDeviceUser.class)
                .where(WeekendSqls.<MiddlewareDeviceUser>custom()
                        .andEqualTo(MiddlewareDeviceUser::getDeviceSn, sn)).build());
        if (Preconditions.isBlank(users)) {
            return;
        }

        SendMsg2AppDto app = new SendMsg2AppDto();
        app.setSn(sn);
        app.setSource(CommonConstants.WEBSOCKET_SOURCE_SERVER);
        app.setOp(CommonConstants.WEBSOCKET_OP_DELETE_USER_INFO);
        app.setMsg("删除用户信息");
        List<DeleteUserInfoDto> pins = new ArrayList<>();
        for (MiddlewareDeviceUser item : users) {
            DeleteUserInfoDto dto = new DeleteUserInfoDto();
            dto.setPin(item.getPin());
            pins.add(dto);
        }
        app.setData(pins);
        sendMsg2App(app);
    }

    @Override
    public void delUser(String sn, List<Long> userIds) {

        if (Preconditions.isBlank(userIds)) {
            return;
        }

        List<MiddlewareDeviceUser> users = middlewareDeviceUserDao.selectByExample(Example.builder(MiddlewareDeviceUser.class)
                .where(WeekendSqls.<MiddlewareDeviceUser>custom()
                        .andEqualTo(MiddlewareDeviceUser::getDeviceSn, sn).andIn(MiddlewareDeviceUser::getUserId, userIds)).build());
        if (Preconditions.isBlank(users)) {
            return;
        }
        List<String> fPins = users.stream().map(MiddlewareDeviceUser::getPin).collect(Collectors.toList());

        SendMsg2AppDto app = new SendMsg2AppDto();
        app.setSn(sn);
        app.setSource(CommonConstants.WEBSOCKET_SOURCE_SERVER);
        app.setOp(CommonConstants.WEBSOCKET_OP_DELETE_USER_INFO);
        app.setMsg("删除用户信息");
        List<DeleteUserInfoDto> pins = new ArrayList<>();
        for (String pin : fPins) {
            DeleteUserInfoDto dto = new DeleteUserInfoDto();
            dto.setPin(pin);
            pins.add(dto);
        }
        app.setData(pins);
        sendMsg2App(app);

    }


}
