package com.yx.face.service.server;

import cn.hutool.core.collection.CollUtil;
import com.yx.face.boot.component.security.JWTUtils;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestCode;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.dao.AdminDao;
import com.yx.face.model.entity.Admin;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.stream.Collectors;

@Component
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class CommonServer {

    private final AdminDao adminDao;

    /*获取所有下级且为区级的adminId ，区级账号（非楼宇账号）和企业返回自己的adminId*/
    public List<Integer> getBelowAdminIdList(){
        //1.获取当前账号
        Admin admin = this.getAdmin(this.getAdminId());
        if(AdminTypeEnum.ENTERPRISE.getCode().equals(admin.getType())){
            return CollUtil.toList(admin.getId());
        } else if (AdminTypeEnum.DISTRICT.getCode().equals(admin.getType())){
            if(AdminPlaceTypeEnum.COMMON_PLACE.getCode().equals(admin.getPlaceType()) || AdminPlaceTypeEnum.APPOINTMENT_PLACE.getCode().equals(admin.getPlaceType())){
                return CollUtil.toList(admin.getId());
            }else {
                List<Admin> adminList = adminDao.select(new Admin().setTenantId(admin.getId()));
                List<Integer> adminIdList = adminList.stream().map(Admin::getId).collect(Collectors.toList());
                adminIdList.add(admin.getId());
                return adminIdList;
            }
        }else {
            return adminDao.selectBelowAdminIdList(admin);
        }
    }
    private Admin getAdmin(Integer adminId){
        return adminDao.selectByPrimaryKey(adminId);
    }
    private Integer getAdminId(){
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (requestAttributes == null) {
            throw new CommonException(RestCode.UNAUTHORIZED);
        }
        HttpServletRequest request = requestAttributes.getRequest();
        String token = request.getHeader("Authorization");
        if (token == null || token.equals("")) {
            throw new ServiceException(RestCode.UNAUTHORIZED.code, "缺少token");
        }
        token = JWTUtils.getUserId(token);
        if (org.apache.commons.lang3.StringUtils.isEmpty(token)) {
            throw new CommonException(RestCode.UNAUTHORIZED);
        }
        return Integer.parseInt(token);
    }
}
