package com.yx.face.service.smartlock.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.uitls.*;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.dao.UserInfoDao;
import com.yx.face.dao.smartlock.SmartlockUserDao;
import com.yx.face.model.dto.smartlock.*;
import com.yx.face.model.entity.DownFaceNew;
import com.yx.face.model.entity.FaceDevice;
import com.yx.face.model.entity.SmartlockUser;
import com.yx.face.model.entity.UserInfo;
import com.yx.face.service.FaceTBService;
import com.yx.face.service.smartlock.AdminSmartlockDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author shisl
 * @package com.yx.face.service.smartlock.impl
 * @class AdminSmartlockDeviceServiceImpl
 * @date 2023/8/3 下午1:40
 * @description 智能门锁
 */
@Slf4j
@Service
public class AdminSmartlockDeviceServiceImpl implements AdminSmartlockDeviceService {

    @Value("${face.service.smartlock.host}")
    private String HOST;

    @Value("${face.service.smartlock.oauthUri}")
    private String OAUTH_URI;

    @Value("${face.service.smartlock.devicesUri}")
    private String DEVICES_URI;

    @Value("${face.service.smartlock.subscribeUri}")
    private String SUBSCRIBE_URI;

    @Value("${face.service.smartlock.callbackUrl}")
    private String CALLBACK_URL;

    @Value("${face.service.smartlock.appId}")
    private String APP_ID;

    @Value("${face.service.smartlock.appSecret}")
    private String APP_SECRET;

    /**
     * 智能门锁token key
     */
    private static String SMART_LOCK_TOKEN_KEY = "smartlock:token:%s";

    @Autowired
    private RedisUtil redisUtil;

    @Resource
    private FaceDeviceDao faceDeviceDao;

    @Resource
    private SmartlockUserDao smartlockUserDao;

   

    @Resource
    private FaceTBService faceTBService;

    @Resource
    private UserInfoDao userInfoDao;


    private static final char[] DIGITS = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

    @Override
    public String oauthToken() {
        String tokenKey = String.format(SMART_LOCK_TOKEN_KEY, APP_ID);
        try {
            if (redisUtil.hasKey(tokenKey)) {
                return (String) redisUtil.get(tokenKey);
            }
        } catch (Exception e) {

        }
        String token = null;
        SmartlockAuthTokenReq req = new SmartlockAuthTokenReq();
        req.setAppId(APP_ID);
        req.setAppSecret(APP_SECRET);
        //获取token 鉴权接口是调用 API 接口的前提，除了调用鉴权接口，其他接口的调用都需要在 Header 中携带 appID 和 accessToken，认证有效期为 24 小时，认证过期后需要重新调用此接口进行认证。
        // SmartlockAuthTokenResp
        log.info("--->获取智能门锁授权token入参:{}", JSONObject.toJSONString(req));
        String resp = OkHttpUtils.builder().url(HOST + OAUTH_URI).localPost(JSONObject.toJSONString(req)).sync();
        log.info("--->获取智能门锁授权token响应参数:{}", resp);
        JSONObject object = JSONObject.parseObject(resp);
        if (Preconditions.isNotBlank(object) && Preconditions.isNotBlank(object.get("accessToken")) && !"".equals(object.get("accessToken").toString())) {
            token = object.get("accessToken").toString();
        }
        if (Preconditions.isNotBlank(token)) {
            //缓存23个小时
            redisUtil.set(tokenKey, token, 82800l);
        }
        return token;
    }

    @Override
    public void deviceInfo(SmartlockDeviceInfoReq infoReq) {
        try {
            String accessToken = oauthToken();
            log.info("--->获取智能门锁信息入参:{}", JSONObject.toJSONString(infoReq));
            String resp = OkHttpUtils.builder().url(HOST + String.format(DEVICES_URI, infoReq.getDeviceSn())).addHeader("appId", APP_ID).addHeader("accessToken", accessToken).get().sync();
            log.info("--->获取智能门锁信息响应参数:{}", resp);
        } catch (Exception e) {
            log.error("--->获取{}智能门锁信息服务异常", infoReq.getDeviceSn(), e);
        }

    }

    @Override
    public SmartlockDeviceResp addUser(SmartlockDeviceAddUserReq addReq) {
        try {
            String accessToken = oauthToken();
            SmartlockDeviceReq req = new SmartlockDeviceReq();
            req.setData(addReq);
            req.setType(addReq.getType());
            log.info("--->智能门锁添加用户入参:{}", JSONObject.toJSONString(req));
            String format = String.format(DEVICES_URI, addReq.getDeviceSn());
            String resp = OkHttpUtils.builder().url(HOST + format).addHeader("appId", APP_ID).addHeader("accessToken", accessToken).localPost(JSONObject.toJSONString(req)).sync();
            log.info("--->智能门锁添加用户响应参数:{}", resp);
            return JSONObject.parseObject(resp, SmartlockDeviceResp.class);
        } catch (Exception e) {
            log.error("--->获取{}智能门锁添加用户服务异常", addReq.getDeviceSn(), e);
        }
        return null;
    }

    @Override
    public void delUser(SmartlockDeviceDelUserReq delReq) {
        try {
            String accessToken = oauthToken();
            SmartlockDeviceReq req = new SmartlockDeviceReq();
            req.setData(delReq);
            req.setType(delReq.getType());
            log.info("--->智能门锁删除用户入参:{}", JSONObject.toJSONString(req));
            String resp = OkHttpUtils.builder().url(HOST + String.format(DEVICES_URI, delReq.getDeviceSn())).addHeader("appId", APP_ID).addHeader("accessToken", accessToken).localPost(JSONObject.toJSONString(req)).sync();
            log.info("--->智能门锁删除用户响应参数:{}", resp);
        } catch (Exception e) {
            log.error("--->智能门锁{}删除用户{}服务异常", delReq.getDeviceSn(), delReq.getUserId(), e);
        }

    }

    @Override
    public void userChangePwd(SmartlockUserChangePwdReq pwdReq) {
        try {
            String accessToken = oauthToken();
            SmartlockDeviceReq req = new SmartlockDeviceReq();
            req.setData(pwdReq);
            req.setType(pwdReq.getType());
            log.info("--->智能门锁操作管理员入参:{}", JSONObject.toJSONString(req));
            String resp = OkHttpUtils.builder().url(HOST + String.format(DEVICES_URI, pwdReq.getDeviceSn())).addHeader("appId", APP_ID).addHeader("accessToken", accessToken).localPost(JSONObject.toJSONString(req)).sync();
            log.info("--->智能门锁操作管理员响应参数:{}", resp);
        } catch (Exception e) {
            log.error("--->{}智能门锁操作管理员服务异常", pwdReq.getDeviceSn(), e);
        }

    }

    @Override
    public void cleanUser(SmartlockDeviceCleanUserReq cleanReq) {
        try {
            String accessToken = oauthToken();
            SmartlockDeviceReq req = new SmartlockDeviceReq();
            req.setData(cleanReq);
            req.setType(cleanReq.getType());
            log.info("--->智能门锁清空用户入参:{}", JSONObject.toJSONString(req));
            String resp = OkHttpUtils.builder().url(HOST + String.format(DEVICES_URI, cleanReq.getDeviceSn())).addHeader("appId", APP_ID).addHeader("accessToken", accessToken).localPost(JSONObject.toJSONString(req)).sync();
            log.info("--->智能门锁清空用户响应参数:{}", resp);
        } catch (Exception e) {
            log.error("--->{}智能门锁清空用户服务异常", cleanReq.getDeviceSn(), e);
        }
    }

    @Override
    public void enableUser(SmartlockDeviceEnableUserReq enableReq) {
        try {
            String accessToken = oauthToken();
            SmartlockDeviceReq req = new SmartlockDeviceReq();
            req.setData(enableReq);
            req.setType(enableReq.getType());
            log.info("--->智能门锁启用用户入参:{}", JSONObject.toJSONString(req));
            String resp = OkHttpUtils.builder().url(HOST + String.format(DEVICES_URI, enableReq.getDeviceSn())).addHeader("appId", APP_ID).addHeader("accessToken", accessToken).localPost(JSONObject.toJSONString(req)).sync();
            log.info("--->智能门锁启用用户响应参数:{}", resp);
        } catch (Exception e) {
            log.error("--->{}智能门锁启用用户服务异常", enableReq.getDeviceSn(), e);
        }
    }

    @Override
    public void disableUser(SmartlockDeviceDisableUserReq disableReq) {
        try {
            String accessToken = oauthToken();
            SmartlockDeviceReq req = new SmartlockDeviceReq();
            req.setData(disableReq);
            req.setType(disableReq.getType());
            log.info("--->智能门锁禁止用户入参:{}", JSONObject.toJSONString(req));
            String resp = OkHttpUtils.builder().url(HOST + String.format(DEVICES_URI, disableReq.getDeviceSn())).addHeader("appId", APP_ID).addHeader("accessToken", accessToken).localPost(JSONObject.toJSONString(req)).sync();
            log.info("--->智能门锁禁止用户响应参数:{}", resp);
        } catch (Exception e) {
            log.error("--->{}智能门锁禁止用户服务异常", disableReq.getDeviceSn(), e);
        }

    }

    @Override
    public void notifyMsg(SmartlockDeviceNotifyMsgReq req) {
        log.info("--->智能门锁异步通知入参：{}", JSONObject.toJSONString(req));
        //通知类型，取值 deviceDataChanged
        req.getNotifyType();
        req.getGatewayId();
        req.getRequestId();
        //设备SN
        String deviceId = req.getDeviceId();
        //设备的通知数据 ServiceData
        ServiceData serviceData = req.getService();
        if (Preconditions.isBlank(serviceData)) {
            return;
        }
        serviceData.getServiceId();
        //服务类型 当前只支持类型 LOCK_DEVICE_DATA_CHANGED
        serviceData.getServiceType();
        serviceData.getEventTime();
        LockDeviceDataChanged data = serviceData.getData();
        if (Preconditions.isBlank(data)) {
            return;
        }
        //数据类型
        String dataType = data.getDataType();
        if (Preconditions.isBlank(dataType)) {
            return;
        }
        Object lockDeviceData = data.getData();
        // 门锁联网状态
        if (dataType.equals(DataTypeEnum.LOCK_NETWORK_STATUE.getType())) {
            log.info("--->门锁联网状态：{}", JSON.toJSONString(lockDeviceData));
            handleLockNetworkStatue(deviceId, lockDeviceData);
        }

        // 门锁开门
        if (dataType.equals(DataTypeEnum.LOCK_OPEN_DOOR.getType())) {
            log.info("--->门锁开门：{}", JSON.toJSONString(lockDeviceData));
            handleLockOpenDoor(deviceId, lockDeviceData);
        }

        // 门锁操作返回
        if (dataType.equals(DataTypeEnum.LOCK_OPERATION_RESP.getType())) {
            log.info("--->门锁操作返回：{}", JSON.toJSONString(lockDeviceData));
            handleLockOperationResp(deviceId, lockDeviceData);
        }
    }

    /**
     * 门锁操作
     *
     * @param deviceId
     * @param lockDeviceData
     */
    private void handleLockOperationResp(String deviceId, Object lockDeviceData) {
        UserOperation userOperation = JSONObject.parseObject(JSON.toJSONString(lockDeviceData), UserOperation.class);
        if (Preconditions.isBlank(userOperation)) {
            return;
        }

        //操作管理员
        if (userOperation.getType() == TypeCommand.USER_CHANGE_PWD) {
            return;
        }

        SmartlockUser smartlockUser = smartlockUserDao.selectOneByExample(Example.builder(SmartlockUser.class)
                .where(WeekendSqls.<SmartlockUser>custom()
                        .andEqualTo(SmartlockUser::getDeviceSn, deviceId)
                        .andEqualTo(SmartlockUser::getSmartlockUserId, userOperation.getData().getUserId())).build());
        if (Preconditions.isBlank(smartlockUser)) {
            log.error("--->门锁{}用户{}不存在或已删除", deviceId, userOperation.getData().getUserId());
            return;
        }

        //删除用户
        if (userOperation.getType() == TypeCommand.USER_DEL) {
            smartlockUserDao.deleteByPrimaryKey(smartlockUser);
        }


    }

    /**
     * 门锁开门
     *
     * @param deviceId
     * @param lockDeviceData
     */
    private void handleLockOpenDoor(String deviceId, Object lockDeviceData) {
        LockOpenDoor lockOpenDoor = JSONObject.parseObject(JSON.toJSONString(lockDeviceData), LockOpenDoor.class);
        if (Preconditions.isBlank(lockOpenDoor) || Preconditions.isBlank(lockOpenDoor.getUserId())) {
            return;
        }

        SmartlockUser smartlockUser = smartlockUserDao.selectOneByExample(Example.builder(SmartlockUser.class)
                .where(WeekendSqls.<SmartlockUser>custom()
                        .andEqualTo(SmartlockUser::getDeviceSn, deviceId)
                        .andEqualTo(SmartlockUser::getSmartlockUserId, lockOpenDoor.getUserId())).build());
        if (Preconditions.isBlank(smartlockUser)) {
            log.error("--->门锁{}用户{}不存在或已删除", deviceId, lockOpenDoor.getUserId());
            return;
        }
        //进出记录
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(smartlockUser.getUserId());
        Map<String, Object> dto = new HashMap<>();
        dto.put("SN", deviceId);
        dto.put("action", 1);
        JSONObject dto1 = new JSONObject();
        dto1.put("recordid", Integer.valueOf(smartlockUser.getUserId() + ""));
        dto1.put("pin", Integer.valueOf(smartlockUser.getUserId() + ""));
        dto1.put("name", Preconditions.isNotBlank(userInfo) ? userInfo.getUsername() : null);
//        dto1.put("photo", Preconditions.isNotBlank(userInfo) ? userInfo.getAvatar() : null);
        dto1.put("cardid", Preconditions.isNotBlank(userInfo) ? userInfo.getIdNumber() : null);
        dto1.put("phone", Preconditions.isNotBlank(userInfo) ? userInfo.getPhone() : null);
        dto1.put("cardIdEx", Preconditions.isNotBlank(userInfo) ? userInfo.getCardIdEx() : null);
        dto1.put("verifytype", 0);
        dto1.put("rightno", 103);
        dto1.put("time", lockOpenDoor.getTime());
        dto1.put("outtype", 1);
        dto.put("data", Arrays.asList(dto1));
        faceTBService.eventRecords(dto);
    }

    @Override
    public void addSmartLockUser(DownFaceNew downFace, Date startTime, Date endTime, Integer adminId) {
        //组装参数
        SmartlockDeviceAddUserReq req = new SmartlockDeviceAddUserReq();
        req.setDeviceSn(downFace.getSn());
        req.setUserName(downFace.getName());
        req.setValidType("VALIDTYPE_VALID_PERIOD");
        req.setValidPeriodStartTime(DateUtil.format(startTime, DateUtils.YYYY_MM_DD_HH_MM_SS));
        req.setValidPeriodEndTime(DateUtil.format(endTime, DateUtils.YYYY_MM_DD_HH_MM_SS));
//        if (Preconditions.isNotBlank(downFace.getCardid())) {
//            //身份证号
//            req.setUserType("CARD_ID");
//            req.setUserData(downFace.getCardid());
//            SmartlockDeviceResp resp = addUser(req);
//            if (Preconditions.isNotBlank(resp) && resp.getCode().intValue() == 0) {
//                handleSmartLockUser(downFace.getUserId(), resp, "CARD_ID", adminId, downFace.getSn());
//            }
//        }

        if (Preconditions.isBlank(downFace.getPhoto())) {
            throw new CommonException("人脸照片不存在");
        }

        //人脸特征
        req.setUserType("SHANGTANG_FACE");
//            StringToHexUtils.stringToHexString(downFace.getPhoto())
        req.setUserData(FileUtils.compressNetworkPictureToCustom(AddressUtil.address, downFace.getPhoto(), 50));
//        req.setUserData(downFace.getPhoto());
        SmartlockDeviceResp resp = addUser(req);
        if (Preconditions.isBlank(resp)) {
            throw new CommonException("智能门锁人脸下发服务异常");
        }

        if (Preconditions.isBlank(resp.getCode()) || resp.getCode().intValue() != 0) {
            throw new CommonException(resp.getDesc());
        }

        if (resp.getCode().intValue() == 0) {
            handleSmartLockUser(downFace.getUserId(), resp, "SHANGTANG_FACE", adminId, downFace.getSn());
        }

    }

    @Override
    public void delSmartlockUser(String deviceSn, List<Long> userIds) {
        if (Preconditions.isBlank(userIds)) {
            return;
        }
        List<SmartlockUser> smartlockUsers = smartlockUserDao.selectByExample(Example.builder(SmartlockUser.class)
                .where(WeekendSqls.<SmartlockUser>custom()
                        .andEqualTo(SmartlockUser::getDeviceSn, deviceSn).andIn(SmartlockUser::getUserId, userIds)).build());
        if (Preconditions.isBlank(smartlockUsers)) {
            return;
        }

        smartlockUsers.stream().forEach(item -> {
            SmartlockDeviceDelUserReq req = new SmartlockDeviceDelUserReq();
            req.setUserId(item.getSmartlockUserId());
            req.setDeviceSn(deviceSn);
            delUser(req);
        });


    }

    @Override
    public void cleanSmartlockUser(String sn) {
        SmartlockDeviceCleanUserReq req = new SmartlockDeviceCleanUserReq();
        req.setDeviceSn(sn);
        cleanUser(req);
        //删除sn用户数据
        handleDelSnSmartlockUser(sn);
    }

    @Override
    public void subscribeCallbackUrl() {
        try {
            String accessToken = oauthToken();
            SmartlockSubscribeCallbackUrlReq req = new SmartlockSubscribeCallbackUrlReq();
            req.setCallbackUrl(CALLBACK_URL);
            log.info("--->智能门锁注册回调地址入参:{}", JSONObject.toJSONString(req));
            String resp = OkHttpUtils.builder().url(HOST + SUBSCRIBE_URI).addHeader("appId", APP_ID).addHeader("accessToken", accessToken).localPost(JSONObject.toJSONString(req)).sync();
            log.info("--->智能门锁注册回调地址响应参数:{}", resp);
        } catch (Exception e) {
            log.error("--->智能门锁注册回调地址服务异常", e);
        }
    }


    private void handleDelSnSmartlockUser(String deviceSn) {
        List<SmartlockUser> smartlockUsers = smartlockUserDao.selectByExample(Example.builder(SmartlockUser.class)
                .where(WeekendSqls.<SmartlockUser>custom()
                        .andEqualTo(SmartlockUser::getDeviceSn, deviceSn)).build());

        if (Preconditions.isNotBlank(smartlockUsers)) {
            smartlockUsers.stream().forEach(item -> smartlockUserDao.delete(item));
        }
    }

    //TODO。。。处理智能门锁添加用户返回信息 保存智能门锁用户id 并关联user_info表
    private void handleSmartLockUser(Long userId, SmartlockDeviceResp resp, String userType, Integer adminId, String deviceSn) {
        if (Preconditions.isBlank(resp.getData())) {
            return;
        }

        RequestDataResp data = resp.getData();
        if (Preconditions.isBlank(data)) {
            return;
        }
        AddUserDataResp userDataResp = JSONObject.parseObject(JSONObject.toJSONString(data.getData()), AddUserDataResp.class);
        if (Preconditions.isBlank(userDataResp)) {
            return;
        }

        SmartlockUser smartlockUser = smartlockUserDao.selectOneByExample(Example.builder(SmartlockUser.class)
                .where(WeekendSqls.<SmartlockUser>custom()
                        .andEqualTo(SmartlockUser::getDeviceSn, deviceSn)
                        .andEqualTo(SmartlockUser::getUserId, userId)
                        .andEqualTo(SmartlockUser::getUserType, userType)).build());
        if (Preconditions.isNotBlank(smartlockUser)) {
            smartlockUser.setAdminId(adminId);
            smartlockUser.setUserId(userId);
            smartlockUser.setSmartlockUserId(userDataResp.getUserId());
            smartlockUser.setTokenId(userDataResp.getTokenId());
            smartlockUser.setRequestId(data.getRequestId());
            smartlockUser.setDeviceSn(deviceSn);
            smartlockUserDao.updateByPrimaryKey(smartlockUser);
        } else {
            smartlockUser = new SmartlockUser();
            smartlockUser.setAdminId(adminId);
            smartlockUser.setUserId(userId);
            smartlockUser.setSmartlockUserId(userDataResp.getUserId());
            smartlockUser.setTokenId(userDataResp.getTokenId());
            smartlockUser.setRequestId(data.getRequestId());
            smartlockUser.setUserType(userType);
            smartlockUser.setCreateTime(new Date());
            smartlockUser.setDeviceSn(deviceSn);
            smartlockUserDao.insert(smartlockUser);
        }
    }

    //门锁联网状态 同步设备在离线状态
    private void handleLockNetworkStatue(String deviceId, Object lockDeviceData) {
        LockNetworkStatue lockNetworkStatue = JSONObject.parseObject(JSON.toJSONString(lockDeviceData), LockNetworkStatue.class);
        if (Preconditions.isBlank(lockNetworkStatue)) {
            return;
        }
        Integer networkStatue = lockNetworkStatue.getNetworkStatue();
        //TODO。。。 deviceId 根据设备id修改设备在离线状态
        FaceDevice faceDevice = faceDeviceDao.selectOneByExample(Example.builder(FaceDevice.class).where(WeekendSqls.<FaceDevice>custom().andEqualTo(FaceDevice::getSn, deviceId)).build());
        if (Preconditions.isBlank(faceDevice)) {
            log.error("--->{}设备不存在或已删除", deviceId);
            return;
        }
        faceDevice.setOnline(networkStatue.intValue() == 0 ? 1 : 0);
        faceDeviceDao.updateByPrimaryKeySelective(faceDevice);
    }


}
