package com.yx.face.service.udistrict;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.core.Constants;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.restful.RestResponse;
import com.yx.face.boot.restful.RestResult;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.*;
import com.yx.face.dao.AdminAuthDao;
import com.yx.face.dao.UserWhitelistDao;
import com.yx.face.model.api.district.DistrictEncryptionRequest;
import com.yx.face.model.api.district.DistrictEncryptionResponse;
import com.yx.face.model.api.district.OpenDistrictCreateRequest;
import com.yx.face.model.api.district.OpenDistrictCreateResponse;
import com.yx.face.model.api.district.OpenDistrictDeleteRequest;
import com.yx.face.model.api.district.OpenDistrictDeleteResponse;
import com.yx.face.model.api.district.OpenDistrictQueryRequest;
import com.yx.face.model.api.district.OpenDistrictQueryResponse;
import com.yx.face.model.api.district.OpenDistrictUpdateRequest;
import com.yx.face.model.api.district.OpenDistrictUpdateResponse;
import com.yx.face.model.api.user.OpenUserCustomFieldQueryResponse;
import com.yx.face.model.api.user.OpenUserWhitelistCreateRequest;
import com.yx.face.model.api.user.OpenUserWhitelistCreateResponse;
import com.yx.face.model.api.user.OpenUserWhitelistDeleteRequest;
import com.yx.face.model.api.user.OpenUserWhitelistDistributeRequest;
import com.yx.face.model.api.user.OpenUserWhitelistQueryRequest;
import com.yx.face.model.api.user.OpenUserWhitelistQueryResponse;
import com.yx.face.model.api.user.UserEncryptionRequest;
import com.yx.face.model.api.user.UserEncryptionResponse;
import com.yx.face.model.dto.UserWhitelistBatchDelDTO;
import com.yx.face.model.dto.UserWhitelistBatchDistributionDTO;
import com.yx.face.model.dto.UserWhitelistDTO;
import com.yx.face.model.entity.AdminAuth;
import com.yx.face.model.entity.UserWhitelist;
import com.yx.face.model.entity.ZoneDeviceR;
import com.yx.face.model.enums.AdminAuthTypeEnum;
import com.yx.face.model.enums.AdminPlaceTypeEnum;
import com.yx.face.model.enums.AdminTypeEnum;
import com.yx.face.model.enums.CustomTypeEnum;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.ContextVO;
import com.yx.face.model.vo.third.ZoneDeviceRDto;
import com.yx.face.service.AdminService;
import com.yx.face.service.UserWhitelistService;
import com.yx.face.service.ZoneDeviceRService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package com.yx.face.service.udistrict
 * @class OpenUserDistrictServiceImpl
 * @date 2024/3/22 下午3:15
 * @description
 */
@Service
@Slf4j
public class OpenUserDistrictServiceImpl implements OpenUserDistrictService {


    @Resource
    private AdminAuthDao adminAuthDao;

    @Resource
    private AdminService adminService;

    @Resource
    private UserWhitelistService userWhitelistService;

    @Resource
    private ZoneDeviceRService zoneDeviceRService;

    @Resource
    private UserWhitelistDao userWhitelistDao;

   

    @Override
    public RestResult<UserEncryptionResponse> userCustomFieldQuery(UserEncryptionRequest req) {
        AdminAuth auth = getAuth(req.getAppId());
        //校验授权
        analysisBizContent(req, auth, false);
        AdminVO adminVO = adminService.getAdminInfoById(auth.getAdminId());
        if (Preconditions.isBlank(adminVO)
                || Preconditions.isBlank(adminVO.getUserManageSwitch())
                || !adminVO.getUserManageSwitch()) {
            throw new CommonException("人员管理权限未开启，请联系管理人员");
        }

        List<OpenUserCustomFieldQueryResponse> bizContents = new ArrayList<>();
        if (Preconditions.isNotBlank(adminVO.getContent())) {
            bizContents = JSON.parseArray(adminVO.getContent(), OpenUserCustomFieldQueryResponse.class);
        }
        UserEncryptionResponse response = new UserEncryptionResponse();
        long currentTimeMillis = System.currentTimeMillis();
        response.setSign(Sm4Util.getSign(auth.getAppKey(), auth.getAppSecret(), String.valueOf(currentTimeMillis)));
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(Sm4Util.encrypt(auth.getPrivateKey(), JSONObject.toJSONString(bizContents)));
        return RestResponse.ok(response);
    }

    @Override
    public RestResult<UserEncryptionResponse> userCreate(UserEncryptionRequest req) {
        AdminAuth auth = getAuth(req.getAppId());
        //校验授权
        String decrypt = analysisBizContent(req, auth, true);
        //数据转换
        OpenUserWhitelistCreateRequest userWhitelistCreate = JSONObject.parseObject(decrypt, OpenUserWhitelistCreateRequest.class);
        if (Preconditions.isBlank(userWhitelistCreate.getHid())
                || Preconditions.isBlank(userWhitelistCreate.getIdNumber())
                || Preconditions.isBlank(userWhitelistCreate.getAvatar())) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        AdminVO info = adminService.getInfo(auth.getAdminId());
        if (Preconditions.isBlank(info)
                || Preconditions.isBlank(info.getUserManageSwitch())
                || !info.getUserManageSwitch()) {
            throw new CommonException("人员管理权限未开启，请联系管理人员");
        }
        String pathString = System.currentTimeMillis() + String.valueOf((int) (Math.random() * 10000)) + ".jpg";
        String filePath = FileUtils.getDirectory(Constants.UPLOAD_IMAGE_NEW) + "/";
        String img = FileUtils.transformBase64Image(userWhitelistCreate.getAvatar(), filePath, pathString);
        UserWhitelistDTO dto = BeanUtil.copyProperties(userWhitelistCreate, UserWhitelistDTO.class);
        String image = userWhitelistCreate.getAvatar();
        if (Preconditions.isNotBlank(img)) {
            image = AddressUtil.address + img;
        }
        Integer adminAreaType = null;
        if (!info.getType().equals(AdminTypeEnum.ENTERPRISE.getCode())) {
            dto.setPrivateAreaIdList(null);
        } else {
            adminAreaType = 2;
        }
        if (info.getType().equals(AdminTypeEnum.DISTRICT.getCode())
                && (AdminPlaceTypeEnum.BUILDING_PLACE.getCode().equals(info.getPlaceType()) || AdminPlaceTypeEnum.SCHOOL_PLACE.getCode().equals(info.getPlaceType()))) {
            throw new ServiceException("常客无法添加入当前配置的（楼宇/学校）区级账号");
        }
        String content = null;
        List<ContextVO> customFieldList = userWhitelistCreate.getCustomFieldList();
        if (Preconditions.isNotBlank(customFieldList)) {
            customFieldList.stream().forEach(item -> item.setCustomType(Preconditions.isNotBlank(item.getCustomType()) ? CustomTypeEnum.valueOf(item.getCustomType()).getDesc() : item.getCustomType()));
            content = JSON.toJSONString(customFieldList);
        }
        UserEncryptionResponse response = new UserEncryptionResponse();
        OpenUserWhitelistCreateResponse bizContent = null;
        UserWhitelist whitelist = userWhitelistDao.selectOneByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andEqualTo(UserWhitelist::getHid, dto.getHid()).andEqualTo(UserWhitelist::getAdminId, auth.getAdminId())).build());
        try {
            long currentTimeMillis = System.currentTimeMillis();
            response.setSign(Sm4Util.getSign(auth.getAppKey(), auth.getAppSecret(), String.valueOf(currentTimeMillis)));
            response.setReqTimestamp(currentTimeMillis);
            if (Preconditions.isBlank(whitelist)) {
                dto.setInputType(2);
                dto.setPushType(2);
                dto.setAdminAreaType(adminAreaType);
                dto.setContent(content);
                dto.setAvatar(image);
                dto.setAdminId(auth.getAdminId());
                whitelist = userWhitelistService.insert(dto, 0);
            } else {
                UserWhitelistDTO whitelistDTO = BeanUtil.copyProperties(whitelist, UserWhitelistDTO.class);
                whitelistDTO.setContent(content);
                whitelistDTO.setAvatar(image);
                whitelistDTO.setAdminId(auth.getAdminId());
                whitelistDTO.setCardIdEx(userWhitelistCreate.getCardIdEx());
                whitelistDTO.setEndTime(userWhitelistCreate.getEndTime());
                whitelistDTO.setIdNumber(userWhitelistCreate.getIdNumber());
                whitelistDTO.setName(userWhitelistCreate.getName());
                whitelistDTO.setPhone(userWhitelistCreate.getPhone());
                whitelistDTO.setStartTime(userWhitelistCreate.getStartTime());
                whitelistDTO.setZoneDevicePassIdList(userWhitelistCreate.getZoneDevicePassIdList());
                whitelist = userWhitelistService.update(whitelistDTO);
            }
            if (Objects.nonNull(whitelist) && Objects.nonNull(whitelist.getId())) {
                bizContent = BeanUtil.copyProperties(whitelist, OpenUserWhitelistCreateResponse.class);
                response.setBizContent(Sm4Util.encrypt(auth.getPrivateKey(), JSONObject.toJSONString(bizContent)));
            }
            return RestResponse.ok(response);
        } catch (Exception e) {
            log.error("--->服务异常", e);
            e.printStackTrace();
            throw new CommonException(e.getMessage());
        }
    }

    /**
     * 获取授权信息
     *
     * @param appId
     * @return
     */
    private AdminAuth getAuth(String appId) {
        AdminAuth auth = adminAuthDao.selectOneByExample(Example.builder(AdminAuth.class)
                .where(WeekendSqls.<AdminAuth>custom()
                        .andEqualTo(AdminAuth::getAppId, appId)
                        .andEqualTo(AdminAuth::getType, AdminAuthTypeEnum.USER_DISTRICT_M.getCode())
                        .andEqualTo(AdminAuth::getIsDeleted, Boolean.FALSE)
                ).build());
        return auth;
    }

    /**
     * 解析参数
     *
     * @param req
     */
    private String analysisBizContent(UserEncryptionRequest req, AdminAuth auth, Boolean decryptContent) {
        log.info("--->用户信息解密前原始入参内容：{}", JSONObject.toJSONString(req));
        if (Preconditions.isBlank(auth)) {
            throw new CommonException("无权限调用，请联系管理人员");
        }
        String appKey = auth.getAppKey();
        String appSecret = auth.getAppSecret();
        String privateKey = auth.getPrivateKey();
        //验证签名
        Boolean checkSign = Sm4Util.checkSign(appKey, appSecret, req.getSign(), String.valueOf(req.getReqTimestamp()));
        if (!checkSign) {
            throw new CommonException("签名错误");
        }
        //内容解密
        String decrypt = Sm4Util.decrypt(privateKey, req.getBizContent());
        log.info("--->用户信息解密后入参内容：{}", decrypt);
        if (decryptContent && Preconditions.isBlank(decrypt)) {
            throw new CommonException("用户信息内容解密失败，请检查后重新提交");
        }
        return decrypt;
    }

    private String analysisDistrictBizContent(DistrictEncryptionRequest req, AdminAuth auth, Boolean decryptContent) {
        log.info("--->区域信息解密前原始入参内容：{}", JSONObject.toJSONString(req));
        if (Preconditions.isBlank(auth)) {
            throw new CommonException("无权限调用，请联系管理人员");
        }
        String appKey = auth.getAppKey();
        String appSecret = auth.getAppSecret();
        String privateKey = auth.getPrivateKey();
        //验证签名
        Boolean checkSign = Sm4Util.checkSign(appKey, appSecret, req.getSign(), String.valueOf(req.getReqTimestamp()));
        if (!checkSign) {
            throw new CommonException("签名错误");
        }
        //内容解密
        String decrypt = Sm4Util.decrypt(privateKey, req.getBizContent());
        log.info("--->区域信息解密后入参内容：{}", decrypt);
        if (decryptContent && Preconditions.isBlank(decrypt)) {
            throw new CommonException("区域信息内容解密失败，请检查后重新提交");
        }
        return decrypt;
    }

    @Override
    public RestResult<UserEncryptionResponse> userDelete(UserEncryptionRequest req) {
        AdminAuth auth = getAuth(req.getAppId());
        AdminVO info = adminService.getInfo(auth.getAdminId());
        if (Preconditions.isBlank(info)
                || Preconditions.isBlank(info.getUserManageSwitch())
                || !info.getUserManageSwitch()) {
            throw new CommonException("人员管理权限未开启，请联系管理人员");
        }
        //校验授权
        String decrypt = analysisBizContent(req, auth, true);
        //数据转换
        OpenUserWhitelistDeleteRequest userWhitelistDelete = JSONObject.parseObject(decrypt, OpenUserWhitelistDeleteRequest.class);
        if (Preconditions.isBlank(userWhitelistDelete.getHidList())) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        List<UserWhitelist> whitelist = userWhitelistDao.selectByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andIn(UserWhitelist::getHid, userWhitelistDelete.getHidList()).andEqualTo(UserWhitelist::getAdminId, auth.getAdminId())).build());
        ClassUtil.throwBusinessException(Preconditions.isBlank(whitelist), "常客数据不存在或已删除");
        UserWhitelistBatchDelDTO dto = new UserWhitelistBatchDelDTO();
        dto.setIds(whitelist.stream().map(UserWhitelist::getId).collect(Collectors.toList()));
        dto.setAdminId(auth.getAdminId());
        userWhitelistService.batchDelete(dto);
        UserEncryptionResponse response = new UserEncryptionResponse();
        long currentTimeMillis = System.currentTimeMillis();
        response.setSign(Sm4Util.getSign(auth.getAppKey(), auth.getAppSecret(), String.valueOf(currentTimeMillis)));
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(null);
        return RestResponse.ok(response);
    }

    @Override
    public RestResult<UserEncryptionResponse> userQuery(UserEncryptionRequest req) {
        AdminAuth auth = getAuth(req.getAppId());
        AdminVO info = adminService.getInfo(auth.getAdminId());
        if (Preconditions.isBlank(info)
                || Preconditions.isBlank(info.getUserManageSwitch())
                || !info.getUserManageSwitch()) {
            throw new CommonException("人员管理权限未开启，请联系管理人员");
        }
        //校验授权
        String decrypt = analysisBizContent(req, auth, true);
        //数据转换
        OpenUserWhitelistQueryRequest userWhitelistQuery = JSONObject.parseObject(decrypt, OpenUserWhitelistQueryRequest.class);
        if (Preconditions.isBlank(userWhitelistQuery.getHid())) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        UserWhitelist whitelist = userWhitelistDao.selectOneByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andEqualTo(UserWhitelist::getHid, userWhitelistQuery.getHid()).andEqualTo(UserWhitelist::getAdminId, auth.getAdminId())).build());
        ClassUtil.throwBusinessException(Preconditions.isBlank(whitelist), "常客数据不存在或已删除");
        OpenUserWhitelistQueryResponse queryResponse = BeanUtil.copyProperties(whitelist, OpenUserWhitelistQueryResponse.class);
        queryResponse.setCustomFieldList(Preconditions.isNotBlank(whitelist.getContent()) ? JSON.parseArray(whitelist.getContent(), ContextVO.class) : null);
        UserEncryptionResponse response = new UserEncryptionResponse();
        long currentTimeMillis = System.currentTimeMillis();
        response.setSign(Sm4Util.getSign(auth.getAppKey(), auth.getAppSecret(), String.valueOf(currentTimeMillis)));
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(Sm4Util.encrypt(auth.getPrivateKey(), JSONObject.toJSONString(queryResponse)));
        return RestResponse.ok(response);
    }

    @Override
    public RestResult<UserEncryptionResponse> userDistribute(UserEncryptionRequest req) {
        AdminAuth auth = getAuth(req.getAppId());
        AdminVO info = adminService.getInfo(auth.getAdminId());
        if (Preconditions.isBlank(info)
                || Preconditions.isBlank(info.getUserManageSwitch())
                || !info.getUserManageSwitch()) {
            throw new CommonException("人员管理权限未开启，请联系管理人员");
        }
        //校验授权
        String decrypt = analysisBizContent(req, auth, true);
        //数据转换
        OpenUserWhitelistDistributeRequest userWhitelistDistribute = JSONObject.parseObject(decrypt, OpenUserWhitelistDistributeRequest.class);
        if (Preconditions.isBlank(userWhitelistDistribute.getHidList())) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        List<UserWhitelist> whitelist = userWhitelistDao.selectByExample(Example.builder(UserWhitelist.class).where(WeekendSqls.<UserWhitelist>custom().andIn(UserWhitelist::getHid, userWhitelistDistribute.getHidList()).andEqualTo(UserWhitelist::getAdminId, auth.getAdminId())).build());
        ClassUtil.throwBusinessException(Preconditions.isBlank(whitelist), "常客数据不存在或已删除");
        UserWhitelistBatchDistributionDTO dto = new UserWhitelistBatchDistributionDTO();
        dto.setIds(whitelist.stream().map(UserWhitelist::getId).collect(Collectors.toList()));
        dto.setAdminId(auth.getAdminId());
        userWhitelistService.batchDistribution(dto);
        UserEncryptionResponse response = new UserEncryptionResponse();
        long currentTimeMillis = System.currentTimeMillis();
        response.setSign(Sm4Util.getSign(auth.getAppKey(), auth.getAppSecret(), String.valueOf(currentTimeMillis)));
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(null);
        return RestResponse.ok(response);
    }

    @Override
    public RestResult<DistrictEncryptionResponse> districtCreate(DistrictEncryptionRequest req) {
        AdminAuth auth = getAuth(req.getAppId());
        AdminVO info = adminService.getInfo(auth.getAdminId());
        if (Preconditions.isBlank(info)
                || Preconditions.isBlank(info.getZoneManageSwitch())
                || !info.getZoneManageSwitch()) {
            throw new CommonException("区域管理权限未开启，请联系管理人员");
        }
        //校验授权
        String decrypt = analysisDistrictBizContent(req, auth, true);
        //数据转换
        OpenDistrictCreateRequest districtCreate = JSONObject.parseObject(decrypt, OpenDistrictCreateRequest.class);
        if (Preconditions.isBlank(districtCreate) || Preconditions.isBlank(districtCreate.getName())) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        ZoneDeviceRDto dto = BeanUtil.copyProperties(districtCreate, ZoneDeviceRDto.class);
        dto.setAdminId(auth.getAdminId());
        if (Preconditions.isNotBlank(districtCreate.getSnList())) {
            dto.setSnJson(districtCreate.getSnList().stream().collect(Collectors.joining(",")));
        }
        Integer id = zoneDeviceRService.create(dto);
        OpenDistrictCreateResponse bizContent = new OpenDistrictCreateResponse();
        bizContent.setId(id);
        DistrictEncryptionResponse response = new DistrictEncryptionResponse();
        long currentTimeMillis = System.currentTimeMillis();
        response.setSign(Sm4Util.getSign(auth.getAppKey(), auth.getAppSecret(), String.valueOf(currentTimeMillis)));
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(Sm4Util.encrypt(auth.getPrivateKey(), JSONObject.toJSONString(bizContent)));
        return RestResponse.ok(response);
    }

    @Override
    public RestResult<DistrictEncryptionResponse> districtUpdate(DistrictEncryptionRequest req) {
        AdminAuth auth = getAuth(req.getAppId());
        AdminVO info = adminService.getInfo(auth.getAdminId());
        if (Preconditions.isBlank(info)
                || Preconditions.isBlank(info.getZoneManageSwitch())
                || !info.getZoneManageSwitch()) {
            throw new CommonException("区域管理权限未开启，请联系管理人员");
        }
        //校验授权
        String decrypt = analysisDistrictBizContent(req, auth, true);
        //数据转换
        OpenDistrictUpdateRequest districtUpdate = JSONObject.parseObject(decrypt, OpenDistrictUpdateRequest.class);
        if (Preconditions.isBlank(districtUpdate) || Preconditions.isBlank(districtUpdate.getName()) || Preconditions.isBlank(districtUpdate.getId())) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        ZoneDeviceRDto dto = BeanUtil.copyProperties(districtUpdate, ZoneDeviceRDto.class);
        dto.setAdminId(auth.getAdminId());
        if (Preconditions.isNotBlank(districtUpdate.getSnList())) {
            dto.setSnJson(districtUpdate.getSnList().stream().collect(Collectors.joining(",")));
        }
        Integer id = zoneDeviceRService.create(dto);
        OpenDistrictUpdateResponse bizContent = new OpenDistrictUpdateResponse();
        bizContent.setId(id);
        DistrictEncryptionResponse response = new DistrictEncryptionResponse();
        long currentTimeMillis = System.currentTimeMillis();
        response.setSign(Sm4Util.getSign(auth.getAppKey(), auth.getAppSecret(), String.valueOf(currentTimeMillis)));
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(Sm4Util.encrypt(auth.getPrivateKey(), JSONObject.toJSONString(bizContent)));
        return RestResponse.ok(response);
    }

    @Override
    public RestResult<DistrictEncryptionResponse> districtDelete(DistrictEncryptionRequest req) {
        AdminAuth auth = getAuth(req.getAppId());
        AdminVO info = adminService.getInfo(auth.getAdminId());
        if (Preconditions.isBlank(info)
                || Preconditions.isBlank(info.getZoneManageSwitch())
                || !info.getZoneManageSwitch()) {
            throw new CommonException("区域管理权限未开启，请联系管理人员");
        }
        //校验授权
        String decrypt = analysisDistrictBizContent(req, auth, true);
        //数据转换
        OpenDistrictDeleteRequest districtDelete = JSONObject.parseObject(decrypt, OpenDistrictDeleteRequest.class);
        if (Preconditions.isBlank(districtDelete) || Preconditions.isBlank(districtDelete.getIdList())) {
            throw new CommonException("参数有误，请检查后重新提交");
        }
        List<OpenDistrictDeleteResponse> bizContent = zoneDeviceRService.delete(districtDelete.getIdList(), auth.getAdminId());
        DistrictEncryptionResponse response = new DistrictEncryptionResponse();
        long currentTimeMillis = System.currentTimeMillis();
        response.setSign(Sm4Util.getSign(auth.getAppKey(), auth.getAppSecret(), String.valueOf(currentTimeMillis)));
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(Sm4Util.encrypt(auth.getPrivateKey(), JSONObject.toJSONString(bizContent)));
        return RestResponse.ok(response);
    }

    @Override
    public RestResult<DistrictEncryptionResponse> districtQuery(DistrictEncryptionRequest req) {
        AdminAuth auth = getAuth(req.getAppId());
        AdminVO info = adminService.getInfo(auth.getAdminId());
        if (Preconditions.isBlank(info)
                || Preconditions.isBlank(info.getZoneManageSwitch())
                || !info.getZoneManageSwitch()) {
            throw new CommonException("区域管理权限未开启，请联系管理人员");
        }
        //校验授权
        String decrypt = analysisDistrictBizContent(req, auth, true);
        //数据转换
        OpenDistrictQueryRequest districtQuery = JSONObject.parseObject(decrypt, OpenDistrictQueryRequest.class);
        ZoneDeviceRDto dto = BeanUtil.copyProperties(districtQuery, ZoneDeviceRDto.class);
        dto.setAdminId(auth.getAdminId());
        List<ZoneDeviceR> list = zoneDeviceRService.query(dto);
        List<OpenDistrictQueryResponse> bizContents = new ArrayList<>();
        if (Preconditions.isNotBlank(list)) {
            list.stream().forEach(item -> {
                OpenDistrictQueryResponse bizContent = BeanUtil.copyProperties(item, OpenDistrictQueryResponse.class);
                if (Preconditions.isNotBlank(item.getSnJson())) {
                    bizContent.setSnList(Arrays.asList(item.getSnJson().split(",")));
                }
                bizContents.add(bizContent);
            });
        }
        DistrictEncryptionResponse response = new DistrictEncryptionResponse();
        long currentTimeMillis = System.currentTimeMillis();
        response.setSign(Sm4Util.getSign(auth.getAppKey(), auth.getAppSecret(), String.valueOf(currentTimeMillis)));
        response.setReqTimestamp(currentTimeMillis);
        response.setBizContent(Sm4Util.encrypt(auth.getPrivateKey(), JSONObject.toJSONString(bizContents)));
        return RestResponse.ok(response);
    }
}
