## 通用打卡进出记录推送接口

**接口地址**:`三方提供接口地址` 

**请求方式**:`POST`

**请求数据类型**:`application/json`

**响应数据类型**:`*/*`

**接口描述**:

- 通用打卡进出记录推送接口
- **姓名、身份证号、手机号采用国密sm4 (ECB模式)进行加密处理，加解密方式参考文末**
- **加密所需私钥 privateKey由管理员提供**



**修正记录**: 

- 2023-05-30 姓名、身份证号、手机号采用国密sm4 (ECB模式)进行加密处理 **加解密方式参考文末**



##### 请求参数示例

```java
{
    "reason": "业务拜访",
    "visiteeCompany": "银行",
    "hsStatus": "",
    "visiteeBm": "塞给他",
    "deviceSn": "hh_96802402",
    "deviceAddress": "东门",
    "provinceName": "省份",
    "cityName": "城市",
    "areaName": "县区",
    "streetName": "街道/乡/镇",
    "address": "详细地址",
    "ymStatus": "0",
    "faceTime": "2023-06-07 16:59:46",
    "rightno": 104,
    "visiteeXm": "天天",
    "cardidex": "",
    "twStatus": "",
    "cardid": "14477f2eaeaa390496fac19e1a584571c93462106745ec0c87093658a9991252",
    "adminId": 105,
    "temperature": "36",
    "action": 0,
    "company": "哈哈哈哈",
    "pcsname": "四季青派出所",
    "xcInfo": "",
    "visiteeRoom": "102",
    "org": "航汇",
    "photo": "照片base64",
    "userId": 49086,
    "outType": 1,
    "orgid": "",
    "recordid": 0,
    "pcsdm": "330102014",
    "jkmStatus": "",
    "extend3": "36",
    "phone": "ad4148e39073967a749ee588e0560d5b",
    "jkmType": "",
    "createTime": 1686128355676,
    "carno": "宁A123456",
    "verifyScore": "0",
    "extend5": "",
    "name": "1db597fb4f36420afcaf6220f81c9811",
    "extend1": "2",
    "time": "2023-06-07 16:59:46",
    "verifyType": 0
}
```

##### 请求参数说明

| **参数名**     | **类型**   | **说明**                                                     |
| -------------- | ---------- | ------------------------------------------------------------ |
| recordid       | number     | 记录唯一序号                                                 |
| action         | number     | 0：离线（即开门提示逻辑由设备离线处理事后上报）1：在线（即开门提示逻辑由平台实时控制 |
| deviceSn       | string     | 设备SN                                                       |
| deviceAddress  | string     | 设备地点                                                     |
| provinceName   | string     | 省份                                                         |
| cityName       | string     | 城市                                                         |
| areaName       | string     | 县区                                                         |
| streetName     | string     | 街道/乡/镇                                                   |
| address        | string     | 详细地址                                                     |
| photo          | string     | 照片base64                                                   |
| userId         | number     | 用户id                                                       |
| **name**       | **string** | **用户姓名（国密sm4加密）**                                  |
| **phone**      | **string** | **手机号码（国密sm4加密）**                                  |
| **cardid**     | **string** | **身份证号** **（国密sm4加密）**                             |
| **idCard**     | **string** | **身份信息（国密sm4加密）**                                  |
| cardidex       | string     | 物理卡号（卡号2）                                            |
| verifyType     | number     | 识别类型 0人脸1卡2身份证3二维码4远程6IC卡+人脸               |
| rightno        | number     | 身份标识 106 (管理员)105（操作员）104（普通员工 默认）103（访客）102（禁止）  101(黑名单） |
| outType        | number     | 0 通用 1进门 2出门                                           |
| faceTime       | string     | 考勤时间格式  yyyy-MM-dd HH:mm:ss                            |
| orgid          | string     | 终端上设置的组织id                                           |
| extend3        | string     | 体温测量度数                                                 |
| extend1        | string     | String类型0：未戴1：已戴2：未启用口罩检测/开启仅检测体温     |
| twStatus       | string     | 体温是否异常 0 正常 1 异常                                   |
| jkmStatus      | string     | 健康码状态00 绿码 01 黄码 10 红码                            |
| hsStatus       | string     | 阴性 阳性                                                    |
| ymStatus       | string     | 疫苗状态 0针  等等依次累加                                   |
| xcInfo         | string     | 行程信息                                                     |
| adminId        | number     | 账号id                                                       |
| createTime     | long       | 创建时间                                                     |
| verifyScore    | number     | 人脸相似度分值                                               |
| extend5        | string     | 健康码组装数据                                               |
| jkmType        | string     | 健康码类型                                                   |
| company        | string     | 来访人单位                                                   |
| temperature    | string     | 来访人体温                                                   |
| carno          | string     | 来访人车牌号码                                               |
| reason         | string     | 访问事由                                                     |
| time           | string     | 来访时间                                                     |
| org            | string     | 设备所在单位                                                 |
| visiteeCompany | string     | 受访人单位                                                   |
| visiteeBm      | string     | 受访人部门                                                   |
| visiteeRoom    | string     | 受访人房间号                                                 |
| visiteeXm      | string     | 受访人姓名                                                   |
| pcsdm          | string     | 派出所代码                                                   |
| pcsname        | string     | 派出所名称                                                   |
|                |            |                                                              |



**国密4加解密算法及签名认证**

```java
// pom.xml中引入
<dependency>
    <groupId>cn.hutool</groupId>
    <artifactId>hutool-all</artifactId>
    <version>5.7.0</version>
</dependency>
```

```java
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.SmUtil;
import cn.hutool.crypto.symmetric.SymmetricCrypto;

@Slf4j
public class Sm4Util {

    /**
     * 加密
     *
     * @param privateKey 管理员提供
     * @param str
     * @return
     */
    public static String encrypt(final String privateKey, final String str) {
      if (StrUtil.isBlank((CharSequence) str)) {
        return "";
      }
      
       try {
            SymmetricCrypto sm4 = new SymmetricCrypto("SM4/ECB/PKCS5Padding", privateKey.getBytes());
            return sm4.encryptHex(str, Charset.forName("UTF-8"));
        } catch (Exception e) {
            log.error("加密失败", e);
            return null;
        }
    }

    /**
     * 解密
     *
     * @param privateKey 管理员提供
     * @param str
     * @return
     */
    public static String decrypt(final String privateKey, final String str) {
        if (StrUtil.isBlank((CharSequence) str)) {
            return null;
        }
        try {
            SymmetricCrypto sm4 = new SymmetricCrypto("SM4/ECB/PKCS5Padding", privateKey.getBytes());
            return sm4.decryptStr(str, Charset.forName("UTF-8"));
        } catch (Exception e) {
            log.error("解密失败", e);
            return null;
        }
    }
  
 }


```

**.net加解密方式**



![net1](./.net1.png)

 加密：

![net2](./.net2.png)

​        解密：

![net3](./.net3.png)





